@@ -1,7 +1,6 @@
 /* ---------------------------------------------------------------------
- * $Id: step-13.cc 31456 2013-10-28 02:59:42Z bangerth $
  *
- * Copyright (C) 2001 - 2013 by the deal.II authors
+ * Copyright (C) 2001 - 2015 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -33,6 +32,7 @@
 #include <deal.II/lac/vector.h>
 #include <deal.II/lac/full_matrix.h>
 #include <deal.II/lac/sparse_matrix.h>
+#include <deal.II/lac/dynamic_sparsity_pattern.h>
 #include <deal.II/lac/solver_cg.h>
 #include <deal.II/lac/precondition.h>
 #include <deal.II/lac/constraint_matrix.h>
@@ -343,15 +343,14 @@ namespace Step13
     // program we shall generate output (from this we will then also generate
     // the suffix of the filename to which we write).
     //
-    // Regarding the output format, the <code>DataOutInterface</code> class
-    // (which is a base class of <code>DataOut</code> through which we will
-    // access its fields) provides an enumeration field
-    // <code>OutputFormat</code>, which lists names for all supported output
+    // Regarding the output format, the DataOutBase namespace
+    // provides an enumeration field
+    // DataOutBase::OutputFormat which lists names for all supported output
     // formats. At the time of writing of this program, the supported graphics
     // formats are represented by the enum values <code>ucd</code>,
     // <code>gnuplot</code>, <code>povray</code>, <code>eps</code>,
     // <code>gmv</code>, <code>tecplot</code>, <code>tecplot_binary</code>,
-    // <code>dx</code>, and <code>vtk</code>, but this list will certainly
+    // <code>dx</code>, <code>vtk</code>, etc, but this list will certainly
     // grow over time. Now, within various functions of that base class, you
     // can use values of this type to get information about these graphics
     // formats (for example the default suffix used for files of each format),
@@ -372,21 +371,21 @@ namespace Step13
     class SolutionOutput : public EvaluationBase<dim>
     {
     public:
-      SolutionOutput (const std::string                         &output_name_base,
-                      const typename DataOut<dim>::OutputFormat  output_format);
+      SolutionOutput (const std::string               &output_name_base,
+                      const DataOutBase::OutputFormat  output_format);
 
       virtual void operator () (const DoFHandler<dim> &dof_handler,
                                 const Vector<double>  &solution) const;
     private:
-      const std::string                         output_name_base;
-      const typename DataOut<dim>::OutputFormat output_format;
+      const std::string               output_name_base;
+      const DataOutBase::OutputFormat output_format;
     };
 
 
     template <int dim>
     SolutionOutput<dim>::
-    SolutionOutput (const std::string                         &output_name_base,
-                    const typename DataOut<dim>::OutputFormat  output_format)
+    SolutionOutput (const std::string               &output_name_base,
+                    const DataOutBase::OutputFormat  output_format)
       :
       output_name_base (output_name_base),
       output_format (output_format)
@@ -396,7 +395,7 @@ namespace Step13
     // After the description above, the function generating the actual output
     // is now relatively straightforward. The only particularly interesting
     // feature over previous example programs is the use of the
-    // <code>DataOut::default_suffix</code> function, returning the usual
+    // DataOutBase::default_suffix function, returning the usual
     // suffix for files of a given format (e.g. ".eps" for encapsulated
     // postscript files, ".gnuplot" for Gnuplot files), and of the generic
     // <code>DataOut::write</code> function with a second argument, which
@@ -472,7 +471,7 @@ namespace Step13
     // the base class.
     //
     // Due to the lack of actual functionality, the programming style of
-    // declaring very abstract base classes reminds of the style used in
+    // declaring very abstract base classes is similar to the style used in
     // Smalltalk or Java programs, where all classes are derived from entirely
     // abstract classes <code>Object</code>, even number representations. The
     // author admits that he does not particularly like the use of such a
@@ -649,17 +648,17 @@ namespace Step13
       // calls the virtual function assembling the right hand side.
       struct AssemblyScratchData
       {
-	AssemblyScratchData (const FiniteElement<dim> &fe,
-			     const Quadrature<dim>    &quadrature);
-	AssemblyScratchData (const AssemblyScratchData &scratch_data);
+        AssemblyScratchData (const FiniteElement<dim> &fe,
+                             const Quadrature<dim>    &quadrature);
+        AssemblyScratchData (const AssemblyScratchData &scratch_data);
 
-	FEValues<dim>     fe_values;
+        FEValues<dim>     fe_values;
       };
 
       struct AssemblyCopyData
       {
-	FullMatrix<double> cell_matrix;
-	std::vector<types::global_dof_index> local_dof_indices;
+        FullMatrix<double> cell_matrix;
+        std::vector<types::global_dof_index> local_dof_indices;
       };
 
       void
@@ -667,8 +666,8 @@ namespace Step13
 
       void
       local_assemble_matrix (const typename DoFHandler<dim>::active_cell_iterator &cell,
-			     AssemblyScratchData                                  &scratch_data,
-			     AssemblyCopyData                                     &copy_data) const;
+                             AssemblyScratchData                                  &scratch_data,
+                             AssemblyCopyData                                     &copy_data) const;
 
       void
       copy_local_to_global(const AssemblyCopyData &copy_data,
@@ -773,25 +772,25 @@ namespace Step13
     Solver<dim>::assemble_linear_system (LinearSystem &linear_system)
     {
       Threads::Task<> rhs_task = Threads::new_task (&Solver<dim>::assemble_rhs,
-						    *this,
-						    linear_system.rhs);
+                                                    *this,
+                                                    linear_system.rhs);
 
       WorkStream::run(dof_handler.begin_active(),
-		      dof_handler.end(),
-		      std_cxx1x::bind(&Solver<dim>::local_assemble_matrix,
-				      this,
-				      std_cxx1x::_1,
-				      std_cxx1x::_2,
-				      std_cxx1x::_3),
-		      std_cxx1x::bind(&Solver<dim>::copy_local_to_global,
-				      this,
-				      std_cxx1x::_1,
-				      std_cxx1x::ref(linear_system)),
-		      AssemblyScratchData(*fe, *quadrature),
-		      AssemblyCopyData());
+                      dof_handler.end(),
+                      std_cxx11::bind(&Solver<dim>::local_assemble_matrix,
+                                      this,
+                                      std_cxx11::_1,
+                                      std_cxx11::_2,
+                                      std_cxx11::_3),
+                      std_cxx11::bind(&Solver<dim>::copy_local_to_global,
+                                      this,
+                                      std_cxx11::_1,
+                                      std_cxx11::ref(linear_system)),
+                      AssemblyScratchData(*fe, *quadrature),
+                      AssemblyCopyData());
       linear_system.hanging_node_constraints.condense (linear_system.matrix);
 
-      // The syntax above using <code>std_cxx1x::bind</code> requires
+      // The syntax above using <code>std_cxx11::bind</code> requires
       // some explanation. There are multiple version of
       // WorkStream::run that expect different arguments. In step-9,
       // we used one version that took a pair of iterators, a pair of
@@ -825,7 +824,7 @@ namespace Step13
       // typical way to generate such function objects is using
       // <code>std::bind</code> (or, if the compiler is too old, a
       // replacement for it, which we generically call
-      // <code>std_cxx1x::bind</code>) which takes a pointer to a
+      // <code>std_cxx11::bind</code>) which takes a pointer to a
       // (member) function and then <i>binds</i> individual arguments
       // to fixed values. For example, you can create a function that
       // takes an iterator, a scratch object and a copy object by
@@ -841,9 +840,9 @@ namespace Step13
       // that can then be filled by WorkStream::run().
       //
       // There remains the question of what the
-      // <code>std_cxx1x::_1</code>, <code>std_cxx1x::_2</code>, etc.,
+      // <code>std_cxx11::_1</code>, <code>std_cxx11::_2</code>, etc.,
       // mean. (These arguments are called <i>placeholders</i>.) The
-      // idea of using <code>std_cxx1x::bind</code> in the first of
+      // idea of using <code>std_cxx11::bind</code> in the first of
       // the two cases above is that it produces an object that can be
       // called with three arguments. But how are the three arguments
       // the function object is being called with going to be
@@ -856,9 +855,9 @@ namespace Step13
       // games in other circumstances. Consider, for example, having a
       // function <code>void f(double x, double y)</code>. Then,
       // creating a variable <code>p</code> of type
-      // <code>std_cxx1x::function@<void f(double,double)@></code> and
-      // initializing <code>p=std_cxx1x::bind(&f, std_cxx1x::_2,
-      // std_cxx1x::_1)</code> then calling <code>p(1,2)</code> will
+      // <code>std_cxx11::function@<void f(double,double)@></code> and
+      // initializing <code>p=std_cxx11::bind(&f, std_cxx11::_2,
+      // std_cxx11::_1)</code> then calling <code>p(1,2)</code> will
       // result in calling <code>f(2,1)</code>.
       //
       // @note Once deal.II can rely on every compiler being able to
@@ -935,29 +934,29 @@ namespace Step13
     template <int dim>
     Solver<dim>::AssemblyScratchData::
     AssemblyScratchData (const FiniteElement<dim> &fe,
-			 const Quadrature<dim>    &quadrature)
-    :
-    fe_values (fe,
-	       quadrature,
-	       update_gradients | update_JxW_values)
+                         const Quadrature<dim>    &quadrature)
+      :
+      fe_values (fe,
+                 quadrature,
+                 update_gradients | update_JxW_values)
     {}
 
 
     template <int dim>
     Solver<dim>::AssemblyScratchData::
     AssemblyScratchData (const AssemblyScratchData &scratch_data)
-    :
-    fe_values (scratch_data.fe_values.get_fe(),
-	       scratch_data.fe_values.get_quadrature(),
-	       update_gradients | update_JxW_values)
+      :
+      fe_values (scratch_data.fe_values.get_fe(),
+                 scratch_data.fe_values.get_quadrature(),
+                 update_gradients | update_JxW_values)
     {}
 
 
     template <int dim>
     void
     Solver<dim>::local_assemble_matrix (const typename DoFHandler<dim>::active_cell_iterator &cell,
-					AssemblyScratchData                                  &scratch_data,
-					AssemblyCopyData                                     &copy_data) const
+                                        AssemblyScratchData                                  &scratch_data,
+                                        AssemblyCopyData                                     &copy_data) const
     {
       const unsigned int dofs_per_cell = fe->dofs_per_cell;
       const unsigned int n_q_points    = quadrature->size();
@@ -972,8 +971,8 @@ namespace Step13
         for (unsigned int i=0; i<dofs_per_cell; ++i)
           for (unsigned int j=0; j<dofs_per_cell; ++j)
             copy_data.cell_matrix(i,j) += (scratch_data.fe_values.shape_grad(i,q_point) *
-					   scratch_data.fe_values.shape_grad(j,q_point) *
-					   scratch_data.fe_values.JxW(q_point));
+                                           scratch_data.fe_values.shape_grad(j,q_point) *
+                                           scratch_data.fe_values.JxW(q_point));
 
       cell->get_dof_indices (copy_data.local_dof_indices);
     }
@@ -986,10 +985,10 @@ namespace Step13
                                       LinearSystem           &linear_system) const
     {
       for (unsigned int i=0; i<copy_data.local_dof_indices.size(); ++i)
-	for (unsigned int j=0; j<copy_data.local_dof_indices.size(); ++j)
-	  linear_system.matrix.add (copy_data.local_dof_indices[i],
-				    copy_data.local_dof_indices[j],
-				    copy_data.cell_matrix(i,j));
+        for (unsigned int j=0; j<copy_data.local_dof_indices.size(); ++j)
+          linear_system.matrix.add (copy_data.local_dof_indices[i],
+                                    copy_data.local_dof_indices[j],
+                                    copy_data.cell_matrix(i,j));
     }
 
 
@@ -1030,27 +1029,29 @@ namespace Step13
       hanging_node_constraints.clear ();
 
       void (*mhnc_p) (const DoFHandler<dim> &,
-          ConstraintMatrix &)
+                      ConstraintMatrix &)
         = &DoFTools::make_hanging_node_constraints;
 
       // Start a side task then continue on the main thread
-      Threads::Task<> side_task(std_cxx1x::bind(mhnc_p,std_cxx1x::cref(dof_handler),
-            std_cxx1x::ref(hanging_node_constraints)));
+      Threads::Task<> side_task
+        = Threads::new_task (mhnc_p,
+                             dof_handler,
+                             hanging_node_constraints);
+
+      DynamicSparsityPattern dsp(dof_handler.n_dofs(), dof_handler.n_dofs());
+      DoFTools::make_sparsity_pattern (dof_handler, dsp);
+
 
-      sparsity_pattern.reinit (dof_handler.n_dofs(),
-                               dof_handler.n_dofs(),
-                               dof_handler.max_couplings_between_dofs());
-      DoFTools::make_sparsity_pattern (dof_handler, sparsity_pattern);
 
       // Wait for the side task to be done before going further
       side_task.join();
 
       hanging_node_constraints.close ();
-      hanging_node_constraints.condense (sparsity_pattern);
+      hanging_node_constraints.condense (dsp);
+      sparsity_pattern.copy_from(dsp);
 
-      // Finally, close the sparsity pattern, initialize the matrix, and set
-      // the right hand side vector to the right size.
-      sparsity_pattern.compress();
+
+      // Finally initialize the matrix and right hand side vector
       matrix.reinit (sparsity_pattern);
       rhs.reinit (dof_handler.n_dofs());
     }
@@ -1489,7 +1490,7 @@ namespace Step13
     // Also generate an evaluator which writes out the solution:
     Evaluation::SolutionOutput<dim>
     postprocessor2 (std::string("solution-")+solver_name,
-                    DataOut<dim>::gnuplot);
+                    DataOutBase::gnuplot);
 
     // Take these two evaluation objects and put them in a list...
     std::list<Evaluation::EvaluationBase<dim> *> postprocessor_list;
@@ -1520,8 +1521,6 @@ int main ()
 {
   try
     {
-      dealii::deallog.depth_console (0);
-
       Step13::solve_problem<2> ("global");
       Step13::solve_problem<2> ("kelly");
     }
