@@ -1,7 +1,6 @@
 /* ---------------------------------------------------------------------
- * $Id: step-15.cc 30526 2013-08-29 20:06:27Z felix.gruber $
  *
- * Copyright (C) 2012 - 2013 by the deal.II authors
+ * Copyright (C) 2012 - 2015 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -31,7 +30,7 @@
 #include <deal.II/lac/vector.h>
 #include <deal.II/lac/full_matrix.h>
 #include <deal.II/lac/sparse_matrix.h>
-#include <deal.II/lac/compressed_sparsity_pattern.h>
+#include <deal.II/lac/dynamic_sparsity_pattern.h>
 #include <deal.II/lac/solver_cg.h>
 #include <deal.II/lac/precondition.h>
 #include <deal.II/lac/constraint_matrix.h>
@@ -40,7 +39,7 @@
 #include <deal.II/grid/grid_generator.h>
 #include <deal.II/grid/tria_accessor.h>
 #include <deal.II/grid/tria_iterator.h>
-#include <deal.II/grid/tria_boundary_lib.h>
+#include <deal.II/grid/manifold_lib.h>
 #include <deal.II/grid/grid_refinement.h>
 
 #include <deal.II/dofs/dof_handler.h>
@@ -209,12 +208,12 @@ namespace Step15
     newton_update.reinit (dof_handler.n_dofs());
     system_rhs.reinit (dof_handler.n_dofs());
 
-    CompressedSparsityPattern c_sparsity(dof_handler.n_dofs());
-    DoFTools::make_sparsity_pattern (dof_handler, c_sparsity);
+    DynamicSparsityPattern dsp(dof_handler.n_dofs());
+    DoFTools::make_sparsity_pattern (dof_handler, dsp);
 
-    hanging_node_constraints.condense (c_sparsity);
+    hanging_node_constraints.condense (dsp);
 
-    sparsity_pattern.copy_from(c_sparsity);
+    sparsity_pattern.copy_from(dsp);
     system_matrix.reinit (sparsity_pattern);
   }
 
@@ -294,17 +293,17 @@ namespace Step15
               {
                 for (unsigned int j=0; j<dofs_per_cell; ++j)
                   {
-                    cell_matrix(i, j) += (fe_values.shape_grad(i, q_point)
-                                          * coeff
-                                          * (fe_values.shape_grad(j, q_point)
-                                             -
-                                             coeff * coeff
+                    cell_matrix(i, j) += ( ((fe_values.shape_grad(i, q_point)
+                                             * coeff
+                                             * fe_values.shape_grad(j, q_point))
+                                            -
+                                            (fe_values.shape_grad(i, q_point)
+                                             * coeff * coeff * coeff
                                              * (fe_values.shape_grad(j, q_point)
                                                 *
                                                 old_solution_gradients[q_point])
-                                             * old_solution_gradients[q_point]
-                                            )
-                                          * fe_values.JxW(q_point));
+                                             * old_solution_gradients[q_point]))
+                                           * fe_values.JxW(q_point));
                   }
 
                 cell_rhs(i) -= (fe_values.shape_grad(i, q_point)
@@ -442,8 +441,8 @@ namespace Step15
     // compute again. To ensure there are no hanging nodes of the old mesh in
     // the object, it's first cleared.  To be on the safe side, we then also
     // make sure that the current solution's vector entries satisfy the
-    // hanging node constraints:
-
+    // hanging node constraints (see the discussion in the documentation of
+    // the SolutionTransfer class for why this is necessary):
     hanging_node_constraints.clear();
 
     DoFTools::make_hanging_node_constraints(dof_handler,
@@ -550,9 +549,9 @@ namespace Step15
 
             for (unsigned int i = 0; i < dofs_per_cell; ++i)
               cell_residual(i) -= (fe_values.shape_grad(i, q_point)
-                              * coeff
-                              * gradients[q_point]
-                              * fe_values.JxW(q_point));
+                                   * coeff
+                                   * gradients[q_point]
+                                   * fe_values.JxW(q_point));
           }
 
         cell->get_dof_indices (local_dof_indices);
@@ -628,8 +627,9 @@ namespace Step15
     // the origin, created in the same way as shown in step-6. The mesh is
     // globally refined twice followed later on by several adaptive cycles:
     GridGenerator::hyper_ball (triangulation);
-    static const HyperBallBoundary<dim> boundary;
-    triangulation.set_boundary (0, boundary);
+    static const SphericalManifold<dim> boundary;
+    triangulation.set_all_manifold_ids_on_boundary(0);
+    triangulation.set_manifold (0, boundary);
     triangulation.refine_global(2);
 
     // The Newton iteration starts next. During the first step we do not have
@@ -639,8 +639,9 @@ namespace Step15
     //
     // At the beginning of the loop, we do a bit of setup work. In the first
     // go around, we compute the solution on the twice globally refined mesh
-    // after setting up the basic data structures. In all following mesh
-    // refinement loops, the mesh will be refined adaptively.
+    // after setting up the basic data structures and ensuring that the first
+    // Newton iterate already has the correct boundary values. In all
+    // following mesh refinement loops, the mesh will be refined adaptively.
     double previous_res = 0;
     while (first_step || (previous_res>1e-3))
       {
@@ -652,6 +653,8 @@ namespace Step15
 
             setup_system (true);
             set_boundary_values ();
+
+            first_step = false;
           }
         else
           {
@@ -683,7 +686,6 @@ namespace Step15
 
             solve ();
 
-            first_step = false;
             std::cout << "  Residual: "
                       << compute_residual(0)
                       << std::endl;
@@ -719,8 +721,6 @@ int main ()
       using namespace dealii;
       using namespace Step15;
 
-      deallog.depth_console (0);
-
       MinimalSurfaceProblem<2> laplace_problem_2d;
       laplace_problem_2d.run ();
     }
