@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2005 - 2015 by the deal.II authors
+ * Copyright (C) 2005 - 2016 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -32,10 +32,6 @@
 #include <deal.II/lac/block_sparse_matrix.h>
 #include <deal.II/lac/solver_cg.h>
 #include <deal.II/lac/precondition.h>
-// For our Schur complement solver, we need two new objects. One is a matrix
-// object which acts as the inverse of a matrix by calling an iterative
-// solver.
-#include <deal.II/lac/iterative_inverse.h>
 
 #include <deal.II/grid/tria.h>
 #include <deal.II/grid/grid_generator.h>
@@ -515,12 +511,12 @@ namespace Step20
                               fe_values.JxW(q);
             }
 
-        for (unsigned int face_no=0;
-             face_no<GeometryInfo<dim>::faces_per_cell;
-             ++face_no)
-          if (cell->at_boundary(face_no))
+        for (unsigned int face_n=0;
+             face_n<GeometryInfo<dim>::faces_per_cell;
+             ++face_n)
+          if (cell->at_boundary(face_n))
             {
-              fe_face_values.reinit (cell, face_no);
+              fe_face_values.reinit (cell, face_n);
 
               pressure_boundary_values
               .value_list (fe_face_values.get_quadrature_points(),
@@ -560,15 +556,64 @@ namespace Step20
   // therefore not discuss the rationale for these classes here any more, but
   // rather only comment on implementational aspects.
 
+  // @sect4{The <code>InverseMatrix</code> class template}
 
-  // @sect4{The <code>SchurComplement</code> class template}
+  // There are a few places in this program where we will need either the
+  // action of the inverse of the mass matrix or the action of the inverse of
+  // the approximate Schur complement. Rather than explicitly calling
+  // SolverCG::solve every time that we need to solve such a system, we will
+  // wrap the action of either inverse in a simple class. The only things we
+  // would like to note are that this class is derived from
+  // <code>Subscriptor</code> and, as mentioned above, it stores a pointer to
+  // the underlying matrix with a <code>SmartPointer</code> object. This class
+  // also appears in step-21 and a more advanced version of it appears in
+  // step-22.
+  template <class MatrixType>
+  class InverseMatrix : public Subscriptor
+  {
+  public:
+    InverseMatrix(const MatrixType &m);
+
+    void vmult(Vector<double>       &dst,
+               const Vector<double> &src) const;
+
+  private:
+    const SmartPointer<const MatrixType> matrix;
+  };
+
+
+  template <class MatrixType>
+  InverseMatrix<MatrixType>::InverseMatrix (const MatrixType &m)
+    :
+    matrix (&m)
+  {}
+
+
+  template <class MatrixType>
+  void InverseMatrix<MatrixType>::vmult (Vector<double>       &dst,
+                                         const Vector<double> &src) const
+  {
+    // To make the control flow simpler, we recreate both the ReductionControl
+    // and SolverCG objects every time this is called. This is not the most
+    // efficient choice because SolverCG instances allocate memory whenever
+    // they are created; this is just a tutorial so such inefficiencies are
+    // acceptable for the sake of exposition.
+    SolverControl solver_control (std::max(src.size(), static_cast<std::size_t> (200)),
+                                  1e-8*src.l2_norm());
+    SolverCG<>    cg (solver_control);
+
+    dst = 0;
+
+    cg.solve (*matrix, dst, src, PreconditionIdentity());
+  }
+
+
+  // @sect4{The <code>SchurComplement</code> class}
 
   // The next class is the Schur complement class. Its rationale has also been
-  // discussed in length in the introduction. The only things we would like to
-  // note is that the class, too, is derived from the <code>Subscriptor</code>
-  // class and that as mentioned above it stores pointers to the entire block
-  // matrix and the inverse of the mass matrix block using
-  // <code>SmartPointer</code> objects.
+  // discussed in length in the introduction. Like <code>InverseMatrix</code>,
+  // this class is derived from Subscriptor and stores SmartPointer&nbsp;s
+  // pointing to the system matrix and <code>InverseMatrix</code> wrapper.
   //
   // The <code>vmult</code> function requires two temporary vectors that we do
   // not want to re-allocate and free every time we call this function. Since
@@ -587,22 +632,23 @@ namespace Step20
   class SchurComplement : public Subscriptor
   {
   public:
-    SchurComplement (const BlockSparseMatrix<double> &A,
-                     const IterativeInverse<Vector<double> > &Minv);
+    SchurComplement (const BlockSparseMatrix<double>            &A,
+                     const InverseMatrix<SparseMatrix<double> > &Minv);
 
     void vmult (Vector<double>       &dst,
                 const Vector<double> &src) const;
 
   private:
     const SmartPointer<const BlockSparseMatrix<double> > system_matrix;
-    const SmartPointer<const IterativeInverse<Vector<double> > > m_inverse;
+    const SmartPointer<const InverseMatrix<SparseMatrix<double> > > m_inverse;
 
     mutable Vector<double> tmp1, tmp2;
   };
 
 
-  SchurComplement::SchurComplement (const BlockSparseMatrix<double> &A,
-                                    const IterativeInverse<Vector<double> > &Minv)
+  SchurComplement
+  ::SchurComplement (const BlockSparseMatrix<double>            &A,
+                     const InverseMatrix<SparseMatrix<double> > &Minv)
     :
     system_matrix (&A),
     m_inverse (&Minv),
@@ -620,32 +666,12 @@ namespace Step20
   }
 
 
-  // @sect4{The <code>ApproximateSchurComplement</code> class template}
+  // @sect4{The <code>ApproximateSchurComplement</code> class}
 
   // The third component of our solver and preconditioner system is the class
-  // that approximates the Schur complement so we can form a an InverseIterate
-  // object that approximates the inverse of the Schur complement. It follows
-  // the same pattern as the Schur complement class, with the only exception
-  // that we do not multiply with the inverse mass matrix in
-  // <code>vmult</code>, but rather just do a single Jacobi
-  // step. Consequently, the class also does not have to store a pointer to an
-  // inverse mass matrix object.
-  //
-  // We will later use this class as a template argument to the
-  // IterativeInverse class which will in turn want to use it as a
-  // template argument for the PointerMatrix class. The latter class
-  // has a function that requires us to also write a function that
-  // provides the product with the transpose of the matrix this object
-  // represents. As a consequence, in the code below, we also
-  // implement a <tt>Tvmult</tt> function here that represents the
-  // product of the transpose matrix with a vector. It is easy to see
-  // how this needs to be implemented here: since the matrix is
-  // symmetric, we can as well call <code>vmult</code> wherever the
-  // product with the transpose matrix is required. (Note, however,
-  // that even though we implement this function here, there will in
-  // fact not be any need for it as long as we use SolverCG as the
-  // solver since that solver does not ever call the function that
-  // provides this operation.)
+  // that approximates the Schur complement with the method described in the
+  // introduction. We will use this class to build a preconditioner for our
+  // system matrix.
   class ApproximateSchurComplement : public Subscriptor
   {
   public:
@@ -653,8 +679,6 @@ namespace Step20
 
     void vmult (Vector<double>       &dst,
                 const Vector<double> &src) const;
-    void Tvmult (Vector<double>       &dst,
-                 const Vector<double> &src) const;
 
   private:
     const SmartPointer<const BlockSparseMatrix<double> > system_matrix;
@@ -663,85 +687,58 @@ namespace Step20
   };
 
 
-  ApproximateSchurComplement::ApproximateSchurComplement (const BlockSparseMatrix<double> &A)
-    :
+
+  ApproximateSchurComplement::ApproximateSchurComplement
+  (const BlockSparseMatrix<double> &A) :
     system_matrix (&A),
     tmp1 (A.block(0,0).m()),
     tmp2 (A.block(0,0).m())
   {}
 
 
-  void ApproximateSchurComplement::vmult (Vector<double>       &dst,
-                                          const Vector<double> &src) const
+  void
+  ApproximateSchurComplement::vmult
+  (Vector<double>       &dst,
+   const Vector<double> &src) const
   {
     system_matrix->block(0,1).vmult (tmp1, src);
     system_matrix->block(0,0).precondition_Jacobi (tmp2, tmp1);
     system_matrix->block(1,0).vmult (dst, tmp2);
   }
 
-
-  void ApproximateSchurComplement::Tvmult (Vector<double>       &dst,
-                                           const Vector<double> &src) const
-  {
-    vmult (dst, src);
-  }
-
-
-
   // @sect4{MixedLaplace::solve}
 
   // After all these preparations, we can finally write the function that
   // actually solves the linear problem. We will go through the two parts it
   // has that each solve one of the two equations, the first one for the
   // pressure (component 1 of the solution), then the velocities (component 0
-  // of the solution). Both parts need an object representing the inverse mass
-  // matrix and an auxiliary vector, and we therefore declare these objects at
-  // the beginning of this function.
+  // of the solution).
   template <int dim>
   void MixedLaplaceProblem<dim>::solve ()
   {
-    PreconditionIdentity identity;
-    IterativeInverse<Vector<double> > m_inverse;
-    m_inverse.initialize(system_matrix.block(0,0), identity);
-    m_inverse.solver.select("cg");
-    static ReductionControl inner_control(1000, 0., 1.e-13);
-    m_inverse.solver.set_control(inner_control);
-
+    InverseMatrix<SparseMatrix<double> > inverse_mass (system_matrix.block(0,0));
     Vector<double> tmp (solution.block(0).size());
 
     // Now on to the first equation. The right hand side of it is $B^TM^{-1}F-G$,
-    // which is what we compute in the first few lines. We then declare the
-    // objects representing the Schur complement, its approximation, and the
-    // inverse of the approximation. Finally, we declare a solver object and
-    // hand off all these matrices and vectors to it to compute block 1 (the
-    // pressure) of the solution:
+    // which is what we compute in the first few lines:
     {
+      SchurComplement schur_complement (system_matrix, inverse_mass);
       Vector<double> schur_rhs (solution.block(1).size());
-
-      m_inverse.vmult (tmp, system_rhs.block(0));
+      inverse_mass.vmult (tmp, system_rhs.block(0));
       system_matrix.block(1,0).vmult (schur_rhs, tmp);
       schur_rhs -= system_rhs.block(1);
 
-
-      SchurComplement
-      schur_complement (system_matrix, m_inverse);
-
-      ApproximateSchurComplement
-      approximate_schur_complement (system_matrix);
-
-      IterativeInverse<Vector<double> >
-      preconditioner;
-      preconditioner.initialize(approximate_schur_complement, identity);
-      preconditioner.solver.select("cg");
-      preconditioner.solver.set_control(inner_control);
-
-
+      // Now that we have the right hand side we can go ahead and solve for the
+      // pressure, using our approximation of the inverse as a preconditioner:
       SolverControl solver_control (solution.block(1).size(),
                                     1e-12*schur_rhs.l2_norm());
-      SolverCG<>    cg (solver_control);
+      SolverCG<> cg (solver_control);
 
+      ApproximateSchurComplement approximate_schur (system_matrix);
+      InverseMatrix<ApproximateSchurComplement> approximate_inverse
+      (approximate_schur);
       cg.solve (schur_complement, solution.block(1), schur_rhs,
-                preconditioner);
+                approximate_inverse);
 
       std::cout << solver_control.last_step()
                 << " CG Schur complement iterations to obtain convergence."
@@ -757,7 +754,7 @@ namespace Step20
       tmp *= -1;
       tmp += system_rhs.block(0);
 
-      m_inverse.vmult (solution.block(0), tmp);
+      inverse_mass.vmult (solution.block(0), tmp);
     }
   }
 
@@ -780,9 +777,9 @@ namespace Step20
   // frequently the case in mixed finite element applications). What we
   // therefore have to do is to `mask' the components that we are interested
   // in. This is easily done: the
-  // <code>VectorTools::integrate_difference</code> function takes as its last
-  // argument a pointer to a weight function (the parameter defaults to the
-  // null pointer, meaning unit weights). What we simply have to do is to pass
+  // <code>VectorTools::integrate_difference</code> function takes as one of its
+  // arguments a pointer to a weight function (the parameter defaults to the
+  // null pointer, meaning unit weights). What we have to do is to pass
   // a function object that equals one in the components we are interested in,
   // and zero in the other ones. For example, to compute the pressure error,
   // we should pass a function that represents the constant vector with a unit
@@ -832,13 +829,17 @@ namespace Step20
                                        cellwise_errors, quadrature,
                                        VectorTools::L2_norm,
                                        &pressure_mask);
-    const double p_l2_error = cellwise_errors.l2_norm();
+    const double p_l2_error = VectorTools::compute_global_error(triangulation,
+                                                                cellwise_errors,
+                                                                VectorTools::L2_norm);
 
     VectorTools::integrate_difference (dof_handler, solution, exact_solution,
                                        cellwise_errors, quadrature,
                                        VectorTools::L2_norm,
                                        &velocity_mask);
-    const double u_l2_error = cellwise_errors.l2_norm();
+    const double u_l2_error = VectorTools::compute_global_error(triangulation,
+                                                                cellwise_errors,
+                                                                VectorTools::L2_norm);
 
     std::cout << "Errors: ||e_p||_L2 = " << p_l2_error
               << ",   ||e_u||_L2 = " << u_l2_error
@@ -849,59 +850,36 @@ namespace Step20
   // @sect4{MixedLaplace::output_results}
 
   // The last interesting function is the one in which we generate graphical
-  // output. Everything here looks obvious and familiar. Note how we construct
-  // unique names for all the solution variables at the beginning, like we did
-  // in step-8 and other programs later on. The only thing worth mentioning is
-  // that for higher order elements, in seems inappropriate to only show a
-  // single bilinear quadrilateral per cell in the graphical output. We
-  // therefore generate patches of size (degree+1)x(degree+1) to capture the
-  // full information content of the solution. See the step-7 tutorial program
-  // for more information on this.
-  //
-  // Note that we output the <code>dim+1</code> components of the solution
-  // vector as a collection of individual scalars here. Most visualization
-  // programs will then only offer to visualize them individually, rather than
-  // allowing us to plot the flow field as a vector field. However, as
-  // explained in the corresponding function of step-22 or the @ref VVOutput
-  // "Generating graphical output" section of the @ref vector_valued module,
-  // instructing the DataOut class to identify components of the FESystem
-  // object as elements of a <code>dim</code>-dimensional vector is not
-  // actually very difficult and will then allow us to show results as vector
-  // plots. We skip this here for simplicity and refer to the links above for
-  // more information.
+  // output. Note that all velocity components get the same solution name
+  // "u". Together with using
+  // DataComponentInterpretation::::component_is_part_of_vector this will
+  // cause DataOut<dim>::write_vtu() to generate a vector representation of
+  // the individual velocity components, see step-22 or the
+  // @ref VVOutput "Generating graphical output"
+  // section of the
+  // @ref vector_valued
+  // module for more information. Finally, it seems inappropriate for higher
+  // order elements to only show a single bilinear quadrilateral per cell in
+  // the graphical output. We therefore generate patches of size
+  // (degree+1)x(degree+1) to capture the full information content of the
+  // solution. See the step-7 tutorial program for more information on this.
   template <int dim>
   void MixedLaplaceProblem<dim>::output_results () const
   {
-    std::vector<std::string> solution_names;
-    switch (dim)
-      {
-      case 2:
-        solution_names.push_back ("u");
-        solution_names.push_back ("v");
-        solution_names.push_back ("p");
-        break;
-
-      case 3:
-        solution_names.push_back ("u");
-        solution_names.push_back ("v");
-        solution_names.push_back ("w");
-        solution_names.push_back ("p");
-        break;
-
-      default:
-        Assert (false, ExcNotImplemented());
-      }
-
+    std::vector<std::string> solution_names(dim, "u");
+    solution_names.push_back ("p");
+    std::vector<DataComponentInterpretation::DataComponentInterpretation>
+    interpretation (dim,
+                    DataComponentInterpretation::component_is_part_of_vector);
+    interpretation.push_back (DataComponentInterpretation::component_is_scalar);
 
     DataOut<dim> data_out;
-
-    data_out.attach_dof_handler (dof_handler);
-    data_out.add_data_vector (solution, solution_names);
+    data_out.add_data_vector (dof_handler, solution, solution_names, interpretation);
 
     data_out.build_patches (degree+1);
 
-    std::ofstream output ("solution.gmv");
-    data_out.write_gmv (output);
+    std::ofstream output ("solution.vtu");
+    data_out.write_vtu (output);
   }
 
 
