@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2006 - 2015 by the deal.II authors
+ * Copyright (C) 2006 - 2016 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -193,15 +193,15 @@ namespace Step25
 
       case 3:
       {
-        double theta  = numbers::PI/4;
-        double phi = numbers::PI/4;
-        double tau = 1.;
-        double c0  = 1.;
-        double s   = 1.;
-        double arg = p[0]*std::cos(theta) +
-                     p[1]*std::sin(theta) * std::cos(phi) +
-                     std::sin(theta) * std::sin(phi) *
-                     (p[2]*std::cosh(tau)+t*std::sinh(tau));
+        const double theta  = numbers::PI/4;
+        const double phi = numbers::PI/4;
+        const double tau = 1.;
+        const double c0  = 1.;
+        const double s   = 1.;
+        const double arg = p[0]*std::cos(theta) +
+                           p[1]*std::sin(theta) * std::cos(phi) +
+                           std::sin(theta) * std::sin(phi) *
+                           (p[2]*std::cosh(tau)+t*std::sinh(tau));
         return 4.*std::atan(c0*std::exp(s*arg));
       }
 
@@ -353,51 +353,36 @@ namespace Step25
 
     SparseMatrix<double> tmp_matrix (sparsity_pattern);
     compute_nl_matrix (old_solution, solution, tmp_matrix);
-    system_matrix.add (-std::pow(time_step*theta,2), tmp_matrix);
+    system_matrix.add (std::pow(time_step*theta,2), tmp_matrix);
 
-    // Then, we compute the right-hand side vector $-F_h(U^{n,l})$.
-    //
-    // We have to first build up the matrix
-    // $M+k^2\theta^2 A$, which we put into <code>tmp_matrix</code>
-    // use it to compute a contribution to the right hand side vector, and
-    // then build the matrix $M-k^2\theta(1-\theta) A$. We could
-    // build it in the same way as before, i.e., using code like
-    // @code
-    // tmp_matrix.copy_from (mass_matrix);
-    // tmp_matrix.add (-std::pow(time_step,2)*theta*(1-theta), laplace_matrix);
-    // @endcode
-    // but we can save the expense of the <code>copy_from</code> operation
-    // by starting from what is already in the <code>tmp_matrix</code>
-    // variable (i.e., $M+k^2\theta^2 A$) and subtracting from this
-    // $k^2\theta^2 A+k^2\theta(1-\theta) A=k^2\theta A$ when computing the
-    // second matrix:
-    system_rhs = 0;
-
-    tmp_matrix.copy_from (mass_matrix);
-    tmp_matrix.add (std::pow(time_step*theta,2), laplace_matrix);
+    // Next we compute the right-hand side vector. This is just the
+    // combination of matrix-vector products implied by the description of
+    // $-F_h(U^{n,l})$ in the introduction.
+    system_rhs = 0.;
 
     Vector<double> tmp_vector (solution.size());
-    tmp_matrix.vmult (tmp_vector, solution);
-    system_rhs += tmp_vector;
 
+    mass_matrix.vmult (system_rhs, solution);
+    laplace_matrix.vmult (tmp_vector, solution);
+    system_rhs.add (std::pow(time_step*theta, 2), tmp_vector);
 
-    tmp_matrix.add(-std::pow(time_step, 2) * theta, laplace_matrix);
-
-    tmp_matrix.vmult (tmp_vector, old_solution);
-    system_rhs -= tmp_vector;
+    mass_matrix.vmult (tmp_vector, old_solution);
+    system_rhs.add (-1.0, tmp_vector);
+    laplace_matrix.vmult (tmp_vector, old_solution);
+    system_rhs.add (std::pow(time_step,2)*theta*(1 - theta), tmp_vector);
 
     system_rhs.add (-time_step, M_x_velocity);
 
     compute_nl_term (old_solution, solution, tmp_vector);
     system_rhs.add (std::pow(time_step,2)*theta, tmp_vector);
 
-    system_rhs *= -1;
+    system_rhs *= -1.;
   }
 
   // @sect4{SineGordonProblem::compute_nl_term}
 
   // This function computes the vector $S(\cdot,\cdot)$, which appears in the
-  // nonlinear term in the both equations of the split formulation. This
+  // nonlinear term in both equations of the split formulation. This
   // function not only simplifies the repeated computation of this term, but
   // it is also a fundamental part of the nonlinear iterative solver that we
   // use when the time stepping is implicit (i.e. $\theta\ne 0$). Moreover, we
