@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2013 - 2015 by the deal.II authors
+ * Copyright (C) 2013 - 2017 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -145,7 +145,8 @@ namespace Step26
   double RightHandSide<dim>::value (const Point<dim> &p,
                                     const unsigned int component) const
   {
-    Assert (component == 0, ExcInternalError());
+    (void) component;
+    Assert (component == 0, ExcIndexRange(component, 0, 1));
     Assert (dim == 2, ExcNotImplemented());
 
     const double time = this->get_time();
@@ -185,7 +186,8 @@ namespace Step26
   double BoundaryValues<dim>::value (const Point<dim> &/*p*/,
                                      const unsigned int component) const
   {
-    Assert(component == 0, ExcInternalError());
+    (void) component;
+    Assert (component == 0, ExcIndexRange(component, 0, 1));
     return 0;
   }
 
@@ -204,7 +206,9 @@ namespace Step26
     :
     fe(1),
     dof_handler(triangulation),
+    time (0.0),
     time_step(1. / 500),
+    timestep_number (0),
     theta(0.5)
   {}
 
@@ -463,9 +467,6 @@ start_time_iteration:
                              old_solution);
     solution = old_solution;
 
-    timestep_number = 0;
-    time            = 0;
-
     output_results();
 
     // Then we start the main loop until the computed time exceeds our
