@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2009 - 2015 by the deal.II authors
+ * Copyright (C) 2009 - 2017 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -74,6 +74,10 @@
 // file:
 #include <deal.II/grid/grid_tools.h>
 
+// We use a little utility class from boost to save the state of an output
+// stream (see the <code>run</code> function below):
+#include <boost/io/ios_state.hpp>
+
 // Here are two more C++ standard headers that we use to define list data
 // types as well as to fine-tune the output we generate:
 #include <list>
@@ -168,7 +172,7 @@ namespace Step28
     sigma_s (n_materials, n_groups, n_groups),
     chi (n_materials, n_groups)
   {
-    switch (n_groups)
+    switch (this->n_groups)
       {
       case 2:
       {
@@ -1133,34 +1137,20 @@ namespace Step28
   void
   EnergyGroup<dim>::output_results (const unsigned int cycle) const
   {
-    {
-      const std::string filename = std::string("grid-") +
-                                   Utilities::int_to_string(group,1) +
-                                   "." +
-                                   Utilities::int_to_string(cycle,1) +
-                                   ".eps";
-      std::ofstream output (filename.c_str());
+    const std::string filename = std::string("solution-") +
+                                 Utilities::int_to_string(group, 2) +
+                                 "." +
+                                 Utilities::int_to_string(cycle, 2) +
+                                 ".vtu";
 
-      GridOut grid_out;
-      grid_out.write_eps (triangulation, output);
-    }
+    DataOut<dim> data_out;
 
-    {
-      const std::string filename = std::string("solution-") +
-                                   Utilities::int_to_string(group,1) +
-                                   "." +
-                                   Utilities::int_to_string(cycle,1) +
-                                   ".gmv";
+    data_out.attach_dof_handler (dof_handler);
+    data_out.add_data_vector (solution, "solution");
+    data_out.build_patches ();
 
-      DataOut<dim> data_out;
-
-      data_out.attach_dof_handler (dof_handler);
-      data_out.add_data_vector (solution, "solution");
-      data_out.build_patches ();
-
-      std::ofstream output (filename.c_str());
-      data_out.write_gmv (output);
-    }
+    std::ofstream output (filename.c_str());
+    data_out.write_vtu (output);
   }
 
 
@@ -1322,7 +1312,8 @@ namespace Step28
     :
     parameters (parameters),
     material_data (parameters.n_groups),
-    fe (parameters.fe_degree)
+    fe (parameters.fe_degree),
+    k_eff (std::numeric_limits<double>::quiet_NaN())
   {}
 
 
@@ -1637,6 +1628,14 @@ namespace Step28
   template <int dim>
   void NeutronDiffusionProblem<dim>::run ()
   {
+    // We would like to change the output precision for just this function and
+    // restore the state of <code>std::cout</code> when this function returns.
+    // Hence, we need a way to undo the output format change. Boost provides a
+    // convenient way to save the state of an output stream and restore it at
+    // the end of the current block (when the destructor of
+    // <code>restore_flags</code> is called) with the
+    // <code>ios_flags_saver</code> class, which we use here.
+    boost::io::ios_flags_saver restore_flags(std::cout);
     std::cout << std::setprecision (12) << std::fixed;
 
     double k_eff_old = k_eff;
@@ -1770,7 +1769,7 @@ int main (int argc, char **argv)
       NeutronDiffusionProblem<dim>::Parameters parameters;
       parameters.declare_parameters (parameter_handler);
 
-      parameter_handler.read_input (filename);
+      parameter_handler.parse_input (filename);
 
       parameters.get_parameters (parameter_handler);
 
