@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2009 - 2015 by the deal.II authors
+ * Copyright (C) 2009 - 2017 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -321,7 +321,12 @@ namespace Step34
     fe(fe_degree),
     dh(tria),
     mapping(mapping_degree, true),
-    wind(dim)
+    wind(dim),
+    singular_quadrature_order(5),
+    n_cycles(4),
+    external_refinement(5),
+    run_in_this_dimension(true),
+    extend_solution(true)
   {}
 
 
@@ -417,7 +422,7 @@ namespace Step34
     // After declaring all these parameters to the ParameterHandler object,
     // let's read an input file that will give the parameters their values. We
     // then proceed to extract these values from the ParameterHandler object:
-    prm.read_input(filename);
+    prm.parse_input(filename);
 
     n_cycles = prm.get_integer("Number of cycles");
     external_refinement = prm.get_integer("External refinement");
@@ -768,8 +773,9 @@ namespace Step34
                                        difference_per_cell,
                                        QGauss<(dim-1)>(2*fe.degree+1),
                                        VectorTools::L2_norm);
-    const double L2_error = difference_per_cell.l2_norm();
-
+    const double L2_error = VectorTools::compute_global_error(tria,
+                                                              difference_per_cell,
+                                                              VectorTools::L2_norm);
 
     // The error in the alpha vector can be computed directly using the
     // Vector::linfty_norm() function, since on each node, the value should be
