@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2009 - 2015 by the deal.II authors
+ * Copyright (C) 2009 - 2017 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -98,7 +98,7 @@ namespace Step35
     {
     public:
       Data_Storage();
-      ~Data_Storage();
+
       void read_data (const char *filename);
       MethodFormulation form;
       double initial_time,
@@ -122,7 +122,22 @@ namespace Step35
     // In the constructor of this class we declare all the parameters. The
     // details of how this works have been discussed elsewhere, for example in
     // step-19 and step-29.
-    Data_Storage::Data_Storage()
+    Data_Storage::Data_Storage() :
+      form (METHOD_ROTATIONAL),
+      initial_time (0.),
+      final_time (1.),
+      Reynolds (1.),
+      dt (5e-4),
+      n_global_refines (0),
+      pressure_degree (1),
+      vel_max_iterations (1000),
+      vel_Krylov_size (30),
+      vel_off_diagonals (60),
+      vel_update_prec (15),
+      vel_eps (1e-12),
+      vel_diag_strength (0.01),
+      verbose (true),
+      output_interval (15)
     {
       prm.declare_entry ("Method_Form", "rotational",
                          Patterns::Selection ("rotational|standard"),
@@ -199,17 +214,12 @@ namespace Step35
 
 
 
-    Data_Storage::~Data_Storage()
-    {}
-
-
-
     void Data_Storage::read_data (const char *filename)
     {
       std::ifstream file (filename);
       AssertThrow (file, ExcFileNotOpen (filename));
 
-      prm.read_input (file);
+      prm.parse_input (file);
 
       if (prm.get ("Method_Form") == std::string ("rotational"))
         form = METHOD_ROTATIONAL;
@@ -888,11 +898,11 @@ namespace Step35
                                  InitGradScratchData &scratch,
                                  InitGradPerTaskData &data)
   {
-    scratch.fe_val_vel.reinit (std_cxx11::get<0> (SI.iterators));
-    scratch.fe_val_pres.reinit (std_cxx11::get<1> (SI.iterators));
+    scratch.fe_val_vel.reinit (std_cxx11::get<0> (*SI));
+    scratch.fe_val_pres.reinit (std_cxx11::get<1> (*SI));
 
-    std_cxx11::get<0> (SI.iterators)->get_dof_indices (data.vel_local_dof_indices);
-    std_cxx11::get<1> (SI.iterators)->get_dof_indices (data.pres_local_dof_indices);
+    std_cxx11::get<0> (*SI)->get_dof_indices (data.vel_local_dof_indices);
+    std_cxx11::get<1> (*SI)->get_dof_indices (data.pres_local_dof_indices);
 
     data.local_grad = 0.;
     for (unsigned int q=0; q<scratch.nqp; ++q)
