@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2009 - 2015 by the deal.II authors
+ * Copyright (C) 2009 - 2016 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -135,7 +135,7 @@ namespace Step36
                               Patterns::Anything(),
                               "A functional description of the potential.");
 
-    parameters.read_input (prm_file);
+    parameters.parse_input (prm_file);
   }
 
 
