@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2010 - 2015 by the deal.II authors
+ * Copyright (C) 2010 - 2017 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -361,12 +361,12 @@ namespace Step43
   // darcy_matrix to match our problem.
   namespace LinearSolvers
   {
-    template <class Matrix, class Preconditioner>
+    template <class MatrixType, class PreconditionerType>
     class InverseMatrix : public Subscriptor
     {
     public:
-      InverseMatrix (const Matrix         &m,
-                     const Preconditioner &preconditioner);
+      InverseMatrix (const MatrixType         &m,
+                     const PreconditionerType &preconditioner);
 
 
       template <typename VectorType>
@@ -374,15 +374,15 @@ namespace Step43
                   const VectorType &src) const;
 
     private:
-      const SmartPointer<const Matrix> matrix;
-      const Preconditioner &preconditioner;
+      const SmartPointer<const MatrixType> matrix;
+      const PreconditionerType &preconditioner;
     };
 
 
-    template <class Matrix, class Preconditioner>
-    InverseMatrix<Matrix,Preconditioner>::
-    InverseMatrix (const Matrix &m,
-                   const Preconditioner &preconditioner)
+    template <class MatrixType, class PreconditionerType>
+    InverseMatrix<MatrixType,PreconditionerType>::InverseMatrix
+    (const MatrixType         &m,
+     const PreconditionerType &preconditioner)
       :
       matrix (&m),
       preconditioner (preconditioner)
@@ -390,12 +390,12 @@ namespace Step43
 
 
 
-    template <class Matrix, class Preconditioner>
+    template <class MatrixType, class PreconditionerType>
     template <typename VectorType>
     void
-    InverseMatrix<Matrix,Preconditioner>::
-    vmult (VectorType       &dst,
-           const VectorType &src) const
+    InverseMatrix<MatrixType,PreconditionerType>::vmult
+    (VectorType       &dst,
+     const VectorType &src) const
     {
       SolverControl solver_control (src.size(), 1e-7*src.l2_norm());
       SolverCG<VectorType> cg (solver_control);
@@ -412,15 +412,15 @@ namespace Step43
         }
     }
 
-    template <class PreconditionerA, class PreconditionerMp>
+    template <class PreconditionerTypeA, class PreconditionerTypeMp>
     class BlockSchurPreconditioner : public Subscriptor
     {
     public:
       BlockSchurPreconditioner (
-        const TrilinosWrappers::BlockSparseMatrix     &S,
+        const TrilinosWrappers::BlockSparseMatrix &S,
         const InverseMatrix<TrilinosWrappers::SparseMatrix,
-        PreconditionerMp>         &Mpinv,
-        const PreconditionerA                         &Apreconditioner);
+        PreconditionerTypeMp>                     &Mpinv,
+        const PreconditionerTypeA                 &Apreconditioner);
 
       void vmult (TrilinosWrappers::MPI::BlockVector       &dst,
                   const TrilinosWrappers::MPI::BlockVector &src) const;
@@ -428,20 +428,20 @@ namespace Step43
     private:
       const SmartPointer<const TrilinosWrappers::BlockSparseMatrix> darcy_matrix;
       const SmartPointer<const InverseMatrix<TrilinosWrappers::SparseMatrix,
-            PreconditionerMp > > m_inverse;
-      const PreconditionerA &a_preconditioner;
+            PreconditionerTypeMp > > m_inverse;
+      const PreconditionerTypeA &a_preconditioner;
 
       mutable TrilinosWrappers::MPI::Vector tmp;
     };
 
 
 
-    template <class PreconditionerA, class PreconditionerMp>
-    BlockSchurPreconditioner<PreconditionerA, PreconditionerMp>::
-    BlockSchurPreconditioner(const TrilinosWrappers::BlockSparseMatrix  &S,
+    template <class PreconditionerTypeA, class PreconditionerTypeMp>
+    BlockSchurPreconditioner<PreconditionerTypeA, PreconditionerTypeMp>::
+    BlockSchurPreconditioner(const TrilinosWrappers::BlockSparseMatrix &S,
                              const InverseMatrix<TrilinosWrappers::SparseMatrix,
-                             PreconditionerMp>      &Mpinv,
-                             const PreconditionerA                      &Apreconditioner)
+                             PreconditionerTypeMp>                     &Mpinv,
+                             const PreconditionerTypeA                 &Apreconditioner)
       :
       darcy_matrix            (&S),
       m_inverse               (&Mpinv),
@@ -450,8 +450,8 @@ namespace Step43
     {}
 
 
-    template <class PreconditionerA, class PreconditionerMp>
-    void BlockSchurPreconditioner<PreconditionerA, PreconditionerMp>::vmult (
+    template <class PreconditionerTypeA, class PreconditionerTypeMp>
+    void BlockSchurPreconditioner<PreconditionerTypeA, PreconditionerTypeMp>::vmult (
       TrilinosWrappers::MPI::BlockVector       &dst,
       const TrilinosWrappers::MPI::BlockVector &src) const
     {
@@ -624,7 +624,7 @@ namespace Step43
   TwoPhaseFlowProblem<dim>::TwoPhaseFlowProblem (const unsigned int degree)
     :
     triangulation (Triangulation<dim>::maximum_smoothing),
-
+    global_Omega_diameter (std::numeric_limits<double>::quiet_NaN()),
     degree (degree),
     darcy_degree (degree),
     darcy_fe (FE_Q<dim>(darcy_degree+1), dim,
@@ -645,6 +645,7 @@ namespace Step43
 
     time_step (0),
     old_time_step (0),
+    timestep_number (0),
     viscosity (0.2),
     porosity (1.0),
     AOS_threshold (3.0),
@@ -1743,6 +1744,10 @@ namespace Step43
       old_saturation_solution = tmp_saturation[1];
       saturation_matching_last_computed_darcy_solution = tmp_saturation[2];
 
+      saturation_constraints.distribute(saturation_solution);
+      saturation_constraints.distribute(old_saturation_solution);
+      saturation_constraints.distribute(saturation_matching_last_computed_darcy_solution);
+
       std::vector<TrilinosWrappers::MPI::BlockVector> tmp_darcy (2);
       tmp_darcy[0].reinit (darcy_solution);
       tmp_darcy[1].reinit (darcy_solution);
@@ -1751,6 +1756,9 @@ namespace Step43
       last_computed_darcy_solution        = tmp_darcy[0];
       second_last_computed_darcy_solution = tmp_darcy[1];
 
+      darcy_constraints.distribute(last_computed_darcy_solution);
+      darcy_constraints.distribute(second_last_computed_darcy_solution);
+
       rebuild_saturation_matrix    = true;
     }
   }
@@ -2190,7 +2198,6 @@ start_time_iteration:
                           SaturationInitialValues<dim>(),
                           old_saturation_solution);
 
-    timestep_number = 0;
     time_step = old_time_step = 0;
     current_macro_time_step = old_macro_time_step = 0;
 
