@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2000 - 2015 by the deal.II authors
+ * Copyright (C) 2000 - 2017 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -327,6 +327,7 @@ namespace Step9
   RightHandSide<dim>::value (const Point<dim>   &p,
                              const unsigned int  component) const
   {
+    (void) component;
     Assert (component == 0, ExcIndexRange (component, 0, 1));
     const double diameter = 0.1;
     return ( (p-center_point).norm_square() < diameter*diameter ?
@@ -374,6 +375,7 @@ namespace Step9
   BoundaryValues<dim>::value (const Point<dim>   &p,
                               const unsigned int  component) const
   {
+    (void) component;
     Assert (component == 0, ExcIndexRange (component, 0, 1));
 
     const double sine_term = std::sin(16*numbers::PI*std::sqrt(p.norm_square()));
@@ -1107,7 +1109,7 @@ namespace Step9
     active_neighbors.reserve (GeometryInfo<dim>::faces_per_cell *
                               GeometryInfo<dim>::max_children_per_face);
 
-    typename DoFHandler<dim>::active_cell_iterator cell_it(std_cxx11::get<0>(cell.iterators));
+    typename DoFHandler<dim>::active_cell_iterator cell_it(std_cxx11::get<0>(*cell));
 
     // First initialize the <code>FEValues</code> object, as well as the
     // <code>Y</code> tensor:
@@ -1142,14 +1144,14 @@ namespace Step9
     // neighbors, of course.
     active_neighbors.clear ();
     for (unsigned int face_no=0; face_no<GeometryInfo<dim>::faces_per_cell; ++face_no)
-      if (! std_cxx11::get<0>(cell.iterators)->at_boundary(face_no))
+      if (! std_cxx11::get<0>(*cell)->at_boundary(face_no))
         {
           // First define an abbreviation for the iterator to the face and
           // the neighbor
           const typename DoFHandler<dim>::face_iterator
-          face = std_cxx11::get<0>(cell.iterators)->face(face_no);
+          face = std_cxx11::get<0>(*cell)->face(face_no);
           const typename DoFHandler<dim>::cell_iterator
-          neighbor = std_cxx11::get<0>(cell.iterators)->neighbor(face_no);
+          neighbor = std_cxx11::get<0>(*cell)->neighbor(face_no);
 
           // Then check whether the neighbor is active. If it is, then it
           // is on the same level or one level coarser (if we are not in
@@ -1192,7 +1194,7 @@ namespace Step9
                   // as an internal error. We therefore use a predefined
                   // exception class to throw here.
                   Assert (neighbor_child->neighbor(face_no==0 ? 1 : 0)
-                          ==std_cxx11::get<0>(cell.iterators),ExcInternalError());
+                          ==std_cxx11::get<0>(*cell),ExcInternalError());
 
                   // If the check succeeded, we push the active neighbor
                   // we just found to the stack we keep:
@@ -1203,7 +1205,7 @@ namespace Step9
                 // `behind' the subfaces of the current face
                 for (unsigned int subface_no=0; subface_no<face->n_children(); ++subface_no)
                   active_neighbors.push_back (
-                    std_cxx11::get<0>(cell.iterators)->neighbor_child_on_subface(face_no,subface_no));
+                    std_cxx11::get<0>(*cell)->neighbor_child_on_subface(face_no,subface_no));
             }
         }
 
@@ -1305,9 +1307,9 @@ namespace Step9
     // at the second element of the pair of iterators, which requires
     // slightly awkward syntax but is not otherwise particularly
     // difficult:
-    *(std_cxx11::get<1>(cell.iterators)) = (std::pow(std_cxx11::get<0>(cell.iterators)->diameter(),
-                                                     1+1.0*dim/2) *
-                                            std::sqrt(gradient.norm_square()));
+    *(std_cxx11::get<1>(*cell)) = (std::pow(std_cxx11::get<0>(*cell)->diameter(),
+                                            1+1.0*dim/2) *
+                                   std::sqrt(gradient.norm_square()));
 
   }
 }
