@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2006 - 2015 by the deal.II authors
+// Copyright (C) 2006 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,6 +13,9 @@
 //
 // ---------------------------------------------------------------------
 
+#ifndef dealii__newton_templates_h
+#define dealii__newton_templates_h
+
 
 #include <deal.II/algorithms/newton.h>
 
@@ -93,7 +96,8 @@ namespace Algorithms
 
   template <typename VectorType>
   void
-  Newton<VectorType>::operator() (AnyData &out, const AnyData &in)
+  Newton<VectorType>::operator() (AnyData &out,
+                                  const AnyData &in)
   {
     Assert (out.size() == 1, ExcNotImplemented());
     deallog.push ("Newton");
@@ -107,6 +111,7 @@ namespace Algorithms
     typename VectorMemory<VectorType>::Pointer Du(mem);
     typename VectorMemory<VectorType>::Pointer res(mem);
 
+    Du->reinit(u);
     res->reinit(u);
     AnyData src1;
     AnyData src2;
@@ -127,15 +132,15 @@ namespace Algorithms
 
     if (debug_vectors)
       {
-        AnyData out;
+        AnyData tmp;
         VectorType *p = &u;
-        out.add<const VectorType *>(p, "solution");
+        tmp.add<const VectorType *>(p, "solution");
         p = Du;
-        out.add<const VectorType *>(p, "update");
+        tmp.add<const VectorType *>(p, "update");
         p = res;
-        out.add<const VectorType *>(p, "residual");
+        tmp.add<const VectorType *>(p, "residual");
         *data_out << step;
-        *data_out << out;
+        *data_out << tmp;
       }
 
     while (control.check(step++, resnorm) == SolverControl::iterate)
@@ -158,15 +163,15 @@ namespace Algorithms
 
         if (debug_vectors)
           {
-            AnyData out;
+            AnyData tmp;
             VectorType *p = &u;
-            out.add<const VectorType *>(p, "solution");
+            tmp.add<const VectorType *>(p, "solution");
             p = Du;
-            out.add<const VectorType *>(p, "update");
+            tmp.add<const VectorType *>(p, "update");
             p = res;
-            out.add<const VectorType *>(p, "residual");
+            tmp.add<const VectorType *>(p, "residual");
             *data_out << step;
-            *data_out << out;
+            *data_out << tmp;
           }
 
         u.add(-1., *Du);
@@ -204,3 +209,5 @@ namespace Algorithms
 
 
 DEAL_II_NAMESPACE_CLOSE
+
+#endif
