@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2016 by the deal.II authors
+// Copyright (C) 1998 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -55,7 +55,7 @@ public:
   /**
    * Destructor.
    */
-  virtual ~ExceptionBase () throw();
+  virtual ~ExceptionBase () DEAL_II_NOEXCEPT;
 
   /**
    * Set the file name and line of where the exception appeared as well as the
@@ -72,7 +72,7 @@ public:
   /**
    * Override the standard function that returns the description of the error.
    */
-  virtual const char *what() const throw();
+  virtual const char *what() const DEAL_II_NOEXCEPT;
 
   /**
    * Get exception name.
@@ -142,6 +142,11 @@ protected:
 #endif
 
 private:
+  /**
+   * Copy operator. Made private since it is not used and not implemented.
+   */
+  ExceptionBase operator= (const ExceptionBase &exc);
+
   /**
    * Internal function that generates the c_string. Called by what().
    */
@@ -219,26 +224,40 @@ namespace deal_II_exceptions
     /**
      * Conditionally abort the program.
      *
-     * Depending on whether disable_abort_on_exception was called, this
-     * function either aborts the program flow by printing the error message
-     * provided by @p exc and calling <tt>std::abort()</tt>, or throws @p exc
-     * instead (if @p nothrow is set to <tt>false</tt>).
+     * Depending on whether deal_II_exceptions::disable_abort_on_exception was
+     * called, this function either aborts the program flow by printing the
+     * error message provided by @p exc and calling <tt>std::abort()</tt>, or
+     * throws @p exc instead (if @p nothrow is set to <tt>false</tt>).
      *
-     * If the boolean @p nothrow is set to true and disable_abort_on_exception
-     * was called, the exception type is just printed to deallog and program
-     * flow continues. This is useful if throwing an exception is prohibited
+     * If the boolean @p nothrow is set to true and
+     * deal_II_exceptions::disable_abort_on_exception was called, the
+     * exception type is just printed to deallog and program flow
+     * continues. This is useful if throwing an exception is prohibited
      * (e.g. in a destructor with <tt>noexcept(true)</tt> or
      * <tt>throw()</tt>).
      */
     void abort (const ExceptionBase &exc, bool nothrow = false);
 
     /**
-     * An enum describing how to treat an exception in issue_error
+     * An enum describing how to treat an exception in issue_error.
      */
     enum ExceptionHandling
     {
+      /**
+       * Abort the program by calling <code>std::abort</code> unless
+       * deal_II_exceptions::disable_abort_on_exception has been called: in
+       * that case the program will throw an exception.
+       */
       abort_on_exception,
+      /**
+       * Throw the exception normally.
+       */
       throw_on_exception,
+      /**
+       * Call <code>std::abort</code> as long as
+       * deal_II_exceptions::disable_abort_on_exception has not been called:
+       * if it has, then just print a description of the exception to deallog.
+       */
       abort_nothrow_on_exception
     };
 
@@ -391,9 +410,9 @@ namespace deal_II_exceptions
   {                                                                       \
   public:                                                                 \
     Exception (const std::string &msg = defaulttext) : arg (msg) {}       \
-    virtual ~Exception () throw () {}                                     \
+    virtual ~Exception () DEAL_II_NOEXCEPT {}                             \
     virtual void print_info (std::ostream &out) const {                   \
-      out << arg << std::endl;                                            \
+      out << "    " << arg << std::endl;                                  \
     }                                                                     \
   private:                                                                \
     const std::string arg;                                                \
@@ -409,9 +428,9 @@ namespace deal_II_exceptions
   class Exception1 : public dealii::ExceptionBase {                       \
   public:                                                                 \
     Exception1 (const type1 a1) : arg1 (a1) {}                            \
-    virtual ~Exception1 () throw () {}                                    \
+    virtual ~Exception1 () DEAL_II_NOEXCEPT {}                            \
     virtual void print_info (std::ostream &out) const {                   \
-      out outsequence << std::endl;                                       \
+      out << "    " outsequence << std::endl;                             \
     }                                                                     \
   private:                                                                \
     const type1 arg1;                                                     \
@@ -429,9 +448,9 @@ namespace deal_II_exceptions
   public:                                                                 \
     Exception2 (const type1 a1, const type2 a2) :                         \
       arg1 (a1), arg2(a2) {}                                              \
-    virtual ~Exception2 () throw () {}                                    \
+    virtual ~Exception2 () DEAL_II_NOEXCEPT {}                            \
     virtual void print_info (std::ostream &out) const {                   \
-      out outsequence << std::endl;                                       \
+      out << "    " outsequence << std::endl;                             \
     }                                                                     \
   private:                                                                \
     const type1 arg1;                                                     \
@@ -450,9 +469,9 @@ namespace deal_II_exceptions
   public:                                                                 \
     Exception3 (const type1 a1, const type2 a2, const type3 a3) :         \
       arg1 (a1), arg2(a2), arg3(a3) {}                                    \
-    virtual ~Exception3 () throw () {}                                    \
+    virtual ~Exception3 () DEAL_II_NOEXCEPT {}                            \
     virtual void print_info (std::ostream &out) const {                   \
-      out outsequence << std::endl;                                       \
+      out << "    " outsequence << std::endl;                             \
     }                                                                     \
   private:                                                                \
     const type1 arg1;                                                     \
@@ -473,9 +492,9 @@ namespace deal_II_exceptions
     Exception4 (const type1 a1, const type2 a2,                           \
                 const type3 a3, const type4 a4) :                         \
       arg1 (a1), arg2(a2), arg3(a3), arg4(a4) {}                          \
-    virtual ~Exception4 () throw () {}                                    \
+    virtual ~Exception4 () DEAL_II_NOEXCEPT {}                            \
     virtual void print_info (std::ostream &out) const {                   \
-      out outsequence << std::endl;                                       \
+      out << "    " outsequence << std::endl;                             \
     }                                                                     \
   private:                                                                \
     const type1 arg1;                                                     \
@@ -497,9 +516,9 @@ namespace deal_II_exceptions
     Exception5 (const type1 a1, const type2 a2, const type3 a3,           \
                 const type4 a4, const type5 a5) :                         \
       arg1 (a1), arg2(a2), arg3(a3), arg4(a4), arg5(a5) {}                \
-    virtual ~Exception5 () throw () {}                                    \
+    virtual ~Exception5 () DEAL_II_NOEXCEPT {}                            \
     virtual void print_info (std::ostream &out) const {                   \
-      out outsequence << std::endl;                                       \
+      out << "    " outsequence << std::endl;                             \
     }                                                                     \
   private:                                                                \
     const type1 arg1;                                                     \
@@ -520,6 +539,7 @@ namespace deal_II_exceptions
  * @ingroup Exceptions
  */
 #define DeclException0(Exception0)                                        \
+  /** @ingroup Exceptions */ \
   static dealii::ExceptionBase& Exception0 ()
 
 /**
@@ -531,6 +551,8 @@ namespace deal_II_exceptions
  * @ingroup Exceptions
  */
 #define DeclExceptionMsg(Exception, defaulttext)                          \
+  /** @ingroup Exceptions */ \
+  /** @dealiiExceptionMessage{defaulttext} */  \
   static dealii::ExceptionBase& Exception ()
 
 /**
@@ -540,7 +562,9 @@ namespace deal_II_exceptions
  * @ingroup Exceptions
  */
 #define DeclException1(Exception1, type1, outsequence)                    \
-  static dealii::ExceptionBase& Exception1 (type1 arg1) throw (errortext outsequence)
+  /** @ingroup Exceptions */ \
+  /** @dealiiExceptionMessage{outsequence} */              \
+  static dealii::ExceptionBase& Exception1 (type1 arg1)
 
 
 /**
@@ -550,7 +574,9 @@ namespace deal_II_exceptions
  * @ingroup Exceptions
  */
 #define DeclException2(Exception2, type1, type2, outsequence)             \
-  static dealii::ExceptionBase& Exception2 (type1 arg1, type2 arg2) throw (errortext outsequence)
+  /** @ingroup Exceptions */ \
+  /** @dealiiExceptionMessage{outsequence} */              \
+  static dealii::ExceptionBase& Exception2 (type1 arg1, type2 arg2)
 
 
 /**
@@ -560,7 +586,9 @@ namespace deal_II_exceptions
  * @ingroup Exceptions
  */
 #define DeclException3(Exception3, type1, type2, type3, outsequence)      \
-  static dealii::ExceptionBase& Exception3 (type1 arg1, type2 arg2, type3 arg3) throw (errortext outsequence)
+  /** @ingroup Exceptions */ \
+  /** @dealiiExceptionMessage{outsequence} */              \
+  static dealii::ExceptionBase& Exception3 (type1 arg1, type2 arg2, type3 arg3)
 
 
 /**
@@ -570,7 +598,9 @@ namespace deal_II_exceptions
  * @ingroup Exceptions
  */
 #define DeclException4(Exception4, type1, type2, type3, type4, outsequence) \
-  static dealii::ExceptionBase& Exception4 (type1 arg1, type2 arg2, type3 arg3, type4 arg4) throw (errortext outsequence)
+  /** @ingroup Exceptions */ \
+  /** @dealiiExceptionMessage{outsequence} */                \
+  static dealii::ExceptionBase& Exception4 (type1 arg1, type2 arg2, type3 arg3, type4 arg4)
 
 
 /**
@@ -580,7 +610,9 @@ namespace deal_II_exceptions
  * @ingroup Exceptions
  */
 #define DeclException5(Exception5, type1, type2, type3, type4, type5, outsequence) \
-  static dealii::ExceptionBase& Exception5 (type1 arg1, type2 arg2, type3 arg3, type4 arg4, type5 arg5) throw (errortext outsequence)
+  /** @ingroup Exceptions */ \
+  /** @dealiiExceptionMessage{outsequence} */                       \
+  static dealii::ExceptionBase& Exception5 (type1 arg1, type2 arg2, type3 arg3, type4 arg4, type5 arg5)
 
 #endif /*ifndef DOXYGEN*/
 
@@ -626,7 +658,7 @@ namespace StandardExceptions
                   << "value is a finite number (as opposed to plus or minus infinity, or "
                   << "NaN/Not a Number). In the current function, we encountered a number "
                   << "that is not finite (its value is " << arg1 << " and therefore "
-                  << "violates the current assertion.\n\n"
+                  << "violates the current assertion).\n\n"
                   << "This may be due to the fact that some operation in this function "
                   << "created such a value, or because one of the arguments you passed "
                   << "to the function already had this value from some previous "
@@ -669,7 +701,7 @@ namespace StandardExceptions
    */
   DeclException1 (ExcMemoryLeak, int,
                   << "Destroying memory handler while " << arg1
-                  << " objects are still allocated");
+                  << " objects are still allocated.");
 
   /**
    * An error occurred reading or writing a file.
@@ -705,7 +737,7 @@ namespace StandardExceptions
    */
   DeclException1 (ExcFileNotOpen,
                   char *,
-                  << "Could not open file " << arg1);
+                  << "Could not open file " << arg1 << ".");
 
   /**
    * Exception denoting a part of the library or application program that has
@@ -755,7 +787,7 @@ namespace StandardExceptions
                     "example would be that the first part of an algorithm "
                     "sorts elements of an array in ascending order, and "
                     "a second part of the algorithm later encounters an "
-                    "an element that is not larger than the previous one."
+                    "element that is not larger than the previous one."
                     "\n\n"
                     "There is usually not very much you can do if you "
                     "encounter such an exception since it indicates an error "
@@ -806,13 +838,6 @@ namespace StandardExceptions
                     "been called; if the answer is 'yes', then you need to "
                     "implement the missing override in your class.");
 
-  /**
-   * Used for constructors that are disabled. Examples are copy constructors
-   * and assignment operators of large objects, which are only allowed for
-   * empty objects.
-   */
-  DeclException0 (ExcInvalidConstructorCall);
-
   /**
    * This exception is used if some object is found uninitialized.
    */
@@ -835,6 +860,20 @@ namespace StandardExceptions
                   << "impossible in " << arg1
                   << "d or simply does not make any sense.");
 
+  /**
+   * This exception is raised if a functionality is not possible in the given
+   * combination of dimension and space-dimension.
+   *
+   * The constructor takes two <tt>int</tt>, denoting the dimension and the
+   * space dimension.
+   */
+  DeclException2 (ExcImpossibleInDimSpacedim,
+                  int, int,
+                  << "You are trying to execute functionality that is "
+                  << "impossible in dimensions <" << arg1 << "," << arg2
+                  << "> or simply does not make any sense.");
+
+
   /**
    * A number is zero, but it should not be here.
    */
@@ -863,7 +902,7 @@ namespace StandardExceptions
    */
   DeclException2 (ExcDimensionMismatch,
                   std::size_t, std::size_t,
-                  << "Dimension " << arg1 << " not equal to " << arg2);
+                  << "Dimension " << arg1 << " not equal to " << arg2 << ".");
 
   /**
    * The first dimension should be either equal to the second or the third,
@@ -872,7 +911,7 @@ namespace StandardExceptions
   DeclException3 (ExcDimensionMismatch2,
                   int, int, int,
                   << "Dimension " << arg1 << " neither equal to " << arg2
-                  << " nor to " << arg3);
+                  << " nor to " << arg3 << ".");
 
   /**
    * This exception indicates that an index is not within the expected range.
@@ -903,7 +942,7 @@ namespace StandardExceptions
    * For example, it may be that you are trying to access an element of a
    * vector which does not exist.
    *
-   * The constructor takes three <tt>int</tt> arguments, namely
+   * The constructor takes three arguments, namely
    * <ol>
    * <li> the violating index
    * <li> the lower bound
@@ -932,7 +971,7 @@ namespace StandardExceptions
   DeclException2 (ExcLowerRange,
                   int, int,
                   << "Number " << arg1 << " must be larger than or equal "
-                  << arg2 << ").");
+                  << arg2 << ".");
 
   /**
    * A generic exception definition for the ExcLowerRange above.
@@ -950,7 +989,7 @@ namespace StandardExceptions
   DeclException2 (ExcNotMultiple,
                   int, int,
                   << "Division " << arg1 << " by " << arg2
-                  << " has remainder different from zero");
+                  << " has remainder different from zero.");
 
   /**
    * This exception is thrown if the iterator you access has corrupted data.
@@ -1022,20 +1061,75 @@ namespace StandardExceptions
   /**
    * This function requires support for the LAPACK library.
    */
-  DeclException0 (ExcNeedsLAPACK);
+  DeclExceptionMsg (ExcNeedsLAPACK,
+                    "You are attempting to use functionality that is only available "
+                    "if deal.II was configured to use LAPACK, but cmake did not "
+                    "find a valid LAPACK library.");
 
   /**
    * This function requires support for the NetCDF library.
    */
-  DeclException0 (ExcNeedsNetCDF);
+  DeclExceptionMsg (ExcNeedsNetCDF,
+                    "You are attempting to use functionality that is only available "
+                    "if deal.II was configured to use NetCDF, but cmake did not "
+                    "find a valid NetCDF library.");
 
   /**
    * This function requires support for the FunctionParser library.
    */
-  DeclException0 (ExcNeedsFunctionparser);
-
+  DeclExceptionMsg (ExcNeedsFunctionparser,
+                    "You are attempting to use functionality that is only available "
+                    "if deal.II was configured to use the function parser which "
+                    "relies on the muparser library, but cmake did not "
+                    "find a valid muparser library on your system and also did "
+                    "not choose the one that comes bundled with deal.II.");
 
+#ifdef DEAL_II_WITH_CUDA
+  /**
+   * This exception is raised if an error happened in a CUDA kernel.
+   *
+   * The constructor takes a single <tt>char*</tt>, the output of
+   * cudaGetErrorString.
+   */
+  DeclException1 (ExcCudaError,
+                  char *,
+                  << arg1);
+#endif
 //@}
+
+#ifdef DEAL_II_WITH_MPI
+  /**
+   * Exception for MPI errors. This exception is only defined if
+   * <code>deal.II</code> is compiled with MPI support. This exception should
+   * be used with <code>AssertThrow</code> to check error codes of MPI
+   * functions. For example:
+   * @code
+   * const int ierr = MPI_Isend(...);
+   * AssertThrow(ierr == MPI_SUCCESS, ExcMPI(ierr));
+   * @endcode
+   * or, using the convenience macro <code>AssertThrowMPI</code>,
+   * @code
+   * const int ierr = MPI_Irecv(...);
+   * AssertThrowMPI(ierr);
+   * @endcode
+   *
+   * If the assertion fails then the error code will be used to print a helpful
+   * message to the screen by utilizing the <code>MPI_Error_string</code>
+   * function.
+   *
+   * @ingroup Exceptions
+   * @author David Wells, 2016
+   */
+  class ExcMPI : public dealii::ExceptionBase
+  {
+  public:
+    ExcMPI (const int error_code);
+
+    virtual void print_info (std::ostream &out) const;
+
+    const int error_code;
+  };
+#endif // DEAL_II_WITH_MPI
 } /*namespace StandardExceptions*/
 
 
@@ -1054,8 +1148,8 @@ namespace StandardExceptions
 
 
 /**
- * Special assertion, testing whether <tt>vec</tt> has size <tt>dim1</tt>, and
- * each entry of the vector has the size <tt>dim2</tt>
+ * An assertion that tests whether <tt>vec</tt> has size <tt>dim1</tt>, and
+ * each entry of the vector is itself an array that has the size <tt>dim2</tt>.
  *
  * @ingroup Exceptions
  * @author Guido Kanschat 2010
@@ -1065,14 +1159,9 @@ namespace StandardExceptions
 
 
 /**
- * Special assertion for index range of nonnegative indices.
- *
- * Since this is used very often and always repeats the arguments, we
- * introduce this special assertion for ExcIndexRange in order to keep the
- * user codes shorter.
- *
- * Called wit arguments <tt>index</tt> and <tt>range</tt> it asserts that
- * <tt>index&lt;range</tt> and throws ExcIndexRange(index,0,range) if it
+ * An assertion that tests that a given index is within the half-open
+ * range <code>[0,range)</code>. It throws an exception object
+ * <code>ExcIndexRange(index,0,range)</code> if the assertion
  * fails.
  *
  * @ingroup Exceptions
@@ -1081,8 +1170,20 @@ namespace StandardExceptions
 #define AssertIndexRange(index,range) Assert((index) < (range), \
                                              dealii::ExcIndexRange((index),0,(range)))
 
+/**
+ * An assertion that tests that a given index is within the half-open
+ * range <code>[0,range)</code>. It throws an exception object
+ * <code>ExcIndexRange(index,0,range)</code> if the assertion
+ * fails.
+ *
+ * This variation of the AssertIndexRange assertion is used for indices of type
+ * types::global_dof_index for which we need to make special accommodations because
+ * they may not fit into the regular 32-bit integer indices used in AssertIndexRange.
+ *
+ * @ingroup Exceptions
+ */
 #define AssertGlobalIndexRange(index,range) Assert((index) < (range), \
-                                                   ExcIndexRange<types::global_dof_index>((index),0,(range)))
+                                                   dealii::ExcIndexRange<types::global_dof_index>((index),0,(range)))
 
 /**
  * An assertion that checks whether a number is finite or not. We explicitly
@@ -1094,7 +1195,38 @@ namespace StandardExceptions
  * @author Wolfgang Bangerth, 2015
  */
 #define AssertIsFinite(number) Assert(dealii::numbers::is_finite(number), \
-                                      ExcNumberNotFinite(std::complex<double>(number)))
+                                      dealii::ExcNumberNotFinite(std::complex<double>(number)))
+
+#ifdef DEAL_II_WITH_MPI
+/**
+ * An assertion that checks whether or not an error code returned by an MPI
+ * function is equal to <code>MPI_SUCCESS</code>. If the check fails then an
+ * exception of type ExcMPI is thrown with the given error code as an
+ * argument.
+ *
+ * @note Active only if deal.II is compiled with MPI
+ * @ingroup Exceptions
+ * @author David Wells, 2016
+ */
+#define AssertThrowMPI(error_code) AssertThrow(error_code == MPI_SUCCESS, \
+                                               dealii::ExcMPI(error_code))
+#else
+#define AssertThrowMPI(error_code) {}
+#endif // DEAL_II_WITH_MPI
+
+#ifdef DEAL_II_WITH_CUDA
+/**
+ * An assertion that checks that the error code produced by calling a CUDA
+ * routine is equal to cudaSuccess.
+ *
+ * @ingroup Exceptions
+ * @author Bruno Turcksin, 2016
+ */
+// For now use AssertThrow instead of Assert because macros are not passed
+// correctly to nvcc.
+#define AssertCuda(error_code) AssertThrow(error_code == cudaSuccess, \
+                                           dealii::ExcCudaError(cudaGetErrorString(error_code)))
+#endif
 
 using namespace StandardExceptions;
 
