@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2015 by the deal.II authors
+// Copyright (C) 2000 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -101,16 +101,6 @@ public:
    */
   std::size_t memory_consumption () const;
 
-  /**
-   * @addtogroup Exceptions
-   * @{
-   */
-
-  /**
-   * Exception.
-   */
-  DeclException0(ExcInvalidFormula);
-  //@}
 private:
   /**
    * Function for differentiation.
