@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2005 - 2015 by the deal.II authors
+// Copyright (C) 2005 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -19,7 +19,7 @@
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/exceptions.h>
-#include <deal.II/base/function.h>
+#include <deal.II/base/auto_derivative_function.h>
 #include <deal.II/base/tensor.h>
 #include <deal.II/base/point.h>
 #include <deal.II/base/thread_local_storage.h>
@@ -130,7 +130,7 @@ template <typename> class Vector;
  * An example of time dependent scalar function is the following:
  * @code
  *    // Empty constants object
- *    std::map<std::string> constants;
+ *    std::map<std::string,double> constants;
  *
  *    // Variables that will be used inside the expressions
  *    std::string variables = "x,y,t";
@@ -155,7 +155,7 @@ template <typename> class Vector;
  * function by using a single string:
  * @code
  *    // Empty constants object
- *    std::map<std::string> constants;
+ *    std::map<std::string,double> constants;
  *
  *    // Variables that will be used inside the expressions
  *    std::string variables = "x,y";
@@ -177,18 +177,21 @@ template <typename> class Vector;
  * @author Luca Heltai, Timo Heister 2005, 2014
  */
 template <int dim>
-class FunctionParser : public Function<dim>
+class FunctionParser : public AutoDerivativeFunction<dim>
 {
 public:
   /**
-   * Constructor for Parsed functions. Its arguments are the same of the base
-   * class Function. The only difference is that this object needs to be
-   * initialized with initialize() method before you can use it. If an attempt
-   * to use this function is made before the initialize() method has been
-   * called, then an exception is thrown.
+   * Constructor for Parsed functions. Its arguments are the same of
+   * the base class Function, with the additional parameter @p h, used
+   * for the computation of gradients using finite differences. This
+   * object needs to be initialized with the initialize() method
+   * before you can use it. If an attempt to use this function is made
+   * before the initialize() method has been called, then an exception
+   * is thrown.
    */
   FunctionParser (const unsigned int n_components = 1,
-                  const double       initial_time = 0.0);
+                  const double       initial_time = 0.0,
+                  const double       h = 1e-8);
 
   /**
    * Destructor. Explicitly delete the FunctionParser objects (there is one
