@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2016 by the deal.II authors
+// Copyright (C) 2009 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -19,9 +19,12 @@
 #include <deal.II/base/config.h>
 #include <deal.II/base/utilities.h>
 #include <deal.II/base/exceptions.h>
+#include <deal.II/base/thread_management.h>
 #include <boost/serialization/vector.hpp>
 #include <vector>
 #include <algorithm>
+#include <iterator>
+
 
 #ifdef DEAL_II_WITH_TRILINOS
 #  include <Epetra_Map.h>
@@ -105,6 +108,29 @@ public:
    */
   explicit IndexSet (const size_type size);
 
+#ifdef DEAL_II_WITH_CXX11
+  /**
+   * Copy constructor.
+   */
+  IndexSet (const IndexSet &) = default;
+
+  /**
+   * Copy assignment operator.
+   */
+  IndexSet &operator= (const IndexSet &) = default;
+
+  /**
+   * Move constructor. Create a new IndexSet by transferring the internal data
+   * of the input set.
+   */
+  IndexSet (IndexSet &&is);
+
+  /**
+   * Move assignment operator. Transfer the internal data of the input set into
+   * the current one.
+   */
+  IndexSet &operator= (IndexSet &&is);
+#endif
 
 #ifdef DEAL_II_WITH_TRILINOS
   /**
@@ -192,6 +218,23 @@ public:
    */
   bool is_contiguous () const;
 
+  /**
+   * Return whether the index set stored by this object contains no elements.
+   * This is similar, but faster than checking <code>n_elements() == 0</code>.
+   */
+  bool is_empty () const;
+
+  /**
+   * Return whether the IndexSets are ascending with respect to MPI process
+   * number and 1:1, i.e., each index is contained in exactly one IndexSet
+   * (among those stored on the different processes), each process stores
+   * contiguous subset of indices, and the index set on process $p+1$ starts
+   * at the index one larger than the last one stored on process $p$.
+   * In case there is only one MPI process, this just means that the IndexSet
+   * is complete.
+   */
+  bool is_ascending_and_one_to_one(const MPI_Comm &communicator) const;
+
   /**
    * Return the number of elements stored in this index set.
    */
@@ -207,7 +250,7 @@ public:
   /**
    * Return the how-manyth element of this set (counted in ascending order) @p
    * global_index is. @p global_index needs to be less than the size(). This
-   * function throws an exception if the index @p global_index is not actually
+   * function returns numbers::invalid_dof_index if the index @p global_index is not actually
    * a member of this index set, i.e. if is_element(global_index) is false.
    */
   size_type index_within_set (const size_type global_index) const;
@@ -279,6 +322,18 @@ public:
    */
   void subtract_set (const IndexSet &other);
 
+  /**
+   * Removes and returns the last element of the last range.
+   * Throws an exception if the IndexSet is empty.
+   */
+  size_type pop_back ();
+
+  /**
+   * Removes and returns the first element of the first range.
+   * Throws an exception if the IndexSet is empty.
+   */
+  size_type pop_front ();
+
   /**
    * Fills the given vector with all indices contained in this IndexSet.
    */
@@ -307,25 +362,25 @@ public:
   void print(StreamType &out) const;
 
   /**
-   * Writes the IndexSet into a text based file format, that can be read in
+   * Write the IndexSet into a text based file format, that can be read in
    * again using the read() function.
    */
   void write(std::ostream &out) const;
 
   /**
-   * Constructs the IndexSet from a text based representation given by the
+   * Construct the IndexSet from a text based representation given by the
    * stream @p in written by the write() function.
    */
   void read(std::istream &in);
 
   /**
-   * Writes the IndexSet into a binary, compact representation, that can be
+   * Write the IndexSet into a binary, compact representation, that can be
    * read in again using the block_read() function.
    */
   void block_write(std::ostream &out) const;
 
   /**
-   * Constructs the IndexSet from a binary representation given by the stream
+   * Construct the IndexSet from a binary representation given by the stream
    * @p in written by the write_block() function.
    */
   void block_read(std::istream &in);
@@ -473,7 +528,7 @@ public:
    * Class that represents an iterator pointing to a contiguous interval
    * $[a,b[$ as returned by IndexSet::begin_interval().
    */
-  class IntervalIterator
+  class IntervalIterator : public std::iterator<std::forward_iterator_tag,IntervalAccessor>
   {
   public:
     /**
@@ -556,14 +611,16 @@ public:
    * Class that represents an iterator pointing to a single element in the
    * IndexSet as returned by IndexSet::begin().
    */
-  class ElementIterator
+  class ElementIterator : public std::iterator<std::forward_iterator_tag,size_type>
   {
   public:
     /**
      * Construct an iterator pointing to the global index @p index in the
      * interval @p range_idx
      */
-    ElementIterator(const IndexSet *idxset, const size_type range_idx, const size_type index);
+    ElementIterator(const IndexSet *idxset,
+                    const size_type range_idx,
+                    const size_type index);
 
     /**
      * Construct an iterator pointing to the end of the IndexSet.
@@ -642,6 +699,22 @@ public:
    */
   ElementIterator begin() const;
 
+  /**
+   * Return an element iterator pointing to the element with global index
+   * @p global_index or the next larger element if the index is not in the
+   * set. This is equivalent to
+   * @code
+   * auto p = begin();
+   * while (*p<global_index)
+   *   ++p;
+   * return p;
+   * @endcode
+   *
+   * If there is no element in this IndexSet at or behind @p global_index,
+   * this method will return end().
+   */
+  ElementIterator at(const size_type global_index) const;
+
   /**
    * Return an iterator that points one after the last index that is contained
    * in this IndexSet.
@@ -731,7 +804,7 @@ private:
               (range_1.end == range_2.end));
     }
 
-    std::size_t memory_consumption () const
+    static std::size_t memory_consumption ()
     {
       return sizeof(Range);
     }
@@ -781,6 +854,12 @@ private:
    */
   mutable size_type largest_range;
 
+  /**
+   * A mutex that is used to synchronize operations of the do_compress() function
+   * that is called from many 'const' functions via compress().
+   */
+  mutable Threads::Mutex compress_mutex;
+
   /**
    * Actually perform the compress() operation.
    */
@@ -1090,7 +1169,7 @@ inline
 IndexSet::ElementIterator
 IndexSet::ElementIterator::operator++ (int)
 {
-  IndexSet::ElementIterator it = *this;
+  const IndexSet::ElementIterator it = *this;
   advance();
   return it;
 }
@@ -1146,7 +1225,8 @@ inline
 IndexSet::Range::Range ()
   :
   begin(numbers::invalid_dof_index),
-  end(numbers::invalid_dof_index)
+  end(numbers::invalid_dof_index),
+  nth_index_in_set(numbers::invalid_dof_index)
 {}
 
 
@@ -1155,7 +1235,8 @@ IndexSet::Range::Range (const size_type i1,
                         const size_type i2)
   :
   begin(i1),
-  end(i2)
+  end(i2),
+  nth_index_in_set(numbers::invalid_dof_index)
 {}
 
 
@@ -1171,6 +1252,55 @@ IndexSet::ElementIterator IndexSet::begin() const
     return end();
 }
 
+inline
+IndexSet::ElementIterator IndexSet::at(const size_type global_index) const
+{
+  compress();
+  Assert (global_index < size(),
+          ExcIndexRangeType<size_type> (global_index, 0, size()));
+
+  if (ranges.empty())
+    return end();
+
+  std::vector<Range>::const_iterator main_range=ranges.begin()+largest_range;
+
+  Range r (global_index, global_index+1);
+  // This optimization makes the bounds for lower_bound smaller by checking
+  // the largest range first.
+  std::vector<Range>::const_iterator range_begin, range_end;
+  if (global_index<main_range->begin)
+    {
+      range_begin = ranges.begin();
+      range_end   = main_range;
+    }
+  else
+    {
+      range_begin = main_range;
+      range_end   = ranges.end();
+    }
+
+  // This will give us the first range p=[a,b[ with b>=global_index using
+  // a binary search
+  const std::vector<Range>::const_iterator
+  p = Utilities::lower_bound(range_begin, range_end, r, Range::end_compare);
+
+  // We couldn't find a range, which means we have no range that contains
+  // global_index and also no range behind it, meaning we need to return end().
+  if (p == ranges.end())
+    return end();
+
+  // Finally, we can have two cases: Either global_index is not in [a,b[,
+  // which means we need to return an iterator to a because global_index, ...,
+  // a-1 is not in the IndexSet (if branch). Alternatively, global_index is in
+  // [a,b[ and we will return an iterator pointing directly at global_index
+  // (else branch).
+  if (global_index < p->begin)
+    return IndexSet::ElementIterator(this, p-ranges.begin(), p->begin);
+  else
+    return IndexSet::ElementIterator(this, p-ranges.begin(), global_index);
+}
+
+
 inline
 IndexSet::ElementIterator IndexSet::end() const
 {
@@ -1218,13 +1348,55 @@ IndexSet::IndexSet (const size_type size)
 
 
 
+#ifdef DEAL_II_WITH_CXX11
+
+inline
+IndexSet::IndexSet (IndexSet &&is)
+  :
+  ranges (std::move(is.ranges)),
+  is_compressed (is.is_compressed),
+  index_space_size (is.index_space_size),
+  largest_range (is.largest_range)
+{
+  is.ranges.clear ();
+  is.is_compressed = true;
+  is.index_space_size = 0;
+  is.largest_range = numbers::invalid_unsigned_int;
+
+  compress ();
+}
+
+
+inline
+IndexSet &IndexSet::operator= (IndexSet &&is)
+{
+  ranges = std::move (is.ranges);
+  is_compressed = is.is_compressed;
+  index_space_size = is.index_space_size;
+  largest_range = is.largest_range;
+
+  is.ranges.clear ();
+  is.is_compressed = true;
+  is.index_space_size = 0;
+  is.largest_range = numbers::invalid_unsigned_int;
+
+  compress ();
+
+  return *this;
+}
+
+#endif
+
+
 inline
 void
 IndexSet::clear ()
 {
+  // reset so that there are no indices in the set any more; however,
+  // as documented, the index set retains its size
   ranges.clear ();
-  largest_range = 0;
   is_compressed = true;
+  largest_range = numbers::invalid_unsigned_int;
 }
 
 
@@ -1374,6 +1546,14 @@ IndexSet::is_contiguous () const
 }
 
 
+inline
+bool
+IndexSet::is_empty () const
+{
+  return ranges.empty();
+}
+
+
 
 inline
 IndexSet::size_type
@@ -1431,11 +1611,10 @@ inline
 IndexSet::size_type
 IndexSet::nth_index_in_set (const unsigned int n) const
 {
-  // to make this call thread-safe, compress() must not be called through this
-  // function
-  Assert (is_compressed == true, ExcMessage ("IndexSet must be compressed."));
   Assert (n < n_elements(), ExcIndexRangeType<size_type> (n, 0, n_elements()));
 
+  compress ();
+
   // first check whether the index is in the largest range
   Assert (largest_range < ranges.size(), ExcInternalError());
   std::vector<Range>::const_iterator main_range=ranges.begin()+largest_range;
@@ -1469,7 +1648,6 @@ IndexSet::nth_index_in_set (const unsigned int n) const
 }
 
 
-
 inline
 IndexSet::size_type
 IndexSet::index_within_set (const size_type n) const
@@ -1477,9 +1655,12 @@ IndexSet::index_within_set (const size_type n) const
   // to make this call thread-safe, compress() must not be called through this
   // function
   Assert (is_compressed == true, ExcMessage ("IndexSet must be compressed."));
-  Assert (is_element(n) == true, ExcIndexNotPresent (n));
   Assert (n < size(), ExcIndexRangeType<size_type> (n, 0, size()));
 
+  // return immediately if the index set is empty
+  if (is_empty())
+    return numbers::invalid_dof_index;
+
   // check whether the index is in the largest range. use the result to
   // perform a one-sided binary search afterward
   Assert (largest_range < ranges.size(), ExcInternalError());
@@ -1504,6 +1685,10 @@ IndexSet::index_within_set (const size_type n) const
   p = Utilities::lower_bound(range_begin, range_end, r,
                              Range::end_compare);
 
+  // if n is not in this set
+  if (p==range_end || p->end == n || p->begin > n)
+    return numbers::invalid_dof_index;
+
   Assert(p!=ranges.end(), ExcInternalError());
   Assert(p->begin<=n, ExcInternalError());
   Assert(n<p->end, ExcInternalError());
@@ -1579,7 +1764,7 @@ IndexSet::print (StreamType &out) const
       else
         out << "[" << p->begin << "," << p->end-1 << "]";
 
-      if (p !=--ranges.end())
+      if (p != --ranges.end())
         out << ", ";
     }
   out << "}" << std::endl;
