@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2016 by the deal.II authors
+// Copyright (C) 1998 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -63,7 +63,7 @@ DEAL_II_NAMESPACE_OPEN
  * can be set via <tt>LogStream::Prefix p("loopname");</tt>. The destructor of
  * the prefix will pop the prefix text from the stack.
  *
- * Writes via the <tt>&lt;&lt;</tt> operator, <tt> deallog << "This is a log
+ * Write via the <tt>&lt;&lt;</tt> operator, <tt> deallog << "This is a log
  * notice";</tt> will be buffered thread locally until a <tt>std::flush</tt>
  * or <tt>std::endl</tt> is encountered, which will trigger a writeout to the
  * console and, if set up, the log file.
@@ -190,13 +190,16 @@ public:
    * Setup the logstream for regression test mode.
    *
    * This sets the parameters #double_threshold, #float_threshold, and #offset
-   * to nonzero values. The exact values being used have been determined
-   * experimentally and can be found in the source code.
+   * to nonzero values. The default values being used have been determined
+   * experimentally.
    *
    * Called with an argument <tt>false</tt>, switches off test mode and sets
    * all involved parameters to zero.
    */
-  void test_mode (bool on=true);
+  void test_mode (const bool on = true,
+                  const double double_threshold = 1.e-10,
+                  const float float_threshold = 1.e-7f,
+                  const double offset = 1.e-7);
 
 
   /**
@@ -398,12 +401,6 @@ public:
    */
   std::size_t memory_consumption () const;
 
-
-  /**
-   * Exception.
-   */
-  DeclException0(ExcNoFileStreamGiven);
-
 private:
 
 
