@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2006 - 2015 by the deal.II authors
+// Copyright (C) 2006 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -26,6 +26,26 @@
 
 DEAL_II_NAMESPACE_OPEN
 
+// forward declarations to support abs or sqrt operations on VectorizedArray
+template <typename Number> class VectorizedArray;
+template <typename T> struct EnableIfScalar;
+
+DEAL_II_NAMESPACE_CLOSE
+
+namespace std
+{
+  template <typename Number> DEAL_II_ALWAYS_INLINE ::dealii::VectorizedArray<Number>
+  sqrt(const ::dealii::VectorizedArray<Number> &);
+  template <typename Number> DEAL_II_ALWAYS_INLINE ::dealii::VectorizedArray<Number>
+  abs(const ::dealii::VectorizedArray<Number> &);
+  template <typename Number> DEAL_II_ALWAYS_INLINE ::dealii::VectorizedArray<Number>
+  max(const ::dealii::VectorizedArray<Number> &, const ::dealii::VectorizedArray<Number> &);
+  template <typename Number> DEAL_II_ALWAYS_INLINE ::dealii::VectorizedArray<Number>
+  min (const ::dealii::VectorizedArray<Number> &, const ::dealii::VectorizedArray<Number> &);
+}
+
+DEAL_II_NAMESPACE_OPEN
+
 /**
  * Namespace for the declaration of universal constants. Since the
  * availability in <tt>math.h</tt> is not always guaranteed, we put them here.
@@ -353,7 +373,36 @@ namespace numbers
 
 }
 
+namespace internal
+{
+  /**
+  * The structs below are needed since VectorizedArray<T1> is a POD-type without a constructor and
+  * can be a template argument for SymmetricTensor<...,T2> where T2 would equal VectorizedArray<T1>.
+  * Internally, in previous versions of deal.II, SymmetricTensor<...,T2> would make use of the constructor
+  * of T2 leading to a compile-time error. However simply adding a constructor for VectorizedArray<T1>
+  * breaks the POD-idioms needed elsewhere. Calls to constructors of T2 subsequently got replaced by a
+  * call to internal::NumberType<T2> which then determines the right function to use by template deduction.
+  * A detailled discussion can be found at https://github.com/dealii/dealii/pull/3967 . Also see
+  * numbers.h for another specialization.
+  */
+  template <typename T>
+  struct NumberType
+  {
+    static T value (const T &t)
+    {
+      return t;
+    }
+  };
 
+  template <typename T>
+  struct NumberType<std::complex<T> >
+  {
+    static std::complex<T> value (const T &t)
+    {
+      return std::complex<T>(t);
+    }
+  };
+}
 
 DEAL_II_NAMESPACE_CLOSE
 
