@@ -1,7 +1,6 @@
 // ---------------------------------------------------------------------
-// $Id: quadrature_selector.h 30036 2013-07-18 16:55:32Z maier $
 //
-// Copyright (C) 2003 - 2013 by the deal.II authors
+// Copyright (C) 2003 - 2015 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -15,8 +14,8 @@
 // ---------------------------------------------------------------------
 
 
-#ifndef __deal2__quadrature_selector_h
-#define __deal2__quadrature_selector_h
+#ifndef dealii__quadrature_selector_h
+#define dealii__quadrature_selector_h
 
 
 #include <deal.II/base/quadrature.h>
@@ -27,14 +26,12 @@
 DEAL_II_NAMESPACE_OPEN
 
 /**
- * This class implements the quadrature rule passed to its constructor
- * as a string. Supported quadratures are QGauss (of all
- * orders), QMidpoint, QMilne, QSimpson,
- * QTrapez and QWeddle.
+ * This class implements the quadrature rule passed to its constructor as a
+ * string. Supported quadratures are QGauss (of all orders), QMidpoint,
+ * QMilne, QSimpson, QTrapez and QWeddle.
  *
- * This class is useful if you want to use flexible quadrature rules,
- * that are read from a parameter file (see ParameterHandler for
- * this).
+ * This class is useful if you want to use flexible quadrature rules, that are
+ * read from a parameter file (see ParameterHandler for this).
  *
  * @ingroup Quadrature
  * @author Ralf Schulz, 2003
@@ -44,29 +41,24 @@ class QuadratureSelector : public Quadrature<dim>
 {
 public:
   /**
-   * Constructor. Takes the name of
-   * the quadrature rule (one of
-   * "gauss", "milne", "weddle",
-   * etc) and, if it iss "gauss",
-   * the order of the quadrature
-   * rule as argument.
+   * Constructor. Takes the name of the quadrature rule (one of "gauss",
+   * "milne", "weddle", etc) and, if it is "gauss", the number of quadrature
+   * points in each coordinate direction.
    */
   QuadratureSelector (const std::string &s,
                       const unsigned int order=0);
 
   /**
-   * This function returns all
-   * possible names for quadratures
-   * as a list separated by <tt>|</tt>,
-   * so that you can use it for the
-   * definition of parameter files
-   * (see ParameterHandler for
-   * details).
+   * This function returns all possible names for quadratures as a list
+   * separated by <tt>|</tt>, so that you can use it for the definition of
+   * parameter files (see ParameterHandler for details).
    */
   static std::string get_quadrature_names();
 
-  /** @addtogroup Exceptions
-   * @{ */
+  /**
+   * @addtogroup Exceptions
+   * @{
+   */
 
 
   /**
@@ -74,8 +66,11 @@ public:
    */
   DeclException1 (ExcInvalidQGaussOrder,
                   int,
-                  << "You tried to generate QGauss with an invalid order of "
-                  << arg1 << " (must be >= 2)");
+                  << "You tried to generate a QGauss object with an invalid "
+                  << "number " << arg1
+                  << " of quadrature points in each coordinate "
+                  << "direction. This number must be greater than or equal "
+                  << "to 1.");
   /**
    * Exception
    */
@@ -83,24 +78,21 @@ public:
                   std::string,
                   unsigned int,
                   << "You tried to generate a " << arg1
-                  << " object; no order is needed (" << arg2
-                  << " was given as parameter)");
+                  << " object; no order is needed for objects of this kind, but "
+                  << arg2 << " was given as argument.");
   /**
    * Exception
    */
   DeclException1 (ExcInvalidQuadrature,
                   std::string,
                   << arg1
-                  << " is not a valid quadrature name for a quadrature rule");
+                  << " is not a valid name for a quadrature rule.");
   //@}
 private:
   /**
-   * This static function creates a
-   * quadrature object according to
-   * the name given as a string,
-   * and the appropriate order (if
-   * the name is "gauss"). It is
-   * called from the constructor.
+   * This static function creates a quadrature object according to the name
+   * given as a string, and the appropriate order (if the name is "gauss"). It
+   * is called from the constructor.
    */
   static
   Quadrature<dim>
