@@ -69,6 +69,16 @@ public:
    */
   Subscriptor(const Subscriptor &);
 
+#ifdef DEAL_II_WITH_CXX11
+  /**
+   * Move constructor.
+   *
+   * An object inheriting from Subscriptor can only be moved if no other
+   * objects are subscribing to it.
+   */
+  Subscriptor(Subscriptor &&);
+#endif
+
   /**
    * Destructor, asserting that the counter is zero.
    */
@@ -82,6 +92,15 @@ public:
    */
   Subscriptor &operator = (const Subscriptor &);
 
+#ifdef DEAL_II_WITH_CXX11
+  /**
+   * Move assignment operator.
+   *
+   * Asserts that the counter for the moved object is zero.
+   */
+  Subscriptor &operator = (Subscriptor &&);
+#endif
+
   /**
    * Subscribes a user of the object. The subscriber may be identified by text
    * supplied as <tt>identifier</tt>.
@@ -196,6 +215,13 @@ private:
    * in between and store it here.
    */
   mutable const std::type_info *object_info;
+
+  /**
+   * Check that there are no objects subscribing to this object and throw an
+   * error if there are, in order to guarantee that it is safe to either move
+   * or destroy this object.
+   */
+  void check_no_subscribers () const;
 };
 
 //---------------------------------------------------------------------------
