@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2002 - 2016 by the deal.II authors
+// Copyright (C) 2002 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -444,6 +444,15 @@ public:
   template <typename T2>
   TableBase (const TableBase<N,T2> &src);
 
+#ifdef DEAL_II_WITH_CXX11
+  /**
+   * Move constructor. Transfers the contents of another Table.
+   *
+   * @note This constructor is only available if deal.II is built with C++11.
+   */
+  TableBase (TableBase<N,T> &&src);
+#endif
+
   /**
    * Destructor. Free allocated memory.
    */
@@ -469,6 +478,16 @@ public:
   template<typename T2>
   TableBase<N,T> &operator = (const TableBase<N,T2> &src);
 
+#ifdef DEAL_II_WITH_CXX11
+  /**
+   * Move assignment operator. Transfer all elements of <tt>src</tt> into the
+   * table.
+   *
+   * @note This operator is only available if deal.II is built with C++11.
+   */
+  TableBase<N,T> &operator = (TableBase<N,T> &&src);
+#endif
+
   /**
    * Test for equality of two tables.
    */
@@ -581,9 +600,10 @@ public:
    * swaps the pointers to the data of the two vectors and therefore does not
    * need to allocate temporary storage and move data around.
    *
-   * This function is analog to the the @p swap function of all C++ standard
-   * containers. Also, there is a global function <tt>swap(u,v)</tt> that
-   * simply calls <tt>u.swap(v)</tt>, again in analogy to standard functions.
+   * This function is analogous to the the @p swap function of all C++
+   * standard containers. Also, there is a global function <tt>swap(u,v)</tt>
+   * that simply calls <tt>u.swap(v)</tt>, again in analogy to standard
+   * functions.
    */
   void swap (TableBase<N,T> &v);
 
@@ -1648,6 +1668,22 @@ TableBase<N,T>::TableBase (const TableBase<N,T2> &src)
 
 
 
+#ifdef DEAL_II_WITH_CXX11
+
+template <int N, typename T>
+TableBase<N,T>::TableBase (TableBase<N,T> &&src)
+  :
+  Subscriptor (std::move(src)),
+  values (std::move(src.values)),
+  table_size (src.table_size)
+{
+  src.table_size = TableIndices<N>();
+}
+
+#endif
+
+
+
 template <int N, typename T>
 template <class Archive>
 inline
@@ -1845,6 +1881,26 @@ TableBase<N,T>::operator = (const TableBase<N,T2> &m)
 }
 
 
+
+#ifdef DEAL_II_WITH_CXX11
+
+template <int N, typename T>
+inline
+TableBase<N,T> &
+TableBase<N,T>::operator = (TableBase<N,T> &&m)
+{
+  static_cast<Subscriptor &>(*this) = std::move(m);
+  values = std::move(m.values);
+  table_size = m.table_size;
+  m.table_size = TableIndices<N>();
+
+  return *this;
+}
+
+#endif
+
+
+
 template <int N, typename T>
 inline
 bool
