@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -131,7 +131,7 @@ namespace internal
     value_type value;
 
     /**
-     * cache the current value as a string
+     * Cache the current value as a string.
      */
     mutable std::string cached_value;
 
@@ -322,9 +322,22 @@ public:
    */
   enum TextOutputFormat
   {
+    /**
+     * Print the table with headers.
+     */
     table_with_headers,
+    /**
+     * Print the table with separate lines for each column label.
+     */
     table_with_separate_column_description,
+    /**
+     * Like table_with_separate_column_description, but without aligning the
+     * column containing the column labels.
+     */
     simple_table_with_separate_column_description,
+    /**
+     * Print the table in org mode format.
+     */
     org_mode_table
   };
 
@@ -383,7 +396,7 @@ public:
   void set_column_order (const std::vector<std::string> &new_order);
 
   /**
-   * Sets the <tt>precision</tt> e.g. double or float variables are written
+   * Set the <tt>precision</tt> e.g. double or float variables are written
    * with. <tt>precision</tt> is the same as in calling
    * <tt>out<<setprecision(precision)</tt>.
    */
@@ -391,14 +404,14 @@ public:
                       const unsigned int precision);
 
   /**
-   * Sets the <tt>scientific_flag</tt>. True means scientific, false means
+   * Set the <tt>scientific_flag</tt>. True means scientific, false means
    * fixed point notation.
    */
   void set_scientific (const std::string &key,
                        const bool         scientific);
 
   /**
-   * Sets the caption of the column <tt>key</tt> for tex output. You may want
+   * Set the caption of the column <tt>key</tt> for tex output. You may want
    * to chose this different from <tt>key</tt>, if it contains formulas or
    * similar constructs.
    */
@@ -406,17 +419,17 @@ public:
                         const std::string &tex_caption);
 
   /**
-   * Sets the tex caption of the entire <tt>table</tt> for tex output.
+   * Set the tex caption of the entire <tt>table</tt> for tex output.
    */
   void set_tex_table_caption (const std::string &table_caption);
 
   /**
-   * Sets the label of this <tt>table</tt> for tex output.
+   * Set the label of this <tt>table</tt> for tex output.
    */
   void set_tex_table_label (const std::string &table_label);
 
   /**
-   * Sets the caption the the supercolumn <tt>superkey</tt> for tex output.
+   * Set the caption the the supercolumn <tt>superkey</tt> for tex output.
    * You may want to chose this different from <tt>superkey</tt>, if it
    * contains formulas or similar constructs.
    */
@@ -424,7 +437,7 @@ public:
                              const std::string &tex_supercaption);
 
   /**
-   * Sets the tex output format of a column, e.g. <tt>c</tt>, <tt>r</tt>,
+   * Set the tex output format of a column, e.g. <tt>c</tt>, <tt>r</tt>,
    * <tt>l</tt>, or <tt>p{3cm}</tt>. The default is <tt>c</tt>. Also if this
    * function is not called for a column, the default is preset to be
    * <tt>c</tt>.
@@ -447,15 +460,16 @@ public:
                    const TextOutputFormat format = table_with_headers) const;
 
   /**
-   * Write table as a tex file. If with_header is set to false (it is true by
-   * default), then no "\documentclass{...}", "\begin{document}" and
-   * "\end{document}" are used. In this way the file can be included into an
-   * existing tex file using a command like "\input{table_file}".
+   * Write table as a tex file. If @p with_header is set to false, then no
+   * <code>\\documentclass{...}</code>, <code>\\begin{document}</code> and
+   * <code>\\end{document}</code> are used. In this way the file can be
+   * included into an existing tex file using a command like
+   * <code>\\input{table_file}</code>.
    */
   void write_tex (std::ostream &file, const bool with_header=true) const;
 
   /**
-   * Clears the rows of the table, i.e. calls clear() on all the underlying
+   * Clear the rows of the table, i.e. calls clear() on all the underlying
    * storage data structures.
    */
   void clear ();
@@ -505,8 +519,8 @@ public:
    */
   DeclException4 (ExcWrongNumberOfDataEntries,
                   std::string, int, std::string, int,
-                  << "Column <" << arg1 << "> has got " << arg2
-                  << " rows, but Column <" << arg3 << "> has got " << arg4 << ".");
+                  << "Column <" << arg1 << "> has " << arg2
+                  << " rows, but Column <" << arg3 << "> has " << arg4 << " rows.");
 
   /**
    * Exception
