@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2015 by the deal.II authors
+// Copyright (C) 1998 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -18,6 +18,7 @@
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/conditional_ostream.h>
+#include <deal.II/base/mpi.h>
 #include <deal.II/base/thread_management.h>
 #include <deal.II/base/utilities.h>
 
@@ -101,7 +102,7 @@ public:
          const bool sync_wall_time = false);
 
   /**
-   * Returns a reference to the data structure with global timing information.
+   * Return a reference to the data structure with global timing information.
    * Filled after calling stop().
    */
   const Utilities::MPI::MinMaxAvg &get_data() const;
@@ -122,7 +123,7 @@ public:
   void start ();
 
   /**
-   * Sets the current time as next starting time and return the elapsed time
+   * Set the current time as next starting time and return the elapsed time
    * in seconds.
    */
   double stop ();
@@ -151,7 +152,7 @@ public:
   double wall_time () const;
 
   /**
-   * Returns the last lap time; the time taken between the last start()/stop()
+   * Return the last lap time; the time taken between the last start()/stop()
    * call.
    */
   double get_lap_time () const;
@@ -430,9 +431,21 @@ public:
    */
   enum OutputFrequency
   {
+    /**
+     * Generate output after every call.
+     */
     every_call,
+    /**
+     * Generate output in summary at the end.
+     */
     summary,
+    /**
+     * Generate output both after every call and in summary at the end.
+     */
     every_call_and_summary,
+    /**
+     * Never generate any output.
+     */
     never
   };
 
@@ -442,8 +455,17 @@ public:
    */
   enum OutputType
   {
+    /**
+     * Output CPU times.
+     */
     cpu_times,
+    /**
+     * Output wall clock times.
+     */
     wall_times,
+    /**
+     * Output both CPU and wall clock times.
+     */
     cpu_and_wall_times
   };
 
@@ -730,7 +752,12 @@ TimerOutput::Scope::Scope(dealii::TimerOutput &timer_, const std::string &sectio
 inline
 TimerOutput::Scope::~Scope()
 {
-  stop();
+  try
+    {
+      stop();
+    }
+  catch (...)
+    {}
 }
 
 inline
