@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2015 by the deal.II authors
+// Copyright (C) 2009 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -130,6 +130,12 @@ namespace types
   typedef unsigned char material_id;
 }
 
+/**
+ * Declare type used in Epetra.
+ */
+typedef double TrilinosScalar;
+
+
 namespace TrilinosWrappers
 {
   namespace types
