@@ -18,7 +18,6 @@
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/exceptions.h>
-#include <deal.II/base/mpi.h>
 
 #include <vector>
 #include <utility>
@@ -390,10 +389,25 @@ namespace Utilities
      */
     struct MemoryStats
     {
-      unsigned long int VmPeak; /** peak virtual memory size in kB */
-      unsigned long int VmSize; /** current virtual memory size in kB */
-      unsigned long int VmHWM; /** peak resident memory size in kB */
-      unsigned long int VmRSS; /** current resident memory size in kB */
+      /**
+       * Peak virtual memory size in kB.
+       */
+      unsigned long int VmPeak;
+
+      /**
+       * Current virtual memory size in kB.
+       */
+      unsigned long int VmSize;
+
+      /**
+       * Peak resident memory size in kB. Also known as "high water mark" (HWM).
+       */
+      unsigned long int VmHWM;
+
+      /**
+       * Current resident memory size in kB. Also known as "resident set size" (RSS).
+       */
+      unsigned long int VmRSS;
     };
 
 
@@ -453,7 +467,7 @@ namespace Utilities
   namespace Trilinos
   {
     /**
-     * Returns a Trilinos Epetra_Comm object needed for creation of
+     * Return a Trilinos Epetra_Comm object needed for creation of
      * Epetra_Maps.
      *
      * If deal.II has been configured to use a compiler that does not support
@@ -464,7 +478,7 @@ namespace Utilities
     const Epetra_Comm &comm_world();
 
     /**
-     * Returns a Trilinos Epetra_Comm object needed for creation of
+     * Return a Trilinos Epetra_Comm object needed for creation of
      * Epetra_Maps.
      *
      * If deal.II has been configured to use a compiler that does not support
