@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2015 by the deal.II authors
+// Copyright (C) 2009 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -30,10 +30,6 @@ namespace parallel
 {
   namespace distributed
   {
-    // forward declarations
-    template <int dim, int spacedim> class Triangulation;
-
-
     /**
      * Collection of functions controlling refinement and coarsening of
      * parallel::distributed::Triangulation objects. This namespace provides
@@ -47,18 +43,20 @@ namespace parallel
     {
       /**
        * Like dealii::GridRefinement::refine_and_coarsen_fixed_number, but for
-       * parallel distributed triangulation.
+       * parallel distributed triangulations.
        *
        * The vector of criteria needs to be a vector of refinement criteria
-       * for all cells active on the current triangulation, i.e.
-       * <code>tria.n_active_cells()</code> (and not
-       * <code>tria.n_locally_owned_active_cells()</code>). However, the
+       * for all cells active on the current triangulation, i.e.,
+       * it needs to be of length <code>tria.n_active_cells()</code> (and not
+       * <code>tria.n_locally_owned_active_cells()</code>). In other words,
+       * the vector needs to include entries for ghost and artificial
+       * cells. However, the current
        * function will only look at the indicators that correspond to those
        * cells that are actually locally owned, and ignore the indicators for
        * all other cells. The function will then coordinate among all
-       * processors that store part of the triangulation so that at the end @p
-       * top_fraction_of_cells are refined, where the fraction is enforced as
-       * a fraction of Triangulation::n_global_active_cells, not
+       * processors that store part of the triangulation so that at the end
+       * a fraction @p top_fraction_of_cells of all Triangulation::n_global_active_cells()
+       * active cells are refined, rather than a fraction of the
        * Triangulation::n_locally_active_cells on each processor individually.
        * In other words, it may be that on some processors, no cells are
        * refined at all.
@@ -76,18 +74,20 @@ namespace parallel
 
       /**
        * Like dealii::GridRefinement::refine_and_coarsen_fixed_fraction, but
-       * for parallel distributed triangulation.
+       * for parallel distributed triangulations.
        *
        * The vector of criteria needs to be a vector of refinement criteria
-       * for all cells active on the current triangulation,
-       * <code>tria.n_active_cells()</code> (and not
-       * <code>tria.n_locally_owned_active_cells()</code>). However, the
+       * for all cells active on the current triangulation, i.e.,
+       * it needs to be of length <code>tria.n_active_cells()</code> (and not
+       * <code>tria.n_locally_owned_active_cells()</code>). In other words,
+       * the vector needs to include entries for ghost and artificial
+       * cells. However, the current
        * function will only look at the indicators that correspond to those
        * cells that are actually locally owned, and ignore the indicators for
        * all other cells. The function will then coordinate among all
        * processors that store part of the triangulation so that at the end
        * the smallest fraction of Triangulation::n_global_active_cells (not
-       * Triangulation::n_locally_active_cells on each processor individually)
+       * Triangulation::n_locally_owned_active_cells() on each processor individually)
        * is refined that together make up a total of @p top_fraction_of_error
        * of the total error. In other words, it may be that on some
        * processors, no cells are refined at all.
