@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2016 by the deal.II authors
+// Copyright (C) 2008 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -27,6 +27,7 @@
 #include <deal.II/base/std_cxx11/tuple.h>
 
 #include <deal.II/distributed/tria_base.h>
+#include <deal.II/distributed/p4est_wrappers.h>
 
 #include <set>
 #include <vector>
@@ -50,8 +51,6 @@
 
 DEAL_II_NAMESPACE_OPEN
 
-template <int, int> class Triangulation;
-
 #ifdef DEAL_II_WITH_P4EST
 
 namespace internal
@@ -65,93 +64,11 @@ namespace internal
   }
 }
 
-
-namespace internal
+namespace FETools
 {
-  namespace p4est
+  namespace internal
   {
-    /**
-     * A structure whose explicit specializations contain typedefs to the
-     * relevant p4est_* and p8est_* types. Using this structure, for example
-     * by saying <tt>types<dim>::connectivity</tt> we can write code in a
-     * dimension independent way, either referring to p4est_connectivity_t or
-     * p8est_connectivity_t, depending on template argument.
-     */
-    template <int> struct types;
-
-    template <>
-    struct types<2>
-    {
-      typedef p4est_connectivity_t connectivity;
-      typedef p4est_t              forest;
-      typedef p4est_tree_t         tree;
-      typedef p4est_quadrant_t     quadrant;
-      typedef p4est_topidx_t       topidx;
-      typedef p4est_locidx_t       locidx;
-#if DEAL_II_P4EST_VERSION_GTE(0,3,4,3)
-      typedef p4est_connect_type_t balance_type;
-#else
-      typedef p4est_balance_type_t balance_type;
-#endif
-      typedef p4est_ghost_t        ghost;
-    };
-
-    template <>
-    struct types<3>
-    {
-      typedef p8est_connectivity_t connectivity;
-      typedef p8est_t              forest;
-      typedef p8est_tree_t         tree;
-      typedef p8est_quadrant_t     quadrant;
-      typedef p4est_topidx_t       topidx;
-      typedef p4est_locidx_t       locidx;
-#if DEAL_II_P4EST_VERSION_GTE(0,3,4,3)
-      typedef p8est_connect_type_t balance_type;
-#else
-      typedef p8est_balance_type_t balance_type;
-#endif
-      typedef p8est_ghost_t        ghost;
-    };
-
-
-    /**
-     * Initialize the GeometryInfo<dim>::max_children_per_cell children of the
-     * cell p4est_cell.
-     */
-    template <int dim>
-    void
-    init_quadrant_children
-    (const typename types<dim>::quadrant &p4est_cell,
-     typename types<dim>::quadrant (&p4est_children)[GeometryInfo<dim>::max_children_per_cell]);
-
-
-    /**
-     * Initialize quadrant to represent a coarse cell.
-     */
-    template <int dim>
-    void
-    init_coarse_quadrant(typename types<dim>::quadrant &quad);
-
-
-
-    /**
-     * Returns whether q1 and q2 are equal
-     */
-    template <int dim>
-    bool
-    quadrant_is_equal (const typename types<dim>::quadrant &q1,
-                       const typename types<dim>::quadrant &q2);
-
-    //TODO: remove these functions from
-    //public interface somehow? [TH]
-
-    /**
-     * returns whether q1 is an ancestor of q2
-     */
-    template <int dim>
-    bool
-    quadrant_is_ancestor (const typename types<dim>::quadrant &q1,
-                          const typename types<dim>::quadrant &q2);
+    template <int, int, class> class ExtrapolateImplementation;
   }
 }
 
@@ -329,7 +246,7 @@ namespace parallel
     {
     public:
       /**
-       * A typedef that is used to to identify cell iterators. The concept of
+       * A typedef that is used to identify cell iterators. The concept of
        * iterators is discussed at length in the
        * @ref Iterators "iterators documentation module".
        *
@@ -348,7 +265,7 @@ namespace parallel
       typedef typename dealii::Triangulation<dim,spacedim>::cell_iterator        cell_iterator;
 
       /**
-       * A typedef that is used to to identify
+       * A typedef that is used to identify
        * @ref GlossActive "active cell iterators".
        * The concept of iterators is discussed at length in the
        * @ref Iterators "iterators documentation module".
@@ -381,7 +298,7 @@ namespace parallel
         default_setting = 0x0,
         /**
          * If set, the deal.II mesh will be reconstructed from the coarse mesh
-         * every time a repartioning in p4est happens. This can be a bit more
+         * every time a repartitioning in p4est happens. This can be a bit more
          * expensive, but guarantees the same memory layout and therefore cell
          * ordering in the deal.II mesh. As assembly is done in the deal.II
          * cell ordering, this flag is required to get reproducible behaviour
@@ -396,7 +313,7 @@ namespace parallel
          */
         construct_multigrid_hierarchy = 0x2,
         /**
-         * Setting this flag will disable automatic repartioning of the cells
+         * Setting this flag will disable automatic repartitioning of the cells
          * after a refinement cycle. It can be executed manually by calling
          * repartition().
          */
@@ -416,6 +333,9 @@ namespace parallel
        * the kinds of smoothing operations that can be applied.
        *
        * @param settings See the description of the Settings enumerator.
+       * Providing <code>construct_multigrid_hierarchy</code> enforces
+       * <code>Triangulation::limit_level_difference_at_vertices</code>
+       * for smooth_grid.
        *
        * @note This class does not currently support the
        * <code>check_for_distorted_cells</code> argument provided by the base
@@ -453,8 +373,14 @@ namespace parallel
 
       /**
        * Implementation of the same function as in the base class.
+       *
+       * @note This function cannot copy a triangulation that has been refined.
+       *
+       * @note This function can be used to copy a serial Triangulation to a
+       * parallel::distributed::Triangulation but only if the serial
+       * Triangulation has never been refined.
        */
-      virtual void copy_triangulation (const dealii::Triangulation<dim, spacedim> &old_tria);
+      virtual void copy_triangulation (const dealii::Triangulation<dim, spacedim> &other_tria);
 
       /**
        * Create a triangulation as documented in the base class.
@@ -607,7 +533,7 @@ namespace parallel
 
 
       /**
-       * Returns true if the triangulation has hanging nodes.
+       * Return true if the triangulation has hanging nodes.
        *
        * In the context of parallel distributed triangulations, every
        * processor stores only that part of the triangulation it locally owns.
@@ -700,11 +626,12 @@ namespace parallel
        * can be different than the coarsen and refine flags set by you). If it
        * is
        *
-       * - CELL_PERIST: the cell won't be refined/coarsened, but might be
-       * moved to a different processor - CELL_REFINE: this cell will be
-       * refined into 4/8 cells, you can not access the children (because they
-       * don't exist yet) - CELL_COARSEN: the children of this cell will be
-       * coarsened into the given cell (you can access the active children!)
+       * - CELL_PERSIST: the cell won't be refined/coarsened, but might be
+       * moved to a different processor
+       * - CELL_REFINE: this cell will be refined into 4/8 cells, you can not
+       * access the children (because they don't exist yet)
+       * - CELL_COARSEN: the children of this cell will be coarsened into the
+       * given cell (you can access the active children!)
        *
        * When unpacking the data with notify_ready_to_unpack() you can access
        * the children of the cell if the status is CELL_REFINE but not for
@@ -759,7 +686,8 @@ namespace parallel
       get_coarse_cell_to_p4est_tree_permutation() const;
 
       /**
-       * Join faces in the p4est forest for periodic boundary conditions. As a
+       * In addition to the action in the base class Triangulation, this function
+       * joins faces in the p4est forest for periodic boundary conditions. As a
        * result, each pair of faces will differ by at most one refinement
        * level and ghost neighbors will be available across these faces.
        *
@@ -775,7 +703,7 @@ namespace parallel
        * once is possible, but not recommended: The function destroys and
        * rebuilds the p4est forest each time it is called.
        */
-      void
+      virtual void
       add_periodicity
       (const std::vector<GridTools::PeriodicFacePair<cell_iterator> > &);
 
@@ -879,13 +807,6 @@ namespace parallel
       std::vector<types::global_dof_index> coarse_cell_to_p4est_tree_permutation;
       std::vector<types::global_dof_index> p4est_tree_to_coarse_cell_permutation;
 
-      /**
-       * If add_periodicity() is called, this variable stores the given
-       * periodic face pairs on level 0 for later access during the
-       * identification of ghost cells for the multigrid hierarchy.
-       */
-      std::vector<GridTools::PeriodicFacePair<cell_iterator> > periodic_face_pairs_level_0;
-
       /**
        * Return a pointer to the p4est tree that belongs to the given
        * dealii_coarse_cell_index()
@@ -954,7 +875,7 @@ namespace parallel
        */
       void
       fill_level_vertices_with_ghost_neighbors
-      (const unsigned int level,
+      (const int level,
        std::map<unsigned int, std::set<dealii::types::subdomain_id> >
        &vertices_with_ghost_neighbors);
 
@@ -968,9 +889,11 @@ namespace parallel
        * Used by DoFHandler::Policy::ParallelDistributed.
        */
       std::vector<bool>
-      mark_locally_active_vertices_on_level(const unsigned int level) const;
+      mark_locally_active_vertices_on_level(const int level) const;
 
       template <int, int> friend class dealii::internal::DoFHandler::Policy::ParallelDistributed;
+
+      template <int,int,class> friend class dealii::FETools::internal::ExtrapolateImplementation;
     };
 
 
@@ -995,7 +918,7 @@ namespace parallel
       virtual ~Triangulation ();
 
       /**
-       * Returns a permutation vector for the order the coarse cells are
+       * Return a permutation vector for the order the coarse cells are
        * handed of to p4est. For example the first element i in this vector
        * denotes that the first cell in hierarchical ordering is the ith deal
        * cell starting from begin(0).
