@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2016 by the deal.II authors
+// Copyright (C) 1998 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -27,7 +27,6 @@
 DEAL_II_NAMESPACE_OPEN
 
 template <typename number> class FullMatrix;
-template <typename number> class SparseMatrix;
 template <typename number> class Vector;
 class ConstraintMatrix;
 
@@ -176,6 +175,23 @@ namespace internal
  * There is a DoFCellAccessor class that provides the equivalent to the
  * CellAccessor class.
  *
+ * @tparam structdim The dimensionality of the objects the accessor
+ *   represents. For example, points have @p structdim equal to zero,
+ *   edges have @p structdim equal to one, etc.
+ * @tparam DoFHandlerType The type of the DoF handler into which accessor
+ *   of this type point. This is either the DoFHandler or hp::DoFHandler
+ *   class. See also the
+ *   @ref ConceptDoFHandlerType "DoFHandlerType concept".
+ * @tparam level_dof_access If @p false, then the accessor simply represents
+ *   a cell, face, or edge in a DoFHandler for which degrees of freedom only
+ *   exist on the finest level. Some operations are not allowed in this case,
+ *   such as asking for DoF indices on non-active cells. On the other hand,
+ *   if this template argument is @p true, then the accessor represents an
+ *   object in a multilevel hierarchy of degrees of freedom. In this case,
+ *   accessing DoF indices of <i>any</i> cell is possible, and will return
+ *   the <i>level</i> indices (which, for active cells, may be different
+ *   from the <i>global</i> indices).
+ *
  * @ingroup dofs
  * @ingroup Accessors
  * @author Wolfgang Bangerth, 1998, 2006, 2008, Timo Heister, Guido Kanschat,
@@ -224,12 +240,25 @@ public:
   DoFAccessor ();
 
   /**
-   * Constructor
+   * Constructor that generates an access that points to a particular cell or
+   * face or edge in a DoFHandler or hp::DoFHandler.
+   *
+   * @param tria The triangulation into which this accessor points.
+   * @param level The level within the mesh hierarchy of the object pointed
+   *   to. For example, coarse mesh cells will have level zero, their children
+   *   level one, and so on. This argument is ignored for faces and edges
+   *   which do not have a level.
+   * @param index The index of the object pointed to within the specified
+   *   refinement level.
+   * @param dof_handler A pointer to the DoFHandler or hp::DoFHandler object
+   *   to which the accessor shall refer. This DoFHandler object must of
+   *   course be built on the same triangulation as the one specified in
+   *   the first argument.
    */
   DoFAccessor (const Triangulation<DoFHandlerType::dimension,DoFHandlerType::space_dimension> *tria,
                const int             level,
                const int             index,
-               const DoFHandlerType *local_data);
+               const DoFHandlerType *dof_handler);
 
   /**
    * Conversion constructor. This constructor exists to make certain
@@ -374,7 +403,7 @@ public:
                            const unsigned int fe_index = DoFHandlerType::default_fe_index) const;
 
   /**
-   * Sets the level DoF indices that are returned by get_mg_dof_indices.
+   * Set the level DoF indices that are returned by get_mg_dof_indices.
    */
   void set_mg_dof_indices (const int level,
                            const std::vector<types::global_dof_index> &dof_indices,
@@ -403,7 +432,7 @@ public:
    const unsigned int fe_index = DoFHandlerType::default_fe_index) const;
 
   /**
-   * Returns the global DoF index of the <code>i</code>th degree of freedom
+   * Return the global DoF index of the <code>i</code>th degree of freedom
    * associated with the <code>vertex</code>th vertex on level @p level. Also
    * see vertex_dof_index().
    */
@@ -445,7 +474,7 @@ public:
    const unsigned int fe_index = DoFHandlerType::default_fe_index) const;
 
   /**
-   * Returns the dof_index on the given level. Also see dof_index.
+   * Return the dof_index on the given level. Also see dof_index.
    */
   types::global_dof_index mg_dof_index (const int level, const unsigned int i) const;
 
@@ -816,17 +845,19 @@ public:
   child (const unsigned int c) const;
 
   /**
-   * Pointer to the @p ith line bounding this object. If the current object is
-   * a line itself, then the only valid index is @p i equals to zero, and the
-   * function returns an iterator to itself.
+   * Pointer to the @p ith line bounding this object.
+   *
+   * Since meshes with dimension 1 do not have quads this method just throws
+   * an exception.
    */
   typename dealii::internal::DoFHandler::Iterators<DoFHandlerType<1,spacedim>, level_dof_access>::line_iterator
   line (const unsigned int i) const;
 
   /**
-   * Pointer to the @p ith quad bounding this object. If the current object is
-   * a quad itself, then the only valid index is @p i equals to zero, and the
-   * function returns an iterator to itself.
+   * Pointer to the @p ith quad bounding this object.
+   *
+   * Since meshes with dimension 1 do not have quads this method just throws
+   * an exception.
    */
   typename dealii::internal::DoFHandler::Iterators<DoFHandlerType<1,spacedim>, level_dof_access>::quad_iterator
   quad (const unsigned int i) const;
@@ -844,14 +875,12 @@ public:
 
   /**
    * Return the <i>global</i> indices of the degrees of freedom located on
-   * this object in the standard ordering defined by the finite element (i.e.,
-   * dofs on vertex 0, dofs on vertex 1, etc, dofs on line 0, dofs on line 1,
-   * etc, dofs on quad 0, etc.) This function is only available on
-   * <i>active</i> objects (see
+   * this object in the standard ordering defined by the finite element. This
+   * function is only available on <i>active</i> objects (see
    * @ref GlossActive "this glossary entry").
    *
-   * The cells needs to be an active cell (and not artificial in a parallel
-   * distributed computation).
+   * The present vertex must belong to an active cell (and not artificial in a
+   * parallel distributed computation).
    *
    * The vector has to have the right size before being passed to this
    * function.
@@ -878,6 +907,16 @@ public:
   void get_dof_indices (std::vector<types::global_dof_index> &dof_indices,
                         const unsigned int fe_index = AccessorData::default_fe_index) const;
 
+  /**
+   * Return the global multilevel indices of the degrees of freedom that live
+   * on the current object with respect to the given level within the
+   * multigrid hierarchy. The indices refer to the local numbering for the
+   * level this line lives on.
+   */
+  void get_mg_dof_indices (const int level,
+                           std::vector<types::global_dof_index> &dof_indices,
+                           const unsigned int fe_index = AccessorData::default_fe_index) const;
+
   /**
    * Global DoF index of the <i>i</i> degree associated with the @p vertexth
    * vertex of the present cell.
@@ -911,20 +950,7 @@ public:
    * cells, as well as vertices, there may therefore be two sets of degrees of
    * freedom, one for each of the finite elements used on the adjacent cells.
    * In order to specify which set of degrees of freedom to work on, the last
-   * argument is used to disambiguate. Finally, if this function is called for
-   * a cell object, there can only be a single set of degrees of freedom, and
-   * fe_index has to match the result of active_fe_index().
-   *
-   * @note While the get_dof_indices() function returns an array that contains
-   * the indices of all degrees of freedom that somehow live on this object
-   * (i.e. on the vertices, edges or interior of this object), the current
-   * dof_index() function only considers the DoFs that really belong to this
-   * particular object's interior. In other words, as an example, if the
-   * current object refers to a quad (a cell in 2d, a face in 3d) and the
-   * finite element associated with it is a bilinear one, then the
-   * get_dof_indices() will return an array of size 4 while dof_index() will
-   * produce an exception because no degrees are defined in the interior of
-   * the face.
+   * argument is used to disambiguate.
    */
   types::global_dof_index dof_index (const unsigned int i,
                                      const unsigned int fe_index = AccessorData::default_fe_index) const;
@@ -943,33 +969,30 @@ public:
   /**
    * Return the number of finite elements that are active on a given object.
    *
-   * For non-hp DoFHandler objects, the answer is of course always one.
-   * However, for hp::DoFHandler objects, this isn't the case: If this is a
-   * cell, the answer is of course one. If it is a face, the answer may be one
-   * or two, depending on whether the two adjacent cells use the same finite
-   * element or not. If it is an edge in 3d, the possible return value may be
-   * one or any other value larger than that.
+   * Since vertices do not store the information necessary for this to be
+   * calculated, this method just raises an exception and only exists to
+   * enable dimension-independent programming.
    */
   unsigned int
   n_active_fe_indices () const;
 
   /**
-   * Return the @p n-th active fe index on this object. For cells and all non-
-   * hp objects, there is only a single active fe index, so the argument must
-   * be equal to zero. For lower-dimensional hp objects, there are
-   * n_active_fe_indices() active finite elements, and this function can be
-   * queried for their indices.
+   * Return the @p n-th active fe index on this object.
+   *
+   * Since vertices do not store the information necessary for this to be
+   * calculated, this method just raises an exception and only exists to
+   * enable dimension-independent programming.
    */
   unsigned int
   nth_active_fe_index (const unsigned int n) const;
 
   /**
    * Return true if the finite element with given index is active on the
-   * present object. For non-hp DoF accessors, this is of course the case only
-   * if @p fe_index equals zero. For cells, it is the case if @p fe_index
-   * equals active_fe_index() of this cell. For faces and other lower-
-   * dimensional objects, there may be more than one @p fe_index that are
-   * active on any given object (see n_active_fe_indices()).
+   * present object.
+   *
+   * Since vertices do not store the information necessary for this to be
+   * calculated, this method just raises an exception and only exists to
+   * enable dimension-independent programming.
    */
   bool
   fe_index_is_active (const unsigned int fe_index) const;
@@ -1234,7 +1257,23 @@ public:
    * accessor without access to the DoF data.
    */
   TriaIterator<DoFCellAccessor<DoFHandlerType, level_dof_access> >
-  neighbor (const unsigned int) const;
+  neighbor (const unsigned int i) const;
+
+  /**
+   * Return the @p ith periodic neighbor as a DoF cell iterator. This function
+   * is needed since the neighbor function of the base class returns a cell
+   * accessor without access to the DoF data.
+   */
+  TriaIterator<DoFCellAccessor<DoFHandlerType, level_dof_access> >
+  periodic_neighbor (const unsigned int i) const;
+
+  /**
+   * Return the @p ith neighbor or periodic neighbor as a DoF cell iterator.
+   * This function is needed since the neighbor function of the base class
+   * returns a cell accessor without access to the DoF data.
+   */
+  TriaIterator<DoFCellAccessor<DoFHandlerType, level_dof_access> >
+  neighbor_or_periodic_neighbor (const unsigned int i) const;
 
   /**
    * Return the @p ith child as a DoF cell iterator. This function is needed
@@ -1242,13 +1281,13 @@ public:
    * without access to the DoF data.
    */
   TriaIterator<DoFCellAccessor<DoFHandlerType, level_dof_access> >
-  child (const unsigned int) const;
+  child (const unsigned int i) const;
 
   /**
    * Return an iterator to the @p ith face of this cell.
    *
-   * This function is not implemented in 1D, and returns DoFAccessor::line in
-   * 2D and DoFAccessor::quad in 3d.
+   * This function returns a DoFAccessor with <code>structdim == 0</code> in
+   * 1D, a DoFAccessor::line in 2D, and a DoFAccessor::quad in 3d.
    */
   face_iterator
   face (const unsigned int i) const;
@@ -1263,6 +1302,16 @@ public:
   neighbor_child_on_subface (const unsigned int face_no,
                              const unsigned int subface_no) const;
 
+  /**
+   * Return the result of the @p periodic_neighbor_child_on_subface function
+   * of the base class, but convert it so that one can also access the DoF
+   * data (the function in the base class only returns an iterator with access
+   * to the triangulation data).
+   */
+  TriaIterator<DoFCellAccessor<DoFHandlerType, level_dof_access> >
+  periodic_neighbor_child_on_subface (const unsigned int face_no,
+                                      const unsigned int subface_no) const;
+
   /**
    * @}
    */
@@ -1619,7 +1668,7 @@ public:
   get_fe () const;
 
   /**
-   * Returns the index inside the hp::FECollection of the FiniteElement used
+   * Return the index inside the hp::FECollection of the FiniteElement used
    * for this cell. This function is only useful if the DoF handler object
    * associated with the current cell is an hp::DoFHandler.
    *
@@ -1633,7 +1682,7 @@ public:
   unsigned int active_fe_index () const;
 
   /**
-   * Sets the index of the FiniteElement used for this cell. This determines
+   * Set the index of the FiniteElement used for this cell. This determines
    * which element in an hp::FECollection to use. This function is only useful
    * if the DoF handler object associated with the current cell is an
    * hp::DoFHandler.
