@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2016 by the deal.II authors
+// Copyright (C) 1998 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -39,6 +39,9 @@
 
 DEAL_II_NAMESPACE_OPEN
 
+template <int dim, int spacedim> class FiniteElement;
+template <int dim, int spacedim> class Triangulation;
+
 namespace internal
 {
   namespace DoFHandler
@@ -65,8 +68,12 @@ namespace internal
 
 
 /**
- * Manage the distribution and numbering of the degrees of freedom for non-
- * multigrid algorithms. This class satisfies the
+ * Given a triangulation and a description of a finite element, this
+ * class enumerates degrees of freedom on all vertices, edges, faces,
+ * and cells of the triangulation. As a result, it also provides a
+ * <i>basis</i> for a discrete space $V_h$ whose elements are finite
+ * element functions defined on each cell by a FiniteElement object.
+ * This class satisfies the
  * @ref ConceptMeshType "MeshType concept"
  * requirements.
  *
@@ -199,7 +206,7 @@ public:
   typedef typename ActiveSelector::active_hex_iterator  active_hex_iterator;
 
   /**
-   * A typedef that is used to to identify
+   * A typedef that is used to identify
    * @ref GlossActive "active cell iterators".
    * The concept of iterators is discussed at length in the
    * @ref Iterators "iterators documentation module".
@@ -221,7 +228,7 @@ public:
   typedef typename ActiveSelector::active_cell_iterator active_cell_iterator;
 
   /**
-   * A typedef that is used to to identify cell iterators. The concept of
+   * A typedef that is used to identify cell iterators. The concept of
    * iterators is discussed at length in the
    * @ref Iterators "iterators documentation module".
    *
@@ -258,11 +265,6 @@ public:
   typedef typename LevelSelector::face_iterator         level_face_iterator;
 
 
-  /**
-   * Alias the @p FunctionMap type declared elsewhere.
-   */
-  typedef typename dealii::FunctionMap<spacedim>::type FunctionMap;
-
   /**
    * Make the dimension available in function templates.
    */
@@ -706,9 +708,12 @@ public:
    * reason that a @p map rather than a @p set is used is the same as
    * described in the section on the @p make_boundary_sparsity_pattern
    * function.
+   *
+   * The type of boundary_ids equals typename FunctionMap<spacedim,number>::type .
    */
+  template<typename number>
   types::global_dof_index
-  n_boundary_dofs (const FunctionMap &boundary_ids) const;
+  n_boundary_dofs (const std::map<types::boundary_id, const Function<spacedim,number>*> &boundary_ids) const;
 
   /**
    * Same function, but with different data type of the argument, which is
@@ -758,14 +763,14 @@ public:
   const IndexSet &locally_owned_dofs() const;
 
   /**
-   * Returns an IndexSet describing the set of locally owned DoFs used for the
+   * Return an IndexSet describing the set of locally owned DoFs used for the
    * given multigrid level as a subset of 0..n_dofs(level).
    */
   const IndexSet &locally_owned_mg_dofs(const unsigned int level) const;
 
 
   /**
-   * Returns a vector that stores the locally owned DoFs of each processor. If
+   * Return a vector that stores the locally owned DoFs of each processor. If
    * you are only interested in the number of elements each processor owns
    * then n_locally_owned_dofs_per_processor() is a better choice.
    *
@@ -1088,6 +1093,16 @@ private:
 
   friend struct dealii::internal::DoFHandler::Implementation;
   friend struct dealii::internal::DoFHandler::Policy::Implementation;
+
+  // explicitly check for sensible template arguments, but not on windows
+  // because MSVC creates bogus warnings during normal compilation
+#ifdef DEAL_II_WITH_CXX11
+#ifndef DEAL_II_MSVC
+  static_assert (dim<=spacedim,
+                 "The dimension <dim> of a DoFHandler must be less than or "
+                 "equal to the space dimension <spacedim> in which it lives.");
+#endif
+#endif
 };
 
 
@@ -1098,7 +1113,6 @@ private:
 #ifndef DOXYGEN
 
 template <> types::global_dof_index DoFHandler<1>::n_boundary_dofs () const;
-template <> types::global_dof_index DoFHandler<1>::n_boundary_dofs (const FunctionMap &) const;
 template <> types::global_dof_index DoFHandler<1>::n_boundary_dofs (const std::set<types::boundary_id> &) const;
 
 template <> void DoFHandler<1>::renumber_dofs(unsigned int,const std::vector<types::global_dof_index>  &new_numbers);
@@ -1232,6 +1246,24 @@ DoFHandler<dim,spacedim>::block_info () const
 }
 
 
+template <int dim, int spacedim>
+template <typename number>
+types::global_dof_index
+DoFHandler<dim,spacedim>::n_boundary_dofs (const std::map<types::boundary_id, const Function<spacedim,number>*> &boundary_ids) const
+{
+  // extract the set of boundary ids and forget about the function object pointers
+  std::set<types::boundary_id> boundary_ids_only;
+  for (typename std::map<types::boundary_id, const Function<spacedim,number>*>::const_iterator
+       p = boundary_ids.begin();
+       p != boundary_ids.end(); ++p)
+    boundary_ids_only.insert (p->first);
+
+  // then just hand everything over to the other function that does the work
+  return n_boundary_dofs(boundary_ids_only);
+}
+
+
+
 namespace internal
 {
   /**
@@ -1243,7 +1275,6 @@ namespace internal
    */
   template<int dim, int spacedim>
   std::string policy_to_string(const dealii::internal::DoFHandler::Policy::PolicyBase<dim,spacedim> &policy);
-
 }
 
 
