@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2015 by the deal.II authors
+// Copyright (C) 1998 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -26,13 +26,9 @@ template <int, int, int> class InvalidAccessor;
 template <int structdim, typename DoFHandlerType, bool lda> class DoFAccessor;
 template <typename DoFHandlerType, bool lda> class DoFCellAccessor;
 
-template <int dim, int spacedim> class FiniteElement;
 template <typename Accessor> class TriaRawIterator;
 template <typename Accessor> class TriaIterator;
 template <typename Accessor> class TriaActiveIterator;
-template <int dim, int spacedim> class Triangulation;
-template <int dim, int spacedim> class DoFHandler;
-
 
 namespace internal
 {
