@@ -33,9 +33,6 @@
 
 DEAL_II_NAMESPACE_OPEN
 
-template<int dim, int spacedim> class FESystem;
-
-
 /**
  * A namespace solely for the purpose of defining the Domination enum as well
  * as associated operators.
@@ -97,10 +94,25 @@ namespace FiniteElementDomination
    */
   enum Domination
   {
+    /**
+     * The current element dominates.
+     */
     this_element_dominates,
+    /**
+     * The other element dominates.
+     */
     other_element_dominates,
+    /**
+     * Neither element dominates.
+     */
     neither_element_dominates,
+    /**
+     * Either element may dominate.
+     */
     either_element_can_dominate,
+    /**
+     * There are no requirements.
+     */
     no_requirements
   };
 
@@ -416,16 +428,6 @@ public:
    */
   const BlockIndices &block_indices() const;
 
-  /**
-   * Return whether the entire finite element is primitive, in the sense that
-   * all its shape functions are primitive. If the finite element is scalar,
-   * then this is always the case.
-   *
-   * Since this is an extremely common operation, the result is cached in the
-   * #cached_primitivity variable which is computed in the constructor.
-   */
-  bool is_primitive () const;
-
   /**
    * Maximal polynomial degree of a shape function in a single coordinate
    * direction.
@@ -446,24 +448,6 @@ public:
    * Comparison operator.
    */
   bool operator == (const FiniteElementData &) const;
-
-protected:
-
-  /**
-   * Set the primitivity of the element. This is usually done by the
-   * constructor of a derived class.  See
-   * @ref GlossPrimitive "primitive"
-   * for details.
-   */
-  void set_primitivity(const bool value);
-
-private:
-  /**
-   * Store whether all shape functions are primitive. Since finding this out
-   * is a very common operation, we cache the result, i.e. compute the value
-   * in the constructor for simpler access.
-   */
-  bool cached_primitivity;
 };
 
 
@@ -619,24 +603,6 @@ FiniteElementData<dim>::n_components () const
 
 
 
-template <int dim>
-inline
-bool
-FiniteElementData<dim>::is_primitive () const
-{
-  return cached_primitivity;
-}
-
-
-template <int dim>
-inline
-void
-FiniteElementData<dim>::set_primitivity (const bool value)
-{
-  cached_primitivity = value;
-}
-
-
 template <int dim>
 inline
 const BlockIndices &
