@@ -20,13 +20,10 @@
 #include <deal.II/base/polynomial.h>
 #include <deal.II/base/polynomial_space.h>
 #include <deal.II/fe/fe.h>
+#include <deal.II/fe/mapping.h>
 
 DEAL_II_NAMESPACE_OPEN
 
-template <int dim> class PolynomialSpace;
-template <int dim, int spacedim> class MappingQ;
-
-
 /*!@addtogroup fe */
 /*@{*/
 
@@ -471,8 +468,8 @@ public:
    * meet at a common face, whether it is the other way around, whether
    * neither dominates, or if either could dominate.
    *
-   * For a definition of domination, see FiniteElementBase::Domination and in
-   * particular the
+   * For a definition of domination, see FiniteElementDomination::Domination
+   * and in particular the
    * @ref hp_paper "hp paper".
    */
   virtual
@@ -600,11 +597,6 @@ private:
   std::vector<unsigned int>
   get_dpo_vector (const unsigned int degree);
 
-  /**
-   * Degree of the polynomials.
-   */
-  const unsigned int degree;
-
   /**
    * Pointer to an object representing the polynomial space used here.
    */
