@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2001 - 2015 by the deal.II authors
+// Copyright (C) 2001 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -75,6 +75,30 @@ template <int dim> class Quadrature;
  * element are exactly the same as those of the FE_Q element where they are
  * shown visually.
  *
+ * <h3>Unit support point distribution and conditioning of interpolation</h3>
+ *
+ * When constructing an FE_DGQ element at polynomial degrees one or two,
+ * equidistant support points at 0 and 1 (linear case) or 0, 0.5, and 1
+ * (quadratic case) are used. The unit support or nodal points
+ * <i>x<sub>i</sub></i> are those points where the <i>j</i>th Lagrange
+ * polynomial satisfies the $\delta_{ij}$ property, i.e., where one polynomial
+ * is one and all the others are zero.  For higher polynomial degrees, the
+ * support points are non-equidistant by default, and chosen to be the support
+ * points of the <tt>(degree+1)</tt>-order Gauss-Lobatto quadrature rule. This
+ * point distribution yields well-conditioned Lagrange interpolation at
+ * arbitrary polynomial degrees. By contrast, polynomials based on equidistant
+ * points get increasingly ill-conditioned as the polynomial degree
+ * increases. In interpolation, this effect is known as the Runge
+ * phenomenon. For Galerkin methods, the Runge phenomenon is typically not
+ * visible in the solution quality but rather in the condition number of the
+ * associated system matrices. For example, the elemental mass matrix of
+ * equidistant points at degree 10 has condition number 2.6e6, whereas the
+ * condition number for Gauss-Lobatto points is around 400.
+ *
+ * The Gauss-Lobatto points in 1D include the end points 0 and +1 of the unit
+ * interval. The interior points are shifted towards the end points, which
+ * gives a denser point distribution close to the element boundary.
+ *
  * @author Ralf Hartmann, Guido Kanschat 2001, 2004
  */
 template <int dim, int spacedim=dim>
@@ -84,7 +108,7 @@ public:
   /**
    * Constructor for tensor product polynomials of degree <tt>p</tt>. The
    * shape functions created using this constructor correspond to Lagrange
-   * interpolation polynomials for equidistantly spaced support points in each
+   * interpolation polynomials for Gauss-Lobatto support (node) points in each
    * coordinate direction.
    */
   FE_DGQ (const unsigned int p);
@@ -250,8 +274,8 @@ public:
    * meet at a common face, whether it is the other way around, whether
    * neither dominates, or if either could dominate.
    *
-   * For a definition of domination, see FiniteElementBase::Domination and in
-   * particular the
+   * For a definition of domination, see FiniteElementDomination::Domination
+   * and in particular the
    * @ref hp_paper "hp paper".
    */
   virtual
@@ -270,7 +294,7 @@ public:
                                     const unsigned int face_index) const;
 
   /**
-   * Returns a list of constant modes of the element. For this element, it
+   * Return a list of constant modes of the element. For this element, it
    * simply returns one row with all entries set to true.
    */
   virtual std::pair<Table<2,bool>, std::vector<unsigned int> >
@@ -289,15 +313,14 @@ public:
 
 protected:
   /**
-   * Constructor for tensor product polynomials based on Polynomials::Lagrange
-   * interpolation of the support points in the quadrature rule
-   * <tt>points</tt>. The degree of these polynomials is
-   * <tt>points.size()-1</tt>.
+   * Constructor for tensor product polynomials based on an arbitrary vector
+   * of polynomials. This constructor is used in derived classes to construct
+   * e.g. elements with arbitrary nodes or elements based on Legendre
+   * polynomials.
    *
-   * Note: The FE_DGQ::clone function does not work properly for FE with
-   * arbitrary nodes!
+   * The degree of these polynomials is <tt>polynomials.size()-1</tt>.
    */
-  FE_DGQ (const Quadrature<1> &points);
+  FE_DGQ (const std::vector<Polynomials::Polynomial<double> > &polynomials);
 
   /**
    * @p clone function instead of a copy constructor.
@@ -360,7 +383,8 @@ private:
  * quadrature points. If this set of quadrature points is then also used in
  * integrating the mass matrix, then it will be diagonal. The number of
  * quadrature points automatically determines the polynomial degree chosen for
- * this element.
+ * this element. The typical applications are the Gauss quadrature or the
+ * Gauss-Lobatto quadrature (provided through the base class).
  *
  * See the base class documentation in FE_DGQ for details.
  *
@@ -395,6 +419,104 @@ protected:
 };
 
 
+
+/**
+ * Implementation of scalar, discontinuous tensor product elements based on
+ * Legendre polynomials, described by the tensor product of the polynomial
+ * space Polynomials::Legendre. As opposed to the basic FE_DGQ element, these
+ * elements are not interpolatory and no support points are defined.
+ *
+ * See the base class documentation in FE_DGQ for details.
+ *
+ * @author Martin Kronbichler, 2017
+ */
+template <int dim,int spacedim=dim>
+class FE_DGQLegendre : public FE_DGQ<dim,spacedim>
+{
+public:
+  /**
+   * Constructor for tensor product polynomials based on Polynomials::Legendre
+   * interpolation.
+   */
+  FE_DGQLegendre (const unsigned int degree);
+
+  /**
+   * Return a list of constant modes of the element. For the Legendre basis,
+   * it returns one row where the first element (corresponding to the constant
+   * mode) is set to true and all other elements are set to false.
+   */
+  virtual std::pair<Table<2,bool>, std::vector<unsigned int> >
+  get_constant_modes () const;
+
+  /**
+   * Return a string that uniquely identifies a finite element. This class
+   * returns <tt>FE_DGQLegendre<dim>(degree)</tt> with <tt>dim</tt> and
+   * <tt>degree</tt> replaced by the values given by the template parameter
+   * and the argument passed to the constructor, respectively.
+   */
+  virtual std::string get_name () const;
+
+protected:
+  /**
+   * @p clone function instead of a copy constructor.
+   *
+   * This function is needed by the constructors of @p FESystem.
+   */
+  virtual FiniteElement<dim,spacedim> *clone() const;
+};
+
+
+
+/**
+ * Implementation of scalar, discontinuous tensor product elements based on
+ * Hermite polynomials, described by the polynomial space
+ * Polynomials::HermiteInterpolation. As opposed to the basic FE_DGQ element,
+ * these elements are not interpolatory and no support points are defined.
+ *
+ * This element is only a Hermite polynomials for degrees larger or equal to
+ * three. For degrees zero to two, a usual Lagrange basis is selected.
+ *
+ * See the base class documentation in FE_DGQ for details.
+ *
+ * @author Martin Kronbichler, 2017
+ */
+template <int dim,int spacedim=dim>
+class FE_DGQHermite : public FE_DGQ<dim,spacedim>
+{
+public:
+  /**
+   * Constructor for tensor product polynomials based on
+   * Polynomials::HermiteInterpolation.
+   */
+  FE_DGQHermite (const unsigned int degree);
+
+  /**
+   * Return a list of constant modes of the element. For the Hermite basis of
+   * degree three and larger, it returns one row where the first two elements
+   * (corresponding to the value left and the value at the right) are set to
+   * true and all other elements are set to false.
+   */
+  virtual std::pair<Table<2,bool>, std::vector<unsigned int> >
+  get_constant_modes () const;
+
+  /**
+   * Return a string that uniquely identifies a finite element. This class
+   * returns <tt>FE_DGQHermite<dim>(degree)</tt>, with <tt>dim</tt> and
+   * <tt>degree</tt> replaced by the values given by the template parameter
+   * and the argument passed to the constructor, respectively.
+   */
+  virtual std::string get_name () const;
+
+protected:
+  /**
+   * @p clone function instead of a copy constructor.
+   *
+   * This function is needed by the constructors of @p FESystem.
+   */
+  virtual FiniteElement<dim,spacedim> *clone() const;
+};
+
+
 /*@}*/
 
 DEAL_II_NAMESPACE_CLOSE
