@@ -13,6 +13,9 @@
 //
 // ---------------------------------------------------------------------
 
+#ifndef dealii__fe_poly_templates_h
+#define dealii__fe_poly_templates_h
+
 
 #include <deal.II/base/qprojector.h>
 #include <deal.II/base/polynomial_space.h>
@@ -199,8 +202,8 @@ FE_Poly<PolynomialType,dim,spacedim>::requires_update_flags (const UpdateFlags f
            | update_hessians | update_gradients
            | update_jacobian_pushed_forward_grads
            | update_jacobian_pushed_forward_2nd_derivatives;
-  if (flags & update_cell_normal_vectors)
-    out |= update_cell_normal_vectors | update_JxW_values;
+  if (flags & update_normal_vectors)
+    out |= update_normal_vectors | update_JxW_values;
 
   return out;
 }
@@ -534,3 +537,5 @@ FE_Poly<PolynomialType,dim,spacedim>::get_poly_space_numbering_inverse () const
 
 
 DEAL_II_NAMESPACE_CLOSE
+
+#endif
