@@ -1,7 +1,6 @@
 // ---------------------------------------------------------------------
-// $Id$
 //
-// Copyright (C) 2012 - 2016 by the deal.II authors
+// Copyright (C) 2012 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -105,33 +104,21 @@ public:
    */
   virtual std::string get_name () const;
 
-  /**
-   * Interpolate a set of scalar values, computed in the generalized support
-   * points.
-   */
+  // documentation inherited from the base class
+  virtual
+  void
+  convert_generalized_support_point_values_to_nodal_values (const std::vector<Vector<double> > &support_point_values,
+                                                            std::vector<double>                &nodal_values) const;
+
   virtual void interpolate(std::vector<double>       &local_dofs,
-                           const std::vector<double> &values) const;
+                           const std::vector<double> &values) const DEAL_II_DEPRECATED;
 
-  /**
-   * Interpolate a set of vector values, computed in the generalized support
-   * points.
-   *
-   * Since a finite element often only interpolates part of a vector,
-   * <tt>offset</tt> is used to determine the first component of the vector to
-   * be interpolated. Maybe consider changing your data structures to use the
-   * next function.
-   */
   virtual void interpolate(std::vector<double>                &local_dofs,
                            const std::vector<Vector<double> > &values,
-                           unsigned int offset = 0) const;
+                           const unsigned int                  offset = 0) const DEAL_II_DEPRECATED;
 
-  /**
-   * Interpolate a set of vector values, computed in the generalized support
-   * points.
-   */
-  virtual void interpolate(
-    std::vector<double>          &local_dofs,
-    const VectorSlice<const std::vector<std::vector<double> > > &values) const;
+  virtual void interpolate(std::vector<double>          &local_dofs,
+                           const VectorSlice<const std::vector<std::vector<double> > > &values) const DEAL_II_DEPRECATED;
 
   /**
    * Return the matrix interpolating from the given finite element to the
@@ -176,7 +163,7 @@ protected:
 private:
 
   /**
-   * Returns the restriction_is_additive flags. Only the last components for
+   * Return the restriction_is_additive flags. Only the last components for
    * the bubble enrichments are true.
    */
   static std::vector<bool> get_riaf_vector(const unsigned int degree);
