@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2002 - 2015 by the deal.II authors
+// Copyright (C) 2002 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -22,9 +22,6 @@
 
 DEAL_II_NAMESPACE_OPEN
 
-template <int dim, int spacedim> class MappingQ;
-
-
 /*!@addtogroup fe */
 /*@{*/
 
@@ -658,8 +655,8 @@ public:
    * meet at a common face, whether it is the other way around, whether
    * neither dominates, or if either could dominate.
    *
-   * For a definition of domination, see FiniteElementBase::Domination and in
-   * particular the
+   * For a definition of domination, see FiniteElementDomination::Domination
+   * and in particular the
    * @ref hp_paper "hp paper".
    */
   virtual
@@ -684,7 +681,7 @@ public:
   std::vector<unsigned int> get_embedding_dofs (const unsigned int sub_degree) const;
 
   /**
-   * Returns a list of constant modes of the element. For this element, the
+   * Return a list of constant modes of the element. For this element, the
    * list consists of true arguments for the first vertex shape functions and
    * false for the remaining ones.
    */
@@ -696,7 +693,7 @@ public:
    */
   virtual
   void interpolate(std::vector<double>       &local_dofs,
-                   const std::vector<double> &values) const;
+                   const std::vector<double> &values) const DEAL_II_DEPRECATED;
 
   /**
    * This function is not implemented and throws an exception if called.
@@ -705,7 +702,7 @@ public:
   void
   interpolate(std::vector<double>                &local_dofs,
               const std::vector<Vector<double> > &values,
-              unsigned int offset = 0) const;
+              const unsigned int offset = 0) const DEAL_II_DEPRECATED;
 
   /**
    * This function is not implemented and throws an exception if called.
@@ -713,7 +710,7 @@ public:
   virtual
   void
   interpolate(std::vector<double> &local_dofs,
-              const VectorSlice<const std::vector<std::vector<double> > > &values) const;
+              const VectorSlice<const std::vector<std::vector<double> > > &values) const DEAL_II_DEPRECATED;
 
 
 protected:
@@ -792,16 +789,16 @@ private:
                                              const std::vector<FullMatrix<double> > &dofs_subcell);
 
   /**
-   * Initialize the @p unit_support_points field of the FiniteElement class.
+   * Initialize the @p generalized_support_points field of the FiniteElement class.
    * Called from the constructor.
    */
-  void initialize_unit_support_points ();
+  void initialize_generalized_support_points ();
 
   /**
-   * Initialize the @p unit_face_support_points field of the FiniteElement
+   * Initialize the @p generalized_face_support_points field of the FiniteElement
    * class. Called from the constructor.
    */
-  void initialize_unit_face_support_points ();
+  void initialize_generalized_face_support_points ();
 
   /**
    * Mapping from lexicographic to shape function numbering on first face.
@@ -822,7 +819,7 @@ private:
 /* -------------- declaration of explicit specializations ------------- */
 
 template <>
-void FE_Q_Hierarchical<1>::initialize_unit_face_support_points ();
+void FE_Q_Hierarchical<1>::initialize_generalized_face_support_points ();
 
 template <>
 bool
