@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2003 - 2015 by the deal.II authors
+// Copyright (C) 2003 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -29,9 +29,6 @@
 
 DEAL_II_NAMESPACE_OPEN
 
-template <int dim, int spacedim> class MappingQ;
-
-
 /*!@addtogroup fe */
 /*@{*/
 
@@ -131,17 +128,24 @@ public:
   virtual bool has_support_on_face (const unsigned int shape_index,
                                     const unsigned int face_index) const;
 
+  // documentation inherited from the base class
+  virtual
+  void
+  convert_generalized_support_point_values_to_nodal_values (const std::vector<Vector<double> > &support_point_values,
+                                                            std::vector<double>                &nodal_values) const;
+
   virtual void interpolate(std::vector<double>                &local_dofs,
-                           const std::vector<double> &values) const;
+                           const std::vector<double>          &values) const DEAL_II_DEPRECATED;
+
   virtual void interpolate(std::vector<double>                &local_dofs,
                            const std::vector<Vector<double> > &values,
-                           unsigned int offset = 0) const;
-  virtual void interpolate(
-    std::vector<double> &local_dofs,
-    const VectorSlice<const std::vector<std::vector<double> > > &values) const;
+                           const unsigned int                  offset = 0) const DEAL_II_DEPRECATED;
+
+  virtual void interpolate(std::vector<double> &local_dofs,
+                           const VectorSlice<const std::vector<std::vector<double> > > &values) const DEAL_II_DEPRECATED;
 
   /**
-   * Returns a list of constant modes of the element. This method is currently
+   * Return a list of constant modes of the element. This method is currently
    * not correctly implemented because it returns ones for all components.
    */
   virtual std::pair<Table<2,bool>, std::vector<unsigned int> >
@@ -262,14 +266,20 @@ public:
 
   virtual FiniteElement<dim> *clone () const;
 
+  // documentation inherited from the base class
+  virtual
+  void
+  convert_generalized_support_point_values_to_nodal_values (const std::vector<Vector<double> > &support_point_values,
+                                                            std::vector<double>                &nodal_values) const;
   virtual void interpolate(std::vector<double>                &local_dofs,
-                           const std::vector<double> &values) const;
+                           const std::vector<double> &values) const DEAL_II_DEPRECATED;
+
   virtual void interpolate(std::vector<double>                &local_dofs,
                            const std::vector<Vector<double> > &values,
-                           unsigned int offset = 0) const;
-  virtual void interpolate(
-    std::vector<double> &local_dofs,
-    const VectorSlice<const std::vector<std::vector<double> > > &values) const;
+                           const unsigned int                  offset = 0) const DEAL_II_DEPRECATED;
+
+  virtual void interpolate(std::vector<double> &local_dofs,
+                           const VectorSlice<const std::vector<std::vector<double> > > &values) const DEAL_II_DEPRECATED;
 
 
   virtual void get_face_interpolation_matrix (const FiniteElement<dim> &source,
