@@ -28,13 +28,18 @@
 
 DEAL_II_NAMESPACE_OPEN
 
+template <int dim, int spacedim> class FE_Enriched;
+
 
 /**
  * This class provides an interface to group several elements together into
  * one. To the outside world, the resulting object looks just like a usual
  * finite element object, which is composed of several other finite elements
  * that are possibly of different type. The result is then a vector-valued
- * finite element. %Vector valued elements are discussed in a number of
+ * finite element. An example is given in the documentation of namespace
+ * FETools::Compositing, when using the "tensor product" strategy.
+ *
+ * %Vector valued elements are discussed in a number of
  * tutorial programs, for example step-8, step-20, step-21, and in particular
  * in the
  * @ref vector_valued
@@ -683,7 +688,7 @@ public:
    * Code implementing this would then look like this:
    * @code
    * for (i=0; i<dofs_per_face; ++i)
-   *  if (fe.is_primitive(fe.face_to_equivalent_cell_index(i, some_face_no)))
+   *  if (fe.is_primitive(fe.face_to_cell_index(i, some_face_no)))
    *   ... do whatever
    * @endcode
    * The function takes additional arguments that account for the fact that
@@ -729,7 +734,7 @@ public:
   unit_face_support_point (const unsigned int index) const;
 
   /**
-   * Returns a list of constant modes of the element. The returns table has as
+   * Return a list of constant modes of the element. The returns table has as
    * many rows as there are components in the element and dofs_per_cell
    * columns. To each component of the finite element, the row in the returned
    * table contains a basis representation of the constant function 1 on the
@@ -826,8 +831,8 @@ public:
    * meet at a common face, whether it is the other way around, whether
    * neither dominates, or if either could dominate.
    *
-   * For a definition of domination, see FiniteElementBase::Domination and in
-   * particular the
+   * For a definition of domination, see FiniteElementDomination::Domination
+   * and in particular the
    * @ref hp_paper "hp paper".
    */
   virtual
@@ -956,7 +961,6 @@ private:
       unsigned int> >
       base_elements;
 
-
   /**
    * Initialize the @p unit_support_points field of the FiniteElement class.
    * Called from the constructor.
@@ -982,16 +986,6 @@ private:
   void initialize (const std::vector<const FiniteElement<dim,spacedim>*> &fes,
                    const std::vector<unsigned int> &multiplicities);
 
-  /**
-   * Used by @p initialize.
-   */
-  void build_cell_tables();
-
-  /**
-   * Used by @p initialize.
-   */
-  void build_face_tables();
-
   /**
    * Used by @p initialize.
    */
@@ -1078,6 +1072,8 @@ private:
    * Mutex for protecting initialization of restriction and embedding matrix.
    */
   mutable Threads::Mutex mutex;
+
+  friend class FE_Enriched<dim,spacedim>;
 };
 
 
