@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2016 by the deal.II authors
+// Copyright (C) 2000 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -24,6 +24,11 @@
 #include <deal.II/base/geometry_info.h>
 #include <deal.II/base/tensor.h>
 #include <deal.II/base/symmetric_tensor.h>
+#include <deal.II/distributed/tria.h>
+#include <deal.II/fe/component_mask.h>
+#include <deal.II/lac/parallel_vector.h>
+
+
 
 #include <vector>
 #include <string>
@@ -32,14 +37,9 @@
 DEAL_II_NAMESPACE_OPEN
 
 template <typename number> class FullMatrix;
-template <typename number> class Vector;
 template <int dim> class Quadrature;
 template <int dim, int spacedim> class FiniteElement;
 template <int dim, int spacedim> class DoFHandler;
-namespace hp
-{
-  template <int dim, int spacedim> class DoFHandler;
-}
 template <int dim> class FiniteElementData;
 class ConstraintMatrix;
 
@@ -71,7 +71,7 @@ namespace FETools
    * degree. Derived classes are called whenever one wants to have a
    * transparent way to create a finite element object.
    *
-   * This class is used in the FETools::get_fe_from_name() and
+   * This class is used in the FETools::get_fe_by_name() and
    * FETools::add_fe_name() functions.
    *
    * @author Guido Kanschat, 2006
@@ -232,35 +232,86 @@ namespace FETools
                              FullMatrix<number> &matrix);
 
   /**
-   * Compute the matrix of nodal values of a finite element applied to all its
-   * shape functions.
+   * This is a rather specialized function used during the construction of
+   * finite element objects. It is used to build the basis of shape functions
+   * for an element, given a set of polynomials and interpolation points. The
+   * function is only implemented for finite elements with exactly @p dim
+   * vector components. In particular, this applies to classes derived from
+   * the FE_PolyTensor class.
    *
-   * This function is supposed to help building finite elements from
-   * polynomial spaces and should be called inside the constructor of an
-   * element. Applied to a completely initialized finite element, the result
-   * should be the unit matrix by definition of the node values.
+   * Specifically, the purpose of this function is as follows: FE_PolyTensor
+   * receives, from its derived classes, an argument that describes a polynomial
+   * space. This space may be parameterized in terms of monomials, or in some
+   * other way, but is in general not in the form that we use for finite
+   * elements where we typically want to use a basis that is derived from
+   * some kind of node functional (e.g., the interpolation at specific points).
+   * Concretely, assume that the basis used by the polynomial space is
+   * $\{\tilde\varphi_j(\mathbf x)\}_{j=1}^N$, and that the node functionals
+   * of the finite element are $\{\Psi_i\}_{i=1}^N$. We then want to compute a
+   * basis $\{\varphi_j(\mathbf x)\}_{j=1}^N$ for the finite element space so
+   * that $\Psi_i[\varphi_j] = \delta_{ij}$. To do this, we can set
+   * $\varphi_j(\mathbf x) = \sum_{k=1}^N c_{jk} \tilde\varphi_k(\mathbf x)$
+   * where we need to determine the expansion coefficients $c_{jk}$. We do this
+   * by applying $\Psi_i$ to both sides of the equation, to obtain
+   * @f{align*}{
+   *   \Psi_i [\varphi_j] = \sum_{k=1}^N c_{jk} \Psi_i[\tilde\varphi_k],
+   * @f}
+   * and we know that the left hand side equals $\delta_{ij}$.
+   * If you think of this as a system of $N\times N$ equations for the
+   * elements of a matrix on the left and on the right, then this can be
+   * written as
+   * @f{align*}{
+   *   I = C X^T
+   * @f}
+   * where $C$ is the matrix of coefficients $c_{jk}$ and
+   * $X_{ik} = \Psi_i[\tilde\varphi_k]$. Consequently, in order to compute
+   * the expansion coefficients $C=X^{-T}$, we need to apply the node
+   * functionals to all functions of the "raw" basis of the polynomial space.
    *
-   * Using this matrix allows the construction of the basis of shape functions
-   * in two steps.
+   * Until the finite element receives this matrix $X$ back, it describes its
+   * shape functions (e.g., in FiniteElement::shape_value()) in the form
+   * $\tilde\varphi_j$. After it calls this function, it has the expansion
+   * coefficients and can describe its shape functions as $\varphi_j$.
    *
-   * <ol>
+   * This function therefore computes this matrix $X$, for the following
+   * specific circumstances:
+   * - That the node functionals $\Psi_i$ are point evaluations at points
+   *   $\mathbf x_i$ that the finite element in question describes via its
+   *   "generalized" support points (through
+   *   FiniteElement::get_generalized_support_points(), see also
+   *   @ref GlossGeneralizedSupport "this glossary entry"). These point
+   *   evaluations need to necessarily evaluate the <i>value</i> of a shape
+   *   function at that point (the shape function may be vector-valued, and
+   *   so the functional may be a linear combination of the individual
+   *   components of the values); but, in particular, the nodal functions may
+   *   not be <i>integrals</i> over entire edges or faces,
+   *   or other non-local functionals. In other words, we assume that
+   *   $\Psi_i[\tilde\varphi_j] = f_j(\tilde\varphi_j(\mathbf x_i))$
+   *   where $f_j$ is a function of the (possibly vector-valued) argument
+   *   that returns a scalar.
+   * - That the finite element has exactly @p dim vector components.
+   * - That the function $f_j$ is given by whatever the element implements
+   *   through the FiniteElement::convert_generalized_support_point_values_to_nodal_values()
+   *   function.
    *
-   * <li>Define the space of shape functions using an arbitrary basis
-   * <i>w<sub>j</sub></i> and compute the matrix <i>M</i> of node functionals
-   * <i>N<sub>i</sub></i> applied to these basis functions.
-   *
-   * <li>Compute the basis <i>v<sub>j</sub></i> of the finite element shape
-   * function space by applying <i>M<sup>-1</sup></i> to the basis
-   * <i>w<sub>j</sub></i>.
-   *
-   * </ol>
-   *
-   * @note The FiniteElement must provide generalized support points and and
-   * interpolation functions.
+   * @param fe The finite element for which the operations above are to be
+   *        performed.
+   * @return The matrix $X$ as discussed above.
    */
   template <int dim, int spacedim>
-  void compute_node_matrix(FullMatrix<double> &M,
-                           const FiniteElement<dim,spacedim> &fe);
+  FullMatrix<double>
+  compute_node_matrix(const FiniteElement<dim,spacedim> &fe);
+
+  /**
+   * Same as the function above, but return the matrix by reference through
+   * the first argument, rather than as the function's return value.
+   *
+   * @deprecated
+   */
+  template <int dim, int spacedim>
+  void
+  compute_node_matrix(FullMatrix<double> &M,
+                      const FiniteElement<dim,spacedim> &fe) DEAL_II_DEPRECATED;
 
   /**
    * For all possible (isotropic and anisotropic) refinement cases compute the
@@ -474,7 +525,7 @@ namespace FETools
                                                      FullMatrix<double>       &I_q);
 
   /**
-   * Computes the projection of tensorial (first-order tensor) data stored at
+   * Compute the projection of tensorial (first-order tensor) data stored at
    * the quadrature points @p vector_of_tensors_at_qp to data @p
    * vector_of_tensors_at_nodes at the support points of the cell.  The data
    * in @p vector_of_tensors_at_qp is ordered sequentially following the
@@ -606,27 +657,19 @@ namespace FETools
    * nodes. For that case use the @p back_interpolate function, below, that
    * takes an additional @p ConstraintMatrix object.
    *
+   * @p dof1 might be a DoFHandler or a hp::DoFHandler onject.
+   *
    * Furthermore note, that for the specific case when the finite element
    * space corresponding to @p fe1 is a subset of the finite element space
    * corresponding to @p fe2, this function is simply an identity mapping.
    */
-  template <int dim, class InVector, class OutVector, int spacedim>
-  void back_interpolate (const DoFHandler<dim,spacedim>    &dof1,
-                         const InVector                    &u1,
-                         const FiniteElement<dim,spacedim> &fe2,
-                         OutVector                         &u1_interpolated);
-
-  /**
-   * Same as last function, except that the dof handler objects might be of
-   * type @p hp::DoFHandler.
-   */
   template <int dim,
-            template <int> class DoFHandlerType,
+            template <int,int> class DoFHandlerType,
             class InVector, class OutVector, int spacedim>
-  void back_interpolate (const DoFHandlerType<dim>         &dof1,
-                         const InVector                    &u1,
-                         const FiniteElement<dim,spacedim> &fe2,
-                         OutVector                         &u1_interpolated);
+  void back_interpolate (const DoFHandlerType<dim,spacedim> &dof1,
+                         const InVector                     &u1,
+                         const FiniteElement<dim,spacedim>  &fe2,
+                         OutVector                          &u1_interpolated);
 
   /**
    * Gives the interpolation of the @p dof1-function @p u1 to a @p
@@ -714,28 +757,29 @@ namespace FETools
    * corresponding cell of `dof2` using the interpolation matrix of the finite
    * element spaces used on these cells and provided by the finite element
    * objects involved. This step is done using the FETools::interpolate()
-   * function. - It then performs a loop over all non-active cells of `dof2`.
+   * function.
+   * - It then performs a loop over all non-active cells of `dof2`.
    * If such a non-active cell has at least one active child, then we call the
    * children of this cell a "patch". We then interpolate from the children of
    * this patch to the patch, using the finite element space associated with
    * `dof2` and immediately interpolate back to the children. In essence, this
    * information throws away all information in the solution vector that lives
-   * on a scale smaller than the patch cell. - Since we traverse non-active
-   * cells from the coarsest to the finest levels, we may find patches that
-   * correspond to child cells of previously treated patches if the mesh had
-   * been refined adaptively (this cannot happen if the mesh has been refined
-   * globally because there the children of a patch are all active). We also
-   * perform the operation described above on these patches, but it is easy to
-   * see that on patches that are children of previously treated patches, the
-   * operation is now the identity operation (since it interpolates from the
-   * children of the current patch a function that had previously been
-   * interpolated to these children from an even coarser patch). Consequently,
-   * this does not alter the solution vector any more.
+   * on a scale smaller than the patch cell.
+   * - Since we traverse non-active cells from the coarsest to the finest
+   * levels, we may find patches that correspond to child cells of previously
+   * treated patches if the mesh had been refined adaptively (this cannot
+   * happen if the  mesh has been refined globally because there the children
+   * of a patch are all active). We also perform the operation described above
+   * on these patches, but it is easy to see that on patches that are children
+   * of previously treated patches, the operation is now the identity operation
+   * (since it interpolates from the children of the current patch a function
+   * that had previously been interpolated to these children from an even coarser
+   * patch). Consequently, this does not alter the solution vector any more.
    *
    * The name of the function originates from the fact that it can be used to
    * construct a representation of a function of higher polynomial degree on a
    * once coarser mesh. For example, if you imagine that you start with a
-   * $Q_1$ function on globally refined mesh, and that @p dof2 is associated
+   * $Q_1$ function on a globally refined mesh, and that @p dof2 is associated
    * with a $Q_2$ element, then this function computes the equivalent of the
    * operator $I_{2h}^{(2)}$ interpolating the original piecewise linear
    * function onto a quadratic function on a once coarser mesh with mesh size
@@ -781,6 +825,7 @@ namespace FETools
                     const DoFHandler<dim,spacedim> &dof2,
                     const ConstraintMatrix         &constraints,
                     OutVector                      &z2);
+
   //@}
   /**
    * The numbering of the degrees of freedom in continuous finite elements is
@@ -843,6 +888,268 @@ namespace FETools
   std::vector<unsigned int>
   lexicographic_to_hierarchic_numbering (const FiniteElementData<dim> &fe_data);
 
+
+  /**
+   * A namespace that contains functions that help setting up internal
+   * data structures when implementing FiniteElement which are build
+   * from simpler ("base") elements, for example FESystem. The things
+   * computed by these functions typically serve as constructor
+   * arguments to the FiniteElement base class of the derived finite
+   * element object being constructed.
+   *
+   * There are generally two ways in which one can build more complex
+   * elements, and this is reflected by several of the functions in
+   * this namespace having arguments called
+   * <code>do_tensor_product</code>:
+   *
+   * <ol>
+   * <li> Tensor product construction (<code>do_tensor_product=true</code>):
+   * The tensor product construction, in the simplest case, builds a
+   * vector-valued element from scalar elements (see
+   * @ref vector_valued "this documentation module" and
+   * @ref GlossComponent "this glossary entry" for more information).
+   * To give an example, consider creating a vector-valued element with
+   * two vector components, where the first should have linear shape
+   * functions and the second quadratic shape functions. In 1d, the
+   * shape functions (on the reference cell) of the base elements are then
+   * @f{align*}{
+   *   Q_1 &= \{ 1-x, x \},
+   *   \\  Q_2 &= \{ 2(\frac 12 - x)(1-x), 2(x - \frac 12)x, 4x(1-x) \},
+   * @f}
+   * where shape functions are ordered in the usual way (first on the
+   * first vertex, then on the second vertex, then in the interior of
+   * the cell). The tensor product construction will create an element with
+   * the following shape functions:
+   * @f{align*}{
+   *   Q_1 \times Q_2 &=
+   *   \left\{
+   *     \begin{pmatrix} 1-x \\ 0 \end{pmatrix},
+   *     \begin{pmatrix} 0 \\ 2(\frac 12 - x)(1-x)  \end{pmatrix},
+   *     \begin{pmatrix} x \\ 0 \end{pmatrix},
+   *     \begin{pmatrix} 0 \\ 2(x - \frac 12)x \end{pmatrix},
+   *     \begin{pmatrix} 0 \\ 4x(1-x) \end{pmatrix}
+   *   \right\}.
+   * @f}
+   * The list here is again in standard order.
+   *
+   * Of course, the procedure also works if the base elements are
+   * already vector valued themselves: in that case, the composed
+   * element simply has as many vector components as the base elements
+   * taken together.
+   *
+   * <li> Combining shape functions
+   * (<code>do_tensor_product=false</code>): In contrast to the
+   * previous strategy, combining shape functions simply takes
+   * <i>all</i> of the shape functions together. In the case above,
+   * this would yield the following element:
+   * @f{align*}{
+   *   Q_1 + Q_2 &= \{ 1-x, 2(\frac 12 - x)(1-x),
+   *                   x, 2(x - \frac 12)x, 4x(1-x) \}.
+   * @f}
+   * In other words, if the base elements are scalar, the resulting
+   * element will also be. In general, the base elements all will
+   * have to have the same number of vector components.
+   *
+   * The element constructed above of course no longer has a linearly
+   * independent set of shape functions. As a consequence, any matrix
+   * one creates by treating all shape functions of the composed
+   * element in the same way will be singular. In practice, this
+   * strategy is therefore typically used in situations where one
+   * explicitly makes sure that certain shape functions are treated
+   * differently (e.g., by multiplying them with weight functions), or
+   * in cases where the shape functions one combines are not linearly
+   * dependent.
+   *
+   * </ol>
+   */
+  namespace Compositing
+  {
+
+    /**
+     * Take vectors of finite elements and multiplicities and multiply out
+     * how many degrees of freedom the composed element has per vertex,
+     * line, etc.
+     *
+     * If @p do_tensor_product is true, the number of components
+     * returned in the FiniteElementData object is the sum over the
+     * product of the number of components in each of the finite
+     * elements times the corresponding multiplicity.  Otherwise the
+     * number of components is taken from the first finite element with
+     * non-zero multiplicity, and all other elements with non-zero
+     * multiplicities need to have the same number of vector components.
+     *
+     * See the documentation of namespace FETools::Compositing for more
+     * information about the @p do_tensor_product argument.
+     */
+    template <int dim, int spacedim>
+    FiniteElementData<dim>
+    multiply_dof_numbers (const std::vector<const FiniteElement<dim,spacedim>*> &fes,
+                          const std::vector<unsigned int>                       &multiplicities,
+                          const bool do_tensor_product = true);
+
+    /**
+     * Same as above but for a specific number of sub-elements.
+     */
+    template <int dim, int spacedim>
+    FiniteElementData<dim>
+    multiply_dof_numbers (const FiniteElement<dim,spacedim> *fe1,
+                          const unsigned int            N1,
+                          const FiniteElement<dim,spacedim> *fe2=NULL,
+                          const unsigned int            N2=0,
+                          const FiniteElement<dim,spacedim> *fe3=NULL,
+                          const unsigned int            N3=0,
+                          const FiniteElement<dim,spacedim> *fe4=NULL,
+                          const unsigned int            N4=0,
+                          const FiniteElement<dim,spacedim> *fe5=NULL,
+                          const unsigned int            N5=0);
+
+    /**
+     * Compute the "restriction is additive" flags (see the
+     * documentation of the FiniteElement class) for a list of finite
+     * elements with multiplicities given in the second argument.
+     *
+     * The "restriction is additive" flags are properties of
+     * individual shape functions that do not depend on whether the
+     * composed element uses the tensor product or combination
+     * strategy outlined in the documentation of the
+     * FETools::Composition namespace. Consequently, this function
+     * does not have a @p do_tensor_product argument.
+     */
+    template <int dim, int spacedim>
+    std::vector<bool>
+    compute_restriction_is_additive_flags (const std::vector<const FiniteElement<dim,spacedim>*> &fes,
+                                           const std::vector<unsigned int>              &multiplicities);
+
+    /**
+     * Take a @p FiniteElement object and return a boolean vector
+     * describing the @p restriction_is_additive_flags (see the
+     * documentation of the FiniteElement class) for each shape function
+     * of the mixed element consisting of @p N1, @p N2, ... copies of
+     * the sub-elements @p fe1, @p fe2, ...
+     *
+     * The "restriction is additive" flags are properties of
+     * individual shape functions that do not depend on whether the
+     * composed element uses the tensor product or combination
+     * strategy outlined in the documentation of the
+     * FETools::Composition namespace. Consequently, this function
+     * does not have a @p do_tensor_product argument.
+     */
+    template <int dim, int spacedim>
+    std::vector<bool>
+    compute_restriction_is_additive_flags (const FiniteElement<dim,spacedim> *fe1,
+                                           const unsigned int        N1,
+                                           const FiniteElement<dim,spacedim> *fe2=NULL,
+                                           const unsigned int        N2=0,
+                                           const FiniteElement<dim,spacedim> *fe3=NULL,
+                                           const unsigned int        N3=0,
+                                           const FiniteElement<dim,spacedim> *fe4=NULL,
+                                           const unsigned int        N4=0,
+                                           const FiniteElement<dim,spacedim> *fe5=NULL,
+                                           const unsigned int        N5=0);
+
+    /**
+     * Compute the nonzero components for each shape function of a
+     * composed finite element described by a list of finite elements
+     * with multiplicities given in the second argument.
+     *
+     * If @p do_tensor_product is true, the number of components (and
+     * thus the size of the ComponentMask objects) is the sum over the
+     * product of the number of components in each of the finite
+     * elements times the corresponding multiplicity.  Otherwise the
+     * number of components is taken from the first finite element with
+     * non-zero multiplicity, and all other elements with non-zero
+     * multiplicities need to have the same number of vector components.
+     *
+     * See the documentation of namespace FETools::Compositing for more
+     * information about the @p do_tensor_product argument.
+     */
+    template <int dim, int spacedim>
+    std::vector<ComponentMask>
+    compute_nonzero_components (const std::vector<const FiniteElement<dim,spacedim>*> &fes,
+                                const std::vector<unsigned int>              &multiplicities,
+                                const bool do_tensor_product = true);
+
+    /**
+     * Compute the non-zero vector components of a composed finite
+     * element. This function is similar to the previous one, except
+     * that the pointers indicate the elements to be composed, and the
+     * arguments @p N1, @p N2, ... the multiplicities. Null pointers
+     * indicate that an argument is to be skipped.
+     *
+     * If @p do_tensor_product is true, the number of components (and
+     * thus the size of the ComponentMask objects) is the sum over the
+     * product of the number of components in each of the finite
+     * elements times the corresponding multiplicity.  Otherwise the
+     * number of components is taken from the first finite element with
+     * non-zero multiplicity, and all other elements with non-zero
+     * multiplicities need to have the same number of vector components.
+     *
+     * See the documentation of namespace FETools::Compositing for more
+     * information about the @p do_tensor_product argument.
+     */
+    template <int dim, int spacedim>
+    std::vector<ComponentMask>
+    compute_nonzero_components (const FiniteElement<dim,spacedim> *fe1,
+                                const unsigned int        N1,
+                                const FiniteElement<dim,spacedim> *fe2=NULL,
+                                const unsigned int        N2=0,
+                                const FiniteElement<dim,spacedim> *fe3=NULL,
+                                const unsigned int        N3=0,
+                                const FiniteElement<dim,spacedim> *fe4=NULL,
+                                const unsigned int        N4=0,
+                                const FiniteElement<dim,spacedim> *fe5=NULL,
+                                const unsigned int        N5=0,
+                                const bool do_tensor_product = true);
+
+    /**
+     * For a given (composite) @p finite_element build @p
+     * system_to_component_table, @p system_to_base_table and @p
+     * component_to_base_table.
+     *
+     * If @p do_tensor_product is true, the number of components
+     * used for the composite element is the sum over the
+     * product of the number of components in each of the finite
+     * elements times the corresponding multiplicity.  Otherwise the
+     * number of components is taken from the first finite element with
+     * non-zero multiplicity, and all other elements with non-zero
+     * multiplicities need to have the same number of vector components.
+     *
+     * See the documentation of namespace FETools::Compositing for more
+     * information about the @p do_tensor_product argument.
+     */
+    template <int dim, int spacedim>
+    void
+    build_cell_tables(std::vector< std::pair< std::pair< unsigned int, unsigned int >, unsigned int > > &system_to_base_table,
+                      std::vector< std::pair< unsigned int, unsigned int > >  &system_to_component_table,
+                      std::vector< std::pair< std::pair< unsigned int, unsigned int >, unsigned int > > &component_to_base_table,
+                      const FiniteElement<dim,spacedim> &finite_element,
+                      const bool do_tensor_product = true);
+
+    /**
+     * For a given (composite) @p finite_element build @p face_system_to_base_table,
+     * and @p face_system_to_component_table.
+     *
+     * If @p do_tensor_product is true, the number of components
+     * used for the composite element is the sum over the
+     * product of the number of components in each of the finite
+     * elements times the corresponding multiplicity.  Otherwise the
+     * number of components is taken from the first finite element with
+     * non-zero multiplicity, and all other elements with non-zero
+     * multiplicities need to have the same number of vector components.
+     *
+     * See the documentation of namespace FETools::Compositing for more
+     * information about the @p do_tensor_product argument.
+     */
+    template <int dim, int spacedim>
+    void
+    build_face_tables(std::vector< std::pair< std::pair< unsigned int, unsigned int >, unsigned int > > &face_system_to_base_table,
+                      std::vector< std::pair< unsigned int, unsigned int > >                            &face_system_to_component_table,
+                      const FiniteElement<dim,spacedim> &finite_element,
+                      const bool do_tensor_product = true);
+
+  }
+
+
   /**
    * Parse the name of a finite element and generate a finite element object
    * accordingly. The parser ignores space characters between words (things
@@ -877,7 +1184,11 @@ namespace FETools
    * function, use the add_fe_name() function.  This function does not work if
    * one wants to get a codimension 1 finite element.
    */
-  template <int dim, int spacedim>
+  template <int dim, int spacedim
+#ifdef DEAL_II_WITH_CXX11
+            =dim
+#endif
+            >
   FiniteElement<dim, spacedim> *
   get_fe_by_name (const std::string &name);
 
@@ -887,12 +1198,12 @@ namespace FETools
    */
   template <int dim>
   FiniteElement<dim,dim> *
-  get_fe_from_name (const std::string &name);
+  get_fe_from_name (const std::string &name) DEAL_II_DEPRECATED;
 
 
   /**
    * Extend the list of finite elements that can be generated by
-   * get_fe_from_name() by the one given as @p name. If get_fe_from_name() is
+   * get_fe_by_name() by the one given as @p name. If get_fe_by_name() is
    * later called with this name, it will use the object given as second
    * argument to create a finite element object.
    *
@@ -907,7 +1218,7 @@ namespace FETools
    * used anymore.
    *
    * In most cases, if you want objects of type <code>MyFE</code> be created
-   * whenever the name <code>my_fe</code> is given to get_fe_from_name, you
+   * whenever the name <code>my_fe</code> is given to get_fe_by_name, you
    * will want the second argument to this function be of type
    * FEFactory@<MyFE@>, but you can of course create your custom finite
    * element factory class.
@@ -917,7 +1228,7 @@ namespace FETools
    * object will be deleted at the end of the program's lifetime.
    *
    * If the name of the element is already in use, an exception is thrown.
-   * Thus, functionality of get_fe_from_name() can only be added, not changed.
+   * Thus, functionality of get_fe_by_name() can only be added, not changed.
    *
    * @note This function manipulates a global table (one table for each space
    * dimension). It is thread safe in the sense that every access to this
@@ -937,7 +1248,7 @@ namespace FETools
                     const FEFactoryBase<dim,spacedim> *factory);
 
   /**
-   * The string used for get_fe_from_name() cannot be translated to a finite
+   * The string used for get_fe_by_name() cannot be translated to a finite
    * element.
    *
    * Either the string is badly formatted or you are using a custom element
@@ -951,7 +1262,7 @@ namespace FETools
                   << arg1 << "'.");
 
   /**
-   * The string used for get_fe_from_name() cannot be translated to a finite
+   * The string used for get_fe_by_name() cannot be translated to a finite
    * element.
    *
    * Dimension arguments in finite element names should be avoided. If they
