@@ -196,7 +196,7 @@ enum UpdateFlags
    */
   update_jacobian_pushed_forward_3rd_derivatives = 0x1000000,
   /**
-   * @deprecated Update quadrature points
+   * @deprecated Use #update_quadrature_points instead.
    */
   update_q_points = update_quadrature_points,
   /**
@@ -207,7 +207,31 @@ enum UpdateFlags
   /**
    * Combination of the flags needed for Piola transform of Hdiv elements.
    */
-  update_piola = update_volume_elements | update_contravariant_transformation
+  update_piola = update_volume_elements | update_contravariant_transformation,
+  /**
+   * Combination of the flags that require a mapping calculation
+   */
+  update_mapping =
+    // Direct data
+    update_quadrature_points |
+    update_JxW_values |
+    update_jacobians |
+    update_jacobian_grads |
+    update_jacobian_pushed_forward_grads |
+    update_jacobian_2nd_derivatives |
+    update_jacobian_pushed_forward_2nd_derivatives |
+    update_jacobian_3rd_derivatives |
+    update_jacobian_pushed_forward_3rd_derivatives |
+    update_inverse_jacobians |
+    update_boundary_forms |
+    update_normal_vectors |
+    // Transformation dependence
+    update_covariant_transformation |
+    update_contravariant_transformation |
+    update_transformation_values |
+    update_transformation_gradients |
+    // Volume data
+    update_volume_elements
 };
 
 
@@ -331,9 +355,22 @@ namespace CellSimilarity
 {
   enum Similarity
   {
+    /**
+     * The cells differ by something besides a translation or inverted
+     * translations.
+     */
     none,
+    /**
+     * The cells differ by a translation.
+     */
     translation,
+    /**
+     * The cells differ by an inverted translation.
+     */
     inverted_translation,
+    /**
+     * The next cell is not valid.
+     */
     invalid_next_cell
   };
 }
