@@ -20,14 +20,20 @@
 #include <deal.II/base/std_cxx11/array.h>
 #include <deal.II/base/smartpointer.h>
 #include <deal.II/fe/mapping_q1.h>
+#include <deal.II/dofs/dof_handler.h>
 
 DEAL_II_NAMESPACE_OPEN
 
+template <typename> class Vector;
+
+
 /*!@addtogroup mapping */
 /*@{*/
 
 /**
- * Eulerian mapping of general unit cells by $d$-linear shape functions. Each
+ * This class provides a mapping that adds to the location of each cell
+ * a $d$-linear displacement field. (The generalization to higher order
+ * polynomials is provided in the MappingQEulerian class.) Each
  * cell is thus shifted in space by values given to the mapping through a
  * finite element field.
  *
@@ -62,8 +68,8 @@ DEAL_II_NAMESPACE_OPEN
  *    FESystem<dim> fe(FE_Q<dim>(1), dim);
  *    DoFHandler<dim> flowfield_dof_handler(triangulation);
  *    flowfield_dof_handler.distribute_dofs(fe);
- *    Vector<double> map_points(flowfield_dof_handler.n_dofs());
- *    MappingQ1Eulerian<dim> mymapping(map_points, flowfield_dof_handler);
+ *    Vector<double> displacement_field(flowfield_dof_handler.n_dofs());
+ *    MappingQ1Eulerian<dim> mymapping(flowfield_dof_handler, displacement_field);
  * @endcode
  *
  * Note that since the vector of shift values and the dof handler are only
@@ -71,11 +77,8 @@ DEAL_II_NAMESPACE_OPEN
  * whenever you use this object, the given objects still represent valid data.
  *
  * To enable the use of the MappingQ1Eulerian class also in the context of
- * parallel codes using the PETSc wrapper classes, the type of the vector can
- * be specified as template parameter <tt>EulerVectorType</tt> Not specifying
- * this template argument in applications using the PETSc vector classes leads
- * to the construction of a copy of the vector which is not acccessible
- * afterwards!
+ * parallel codes using the PETSc or Trilinos wrapper classes, the type of
+ * the vector can be specified as template parameter <tt>VectorType</tt>.
  *
  * For more information about the <tt>spacedim</tt> template parameter check
  * the documentation of FiniteElement or the one of Triangulation.
@@ -86,19 +89,29 @@ template <int dim, typename VectorType = Vector<double>, int spacedim=dim >
 class MappingQ1Eulerian : public MappingQGeneric<dim,spacedim>
 {
 public:
+  /**
+   * Constructor.
+   *
+   * @param[in] euler_dof_handler A DoFHandler object that defines a finite
+   * element space. This space needs to have exactly dim components
+   * and these will be considered displacements
+   * relative to the original positions of the cells of the triangulation.
+   * This DoFHandler must be based on a <code>FESystem(FE_Q(1),dim)</code>
+   * finite element.
+   * @param[in] euler_vector A finite element function in the space defined by
+   * the first argument. The dim components of this function will be
+   * interpreted as the displacement we use in defining the mapping, relative
+   * to the location of cells of the underlying triangulation.
+   */
+  MappingQ1Eulerian (const DoFHandler<dim,spacedim> &euler_dof_handler,
+                     const VectorType               &euler_vector);
 
   /**
-   * Constructor. It takes a <tt>Vector<double> &</tt> as its first argument
-   * to specify the transformation of the whole problem from the reference to
-   * the current configuration. The organization of the elements in the @p
-   * Vector must follow the concept how deal.II stores solutions that are
-   * associated to a triangulation.  This is automatically the case if the @p
-   * Vector represents the solution of the previous step of a nonlinear
-   * problem. Alternatively, the @p Vector can be initialized by
-   * <tt>DoFAccessor::set_dof_values()</tt>.
+   * @deprecated Use the constructor with the reverse order of first and
+   * second argument.
    */
-  MappingQ1Eulerian (const VectorType  &euler_transform_vectors,
-                     const DoFHandler<dim,spacedim> &shiftmap_dof_handler);
+  MappingQ1Eulerian (const VectorType               &euler_vector,
+                     const DoFHandler<dim,spacedim> &euler_dof_handler) DEAL_II_DEPRECATED;
 
   /**
    * Return the mapped vertices of the cell. For the current class, this
