@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2002 - 2016 by the deal.II authors
+// Copyright (C) 2002 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -19,9 +19,13 @@
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/exceptions.h>
+#include <deal.II/base/iterator_range.h>
 #include <deal.II/grid/tria_iterator_base.h>
 
 #include <set>
+#ifdef DEAL_II_WITH_CXX11
+#include <tuple>
+#endif
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -299,6 +303,24 @@ namespace IteratorFilters
      */
     const bool only_locally_owned;
   };
+
+  /**
+   * Filter for iterators that evaluates to true if the iterator of the object
+   * pointed to is on the boundary.
+   *
+   * @author Bruno Turcksin, 2016
+   *
+   * @ingroup Iterators
+   */
+  class AtBoundary
+  {
+  public:
+    /**
+     * Evaluate the iterator and return true if the object at the boundary.
+     */
+    template <class Iterator>
+    bool operator () (const Iterator &i) const;
+  };
 }
 
 
@@ -354,11 +376,11 @@ namespace IteratorFilters
  * @endcode
  * then
  * @code
- *   std::bind2nd (std::ptr_fun(&level_equal_to<active_cell_iterator>), 3)
+ *   std_cxx11::bind (&level_equal_to<active_cell_iterator>, std_cxx11::_1, 3)
  * @endcode
  * is another valid predicate (here: a function that returns true if either
  * the iterator is past the end or the level is equal to the second argument;
- * this second argument is bound to a fixed value using the @p std::bind2nd
+ * this second argument is bound to a fixed value using the @p std::bind
  * function).
  *
  * Finally, classes can be predicates. The following class is one:
@@ -675,7 +697,7 @@ private:
 
     /**
      * Abstract function which in derived classes denotes the evaluation of
-     * the predicate on the give iterator.
+     * the predicate on the given iterator.
      */
     virtual bool operator () (const BaseIterator &bi) const = 0;
 
@@ -753,6 +775,128 @@ make_filtered_iterator (const BaseIterator &i,
 
 
 
+#ifdef DEAL_II_WITH_CXX11
+namespace internal
+{
+  namespace FilteredIterator
+  {
+    // The following classes create a nested sequencee of
+    // FilteredIterator<FilteredIterator<...<BaseIterator>...>> with as many
+    // levels of FilteredIterator classes as there are elements in the TypeList
+    // if the latter is given as a std::tuple<Args...>.
+    template <typename BaseIterator, typename TypeList>
+    struct NestFilteredIterators;
+
+    template <typename BaseIterator, typename Predicate>
+    struct NestFilteredIterators<BaseIterator, std::tuple<Predicate> >
+    {
+      typedef ::dealii::FilteredIterator<BaseIterator> type;
+    };
+
+    template <typename BaseIterator, typename Predicate, typename... Targs>
+    struct NestFilteredIterators<BaseIterator, std::tuple<Predicate, Targs...> >
+    {
+      typedef ::dealii::FilteredIterator<typename NestFilteredIterators<BaseIterator,
+              std::tuple<Targs...> >::type> type;
+    };
+  }
+}
+
+
+
+/**
+ * Filter the  given range of iterators using a Predicate. This allows to
+ * replace:
+ * @code
+ *   DoFHandler<dim> dof_handler;
+ *   ...
+ *   for (auto cell : dof_handler.active_cell_iterators())
+ *     {
+ *       if (cell->is_locally_owned())
+ *         {
+ *           fe_values.reinit (cell);
+ *           ...do the local integration on 'cell'...;
+ *         }
+ *     }
+ * @endcode
+ * by:
+ * @code
+ *   DoFHandler<dim> dof_handler;
+ *   ...
+ *   for (auto cell : filter_iterators(dof_handler.active_cell_iterators(),
+ *                                    IteratorFilters::LocallyOwnedCell())
+ *     {
+ *       fe_values.reinit (cell);
+ *       ...do the local integration on 'cell'...;
+ *     }
+ * @endcode
+ *
+ * @author Bruno Turcksin, 2016
+ * @relates FilteredIterator
+ * @ingroup CPP11
+ */
+template <typename BaseIterator, typename Predicate>
+IteratorRange<FilteredIterator<BaseIterator> >
+filter_iterators (IteratorRange<BaseIterator> i,
+                  const Predicate &p)
+{
+  FilteredIterator<BaseIterator> fi(p, *(i.begin()));
+  FilteredIterator<BaseIterator> fi_end(p, *(i.end()));
+
+  return IteratorRange<FilteredIterator<BaseIterator> > (fi, fi_end);
+}
+
+
+
+/**
+ * Filter the given range of iterators through an arbitrary number of
+ * Predicates. This allows to replace:
+ * @code
+ *   DoFHandler<dim> dof_handler;
+ *   ...
+ *   for (auto cell : dof_handler.active_cell_iterators())
+ *     {
+ *       if (cell->is_locally_owned())
+ *         {
+ *           if (cell->at_boundary())
+ *             {
+ *               fe_values.reinit (cell);
+ *               ...do the local integration on 'cell'...;
+ *             }
+ *         }
+ *     }
+ * @endcode
+ * by:
+ * @code
+ *   DoFHandler<dim> dof_handler;
+ *   ...
+ *   for (auto cell : filter_iterators(dof_handler.active_cell_iterators(),
+ *                                    IteratorFilters::LocallyOwnedCell(),
+ *                                    IteratorFilters::AtBoundary())
+ *     {
+ *       fe_values.reinit (cell);
+ *       ...do the local integration on 'cell'...;
+ *     }
+ * @endcode
+ *
+ * @author Bruno Turcksin, 2016
+ * @relates FilteredIterator
+ * @ingroup CPP11
+ */
+template <typename BaseIterator, typename Predicate, typename... Targs>
+IteratorRange<typename internal::FilteredIterator::
+NestFilteredIterators<BaseIterator,std::tuple<Predicate, Targs...> >::type>
+filter_iterators (IteratorRange<BaseIterator> i,
+                  const Predicate &p,
+                  const Targs... args)
+{
+  // Recursively create filtered iterators, one predicate at a time
+  auto fi = filter_iterators(i,p);
+  return filter_iterators(fi, args...);
+}
+#endif
+
+
 /* ------------------ Inline functions and templates ------------ */
 
 
@@ -816,10 +960,13 @@ FilteredIterator<BaseIterator> &
 FilteredIterator<BaseIterator>::
 operator = (const FilteredIterator &fi)
 {
-  Assert ((fi.state() != IteratorState::valid) || (*predicate)(fi),
-          ExcInvalidElement(fi));
-  BaseIterator::operator = (fi);
-  return *this;
+  // Using equivalent code to the one for 'operator=(const BaseIterator &bi)'
+  // below, some compiler would not cast fi to the base class of type
+  // BaseIterator but try to go through constructing a new Accessor from fi
+  // which fails. Hence, we just use an explicit upcast and call the above-
+  // mentioned method.
+  const BaseIterator &bi = fi;
+  return operator = (bi);
 }
 
 
@@ -1215,6 +1362,17 @@ namespace IteratorFilters
            active_fe_indices.find(i->active_fe_index()) != active_fe_indices.end();
   }
 
+
+
+// ---------------- IteratorFilters::AtBoundary ---------
+
+  template <class Iterator>
+  inline
+  bool
+  AtBoundary::operator () (const Iterator &i) const
+  {
+    return (i->at_boundary());
+  }
 }
 
 
