@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -28,11 +28,6 @@
 
 DEAL_II_NAMESPACE_OPEN
 
-template <int dim, int spacedim> class Triangulation;
-template <typename number> class Vector;
-template <typename number> class SparseMatrix;
-
-
 /**
  * This namespace provides a collection of functions for generating
  * triangulations for some basic geometries.
@@ -57,11 +52,12 @@ namespace GridGenerator
    * in 2D, etc) consisting of exactly one cell. The hypercube volume is the
    * tensor product interval $[left,right]^{\text{dim}}$ in the present number
    * of dimensions, where the limits are given as arguments. They default to
-   * zero and unity, then producing the unit hypercube. If the argument @p
-   * colorize is false, all boundary indicators are set to zero ("not
-   * colorized") for 2d and 3d. If it is true, the boundary is colorized as in
-   * hyper_rectangle(). In 1d the indicators are always colorized, see
-   * hyper_rectangle().
+   * zero and unity, then producing the unit hypercube.
+   *
+   * If the argument @p colorize is false, all boundary indicators are set to
+   * zero ("not colorized") for 2d and 3d. If it is true, the boundary is
+   * colorized as in hyper_rectangle(). In 1d the indicators are always
+   * colorized, see hyper_rectangle().
    *
    * @image html hyper_cubes.png
    *
@@ -133,14 +129,26 @@ namespace GridGenerator
    * Create a coordinate-parallel brick from the two diagonally opposite
    * corner points @p p1 and @p p2.
    *
-   * If the @p colorize flag is set, the @p boundary_ids of the surfaces are
-   * assigned, such that the lower one in @p x-direction is 0, the upper one
-   * is 1. The indicators for the surfaces in @p y-direction are 2 and 3, the
-   * ones for @p z are 4 and 5. Additionally, material ids are assigned to the
+   * If the @p colorize flag is @p true, the @p boundary_ids of the boundary
+   * faces are assigned, such that the lower one in @p x-direction is 0, the
+   * upper one is 1. The indicators for the surfaces in @p y-direction are 2
+   * and 3, the ones for @p z are 4 and 5. This corresponds to the numbers of
+   * faces of the unit square of cube as laid out in the documentation of the
+   * GeometryInfo class. Importantly, however, in 3d colorization does not set
+   * @p boundary_ids of <i>edges</i>, but only of <i>faces</i>, because each
+   * boundary edge is shared between two faces and it is not clear how the
+   * boundary id of an edge should be set in that case. This may later on lead
+   * to problems if one wants to assign boundary or manifold objects to parts
+   * of the boundary with certain boundary indicators since then the boundary
+   * object may not apply to the edges bounding the face it is meant to
+   * describe.
+   *
+   * Additionally, if @p colorize is @p true, material ids are assigned to the
    * cells according to the octant their center is in: being in the right half
-   * plane for any coordinate direction <i>x<sub>i</sub></i> adds
-   * 2<sup>i</sup>. For instance, the center point (1,-1,1) yields a material
-   * id 5.
+   * space for any coordinate direction <i>x<sub>i</sub></i> adds
+   * 2<sup>i</sup>. For instance, a cell with center point (1,-1,1) yields a
+   * material id 5, assuming that the center of the hyper rectangle lies at
+   * the origin.
    *
    * If @p dim < @p spacedim, this will create a @p dim dimensional object in
    * the first @p dim coordinate directions embedded into the @p spacedim
@@ -157,15 +165,14 @@ namespace GridGenerator
                         const bool                  colorize = false);
 
   /**
-   * Create a coordinate-parallel parallelepiped from the two diagonally
-   * opposite corner points @p p1 and @p p2. In direction @p i,
-   * <tt>repetitions[i]</tt> cells are generated.
+   * Create a coordinate-parallel brick from the two diagonally opposite
+   * corner points @p p1 and @p p2. The number of cells in coordinate
+   * direction @p i is given by the integer <tt>repetitions[i]</tt>.
    *
    * To get cells with an aspect ratio different from that of the domain, use
-   * different numbers of subdivisions in different coordinate directions. The
-   * minimum number of subdivisions in each direction is 1. @p repetitions is
-   * a list of integers denoting the number of subdivisions in each coordinate
-   * direction.
+   * different numbers of subdivisions, given by @p repetitions, in different
+   * coordinate directions. The minimum number of subdivisions in each
+   * direction is 1.
    *
    * If the @p colorize flag is set, the @p boundary_ids of the surfaces are
    * assigned, such that the lower one in @p x-direction is 0, the upper one
@@ -194,14 +201,15 @@ namespace GridGenerator
    * @param tria The Triangulation to create. It needs to be empty upon
    * calling this function.
    *
-   * @param repetitions A vector of dim positive values denoting the number of
-   * cells to generate in that direction.
+   * @param repetitions A vector of @p dim positive values denoting the number
+   * of cells to generate in that direction.
    *
    * @param p1 First corner point.
    *
    * @param p2 Second corner opposite to @p p1.
    *
-   * @param colorize Assign different boundary ids if set to true.
+   * @param colorize Assign different boundary ids if set to true. The same
+   * comments apply as for the hyper_rectangle() function.
    *
    */
   template <int dim, int spacedim>
@@ -217,7 +225,7 @@ namespace GridGenerator
    * denote the number of subdivisions in each coordinate direction, but a
    * sequence of step sizes for each coordinate direction. The domain will
    * therefore be subdivided into <code>step_sizes[i].size()</code> cells in
-   * coordinate direction <code>i</code>, with widths
+   * coordinate direction <code>i</code>, with width
    * <code>step_sizes[i][j]</code> for the <code>j</code>th cell.
    *
    * This function is therefore the right one to generate graded meshes where
@@ -233,7 +241,7 @@ namespace GridGenerator
                               const std::vector<std::vector<double> > &step_sizes,
                               const Point<dim>                        &p_1,
                               const Point<dim>                        &p_2,
-                              const bool                              colorize);
+                              const bool                              colorize=false);
 
   /**
    * Like the previous function, but with the following twist: the @p
@@ -284,6 +292,26 @@ namespace GridGenerator
   cheese (Triangulation<dim, spacedim> &tria,
           const std::vector<unsigned int> &holes);
 
+  /**
+   * A general quadrilateral in 2d or a general hexahedron in 3d. It is the
+   * responsibility of the user to provide the vertices in the right order (see
+   * the documentation of the GeometryInfo class) because the vertices are stored
+   * in the same order as they are given. It is also important to make sure that
+   * the volume of the cell is positive.
+   *
+   * If the argument @p colorize is false, all boundary indicators are set to
+   * zero ("not colorized") for 2d and 3d. If it is true, the boundary is
+   * colorized as in hyper_rectangle(). In 1d the indicators are always
+   * colorized, see hyper_rectangle().
+   *
+   * @author Bruno Turcksin
+   */
+  template <int dim>
+  void
+  general_cell(Triangulation<dim> &tria,
+               const std::vector<Point<dim> > &vertices,
+               const bool colorize = false);
+
   /**
    * A parallelogram. The first corner point is the origin. The @p dim
    * adjacent points are the ones given in the second argument and the fourth
@@ -466,6 +494,22 @@ namespace GridGenerator
                      const Point<spacedim>   &center = Point<spacedim>(),
                      const double        radius = 1.);
 
+  /**
+   * This class produces a hyper-ball intersected with the positive orthant
+   * relative to @p center, which contains three elements in 2d and four in 3d.
+   *
+   * The boundary indicators for the final triangulation are 0 for the curved
+   * boundary and 1 for the cut plane.
+   *
+   * The appropriate boundary class is HyperBallBoundary.
+   *
+   * @note The triangulation needs to be void upon calling this function.
+   */
+  template <int dim>
+  void quarter_hyper_ball (Triangulation<dim> &tria,
+                           const Point<dim>   &center = Point<dim>(),
+                           const double        radius = 1.);
+
   /**
    * This class produces a half hyper-ball around @p center, which contains
    * four elements in 2d and 6 in 3d. The cut plane is perpendicular to the
@@ -575,15 +619,32 @@ namespace GridGenerator
 
   /**
    * Initialize the given triangulation with a hyper-L (in 2d or 3d)
-   * consisting of exactly <tt>2^dim-1</tt> cells. It produces the hypercube
-   * with the interval [<i>left,right</i>] without the hypercube made out of
-   * the interval [<i>(left+right)/2,right</i>] for each coordinate.  All
-   * faces will have boundary indicator 0. This function will create the
-   * classical L-shape in 2d and it will look like the following in 3d:
+   * consisting of exactly <tt>2^dim-1</tt> cells. It produces the
+   * hypercube with the interval [<i>left,right</i>] without the
+   * hypercube made out of the interval [<i>(left+right)/2,right</i>]
+   * for each coordinate. Because the domain is about the simplest one
+   * with a reentrant (i.e., non-convex) corner, solutions of many
+   * partial differential equation have singularities at this
+   * corner. That is, at the corner, the gradient or a higher
+   * derivative (depending on the boundary conditions chosen) does not
+   * remain bounded. As a consequence, this domain is often used to
+   * test convergence of schemes when the solution lacks regularity.
+   *
+   * If the @p colorize flag is set, the @p boundary_ids of the
+   * surfaces are assigned, such that the left boundary is 0, and the
+   * others are set with growing number accordingly to the
+   * counterclockwise. Colorize option works only with 2-dimensional
+   * problem. This function will create the classical L-shape in 2d
+   * and it will look like the following in 3d:
    *
    * @image html hyper_l.png
    *
-   * This function is declared to exist for triangulations of all space
+   * @note The 3d domain is also often referred to as the "Fichera corner",
+   * named after Gaetano Fichera (1922-1996) who first computed an
+   * approximation of the corner singularity exponent of the lowest
+   * eigenfunction of the domain.
+   *
+   * This function exists for triangulations of all space
    * dimensions, but throws an error if called in 1d.
    *
    * @note The triangulation needs to be void upon calling this function.
@@ -591,7 +652,8 @@ namespace GridGenerator
   template <int dim>
   void hyper_L (Triangulation<dim> &tria,
                 const double        left = -1.,
-                const double        right= 1.);
+                const double        right= 1.,
+                const bool          colorize = false);
 
   /**
    * Initialize the given Triangulation with a hypercube with a slit. In each
@@ -605,6 +667,9 @@ namespace GridGenerator
    * declared to exist for triangulations of all space dimensions, but throws
    * an error if called in 1d.
    *
+   * If @p colorize is set to @p true, the faces forming the slit are marked
+   * with boundary id 1 and 2, respectively.
+   *
    * @note The triangulation needs to be void upon calling this function.
    */
   template <int dim>
@@ -745,13 +810,17 @@ namespace GridGenerator
 
   /**
    * Produce a domain that is the space between two cylinders in 3d, with
-   * given length, inner and outer radius and a given number of elements for
-   * this initial triangulation. If @p n_radial_cells is zero (as is the
+   * given length, inner and outer radius and a given number of elements. The
+   * cylinder shell is built around the $z$-axis with the two faces located
+   * at $z = 0$ and $z = $ @p length.
+   *
+   * If @p n_radial_cells is zero (as is the
    * default), then it is computed adaptively such that the resulting elements
    * have the least aspect ratio. The same holds for @p n_axial_cells.
    *
    * @note Although this function is declared as a template, it does not make
-   * sense in 1D and 2D.
+   * sense in 1D and 2D. Also keep in mind that this object is rotated
+   * and positioned differently than the one created by cylinder().
    *
    * @note The triangulation needs to be void upon calling this function.
    */
@@ -766,9 +835,17 @@ namespace GridGenerator
 
 
   /**
-   * Produce the surface meshing of the torus. The axis of the torus is the
-   * $y$-axis while the plane of the torus is the $x$-$z$ plane. The boundary
-   * of this object can be described by the TorusBoundary class.
+   * Produce the volume or surface mesh of a torus. The axis of the torus is
+   * the $y$-axis while the plane of the torus is the $x$-$z$ plane.
+   *
+   * If @p dim is 3, the mesh will be the volume of the torus. By default,
+   * the boundary faces will have manifold id 0 and you should attach a
+   * TorusManifold to it. The cells will have manifold id 1 and you should
+   * attach a SphericalManifold to it.
+   *
+   * If @p dim is 2, the mesh will describe the surface of the torus. All
+   * cells and faces will have manifold id 0 and you should attach a
+   * TorusManifold to it.
    *
    * @param tria The triangulation to be filled.
    *
@@ -776,10 +853,14 @@ namespace GridGenerator
    * torus containing the loop of cells. Must be greater than @p r.
    *
    * @param r The inner radius of the torus.
+   *
+   * @note Implemented for Triangulation<2,3> and Triangulation<3,3>.
    */
-  void torus (Triangulation<2,3> &tria,
-              const double        R,
-              const double        r);
+  template <int dim, int spacedim>
+  void torus (Triangulation<dim,spacedim> &tria,
+              const double R,
+              const double r);
+
 
 
   /**
@@ -822,7 +903,7 @@ namespace GridGenerator
    * before gluing the loop together.
    * @param R           The radius of the circle, which forms the middle line
    * of the torus containing the loop of cells. Must be greater than @p r.
-   * @param r           The radius of the cylinder bend together as loop.
+   * @param r           The radius of the cylinder bent together as a loop.
    */
   void moebius (Triangulation<3,3> &tria,
                 const unsigned int  n_cells,
@@ -1126,8 +1207,14 @@ namespace GridGenerator
   DeclException1 (ExcInvalidRepetitionsDimension,
                   int,
                   << "The vector of repetitions  must have "
-                  << arg1 <<" elements.");
+                  << arg1 << " elements.");
 
+  /**
+   * Exception for input that is not properly oriented.
+   */
+  DeclExceptionMsg (ExcInvalidInputOrientation,
+                    "The input to this function is oriented in a way that will"
+                    " cause all cells to have negative measure.");
   ///@}
 }
 
