@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -29,27 +29,31 @@ DEAL_II_NAMESPACE_OPEN
 
 template <int dim, int space_dim> class Triangulation;
 template <int dim> struct CellData;
-struct SubCellData;
-
 
 /**
  * This class implements an input mechanism for grid data. It allows to read a
  * grid structure into a triangulation object. At present, UCD (unstructured
  * cell data), DB Mesh, XDA, Gmsh, Tecplot, NetCDF, UNV, VTK, and Cubit are
  * supported as input format for grid data. Any numerical data other than
- * geometric (vertex locations) and topological (how vertices form cells)
- * information is ignored.
+ * geometric (vertex locations) and topological (how vertices form cells,
+ * faces, and edges) information is ignored, but the readers for the various
+ * formats generally do read information that associates material ids or
+ * boundary ids to cells or faces (see
+ * @ref GlossMaterialId "this"
+ * and
+ * @ref GlossBoundaryIndicator "this"
+ * glossary entry for more information).
  *
  * @note Since deal.II only supports line, quadrilateral and hexahedral
  * meshes, the functions in this class can only read meshes that consist
  * exclusively of such cells. If you absolutely need to work with a mesh that
  * uses triangles or tetrahedra, then your only option is to convert the mesh
- * to quadrilaterals and hexahedra. A tool that can do this is tethex, see
- * http://code.google.com/p/tethex/wiki/Tethex .
+ * to quadrilaterals and hexahedra. A tool that can do this is tethex,
+ * available <a href="https://github.com/martemyev/tethex">here</a>.
  *
  * The mesh you read will form the coarsest level of a @p Triangulation
- * object. As such, it must not contain hanging nodes or other forms or
- * adaptive refinement and strange things will happen if the mesh represented
+ * object. As such, it must not contain hanging nodes or other forms of
+ * adaptive refinement, or strange things will happen if the mesh represented
  * by the input file does in fact have them. This is due to the fact that most
  * mesh description formats do not store neighborship information between
  * cells, so the grid reading functions have to regenerate it. They do so by
@@ -67,25 +71,25 @@ struct SubCellData;
  * has been adaptively refined, then this class is not your solution; rather
  * take a look at the PersistentTriangulation class.
  *
- * @note It is not uncommon to experience unexpected problems when reading
- * generated meshes for the first time using this class. If this applies to
- * you, be sure to read the documentation right until the end, and also read
- * the documentation of the GridReordering class.
+ * To read grid data, the triangulation to be filled has to be empty.
+ * Upon calling the functions of this class, the input file may
+ * contain only lines in one dimension; lines and quads in two
+ * dimensions; and lines, quads, and hexes in three dimensions. All
+ * other cell types (e.g. triangles in two dimensions, triangles or
+ * tetrahedra in 3d) are rejected.  (Here, the "dimension" refers to
+ * the dimensionality of the mesh; it may be embedded in a higher
+ * dimensional space, such as a mesh on the two-dimensional surface of
+ * the sphere embedded in 3d, or a 1d mesh that discretizes a line in
+ * 3d.) The result will be a triangulation that consists of the cells
+ * described in the input file, and to the degree possible with
+ * material indicators and boundary indicators correctly set as
+ * described in the input file.
  *
- * To read grid data, the triangulation to be fed with has to be empty. When
- * giving a file which does not contain the assumed information or which does
- * not keep to the right format, the state of the triangulation will be
- * undefined afterwards. Upon input, only lines in one dimension and line and
- * quads in two dimensions are accepted. All other cell types (e.g. triangles
- * in two dimensions, quads and hexes in 3d) are rejected. The vertex and cell
- * numbering in the input file, which need not be consecutively, is lost upon
- * transfer to the triangulation object, since this one needs consecutively
- * numbered elements.
- *
- * Material indicators are accepted to denote the material ID of cells and to
- * denote boundary part indication for lines in 2D. Read the according
- * sections in the documentation of the Triangulation class for further
- * details.
+ * @note You can not expect vertex and cell numbers in the triangulation
+ * to match those in the input file. (This is already clear based on the
+ * fact that we number cells and vertices separately, whereas this is not
+ * the case for some input file formats; some formats also do not require
+ * consecutive numbering, or start numbering at indices other than zero.)
  *
  *
  * <h3>Supported input formats</h3>
@@ -154,15 +158,15 @@ struct SubCellData;
  * generator, see http://www.salome-platform.org/ . The sections of the format
  * that the GridIn::read_unv function supports are documented here:
  * <ul>
- * <li> section 2411: http://www.sdrl.uc.edu/universal-file-formats-for-modal-
- * analysis-testing-1/file-format-storehouse/unv_2411.htm
- * <li> section 2412: http://www.sdrl.uc.edu/universal-file-formats-for-modal-
- * analysis-testing-1/file-format-storehouse/unv_2412.htm
- * <li> section 2467: http://www.sdrl.uc.edu/universal-file-formats-for-modal-
- * analysis-testing-1/file-format-storehouse/unv_2467.htm
+ * <li> section 2411:
+ * http://www.sdrl.uc.edu/sdrl/referenceinfo/universalfileformats/file-format-storehouse/universal-dataset-number-2411
+ * <li> section 2412:
+ * http://www.sdrl.uc.edu/sdrl/referenceinfo/universalfileformats/file-format-storehouse/universal-dataset-number-2412
+ * <li> section 2467:
+ * http://www.sdrl.uc.edu/sdrl/referenceinfo/universalfileformats/file-format-storehouse/universal-dataset-number-2467
  * <li> all sections of this format, even if they may not be supported in our
- * reader, can be found here: http://www.sdrl.uc.edu/universal-file-formats-
- * for-modal-analysis-testing-1/file-format-storehouse/file-formats
+ * reader, can be found here:
+ * http://www.sdrl.uc.edu/sdrl/referenceinfo/universalfileformats/file-format-storehouse
  * </ul>
  * Note that Salome, let's say in 2D, can only make a quad mesh on an object
  * that has exactly 4 edges (or 4 pieces of the boundary). That means, that if
@@ -206,14 +210,14 @@ struct SubCellData;
  * <li> <tt>Cubit</tt> format: deal.II doesn't directly support importing from
  * Cubit at this time. However, Cubit can export in UCD format using a simple
  * plug-in, and the resulting UCD file can then be read by this class. The
- * plug-in script can be found on the deal.II wiki page,
- * http://code.google.com/p/dealii/wiki/MeshInputAndOutput .
+ * plug-in script can be found on the deal.II wiki page under
+ * <a href="https://github.com/dealii/dealii/wiki/Mesh-Input-And-Output">Mesh
+ * Input and Output</a>.
  *
  * Alternatively, Cubit can generate ABAQUS files that can be read in via the
  * read_abaqus() function. This may be a better option for geometries with
- * complex boundary condition surfaces and multiple materials
- *  - information which is currently not easily obtained through
- * Cubit's python interface.
+ * complex boundary condition surfaces and multiple materials - information
+ * which is currently not easily obtained through Cubit's python interface.
  *
  * </ul>
  *
@@ -365,12 +369,21 @@ public:
 
   /**
    * Read grid data from an ucd file. Numerical data is ignored.
+   * It is not possible to use a ucd file to set both boundary_id and
+   * manifold_id for the same cell. Yet it is possible to use
+   * the flag apply_all_indicators_to_manifolds to decide if
+   * the indicators in the file refer to manifolds (flag set to true)
+   * or boundaries (flag set to false).
    */
-  void read_ucd (std::istream &in);
+  void read_ucd (std::istream                                  &in,
+                 const bool apply_all_indicators_to_manifolds=false);
 
   /**
    * Read grid data from an Abaqus file. Numerical and constitutive data is
-   * ignored.
+   * ignored. As in the case of the ucd file format, it is possible to use
+   * the flag apply_all_indicators_to_manifolds to decide if
+   * the indicators in the file refer to manifolds (flag set to true)
+   * or boundaries (flag set to false).
    *
    * @note The current implementation of this mesh reader is suboptimal, and
    * may therefore be slow for large meshes.
@@ -389,8 +402,8 @@ public:
    * - Files generated in Abaqus CAE 6.12 have been verified to be
    * correctly imported, but older (or newer) versions of Abaqus may also
    * generate valid input decks.
-   * - Files generated using Cubit 11.x, 12.x and 13.x are valid, but only
-   * when using a specific set of export steps. These are as follows:
+   * - Files generated using Cubit 11.x, 12.x, 13.x, 14.x and 15.x are valid,
+   * but only when using a specific set of export steps. These are as follows:
    *     - Go to "Analysis setup mode" by clicking on the disc icon in the
    * toolbar on the right.
    *     - Select "Export Mesh" under "Operation" by clicking on the
@@ -404,7 +417,8 @@ public:
    * ID's". An invalid file will encounter errors if this box is left checked.
    *     - Click apply.
    */
-  void read_abaqus (std::istream &in);
+  void read_abaqus (std::istream                                  &in,
+                    const bool apply_all_indicators_to_manifolds=false);
 
   /**
    * Read grid data from a file containing data in the DB mesh format.
@@ -442,7 +456,7 @@ public:
   void read_tecplot (std::istream &in);
 
   /**
-   * Returns the standard suffix for a file in this format.
+   * Return the standard suffix for a file in this format.
    */
   static std::string default_suffix (const Format format);
 
@@ -499,6 +513,15 @@ public:
   /**
    * Exception
    */
+  DeclException3 (ExcInvalidVertexIndexGmsh,
+                  int, int, int,
+                  << "While creating cell " << arg1
+                  << " (which is numbered as " << arg2
+                  << " in the input file), you are referencing a vertex with index " << arg3
+                  << " but no vertex with this index has been described in the input file.");
+  /**
+   * Exception
+   */
   DeclException0 (ExcInvalidDBMeshFormat);
   /**
    * Exception
