@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2001 - 2016 by the deal.II authors
+// Copyright (C) 2001 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -18,11 +18,14 @@
 
 
 #include <deal.II/base/config.h>
+#include <deal.II/base/geometry_info.h>
+#include <deal.II/dofs/dof_handler.h>
 #include <deal.II/fe/mapping.h>
+#include <deal.II/fe/mapping_q1.h>
 #include <deal.II/grid/tria.h>
 #include <deal.II/grid/tria_accessor.h>
 #include <deal.II/grid/tria_iterator.h>
-#include <deal.II/fe/mapping_q1.h>
+#include <deal.II/hp/dof_handler.h>
 
 #include <bitset>
 #include <list>
@@ -30,12 +33,16 @@
 
 DEAL_II_NAMESPACE_OPEN
 
+namespace parallel
+{
+  namespace distributed
+  {
+    template <int, int> class Triangulation;
+  }
+}
 
-template <int, int> class DoFHandler;
-template <int, int> class Mapping;
 namespace hp
 {
-  template <int, int> class DoFHandler;
   template <int, int> class MappingCollection;
 }
 
@@ -155,6 +162,14 @@ namespace GridTools
   double cell_measure (const std::vector<Point<dim> > &all_vertices,
                        const unsigned int (&vertex_indices)[GeometryInfo<dim>::vertices_per_cell]);
 
+  /**
+   * A version of the last function that can accept input for nonzero
+   * codimension cases. This function only exists to aid generic programming
+   * and calling it will just raise an exception.
+   */
+  template <int dim, typename T>
+  double cell_measure (const T &, ...);
+
   /*@}*/
   /**
    * @name Functions supporting the creation of meshes
@@ -257,17 +272,50 @@ namespace GridTools
   void rotate (const double      angle,
                Triangulation<2> &triangulation);
 
+  /**
+   * Rotate all vertices of the given @p triangulation in counter-clockwise
+   * direction around the axis with the given index. Otherwise like the
+   * function above.
+   *
+   * @param[in] angle Angle in radians to rotate the Triangulation by.
+   * @param[in] axis Index of the coordinate axis to rotate around, keeping
+   * that coordinate fixed (0=x axis, 1=y axis, 2=z axis).
+   * @param[in,out] triangulation The Triangulation object to rotate.
+   *
+   * @note Implemented for dim=1, 2, and 3.
+   */
+  template<int dim>
+  void
+  rotate (const double          angle,
+          const unsigned int    axis,
+          Triangulation<dim,3> &triangulation);
+
   /**
    * Transform the given triangulation smoothly to a different domain where,
    * typically, each of the vertices at the boundary of the triangulation is
    * mapped to the corresponding points in the @p new_points map.
    *
-   * The way this function works is that it solves a Laplace equation for each
+   * The unknown displacement field $u_d(\mathbf x)$ in direction $d$ is obtained from
+   * the minimization problem
+   * \f[
+   *   \min\, \int
+   *   \frac{1}{2}
+   *   c(\mathbf x)
+   *   \mathbf \nabla u_d(\mathbf x) \cdot
+   *   \mathbf \nabla u_d(\mathbf x)
+   *   \,\rm d x
+   * \f]
+   * subject to prescribed constraints. The minimizer is obtained by solving the Laplace equation
    * of the dim components of a displacement field that maps the current
-   * domain into one described by @p new_points . The @p new_points array
-   * therefore represents the boundary values of this displacement field. The
-   * function then evaluates this displacement field at each vertex in the
-   * interior and uses it to place the mapped vertex where the displacement
+   * domain into one described by @p new_points . Linear finite elements with
+   * four Gaussian quadrature points in each direction are used. The difference
+   * between the vertex positions specified in @p new_points and their current
+   * value in @p tria therefore represents the prescribed values of this
+   * displacement field at the boundary of the domain, or more precisely at all
+   * of those locations for which @p new_points provides values (which may be
+   * at part of the boundary, or even in the interior of the domain). The
+   * function then evaluates this displacement field at each unconstrained
+   * vertex and uses it to place the mapped vertex where the displacement
    * field locates it. Because the solution of the Laplace equation is smooth,
    * this guarantees a smooth mapping from the old domain to the new one.
    *
@@ -290,15 +338,24 @@ namespace GridTools
    * Should this function be provided, sensible results can only be expected
    * if all coefficients are positive.
    *
+   * @param[in] solve_for_absolute_positions If set to <code>true</code>, the
+   * minimization problem is formulated with respect to the final vertex positions
+   * as opposed to their displacement. The two formulations are equivalent for
+   * the homogeneous problem (default value of @p coefficient), but they
+   * result in very different mesh motion otherwise. Since in most cases one will
+   * be using a non-constant coefficient in displacement formulation, the default
+   * value of this parameter is <code>false</code>.
+   *
    * @note This function is not currently implemented for the 1d case.
    */
   template <int dim>
   void laplace_transform (const std::map<unsigned int,Point<dim> > &new_points,
                           Triangulation<dim> &tria,
-                          const Function<dim,double> *coefficient = 0);
+                          const Function<dim,double> *coefficient = 0,
+                          const bool solve_for_absolute_positions = false);
 
   /**
-   * Returns a std::map with all vertices of faces located in the boundary
+   * Return a std::map with all vertices of faces located in the boundary
    *
    * @param[in] tria The Triangulation object.
    */
@@ -332,6 +389,78 @@ namespace GridTools
                        Triangulation<dim, spacedim> &triangulation,
                        const bool   keep_boundary=true);
 
+  /**
+   * Remove hanging nodes from a grid. If the @p isotropic parameter is set
+   * to @p false (default) this function detects cells with hanging nodes and
+   * refines the neighbours in the direction that removes hanging nodes.
+   * If the @p isotropic parameter is set
+   * to @p true, the neighbours refinement is made in each directions.
+   * In order to remove all hanging nodes this procedure has to be repeated:
+   * this could require a large number of iterations.
+   * To avoid this a max number (@p max_iterations) of iteration is provided.
+   *
+   * Consider the following grid:
+   * @image html remove_hanging_nodes-hanging.png
+   *
+   * @p isotropic == @p false would return:
+   * @image html remove_hanging_nodes-aniso.png
+   *
+   * @p isotropic == @p true would return:
+   * @image html remove_hanging_nodes-isotro.png
+   *
+   * @param[in,out] tria Triangulation to refine.
+   *
+   * @param[in] isotropic If true refine cells in each directions, otherwise
+   * (default value) refine the cell in the direction that removes hanging node.
+   *
+   * @param[in] max_iterations At each step only closest cells to hanging nodes
+   * are refined. The code may require a lot of iterations to remove all
+   * hanging nodes. @p max_iterations is the maximum number of iteration
+   * allowed. If @p max_iterations == numbers::invalid_unsigned_int this
+   * function continues refining until there are no hanging nodes.
+   *
+   * @note In the case of parallel codes, this function should be combined
+   * with GridGenerator::flatten_triangulation.
+   *
+   * @author Mauro Bardelloni, Luca Heltai, Andrea Mola, 2016
+   */
+  template<int dim, int spacedim>
+  void
+  remove_hanging_nodes( Triangulation<dim,spacedim> &tria,
+                        const bool isotropic = false,
+                        const unsigned int max_iterations = 100);
+
+  /**
+   * Refine a mesh anisotropically such that the resulting mesh is composed by
+   * cells with maximum ratio between dimensions less than @p max_ratio.
+   * This procedure requires an algorithm that may not terminate. Consequently,
+   * it is possible to set a maximum number of iterations through the
+   * @p max_iterations parameter.
+   *
+   * Starting from a cell like this:
+   * @image html remove_anisotropy-coarse.png
+   *
+   * This function would return:
+   * @image html remove_anisotropy-refined.png
+   *
+   * @param[in,out] tria Triangulation to refine.
+   *
+   * @param[in] max_ratio Maximum value allowed among the ratio between
+   * the dimensions of each cell.
+   *
+   * @param[in] max_iterations Maximum number of iterations allowed.
+   *
+   * @note In the case of parallel codes, this function should be combined
+   * with GridGenerator::flatten_triangulation and GridTools::remove_hanging_nodes.
+   *
+   * @author Mauro Bardelloni, Luca Heltai, Andrea Mola, 2016
+   */
+  template<int dim, int spacedim>
+  void
+  remove_anisotropy(  Triangulation<dim,spacedim> &tria,
+                      const double max_ratio = 1.6180339887,
+                      const unsigned int max_iterations = 5);
+
   /*@}*/
   /**
    * @name Finding cells and vertices of a triangulation
@@ -656,6 +785,22 @@ namespace GridTools
   compute_local_to_global_vertex_index_map(
     const parallel::distributed::Triangulation<dim,spacedim> &triangulation);
 
+  /**
+   * Return the highest value among ratios between extents in each of the
+   * coordinate directions of a @p cell. Moreover, return the dimension
+   * relative to the highest elongation.
+   *
+   * @param[in] cell an iterator pointing to the cell.
+   *
+   * @return  A std::pair<unsigned int, double> such that the @p first value
+   * is the dimension of the highest elongation and the @p second value is the
+   * ratio among the dimensions of the @p cell.
+   *
+   * @author Mauro Bardelloni, Luca Heltai, Andrea Mola, 2016
+   */
+  template<int dim, int spacedim>
+  std::pair<unsigned int, double>
+  get_longest_direction(typename Triangulation<dim, spacedim>::active_cell_iterator cell);
 
   /*@}*/
   /**
@@ -1019,6 +1164,13 @@ namespace GridTools
    * identify the cells in the output Triangulation and corresponding cells in
    * the input list.
    *
+   * The function copies the location of vertices of cells from the cells of the
+   * source triangulation to the triangulation that is built from the list of
+   * patch cells.  This adds support for triangulations which have been
+   * perturbed or smoothed in some manner which makes the triangulation
+   * deviate from the standard deal.ii refinement strategy of placing new
+   * vertices at midpoints of faces or edges.
+   *
    * The operation implemented by this function is frequently used in the
    * definition of error estimators that need to solve "local" problems on
    * each cell and its neighbors. A similar construction is necessary in the
@@ -1119,30 +1271,7 @@ namespace GridTools
 
 
   /*@}*/
-  /**
-   * @name Lower-dimensional meshes for parts of higher-dimensional meshes
-   */
-  /*@{*/
 
-
-#ifdef _MSC_VER
-  // Microsoft's VC++ has a bug where it doesn't want to recognize that
-  // an implementation (definition) of the extract_boundary_mesh function
-  // matches a declaration. This can apparently only be avoided by
-  // doing some contortion with the return type using the following
-  // intermediate type. This is only used when using MS VC++ and uses
-  // the direct way of doing it otherwise
-  template <template <int,int> class MeshType, int dim, int spacedim>
-  struct ExtractBoundaryMesh
-  {
-    typedef
-    std::map<typename MeshType<dim-1,spacedim>::cell_iterator,
-        typename MeshType<dim,spacedim>::face_iterator>
-        return_type;
-  };
-#endif
-
-  /*@}*/
   /**
    * @name Dealing with periodic domains
    */
@@ -1329,6 +1458,12 @@ namespace GridTools
    * times with different boundary ids to generate a vector with all periodic
    * pairs.
    *
+   * @note Since the periodic face pairs are found on the coarsest mesh level,
+   * it is necessary to ensure that the coarsest level faces have the correct
+   * boundary indicators set. In general, this means that one must first set
+   * all boundary indicators on the coarse grid before performing any global
+   * or local grid refinement.
+   *
    * @author Daniel Arndt, Matthias Maier, 2013 - 2015
    */
   template <typename MeshType>
@@ -1344,7 +1479,7 @@ namespace GridTools
 
 
   /**
-   * This compatibility version of collect_periodic_face_pairs() only works on
+   * This compatibility version of collect_periodic_faces() only works on
    * grids with cells in
    * @ref GlossFaceOrientation "standard orientation".
    *
@@ -1359,7 +1494,7 @@ namespace GridTools
    *
    * See above function for further details.
    *
-   * @note This version of collect_periodic_face_pairs() will not work on
+   * @note This version of collect_periodic_faces() will not work on
    * meshes with cells not in
    * @ref GlossFaceOrientation "standard orientation".
    *
@@ -1474,7 +1609,7 @@ namespace GridTools
    */
   DeclException1 (ExcScalingFactorNotPositive,
                   double,
-                  << "The scaling factor must be positive, but is " << arg1);
+                  << "The scaling factor must be positive, but it is " << arg1 << ".");
   /**
    * Exception
    */
@@ -1499,8 +1634,8 @@ namespace GridTools
    */
   DeclException1 (ExcVertexNotUsed,
                   unsigned int,
-                  << "The given vertex " << arg1
-                  << " is not used in the given triangulation");
+                  << "The given vertex with index " << arg1
+                  << " is not used in the given triangulation.");
 
 
   /*@}*/
@@ -1515,6 +1650,12 @@ namespace GridTools
 
 namespace GridTools
 {
+  template <int dim, typename T>
+  double cell_measure (const T &, ...)
+  {
+    Assert(false, ExcNotImplemented());
+    return std::numeric_limits<double>::quiet_NaN();
+  }
 
   template <int dim, typename Predicate, int spacedim>
   void transform (const Predicate    &predicate,
@@ -1586,6 +1727,9 @@ namespace GridTools
                      + cell->face(face)->vertex(2) + cell->face(face)->vertex(3)) / 4.0;
               }
       }
+
+    // Make sure FEValues notices that the mesh has changed
+    triangulation.signals.mesh_movement();
   }
 
 
@@ -1662,20 +1806,6 @@ namespace GridTools
             }
         }
   }
-
-
-
-// declaration of explicit specializations
-
-  template <>
-  double
-  cell_measure<3>(const std::vector<Point<3> > &all_vertices,
-                  const unsigned int (&vertex_indices) [GeometryInfo<3>::vertices_per_cell]);
-
-  template <>
-  double
-  cell_measure<2>(const std::vector<Point<2> > &all_vertices,
-                  const unsigned int (&vertex_indices) [GeometryInfo<2>::vertices_per_cell]);
 }
 
 #endif
