@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2015 by the deal.II authors
+// Copyright (C) 1999 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -35,16 +35,6 @@ DEAL_II_NAMESPACE_OPEN
 namespace parallel
 {
   template <int, int> class Triangulation;
-
-  namespace distributed
-  {
-    template <int, int> class Triangulation;
-  }
-
-  namespace shared
-  {
-    template <int, int> class Triangulation;
-  }
 }
 
 
@@ -391,7 +381,7 @@ TriaAccessorBase<structdim,dim,spacedim>::objects() const
     // get sub-objects. note that the
     // current class is only used for
     // objects that are *not* cells
-    return *dealii::internal::TriaAccessorBase::get_objects (this->tria->faces,
+    return *dealii::internal::TriaAccessorBase::get_objects (this->tria->faces.get(),
                                                              dealii::internal::int2type<structdim> ());
   else
     return *dealii::internal::TriaAccessorBase::get_objects (&this->tria->levels[this->present_level]->cells,
@@ -1854,16 +1844,6 @@ TriaAccessor<structdim, dim, spacedim>::boundary_id () const
 
 
 
-template <int structdim, int dim, int spacedim>
-types::boundary_id
-TriaAccessor<structdim, dim, spacedim>::boundary_indicator () const
-{
-  return boundary_id();
-}
-
-
-
-
 template <int structdim, int dim, int spacedim>
 void
 TriaAccessor<structdim, dim, spacedim>::
@@ -1871,6 +1851,10 @@ set_boundary_id (const types::boundary_id boundary_ind) const
 {
   Assert (structdim<dim, ExcImpossibleInDim(dim));
   Assert (this->used(), TriaAccessorExceptions::ExcCellNotUsed());
+  Assert (boundary_ind != numbers::internal_face_boundary_id,
+          ExcMessage("You are trying to set the boundary_id to an illegal value (numbers::internal_face_boundary_id is reserved)."));
+  Assert (this->at_boundary(),
+          ExcMessage("You are trying to set the boundary_id of an internal object, which is illegal!"));
 
   this->objects().boundary_or_material_id[this->present_index].boundary_id = boundary_ind;
 }
@@ -1880,9 +1864,12 @@ set_boundary_id (const types::boundary_id boundary_ind) const
 template <int structdim, int dim, int spacedim>
 void
 TriaAccessor<structdim, dim, spacedim>::
-set_boundary_indicator (const types::boundary_id boundary_ind) const
+set_boundary_id_internal (const types::boundary_id boundary_ind) const
 {
-  set_boundary_id (boundary_ind);
+  Assert (structdim<dim, ExcImpossibleInDim(dim));
+  Assert (this->used(), TriaAccessorExceptions::ExcCellNotUsed());
+
+  this->objects().boundary_or_material_id[this->present_index].boundary_id = boundary_ind;
 }
 
 
@@ -1915,16 +1902,6 @@ set_all_boundary_ids (const types::boundary_id boundary_ind) const
 
 
 
-template <int structdim, int dim, int spacedim>
-void
-TriaAccessor<structdim, dim, spacedim>::
-set_all_boundary_indicators (const types::boundary_id boundary_ind) const
-{
-  set_all_boundary_ids (boundary_ind);
-}
-
-
-
 template <int structdim, int dim, int spacedim>
 bool
 TriaAccessor<structdim, dim, spacedim>::at_boundary () const
@@ -2772,16 +2749,6 @@ TriaAccessor<0, 1, spacedim>::boundary_id () const
 
 
 
-template <int spacedim>
-inline
-types::boundary_id
-TriaAccessor<0, 1, spacedim>::boundary_indicator () const
-{
-  return boundary_id();
-}
-
-
-
 template <int spacedim>
 inline
 types::manifold_id
@@ -2922,17 +2889,6 @@ TriaAccessor<0, 1, spacedim>::set_boundary_id (const types::boundary_id b)
 
 
 
-template <int spacedim>
-inline
-void
-TriaAccessor<0, 1, spacedim>::set_boundary_indicator (const types::boundary_id b)
-{
-  set_boundary_id (b);
-}
-
-
-
-
 template <int spacedim>
 inline
 void
@@ -2952,15 +2908,6 @@ void TriaAccessor<0, 1, spacedim>::set_all_boundary_ids (const types::boundary_i
 
 
 
-template <int spacedim>
-inline
-void TriaAccessor<0, 1, spacedim>::set_all_boundary_indicators (const types::boundary_id b)
-{
-  set_all_boundary_ids (b);
-}
-
-
-
 template <int spacedim>
 inline
 void TriaAccessor<0, 1, spacedim>::set_all_manifold_ids (const types::manifold_id b)
@@ -3275,7 +3222,6 @@ CellAccessor<3>::subface_case(const unsigned int face_no) const
     {
     case RefinementCase<3>::no_refinement:
       return dealii::internal::SubfaceCase<3>::case_none;
-      break;
     case RefinementCase<3>::cut_x:
       if (face(face_no)->child(0)->has_children())
         {
@@ -3301,7 +3247,6 @@ CellAccessor<3>::subface_case(const unsigned int face_no) const
           else
             return dealii::internal::SubfaceCase<3>::case_x;
         }
-      break;
     case RefinementCase<3>::cut_y:
       if (face(face_no)->child(0)->has_children())
         {
@@ -3327,10 +3272,8 @@ CellAccessor<3>::subface_case(const unsigned int face_no) const
           else
             return dealii::internal::SubfaceCase<3>::case_y;
         }
-      break;
     case RefinementCase<3>::cut_xy:
       return dealii::internal::SubfaceCase<3>::case_xy;
-      break;
     default:
       Assert(false, ExcInternalError());
     }
