@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2006 - 2015 by the deal.II authors
+// Copyright (C) 2006 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -117,7 +117,7 @@ namespace internal
 
       /**
        * We use this union to store boundary and material data. Because only
-       * one one out of these two is actually needed here, we use an union.
+       * one out of these two is actually needed here, we use an union.
        */
       struct BoundaryOrMaterialId
       {
@@ -436,7 +436,7 @@ namespace internal
        * we have this convention. We also know that it is always possible to
        * have opposite faces have parallel normal vectors. (For both things,
        * see the Agelek, Anderson, Bangerth, Barth paper mentioned in the
-       * publications list.)
+       * documentation of the GridReordering class.)
        *
        * The problem is that we originally had another condition, namely that
        * faces 0, 2 and 6 have normals that point into the cell, while the
@@ -667,6 +667,8 @@ namespace internal
     inline
     TriaObjects<G>::TriaObjects()
       :
+      next_free_single (numbers::invalid_unsigned_int),
+      next_free_pair (numbers::invalid_unsigned_int),
       reverse_order_next_free_single (false),
       user_data_type(data_unknown)
     {}
