@@ -1,7 +1,6 @@
 // ---------------------------------------------------------------------
-// $Id: dof_faces.h 31932 2013-12-08 02:15:54Z heister $
 //
-// Copyright (C) 2006 - 2013 by the deal.II authors
+// Copyright (C) 2006 - 2015 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef __deal2__hp_dof_faces_h
-#define __deal2__hp_dof_faces_h
+#ifndef dealii__hp_dof_faces_h
+#define dealii__hp_dof_faces_h
 
 
 #include <deal.II/base/config.h>
@@ -38,63 +37,61 @@ namespace internal
   {
     /**
      * Store the indices of the degrees of freedom which are located on
-     * objects of dimension @p structdim < dim, i.e., for faces or edges
-     * of cells. This is opposed to the internal::hp::DoFLevels class
-     * that stores the DoF indices on cells.
+     * objects of dimension @p structdim < dim, i.e., for faces or edges of
+     * cells. This is opposed to the internal::hp::DoFLevels class that stores
+     * the DoF indices on cells.
      *
      * The things we store here is very similar to what is stored in the
      * internal::DoFHandler::DoFObjects classes (see there for more
-     * information, in particular on the layout of the class hierarchy,
-     * and the use of file names).
+     * information, in particular on the layout of the class hierarchy, and
+     * the use of file names).
      *
      * <h4>Offset computations</h4>
      *
-     * For hp methods, not all cells may use the same finite element, and
-     * it is consequently more complicated to determine where the DoF
-     * indices for a given line, quad, or hex are stored. As described in
-     * the documentation of the internal::DoFHandler::DoFLevel class, we
-     * can compute the location of the first line DoF, for example, by
-     * calculating the offset as <code>line_index *
-     * dof_handler.get_fe().dofs_per_line</code>. This of course doesn't
-     * work any more if different lines may have different numbers of
-     * degrees of freedom associated with them. Consequently, rather than
-     * using this simple multiplication, the dofs array has an associated
-     * array dof_offsets. The data corresponding to a
-     * line then starts at index <code>line_dof_offsets[line_index]</code>
-     * within the <code>line_dofs</code> array.
+     * For hp methods, not all cells may use the same finite element, and it
+     * is consequently more complicated to determine where the DoF indices for
+     * a given line, quad, or hex are stored. As described in the
+     * documentation of the internal::DoFHandler::DoFLevel class, we can
+     * compute the location of the first line DoF, for example, by calculating
+     * the offset as <code>line_index *
+     * dof_handler.get_fe().dofs_per_line</code>. This of course doesn't work
+     * any more if different lines may have different numbers of degrees of
+     * freedom associated with them. Consequently, rather than using this
+     * simple multiplication, the dofs array has an associated array
+     * dof_offsets. The data corresponding to a line then starts at index
+     * <code>line_dof_offsets[line_index]</code> within the
+     * <code>line_dofs</code> array.
      *
      *
      * <h4>Multiple data sets per object</h4>
      *
-     * If two adjacent cells use different finite elements, then
-     * the face that they share needs to store DoF indices for both
-     * involved finite elements. While faces therefore have to have at
-     * most two sets of DoF indices, it is easy to see that edges and
-     * vertices can have as many sets of DoF indices associated with them
-     * as there are adjacent cells.
+     * If two adjacent cells use different finite elements, then the face that
+     * they share needs to store DoF indices for both involved finite
+     * elements. While faces therefore have to have at most two sets of DoF
+     * indices, it is easy to see that edges and vertices can have as many
+     * sets of DoF indices associated with them as there are adjacent cells.
      *
-     * Consequently, for objects that have a lower dimensionality than
-     * cells, we have to store a map from the finite element index to the
-     * set of DoF indices associated. Since real sets are typically very
-     * inefficient to store, and since most of the time we expect the
-     * number of individual keys to be small (frequently, adjacent cells
-     * will have the same finite element, and only a single entry will
-     * exist in the map), what we do is instead to store a linked list. In
-     * this format, the first entry starting at position
-     * <code>lines.dofs[lines.dof_offsets[line_index]]</code> will denote
-     * the finite element index of the set of DoF indices following; after
-     * this set, we will store the finite element index of the second set
-     * followed by the corresponding DoF indices; and so on. Finally, when
-     * all finite element indices adjacent to this object have been
-     * covered, we write a -1 to indicate the end of the list.
+     * Consequently, for objects that have a lower dimensionality than cells,
+     * we have to store a map from the finite element index to the set of DoF
+     * indices associated. Since real sets are typically very inefficient to
+     * store, and since most of the time we expect the number of individual
+     * keys to be small (frequently, adjacent cells will have the same finite
+     * element, and only a single entry will exist in the map), what we do is
+     * instead to store a linked list. In this format, the first entry
+     * starting at position
+     * <code>lines.dofs[lines.dof_offsets[line_index]]</code> will denote the
+     * finite element index of the set of DoF indices following; after this
+     * set, we will store the finite element index of the second set followed
+     * by the corresponding DoF indices; and so on. Finally, when all finite
+     * element indices adjacent to this object have been covered, we write a
+     * -1 to indicate the end of the list.
      *
-     * Access to this kind of data, as well as the distinction between
-     * cells and objects of lower dimensionality are encoded in the
-     * accessor functions, DoFObjects::set_dof_index() and
-     * DoFLevel::get_dof_index(). They are able to traverse this
-     * list and pick out or set a DoF index given the finite element index
-     * and its location within the set of DoFs corresponding to this
-     * finite element.
+     * Access to this kind of data, as well as the distinction between cells
+     * and objects of lower dimensionality are encoded in the accessor
+     * functions, DoFObjects::set_dof_index() and DoFLevel::get_dof_index().
+     * They are able to traverse this list and pick out or set a DoF index
+     * given the finite element index and its location within the set of DoFs
+     * corresponding to this finite element.
      *
      *
      * @ingroup hp
@@ -105,45 +102,30 @@ namespace internal
     {
     public:
       /**
-       * Store the start index for
-       * the degrees of freedom of each
-       * object in the @p dofs array.
+       * Store the start index for the degrees of freedom of each object in
+       * the @p dofs array.
        *
-       * The type we store is then obviously the type the @p dofs array
-       * uses for indexing.
+       * The type we store is then obviously the type the @p dofs array uses
+       * for indexing.
        */
       std::vector<unsigned int> dof_offsets;
 
       /**
-       * Store the global indices of
-       * the degrees of freedom. See
-       * DoFLevel() for detailed
-       * information.
+       * Store the global indices of the degrees of freedom. See DoFLevel()
+       * for detailed information.
        */
       std::vector<types::global_dof_index> dofs;
 
       /**
-       * Set the global index of
-       * the @p local_index-th
-       * degree of freedom located
-       * on the object with number @p
-       * obj_index to the value
-       * given by @p global_index. The @p
-       * dof_handler argument is
-       * used to access the finite
-       * element that is to be used
-       * to compute the location
-       * where this data is stored.
+       * Set the global index of the @p local_index-th degree of freedom
+       * located on the object with number @p obj_index to the value given by
+       * @p global_index. The @p dof_handler argument is used to access the
+       * finite element that is to be used to compute the location where this
+       * data is stored.
        *
-       * The third argument, @p
-       * fe_index, denotes which of
-       * the finite elements
-       * associated with this
-       * object we shall
-       * access. Refer to the
-       * general documentation of
-       * the internal::hp::DoFLevel
-       * class template for more
+       * The third argument, @p fe_index, denotes which of the finite elements
+       * associated with this object we shall access. Refer to the general
+       * documentation of the internal::hp::DoFLevel class template for more
        * information.
        */
       template <int dim, int spacedim>
@@ -156,26 +138,14 @@ namespace internal
                      const unsigned int               obj_level);
 
       /**
-       * Return the global index of
-       * the @p local_index-th
-       * degree of freedom located
-       * on the object with number @p
-       * obj_index. The @p
-       * dof_handler argument is
-       * used to access the finite
-       * element that is to be used
-       * to compute the location
-       * where this data is stored.
+       * Return the global index of the @p local_index-th degree of freedom
+       * located on the object with number @p obj_index. The @p dof_handler
+       * argument is used to access the finite element that is to be used to
+       * compute the location where this data is stored.
        *
-       * The third argument, @p
-       * fe_index, denotes which of
-       * the finite elements
-       * associated with this
-       * object we shall
-       * access. Refer to the
-       * general documentation of
-       * the internal::hp::DoFLevel
-       * class template for more
+       * The third argument, @p fe_index, denotes which of the finite elements
+       * associated with this object we shall access. Refer to the general
+       * documentation of the internal::hp::DoFLevel class template for more
        * information.
        */
       template <int dim, int spacedim>
@@ -187,27 +157,15 @@ namespace internal
                      const unsigned int               obj_level) const;
 
       /**
-       * Return the number of
-       * finite elements that are
-       * active on a given
-       * object. If this is a cell,
-       * the answer is of course
-       * one. If it is a face, the
-       * answer may be one or two,
-       * depending on whether the
-       * two adjacent cells use the
-       * same finite element or
-       * not. If it is an edge in
-       * 3d, the possible return
-       * value may be one or any
-       * other value larger than
-       * that.
+       * Return the number of finite elements that are active on a given
+       * object. If this is a cell, the answer is of course one. If it is a
+       * face, the answer may be one or two, depending on whether the two
+       * adjacent cells use the same finite element or not. If it is an edge
+       * in 3d, the possible return value may be one or any other value larger
+       * than that.
        *
-       * If the object is not part
-       * of an active cell, then no
-       * degrees of freedom have
-       * been distributed and zero
-       * is returned.
+       * If the object is not part of an active cell, then no degrees of
+       * freedom have been distributed and zero is returned.
        */
       template <int dim, int spacedim>
       unsigned int
@@ -215,9 +173,7 @@ namespace internal
                            const unsigned int               obj_index) const;
 
       /**
-       * Return the fe_index of the
-       * n-th active finite element
-       * on this object.
+       * Return the fe_index of the n-th active finite element on this object.
        */
       template <int dim, int spacedim>
       types::global_dof_index
@@ -227,9 +183,7 @@ namespace internal
                            const unsigned int               n) const;
 
       /**
-       * Check whether a given
-       * finite element index is
-       * used on the present
+       * Check whether a given finite element index is used on the present
        * object or not.
        */
       template <int dim, int spacedim>
@@ -240,35 +194,45 @@ namespace internal
                           const unsigned int               obj_level) const;
 
       /**
-       * Determine an estimate for the
-       * memory consumption (in bytes)
-       * of this object.
+       * Determine an estimate for the memory consumption (in bytes) of this
+       * object.
        */
       std::size_t memory_consumption () const;
+
+      /**
+       * Read or write the data of this object to or from a stream for the
+       * purpose of serialization
+       */
+      template <class Archive>
+      void serialize(Archive &ar,
+                     const unsigned int version);
     };
 
 
 
     /**
-     * These classes are similar to the internal::hp::DoFLevel classes. We here store
-     * information that is associated with faces, rather than cells, as this information is
-     * independent of the hierarchical structure of cells, which are organized in levels. In 2D
-     * we store information on degrees of freedom located on lines whereas in 3D we store
-     * information on drefrees of freedom located on quads and lines. In 1D we do nothing, as
-     * the faces of lines are vertices which are treated separately.
+     * These classes are similar to the internal::hp::DoFLevel classes. We
+     * here store information that is associated with faces, rather than
+     * cells, as this information is independent of the hierarchical structure
+     * of cells, which are organized in levels. In 2D we store information on
+     * degrees of freedom located on lines whereas in 3D we store information
+     * on degrees of freedom located on quads and lines. In 1D we do nothing,
+     * as the faces of lines are vertices which are treated separately.
      *
-     * Apart from the internal::hp::DoFObjects object containing the data to store
-     * (degree of freedom indices) and all the access-functionality to this data, we do not
-     * store any data or provide any functionality. However, we do implement a function to
-     * determine an estimate of the memory consumption of the contained
-     * internal::hp::DoFObjects object(s).
+     * Apart from the internal::hp::DoFObjects object containing the data to
+     * store (degree of freedom indices) and all the access-functionality to
+     * this data, we do not store any data or provide any functionality.
+     * However, we do implement a function to determine an estimate of the
+     * memory consumption of the contained internal::hp::DoFObjects object(s).
      *
-     * The data contained isn't usually directly accessed. Rather, except for some access from
-     * the DoFHandler class, access is usually through the DoFAccessor::set_dof_index() and
-     * DoFAccessor::dof_index() functions or similar functions of derived classes that in turn
-     * access the member variables using the DoFHandler::get_dof_index() and corresponding
-     * setter functions. Knowledge of the actual data format is therefore encapsulated to the
-     * present hierarchy of classes as well as the ::DoFHandler class.
+     * The data contained isn't usually directly accessed. Rather, except for
+     * some access from the DoFHandler class, access is usually through the
+     * DoFAccessor::set_dof_index() and DoFAccessor::dof_index() functions or
+     * similar functions of derived classes that in turn access the member
+     * variables using the DoFHandler::get_dof_index() and corresponding
+     * setter functions. Knowledge of the actual data format is therefore
+     * encapsulated to the present hierarchy of classes as well as the
+     * ::DoFHandler class.
      *
      * @ingroup dofs
      * @author Tobias Leicht, 2006
@@ -278,8 +242,8 @@ namespace internal
 
 
     /**
-     * Store the indices of degrees of freedom on faces in 1D. As these would be vertices, which
-     * are treated separately, don't do anything.
+     * Store the indices of degrees of freedom on faces in 1D. As these would
+     * be vertices, which are treated separately, don't do anything.
      *
      * @ingroup hp
      * @author Tobias Leicht, 2006
@@ -289,15 +253,23 @@ namespace internal
     {
     public:
       /**
-       * Determine an estimate for the
-       * memory consumption (in bytes)
-       * of this object.
+       * Determine an estimate for the memory consumption (in bytes) of this
+       * object.
        */
       std::size_t memory_consumption () const;
+
+      /**
+       * Read or write the data of this object to or from a stream for the
+       * purpose of serialization
+       */
+      template <class Archive>
+      void serialize(Archive &ar,
+                     const unsigned int version);
     };
 
     /**
-     * Store the indices of degrees of freedom on faces in 2D, which are lines.
+     * Store the indices of degrees of freedom on faces in 2D, which are
+     * lines.
      *
      * @ingroup hp
      * @author Tobias Leicht, 2006
@@ -312,11 +284,18 @@ namespace internal
       internal::hp::DoFIndicesOnFacesOrEdges<1> lines;
 
       /**
-       * Determine an estimate for the
-       * memory consumption (in bytes)
-       * of this object.
+       * Determine an estimate for the memory consumption (in bytes) of this
+       * object.
        */
       std::size_t memory_consumption () const;
+
+      /**
+       * Read or write the data of this object to or from a stream for the
+       * purpose of serialization
+       */
+      template <class Archive>
+      void serialize(Archive &ar,
+                     const unsigned int version);
     };
 
     /**
@@ -341,15 +320,42 @@ namespace internal
       internal::hp::DoFIndicesOnFacesOrEdges<2> quads;
 
       /**
-       * Determine an estimate for the
-       * memory consumption (in bytes)
-       * of this object.
+       * Determine an estimate for the memory consumption (in bytes) of this
+       * object.
        */
       std::size_t memory_consumption () const;
+
+      /**
+       * Read or write the data of this object to or from a stream for the
+       * purpose of serialization
+       */
+      template <class Archive>
+      void serialize(Archive &ar,
+                     const unsigned int version);
     };
 
 
     // --------------------- inline and template functions ------------------
+    template <class Archive>
+    void DoFIndicesOnFaces<1>::serialize(Archive &,
+                                         const unsigned int)
+    {}
+
+
+    template <class Archive>
+    void DoFIndicesOnFaces<2>::serialize(Archive &ar,
+                                         const unsigned int)
+    {
+      ar &lines;
+    }
+
+
+    template <class Archive>
+    void DoFIndicesOnFaces<3>::serialize(Archive &ar,
+                                         const unsigned int)
+    {
+      ar &lines &quads;
+    }
 
     template <int structdim>
     template <int dim, int spacedim>
@@ -360,16 +366,11 @@ namespace internal
                    const unsigned int                obj_index,
                    const unsigned int                fe_index,
                    const unsigned int                local_index,
-                   const unsigned int                obj_level) const
+                   const unsigned int                /*obj_level*/) const
     {
       Assert ((fe_index != dealii::hp::DoFHandler<dim,spacedim>::default_fe_index),
               ExcMessage ("You need to specify a FE index when working "
                           "with hp DoFHandlers"));
-      Assert (&dof_handler != 0,
-              ExcMessage ("No DoFHandler is specified for this iterator"));
-      Assert (&dof_handler.get_fe() != 0,
-              ExcMessage ("No finite element collection is associated with "
-                          "this DoFHandler"));
       Assert (fe_index < dof_handler.get_fe().size(),
               ExcIndexRange (fe_index, 0, dof_handler.get_fe().size()));
       Assert (local_index <
@@ -422,16 +423,11 @@ namespace internal
                    const unsigned int                fe_index,
                    const unsigned int                local_index,
                    const types::global_dof_index     global_index,
-                   const unsigned int                obj_level)
+                   const unsigned int                /*obj_level*/)
     {
       Assert ((fe_index != dealii::hp::DoFHandler<dim,spacedim>::default_fe_index),
               ExcMessage ("You need to specify a FE index when working "
                           "with hp DoFHandlers"));
-      Assert (&dof_handler != 0,
-              ExcMessage ("No DoFHandler is specified for this iterator"));
-      Assert (&dof_handler.get_fe() != 0,
-              ExcMessage ("No finite element collection is associated with "
-                          "this DoFHandler"));
       Assert (fe_index < dof_handler.get_fe().size(),
               ExcIndexRange (fe_index, 0, dof_handler.get_fe().size()));
       Assert (local_index <
@@ -484,11 +480,6 @@ namespace internal
     n_active_fe_indices (const dealii::hp::DoFHandler<dim,spacedim> &dof_handler,
                          const unsigned int                obj_index) const
     {
-      Assert (&dof_handler != 0,
-              ExcMessage ("No DoFHandler is specified for this iterator"));
-      Assert (&dof_handler.get_fe() != 0,
-              ExcMessage ("No finite element collection is associated with "
-                          "this DoFHandler"));
       Assert (obj_index < dof_offsets.size(),
               ExcIndexRange (obj_index, 0, dof_offsets.size()));
 
@@ -530,15 +521,10 @@ namespace internal
     types::global_dof_index
     DoFIndicesOnFacesOrEdges<structdim>::
     nth_active_fe_index (const dealii::hp::DoFHandler<dim,spacedim> &dof_handler,
-                         const unsigned int                obj_level,
+                         const unsigned int                /*obj_level*/,
                          const unsigned int                obj_index,
                          const unsigned int                n) const
     {
-      Assert (&dof_handler != 0,
-              ExcMessage ("No DoFHandler is specified for this iterator"));
-      Assert (&dof_handler.get_fe() != 0,
-              ExcMessage ("No finite element collection is associated with "
-                          "this DoFHandler"));
       Assert (obj_index < dof_offsets.size(),
               ExcIndexRange (obj_index, 0, dof_offsets.size()));
 
@@ -593,13 +579,8 @@ namespace internal
     fe_index_is_active (const dealii::hp::DoFHandler<dim,spacedim> &dof_handler,
                         const unsigned int                obj_index,
                         const unsigned int                fe_index,
-                        const unsigned int                obj_level) const
+                        const unsigned int                /*obj_level*/) const
     {
-      Assert (&dof_handler != 0,
-              ExcMessage ("No DoFHandler is specified for this iterator"));
-      Assert (&dof_handler.get_fe() != 0,
-              ExcMessage ("No finite element collection is associated with "
-                          "this DoFHandler"));
       Assert (obj_index < dof_offsets.size(),
               ExcIndexRange (obj_index, 0, static_cast<unsigned int>(dof_offsets.size())));
       Assert ((fe_index != dealii::hp::DoFHandler<dim,spacedim>::default_fe_index),
@@ -639,6 +620,15 @@ namespace internal
         }
     }
 
+    template <int structdim>
+    template <class Archive>
+    void DoFIndicesOnFacesOrEdges<structdim>::serialize(Archive &ar,
+                                                        const unsigned int)
+    {
+      ar &dofs;
+      ar &dof_offsets;
+    }
+
 
   } // namespace hp
 
