@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2005 - 2016 by the deal.II authors
+// Copyright (C) 2005 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -24,6 +24,7 @@
 #include <deal.II/base/smartpointer.h>
 #include <deal.II/base/iterator_range.h>
 #include <deal.II/dofs/function_map.h>
+#include <deal.II/dofs/dof_accessor.h>
 #include <deal.II/dofs/dof_iterator_selector.h>
 #include <deal.II/dofs/number_cache.h>
 #include <deal.II/hp/fe_collection.h>
@@ -36,6 +37,8 @@
 
 DEAL_II_NAMESPACE_OPEN
 
+template <int dim, int spacedim> class Triangulation;
+
 namespace internal
 {
   namespace hp
@@ -100,15 +103,18 @@ namespace hp
    * hp::DoFHandler::distribute_dofs(), and then assemble a linear system and
    * solve a problem on this finite element space. However, one can skip
    * assigning active FE indices upon mesh refinement in certain
-   * circumstances. In particular, the following rules apply: - Upon mesh
-   * refinement, child cells inherit the active FE index of the parent. - On
-   * the other hand, when coarsening cells, the (now active) parent cell will
-   * not have an active FE index set and you will have to set it explicitly
-   * before calling hp::DoFHandler::distribute_dofs(). In particular, to avoid
-   * stale information to be used by accident, this class deletes the active
-   * FE index of cells that are refined after inheriting this index to the
-   * children; this implies that if the children are coarsened away, the old
-   * value is no longer available on the parent cell.
+   * circumstances. In particular, the following rules apply:
+   * - Upon mesh refinement, child cells inherit the active FE index of
+   *   the parent.
+   * - On the other hand, when coarsening cells, the (now active)
+   *   parent cell will not have an active FE index set and you will
+   *   have to set it explicitly before calling
+   *   hp::DoFHandler::distribute_dofs(). In particular, to avoid
+   *   stale information to be used by accident, this class deletes
+   *   the active FE index of cells that are refined after inheriting
+   *   this index to the children; this implies that if the children
+   *   are coarsened away, the old value is no longer available on the
+   *   parent cell.
    *
    * @ingroup dofs
    * @ingroup hp
@@ -134,7 +140,7 @@ namespace hp
     typedef typename ActiveSelector::active_hex_iterator  active_hex_iterator;
 
     /**
-     * A typedef that is used to to identify
+     * A typedef that is used to identify
      * @ref GlossActive "active cell iterators".
      * The concept of iterators is discussed at length in the
      * @ref Iterators "iterators documentation module".
@@ -162,7 +168,7 @@ namespace hp
     typedef typename LevelSelector::cell_iterator         level_cell_iterator;
 
     /**
-     * A typedef that is used to to identify cell iterators. The concept of
+     * A typedef that is used to identify cell iterators. The concept of
      * iterators is discussed at length in the
      * @ref Iterators "iterators documentation module".
      *
@@ -202,11 +208,6 @@ namespace hp
 
     typedef typename LevelSelector::face_iterator         level_face_iterator;
 
-    /**
-     * Alias the @p FunctionMap type declared elsewhere.
-     */
-    typedef typename FunctionMap<spacedim>::type FunctionMap;
-
     /**
      * Make the dimension available in function templates.
      */
@@ -526,9 +527,12 @@ namespace hp
      * reason that a @p map rather than a @p set is used is the same as
      * described in the section on the @p make_boundary_sparsity_pattern
      * function.
+     *
+     * The type of @p boundary_ids equals typename FunctionMap<spacedim,number>::type.
      */
+    template <typename number>
     types::global_dof_index
-    n_boundary_dofs (const FunctionMap &boundary_ids) const;
+    n_boundary_dofs (const std::map<types::boundary_id, const Function<spacedim,number>*> &boundary_ids) const;
 
     /**
      * Same function, but with different data type of the argument, which is
@@ -561,7 +565,7 @@ namespace hp
 
 
     /**
-     * Returns a vector that stores the locally owned DoFs of each processor.
+     * Return a vector that stores the locally owned DoFs of each processor.
      * If you are only interested in the number of elements each processor
      * owns then n_dofs_per_processor() is a better choice.
      *
@@ -902,6 +906,25 @@ namespace hp
 
   /* ----------------------- Inline functions ---------------------------------- */
 
+
+  template <int dim, int spacedim>
+  template <typename number>
+  types::global_dof_index
+  DoFHandler<dim,spacedim>::n_boundary_dofs (const std::map<types::boundary_id, const Function<spacedim,number>*> &boundary_ids) const
+  {
+    // extract the set of boundary ids and forget about the function object pointers
+    std::set<types::boundary_id> boundary_ids_only;
+    for (typename std::map<types::boundary_id, const Function<spacedim,number>*>::const_iterator
+         p = boundary_ids.begin();
+         p != boundary_ids.end(); ++p)
+      boundary_ids_only.insert (p->first);
+
+    // then just hand everything over to the other function that does the work
+    return n_boundary_dofs(boundary_ids_only);
+  }
+
+
+
   template <int dim, int spacedim>
   template <class Archive>
   void DoFHandler<dim, spacedim>::save(Archive &ar, unsigned int) const
