@@ -1,7 +1,6 @@
 // ---------------------------------------------------------------------
-// $Id: fe_values.h 30364 2013-08-21 05:35:12Z kronbichler $
 //
-// Copyright (C) 2003 - 2013 by the deal.II authors
+// Copyright (C) 2003 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef __deal2__hp_fe_values_h
-#define __deal2__hp_fe_values_h
+#ifndef dealii__hp_fe_values_h
+#define dealii__hp_fe_values_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/fe/fe.h>
@@ -25,11 +24,10 @@
 #include <deal.II/fe/fe_values.h>
 
 #include <map>
-#include <deal.II/base/std_cxx1x/shared_ptr.h>
+#include <deal.II/base/std_cxx11/shared_ptr.h>
 
 DEAL_II_NAMESPACE_OPEN
 
-template <int dim, int spacedim> class MappingQ1;
 template <int dim, int spacedim> class FiniteElement;
 
 
@@ -42,24 +40,23 @@ namespace internal
      * Base class for the <tt>hp::FE*Values</tt> classes, storing the data
      * that is common to them. The main task of this class is to provide a
      * table where for every combination of finite element, mapping, and
-     * quadrature object from their corresponding collection objects there
-     * is a matching ::FEValues, ::FEFaceValues, or ::FESubfaceValues
-     * object. To make things more efficient, however, these FE*Values
-     * objects are only created once requested (lazy allocation).
+     * quadrature object from their corresponding collection objects there is
+     * a matching ::FEValues, ::FEFaceValues, or ::FESubfaceValues object. To
+     * make things more efficient, however, these FE*Values objects are only
+     * created once requested (lazy allocation).
      *
-     * The first template parameter denotes the space dimension we are in,
-     * the second the dimensionality of the object that we integrate on,
-     * i.e. for usual @p hp::FEValues it is equal to the first one, while
-     * for face integration it is one less. The third template parameter
-     * indicates the type of underlying non-hp FE*Values base type,
-     * i.e. it could either be dealii::FEValues, dealii::FEFaceValues, or
-     * dealii::FESubfaceValues.
+     * The first template parameter denotes the space dimension we are in, the
+     * second the dimensionality of the object that we integrate on, i.e. for
+     * usual @p hp::FEValues it is equal to the first one, while for face
+     * integration it is one less. The third template parameter indicates the
+     * type of underlying non-hp FE*Values base type, i.e. it could either be
+     * dealii::FEValues, dealii::FEFaceValues, or dealii::FESubfaceValues.
      *
      * @ingroup hp
      *
      * @author Wolfgang Bangerth, 2003
      */
-    template <int dim, int q_dim, class FEValues>
+    template <int dim, int q_dim, class FEValuesType>
     class FEValuesBase
     {
     public:
@@ -67,16 +64,16 @@ namespace internal
        * Constructor. Set the fields of this class to the values indicated by
        * the parameters to the constructor.
        */
-      FEValuesBase (const dealii::hp::MappingCollection<dim,FEValues::space_dimension> &mapping_collection,
-                    const dealii::hp::FECollection<dim,FEValues::space_dimension>      &fe_collection,
+      FEValuesBase (const dealii::hp::MappingCollection<dim,FEValuesType::space_dimension> &mapping_collection,
+                    const dealii::hp::FECollection<dim,FEValuesType::space_dimension>      &fe_collection,
                     const dealii::hp::QCollection<q_dim>     &q_collection,
-                    const UpdateFlags             update_flags);
+                    const dealii::UpdateFlags             update_flags);
       /**
-       * Constructor. Set the fields of this class to the values indicated by
-       * the parameters to the constructor, and choose a @p MappingQ1 object
-       * for the mapping object.
+       * Constructor. This constructor is equivalent to the other one except
+       * that it makes the object use a $Q_1$ mapping (i.e., an object of type
+       * MappingQGeneric(1)) implicitly.
        */
-      FEValuesBase (const dealii::hp::FECollection<dim,FEValues::space_dimension> &fe_collection,
+      FEValuesBase (const dealii::hp::FECollection<dim,FEValuesType::space_dimension> &fe_collection,
                     const dealii::hp::QCollection<q_dim> &q_collection,
                     const UpdateFlags         update_flags);
 
@@ -84,19 +81,17 @@ namespace internal
        * Get a reference to the collection of finite element objects used
        * here.
        */
-      const dealii::hp::FECollection<dim,FEValues::space_dimension> &
+      const dealii::hp::FECollection<dim,FEValuesType::space_dimension> &
       get_fe_collection () const;
 
       /**
-       * Get a reference to the collection of mapping objects used
-       * here.
+       * Get a reference to the collection of mapping objects used here.
        */
-      const dealii::hp::MappingCollection<dim,FEValues::space_dimension> &
+      const dealii::hp::MappingCollection<dim,FEValuesType::space_dimension> &
       get_mapping_collection () const;
 
       /**
-       * Get a reference to the collection of quadrature objects used
-       * here.
+       * Get a reference to the collection of quadrature objects used here.
        */
       const dealii::hp::QCollection<q_dim> &
       get_quadrature_collection () const;
@@ -112,7 +107,7 @@ namespace internal
        * you called the @p reinit function of the <tt>hp::FE*Values</tt> class
        * the last time.
        */
-      const FEValues &get_present_fe_values () const;
+      const FEValuesType &get_present_fe_values () const;
 
     protected:
 
@@ -123,7 +118,7 @@ namespace internal
        * The function returns a writable reference so that derived classes can
        * also reinit() the selected FEValues object.
        */
-      FEValues &
+      FEValuesType &
       select_fe_values (const unsigned int fe_index,
                         const unsigned int mapping_index,
                         const unsigned int q_index);
@@ -132,14 +127,14 @@ namespace internal
       /**
        * A pointer to the collection of finite elements to be used.
        */
-      const SmartPointer<const dealii::hp::FECollection<dim,FEValues::space_dimension>,
-            FEValuesBase<dim,q_dim,FEValues> > fe_collection;
+      const SmartPointer<const dealii::hp::FECollection<dim,FEValuesType::space_dimension>,
+            FEValuesBase<dim,q_dim,FEValuesType> > fe_collection;
 
       /**
        * A pointer to the collection of mappings to be used.
        */
-      const SmartPointer<const dealii::hp::MappingCollection<dim, FEValues::space_dimension>,
-            FEValuesBase<dim,q_dim,FEValues> > mapping_collection;
+      const SmartPointer<const dealii::hp::MappingCollection<dim, FEValuesType::space_dimension>,
+            FEValuesBase<dim,q_dim,FEValuesType> > mapping_collection;
 
       /**
        * Copy of the quadrature collection object provided to the constructor.
@@ -149,16 +144,16 @@ namespace internal
     private:
       /**
        * A table in which we store pointers to fe_values objects for different
-       * finite element, mapping, and quadrature objects from our
-       * collection. The first index indicates the index of the finite element
-       * within the fe_collection, the second the index of the mapping within
-       * the mapping collection, and the last one the index of the quadrature
-       * formula within the q_collection.
+       * finite element, mapping, and quadrature objects from our collection.
+       * The first index indicates the index of the finite element within the
+       * fe_collection, the second the index of the mapping within the mapping
+       * collection, and the last one the index of the quadrature formula
+       * within the q_collection.
        *
        * Initially, all entries have zero pointers, and we will allocate them
        * lazily as needed in select_fe_values().
        */
-      Table<3,std_cxx1x::shared_ptr<FEValues> > fe_values_table;
+      dealii::Table<3,std_cxx11::shared_ptr<FEValuesType> > fe_values_table;
 
       /**
        * Set of indices pointing at the fe_values object selected last time
@@ -181,44 +176,46 @@ namespace hp
 {
 
   /**
-   * An hp equivalent of the ::FEValues class. See the step-27
-   * tutorial program for examples of use.
+   * An hp equivalent of the ::FEValues class. See the step-27 tutorial
+   * program for examples of use.
    *
-   * The idea of this class is as follows: when one assembled matrices in the hp
-   * finite element method, there may be different finite elements on different
-   * cells, and consequently one may also want to use different quadrature
-   * formulas for different cells. On the other hand, the ::FEValues efficiently
-   * handles pre-evaluating whatever information is necessary for a single
-   * finite element and quadrature object. This class brings these concepts
-   * together: it provides a "collection" of ::FEValues objects.
+   * The idea of this class is as follows: when one assembled matrices in the
+   * hp finite element method, there may be different finite elements on
+   * different cells, and consequently one may also want to use different
+   * quadrature formulas for different cells. On the other hand, the
+   * ::FEValues efficiently handles pre-evaluating whatever information is
+   * necessary for a single finite element and quadrature object. This class
+   * brings these concepts together: it provides a "collection" of ::FEValues
+   * objects.
    *
-   * Upon construction, one passes not one finite element and quadrature object
-   * (and possible a mapping), but a whole collection of type hp::FECollection
-   * and hp::QCollection. Later on, when one sits on a concrete cell, one would
-   * call the reinit() function for this particular cell, just as one does for a
-   * regular ::FEValues object. The difference is that this time, the reinit()
-   * function looks up the active_fe_index of that cell, if necessary creates a
-   * ::FEValues object that matches the finite element and quadrature formulas
-   * with that particular index in their collections, and then re-initializes it
-   * for the current cell. The ::FEValues object that then fits the finite
-   * element and quadrature formula for the current cell can then be accessed
-   * using the get_present_fe_values() function, and one would work with it just
-   * like with any ::FEValues object for non-hp DoF handler objects.
+   * Upon construction, one passes not one finite element and quadrature
+   * object (and possible a mapping), but a whole collection of type
+   * hp::FECollection and hp::QCollection. Later on, when one sits on a
+   * concrete cell, one would call the reinit() function for this particular
+   * cell, just as one does for a regular ::FEValues object. The difference is
+   * that this time, the reinit() function looks up the active_fe_index of
+   * that cell, if necessary creates a ::FEValues object that matches the
+   * finite element and quadrature formulas with that particular index in
+   * their collections, and then re-initializes it for the current cell. The
+   * ::FEValues object that then fits the finite element and quadrature
+   * formula for the current cell can then be accessed using the
+   * get_present_fe_values() function, and one would work with it just like
+   * with any ::FEValues object for non-hp DoF handler objects.
    *
    * The reinit() functions have additional arguments with default values. If
    * not specified, the function takes the index into the hp::FECollection,
-   * hp::QCollection, and hp::MappingCollection objects from the active_fe_index
-   * of the cell, as explained above. However, one can also select different
-   * indices for a current cell. For example, by specifying a different index
-   * into the hp::QCollection class, one does not need to sort the quadrature
-   * objects in the quadrature collection so that they match one-to-one the
-   * order of finite element objects in the FE collection (even though choosing
-   * such an order is certainly convenient).
+   * hp::QCollection, and hp::MappingCollection objects from the
+   * active_fe_index of the cell, as explained above. However, one can also
+   * select different indices for a current cell. For example, by specifying a
+   * different index into the hp::QCollection class, one does not need to sort
+   * the quadrature objects in the quadrature collection so that they match
+   * one-to-one the order of finite element objects in the FE collection (even
+   * though choosing such an order is certainly convenient).
    *
-   * Note that ::FEValues objects are created on the fly, i.e. only as they are
-   * needed. This ensures that we do not create objects for every combination of
-   * finite element, quadrature formula and mapping, but only those that will
-   * actually be needed.
+   * Note that ::FEValues objects are created on the fly, i.e. only as they
+   * are needed. This ensures that we do not create objects for every
+   * combination of finite element, quadrature formula and mapping, but only
+   * those that will actually be needed.
    *
    * This class has not yet been implemented for the use in the codimension
    * one case (<tt>spacedim != dim </tt>).
@@ -242,7 +239,7 @@ namespace hp
      * the signature of this function to make it compatible with the signature
      * of the respective constructor of the usual FEValues object, with the
      * respective parameter in that function also being the return value of
-     * the <tt>DoFHandler::get_fe()</tt> function.
+     * the DoFHandler::get_fe() function.
      */
     FEValues (const dealii::hp::MappingCollection<dim,spacedim> &mapping_collection,
               const dealii::hp::FECollection<dim,spacedim>  &fe_collection,
@@ -251,14 +248,15 @@ namespace hp
 
 
     /**
-     * Constructor. Initialize this object with the given parameters, and
-     * choose a @p MappingQ1 object for the mapping object.
+     * Constructor. This constructor is equivalent to the other one except
+     * that it makes the object use a $Q_1$ mapping (i.e., an object of type
+     * MappingQGeneric(1)) implicitly.
      *
      * The finite element collection parameter is actually ignored, but is in
      * the signature of this function to make it compatible with the signature
      * of the respective constructor of the usual FEValues object, with the
      * respective parameter in that function also being the return value of
-     * the <tt>DoFHandler::get_fe()</tt> function.
+     * the DoFHandler::get_fe() function.
      */
     FEValues (const hp::FECollection<dim,spacedim> &fe_collection,
               const hp::QCollection<dim>      &q_collection,
@@ -269,12 +267,12 @@ namespace hp
      * Reinitialize the object for the given cell.
      *
      * After the call, you can get an FEValues object using the
-     * get_present_fe_values() function that corresponds to the present
-     * cell. For this FEValues object, we use the additional arguments
-     * described below to determine which finite element, mapping, and
-     * quadrature formula to use. They are order in such a way that the
-     * arguments one may want to change most frequently come first. The rules
-     * for these arguments are as follows:
+     * get_present_fe_values() function that corresponds to the present cell.
+     * For this FEValues object, we use the additional arguments described
+     * below to determine which finite element, mapping, and quadrature
+     * formula to use. They are order in such a way that the arguments one may
+     * want to change most frequently come first. The rules for these
+     * arguments are as follows:
      *
      * If the @p fe_index argument to this function is left at its default
      * value, then we use that finite element within the hp::FECollection
@@ -306,17 +304,16 @@ namespace hp
      * constructor of this class with index given by
      * <code>cell-@>active_fe_index()</code>, i.e. the same index as that of
      * the finite element. As above, if the mapping collection contains only a
-     * single element (a frequent case if one wants to use a MappingQ1 object
-     * for all finite elements in an hp discretization), then this single
-     * mapping is used unless a different value for this argument is
-     * specified.
+     * single element (a frequent case if one wants to use a $Q_1$ mapping for
+     * all finite elements in an hp discretization), then this single mapping
+     * is used unless a different value for this argument is specified.
      */
-    template <class DH, bool lda>
+    template <typename DoFHandlerType, bool lda>
     void
-    reinit (const TriaIterator<DoFCellAccessor<DH,lda> > cell,
-            const unsigned int q_index = numbers::invalid_unsigned_int,
+    reinit (const TriaIterator<DoFCellAccessor<DoFHandlerType,lda> > cell,
+            const unsigned int q_index       = numbers::invalid_unsigned_int,
             const unsigned int mapping_index = numbers::invalid_unsigned_int,
-            const unsigned int fe_index = numbers::invalid_unsigned_int);
+            const unsigned int fe_index      = numbers::invalid_unsigned_int);
 
     /**
      * Like the previous function, but for non-hp iterators. The reason this
@@ -334,9 +331,9 @@ namespace hp
      */
     void
     reinit (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
-            const unsigned int q_index = numbers::invalid_unsigned_int,
+            const unsigned int q_index       = numbers::invalid_unsigned_int,
             const unsigned int mapping_index = numbers::invalid_unsigned_int,
-            const unsigned int fe_index = numbers::invalid_unsigned_int);
+            const unsigned int fe_index      = numbers::invalid_unsigned_int);
 
 
   };
@@ -344,8 +341,9 @@ namespace hp
 
 
   /**
-   * This is the equivalent of the hp::FEValues class but for face integrations,
-   * i.e. it is to hp::FEValues what ::FEFaceValues is to ::FEValues.
+   * This is the equivalent of the hp::FEValues class but for face
+   * integrations, i.e. it is to hp::FEValues what ::FEFaceValues is to
+   * ::FEValues.
    *
    * The same comments apply as in the documentation of the hp::FEValues
    * class. However, it is important to note that it is here more common that
@@ -386,8 +384,9 @@ namespace hp
 
 
     /**
-     * Constructor. Initialize this object with the given parameters, and
-     * choose a @p MappingQ1 object for the mapping object.
+     * Constructor. This constructor is equivalent to the other one except
+     * that it makes the object use a $Q_1$ mapping (i.e., an object of type
+     * MappingQGeneric(1)) implicitly.
      *
      * The finite element collection parameter is actually ignored, but is in
      * the signature of this function to make it compatible with the signature
@@ -403,12 +402,12 @@ namespace hp
      * Reinitialize the object for the given cell and face.
      *
      * After the call, you can get an FEFaceValues object using the
-     * get_present_fe_values() function that corresponds to the present
-     * cell. For this FEFaceValues object, we use the additional arguments
-     * described below to determine which finite element, mapping, and
-     * quadrature formula to use. They are order in such a way that the
-     * arguments one may want to change most frequently come first. The rules
-     * for these arguments are as follows:
+     * get_present_fe_values() function that corresponds to the present cell.
+     * For this FEFaceValues object, we use the additional arguments described
+     * below to determine which finite element, mapping, and quadrature
+     * formula to use. They are order in such a way that the arguments one may
+     * want to change most frequently come first. The rules for these
+     * arguments are as follows:
      *
      * If the @p fe_index argument to this function is left at its default
      * value, then we use that finite element within the hp::FECollection
@@ -440,18 +439,17 @@ namespace hp
      * constructor of this class with index given by
      * <code>cell-@>active_fe_index()</code>, i.e. the same index as that of
      * the finite element. As above, if the mapping collection contains only a
-     * single element (a frequent case if one wants to use a MappingQ1 object
-     * for all finite elements in an hp discretization), then this single
-     * mapping is used unless a different value for this argument is
-     * specified.
+     * single element (a frequent case if one wants to use a $Q_1$ mapping for
+     * all finite elements in an hp discretization), then this single mapping
+     * is used unless a different value for this argument is specified.
      */
-    template <class DH, bool lda>
+    template <typename DoFHandlerType, bool lda>
     void
-    reinit (const TriaIterator<DoFCellAccessor<DH,lda> > cell,
+    reinit (const TriaIterator<DoFCellAccessor<DoFHandlerType,lda> > cell,
             const unsigned int face_no,
-            const unsigned int q_index = numbers::invalid_unsigned_int,
+            const unsigned int q_index       = numbers::invalid_unsigned_int,
             const unsigned int mapping_index = numbers::invalid_unsigned_int,
-            const unsigned int fe_index = numbers::invalid_unsigned_int);
+            const unsigned int fe_index      = numbers::invalid_unsigned_int);
 
     /**
      * Like the previous function, but for non-hp iterators. The reason this
@@ -470,16 +468,16 @@ namespace hp
     void
     reinit (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
             const unsigned int face_no,
-            const unsigned int q_index = numbers::invalid_unsigned_int,
+            const unsigned int q_index       = numbers::invalid_unsigned_int,
             const unsigned int mapping_index = numbers::invalid_unsigned_int,
-            const unsigned int fe_index = numbers::invalid_unsigned_int);
+            const unsigned int fe_index      = numbers::invalid_unsigned_int);
   };
 
 
 
   /**
-   * This class implements for subfaces what hp::FEFaceValues does for
-   * faces. See there for further documentation.
+   * This class implements for subfaces what hp::FEFaceValues does for faces.
+   * See there for further documentation.
    *
    * @ingroup hp hpcollection
    * @author Wolfgang Bangerth, 2003
@@ -504,8 +502,9 @@ namespace hp
 
 
     /**
-     * Constructor. Initialize this object with the given parameters, and
-     * choose a @p MappingQ1 object for the mapping object.
+     * Constructor. This constructor is equivalent to the other one except
+     * that it makes the object use a $Q_1$ mapping (i.e., an object of type
+     * MappingQGeneric(1)) implicitly.
      *
      * The finite element collection parameter is actually ignored, but is in
      * the signature of this function to make it compatible with the signature
@@ -521,8 +520,8 @@ namespace hp
      * Reinitialize the object for the given cell, face, and subface.
      *
      * After the call, you can get an FESubfaceValues object using the
-     * get_present_fe_values() function that corresponds to the present
-     * cell. For this FESubfaceValues object, we use the additional arguments
+     * get_present_fe_values() function that corresponds to the present cell.
+     * For this FESubfaceValues object, we use the additional arguments
      * described below to determine which finite element, mapping, and
      * quadrature formula to use. They are order in such a way that the
      * arguments one may want to change most frequently come first. The rules
@@ -548,19 +547,18 @@ namespace hp
      * constructor of this class with index given by
      * <code>cell-@>active_fe_index()</code>, i.e. the same index as that of
      * the finite element. As above, if the mapping collection contains only a
-     * single element (a frequent case if one wants to use a MappingQ1 object
-     * for all finite elements in an hp discretization), then this single
-     * mapping is used unless a different value for this argument is
-     * specified.
+     * single element (a frequent case if one wants to use a $Q_1$ mapping for
+     * all finite elements in an hp discretization), then this single mapping
+     * is used unless a different value for this argument is specified.
      */
-    template <class DH, bool lda>
+    template <typename DoFHandlerType, bool lda>
     void
-    reinit (const TriaIterator<DoFCellAccessor<DH,lda> > cell,
+    reinit (const TriaIterator<DoFCellAccessor<DoFHandlerType,lda> > cell,
             const unsigned int face_no,
             const unsigned int subface_no,
-            const unsigned int q_index = numbers::invalid_unsigned_int,
+            const unsigned int q_index       = numbers::invalid_unsigned_int,
             const unsigned int mapping_index = numbers::invalid_unsigned_int,
-            const unsigned int fe_index = numbers::invalid_unsigned_int);
+            const unsigned int fe_index      = numbers::invalid_unsigned_int);
 
     /**
      * Like the previous function, but for non-hp iterators. The reason this
@@ -580,9 +578,9 @@ namespace hp
     reinit (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
             const unsigned int face_no,
             const unsigned int subface_no,
-            const unsigned int q_index = numbers::invalid_unsigned_int,
+            const unsigned int q_index       = numbers::invalid_unsigned_int,
             const unsigned int mapping_index = numbers::invalid_unsigned_int,
-            const unsigned int fe_index = numbers::invalid_unsigned_int);
+            const unsigned int fe_index      = numbers::invalid_unsigned_int);
   };
 
 }
@@ -594,50 +592,50 @@ namespace internal
 {
   namespace hp
   {
-    template <int dim, int q_dim, class FEValues>
+    template <int dim, int q_dim, class FEValuesType>
     inline
-    const FEValues &
-    FEValuesBase<dim,q_dim,FEValues>::get_present_fe_values () const
+    const FEValuesType &
+    FEValuesBase<dim,q_dim,FEValuesType>::get_present_fe_values () const
     {
       return *fe_values_table(present_fe_values_index);
     }
 
 
 
-    template <int dim, int q_dim, class FEValues>
+    template <int dim, int q_dim, class FEValuesType>
     inline
-    const dealii::hp::FECollection<dim,FEValues::space_dimension> &
-    FEValuesBase<dim,q_dim,FEValues>::get_fe_collection () const
+    const dealii::hp::FECollection<dim,FEValuesType::space_dimension> &
+    FEValuesBase<dim,q_dim,FEValuesType>::get_fe_collection () const
     {
       return *fe_collection;
     }
 
 
 
-    template <int dim, int q_dim, class FEValues>
+    template <int dim, int q_dim, class FEValuesType>
     inline
-    const dealii::hp::MappingCollection<dim,FEValues::space_dimension> &
-    FEValuesBase<dim,q_dim,FEValues>::get_mapping_collection () const
+    const dealii::hp::MappingCollection<dim,FEValuesType::space_dimension> &
+    FEValuesBase<dim,q_dim,FEValuesType>::get_mapping_collection () const
     {
       return *mapping_collection;
     }
 
 
 
-    template <int dim, int q_dim, class FEValues>
+    template <int dim, int q_dim, class FEValuesType>
     inline
     const dealii::hp::QCollection<q_dim> &
-    FEValuesBase<dim,q_dim,FEValues>::get_quadrature_collection () const
+    FEValuesBase<dim,q_dim,FEValuesType>::get_quadrature_collection () const
     {
       return q_collection;
     }
 
 
 
-    template <int dim, int q_dim, class FEValues>
+    template <int dim, int q_dim, class FEValuesType>
     inline
     dealii::UpdateFlags
-    FEValuesBase<dim,q_dim,FEValues>::get_update_flags () const
+    FEValuesBase<dim,q_dim,FEValuesType>::get_update_flags () const
     {
       return update_flags;
     }
