@@ -1,7 +1,6 @@
 // ---------------------------------------------------------------------
-// $Id: patches.h 30036 2013-07-18 16:55:32Z maier $
 //
-// Copyright (C) 2010 - 2013 by the deal.II authors
+// Copyright (C) 2010 - 2015 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef __deal2__integrators_patches_h
-#define __deal2__integrators_patches_h
+#ifndef dealii__integrators_patches_h
+#define dealii__integrators_patches_h
 
 
 #include <deal.II/base/config.h>
