@@ -29,26 +29,29 @@ DEAL_II_NAMESPACE_OPEN
 
 
 /**
- * @brief Auxiliary class aiding in the handling of block structures like in
- * BlockVector or FESystem.
+ * BlockIndices represents a range of indices (such as the range $[0,N)$
+ * of valid indices for elements of a vector) and how this one range
+ * is broken down into smaller but contiguous "blocks" (such as the velocity
+ * and pressure parts of a solution vector). In particular, it provides the
+ * ability to translate between global indices and the indices <i>within</i>
+ * a block. This class is used, for example, in the BlockVector,
+ * BlockSparsityPattern, and BlockMatrixBase classes.
  *
- * The information obtained from this class falls into two groups. First, it
- * is possible to obtain the number of blocks, namely size(), the block_size()
- * for each block and the total_size() of the object described by the block
- * indices, namely the length of the whole index set. These functions do not
- * make any assumption on the ordering of the index set.
+ * The information that can be obtained from this class falls into two groups.
+ * First, it is possible to query the global size of the index space (through
+ * the total_size() member function), and the number of blocks and their sizes
+ * (via size() and the block_size() functions).
  *
- * If on the other hand the index set is ordered "by blocks", such that each
- * block forms a consecutive set of indices, this class that manages the
- * conversion of global indices into a block vector or matrix to the local
- * indices within this block. This is required, for example, when you address
- * a global element in a block vector and want to know which element within
- * which block this is. It is also useful if a matrix is composed of several
+ * Secondly, this class manages the conversion of global indices to the
+ * local indices within this block, and the other way around. This is required,
+ * for example, when you address a global element in a block vector and want to
+ * know within which block this is, and which index within this block it
+ * corresponds to. It is also useful if a matrix is composed of several
  * blocks, where you have to translate global row and column indices to local
  * ones.
  *
- * @ingroup data @see
- * @ref GlossBlockLA "Block (linear algebra)"
+ * @ingroup data
+ * @see @ref GlossBlockLA "Block (linear algebra)"
  * @author Wolfgang Bangerth, Guido Kanschat, 2000, 2007, 2011
  */
 class BlockIndices : public Subscriptor
@@ -71,6 +74,22 @@ public:
    */
   BlockIndices (const std::vector<size_type> &block_sizes);
 
+#ifdef DEAL_II_WITH_CXX11
+  /**
+   * Move constructor. Initialize a new object by stealing the internal data of
+   * another BlockIndices object.
+   *
+   * @note This constructor is only available if deal.II is configured with
+   * C++11 support.
+   */
+  BlockIndices (BlockIndices &&b);
+
+  /**
+   * Copy constructor.
+   */
+  BlockIndices (const BlockIndices &) = default;
+#endif
+
   /**
    * Specialized constructor for a structure with blocks of equal size.
    */
@@ -121,7 +140,7 @@ public:
   /**
    * String representation of the block sizes. The output is of the form
    * `[nb->b1,b2,b3|s]`, where `nb` is n_blocks(), `s` is total_size() and
-   * `b1` etc. are the values of block_size().
+   * `b1` etc. are the values returned by block_size() for each of the blocks.
    */
   std::string to_string () const;
 
@@ -162,6 +181,14 @@ public:
    */
   BlockIndices &operator = (const BlockIndices &b);
 
+#ifdef DEAL_II_WITH_CXX11
+  /**
+   * Move assignment operator. Move another BlockIndices object onto the
+   * current one by transferring its contents.
+   */
+  BlockIndices &operator = (BlockIndices &&);
+#endif
+
   /**
    * Compare whether two objects are the same, i.e. whether the number of
    * blocks and the sizes of all blocks are equal.
@@ -282,6 +309,23 @@ BlockIndices::BlockIndices (const std::vector<size_type> &block_sizes)
 }
 
 
+
+#ifdef DEAL_II_WITH_CXX11
+
+inline
+BlockIndices::BlockIndices (BlockIndices &&b)
+  :
+  n_blocks(b.n_blocks),
+  start_indices(std::move(b.start_indices))
+{
+  b.n_blocks = 0;
+  b.start_indices = std::vector<size_type>(1, 0);
+}
+
+#endif
+
+
+
 inline
 void
 BlockIndices::push_back(const size_type sz)
@@ -388,6 +432,23 @@ BlockIndices::operator = (const BlockIndices &b)
 
 
 
+#ifdef DEAL_II_WITH_CXX11
+inline
+BlockIndices &
+BlockIndices::operator = (BlockIndices &&b)
+{
+  start_indices = std::move(b.start_indices);
+  n_blocks = b.n_blocks;
+
+  b.start_indices = std::vector<size_type>(1, 0);
+  b.n_blocks = 0;
+
+  return *this;
+}
+#endif
+
+
+
 inline
 bool
 BlockIndices::operator == (const BlockIndices &b) const
