@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2010 - 2016 by the deal.II authors
+// Copyright (C) 2010 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -27,35 +27,49 @@ DEAL_II_NAMESPACE_OPEN
 
 // Forward declarations:
 
+namespace internal
+{
+  namespace BlockLinearOperator
+  {
+    template<typename PayloadBlockType = internal::LinearOperator::EmptyPayload>
+    class EmptyBlockPayload;
+  }
+}
+
 template <typename Number> class BlockVector;
 
 template <typename Range = BlockVector<double>,
-          typename Domain = Range>
+          typename Domain = Range,
+          typename BlockPayload = internal::BlockLinearOperator::EmptyBlockPayload<> >
 class BlockLinearOperator;
 
 template <typename Range = BlockVector<double>,
           typename Domain = Range,
+          typename BlockPayload = internal::BlockLinearOperator::EmptyBlockPayload<>,
           typename BlockMatrixType>
-BlockLinearOperator<Range, Domain>
+BlockLinearOperator<Range, Domain, BlockPayload>
 block_operator(const BlockMatrixType &matrix);
 
 template <size_t m, size_t n,
           typename Range = BlockVector<double>,
-          typename Domain = Range>
-BlockLinearOperator<Range, Domain>
-block_operator(const std::array<std::array<LinearOperator<typename Range::BlockType, typename Domain::BlockType>, n>, m> &);
+          typename Domain = Range,
+          typename BlockPayload = internal::BlockLinearOperator::EmptyBlockPayload<> >
+BlockLinearOperator<Range, Domain, BlockPayload>
+block_operator(const std::array<std::array<LinearOperator<typename Range::BlockType, typename Domain::BlockType, typename BlockPayload::BlockType>, n>, m> &);
 
 template <size_t m,
           typename Range = BlockVector<double>,
-          typename Domain = Range>
-BlockLinearOperator<Range, Domain>
-block_diagonal_operator(const std::array<LinearOperator<typename Range::BlockType, typename Domain::BlockType>, m> &);
+          typename Domain = Range,
+          typename BlockPayload = internal::BlockLinearOperator::EmptyBlockPayload<> >
+BlockLinearOperator<Range, Domain, BlockPayload>
+block_diagonal_operator(const std::array<LinearOperator<typename Range::BlockType, typename Domain::BlockType, typename BlockPayload::BlockType>, m> &);
 
 template <size_t m,
           typename Range = BlockVector<double>,
-          typename Domain = Range>
-BlockLinearOperator<Range, Domain>
-block_diagonal_operator(const LinearOperator<typename Range::BlockType, typename Domain::BlockType> &op);
+          typename Domain = Range,
+          typename BlockPayload = internal::BlockLinearOperator::EmptyBlockPayload<> >
+BlockLinearOperator<Range, Domain, BlockPayload>
+block_diagonal_operator(const LinearOperator<typename Range::BlockType, typename Domain::BlockType, typename BlockPayload::BlockType> &op);
 
 // This is a workaround for a bug in <=gcc-4.7 that does not like partial
 // template default values in combination with local lambda expressions [1]
@@ -66,21 +80,24 @@ block_diagonal_operator(const LinearOperator<typename Range::BlockType, typename
 
 template <typename Range = BlockVector<double>,
           typename Domain = Range,
+          typename BlockPayload = internal::BlockLinearOperator::EmptyBlockPayload<>,
           typename BlockMatrixType>
-BlockLinearOperator<Range, Domain>
+BlockLinearOperator<Range, Domain, BlockPayload>
 block_diagonal_operator(const BlockMatrixType &block_matrix);
 
 template <typename Range = BlockVector<double>,
-          typename Domain = Range>
-LinearOperator<Domain, Range>
-block_forward_substitution(const BlockLinearOperator<Range, Domain> &,
-                           const BlockLinearOperator<Domain, Range> &);
+          typename Domain = Range,
+          typename BlockPayload = internal::BlockLinearOperator::EmptyBlockPayload<> >
+LinearOperator<Domain, Range, typename BlockPayload::BlockType>
+block_forward_substitution(const BlockLinearOperator<Range, Domain, BlockPayload> &,
+                           const BlockLinearOperator<Domain, Range, BlockPayload> &);
 
 template <typename Range = BlockVector<double>,
-          typename Domain = Range>
-LinearOperator<Domain, Range>
-block_back_substitution(const BlockLinearOperator<Range, Domain> &,
-                        const BlockLinearOperator<Domain, Range> &);
+          typename Domain = Range,
+          typename BlockPayload = internal::BlockLinearOperator::EmptyBlockPayload<> >
+LinearOperator<Domain, Range, typename BlockPayload::BlockType>
+block_back_substitution(const BlockLinearOperator<Range, Domain, BlockPayload> &,
+                        const BlockLinearOperator<Domain, Range, BlockPayload> &);
 
 // end of workaround
 
@@ -125,12 +142,12 @@ block_back_substitution(const BlockLinearOperator<Range, Domain> &,
  *
  * @ingroup LAOperators
  */
-template <typename Range, typename Domain>
-class BlockLinearOperator : public LinearOperator<Range, Domain>
+template <typename Range, typename Domain, typename BlockPayload>
+class BlockLinearOperator : public LinearOperator<Range, Domain, typename BlockPayload::BlockType>
 {
 public:
 
-  typedef LinearOperator<typename Range::BlockType, typename Domain::BlockType> BlockType;
+  typedef LinearOperator<typename Range::BlockType, typename Domain::BlockType, typename BlockPayload::BlockType> BlockType;
 
   /**
    * Create an empty BlockLinearOperator object.
@@ -139,10 +156,9 @@ public:
    * class LinearOperator are initialized with default variants that throw an
    * exception upon invocation.
    */
-  BlockLinearOperator()
-    : LinearOperator<Range, Domain>()
+  BlockLinearOperator(const BlockPayload &payload)
+    : LinearOperator<Range, Domain, typename BlockPayload::BlockType>(typename BlockPayload::BlockType(payload,payload))
   {
-
     n_block_rows = []() -> unsigned int
     {
       Assert(false, ExcMessage("Uninitialized BlockLinearOperator<Range, Domain>::n_block_rows called"));
@@ -165,7 +181,7 @@ public:
   /**
    * Default copy constructor.
    */
-  BlockLinearOperator(const BlockLinearOperator<Range, Domain> &) =
+  BlockLinearOperator(const BlockLinearOperator<Range, Domain, BlockPayload> &) =
     default;
 
   /**
@@ -176,7 +192,7 @@ public:
   template<typename Op>
   BlockLinearOperator(const Op &op)
   {
-    *this = block_operator<Range, Domain, Op>(op);
+    *this = block_operator<Range, Domain, BlockPayload, Op>(op);
   }
 
   /**
@@ -187,7 +203,7 @@ public:
   template<size_t m, size_t n>
   BlockLinearOperator(const std::array<std::array<BlockType, n>, m> &ops)
   {
-    *this = block_operator<m, n, Range, Domain>(ops);
+    *this = block_operator<m, n, Range, Domain, BlockPayload>(ops);
   }
 
   /**
@@ -198,23 +214,23 @@ public:
   template<size_t m>
   BlockLinearOperator(const std::array<BlockType, m> &ops)
   {
-    *this = block_diagonal_operator<m, Range, Domain>(ops);
+    *this = block_diagonal_operator<m, Range, Domain, BlockPayload>(ops);
   }
 
   /**
    * Default copy assignment operator.
    */
-  BlockLinearOperator<Range, Domain> &
-  operator=(const BlockLinearOperator<Range, Domain> &) = default;
+  BlockLinearOperator<Range, Domain, BlockPayload> &
+  operator=(const BlockLinearOperator<Range, Domain, BlockPayload> &) = default;
 
   /**
    * Templated copy assignment operator for an object @p op for which the
    * conversion function <code>block_operator</code> is defined.
    */
   template <typename Op>
-  BlockLinearOperator<Range, Domain> &operator=(const Op &op)
+  BlockLinearOperator<Range, Domain, BlockPayload> &operator=(const Op &op)
   {
-    *this = block_operator<Range, Domain, Op>(op);
+    *this = block_operator<Range, Domain, BlockPayload, Op>(op);
     return *this;
   }
 
@@ -224,10 +240,10 @@ public:
    * specialization.
    */
   template <size_t m, size_t n>
-  BlockLinearOperator<Range, Domain> &
+  BlockLinearOperator<Range, Domain, BlockPayload> &
   operator=(const std::array<std::array<BlockType, n>, m> &ops)
   {
-    *this = block_operator<m, n, Range, Domain>(ops);
+    *this = block_operator<m, n, Range, Domain, BlockPayload>(ops);
     return *this;
   }
 
@@ -237,10 +253,10 @@ public:
    * operator calls the corresponding block_operator() specialization.
    */
   template <size_t m>
-  BlockLinearOperator<Range, Domain> &
+  BlockLinearOperator<Range, Domain, BlockPayload> &
   operator=(const std::array<BlockType, m> &ops)
   {
-    *this = block_diagonal_operator<m, Range, Domain>(ops);
+    *this = block_diagonal_operator<m, Range, Domain, BlockPayload>(ops);
     return *this;
   }
 
@@ -272,10 +288,10 @@ namespace internal
   {
     // Populate the LinearOperator interfaces with the help of the
     // BlockLinearOperator functions
-    template <typename Range, typename Domain>
+    template <typename Range, typename Domain, typename BlockPayload>
     inline void
     populate_linear_operator_functions(
-      dealii::BlockLinearOperator<Range, Domain> &op)
+      dealii::BlockLinearOperator<Range, Domain, BlockPayload> &op)
     {
       op.reinit_range_vector = [=](Range &v, bool omit_zeroing_entries)
       {
@@ -359,6 +375,43 @@ namespace internal
             op.block(j, i).Tvmult_add(v.block(i), u.block(j));
       };
     }
+
+
+    /**
+     * A dummy class for BlockLinearOperators that do not require any
+     * extensions to facilitate the operations of the block matrix or its
+     * subblocks.
+     *
+     * This is the Payload class typically associated with deal.II's native
+     * BlockSparseMatrix. To use Trilinos and PETSc BlockSparseMatrices it is
+     * necessary to initialize a BlockLinearOperator with their associated
+     * BlockPayload.
+     *
+     * @author Jean-Paul Pelteret, Matthias Maier, 2016
+     *
+     * @ingroup LAOperators
+     */
+    template<typename PayloadBlockType>
+    class EmptyBlockPayload
+    {
+    public:
+      /**
+       * Type of payload held by each subblock
+       */
+      typedef PayloadBlockType BlockType;
+
+      /**
+       * Default constructor
+       *
+       * Since this class does not do anything in particular and needs no
+       * special configuration, we have only one generic constructor that can
+       * be called under any conditions.
+       */
+      template <typename... Args>
+      EmptyBlockPayload (const Args &...)
+      { }
+    };
+
   } /*namespace BlockLinearOperator*/
 } /*namespace internal*/
 
@@ -382,13 +435,14 @@ namespace internal
  */
 template <typename Range,
           typename Domain,
+          typename BlockPayload,
           typename BlockMatrixType>
-BlockLinearOperator<Range, Domain>
+BlockLinearOperator<Range, Domain, BlockPayload>
 block_operator(const BlockMatrixType &block_matrix)
 {
-  typedef typename BlockLinearOperator<Range, Domain>::BlockType BlockType;
+  typedef typename BlockLinearOperator<Range, Domain, BlockPayload>::BlockType BlockType;
 
-  BlockLinearOperator<Range, Domain> return_op;
+  BlockLinearOperator<Range, Domain, BlockPayload> return_op (BlockPayload(block_matrix,block_matrix));
 
   return_op.n_block_rows = [&block_matrix]() -> unsigned int
   {
@@ -445,16 +499,16 @@ block_operator(const BlockMatrixType &block_matrix)
  *
  * @ingroup LAOperators
  */
-template <size_t m, size_t n, typename Range, typename Domain>
-BlockLinearOperator<Range, Domain>
-block_operator(const std::array<std::array<LinearOperator<typename Range::BlockType, typename Domain::BlockType>, n>, m> &ops)
+template <size_t m, size_t n, typename Range, typename Domain, typename BlockPayload>
+BlockLinearOperator<Range, Domain, BlockPayload>
+block_operator(const std::array<std::array<LinearOperator<typename Range::BlockType, typename Domain::BlockType, typename BlockPayload::BlockType>, n>, m> &ops)
 {
   static_assert(m > 0 && n > 0,
                 "a blocked LinearOperator must consist of at least one block");
 
-  typedef typename BlockLinearOperator<Range, Domain>::BlockType BlockType;
+  typedef typename BlockLinearOperator<Range, Domain, BlockPayload>::BlockType BlockType;
 
-  BlockLinearOperator<Range, Domain> return_op;
+  BlockLinearOperator<Range, Domain, BlockPayload> return_op ((BlockPayload())); // TODO: Create block payload so that this can be initialised correctly
 
   return_op.n_block_rows = []() -> unsigned int
   {
@@ -497,13 +551,14 @@ block_operator(const std::array<std::array<LinearOperator<typename Range::BlockT
  */
 template <typename Range,
           typename Domain,
+          typename BlockPayload,
           typename BlockMatrixType>
-BlockLinearOperator<Range, Domain>
+BlockLinearOperator<Range, Domain, BlockPayload>
 block_diagonal_operator(const BlockMatrixType &block_matrix)
 {
-  typedef typename BlockLinearOperator<Range, Domain>::BlockType BlockType;
+  typedef typename BlockLinearOperator<Range, Domain, BlockPayload>::BlockType BlockType;
 
-  BlockLinearOperator<Range, Domain> return_op;
+  BlockLinearOperator<Range, Domain, BlockPayload> return_op (BlockPayload(block_matrix,block_matrix));
 
   return_op.n_block_rows = [&block_matrix]() -> unsigned int
   {
@@ -553,14 +608,14 @@ block_diagonal_operator(const BlockMatrixType &block_matrix)
  *
  * @ingroup LAOperators
  */
-template <size_t m, typename Range, typename Domain>
-BlockLinearOperator<Range, Domain>
-block_diagonal_operator(const std::array<LinearOperator<typename Range::BlockType, typename Domain::BlockType>, m> &ops)
+template <size_t m, typename Range, typename Domain, typename BlockPayload>
+BlockLinearOperator<Range, Domain, BlockPayload>
+block_diagonal_operator(const std::array<LinearOperator<typename Range::BlockType, typename Domain::BlockType, typename BlockPayload::BlockType>, m> &ops)
 {
   static_assert(m > 0,
                 "a blockdiagonal LinearOperator must consist of at least one block");
 
-  typedef typename BlockLinearOperator<Range, Domain>::BlockType BlockType;
+  typedef typename BlockLinearOperator<Range, Domain, BlockPayload>::BlockType BlockType;
 
   std::array<std::array<BlockType, m>, m> new_ops;
 
@@ -597,15 +652,15 @@ block_diagonal_operator(const std::array<LinearOperator<typename Range::BlockTyp
  *
  * @ingroup LAOperators
  */
-template <size_t m, typename Range, typename Domain>
-BlockLinearOperator<Range, Domain>
-block_diagonal_operator(const LinearOperator<typename Range::BlockType, typename Domain::BlockType> &op)
+template <size_t m, typename Range, typename Domain, typename BlockPayload>
+BlockLinearOperator<Range, Domain, BlockPayload>
+block_diagonal_operator(const LinearOperator<typename Range::BlockType, typename Domain::BlockType, typename BlockPayload::BlockType> &op)
 {
   static_assert(m > 0,
                 "a blockdiagonal LinearOperator must consist of at least "
                 "one block");
 
-  typedef typename BlockLinearOperator<Range, Domain>::BlockType BlockType;
+  typedef typename BlockLinearOperator<Range, Domain, BlockPayload>::BlockType BlockType;
   std::array<BlockType, m> new_ops;
   new_ops.fill(op);
 
@@ -655,12 +710,12 @@ block_diagonal_operator(const LinearOperator<typename Range::BlockType, typename
  *
  * @ingroup LAOperators
  */
-template <typename Range, typename Domain>
-LinearOperator<Domain, Range>
-block_forward_substitution(const BlockLinearOperator<Range, Domain> &block_operator,
-                           const BlockLinearOperator<Domain, Range> &diagonal_inverse)
+template <typename Range, typename Domain, typename BlockPayload>
+LinearOperator<Domain, Range, typename BlockPayload::BlockType>
+block_forward_substitution(const BlockLinearOperator<Range, Domain, BlockPayload> &block_operator,
+                           const BlockLinearOperator<Domain, Range, BlockPayload> &diagonal_inverse)
 {
-  LinearOperator<Range, Range> return_op;
+  LinearOperator<Range, Range, typename BlockPayload::BlockType> return_op ((typename BlockPayload::BlockType(diagonal_inverse)));
 
   return_op.reinit_range_vector = diagonal_inverse.reinit_range_vector;
   return_op.reinit_domain_vector = diagonal_inverse.reinit_domain_vector;
@@ -767,12 +822,12 @@ block_forward_substitution(const BlockLinearOperator<Range, Domain> &block_opera
  *
  * @ingroup LAOperators
  */
-template <typename Range, typename Domain>
-LinearOperator<Domain, Range>
-block_back_substitution(const BlockLinearOperator<Range, Domain> &block_operator,
-                        const BlockLinearOperator<Domain, Range> &diagonal_inverse)
+template <typename Range, typename Domain, typename BlockPayload>
+LinearOperator<Domain, Range, typename BlockPayload::BlockType>
+block_back_substitution(const BlockLinearOperator<Range, Domain, BlockPayload> &block_operator,
+                        const BlockLinearOperator<Domain, Range, BlockPayload> &diagonal_inverse)
 {
-  LinearOperator<Range, Range> return_op;
+  LinearOperator<Range, Range, typename BlockPayload::BlockType> return_op ((typename BlockPayload::BlockType(diagonal_inverse)));
 
   return_op.reinit_range_vector = diagonal_inverse.reinit_range_vector;
   return_op.reinit_domain_vector = diagonal_inverse.reinit_domain_vector;
