@@ -334,6 +334,13 @@ private:
    * number of blocks per row.
    */
   unsigned int block_cols;
+
+  /**
+   * Assignment operator. Since the copy constructor is destructive (see its
+   * documentation) and only exists for convenience there is no reasonable way
+   * to implement this. Hence this operator is both private and unimplemented.
+   */
+  Entry &operator= (const Entry &);
 };
 
 /*@}*/
