@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2015 by the deal.II authors
+// Copyright (C) 2000 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -24,23 +24,14 @@
 #include <deal.II/base/smartpointer.h>
 #include <deal.II/lac/sparsity_pattern.h>
 #include <deal.II/lac/trilinos_sparsity_pattern.h>
-#include <deal.II/lac/compressed_sparsity_pattern.h>
-#include <deal.II/lac/compressed_set_sparsity_pattern.h>
-#include <deal.II/lac/compressed_simple_sparsity_pattern.h>
+#include <deal.II/lac/dynamic_sparsity_pattern.h>
 #include <deal.II/lac/block_indices.h>
 
 DEAL_II_NAMESPACE_OPEN
 
 
 template <typename number> class BlockSparseMatrix;
-class BlockSparsityPattern;
 class BlockDynamicSparsityPattern;
-#ifdef DEAL_II_WITH_TRILINOS
-namespace TrilinosWrappers
-{
-  class BlockSparsityPattern;
-}
-#endif
 
 /*! @addtogroup Sparsity
  *@{
@@ -317,10 +308,6 @@ public:
                   int, int, int, int,
                   << "The blocks [" << arg1 << ',' << arg2 << "] and ["
                   << arg3 << ',' << arg4 << "] have differing column numbers.");
-  /**
-   * Exception
-   */
-  DeclException0 (ExcInvalidConstructorCall);
   //@}
 
 protected:
@@ -457,7 +444,7 @@ public:
  * rather acts as a @p typedef to introduce the name of this class, without
  * requiring the user to specify the templated name of the base class. For
  * information on the interface of this class refer to the base class. The
- * individual blocks are based on the CompressedSparsityPattern class.
+ * individual blocks are based on the DynamicSparsityPattern class.
  *
  * This class is an example of the "dynamic" type of
  * @ref Sparsity.
@@ -575,48 +562,38 @@ public:
   using BlockSparsityPatternBase<DynamicSparsityPattern>::reinit;
 };
 
-/**
- * @deprecated Use  BlockDynamicSparsityPattern instead.
- */
-typedef BlockDynamicSparsityPattern BlockCompressedSparsityPattern DEAL_II_DEPRECATED;
-
-/**
- * @deprecated Use  BlockDynamicSparsityPattern instead.
- */
-typedef BlockDynamicSparsityPattern BlockCompressedSetSparsityPattern DEAL_II_DEPRECATED;
-
-/**
- * @deprecated Use  BlockDynamicSparsityPattern instead.
- */
-typedef BlockDynamicSparsityPattern BlockCompressedSimpleSparsityPattern DEAL_II_DEPRECATED;
-
-
+/*@}*/
 
 
 #ifdef DEAL_II_WITH_TRILINOS
 
 
-/**
- * This class extends the base class to implement an array of Trilinos
- * sparsity patterns that can be used to initialize Trilinos block sparse
- * matrices that can be distributed among different processors. It is used in
- * the same way as the BlockSparsityPattern except that it builds upon the
- * TrilinosWrappers::SparsityPattern instead of the dealii::SparsityPattern.
- * See the documentation of the BlockSparsityPattern for examples.
- *
- * This class is has properties of the "dynamic" type of
- * @ref Sparsity
- * (in the sense that it can extend the memory if too little elements were
- * allocated), but otherwise is more like the basic deal.II SparsityPattern
- * (in the sense that the method compress() needs to be called before the
- * pattern can be used).
- *
- * This class is used in step-32.
- *
- * @author Martin Kronbichler, 2008, 2009
- */
 namespace TrilinosWrappers
 {
+
+  /*! @addtogroup TrilinosWrappers
+   *@{
+   */
+
+  /**
+   * This class extends the base class to implement an array of Trilinos
+   * sparsity patterns that can be used to initialize Trilinos block sparse
+   * matrices that can be distributed among different processors. It is used in
+   * the same way as the dealii::BlockSparsityPattern except that it builds upon
+   * the TrilinosWrappers::SparsityPattern instead of the
+   * dealii::SparsityPattern.
+   *
+   * This class is has properties of the "dynamic" type of
+   * @ref Sparsity
+   * (in the sense that it can extend the memory if too little elements were
+   * allocated), but otherwise is more like the basic deal.II SparsityPattern
+   * (in the sense that the method compress() needs to be called before the
+   * pattern can be used).
+   *
+   * This class is used in step-32.
+   *
+   * @author Martin Kronbichler, 2008, 2009
+   */
   class BlockSparsityPattern :
     public dealii::BlockSparsityPatternBase<SparsityPattern>
   {
@@ -739,12 +716,13 @@ namespace TrilinosWrappers
      */
     using BlockSparsityPatternBase<SparsityPattern>::reinit;
   };
-}
+
+  /*@}*/
+
+} /* namespace TrilinosWrappers */
 
 #endif
 
-
-/*@}*/
 /*---------------------- Template functions -----------------------------------*/
 
 
