@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2015 by the deal.II authors
+// Copyright (C) 1999 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -21,6 +21,7 @@
 #include <deal.II/base/exceptions.h>
 #include <deal.II/lac/block_indices.h>
 #include <deal.II/lac/block_vector_base.h>
+#include <deal.II/lac/vector_type_traits.h>
 
 #include <cstdio>
 #include <vector>
@@ -31,7 +32,6 @@ DEAL_II_NAMESPACE_OPEN
 #ifdef DEAL_II_WITH_TRILINOS
 namespace TrilinosWrappers
 {
-  class Vector;
   class BlockVector;
 }
 #endif
@@ -108,12 +108,12 @@ public:
 #ifdef DEAL_II_WITH_CXX11
   /**
    * Move constructor. Creates a new vector by stealing the internal data of
-   * the vector @p v.
+   * the given argument vector.
    *
    * @note This constructor is only available if deal.II is configured with
    * C++11 support.
    */
-  BlockVector (BlockVector<Number> &&v);
+  BlockVector (BlockVector<Number> &&/*v*/) = default;
 #endif
 
 
@@ -203,13 +203,13 @@ public:
 
 #ifdef DEAL_II_WITH_CXX11
   /**
-   * Move the given vector. This operator replaces the present vector with @p
-   * v by efficiently swapping the internal data structures.
+   * Move the given vector. This operator replaces the present vector with
+   * the contents of the given argument vector.
    *
    * @note This operator is only available if deal.II is configured with C++11
    * support.
    */
-  BlockVector<Number> &operator= (BlockVector<Number> &&v);
+  BlockVector<Number> &operator= (BlockVector<Number> &&/*v*/) = default;
 #endif
 
   /**
@@ -313,7 +313,7 @@ public:
    * only swaps the pointers to the data of the two vectors and therefore does
    * not need to allocate temporary storage and move data around.
    *
-   * This function is analog to the the swap() function of all C++ standard
+   * This function is analogous to the the swap() function of all C++ standard
    * containers. Also, there is a global function swap(u,v) that simply calls
    * <tt>u.swap(v)</tt>, again in analogy to standard functions.
    */
@@ -423,20 +423,6 @@ BlockVector<Number>::operator= (const BlockVector<Number> &v)
 
 
 
-#ifdef DEAL_II_WITH_CXX11
-template <typename Number>
-inline
-BlockVector<Number> &
-BlockVector<Number>::operator= (BlockVector<Number> &&v)
-{
-  swap(v);
-
-  return *this;
-}
-#endif
-
-
-
 template <typename Number>
 inline
 BlockVector<Number> &
@@ -503,7 +489,7 @@ namespace internal
     template <typename> class ReinitHelper;
 
     /**
-     * A helper class internally used in linear_operator.h. Specialization for
+     * A helper class used internally in linear_operator.h. Specialization for
      * BlockVector<number>.
      */
     template<typename number>
@@ -532,6 +518,17 @@ namespace internal
   } /* namespace LinearOperator */
 } /* namespace internal */
 
+
+/**
+ * Declare dealii::BlockVector< Number > as serial vector.
+ *
+ * @author Uwe Koecher, 2017
+ */
+template <typename Number>
+struct is_serial_vector< BlockVector< Number > > : std_cxx11::true_type
+{
+};
+
 DEAL_II_NAMESPACE_CLOSE
 
 #endif
