@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2015 by the deal.II authors
+// Copyright (C) 2004 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -277,9 +277,9 @@ namespace internal
 
 
       /**
-       * Copy constructor.
+       * Copy constructor from an iterator with the same constness.
        */
-      Iterator (const Iterator<BlockVectorType,Constness> &c);
+      Iterator (const Iterator &c);
 
     private:
       /**
@@ -343,47 +343,47 @@ namespace internal
        * Compare for equality of iterators. This operator checks whether the
        * vectors pointed to are the same, and if not it throws an exception.
        */
-      template <bool _Constness>
-      bool operator == (const Iterator<BlockVectorType, _Constness> &i) const;
+      template <bool OtherConstness>
+      bool operator == (const Iterator<BlockVectorType, OtherConstness> &i) const;
 
       /**
        * Compare for inequality of iterators. This operator checks whether the
        * vectors pointed to are the same, and if not it throws an exception.
        */
-      template <bool _Constness>
-      bool operator != (const Iterator<BlockVectorType, _Constness> &i) const;
+      template <bool OtherConstness>
+      bool operator != (const Iterator<BlockVectorType, OtherConstness> &i) const;
 
       /**
        * Check whether this iterators points to an element previous to the one
        * pointed to by the given argument. This operator checks whether the
        * vectors pointed to are the same, and if not it throws an exception.
        */
-      template <bool _Constness>
-      bool operator < (const Iterator<BlockVectorType, _Constness> &i) const;
+      template <bool OtherConstness>
+      bool operator < (const Iterator<BlockVectorType, OtherConstness> &i) const;
 
       /**
        * Comparison operator alike to the one above.
        */
-      template <bool _Constness>
-      bool operator <= (const Iterator<BlockVectorType, _Constness> &i) const;
+      template <bool OtherConstness>
+      bool operator <= (const Iterator<BlockVectorType, OtherConstness> &i) const;
 
       /**
        * Comparison operator alike to the one above.
        */
-      template <bool _Constness>
-      bool operator > (const Iterator<BlockVectorType, _Constness> &i) const;
+      template <bool OtherConstness>
+      bool operator > (const Iterator<BlockVectorType, OtherConstness> &i) const;
 
       /**
        * Comparison operator alike to the one above.
        */
-      template <bool _Constness>
-      bool operator >= (const Iterator<BlockVectorType, _Constness> &i) const;
+      template <bool OtherConstness>
+      bool operator >= (const Iterator<BlockVectorType, OtherConstness> &i) const;
 
       /**
        * Return the distance between the two iterators, in elements.
        */
-      template <bool _Constness>
-      difference_type operator - (const Iterator<BlockVectorType, _Constness> &i) const;
+      template <bool OtherConstness>
+      difference_type operator - (const Iterator<BlockVectorType, OtherConstness> &i) const;
 
       /**
        * Return an iterator which is the given number of elements in front of
@@ -569,14 +569,34 @@ public:
    *
    * For the current class, the variable equals the value declared for the
    * type of the individual blocks.
+   *
+   * @deprecated instead of using this variable, please use the type trait value
+   * <code>is_serial_vector< VectorType >::value</code>
    */
-  static const bool supports_distributed_data = BlockType::supports_distributed_data;
+  static const bool supports_distributed_data DEAL_II_DEPRECATED = !is_serial_vector<BlockType>::value;
 
   /**
    * Default constructor.
    */
   BlockVectorBase ();
 
+#ifdef DEAL_II_WITH_CXX11
+  /**
+   * Copy constructor.
+   */
+  BlockVectorBase (const BlockVectorBase &V) = default;
+
+  /**
+   * Move constructor. Each block of the argument vector is moved into the current
+   * object if the underlying <code>VectorType</code> is move-constructible,
+   * otherwise they are copied.
+   *
+   * @note This constructor is only available if deal.II is configured with
+   * C++11 support.
+   */
+  BlockVectorBase (BlockVectorBase &&/*V*/) = default;
+#endif
+
   /**
    * Update internal structures after resizing vectors. Whenever you reinited
    * a block of a block vector, the internal data structures are corrupted.
@@ -723,6 +743,15 @@ public:
   BlockVectorBase &
   operator= (const BlockVectorBase &V);
 
+#ifdef DEAL_II_WITH_CXX11
+  /**
+   * Move assignment operator. Move each block of the given argument
+   * vector into the current object if `VectorType` is
+   * move-constructible, otherwise copy them.
+   */
+  BlockVectorBase &operator= (BlockVectorBase &&/*V*/) = default;
+#endif
+
   /**
    * Copy operator for template arguments of different types.
    */
@@ -788,7 +817,7 @@ public:
    * The reason this function exists is that this operation involves less
    * memory transfer than calling the two functions separately on deal.II's
    * vector classes (Vector<Number> and
-   * parallel::distributed::Vector<double>). This method only needs to load
+   * LinearAlgebra::distributed::Vector<double>). This method only needs to load
    * three vectors, @p this, @p V, @p W, whereas calling separate methods
    * means to load the calling vector @p this twice. Since most vector
    * operations are memory transfer limited, this reduces the time by 25\% (or
@@ -799,7 +828,7 @@ public:
                           const BlockVectorBase &W);
 
   /**
-   * Returns true if the given global index is in the local range of this
+   * Return true if the given global index is in the local range of this
    * processor. Asks the corresponding block.
    */
   bool in_local_range (const size_type global_index) const;
@@ -1150,11 +1179,11 @@ namespace internal
 
 
     template <class BlockVectorType, bool Constness>
-    template <bool _Constness>
+    template <bool OtherConstness>
     inline
     bool
     Iterator<BlockVectorType,Constness>::
-    operator == (const Iterator<BlockVectorType, _Constness> &i) const
+    operator == (const Iterator<BlockVectorType, OtherConstness> &i) const
     {
       Assert (parent == i.parent, ExcPointerToDifferentVectors());
 
@@ -1164,11 +1193,11 @@ namespace internal
 
 
     template <class BlockVectorType, bool Constness>
-    template <bool _Constness>
+    template <bool OtherConstness>
     inline
     bool
     Iterator<BlockVectorType,Constness>::
-    operator != (const Iterator<BlockVectorType, _Constness> &i) const
+    operator != (const Iterator<BlockVectorType, OtherConstness> &i) const
     {
       Assert (parent == i.parent, ExcPointerToDifferentVectors());
 
@@ -1178,11 +1207,11 @@ namespace internal
 
 
     template <class BlockVectorType, bool Constness>
-    template <bool _Constness>
+    template <bool OtherConstness>
     inline
     bool
     Iterator<BlockVectorType,Constness>::
-    operator < (const Iterator<BlockVectorType, _Constness> &i) const
+    operator < (const Iterator<BlockVectorType, OtherConstness> &i) const
     {
       Assert (parent == i.parent, ExcPointerToDifferentVectors());
 
@@ -1192,11 +1221,11 @@ namespace internal
 
 
     template <class BlockVectorType, bool Constness>
-    template <bool _Constness>
+    template <bool OtherConstness>
     inline
     bool
     Iterator<BlockVectorType,Constness>::
-    operator <= (const Iterator<BlockVectorType, _Constness> &i) const
+    operator <= (const Iterator<BlockVectorType, OtherConstness> &i) const
     {
       Assert (parent == i.parent, ExcPointerToDifferentVectors());
 
@@ -1206,11 +1235,11 @@ namespace internal
 
 
     template <class BlockVectorType, bool Constness>
-    template <bool _Constness>
+    template <bool OtherConstness>
     inline
     bool
     Iterator<BlockVectorType,Constness>::
-    operator > (const Iterator<BlockVectorType, _Constness> &i) const
+    operator > (const Iterator<BlockVectorType, OtherConstness> &i) const
     {
       Assert (parent == i.parent, ExcPointerToDifferentVectors());
 
@@ -1220,11 +1249,11 @@ namespace internal
 
 
     template <class BlockVectorType, bool Constness>
-    template <bool _Constness>
+    template <bool OtherConstness>
     inline
     bool
     Iterator<BlockVectorType,Constness>::
-    operator >= (const Iterator<BlockVectorType, _Constness> &i) const
+    operator >= (const Iterator<BlockVectorType, OtherConstness> &i) const
     {
       Assert (parent == i.parent, ExcPointerToDifferentVectors());
 
@@ -1234,11 +1263,11 @@ namespace internal
 
 
     template <class BlockVectorType, bool Constness>
-    template <bool _Constness>
+    template <bool OtherConstness>
     inline
     typename Iterator<BlockVectorType,Constness>::difference_type
     Iterator<BlockVectorType,Constness>::
-    operator - (const Iterator<BlockVectorType, _Constness> &i) const
+    operator - (const Iterator<BlockVectorType, OtherConstness> &i) const
     {
       Assert (parent == i.parent, ExcPointerToDifferentVectors());
 
@@ -1468,6 +1497,7 @@ namespace internal
 } //namespace internal
 
 
+
 template <class VectorType>
 inline
 BlockVectorBase<VectorType>::BlockVectorBase ()
@@ -1686,10 +1716,11 @@ typename BlockVectorBase<VectorType>::value_type
 BlockVectorBase<VectorType>::mean_value () const
 {
   value_type sum = 0.;
+  // need to do static_cast as otherwise it won't work with value_type=complex<T>
   for (size_type i=0; i<n_blocks(); ++i)
-    sum += components[i].mean_value() * components[i].size();
+    sum += components[i].mean_value() * static_cast<double>(components[i].size());
 
-  return sum/size();
+  return sum/static_cast<double>(size());
 }
 
 
@@ -2021,11 +2052,9 @@ template <class VectorType>
 std::size_t
 BlockVectorBase<VectorType>::memory_consumption () const
 {
-  std::size_t mem = sizeof(this->n_blocks());
-  for (size_type i=0; i<this->components.size(); ++i)
-    mem += MemoryConsumption::memory_consumption (this->components[i]);
-  mem += MemoryConsumption::memory_consumption (this->block_indices);
-  return mem;
+  return (MemoryConsumption::memory_consumption (this->block_indices)
+          +
+          MemoryConsumption::memory_consumption (this->components));
 }
 
 
