@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2015 by the deal.II authors
+// Copyright (C) 2008 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -300,9 +300,11 @@ ChunkSparseMatrix<number>::ChunkSparseMatrix (const ChunkSparseMatrix &m)
   val(0),
   max_len(0)
 {
-  Assert (m.cols==0, ExcInvalidConstructorCall());
-  Assert (m.val==0, ExcInvalidConstructorCall());
-  Assert (m.max_len==0, ExcInvalidConstructorCall());
+  Assert (m.cols==0 && m.val==0 && m.max_len==0,
+          ExcMessage("This constructor can only be called if the provided argument "
+                     "is an empty matrix. This constructor can not be used to "
+                     "copy-construct a non-empty matrix. Use the "
+                     "ChunkSparseMatrix::copy_from() function for that purpose."));
 }
 
 
@@ -312,9 +314,11 @@ ChunkSparseMatrix<number> &
 ChunkSparseMatrix<number>::operator = (const ChunkSparseMatrix<number> &m)
 {
   (void)m;
-  Assert (m.cols==0, ExcInvalidConstructorCall());
-  Assert (m.val==0, ExcInvalidConstructorCall());
-  Assert (m.max_len==0, ExcInvalidConstructorCall());
+  Assert (m.cols==0 && m.val==0 && m.max_len==0,
+          ExcMessage("This operator can only be called if the provided right "
+                     "hand side is an empty matrix. This operator can not be "
+                     "used to copy a non-empty matrix. Use the "
+                     "ChunkSparseMatrix::copy_from() function for that purpose."));
 
   return *this;
 }
@@ -519,7 +523,7 @@ ChunkSparseMatrix<number>::n_actually_nonzero_elements () const
   return std::count_if(&val[0],
                        &val[cols->sparsity_pattern.n_nonzero_elements () *
                             chunk_size * chunk_size],
-                       std::bind2nd(std::not_equal_to<double>(), 0));
+                       std_cxx11::bind(std::not_equal_to<double>(), std_cxx11::_1, 0));
 }
 
 
