@@ -1,7 +1,6 @@
 // ---------------------------------------------------------------------
-// $Id: chunk_sparsity_pattern.h 31414 2013-10-24 22:15:50Z bangerth $
 //
-// Copyright (C) 2008 - 2013 by the deal.II authors
+// Copyright (C) 2008 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef __deal2__chunk_sparsity_pattern_h
-#define __deal2__chunk_sparsity_pattern_h
+#ifndef dealii__chunk_sparsity_pattern_h
+#define dealii__chunk_sparsity_pattern_h
 
 
 #include <deal.II/base/config.h>
@@ -49,13 +48,13 @@ namespace ChunkSparsityPatternIterators
   class Iterator;
 
   /**
-   * Accessor class for iterators into sparsity patterns. This class is
-   * also the base class for both const and non-const accessor classes
-   * into sparse matrices.
+   * Accessor class for iterators into sparsity patterns. This class is also
+   * the base class for both const and non-const accessor classes into sparse
+   * matrices.
    *
-   * Note that this class only allows read access to elements, providing
-   * their row and column number. It does not allow modifying the
-   * sparsity pattern itself.
+   * Note that this class only allows read access to elements, providing their
+   * row and column number. It does not allow modifying the sparsity pattern
+   * itself.
    *
    * @author Martin Kronbichler
    * @date 2013
@@ -81,7 +80,7 @@ namespace ChunkSparsityPatternIterators
     unsigned int row () const;
 
     /**
-     * Returns the global index from the reduced sparsity pattern.
+     * Return the global index from the reduced sparsity pattern.
      */
     std::size_t reduced_index() const;
 
@@ -97,8 +96,8 @@ namespace ChunkSparsityPatternIterators
      * entries are valid. However, before compression, the sparsity pattern
      * allocated some memory to be used while still adding new nonzero
      * entries; if you create iterators in this phase of the sparsity
-     * pattern's lifetime, you will iterate over elements that are not
-     * valid. If this is so, then this function will return false.
+     * pattern's lifetime, you will iterate over elements that are not valid.
+     * If this is so, then this function will return false.
      */
     bool is_valid_entry () const;
 
@@ -153,7 +152,7 @@ namespace ChunkSparsityPatternIterators
 
 
   /**
-   * STL conforming iterator walking over the elements of a sparsity pattern.
+   * Iterator that walks over the elements of a sparsity pattern.
    */
   class Iterator
   {
@@ -215,8 +214,9 @@ namespace ChunkSparsityPatternIterators
 
 
 /**
- * Structure representing the sparsity pattern of a sparse matrix.
- * This class is an example of the "static" type of @ref Sparsity.
+ * Structure representing the sparsity pattern of a sparse matrix. This class
+ * is an example of the "static" type of
+ * @ref Sparsity.
  * It uses the compressed row storage (CSR) format to store data.
  *
  * The use of this class is demonstrated in step-51.
@@ -272,14 +272,14 @@ public:
    * Copy constructor. This constructor is only allowed to be called if the
    * matrix structure to be copied is empty. This is so in order to prevent
    * involuntary copies of objects for temporaries, which can use large
-   * amounts of computing time.  However, copy constructors are needed if yo
-   * want to use the STL data types on classes like this, e.g. to write such
+   * amounts of computing time. However, copy constructors are needed if one
+   * wants to place a ChunkSparsityPattern in a container, e.g., to write such
    * statements like <tt>v.push_back (ChunkSparsityPattern());</tt>, with
    * <tt>v</tt> a vector of ChunkSparsityPattern objects.
    *
    * Usually, it is sufficient to use the explicit keyword to disallow
-   * unwanted temporaries, but for the STL vectors, this does not work. Since
-   * copying a structure like this is not useful anyway because multiple
+   * unwanted temporaries, but this does not work for <tt>std::vector</tt>.
+   * Since copying a structure like this is not useful anyway because multiple
    * matrices can use the same sparsity structure, copies are only allowed for
    * empty objects, as described above.
    */
@@ -288,48 +288,26 @@ public:
   /**
    * Initialize a rectangular matrix.
    *
-   * @arg m number of rows
-   * @arg n number of columns
-   * @arg max_per_row maximum number of nonzero entries per row
+   * @arg m number of rows @arg n number of columns @arg max_per_row maximum
+   * number of nonzero entries per row
    */
   ChunkSparsityPattern (const size_type m,
                         const size_type n,
                         const size_type max_chunks_per_row,
                         const size_type chunk_size);
 
-  /**
-   * @deprecated This constructor is deprecated. Use the version
-   * without the last argument
-   */
-  ChunkSparsityPattern (const size_type m,
-                        const size_type n,
-                        const size_type max_chunks_per_row,
-                        const size_type chunk_size,
-                        const bool optimize_diagonal) DEAL_II_DEPRECATED;
-
   /**
    * Initialize a rectangular matrix.
    *
-   * @arg m number of rows
-   * @arg n number of columns
-   * @arg row_lengths possible number of nonzero entries for each row.  This
-   * vector must have one entry for each row.
+   * @arg m number of rows @arg n number of columns @arg row_lengths possible
+   * number of nonzero entries for each row.  This vector must have one entry
+   * for each row.
    */
   ChunkSparsityPattern (const size_type m,
                         const size_type n,
                         const std::vector<size_type> &row_lengths,
                         const size_type chunk_size);
 
-  /**
-   * @deprecated This constructor is deprecated. Use the version
-   * without the last argument
-   */
-  ChunkSparsityPattern (const size_type               m,
-                        const size_type               n,
-                        const std::vector<size_type> &row_lengths,
-                        const size_type chunk_size,
-                        const bool optimize_diagonal) DEAL_II_DEPRECATED;
-
   /**
    * Initialize a quadratic matrix of dimension <tt>n</tt> with at most
    * <tt>max_per_row</tt> nonzero entries per row.
@@ -345,23 +323,14 @@ public:
   /**
    * Initialize a quadratic matrix.
    *
-   * @arg m number of rows and columns
-   * @arg row_lengths possible number of nonzero entries for each row.  This
-   * vector must have one entry for each row.
+   * @arg m number of rows and columns @arg row_lengths possible number of
+   * nonzero entries for each row.  This vector must have one entry for each
+   * row.
    */
   ChunkSparsityPattern (const size_type                m,
                         const std::vector<size_type>  &row_lengths,
                         const size_type                chunk_size);
 
-  /**
-   * @deprecated This constructor is deprecated. Use the version
-   * without the last argument
-   */
-  ChunkSparsityPattern (const size_type               m,
-                        const std::vector<size_type> &row_lengths,
-                        const size_type               chunk_size,
-                        const bool optimize_diagonal) DEAL_II_DEPRECATED;
-
   /**
    * Destructor.
    */
@@ -387,16 +356,6 @@ public:
                const size_type max_per_row,
                const size_type chunk_size);
 
-  /**
-   * @deprecated This function is deprecated. Use the function
-   * without the last argument
-   */
-  void reinit (const size_type m,
-               const size_type n,
-               const size_type max_per_row,
-               const size_type chunk_size,
-               const bool optimize_diagonal) DEAL_II_DEPRECATED;
-
   /**
    * Reallocate memory for a matrix of size <tt>m x n</tt>. The number of
    * entries for each row is taken from the array <tt>row_lengths</tt> which
@@ -407,11 +366,8 @@ public:
    * allocated if the new size extends the old one. This is done to save time
    * and to avoid fragmentation of the heap.
    *
-   * If the number of rows equals
-   * the number of columns then
-   * diagonal elements are stored
-   * first in each row to allow
-   * optimized access in relaxation
+   * If the number of rows equals the number of columns then diagonal elements
+   * are stored first in each row to allow optimized access in relaxation
    * methods of SparseMatrix.
    */
   void reinit (const size_type m,
@@ -419,16 +375,6 @@ public:
                const std::vector<size_type> &row_lengths,
                const size_type chunk_size);
 
-  /**
-   * @deprecated This function is deprecated. Use the function
-   * without the last argument
-   */
-  void reinit (const size_type                m,
-               const size_type                n,
-               const std::vector<size_type > &row_lengths,
-               const size_type                chunk_size,
-               const bool                     optimize_diagonal) DEAL_II_DEPRECATED;
-
   /**
    * Same as above, but with a VectorSlice argument instead.
    */
@@ -437,16 +383,6 @@ public:
                const VectorSlice<const std::vector<size_type> > &row_lengths,
                const size_type chunk_size);
 
-  /**
-   * @deprecated This function is deprecated. Use the function
-   * without the last argument
-   */
-  void reinit (const size_type m,
-               const size_type n,
-               const VectorSlice<const std::vector<size_type> > &row_lengths,
-               const size_type chunk_size,
-               const bool optimize_diagonal) DEAL_II_DEPRECATED;
-
   /**
    * This function compresses the sparsity structure that this object
    * represents.  It does so by eliminating unused entries and sorting the
@@ -545,35 +481,13 @@ public:
                   const size_type chunk_size);
 
   /**
-   * @deprecated This function is deprecated. Use the function
-   * without the last argument
+   * Copy data from an object of type DynamicSparsityPattern. Previous content
+   * of this object is lost, and the sparsity pattern is in compressed mode
+   * afterwards.
    */
-  template <typename ForwardIterator>
-  void copy_from (const size_type       n_rows,
-                  const size_type       n_cols,
-                  const ForwardIterator begin,
-                  const ForwardIterator end,
-                  const size_type       chunk_size,
-                  const bool optimize_diagonal) DEAL_II_DEPRECATED;
-
-  /**
-   * Copy data from an object of type CompressedSparsityPattern,
-   * CompressedSetSparsityPattern or CompressedSimpleSparsityPattern.
-   * Previous content of this object is lost, and the sparsity pattern is in
-   * compressed mode afterwards.
-   */
-  template <typename SparsityType>
-  void copy_from (const SparsityType &csp,
-                  const size_type     chunk_size);
-
-  /**
-   * @deprecated This function is deprecated. Use the function
-   * without the last argument
-   */
-  template <typename SparsityType>
-  void copy_from (const SparsityType &csp,
-                  const size_type     chunk_size,
-                  const bool          optimize_diagonal) DEAL_II_DEPRECATED;
+  template <typename SparsityPatternType>
+  void copy_from (const SparsityPatternType &dsp,
+                  const size_type            chunk_size);
 
   /**
    * Take a full matrix and use its nonzero entries to generate a sparse
@@ -586,15 +500,6 @@ public:
   void copy_from (const FullMatrix<number> &matrix,
                   const size_type chunk_size);
 
-  /**
-   * @deprecated This function is deprecated. Use the function
-   * without the last argument
-   */
-  template <typename number>
-  void copy_from (const FullMatrix<number> &matrix,
-                  const size_type chunk_size,
-                  const bool optimize_diagonal) DEAL_II_DEPRECATED;
-
   /**
    * Set the sparsity pattern of the chunk sparsity pattern to be given by
    * <tt>chunk_size*chunksize</tt> blocks of the sparsity pattern for chunks
@@ -701,21 +606,13 @@ public:
    */
   bool is_compressed () const;
 
-  /**
-   * Determine whether the matrix uses the special convention for quadratic
-   * matrices that the diagonal entries are stored first in each row.
-   *
-   * @deprecated The function always returns true if the matrix is
-   * square and false if it is not.
-   */
-  bool optimize_diagonal () const DEAL_II_DEPRECATED;
-
   /**
    * Return whether this object stores only those entries that have been added
    * explicitly, or if the sparsity pattern contains elements that have been
    * added through other means (implicitly) while building it. For the current
    * class, the result is true if and only if it is square because it then
-   * unconditionally stores the diagonal entries whether they have been added explicitly or not.
+   * unconditionally stores the diagonal entries whether they have been added
+   * explicitly or not.
    *
    * This function mainly serves the purpose of describing the current class
    * in cases where several kinds of sparsity patterns can be passed as
@@ -724,7 +621,7 @@ public:
   bool stores_only_added_elements () const;
 
   /**
-   * STL-like iterator with the first entry of the matrix. The resulting
+   * Iterator starting at the first entry of the matrix. The resulting
    * iterator can be used to walk over all nonzero entries of the sparsity
    * pattern.
    */
@@ -736,7 +633,7 @@ public:
   iterator end () const;
 
   /**
-   * STL-like iterator with the first entry of row <tt>r</tt>.
+   * Iterator starting at the first entry of row <tt>r</tt>.
    *
    * Note that if the given row is empty, i.e. does not contain any nonzero
    * entries, then the iterator returned by this function equals
@@ -768,10 +665,10 @@ public:
   void block_write (std::ostream &out) const;
 
   /**
-   * Read data that has previously been written by block_write() from a
-   * file. This is done using the inverse operations to the above function, so
-   * it is reasonably fast because the bitstream is not interpreted except for
-   * a few numbers up front.
+   * Read data that has previously been written by block_write() from a file.
+   * This is done using the inverse operations to the above function, so it is
+   * reasonably fast because the bitstream is not interpreted except for a few
+   * numbers up front.
    *
    * The object is resized on this operation, and all previous contents are
    * lost.
@@ -810,8 +707,10 @@ public:
    */
   std::size_t memory_consumption () const;
 
-  /** @addtogroup Exceptions
-   * @{ */
+  /**
+   * @addtogroup Exceptions
+   * @{
+   */
   /**
    * Exception
    */
@@ -835,21 +734,23 @@ public:
                   << "(Maximum number of entries for this row: "
                   << arg2 << "; maybe the matrix is already compressed?)");
   /**
-   * Exception
+   * The operation is only allowed after the SparsityPattern has been set up
+   * and compress() was called.
    */
-  DeclException0 (ExcNotCompressed);
+  DeclExceptionMsg (ExcNotCompressed,
+                    "The operation you attempted is only allowed after the SparsityPattern "
+                    "has been set up and compress() was called.");
   /**
-   * Exception
+   * This operation changes the structure of the SparsityPattern and is not
+   * possible after compress() has been called.
    */
-  DeclException0 (ExcMatrixIsCompressed);
+  DeclExceptionMsg (ExcMatrixIsCompressed,
+                    "The operation you attempted changes the structure of the SparsityPattern "
+                    "and is not possible after compress() has been called.");
   /**
    * Exception
    */
   DeclException0 (ExcEmptyObject);
-  /**
-   * Exception
-   */
-  DeclException0 (ExcInvalidConstructorCall);
   /**
    * Exception
    */
@@ -1254,30 +1155,6 @@ ChunkSparsityPattern::is_compressed () const
 
 
 
-inline
-bool
-ChunkSparsityPattern::optimize_diagonal () const
-{
-  return sparsity_pattern.store_diagonal_first_in_row;
-}
-
-
-
-template <typename ForwardIterator>
-inline
-void
-ChunkSparsityPattern::copy_from (const size_type n_rows,
-                                 const size_type n_cols,
-                                 const ForwardIterator begin,
-                                 const ForwardIterator end,
-                                 const size_type chunk_size,
-                                 const bool)
-{
-  copy_from (n_rows, n_cols, begin, end, chunk_size);
-}
-
-
-
 template <typename ForwardIterator>
 void
 ChunkSparsityPattern::copy_from (const size_type       n_rows,
