@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2016 by the deal.II authors
+// Copyright (C) 1998 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -22,15 +22,12 @@
 #include <deal.II/base/index_set.h>
 #include <deal.II/base/subscriptor.h>
 #include <deal.II/base/template_constraints.h>
-#include <deal.II/base/thread_local_storage.h>
 
 #include <deal.II/lac/vector.h>
 
 #include <vector>
-#include <map>
 #include <set>
 #include <utility>
-#include <complex>
 
 
 DEAL_II_NAMESPACE_OPEN
@@ -43,8 +40,6 @@ class BlockSparsityPattern;
 class BlockDynamicSparsityPattern;
 template <typename number> class SparseMatrix;
 template <typename number> class BlockSparseMatrix;
-class BlockIndices;
-
 
 namespace internals
 {
@@ -213,7 +208,7 @@ public:
   bool can_store_line (const size_type line_index) const;
 
   /**
-   * Returns the index set describing locally relevant lines if any are
+   * Return the index set describing locally relevant lines if any are
    * present. Note that if no local lines were given, this represents an empty
    * IndexSet, whereas otherwise it contains the global problem size and the
    * local range.
@@ -364,13 +359,23 @@ public:
    * second. If this is nevertheless the case, an exception is thrown.
    * However, this behavior can be changed by providing a different value for
    * the second argument.
+   *
+   * By default, merging two ConstraintMatrix objects that are initialized
+   * with different IndexSet objects is not allowed.
+   * This behavior can be altered by setting @p allow_different_local_lines
+   * appropriately.
+   *
+   * Merging a ConstraintMatrix that is initialized with an IndexSet
+   * and one that is not initialized with an IndexSet is not yet implemented.
    */
   void merge (const ConstraintMatrix &other_constraints,
-              const MergeConflictBehavior merge_conflict_behavior = no_conflicts_allowed);
+              const MergeConflictBehavior merge_conflict_behavior = no_conflicts_allowed,
+              const bool allow_different_local_lines = false);
 
   /**
    * Shift all entries of this matrix down @p offset rows and over @p offset
-   * columns.
+   * columns. If this object is initialized with an IndexSet, local_lines are
+   * shifted as well.
    *
    * This function is useful if you are building block matrices, where all
    * blocks are built by the same DoFHandler object, i.e. the matrix size is
@@ -451,40 +456,52 @@ public:
   size_type max_constraint_indirections () const;
 
   /**
-   * Returns <tt>true</tt> in case the dof is constrained and there is a non-
+   * Return <tt>true</tt> in case the dof is constrained and there is a non-
    * trivial inhomogeneous values set to the dof.
    */
   bool is_inhomogeneously_constrained (const size_type index) const;
 
   /**
-   * Returns <tt>false</tt> if all constraints in the ConstraintMatrix are
+   * Return <tt>false</tt> if all constraints in the ConstraintMatrix are
    * homogeneous ones, and <tt>true</tt> if there is at least one
    * inhomogeneity.
    */
   bool has_inhomogeneities () const;
 
   /**
-   * Returns a pointer to the the vector of entries if a line is constrained,
+   * Return a pointer to the the vector of entries if a line is constrained,
    * and a zero pointer in case the dof is not constrained.
    */
   const std::vector<std::pair<size_type,double> > *
   get_constraint_entries (const size_type line) const;
 
   /**
-   * Returns the value of the inhomogeneity stored in the constrained dof @p
+   * Return the value of the inhomogeneity stored in the constrained dof @p
    * line. Unconstrained dofs also return a zero value.
    */
   double get_inhomogeneity (const size_type line) const;
 
   /**
-   * Print the constraint lines. Mainly for debugging purposes.
+   * Print the constraints represented by the current object to the
+   * given stream.
    *
-   * This function writes out all entries in the constraint matrix lines with
-   * their value in the form <tt>row col : value</tt>. Unconstrained lines
-   * containing only one identity entry are not stored in this object and are
-   * not printed.
+   * For each constraint of the form
+   * @f[
+   *  x_{42} = 0.5 x_2 + 0.25 x_{14} + 2.75
+   * @f]
+   * this function will write a sequence of lines that look like this:
+   * @code
+   *   42 2 : 0.5
+   *   42 14 : 0.25
+   *   42 : 2.75
+   * @endcode
+   * The last line is only shown if the inhomogeneity (here: 2.75) is
+   * nonzero.
+   *
+   * A block of lines such as the one above is repeated for each
+   * constrained degree of freedom.
    */
-  void print (std::ostream &) const;
+  void print (std::ostream &out) const;
 
   /**
    * Write the graph of constraints in 'dot' format. 'dot' is a program that
@@ -625,7 +642,7 @@ public:
                  BlockVectorType           &vector) const;
 
   /**
-   * Sets the values of all constrained DoFs in a vector to zero.  The @p
+   * Set the values of all constrained DoFs in a vector to zero.  The @p
    * VectorType may be a Vector<float>, Vector<double>,
    * BlockVector<tt><...></tt>, a PETSc or Trilinos vector wrapper class, or
    * any other type having the same interface.
@@ -740,6 +757,32 @@ public:
                               VectorType                   &global_vector,
                               const FullMatrix<LocalType>  &local_matrix) const;
 
+  /**
+   * Same as the previous function, except that it uses two (possibly) different
+   * index sets to correctly handle inhomogeneities when the local matrix is
+   * computed from a combination of two neighboring elements, for example for an
+   * edge integral term in DG. Note that in the case that these two elements have
+   * different polynomial degree, the local matrix is rectangular.
+   *
+   * <tt>local_dof_indices_row</tt> is the set of row indices and
+   * <tt>local_dof_indices_col</tt> is the set of column indices of the local matrix.
+   * <tt>diagonal=false</tt> says whether the two index sets are equal or not.
+   *
+   * If both index sets are equal, <tt>diagonal</tt> must be set to true or we
+   * simply use the previous function. If both index sets are different (diagonal=false)
+   * the <tt>global_vector</tt> is modified to handle inhomogeneities but no
+   * entries from <tt>local_vector</tt> are added. Note that the edge integrals for inner
+   * edged for DG do not contribute any values to the right hand side.
+   */
+  template <typename VectorType, typename LocalType>
+  void
+  distribute_local_to_global (const Vector<LocalType>      &local_vector,
+                              const std::vector<size_type> &local_dof_indices_row,
+                              const std::vector<size_type> &local_dof_indices_col,
+                              VectorType                   &global_vector,
+                              const FullMatrix<LocalType>  &local_matrix,
+                              bool diagonal = false) const;
+
   /**
    * Enter a single value into a result vector, obeying constraints.
    */
