@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -52,11 +52,11 @@ template <typename number> class LAPACKFullMatrix;
  * matrix number type is used.
  *
  * @note Instantiations for this template are provided for <tt>@<float@>,
- * @<double@>, @<long double@>, @<std::complex@<float@>@>,
- * @<std::complex@<double@>@>, @<std::complex@<long double@>@></tt>; others
- * can be generated in application programs (see the section on
+ * @<double@>, @<std::complex@<float@>@>,
+ * @<std::complex@<double@>@></tt>. Others can be generated in application
+ * programs, see
  * @ref Instantiations
- * in the manual).
+ * for details.
  *
  * @author Guido Kanschat, Franz-Theo Suttmeier, Wolfgang Bangerth, 1993-2004
  */
@@ -226,17 +226,6 @@ public:
   FullMatrix (const size_type rows,
               const size_type cols);
 
-  /**
-   * Copy constructor. This constructor does a deep copy of the matrix.
-   * Therefore, it poses a possible efficiency problem, if for example,
-   * function arguments are passed by value rather than by reference.
-   * Unfortunately, we can't mark this copy constructor <tt>explicit</tt>,
-   * since that prevents the use of this class in containers, such as
-   * <tt>std::vector</tt>. The responsibility to check performance of programs
-   * must therefore remain with the user of this class.
-   */
-  FullMatrix (const FullMatrix &);
-
   /**
    * Constructor initializing from an array of numbers. The array is arranged
    * line by line. No range checking is performed.
@@ -265,14 +254,6 @@ public:
    * @{
    */
 
-  /**
-   * Assignment operator.
-   *
-   * @dealiiOperationIsMultithreaded
-   */
-  FullMatrix<number> &
-  operator = (const FullMatrix<number> &);
-
   /**
    * Variable assignment operator.
    */
@@ -466,14 +447,14 @@ public:
   bool operator == (const FullMatrix<number> &) const;
 
   /**
-   * Number of rows of this matrix.  To remember: this matrix is an <i>m x
-   * n</i>-matrix.
+   * Number of rows of this matrix.  Note that the matrix is of dimension <i>m
+   * x n</i>.
    */
   size_type m () const;
 
   /**
-   * Number of columns of this matrix.  To remember: this matrix is an <i>m x
-   * n</i>-matrix.
+   * Number of columns of this matrix.  Note that the matrix is of dimension
+   * <i>m x n</i>.
    */
   size_type n () const;
 
@@ -547,7 +528,7 @@ public:
   real_type relative_symmetry_norm2 () const;
 
   /**
-   * Computes the determinant of a matrix.  This is only implemented for one,
+   * Compute the determinant of a matrix.  This is only implemented for one,
    * two, and three dimensions, since for higher dimensions the numerical work
    * explodes.  Obviously, the matrix needs to be quadratic for this function.
    */
