@@ -1,7 +1,6 @@
 // ---------------------------------------------------------------------
-// $Id: householder.h 30036 2013-07-18 16:55:32Z maier $
 //
-// Copyright (C) 2005 - 2013 by the deal.II authors
+// Copyright (C) 2005 - 2015 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef __deal2__householder_h
-#define __deal2__householder_h
+#ifndef dealii__householder_h
+#define dealii__householder_h
 
 
 #include <cmath>
@@ -40,15 +39,16 @@ template<typename number> class Vector;
 /**
  * QR-decomposition of a full matrix.
  *
- * Whenever an object of this class is created, it copies the matrix
- * given and computes its QR-decomposition by Householder
- * algorithm. Then, the function least_squares() can be used to
- * compute the vector <i>x</i> minimizing <i>||Ax-b||</i> for a given
- * vector <i>b</i>.
+ * Whenever an object of this class is created, it copies the matrix given and
+ * computes its QR-decomposition by Householder algorithm. Then, the function
+ * least_squares() can be used to compute the vector <i>x</i> minimizing
+ * <i>||Ax-b||</i> for a given vector <i>b</i>.
  *
  * @note Instantiations for this template are provided for <tt>@<float@> and
  * @<double@></tt>; others can be generated in application programs (see the
- * section on @ref Instantiations in the manual).
+ * section on
+ * @ref Instantiations
+ * in the manual).
  *
  * @author Guido Kanschat, 2005
  */
@@ -67,66 +67,53 @@ public:
   Householder ();
 
   /**
-   * Create an object holding the
-   * QR-decomposition of a matrix.
+   * Create an object holding the QR-decomposition of a matrix.
    */
   template<typename number2>
   Householder (const FullMatrix<number2> &);
 
   /**
-   * Compute the QR-decomposition
-   * of another matrix.
+   * Compute the QR-decomposition of another matrix.
    */
   template<typename number2>
   void
   initialize (const FullMatrix<number2> &);
 
   /**
-   * Solve the least-squares
-   * problem for the right hand
-   * side <tt>src</tt>. The return
-   * value is the Euclidean norm of
-   * the approximation error.
+   * Solve the least-squares problem for the right hand side <tt>src</tt>. The
+   * return value is the Euclidean norm of the approximation error.
    *
-   * @arg @c dst contains the
-   * solution of the least squares
-   * problem on return.
+   * @arg @c dst contains the solution of the least squares problem on return.
    *
-   * @arg @c src contains the right
-   * hand side <i>b</i> of the
-   * least squares problem. It will
-   * be changed during the algorithm
-   * and is unusable on return.
+   * @arg @c src contains the right hand side <i>b</i> of the least squares
+   * problem. It will be changed during the algorithm and is unusable on
+   * return.
    */
   template<typename number2>
   double least_squares (Vector<number2> &dst,
                         const Vector<number2> &src) const;
 
   /**
-   * This function does the same as
-   * the one for BlockVectors.
+   * This function does the same as the one for BlockVectors.
    */
   template<typename number2>
   double least_squares (BlockVector<number2> &dst,
                         const BlockVector<number2> &src) const;
 
   /**
-   * A wrapper to least_squares(),
-   * implementing the standard
-   * MATRIX interface.
+   * A wrapper to least_squares(), implementing the standard MatrixType
+   * interface.
    */
-  template<class VECTOR>
-  void vmult (VECTOR &dst, const VECTOR &src) const;
+  template<class VectorType>
+  void vmult (VectorType &dst, const VectorType &src) const;
 
-  template<class VECTOR>
-  void Tvmult (VECTOR &dst, const VECTOR &src) const;
+  template<class VectorType>
+  void Tvmult (VectorType &dst, const VectorType &src) const;
 
 
 private:
   /**
-   * Storage for the diagonal
-   * elements of the orthogonal
-   * transformation.
+   * Storage for the diagonal elements of the orthogonal transformation.
    */
   std::vector<number> diagonal;
 };
@@ -185,7 +172,7 @@ Householder<number>::initialize(const FullMatrix<number2> &M)
 
 
       // For all subsequent columns do
-      // the Householder reflexion
+      // the Householder reflection
       for (size_type k=j+1 ; k<n ; ++k)
         {
           number2 sum = diagonal[j]*this->el(j,k);
@@ -243,7 +230,7 @@ Householder<number>::least_squares (Vector<number2> &dst,
   number2 sum = 0.;
   for (size_type i=n ; i<m ; ++i)
     sum += (*aux)(i) * (*aux)(i);
-  Assert(numbers::is_finite(sum), ExcNumberNotFinite());
+  AssertIsFinite(sum);
 
   // Compute solution
   this->backward(dst, *aux);
@@ -288,7 +275,7 @@ Householder<number>::least_squares (BlockVector<number2> &dst,
   number2 sum = 0.;
   for (size_type i=n ; i<m ; ++i)
     sum += (*aux)(i) * (*aux)(i);
-  Assert(numbers::is_finite(sum), ExcNumberNotFinite());
+  AssertIsFinite(sum);
 
   //backward works for
   //Vectors only, so copy
@@ -309,18 +296,18 @@ Householder<number>::least_squares (BlockVector<number2> &dst,
 
 
 template <typename number>
-template <class VECTOR>
+template <class VectorType>
 void
-Householder<number>::vmult (VECTOR &dst, const VECTOR &src) const
+Householder<number>::vmult (VectorType &dst, const VectorType &src) const
 {
   least_squares (dst, src);
 }
 
 
 template <typename number>
-template <class VECTOR>
+template <class VectorType>
 void
-Householder<number>::Tvmult (VECTOR &, const VECTOR &) const
+Householder<number>::Tvmult (VectorType &, const VectorType &) const
 {
   Assert(false, ExcNotImplemented());
 }
@@ -332,4 +319,3 @@ Householder<number>::Tvmult (VECTOR &, const VECTOR &) const
 DEAL_II_NAMESPACE_CLOSE
 
 #endif
-
