@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2005 - 2015 by the deal.II authors
+// Copyright (C) 2005 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -121,6 +121,18 @@ public:
   LAPACKFullMatrix<number> &
   operator = (const double d);
 
+  /**
+   * This operator multiplies all entries by a fixed factor.
+   */
+  LAPACKFullMatrix<number> &
+  operator*= (const number factor);
+
+  /**
+   * This operator divides all entries by a fixed factor.
+   */
+  LAPACKFullMatrix<number> &
+  operator/= (const number factor);
+
   /**
    * Assignment from different matrix classes, performing the usual conversion
    * to the transposed format expected by LAPACK. This assignment operator
@@ -580,6 +592,10 @@ public:
    *
    * @arg <tt>threshold</tt>: all entries with absolute value smaller than
    * this are considered zero.
+   *
+   * @note The entries stored resemble a matrix only if the state is either
+   * LAPACKSupport::matrix or LAPACK::inverse_matrix. Otherwise, calling this
+   * function is not allowed.
    */
   void print_formatted (std::ostream       &out,
                         const unsigned int  precision   = 3,
