@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2015 by the deal.II authors
+// Copyright (C) 2004 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -206,6 +206,19 @@ namespace PETScWrappers
     void Tvmult (Vector       &dst,
                  const Vector &src) const;
 
+    /**
+     * Return the partitioning of the domain space of this matrix, i.e., the
+     * partitioning of the vectors this matrix has to be multiplied with.
+     */
+    std::vector< size_type > locally_owned_domain_sizes() const;
+
+    /**
+     * Return the partitioning of the range space of this matrix, i.e., the
+     * partitioning of the vectors that are result from matrix-vector
+     * products.
+     */
+    std::vector< size_type > locally_owned_range_sizes() const;
+
     /**
      * Make the clear() function in the base class visible, though it is
      * protected.
