@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2016 by the deal.II authors
+// Copyright (C) 2004 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -26,6 +26,7 @@
 #  include <deal.II/lac/block_indices.h>
 #  include <deal.II/lac/block_vector_base.h>
 #  include <deal.II/lac/exceptions.h>
+#  include <deal.II/lac/vector_type_traits.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -222,9 +223,10 @@ namespace PETScWrappers
      * same number of blocks. If needed, the numbers of blocks should be
      * exchanged, too.
      *
-     * This function is analog to the the swap() function of all C++ standard
-     * containers. Also, there is a global function swap(u,v) that simply
-     * calls <tt>u.swap(v)</tt>, again in analogy to standard functions.
+     * This function is analogous to the the swap() function of all C++
+     * standard containers. Also, there is a global function swap(u,v) that
+     * simply calls <tt>u.swap(v)</tt>, again in analogy to standard
+     * functions.
      */
     void swap (BlockVector &v);
 
@@ -459,6 +461,54 @@ namespace PETScWrappers
 }
 
 
+namespace internal
+{
+  namespace LinearOperator
+  {
+    template <typename> class ReinitHelper;
+
+    /**
+     * A helper class used internally in linear_operator.h. Specialization for
+     * PETScWrappers::BlockVector.
+     */
+    template<>
+    class ReinitHelper<PETScWrappers::BlockVector>
+    {
+    public:
+      template <typename Matrix>
+      static
+      void reinit_range_vector (const Matrix &matrix,
+                                PETScWrappers::BlockVector &v,
+                                bool omit_zeroing_entries)
+      {
+        v.reinit(matrix.locally_owned_range_sizes(), omit_zeroing_entries);
+      }
+
+      template <typename Matrix>
+      static
+      void reinit_domain_vector(const Matrix &matrix,
+                                PETScWrappers::BlockVector &v,
+                                bool omit_zeroing_entries)
+      {
+        v.reinit(matrix.locally_owned_domain_sizes(), omit_zeroing_entries);
+      }
+    };
+
+  } /* namespace LinearOperator */
+} /* namespace internal */
+
+
+/**
+ * Declare dealii::PETScWrappers::BlockVector as serial vector.
+ *
+ * @author Uwe Koecher, 2017
+ */
+template <>
+struct is_serial_vector< PETScWrappers::BlockVector > : std_cxx11::true_type
+{
+};
+
+
 DEAL_II_NAMESPACE_CLOSE
 
 #endif  // DEAL_II_WITH_PETSC
