@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2016 by the deal.II authors
+// Copyright (C) 2004 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -22,8 +22,9 @@
 #ifdef DEAL_II_WITH_PETSC
 
 #  include <deal.II/base/subscriptor.h>
-#  include <deal.II/lac/full_matrix.h>
 #  include <deal.II/lac/exceptions.h>
+#  include <deal.II/lac/full_matrix.h>
+#  include <deal.II/lac/petsc_compatibility.h>
 #  include <deal.II/lac/vector.h>
 
 #  include <petscmat.h>
@@ -81,11 +82,6 @@ namespace PETScWrappers
                   const size_type   row,
                   const size_type   index);
 
-        /**
-         * Copy constructor.
-         */
-        Accessor (const Accessor &a);
-
         /**
          * Row number of the element represented by this object.
          */
@@ -701,12 +697,22 @@ namespace PETScWrappers
      */
     MatrixBase &operator /= (const PetscScalar factor);
 
+
     /**
      * Add the matrix @p other scaled by the factor @p factor to the current
      * matrix.
      */
+    MatrixBase &add (const PetscScalar factor,
+                     const MatrixBase &other);
+
+
+    /**
+     * Add the matrix @p other scaled by the factor @p factor to the current
+     * matrix.
+     * @deprecated Use the function with order of arguments reversed instead.
+     */
     MatrixBase &add (const MatrixBase &other,
-                     const PetscScalar factor);
+                     const PetscScalar factor) DEAL_II_DEPRECATED;
 
     /**
      * Matrix-vector multiplication: let <i>dst = M*src</i> with <i>M</i>
@@ -831,11 +837,7 @@ namespace PETScWrappers
      * Test whether a matrix is symmetric.  Default tolerance is
      * $1000\times32$-bit machine precision.
      */
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-    PetscTruth
-#else
-    PetscBool
-#endif
+    PetscBooleanType
     is_symmetric (const double tolerance = 1.e-12);
 
     /**
@@ -843,19 +845,14 @@ namespace PETScWrappers
      * its transpose. Default tolerance is $1000\times32$-bit machine
      * precision.
      */
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-    PetscTruth
-#else
-    PetscBool
-#endif
+    PetscBooleanType
     is_hermitian (const double tolerance = 1.e-12);
 
     /**
      * Print the PETSc matrix object values using PETSc internal matrix viewer
      * function <tt>MatView</tt>. The default format prints the non- zero
      * matrix elements. For other valid view formats, consult
-     * http://www.mcs.anl.gov/petsc/petsc-
-     * current/docs/manualpages/Mat/MatView.html
+     * http://www.mcs.anl.gov/petsc/petsc-current/docs/manualpages/Mat/MatView.html
      */
     void write_ascii (const PetscViewerFormat format = PETSC_VIEWER_DEFAULT);
 
@@ -870,17 +867,10 @@ namespace PETScWrappers
                 const bool    alternative_output = false) const;
 
     /**
-     * Returns the number bytes consumed by this matrix on this CPU.
+     * Return the number bytes consumed by this matrix on this CPU.
      */
     std::size_t memory_consumption() const;
 
-    /**
-     * Exception
-     */
-    DeclException1 (ExcPETScError,
-                    int,
-                    << "An error with error number " << arg1
-                    << " occurred while calling a PETSc function");
     /**
      * Exception
      */
@@ -1004,17 +994,6 @@ namespace PETScWrappers
     }
 
 
-    inline
-    const_iterator::Accessor::
-    Accessor (const Accessor &a)
-      :
-      matrix(a.matrix),
-      a_row(a.a_row),
-      a_index(a.a_index),
-      colnum_cache (a.colnum_cache),
-      value_cache (a.value_cache)
-    {}
-
 
     inline
     const_iterator::Accessor::size_type
@@ -1276,9 +1255,9 @@ namespace PETScWrappers
         col_value_ptr = &column_values[0];
       }
 
-    const int ierr
-      = MatSetValues (matrix, 1, &petsc_i, n_columns, col_index_ptr,
-                      col_value_ptr, INSERT_VALUES);
+    const PetscErrorCode ierr = MatSetValues (matrix, 1, &petsc_i, n_columns,
+                                              col_index_ptr,
+                                              col_value_ptr, INSERT_VALUES);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
@@ -1418,9 +1397,9 @@ namespace PETScWrappers
         col_value_ptr = &column_values[0];
       }
 
-    const int ierr
-      = MatSetValues (matrix, 1, &petsc_i, n_columns, col_index_ptr,
-                      col_value_ptr, ADD_VALUES);
+    const PetscErrorCode ierr = MatSetValues (matrix, 1, &petsc_i, n_columns,
+                                              col_index_ptr, col_value_ptr,
+                                              ADD_VALUES);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
@@ -1493,8 +1472,8 @@ namespace PETScWrappers
   {
     PetscInt begin, end;
 
-    const int ierr = MatGetOwnershipRange (static_cast<const Mat &>(matrix),
-                                           &begin, &end);
+    const PetscErrorCode ierr = MatGetOwnershipRange (static_cast<const Mat &>(matrix),
+                                                      &begin, &end);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return ((index >= static_cast<size_type>(begin)) &&
