@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2015 by the deal.II authors
+// Copyright (C) 2004 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -149,8 +149,8 @@ namespace PETScWrappers
        * the BlockSparsityPattern of the Simple type can efficiently store
        * large sparsity patterns in parallel, so this is the only supported
        * argument. The IndexSets describe the locally owned range of DoFs for
-       * each block. Note that each IndexSet needs to be contiguous. For a
-       * symmetric structure hand in the same vector for the first two
+       * each block. Note that the IndexSets needs to be ascending and 1:1.
+       * For a symmetric structure hand in the same vector for the first two
        * arguments.
        */
       void reinit(const std::vector<IndexSet> &rows,
@@ -234,6 +234,19 @@ namespace PETScWrappers
        */
       void collect_sizes ();
 
+      /**
+       * Return the partitioning of the domain space of this matrix, i.e., the
+       * partitioning of the vectors this matrix has to be multiplied with.
+       */
+      std::vector< IndexSet > locally_owned_domain_indices() const;
+
+      /**
+       * Return the partitioning of the range space of this matrix, i.e., the
+       * partitioning of the vectors that are result from matrix-vector
+       * products.
+       */
+      std::vector< IndexSet > locally_owned_range_indices() const;
+
       /**
        * Return a reference to the MPI communicator object in use with this
        * matrix.
