@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2015 by the deal.II authors
+// Copyright (C) 2004 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -101,7 +101,7 @@ namespace PETScWrappers
      *
      * The only way to avoid this is to tell PETSc where the actual entries of
      * the matrix will be. For this, there are constructors and reinit()
-     * functions of this class that take a CompressedSparsityPattern object
+     * functions of this class that take a DynamicSparsityPattern object
      * containing all this information. While in the general case it is
      * sufficient if the constructors and reinit() functions know the number
      * of local rows and columns, the functions getting a sparsity pattern
@@ -317,7 +317,7 @@ namespace PETScWrappers
        * Create a matrix where the size() of the IndexSets determine the
        * global number of rows and columns and the entries of the IndexSet
        * give the rows and columns for the calling processor. Note that only
-       * contiguous IndexSets are supported.
+       * ascending, 1:1 IndexSets are supported.
        */
       template <typename SparsityPatternType>
       void reinit (const IndexSet            &local_rows,
@@ -379,6 +379,19 @@ namespace PETScWrappers
       PetscScalar matrix_scalar_product (const Vector &u,
                                          const Vector &v) const;
 
+      /**
+       * Return the partitioning of the domain space of this matrix, i.e., the
+       * partitioning of the vectors this matrix has to be multiplied with.
+       */
+      IndexSet locally_owned_domain_indices() const;
+
+      /**
+       * Return the partitioning of the range space of this matrix, i.e., the
+       * partitioning of the vectors that result from matrix-vector
+       * products.
+       */
+      IndexSet locally_owned_range_indices() const;
+
     private:
 
       /**
