@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2016 by the deal.II authors
+// Copyright (C) 2004 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -26,16 +26,11 @@
 #  include <deal.II/lac/vector.h>
 #  include <deal.II/lac/petsc_vector_base.h>
 #  include <deal.II/base/index_set.h>
+#  include <deal.II/lac/vector_type_traits.h>
 
 DEAL_II_NAMESPACE_OPEN
 
 
-
-// forward declaration
-template <typename> class Vector;
-class IndexSet;
-
-
 /*! @addtogroup PETScWrappers
  *@{
  */
@@ -121,17 +116,16 @@ namespace PETScWrappers
      * @code
      *   PETScWrappers::MPI::Vector vector;
      *   ...
-     *                   // do some write operations on the vector
+     *   // do some write operations on the vector
      *   for (unsigned int i=0; i<vector.size(); ++i)
      *     vector(i) = i;
      *
-     *                   // do some additions to vector elements, but
-     *                   // only for some elements
+     *   // do some additions to vector elements, but only for some elements
      *   for (unsigned int i=0; i<vector.size(); ++i)
      *     if (some_condition(i) == true)
      *       vector(i) += 1;
      *
-     *                   // do another collective operation
+     *   // do another collective operation
      *   const double norm = vector.l2_norm();
      * @endcode
      *
@@ -177,8 +171,11 @@ namespace PETScWrappers
        *
        * For the current class, the variable equals true, since it does
        * support parallel data storage.
+       *
+       * @deprecated instead of using this variable, please use the type trait
+       * value <code>is_serial_vector< VectorType >::value</code>
        */
-      static const bool supports_distributed_data = true;
+      static const bool supports_distributed_data DEAL_II_DEPRECATED = true;
 
       /**
        * Default constructor. Initialize the vector as empty.
@@ -230,16 +227,20 @@ namespace PETScWrappers
        *
        * @arg communicator denotes the MPI communicator over which the
        * different parts of the vector shall communicate
+       *
+       * @deprecated The use of objects that are explicitly of type VectorBase
+       * is deprecated: use PETScWrappers::MPI::Vector instead.
        */
       explicit Vector (const MPI_Comm     &communicator,
                        const VectorBase   &v,
-                       const size_type     local_size);
+                       const size_type     local_size) DEAL_II_DEPRECATED;
 
       /**
-       * Constructs a new parallel ghosted PETSc vector from an IndexSet. Note
-       * that @p local must be contiguous and the global size of the vector is
-       * determined by local.size(). The global indices in @p ghost are
-       * supplied as ghost indices that can also be read locally.
+       * Construct a new parallel ghosted PETSc vector from an IndexSet.
+       * Note that @p local must be ascending and 1:1.
+       * The global size of the vector is determined by local.size().
+       * The global indices in @p ghost are supplied as ghost indices
+       * that can also be read locally.
        *
        * Note that the @p ghost IndexSet may be empty and that any indices
        * already contained in @p local are ignored during construction. That
@@ -256,7 +257,7 @@ namespace PETScWrappers
               const MPI_Comm &communicator);
 
       /**
-       * Constructs a new parallel PETSc vector from an IndexSet. This creates
+       * Construct a new parallel PETSc vector from an IndexSet. This creates
        * a non ghosted vector.
        */
       explicit Vector (const IndexSet &local,
@@ -345,7 +346,7 @@ namespace PETScWrappers
                    const bool    omit_zeroing_entries = false);
 
       /**
-       * Reinit as a vector without ghost elements. See the constructor with
+       * Reinit as a vector with ghost elements. See the constructor with
        * same signature for more details.
        *
        * @see
@@ -486,15 +487,15 @@ namespace PETScWrappers
       if (v.size()==0)
         {
           // this happens if v has not been initialized to something useful:
-          // Vector x,v;x=v;
+          //   Vector x,v;
+          //   x=v;
           // we skip the code below and create a simple serial vector of
           // length 0
 
-          int ierr;
 #if DEAL_II_PETSC_VERSION_LT(3,2,0)
-          ierr = VecDestroy (vector);
+          PetscErrorCode ierr = VecDestroy (vector);
 #else
-          ierr = VecDestroy (&vector);
+          PetscErrorCode ierr = VecDestroy (&vector);
 #endif
           AssertThrow (ierr == 0, ExcPETScError(ierr));
 
@@ -516,13 +517,11 @@ namespace PETScWrappers
             reinit (v.communicator, v.size(), v.local_size(), true);
         }
 
-      const int ierr = VecCopy (v.vector, vector);
+      PetscErrorCode ierr = VecCopy (v.vector, vector);
       AssertThrow (ierr == 0, ExcPETScError(ierr));
 
       if (has_ghost_elements())
         {
-          int ierr;
-
           ierr = VecGhostUpdateBegin(vector, INSERT_VALUES, SCATTER_FORWARD);
           AssertThrow (ierr == 0, ExcPETScError(ierr));
           ierr = VecGhostUpdateEnd(vector, INSERT_VALUES, SCATTER_FORWARD);
@@ -588,8 +587,56 @@ namespace PETScWrappers
   }
 }
 
+namespace internal
+{
+  namespace LinearOperator
+  {
+    template <typename> class ReinitHelper;
+
+    /**
+     * A helper class used internally in linear_operator.h. Specialization for
+     * PETScWrappers::MPI::Vector.
+     */
+    template<>
+    class ReinitHelper<PETScWrappers::MPI::Vector>
+    {
+    public:
+      template <typename Matrix>
+      static
+      void reinit_range_vector (const Matrix &matrix,
+                                PETScWrappers::MPI::Vector &v,
+                                bool /*omit_zeroing_entries*/)
+      {
+        v.reinit(matrix.locally_owned_range_indices(), matrix.get_mpi_communicator());
+      }
+
+      template <typename Matrix>
+      static
+      void reinit_domain_vector(const Matrix &matrix,
+                                PETScWrappers::MPI::Vector &v,
+                                bool /*omit_zeroing_entries*/)
+      {
+        v.reinit(matrix.locally_owned_domain_indices(), matrix.get_mpi_communicator());
+      }
+    };
+
+  } /* namespace LinearOperator */
+} /* namespace internal */
+
 /**@}*/
 
+
+/**
+ * Declare dealii::PETScWrappers::MPI::Vector as distributed vector.
+ *
+ * @author Uwe Koecher, 2017
+ */
+template <>
+struct is_serial_vector< PETScWrappers::MPI::Vector > : std_cxx11::false_type
+{
+};
+
+
 DEAL_II_NAMESPACE_CLOSE
 
 #endif // DEAL_II_WITH_PETSC
