@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2016 by the deal.II authors
+// Copyright (C) 2004 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -26,6 +26,7 @@
 #  include <deal.II/lac/petsc_vector_base.h>
 #  include <deal.II/lac/petsc_parallel_vector.h>
 #  include <deal.II/lac/vector.h>
+#  include <deal.II/lac/vector_type_traits.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -67,8 +68,11 @@ namespace PETScWrappers
      * For the current class, the variable equals false, since it does not
      * support parallel data storage. If you do need parallel data storage,
      * use PETScWrappers::MPI::Vector.
+     *
+     * @deprecated instead of using this variable, please use the type trait
+     * value <code>is_serial_vector< VectorType >::value</code>
      */
-    static const bool supports_distributed_data = false;
+    static const bool supports_distributed_data DEAL_II_DEPRECATED = false;
 
     /**
      * Default constructor. Initialize the vector as empty.
@@ -249,7 +253,7 @@ namespace PETScWrappers
     if (size() != v.size())
       reinit (v.size(), true);
 
-    const int ierr = VecCopy (v.vector, vector);
+    const PetscErrorCode ierr = VecCopy (v.vector, vector);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return *this;
@@ -261,7 +265,7 @@ namespace PETScWrappers
   Vector &
   Vector::operator = (const MPI::Vector &v)
   {
-    int ierr;
+    PetscErrorCode ierr;
     if (attained_ownership)
       {
         // the petsc function we call wants to
@@ -391,6 +395,56 @@ namespace PETScWrappers
 
 }
 
+namespace internal
+{
+  namespace LinearOperator
+  {
+    template <typename> class ReinitHelper;
+
+    /**
+     * A helper class used internally in linear_operator.h. Specialization for
+     * PETScWrappers::Vector.
+     */
+    template<>
+    class ReinitHelper<PETScWrappers::Vector>
+    {
+    public:
+      template <typename Matrix>
+      static
+      void reinit_range_vector (const Matrix &matrix,
+                                PETScWrappers::Vector &v,
+                                bool omit_zeroing_entries)
+      {
+        v.reinit( matrix.m(),
+                  omit_zeroing_entries);
+      }
+
+      template <typename Matrix>
+      static
+      void reinit_domain_vector(const Matrix &matrix,
+                                PETScWrappers::Vector &v,
+                                bool omit_zeroing_entries)
+      {
+        v.reinit( matrix.n(),
+                  omit_zeroing_entries);
+      }
+    };
+
+  } /* namespace LinearOperator */
+} /* namespace internal */
+
+
+/**
+ * Declare dealii::PETScWrappers::Vector as serial vector.
+ *
+ * @author Uwe Koecher, 2017
+ */
+template <>
+struct is_serial_vector< PETScWrappers::Vector > : std_cxx11::true_type
+{
+};
+
+
 DEAL_II_NAMESPACE_CLOSE
 
 #endif // DEAL_II_WITH_PETSC
