@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2015 by the deal.II authors
+// Copyright (C) 2004 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -154,14 +154,6 @@ namespace PETScWrappers
        * the referenced element of the vector.
        */
       operator PetscScalar () const;
-
-      /**
-       * Exception
-       */
-      DeclException1 (ExcPETScError,
-                      int,
-                      << "An error with error number " << arg1
-                      << " occurred while calling a PETSc function");
       /**
        * Exception
        */
@@ -473,6 +465,10 @@ namespace PETScWrappers
 
     /**
      * $l_1$-norm of the vector. The sum of the absolute values.
+     *
+     * @note In complex-valued PETSc priori to 3.7.0 this norm is implemented
+     * as the sum of absolute values of real and imaginary parts of elements
+     * of a complex vector.
      */
     real_type l1_norm () const;
 
@@ -704,8 +700,8 @@ namespace PETScWrappers
      * Prints the PETSc vector object values using PETSc internal vector
      * viewer function <tt>VecView</tt>. The default format prints the
      * vector's contents, including indices of vector elements. For other
-     * valid view formats, consult http://www.mcs.anl.gov/petsc/petsc-
-     * current/docs/manualpages/Vec/VecView.html
+     * valid view formats, consult
+     * http://www.mcs.anl.gov/petsc/petsc-current/docs/manualpages/Vec/VecView.html
      */
     void write_ascii (const PetscViewerFormat format = PETSC_VIEWER_DEFAULT) ;
 
@@ -728,10 +724,10 @@ namespace PETScWrappers
      * only swaps the pointers to the data of the two vectors and therefore
      * does not need to allocate temporary storage and move data around.
      *
-     * This function is analog to the the @p swap function of all C++ standard
-     * containers. Also, there is a global function <tt>swap(u,v)</tt> that
-     * simply calls <tt>u.swap(v)</tt>, again in analogy to standard
-     * functions.
+     * This function is analogous to the the @p swap function of all C++
+     * standard containers. Also, there is a global function
+     * <tt>swap(u,v)</tt> that simply calls <tt>u.swap(v)</tt>, again in
+     * analogy to standard functions.
      */
     void swap (VectorBase &v);
 
@@ -805,7 +801,13 @@ namespace PETScWrappers
                                const PetscScalar  *values,
                                const bool         add_values);
 
-
+  private:
+    /**
+     * Assignment operator. This is currently not implemented, so it is
+     * deliberately left as private (and undefined) to prevent accidental
+     * usage.
+     */
+    VectorBase &operator=(const VectorBase &);
   };
 
 
@@ -882,8 +884,8 @@ namespace PETScWrappers
 
       const PetscInt petsc_i = index;
 
-      const int ierr
-        = VecSetValues (vector, 1, &petsc_i, &value, INSERT_VALUES);
+      const PetscErrorCode ierr = VecSetValues (vector, 1, &petsc_i, &value,
+                                                INSERT_VALUES);
       AssertThrow (ierr == 0, ExcPETScError(ierr));
 
       vector.last_action = VectorOperation::insert;
@@ -919,8 +921,8 @@ namespace PETScWrappers
 
       // use the PETSc function to add something
       const PetscInt petsc_i = index;
-      const int ierr
-        = VecSetValues (vector, 1, &petsc_i, &value, ADD_VALUES);
+      const PetscErrorCode ierr = VecSetValues (vector, 1, &petsc_i, &value,
+                                                ADD_VALUES);
       AssertThrow (ierr == 0, ExcPETScError(ierr));
 
 
@@ -957,8 +959,8 @@ namespace PETScWrappers
       // add something
       const PetscInt petsc_i = index;
       const PetscScalar subtractand = -value;
-      const int ierr
-        = VecSetValues (vector, 1, &petsc_i, &subtractand, ADD_VALUES);
+      const PetscErrorCode ierr = VecSetValues (vector, 1, &petsc_i, &subtractand,
+                                                ADD_VALUES);
       AssertThrow (ierr == 0, ExcPETScError(ierr));
 
       return *this;
@@ -994,8 +996,8 @@ namespace PETScWrappers
       const PetscScalar new_value
         = static_cast<PetscScalar>(*this) * value;
 
-      const int ierr
-        = VecSetValues (vector, 1, &petsc_i, &new_value, INSERT_VALUES);
+      const PetscErrorCode ierr = VecSetValues (vector, 1, &petsc_i, &new_value,
+                                                INSERT_VALUES);
       AssertThrow (ierr == 0, ExcPETScError(ierr));
 
       return *this;
@@ -1031,8 +1033,8 @@ namespace PETScWrappers
       const PetscScalar new_value
         = static_cast<PetscScalar>(*this) / value;
 
-      const int ierr
-        = VecSetValues (vector, 1, &petsc_i, &new_value, INSERT_VALUES);
+      const PetscErrorCode ierr = VecSetValues (vector, 1, &petsc_i, &new_value,
+                                                INSERT_VALUES);
       AssertThrow (ierr == 0, ExcPETScError(ierr));
 
       return *this;
@@ -1071,8 +1073,8 @@ namespace PETScWrappers
   VectorBase::in_local_range (const size_type index) const
   {
     PetscInt begin, end;
-    const int ierr = VecGetOwnershipRange (static_cast<const Vec &>(vector),
-                                           &begin, &end);
+    const PetscErrorCode ierr = VecGetOwnershipRange (static_cast<const Vec &>(vector),
+                                                      &begin, &end);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return ((index >= static_cast<size_type>(begin)) &&
@@ -1167,9 +1169,6 @@ namespace PETScWrappers
     // with a parallel vector
     if (ghosted )
       {
-
-        int ierr;
-
         // there is the possibility
         // that the vector has
         // ghost elements. in that
@@ -1188,7 +1187,7 @@ namespace PETScWrappers
         // position we can get from
         // an index set
         PetscInt begin, end;
-        ierr = VecGetOwnershipRange (vector, &begin, &end);
+        PetscErrorCode ierr = VecGetOwnershipRange (vector, &begin, &end);
         AssertThrow (ierr == 0, ExcPETScError(ierr));
 
         Vec locally_stored_elements = PETSC_NULL;
@@ -1235,10 +1234,8 @@ namespace PETScWrappers
     // element we are interested in
     else
       {
-        int ierr;
-
         PetscInt begin, end;
-        ierr = VecGetOwnershipRange (vector, &begin, &end);
+        PetscErrorCode ierr = VecGetOwnershipRange (vector, &begin, &end);
         AssertThrow (ierr == 0, ExcPETScError(ierr));
 
         PetscScalar *ptr;
