@@ -27,9 +27,6 @@
 
 DEAL_II_NAMESPACE_OPEN
 
-template<typename MatrixType, typename inverse_type>
-class PreconditionBlockJacobi;
-
 /*! @addtogroup Preconditioners
  *@{
  */
@@ -215,8 +212,7 @@ protected:
   /**
    * Replacement of invert_diagblocks() for permuted preconditioning.
    */
-  void invert_permuted_diagblocks(const std::vector<size_type> &permutation,
-                                  const std::vector<size_type> &inverse_permutation);
+  void invert_permuted_diagblocks();
 public:
   /**
    * Deletes the inverse diagonal block matrices if existent, sets the
@@ -226,7 +222,7 @@ public:
   void clear();
 
   /**
-   * Checks whether the object is empty.
+   * Check whether the object is empty.
    */
   bool empty () const;
 
@@ -501,6 +497,7 @@ public:
   /**
    * import functions from private base class
    */
+  using typename PreconditionBlock<MatrixType,inverse_type>::AdditionalData;
   using PreconditionBlock<MatrixType, inverse_type>::initialize;
   using PreconditionBlock<MatrixType, inverse_type>::clear;
   using PreconditionBlock<MatrixType, inverse_type>::empty;
