@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2015 by the deal.II authors
+// Copyright (C) 1999 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -54,7 +54,8 @@ template <typename MatrixType, typename inverse_type>
 PreconditionBlock<MatrixType,inverse_type>::PreconditionBlock (bool store)
   : PreconditionBlockBase<inverse_type>(store),
     blocksize(0),
-    A(0, typeid(*this).name())
+    A(0, typeid(*this).name()),
+    relaxation(1.0)
 {}
 
 
@@ -93,7 +94,7 @@ void PreconditionBlock<MatrixType,inverse_type>::initialize
   if (parameters.invert_diagonal)
     {
       if (permutation.size() == M.m())
-        invert_permuted_diagblocks(permutation, inverse_permutation);
+        invert_permuted_diagblocks();
       else
         invert_diagblocks();
     }
@@ -103,18 +104,16 @@ void PreconditionBlock<MatrixType,inverse_type>::initialize
 template <typename MatrixType, typename inverse_type>
 void PreconditionBlock<MatrixType,inverse_type>::initialize
 (const MatrixType             &M,
- const std::vector<size_type> &permutation,
- const std::vector<size_type> &inverse_permutation,
+ const std::vector<size_type> &new_permutation,
+ const std::vector<size_type> &new_inverse_permutation,
  const AdditionalData          parameters)
 {
-  set_permutation(permutation, inverse_permutation);
+  set_permutation(new_permutation, new_inverse_permutation);
   initialize(M, parameters);
 }
 
 template <typename MatrixType, typename inverse_type>
-void PreconditionBlock<MatrixType,inverse_type>::invert_permuted_diagblocks
-(const std::vector<size_type> &permutation,
- const std::vector<size_type> &inverse_permutation)
+void PreconditionBlock<MatrixType,inverse_type>::invert_permuted_diagblocks()
 {
   Assert (A!=0, ExcNotInitialized());
   Assert (blocksize!=0, ExcNotInitialized());
@@ -128,8 +127,6 @@ void PreconditionBlock<MatrixType,inverse_type>::invert_permuted_diagblocks
 
   if (this->same_diagonal())
     {
-      deallog << "PreconditionBlock uses only one diagonal block" << std::endl;
-
       for (size_type row_cell=0; row_cell<blocksize; ++row_cell)
         {
           typename MatrixType::const_iterator entry = M.begin(row_cell);
@@ -438,7 +435,6 @@ void PreconditionBlock<MatrixType,inverse_type>::invert_diagblocks()
 
   if (this->same_diagonal())
     {
-      deallog << "PreconditionBlock uses only one diagonal block" << std::endl;
       for (size_type row_cell=0; row_cell<blocksize; ++row_cell)
         {
           typename MatrixType::const_iterator entry = M.begin(row_cell);
@@ -519,23 +515,19 @@ void PreconditionBlock<MatrixType,inverse_type>::invert_diagblocks()
 
 template <typename MatrixType, typename inverse_type>
 void PreconditionBlock<MatrixType,inverse_type>::set_permutation
-(const std::vector<size_type> &p,
- const std::vector<size_type> &i)
+(const std::vector<size_type> &new_permutation,
+ const std::vector<size_type> &new_inverse_permutation)
 {
-  Assert (p.size() == i.size(), ExcDimensionMismatch(p.size(), i.size()));
+  AssertDimension (new_permutation.size(),
+                   new_inverse_permutation.size());
 
   if (this->inverses_ready())
     {
-      AssertDimension(p.size(), this->size());
+      AssertDimension(new_permutation.size(), this->size());
     }
 
-  permutation.resize(p.size());
-  inverse_permutation.resize(p.size());
-  for (unsigned int k=0; k<p.size(); ++k)
-    {
-      permutation[k] = p[k];
-      inverse_permutation[k] = i[k];
-    }
+  permutation = new_permutation;
+  inverse_permutation = new_inverse_permutation;
 }
 
 
