@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2016 by the deal.II authors
+// Copyright (C) 1998 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -214,12 +214,6 @@ public:
     const bool every_iteration=false);
 
 protected:
-  /**
-   * Implementation of the computation of the norm of the residual. This can
-   * be replaced by a more problem oriented functional in a derived class.
-   */
-  virtual double criterion();
-
   /**
    * Interface for derived class. This function gets the current iteration
    * vector, the residual and the update vector in each step. It can be used
@@ -254,14 +248,6 @@ protected:
   VectorType *Vp;
   VectorType *Vz;
 
-  /**
-   * Within the iteration loop, the square of the residual vector is stored in
-   * this variable. The function @p criterion uses this variable to compute
-   * the convergence value, which in this class is the norm of the residual
-   * vector and thus the square root of the @p res2 value.
-   */
-  double res2;
-
   /**
    * Additional parameters.
    */
@@ -341,6 +327,9 @@ SolverCG<VectorType>::SolverCG (SolverControl        &cn,
                                 const AdditionalData     &data)
   :
   Solver<VectorType>(cn,mem),
+  Vr(NULL),
+  Vp(NULL),
+  Vz(NULL),
   additional_data(data)
 {}
 
@@ -351,6 +340,9 @@ SolverCG<VectorType>::SolverCG (SolverControl        &cn,
                                 const AdditionalData &data)
   :
   Solver<VectorType>(cn),
+  Vr(NULL),
+  Vp(NULL),
+  Vz(NULL),
   additional_data(data)
 {}
 
@@ -362,15 +354,6 @@ SolverCG<VectorType>::~SolverCG ()
 
 
 
-template <typename VectorType>
-double
-SolverCG<VectorType>::criterion()
-{
-  return std::sqrt(res2);
-}
-
-
-
 template <typename VectorType>
 void
 SolverCG<VectorType>::cleanup()
@@ -469,13 +452,12 @@ SolverCG<VectorType>::solve (const MatrixType         &A,
   // Should we build the matrix for
   // eigenvalue computations?
   const bool do_eigenvalues = !condition_number_signal.empty()
-                              |!all_condition_numbers_signal.empty()
-                              |!eigenvalues_signal.empty()
-                              |!all_eigenvalues_signal.empty()
-                              | additional_data.compute_condition_number
-                              | additional_data.compute_all_condition_numbers
-                              | additional_data.compute_eigenvalues;
-  double eigen_beta_alpha = 0;
+                              ||!all_condition_numbers_signal.empty()
+                              ||!eigenvalues_signal.empty()
+                              ||!all_eigenvalues_signal.empty()
+                              || additional_data.compute_condition_number
+                              || additional_data.compute_all_condition_numbers
+                              || additional_data.compute_eigenvalues;
 
   // vectors used for eigenvalue
   // computations
@@ -487,6 +469,8 @@ SolverCG<VectorType>::solve (const MatrixType         &A,
 
   try
     {
+      double eigen_beta_alpha = 0;
+
       // define some aliases for simpler access
       VectorType &g = *Vr;
       VectorType &d = *Vz;
@@ -498,7 +482,7 @@ SolverCG<VectorType>::solve (const MatrixType         &A,
       d.reinit(x, true);
       h.reinit(x, true);
 
-      double gh,alpha,beta;
+      double gh,beta;
 
       // compute residual. if vector is
       // zero, then short-circuit the
@@ -538,7 +522,7 @@ SolverCG<VectorType>::solve (const MatrixType         &A,
           it++;
           A.vmult(h,d);
 
-          alpha = d*h;
+          double alpha = d*h;
           Assert(alpha != 0., ExcDivideByZero());
           alpha = gh/alpha;
 
