@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2015 by the deal.II authors
+// Copyright (C) 1998 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -99,7 +99,7 @@ public:
       : last_step (last_step), last_residual(last_residual)
     {}
 
-    virtual ~NoConvergence () throw () {}
+    virtual ~NoConvergence () DEAL_II_NOEXCEPT {}
 
     virtual void print_info (std::ostream &out) const
     {
@@ -277,7 +277,7 @@ public:
   void log_history (const bool);
 
   /**
-   * Returns the @p log_history flag.
+   * Return the @p log_history flag.
    */
   bool log_history () const;
 
@@ -292,7 +292,7 @@ public:
   void log_result (const bool);
 
   /**
-   * Returns the @p log_result flag.
+   * Return the @p log_result flag.
    */
   bool log_result () const;
 
