@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2016 by the deal.II authors
+// Copyright (C) 1998 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -83,6 +83,13 @@ namespace internal
       VectorType &operator() (const unsigned int i,
                               const VectorType   &temp);
 
+      /**
+       * Return size of data vector. It is used in the solver to store
+       * the Arnoldi vectors.
+       */
+      unsigned int size() const;
+
+
     private:
       /**
        * Pool were vectors are obtained from.
@@ -288,6 +295,28 @@ public:
     const std_cxx11::function<void (const std::vector<std::complex<double> > &)> &slot,
     const bool every_iteration=false);
 
+  /**
+   * Connect a slot to retrieve the Hessenberg matrix obtained by the
+   * projection of the initial matrix on the Krylov basis. Called on each
+   * outer iteration if every_iteration=true, otherwise called once when
+   * iterations are ended (i.e., either because convergence has been achieved,
+   * or because divergence has been detected).
+   */
+  boost::signals2::connection
+  connect_hessenberg_slot(
+    const std_cxx11::function<void (const FullMatrix<double> &)> &slot,
+    const bool every_iteration=true);
+
+  /**
+   * Connect a slot to retrieve the basis vectors of the Krylov space
+   * generated by the Arnoldi algorithm. Called at once when iterations
+   * are completed (i.e., either because convergence has been achieved,
+   * or because divergence has been detected).
+   */
+  boost::signals2::connection
+  connect_krylov_space_slot(
+    const std_cxx11::function<void (const internal::SolverGMRES::TmpVectors<VectorType> &)> &slot);
+
 
   DeclException1 (ExcTooFewTmpVectors,
                   int,
@@ -326,6 +355,23 @@ protected:
    */
   boost::signals2::signal<void (const std::vector<std::complex<double> > &)> all_eigenvalues_signal;
 
+  /**
+   * Signal used to retrieve the Hessenberg matrix. Called once when
+   * all iterations are ended.
+   */
+  boost::signals2::signal<void (const FullMatrix<double> &)> hessenberg_signal;
+
+  /**
+   * Signal used to retrieve the Hessenberg matrix. Called on each outer
+   * iteration.
+   */
+  boost::signals2::signal<void (const FullMatrix<double> &)> all_hessenberg_signal;
+
+  /**
+   * Signal used to retrieve the Krylov space basis vectors. Called once
+   * when all iterations are ended.
+   */
+  boost::signals2::signal<void (const internal::SolverGMRES::TmpVectors<VectorType> &)> krylov_space_signal;
 
   /**
    * Implementation of the computation of the norm of the residual.
@@ -369,6 +415,7 @@ protected:
     const FullMatrix<double> &H_orig ,
     const unsigned int dim,
     const boost::signals2::signal<void (const std::vector<std::complex<double> > &)> &eigenvalues_signal,
+    const boost::signals2::signal<void (const FullMatrix<double> &)> &hessenberg_signal,
     const boost::signals2::signal<void(double)> &cond_signal,
     const bool log_eigenvalues);
 
@@ -537,6 +584,15 @@ namespace internal
       return *data[i-offset];
     }
 
+
+    template <class VectorType>
+    unsigned int
+    TmpVectors<VectorType>::size() const
+    {
+      return (data.size() > 0 ? data.size()-1 : 0);
+    }
+
+
     // A comparator for better printing eigenvalues
     inline
     bool complex_less_pred(const std::complex<double> &x,
@@ -701,16 +757,19 @@ SolverGMRES<VectorType>::compute_eigs_and_cond
 (const FullMatrix<double>                     &H_orig,
  const unsigned int                           dim,
  const boost::signals2::signal<void (const std::vector<std::complex<double> > &)> &eigenvalues_signal,
+ const boost::signals2::signal<void (const FullMatrix<double> &)> &hessenberg_signal,
  const boost::signals2::signal<void (double)> &cond_signal,
  const bool                                   log_eigenvalues)
 {
   //Avoid copying the Hessenberg matrix if it isn't needed.
-  if (!eigenvalues_signal.empty() || !cond_signal.empty() || log_eigenvalues )
+  if (!eigenvalues_signal.empty() || !hessenberg_signal.empty()
+      || !cond_signal.empty() || log_eigenvalues )
     {
       LAPACKFullMatrix<double> mat(dim,dim);
       for (unsigned int i=0; i<dim; ++i)
         for (unsigned int j=0; j<dim; ++j)
           mat(i,j) = H_orig(i,j);
+      hessenberg_signal(H_orig);
       //Avoid computing eigenvalues if they are not needed.
       if (!eigenvalues_signal.empty() || log_eigenvalues )
         {
@@ -774,10 +833,12 @@ SolverGMRES<VectorType>::solve (const MatrixType         &A,
 
   const bool do_eigenvalues=
     !condition_number_signal.empty()
-    |!all_condition_numbers_signal.empty()
-    |!eigenvalues_signal.empty()
-    |!all_eigenvalues_signal.empty()
-    |additional_data.compute_eigenvalues;
+    ||!all_condition_numbers_signal.empty()
+    ||!eigenvalues_signal.empty()
+    ||!all_eigenvalues_signal.empty()
+    ||!hessenberg_signal.empty()
+    ||!all_hessenberg_signal.empty()
+    ||additional_data.compute_eigenvalues;
   // for eigenvalue computation, need to collect the Hessenberg matrix (before
   // applying Givens rotations)
   FullMatrix<double> H_orig;
@@ -1003,6 +1064,7 @@ SolverGMRES<VectorType>::solve (const MatrixType         &A,
           H1(i,j) = H(i,j);
 
       compute_eigs_and_cond(H_orig,dim,all_eigenvalues_signal,
+                            all_hessenberg_signal,
                             all_condition_numbers_signal,
                             additional_data.compute_eigenvalues);
 
@@ -1024,8 +1086,13 @@ SolverGMRES<VectorType>::solve (const MatrixType         &A,
     }
   while (iteration_state == SolverControl::iterate);
 
-  compute_eigs_and_cond(H_orig,dim,eigenvalues_signal,condition_number_signal,
+  compute_eigs_and_cond(H_orig,dim,eigenvalues_signal,hessenberg_signal,
+                        condition_number_signal,
                         false);
+
+  if (!krylov_space_signal.empty())
+    krylov_space_signal(tmp_vectors);
+
   if (!use_default_residual)
     {
       this->memory.free(r);
@@ -1080,6 +1147,34 @@ SolverGMRES<VectorType>::connect_eigenvalues_slot
 
 
 
+template<class VectorType>
+boost::signals2::connection
+SolverGMRES<VectorType>::connect_hessenberg_slot
+(const std_cxx11::function<void (const FullMatrix<double> &)> &slot,
+ const bool every_iteration)
+{
+  if (every_iteration)
+    {
+      return all_hessenberg_signal.connect(slot);
+    }
+  else
+    {
+      return hessenberg_signal.connect(slot);
+    }
+}
+
+
+
+template<class VectorType>
+boost::signals2::connection
+SolverGMRES<VectorType>::connect_krylov_space_slot
+(const std_cxx11::function<void (const internal::SolverGMRES::TmpVectors<VectorType> &)> &slot)
+{
+  return krylov_space_signal.connect(slot);
+}
+
+
+
 template<class VectorType>
 double
 SolverGMRES<VectorType>::criterion ()
