@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2002 - 2015 by the deal.II authors
+// Copyright (C) 2002 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -31,6 +31,7 @@ template<typename number>
 SparseLUDecomposition<number>::SparseLUDecomposition()
   :
   SparseMatrix<number>(),
+  strengthen_diagonal (0),
   own_sparsity(0)
 {}
 
@@ -118,7 +119,8 @@ void SparseLUDecomposition<number>::initialize (
 
   // now use this sparsity pattern
   Assert (sparsity_pattern_to_use->n_rows()==sparsity_pattern_to_use->n_cols(),
-          typename SparsityPattern::ExcDiagonalNotOptimized());
+          ExcMessage ("It is not possible to compute this matrix decomposition for "
+                      "matrices that are not square."));
   {
     std::vector<const size_type *> tmp;
     tmp.swap (prebuilt_lower_bound);
