@@ -36,8 +36,9 @@
 DEAL_II_NAMESPACE_OPEN
 
 /**
- * This class provides an interface to the sparse direct solver UMFPACK (see
- * <a href="http://www.cise.ufl.edu/research/sparse/umfpack">this link</a>).
+ * This class provides an interface to the sparse direct solver UMFPACK, which
+ * is part of the SuiteSparse library (see <a
+ * href="http://faculty.cse.tamu.edu/davis/suitesparse.html">this link</a>).
  * UMFPACK is a set of routines for solving non-symmetric sparse linear
  * systems, Ax=b, using the Unsymmetric-pattern MultiFrontal method and direct
  * sparse LU factorization. Matrices may have symmetric or unsymmetric
@@ -63,7 +64,8 @@ DEAL_II_NAMESPACE_OPEN
  * @note UMFPACK has its own license, independent of that of deal.II. If you
  * want to use the UMFPACK you have to accept that license. It is linked to
  * from the deal.II ReadMe file. UMFPACK is included courtesy of its author,
- * <a href="http://www.cise.ufl.edu/~davis/">Timothy A. Davis</a>.
+ * <a href="http://faculty.cse.tamu.edu/davis/welcome.html">Timothy
+ * A. Davis</a>.
  *
  *
  * <h4>Instantiations</h4>
@@ -187,13 +189,13 @@ public:
                const BlockVector<double> &src) const;
 
   /**
-   * Return the dimension of the codomain (or range) space. To remember: the
+   * Return the dimension of the codomain (or range) space. Note that the
    * matrix is of dimension $m \times n$.
    */
   size_type m () const;
 
   /**
-   * Return the dimension of the domain space. To remember: the matrix is of
+   * Return the dimension of the domain space. Note that the matrix is of
    * dimension $m \times n$.
    */
   size_type n () const;
@@ -224,12 +226,14 @@ public:
    * If @p transpose is set to true this function solves for the transpose of
    * the matrix, i.e. $x=A^{-T}b$.
    */
-  void solve (Vector<double> &rhs_and_solution, bool transpose = false) const;
+  void solve (Vector<double> &rhs_and_solution,
+              const bool      transpose = false) const;
 
   /**
    * Same as before, but for block vectors.
    */
-  void solve (BlockVector<double> &rhs_and_solution, bool transpose = false) const;
+  void solve (BlockVector<double> &rhs_and_solution,
+              const bool           transpose = false) const;
 
   /**
    * Call the two functions factorize() and solve() in that order, i.e.
@@ -240,7 +244,7 @@ public:
   template <class Matrix>
   void solve (const Matrix   &matrix,
               Vector<double> &rhs_and_solution,
-              bool            transpose = false);
+              const bool      transpose = false);
 
   /**
    * Same as before, but for block vectors.
@@ -248,7 +252,7 @@ public:
   template <class Matrix>
   void solve (const Matrix        &matrix,
               BlockVector<double> &rhs_and_solution,
-              bool                 transpose = false);
+              const bool           transpose = false);
 
   /**
    * @}
