@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2015 by the deal.II authors
+// Copyright (C) 1999 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -62,27 +62,67 @@ SparseMatrix<number>::SparseMatrix (const SparseMatrix &m)
   val(0),
   max_len(0)
 {
-  Assert (m.cols==0, ExcInvalidConstructorCall());
-  Assert (m.val==0, ExcInvalidConstructorCall());
-  Assert (m.max_len==0, ExcInvalidConstructorCall());
+  Assert (m.cols==0 && m.val==0 && m.max_len==0,
+          ExcMessage("This constructor can only be called if the provided argument "
+                     "is an empty matrix. This constructor can not be used to "
+                     "copy-construct a non-empty matrix. Use the "
+                     "SparseMatrix::copy_from() function for that purpose."));
 }
 
 
 
+#ifdef DEAL_II_WITH_CXX11
+template <typename number>
+SparseMatrix<number>::SparseMatrix (SparseMatrix<number> &&m)
+  :
+  Subscriptor(std::move(m)),
+  cols(m.cols),
+  val(m.val),
+  max_len(m.max_len)
+{
+  m.cols = nullptr;
+  m.val = nullptr;
+  m.max_len = 0;
+}
+#endif
+
+
+
 template <typename number>
 SparseMatrix<number> &
 SparseMatrix<number>::operator = (const SparseMatrix<number> &m)
 {
   (void)m;
-  Assert (m.cols==0, ExcInvalidConstructorCall());
-  Assert (m.val==0, ExcInvalidConstructorCall());
-  Assert (m.max_len==0, ExcInvalidConstructorCall());
+  Assert (m.cols==0 && m.val==0 && m.max_len==0,
+          ExcMessage("This operator can only be called if the provided right "
+                     "hand side is an empty matrix. This operator can not be "
+                     "used to copy a non-empty matrix. Use the "
+                     "SparseMatrix::copy_from() function for that purpose."));
 
   return *this;
 }
 
 
 
+#ifdef DEAL_II_WITH_CXX11
+template <typename number>
+SparseMatrix<number> &
+SparseMatrix<number>::operator = (SparseMatrix<number> &&m)
+{
+  cols = m.cols;
+  val = m.val;
+  max_len = m.max_len;
+
+  m.cols = nullptr;
+  m.val = nullptr;
+  m.max_len = 0;
+
+  return *this;
+}
+#endif
+
+
+
 template <typename number>
 SparseMatrix<number>::SparseMatrix (const SparsityPattern &c)
   :
@@ -160,7 +200,7 @@ SparseMatrix<number>::operator = (const double d)
   // operator=. The grain size is chosen to reflect the number of rows in
   // minimum_parallel_grain_size, weighted by the number of nonzero entries
   // per row on average.
-  const size_type matrix_size = cols->n_nonzero_elements();
+  const std::size_t matrix_size = cols->n_nonzero_elements();
   const size_type grain_size =
     internal::SparseMatrix::minimum_parallel_grain_size *
     (cols->n_nonzero_elements()+m()) / m();
@@ -262,7 +302,7 @@ SparseMatrix<number>::get_row_length (const size_type row) const
 
 
 template <typename number>
-typename SparseMatrix<number>::size_type
+std::size_t
 SparseMatrix<number>::n_nonzero_elements () const
 {
   Assert (cols != 0, ExcNotInitialized());
@@ -272,13 +312,13 @@ SparseMatrix<number>::n_nonzero_elements () const
 
 
 template <typename number>
-typename SparseMatrix<number>::size_type
+std::size_t
 SparseMatrix<number>::n_actually_nonzero_elements (const double threshold) const
 {
   Assert (cols != 0, ExcNotInitialized());
   Assert (threshold >= 0, ExcMessage ("Negative threshold!"));
   size_type nnz = 0;
-  const size_type nnz_alloc = n_nonzero_elements();
+  const std::size_t nnz_alloc = n_nonzero_elements();
   for (size_type i=0; i<nnz_alloc; ++i)
     if (std::abs(val[i]) > threshold)
       ++nnz;
