@@ -374,13 +374,13 @@ public:
   bool empty () const;
 
   /**
-   * Return the dimension of the codomain (or range) space. To remember: the
+   * Return the dimension of the codomain (or range) space. Note that the
    * matrix is of dimension $m \times n$.
    */
   size_type m () const;
 
   /**
-   * Return the dimension of the domain space. To remember: the matrix is of
+   * Return the dimension of the domain space. Note that the matrix is of
    * dimension $m \times n$.
    */
   size_type n () const;
@@ -541,7 +541,8 @@ public:
    */
   template <typename MatrixType>
   SparseMatrixEZ<number> &
-  copy_from (const MatrixType &source, const bool elide_zero_values = true);
+  copy_from (const MatrixType &source,
+             const bool elide_zero_values = true);
 
   /**
    * Add @p matrix scaled by @p factor to this matrix.
