@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2015 by the deal.II authors
+// Copyright (C) 2000 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -223,7 +223,8 @@ void SwappableVector<number>::kill_file ()
   if (filename != "")
     {
       int status = std::remove (filename.c_str());
-      AssertThrow (status == 0, ExcInternalError());
+      (void)status;
+      AssertThrow (status == 0, ExcIO());
 
       filename = "";
     };
