@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2015 by the deal.II authors
+// Copyright (C) 2008 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -31,8 +31,6 @@
 
 #  include <cmath>
 
-#  define TrilinosScalar double
-
 DEAL_II_NAMESPACE_OPEN
 
 // forward declarations
@@ -200,7 +198,7 @@ namespace TrilinosWrappers
                  const double                               drop_tolerance=1e-13);
 
     /**
-     * Returns the state of the matrix, i.e., whether compress() needs to be
+     * Return the state of the matrix, i.e., whether compress() needs to be
      * called after an operation requiring data exchange. Does only return
      * non-true values when used in <tt>debug</tt> mode, since it is quite
      * expensive to keep track of all operations that lead to the need for
@@ -221,7 +219,8 @@ namespace TrilinosWrappers
     void collect_sizes ();
 
     /**
-     * Return the number of nonzero elements of this matrix.
+     * Return the total number of nonzero elements of this matrix (summed
+     * over all MPI processes).
      */
     size_type n_nonzero_elements () const;
 
@@ -564,7 +563,63 @@ namespace TrilinosWrappers
       BaseClass::vmult_nonblock_nonblock (dst, src);
   }
 
-}
+
+#ifdef DEAL_II_WITH_CXX11
+
+
+  namespace internal
+  {
+    namespace BlockLinearOperator
+    {
+
+      /**
+       * This is an extension class to BlockLinearOperators for Trilinos block
+       * sparse matrices.
+       *
+       * @note This class does very little at the moment other than to check
+       * that the correct Payload type for each subblock has been chosen
+       * correctly. Further extensions to the class may be necessary in the
+       * future in order to add further functionality to BlockLinearOperators
+       * while retaining compatability with the Trilinos sparse matrix and
+       * preconditioner classes.
+       *
+       * @author Jean-Paul Pelteret, 2016
+       *
+       * @ingroup TrilinosWrappers
+       */
+      template<typename PayloadBlockType>
+      class TrilinosBlockPayload
+      {
+      public:
+        /**
+         * Type of payload held by each subblock
+         */
+        typedef PayloadBlockType BlockType;
+
+        /**
+         * Default constructor
+         *
+         * This simply checks that the payload for each block has been chosen
+         * correctly (i.e. is of type TrilinosPayload). Apart from this, this
+         * class does not do anything in particular and needs no special
+         * configuration, we have only one generic constructor that can be
+         * called under any conditions.
+         */
+        template <typename... Args>
+        TrilinosBlockPayload (const Args &...)
+        {
+          static_assert(typeid(PayloadBlockType)==typeid(internal::LinearOperator::TrilinosPayload),
+                        "TrilinosBlockPayload can only accept a payload of type TrilinosPayload.");
+        }
+      };
+
+    } /*namespace BlockLinearOperator*/
+  } /* namespace internal */
+
+#endif // DEAL_II_WITH_CXX11
+
+}/* namespace TrilinosWrappers */
+
 
 DEAL_II_NAMESPACE_CLOSE
 
