@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2016 by the deal.II authors
+// Copyright (C) 2008 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -26,6 +26,7 @@
 #  include <deal.II/lac/block_indices.h>
 #  include <deal.II/lac/block_vector_base.h>
 #  include <deal.II/lac/exceptions.h>
+#  include <deal.II/lac/vector_type_traits.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -272,9 +273,10 @@ namespace TrilinosWrappers
      * same number of blocks. If needed, the numbers of blocks should be
      * exchanged, too.
      *
-     * This function is analog to the the swap() function of all C++ standard
-     * containers. Also, there is a global function swap(u,v) that simply
-     * calls <tt>u.swap(v)</tt>, again in analogy to standard functions.
+     * This function is analogous to the the swap() function of all C++
+     * standard containers. Also, there is a global function swap(u,v) that
+     * simply calls <tt>u.swap(v)</tt>, again in analogy to standard
+     * functions.
      */
     void swap (BlockVector &v);
 
@@ -464,7 +466,7 @@ namespace internal
     template <typename> class ReinitHelper;
 
     /**
-     * A helper class internally used in linear_operator.h. Specialization for
+     * A helper class used internally in linear_operator.h. Specialization for
      * TrilinosWrappers::BlockVector.
      */
     template<>
@@ -494,6 +496,28 @@ namespace internal
 } /* namespace internal */
 
 
+/**
+ * Declare dealii::TrilinosWrappers::BlockVector as serial vector.
+ *
+ * @author Uwe Koecher, 2017
+ */
+template <>
+struct is_serial_vector< TrilinosWrappers::BlockVector > : std_cxx11::true_type
+{
+};
+
+
+/**
+ * Declare dealii::TrilinosWrappers::MPI::BlockVector as distributed vector.
+ *
+ * @author Uwe Koecher, 2017
+ */
+template <>
+struct is_serial_vector< TrilinosWrappers::MPI::BlockVector > : std_cxx11::false_type
+{
+};
+
+
 DEAL_II_NAMESPACE_CLOSE
 
 #endif  // DEAL_II_WITH_TRILINOS
