@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2016 by the deal.II authors
+// Copyright (C) 2008 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -25,7 +25,7 @@
 #  include <deal.II/base/std_cxx11/shared_ptr.h>
 
 #  include <deal.II/lac/trilinos_vector_base.h>
-#  include <deal.II/lac/parallel_vector.h>
+#  include <deal.II/lac/la_parallel_vector.h>
 
 DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #  ifdef DEAL_II_WITH_MPI
@@ -72,7 +72,8 @@ namespace TrilinosWrappers
    *
    * @ingroup TrilinosWrappers
    * @ingroup Preconditioners
-   * @author Martin Kronbichler, 2008
+   * @author Martin Kronbichler, 2008; extension for full compatibility with
+   * LinearOperator class: Jean-Paul Pelteret, 2015
    */
   class PreconditionBase : public Subscriptor
   {
@@ -112,6 +113,22 @@ namespace TrilinosWrappers
      */
     void clear ();
 
+    /**
+     * Return the MPI communicator object in use with this matrix.
+     */
+    MPI_Comm get_mpi_communicator () const;
+
+    /**
+     * Sets an internal flag so that all operations performed by the matrix,
+     * i.e., multiplications, are done in transposed order. However, this does
+     * not reshape the matrix to transposed form directly, so care should be
+     * taken when using this flag.
+     *
+     * @note Calling this function any even number of times in succession will
+     * return the object to its original state.
+     */
+    void transpose ();
+
     /**
      * Apply the preconditioner.
      */
@@ -142,25 +159,53 @@ namespace TrilinosWrappers
      * Apply the preconditioner on deal.II parallel data structures instead of
      * the ones provided in the Trilinos wrapper class.
      */
-    virtual void vmult (dealii::parallel::distributed::Vector<double>       &dst,
-                        const dealii::parallel::distributed::Vector<double> &src) const;
+    virtual void vmult (dealii::LinearAlgebra::distributed::Vector<double>       &dst,
+                        const dealii::LinearAlgebra::distributed::Vector<double> &src) const;
 
     /**
      * Apply the transpose preconditioner on deal.II parallel data structures
      * instead of the ones provided in the Trilinos wrapper class.
      */
-    virtual void Tvmult (dealii::parallel::distributed::Vector<double>       &dst,
-                         const dealii::parallel::distributed::Vector<double> &src) const;
+    virtual void Tvmult (dealii::LinearAlgebra::distributed::Vector<double>       &dst,
+                         const dealii::LinearAlgebra::distributed::Vector<double> &src) const;
 
     /**
-     * Return a reference to the underlaying Trilinos Epetra_Operator. So you
-     * can use the preconditioner with unwrapped Trilinos solver.
+     * @name Access to underlying Trilinos data
+     */
+//@{
+    /**
      *
      * Calling this function from an uninitialized object will cause an
      * exception.
      */
-    Epetra_Operator &trilinos_operator() const;
+    Epetra_Operator &trilinos_operator () const;
+    //@}
 
+    /**
+     * @name Partitioners
+     */
+//@{
+
+    /**
+     * Return the partitioning of the domain space of this matrix, i.e., the
+     * partitioning of the vectors this matrix has to be multiplied with.
+     */
+    IndexSet locally_owned_domain_indices() const;
+
+    /**
+     * Return the partitioning of the range space of this matrix, i.e., the
+     * partitioning of the vectors that are result from matrix-vector
+     * products.
+     */
+    IndexSet locally_owned_range_indices() const;
+
+//@}
+
+    /**
+     * @addtogroup Exceptions
+     *
+     */
+//@{
     /**
      * Exception.
      */
@@ -170,6 +215,7 @@ namespace TrilinosWrappers
                     << "uses a map that is not compatible to the one in vector "
                     << arg1
                     << ". Check preconditioner and matrix setup.");
+//@}
 
     friend class SolverBase;
 
@@ -507,11 +553,11 @@ namespace TrilinosWrappers
        * subdivision of the rows, set the damping parameter to one, and do not
        * modify the diagonal.
        */
-      AdditionalData (const unsigned int block_size = 1,
-                      const std::string  block_creation_type = "linear",
-                      const double       omega = 1,
-                      const double       min_diagonal = 0,
-                      const unsigned int n_sweeps = 1);
+      AdditionalData (const unsigned int  block_size          = 1,
+                      const std::string  &block_creation_type = "linear",
+                      const double        omega               = 1,
+                      const double        min_diagonal        = 0,
+                      const unsigned int  n_sweeps            = 1);
 
       /**
        * This specifies the size of blocks.
@@ -612,12 +658,12 @@ namespace TrilinosWrappers
        * run a BlockJacobi preconditioner, where each block is inverted
        * approximately by a block SOR).
        */
-      AdditionalData (const unsigned int block_size = 1,
-                      const std::string  block_creation_type = "linear",
-                      const double       omega = 1,
-                      const double       min_diagonal = 0,
-                      const unsigned int overlap = 0,
-                      const unsigned int n_sweeps = 1);
+      AdditionalData (const unsigned int  block_size          = 1,
+                      const std::string  &block_creation_type = "linear",
+                      const double        omega               = 1,
+                      const double        min_diagonal        = 0,
+                      const unsigned int  overlap             = 0,
+                      const unsigned int  n_sweeps            = 1);
 
       /**
        * This specifies the size of blocks.
@@ -725,12 +771,12 @@ namespace TrilinosWrappers
        * run a BlockJacobi preconditioner, where each block is inverted
        * approximately by a block SOR).
        */
-      AdditionalData (const unsigned int block_size = 1,
-                      const std::string  block_creation_type = "linear",
-                      const double       omega = 1,
-                      const double       min_diagonal = 0,
-                      const unsigned int overlap = 0,
-                      const unsigned int n_sweeps = 1);
+      AdditionalData (const unsigned int  block_size          = 1,
+                      const std::string  &block_creation_type = "linear",
+                      const double        omega               = 1,
+                      const double        min_diagonal        = 0,
+                      const unsigned int  overlap             = 0,
+                      const unsigned int  n_sweeps            = 1);
 
       /**
        * This specifies the size of blocks.
@@ -1572,9 +1618,10 @@ namespace TrilinosWrappers
    * except for the higher_order_elements parameter which does not exist in
    * PreconditionerAMGMueLu.
    *
-   * This class requires Trilinos 11.14 or higher.
+   * @note This class requires Trilinos 11.14 or higher. At the moment 64bit-indices
+   * are not supported.
    *
-   * This interface should not be considered as stable.
+   * @warning This interface should not be considered as stable.
    *
    * @ingroup TrilinosWrappers
    * @ingroup Preconditioners
@@ -1716,6 +1763,11 @@ namespace TrilinosWrappers
       const char *coarse_type;
     };
 
+    /**
+     * Constructor.
+     */
+    PreconditionAMGMueLu();
+
     /**
      * Destructor.
      */
@@ -1798,12 +1850,34 @@ namespace TrilinosWrappers
    *
    * @ingroup TrilinosWrappers
    * @ingroup Preconditioners
-   * @author Bruno Turcksin, 2013
+   * @author Bruno Turcksin, 2013; extension for full compatibility with
+   * LinearOperator class: Jean-Paul Pelteret, 2016
    */
   class PreconditionIdentity : public PreconditionBase
   {
   public:
 
+    /**
+     * This function is only present to provide the interface of a
+     * preconditioner to be handed to a smoother.  This does nothing.
+     */
+    struct AdditionalData
+    {
+      /**
+       * Constructor.
+       */
+      AdditionalData () {}
+    };
+
+    /**
+     * The matrix argument is ignored and here just for compatibility with more
+     * complex preconditioners.
+     * @note This function must be called when this preconditioner is to be
+     * wrapped in a LinearOperator without an exemplar materix.
+     */
+    void initialize (const SparseMatrix   &matrix,
+                     const AdditionalData &additional_data = AdditionalData());
+
     /**
      * Apply the preconditioner, i.e., dst = src.
      */
@@ -1834,16 +1908,16 @@ namespace TrilinosWrappers
      * Apply the preconditioner on deal.II parallel data structures instead of
      * the ones provided in the Trilinos wrapper class, i.e., dst = src.
      */
-    void vmult (parallel::distributed::Vector<double>       &dst,
-                const dealii::parallel::distributed::Vector<double> &src) const;
+    void vmult (LinearAlgebra::distributed::Vector<double>       &dst,
+                const dealii::LinearAlgebra::distributed::Vector<double> &src) const;
 
     /**
      * Apply the transpose preconditioner on deal.II parallel data structures
      * instead of the ones provided in the Trilinos wrapper class, i.e., dst =
      * src.
      */
-    void Tvmult (parallel::distributed::Vector<double>       &dst,
-                 const dealii::parallel::distributed::Vector<double> &src) const;
+    void Tvmult (LinearAlgebra::distributed::Vector<double>       &dst,
+                 const dealii::LinearAlgebra::distributed::Vector<double> &src) const;
   };
 
 
@@ -1853,6 +1927,31 @@ namespace TrilinosWrappers
 
 #ifndef DOXYGEN
 
+
+  inline
+  void
+  PreconditionBase::transpose ()
+  {
+    // This only flips a flag that tells
+    // Trilinos that any vmult operation
+    // should be done with the
+    // transpose. However, the matrix
+    // structure is not reset.
+    int ierr;
+
+    if (!preconditioner->UseTranspose())
+      {
+        ierr = preconditioner->SetUseTranspose (true);
+        AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+      }
+    else
+      {
+        ierr = preconditioner->SetUseTranspose (false);
+        AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+      }
+  }
+
+
   inline
   void
   PreconditionBase::vmult (VectorBase       &dst,
@@ -1936,8 +2035,8 @@ namespace TrilinosWrappers
 
   inline
   void
-  PreconditionBase::vmult (parallel::distributed::Vector<double>       &dst,
-                           const parallel::distributed::Vector<double> &src) const
+  PreconditionBase::vmult (LinearAlgebra::distributed::Vector<double>       &dst,
+                           const LinearAlgebra::distributed::Vector<double> &src) const
   {
     AssertDimension (static_cast<TrilinosWrappers::types::int_type>(dst.local_size()),
                      preconditioner->OperatorDomainMap().NumMyElements());
@@ -1954,8 +2053,8 @@ namespace TrilinosWrappers
 
   inline
   void
-  PreconditionBase::Tvmult (parallel::distributed::Vector<double>       &dst,
-                            const parallel::distributed::Vector<double> &src) const
+  PreconditionBase::Tvmult (LinearAlgebra::distributed::Vector<double>       &dst,
+                            const LinearAlgebra::distributed::Vector<double> &src) const
   {
     AssertDimension (static_cast<TrilinosWrappers::types::int_type>(dst.local_size()),
                      preconditioner->OperatorDomainMap().NumMyElements());
@@ -1972,14 +2071,6 @@ namespace TrilinosWrappers
     preconditioner->SetUseTranspose(false);
   }
 
-  inline
-  Epetra_Operator &
-  PreconditionBase::trilinos_operator () const
-  {
-    AssertThrow (preconditioner, ExcMessage("Trying to dereference a null pointer."));
-    return (*preconditioner);
-  }
-
 #endif
 
 }
