@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2015 by the deal.II authors
+// Copyright (C) 2008 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -25,7 +25,7 @@
 #  include <deal.II/lac/exceptions.h>
 #  include <deal.II/lac/solver_control.h>
 #  include <deal.II/lac/vector.h>
-#  include <deal.II/lac/parallel_vector.h>
+#  include <deal.II/lac/la_parallel_vector.h>
 
 DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #  include <Epetra_LinearProblem.h>
@@ -61,7 +61,8 @@ namespace TrilinosWrappers
    * ParameterList) and is similar to the deal.II class SolverSelector.
    *
    * @ingroup TrilinosWrappers
-   * @author Martin Kronbichler, 2008, 2009
+   * @author Martin Kronbichler, 2008, 2009; extension for full compatibility
+   * with LinearOperator class: Jean-Paul Pelteret, 2015
    */
   class SolverBase
   {
@@ -74,7 +75,29 @@ namespace TrilinosWrappers
      * one of the specialized derived classes when the solver should be set at
      * runtime. Currently enabled options are:
      */
-    enum SolverName {cg, cgs, gmres, bicgstab, tfqmr} solver_name;
+    enum SolverName
+    {
+      /**
+       * Use the conjugate gradient (CG) algorithm.
+       */
+      cg,
+      /**
+       * Use the conjugate gradient squared (CGS) algorithm.
+       */
+      cgs,
+      /**
+       * Use the generalized minimum residual (GMRES) algorithm.
+       */
+      gmres,
+      /**
+       * Use the biconjugate gradient stabilized (BICGStab) algorithm.
+       */
+      bicgstab,
+      /**
+       * Use the transpose-free quasi-minimal residual (TFQMR) method.
+       */
+      tfqmr
+    } solver_name;
 
     /**
      * Standardized data struct to pipe additional data to the solver.
@@ -83,7 +106,7 @@ namespace TrilinosWrappers
     struct AdditionalData
     {
       /**
-       * Sets the additional data field to the desired output format and puts
+       * Set the additional data field to the desired output format and puts
        * the restart parameter in case the derived class is GMRES.
        *
        * TODO: Find a better way for setting the GMRES restart parameter since
@@ -142,11 +165,59 @@ namespace TrilinosWrappers
      * Trilinos is chosen.
      */
     void
-    solve (Epetra_Operator        &A,
+    solve (const Epetra_Operator  &A,
            VectorBase             &x,
            const VectorBase       &b,
            const PreconditionBase &preconditioner);
 
+    /**
+     * Solve the linear system <tt>Ax=b</tt> where both <tt>A</tt> and its
+     * @p precondtioner are an operator.
+     * This function can be used when both <tt>A</tt> and the @p preconditioner
+     * are LinearOperators derived from a TrilinosPayload.
+     * Depending on the information provided by derived classes and the object
+     * passed as a preconditioner, one of the linear solvers and preconditioners
+     * of Trilinos is chosen.
+     */
+    void
+    solve (const Epetra_Operator  &A,
+           VectorBase             &x,
+           const VectorBase       &b,
+           const Epetra_Operator  &preconditioner);
+
+    /**
+     * Solve the linear system <tt>Ax=b</tt> where <tt>A</tt> is an operator,
+     * and the vectors @p x and @p b are native Trilinos vector types.
+     * This function can be used when <tt>A</tt> is a LinearOperators derived
+     * from a TrilinosPayload.
+     * Depending on the information provided by derived classes and the object
+     * passed as a preconditioner, one of the linear solvers and preconditioners
+     * of Trilinos is chosen.
+     */
+    void
+    solve (const Epetra_Operator    &A,
+           Epetra_MultiVector       &x,
+           const Epetra_MultiVector &b,
+           const PreconditionBase   &preconditioner);
+
+    /**
+     * Solve the linear system <tt>Ax=b</tt> where both <tt>A</tt> and its
+     * @p precondtioner are an operator, and the vectors @p x and @p b are
+     * native Trilinos vector types.
+     * This function can be used when both <tt>A</tt> and the @p preconditioner
+     * are LinearOperators derived from a TrilinosPayload.
+     * Depending on the information provided by derived classes and the object
+     * passed as a preconditioner, one of the linear solvers and preconditioners
+     * of Trilinos is chosen.
+     */
+    void
+    solve (const Epetra_Operator    &A,
+           Epetra_MultiVector       &x,
+           const Epetra_MultiVector &b,
+           const Epetra_Operator    &preconditioner);
+
+
+
     /**
      * Solve the linear system <tt>Ax=b</tt>. Depending on the information
      * provided by derived classes and the object passed as a preconditioner,
@@ -188,8 +259,8 @@ namespace TrilinosWrappers
      */
     void
     solve (const SparseMatrix                                  &A,
-           dealii::parallel::distributed::Vector<double>       &x,
-           const dealii::parallel::distributed::Vector<double> &b,
+           dealii::LinearAlgebra::distributed::Vector<double>       &x,
+           const dealii::LinearAlgebra::distributed::Vector<double> &b,
            const PreconditionBase                              &preconditioner);
 
     /**
@@ -201,8 +272,8 @@ namespace TrilinosWrappers
      */
     void
     solve (Epetra_Operator                                     &A,
-           dealii::parallel::distributed::Vector<double>       &x,
-           const dealii::parallel::distributed::Vector<double> &b,
+           dealii::LinearAlgebra::distributed::Vector<double>       &x,
+           const dealii::LinearAlgebra::distributed::Vector<double> &b,
            const PreconditionBase                              &preconditioner);
 
 
@@ -235,7 +306,15 @@ namespace TrilinosWrappers
      * The solve function is used to set properly the Epetra_LinearProblem,
      * once it is done this function solves the linear problem.
      */
-    void do_solve(const PreconditionBase &preconditioner);
+    template<typename Preconditioner>
+    void do_solve(const Preconditioner &preconditioner);
+
+    /**
+     * A function that sets the preconditioner that the solver will apply
+     */
+    template<typename Preconditioner>
+    void set_preconditioner (AztecOO              &solver,
+                             const Preconditioner &preconditioner);
 
     /**
      * A structure that collects the Trilinos sparse matrix, the right hand
@@ -244,6 +323,12 @@ namespace TrilinosWrappers
      */
     std_cxx11::shared_ptr<Epetra_LinearProblem> linear_problem;
 
+    /**
+     * A structure that contains a Trilinos object that can query the linear
+     * solver and determine whether the convergence criterion have been met.
+     */
+    std_cxx11::shared_ptr<AztecOO_StatusTest> status_test;
+
     /**
      * A structure that contains the Trilinos solver and preconditioner
      * objects.
@@ -254,10 +339,21 @@ namespace TrilinosWrappers
      * Store a copy of the flags for this particular solver.
      */
     const AdditionalData additional_data;
-
   };
 
 
+  // provide a declaration for two explicit specializations
+  template <>
+  void
+  SolverBase::set_preconditioner(AztecOO                &solver,
+                                 const PreconditionBase &preconditioner);
+
+  template <>
+  void
+  SolverBase::set_preconditioner(AztecOO               &solver,
+                                 const Epetra_Operator &preconditioner);
+
+
 
   /**
    * An implementation of the solver interface using the Trilinos CG solver.
@@ -275,7 +371,7 @@ namespace TrilinosWrappers
     struct AdditionalData
     {
       /**
-       * Sets the additional data field to the desired output format.
+       * Set the additional data field to the desired output format.
        */
       explicit
       AdditionalData (const bool output_solver_details = false);
@@ -321,7 +417,7 @@ namespace TrilinosWrappers
     struct AdditionalData
     {
       /**
-       * Sets the additional data field to the desired output format.
+       * Set the additional data field to the desired output format.
        */
       explicit
       AdditionalData (const bool output_solver_details = false);
@@ -421,7 +517,7 @@ namespace TrilinosWrappers
     struct AdditionalData
     {
       /**
-       * Sets the additional data field to the desired output format.
+       * Set the additional data field to the desired output format.
        */
       explicit
       AdditionalData (const bool output_solver_details = false);
@@ -468,7 +564,7 @@ namespace TrilinosWrappers
     struct AdditionalData
     {
       /**
-       * Sets the additional data field to the desired output format.
+       * Set the additional data field to the desired output format.
        */
       explicit
       AdditionalData (const bool output_solver_details = false);
@@ -523,7 +619,7 @@ namespace TrilinosWrappers
     struct AdditionalData
     {
       /**
-       * Sets the additional data field to the desired output format.
+       * Set the additional data field to the desired output format.
        */
       explicit
       AdditionalData (const bool output_solver_details = false,
@@ -567,6 +663,21 @@ namespace TrilinosWrappers
      */
     virtual ~SolverDirect ();
 
+    /**
+     * Initializes the direct solver for the matrix <tt>A</tt> and creates a
+     * factorization for it with the package chosen from the additional
+     * data structure. Note that there is no need for a preconditioner
+     * here and solve() is not called.
+     */
+    void initialize (const SparseMatrix &A);
+
+    /**
+     * Solve the linear system <tt>Ax=b</tt> based on the
+     * package set in intialize(). Note the matrix is not refactorized during
+     * this call.
+     */
+    void solve (VectorBase &x, const VectorBase &b);
+
     /**
      * Solve the linear system <tt>Ax=b</tt>. Creates a factorization of the
      * matrix with the package chosen from the additional data structure and
@@ -598,8 +709,8 @@ namespace TrilinosWrappers
      */
     void
     solve (const SparseMatrix                                  &A,
-           dealii::parallel::distributed::Vector<double>       &x,
-           const dealii::parallel::distributed::Vector<double> &b);
+           dealii::LinearAlgebra::distributed::Vector<double>       &x,
+           const dealii::LinearAlgebra::distributed::Vector<double> &b);
 
     /**
      * Access to object that controls convergence.
