@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2015 by the deal.II authors
+// Copyright (C) 2008 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -53,6 +53,7 @@ namespace TrilinosWrappers
 {
   // forward declarations
   class SparsityPattern;
+  class SparseMatrix;
 
   namespace SparsityPatternIterators
   {
@@ -87,11 +88,6 @@ namespace TrilinosWrappers
                 const size_type        row,
                 const size_type        index);
 
-      /**
-       * Copy constructor.
-       */
-      Accessor (const Accessor &a);
-
       /**
        * Row number of the element represented by this object.
        */
@@ -829,7 +825,7 @@ namespace TrilinosWrappers
 //@{
 
     /**
-     * Returns the state of the sparsity pattern, i.e., whether compress()
+     * Return the state of the sparsity pattern, i.e., whether compress()
      * needs to be called after an operation requiring data exchange.
      */
     bool is_compressed () const;
@@ -1164,7 +1160,7 @@ namespace TrilinosWrappers
      */
     std_cxx11::shared_ptr<Epetra_CrsGraph> nonlocal_graph;
 
-    friend class SparseMatrix;
+    friend class TrilinosWrappers::SparseMatrix;
     friend class SparsityPatternIterators::Accessor;
     friend class SparsityPatternIterators::Iterator;
   };
@@ -1192,15 +1188,6 @@ namespace TrilinosWrappers
     }
 
 
-    inline
-    Accessor::Accessor (const Accessor &a)
-      :
-      sparsity_pattern(a.sparsity_pattern),
-      a_row(a.a_row),
-      a_index(a.a_index),
-      colnum_cache (a.colnum_cache)
-    {}
-
 
     inline
     Accessor::size_type
