@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2016 by the deal.II authors
+// Copyright (C) 2008 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -28,6 +28,7 @@
 #  include <deal.II/lac/exceptions.h>
 #  include <deal.II/lac/vector.h>
 #  include <deal.II/lac/trilinos_vector_base.h>
+#  include <deal.II/lac/vector_type_traits.h>
 
 DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #  include "Epetra_Map.h"
@@ -267,8 +268,11 @@ namespace TrilinosWrappers
        *
        * For the current class, the variable equals true, since it does
        * support parallel data storage.
+       *
+       * @deprecated instead of using this variable, please use the type trait
+       * value <code>is_serial_vector< VectorType >::value</code>
        */
-      static const bool supports_distributed_data = true;
+      static const bool supports_distributed_data DEAL_II_DEPRECATED = true;
 
       /**
        * @name Basic constructors and initialization.
@@ -307,8 +311,12 @@ namespace TrilinosWrappers
        * dimension and the parallel distribution of the input vector, but does
        * not copy the elements in <tt>v</tt>. If <tt>omit_zeroing_entries</tt>
        * is not <tt>true</tt>, the elements in the vector are initialized with
-       * zero, otherwise the content will be left unchanged and the user has
-       * to set all elements.
+       * zero. If it is set to <tt>true</tt>, the vector entries are in an
+       * unspecified state and the user has to set all elements. In the
+       * current implementation, this method does not touch the vector entries
+       * in case the vector layout is unchanged from before, otherwise entries
+       * are set to zero.  Note that this behavior might change between
+       * releases without notification.
        *
        * This function has a third argument, <tt>allow_different_maps</tt>,
        * that allows for an exchange of data between two equal-sized vectors
@@ -507,6 +515,10 @@ namespace TrilinosWrappers
        * or may not have ghost elements. See the general documentation of this
        * class for more information.
        *
+       * In case the provided IndexSet forms an overlapping partitioning,
+       * it it not clear which elements are owned by which process and
+       * locally_owned_elements() will return an IndexSet of size zero.
+       *
        * @see
        * @ref GlossGhostedVector "vectors with ghost elements"
        */
@@ -567,7 +579,11 @@ namespace TrilinosWrappers
        * Reinit functionality. This function destroys the old vector content
        * and generates a new one based on the input partitioning.  The flag
        * <tt>omit_zeroing_entries</tt> determines whether the vector should be
-       * filled with zero (false) or left untouched (true).
+       * filled with zero (false). If the flag is set to <tt>true</tt>, the
+       * vector entries are in an unspecified state and the user has to set
+       * all elements. In the current implementation, this method still sets
+       * the entries to zero, but this might change between releases without
+       * notification.
        *
        *
        * Depending on whether the @p parallel_partitioning argument uniquely
@@ -575,6 +591,10 @@ namespace TrilinosWrappers
        * or may not have ghost elements. See the general documentation of this
        * class for more information.
        *
+       * In case @p parallel_partitioning is overlapping, it is not clear which
+       * process should own which elements. Hence, locally_owned_elements()
+       * returns an empty IndexSet in this case.
+       *
        * @see
        * @ref GlossGhostedVector "vectors with ghost elements"
        */
@@ -653,6 +673,7 @@ namespace TrilinosWrappers
     {
       *this = Vector(parallel_partitioner.make_trilinos_map (communicator, true),
                      v);
+      owned_elements = parallel_partitioner;
     }
 
 
@@ -663,9 +684,7 @@ namespace TrilinosWrappers
                          const dealii::Vector<number> &v)
     {
       if (vector.get() == 0 || vector->Map().SameAs(parallel_partitioner) == false)
-        vector.reset (new Epetra_FEVector(parallel_partitioner));
-
-      has_ghosts = vector->Map().UniqueGIDs()==false;
+        reinit(parallel_partitioner);
 
       const int size = parallel_partitioner.NumMyElements();
 
@@ -673,6 +692,7 @@ namespace TrilinosWrappers
       // that a direct access is not possible.
       for (int i=0; i<size; ++i)
         (*vector)[0][i] = v(gid(parallel_partitioner,i));
+
     }
 
 
@@ -702,7 +722,14 @@ namespace TrilinosWrappers
                                             )));
         }
 
-      reinit (vector_partitioner(), v);
+      const Epetra_Map &map = vector_partitioner();
+      const int size = map.NumMyElements();
+
+      // Need to copy out values, since the deal.II might not use doubles, so
+      // that a direct access is not possible.
+      for (int i=0; i<size; ++i)
+        (*vector)[0][i] = v(gid(map,i));
+
       return *this;
     }
 
@@ -743,8 +770,11 @@ namespace TrilinosWrappers
      * For the current class, the variable equals false, since it does not
      * support parallel data storage.  If you do need parallel data storage,
      * use TrilinosWrappers::MPI::Vector.
+     *
+     * @deprecated instead of using this variable, please use the type trait
+     * value <code>is_serial_vector< VectorType >::value</code>
      */
-    static const bool supports_distributed_data = false;
+    static const bool supports_distributed_data DEAL_II_DEPRECATED = false;
 
     /**
      * Default constructor that generates an empty (zero size) vector. The
@@ -796,7 +826,12 @@ namespace TrilinosWrappers
 
     /**
      * Reinit function that resizes the vector to the size specified by
-     * <tt>n</tt>.
+     * <tt>n</tt>. The flag <tt>omit_zeroing_entries</tt> specifies whether
+     * the vector entries should be initialized to zero (false). If it is set
+     * to <tt>true</tt>, the vector entries are in an unspecified state and
+     * the user has to set all elements. In the current implementation, this
+     * method still sets the entries to zero, but this might change between
+     * releases without notification.
      */
     void reinit (const size_type n,
                  const bool      omit_zeroing_entries = false);
@@ -808,7 +843,11 @@ namespace TrilinosWrappers
      * in the localized vector should be imported from a distributed vector
      * that has been initialized with the same communicator. The variable
      * <tt>omit_zeroing_entries</tt> determines whether the vector should be
-     * filled with zero or left untouched.
+     * filled with zero (false). If it is set to <tt>true</tt>, the vector
+     * entries are in an unspecified state and the user has to set all
+     * elements. In the current implementation, this method still sets the
+     * entries to zero, but this might change between releases without
+     * notification.
      *
      * Which element of the @p input_map argument are set is in fact ignored,
      * the only thing that matters is the size of the index space described by
@@ -824,7 +863,11 @@ namespace TrilinosWrappers
      * in the localized vector should be imported from a distributed vector
      * that has been initialized with the same communicator. The variable
      * <tt>omit_zeroing_entries</tt> determines whether the vector should be
-     * filled with zero (false) or left untouched (true).
+     * filled with zero (false). If it is set to <tt>true</tt>, the vector
+     * entries are in an unspecified state and the user has to set all
+     * elements. In the current implementation, this method still sets the
+     * entries to zero, but this might change between releases without
+     * notification.
      *
      * Which element of the @p input_map argument are set is in fact ignored,
      * the only thing that matters is the size of the index space described by
@@ -837,6 +880,15 @@ namespace TrilinosWrappers
     /**
      * Reinit function. Takes the information of a Vector and copies
      * everything to the calling vector, now also allowing different maps.
+     *
+     * If the optional flag <tt>omit_zeroing_entries</tt> is not
+     * <tt>true</tt>, the elements in the vector are initialized with zero. If
+     * it is set to <tt>true</tt>, the vector entries are in an unspecified
+     * state and the user has to set all elements. In the current
+     * implementation, this method does not touch the vector entries in case
+     * the vector layout is unchanged from before, otherwise entries are set
+     * to zero.  Note that this behavior might change between releases without
+     * notification.
      */
     void reinit (const VectorBase &V,
                  const bool        omit_zeroing_entries = false,
@@ -851,13 +903,13 @@ namespace TrilinosWrappers
     Vector &operator= (const TrilinosScalar s);
 
     /**
-     * Sets the left hand argument to the (parallel) Trilinos Vector.
+     * Set the left hand argument to the (parallel) Trilinos Vector.
      * Equivalent to the @p reinit function.
      */
     Vector &operator= (const MPI::Vector &v);
 
     /**
-     * Sets the left hand argument to the deal.II vector.
+     * Set the left hand argument to the deal.II vector.
      */
     template <typename Number>
     Vector &operator= (const ::dealii::Vector<Number> &v);
@@ -883,7 +935,6 @@ namespace TrilinosWrappers
   };
 
 
-
 // ------------------- inline and template functions --------------
 
 
@@ -936,6 +987,7 @@ namespace TrilinosWrappers
         Epetra_LocalMap map ((TrilinosWrappers::types::int_type)v.size(), 0,
                              Utilities::Trilinos::comm_self());
         vector.reset (new Epetra_FEVector(map));
+        owned_elements = v.locally_owned_elements();
       }
 
     const Epetra_Map &map = vector_partitioner();
@@ -975,7 +1027,7 @@ namespace internal
     template <typename> class ReinitHelper;
 
     /**
-     * A helper class internally used in linear_operator.h. Specialization for
+     * A helper class used internally in linear_operator.h. Specialization for
      * TrilinosWrappers::MPI::Vector.
      */
     template<>
@@ -1002,7 +1054,7 @@ namespace internal
     };
 
     /**
-     * A helper class internally used in linear_operator.h. Specialization for
+     * A helper class used internally in linear_operator.h. Specialization for
      * TrilinosWrappers::Vector.
      */
     template<>
@@ -1036,6 +1088,28 @@ namespace internal
 } /* namespace internal */
 
 
+/**
+ * Declare dealii::TrilinosWrappers::Vector as serial vector.
+ *
+ * @author Uwe Koecher, 2017
+ */
+template <>
+struct is_serial_vector< TrilinosWrappers::Vector > : std_cxx11::true_type
+{
+};
+
+
+/**
+ * Declare dealii::TrilinosWrappers::MPI::Vector as distributed vector.
+ *
+ * @author Uwe Koecher, 2017
+ */
+template <>
+struct is_serial_vector< TrilinosWrappers::MPI::Vector > : std_cxx11::false_type
+{
+};
+
+
 DEAL_II_NAMESPACE_CLOSE
 
 #endif // DEAL_II_WITH_TRILINOS
