@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2015 by the deal.II authors
+// Copyright (C) 2008 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -26,13 +26,13 @@
 #  include <deal.II/base/subscriptor.h>
 #  include <deal.II/lac/exceptions.h>
 #  include <deal.II/lac/vector.h>
+#  include <deal.II/base/mpi.h>
 
 #  include <vector>
 #  include <utility>
 #  include <memory>
 
 DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
-#  define TrilinosScalar double
 #  include "Epetra_ConfigDefs.h"
 #  ifdef DEAL_II_WITH_MPI // only if MPI is installed
 #    include "mpi.h"
@@ -53,6 +53,12 @@ template <typename number> class Vector;
  * @addtogroup TrilinosWrappers
  * @{
  */
+
+/**
+ * A namespace in which wrapper classes for Trilinos objects reside.
+ *
+ * @ingroup TrilinosWrappers
+ */
 namespace TrilinosWrappers
 {
   // forward declaration
@@ -113,7 +119,7 @@ namespace TrilinosWrappers
       /**
        * Same as above but for non-const reference objects.
        */
-      const VectorReference &
+      VectorReference &
       operator = (const VectorReference &r);
 
       /**
@@ -285,7 +291,7 @@ namespace TrilinosWrappers
     void compress (::dealii::VectorOperation::values operation);
 
     /**
-     * Returns the state of the vector, i.e., whether compress() has already
+     * Return the state of the vector, i.e., whether compress() has already
      * been called after an operation requiring data exchange.
      *
      * This function is deprecated.
@@ -849,10 +855,10 @@ namespace TrilinosWrappers
      * does not need to allocate temporary storage and move data around. Note
      * that the vectors need to be of the same size and base on the same map.
      *
-     * This function is analog to the the @p swap function of all C standard
-     * containers. Also, there is a global function <tt>swap(u,v)</tt> that
-     * simply calls <tt>u.swap(v)</tt>, again in analogy to standard
-     * functions.
+     * This function is analogous to the the @p swap function of all C++
+     * standard containers. Also, there is a global function
+     * <tt>swap(u,v)</tt> that simply calls <tt>u.swap(v)</tt>, again in
+     * analogy to standard functions.
      */
     void swap (VectorBase &v);
 
@@ -937,6 +943,11 @@ namespace TrilinosWrappers
      */
     std_cxx11::shared_ptr<Epetra_MultiVector> nonlocal_vector;
 
+    /**
+     * An IndexSet storing the indices this vector owns exclusively.
+     */
+    IndexSet owned_elements;
+
     /**
      * Make the reference class a friend.
      */
@@ -994,7 +1005,7 @@ namespace TrilinosWrappers
 
 
     inline
-    const VectorReference &
+    VectorReference &
     VectorReference::operator = (const VectorReference &r)
     {
       // as above
@@ -1082,27 +1093,11 @@ namespace TrilinosWrappers
   IndexSet
   VectorBase::locally_owned_elements() const
   {
-    IndexSet is (size());
-
-    // easy case: local range is contiguous
-    if (vector->Map().LinearMap())
-      {
-        const std::pair<size_type, size_type> x = local_range();
-        is.add_range (x.first, x.second);
-      }
-    else if (vector->Map().NumMyElements() > 0)
-      {
-        const size_type n_indices = vector->Map().NumMyElements();
-#ifndef DEAL_II_WITH_64BIT_INDICES
-        unsigned int *vector_indices = (unsigned int *)vector->Map().MyGlobalElements();
-#else
-        size_type *vector_indices = (size_type *)vector->Map().MyGlobalElements64();
-#endif
-        is.add_indices(vector_indices, vector_indices+n_indices);
-        is.compress();
-      }
-
-    return is;
+    Assert(owned_elements.size()==size(),
+           ExcMessage("The locally owned elements have not been properly initialized!"
+                      " This happens for example if this object has been initialized"
+                      " with exactly one overlapping IndexSet."));
+    return owned_elements;
   }
 
 
@@ -1218,6 +1213,8 @@ namespace TrilinosWrappers
 
     if (v.nonlocal_vector.get() != 0)
       nonlocal_vector.reset(new Epetra_MultiVector(v.nonlocal_vector->Map(), 1));
+
+    owned_elements = v.owned_elements;
   }
 
 
@@ -1228,12 +1225,14 @@ namespace TrilinosWrappers
   {
     AssertIsFinite(s);
 
-    const int ierr = vector->PutScalar(s);
-
+    int ierr = vector->PutScalar(s);
     AssertThrow (ierr == 0, ExcTrilinosError(ierr));
 
     if (nonlocal_vector.get() != 0)
-      nonlocal_vector->PutScalar(0.);
+      {
+        ierr = nonlocal_vector->PutScalar(0.);
+        AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+      }
 
     return *this;
   }
@@ -1285,7 +1284,10 @@ namespace TrilinosWrappers
     Assert (!has_ghost_elements(), ExcGhostsPresent());
 
     if (last_action == Add)
-      vector->GlobalAssemble(Add);
+      {
+        const int ierr = vector->GlobalAssemble(Add);
+        AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+      }
 
     if (last_action != Insert)
       last_action = Insert;
@@ -1359,7 +1361,10 @@ namespace TrilinosWrappers
     if (last_action != Add)
       {
         if (last_action == Insert)
-          vector->GlobalAssemble(Insert);
+          {
+            const int ierr = vector->GlobalAssemble(Insert);
+            AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+          }
         last_action = Add;
       }
 
