@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -22,7 +22,16 @@
 #include <deal.II/base/exceptions.h>
 #include <deal.II/base/subscriptor.h>
 #include <deal.II/base/index_set.h>
-#include <boost/serialization/array.hpp>
+#include <deal.II/lac/vector_type_traits.h>
+
+// boost::serialization::make_array used to be in array.hpp, but was
+// moved to a different file in BOOST 1.64
+#include <boost/version.hpp>
+#if BOOST_VERSION >= 106400
+#  include <boost/serialization/array_wrapper.hpp>
+#else
+#  include <boost/serialization/array.hpp>
+#endif
 #include <boost/serialization/split_member.hpp>
 
 #include <cstdio>
@@ -61,6 +70,13 @@ template <typename> class BlockVector;
 
 template <typename> class VectorView;
 
+namespace parallel
+{
+  namespace internal
+  {
+    class TBBPartitioner;
+  }
+}
 
 
 
@@ -81,7 +97,21 @@ template <typename> class VectorView;
  */
 struct VectorOperation
 {
-  enum values { unknown, insert, add };
+  enum values
+  {
+    /**
+     * The current operation is unknown.
+     */
+    unknown,
+    /**
+     * The current operation is an insertion.
+     */
+    insert,
+    /**
+     * The current operation is an addition.
+     */
+    add
+  };
 };
 
 
@@ -97,9 +127,8 @@ struct VectorOperation
  * suitable for numerical computations.
  *
  * @note Instantiations for this template are provided for <tt>@<float@>,
- * @<double@>, @<long double@>, @<std::complex@<float@>@>,
- * @<std::complex@<double@>@>, @<std::complex@<long double@>@></tt>; others
- * can be generated in application programs (see the section on
+ * @<double@>, @<std::complex@<float@>@>, @<std::complex@<double@>@></tt>;
+ * others can be generated in application programs (see the section on
  * @ref Instantiations
  * in the manual).
  *
@@ -141,8 +170,11 @@ public:
    *
    * For the current class, the variable equals false, since it does not
    * support parallel data storage.
+   *
+   * @deprecated instead of using this variable, please use the type trait value
+   * <code>is_serial_vector< VectorType >::value</code>
    */
-  static const bool supports_distributed_data = false;
+  static const bool supports_distributed_data DEAL_II_DEPRECATED = false;
 
 public:
 
@@ -200,7 +232,7 @@ public:
    * vector class. This copy constructor is only available if PETSc was
    * detected during configuration time.
    */
-  Vector (const PETScWrappers::Vector &v);
+  explicit Vector (const PETScWrappers::Vector &v);
 
   /**
    * Another copy constructor: copy the values from a parallel PETSc wrapper
@@ -212,7 +244,7 @@ public:
    * possible for only one process to obtain a copy of a parallel vector while
    * the other jobs do something else.
    */
-  Vector (const PETScWrappers::MPI::Vector &v);
+  explicit Vector (const PETScWrappers::MPI::Vector &v);
 #endif
 
 #ifdef DEAL_II_WITH_TRILINOS
@@ -227,14 +259,14 @@ public:
    * vector while the other jobs do something else. This call will rather
    * result in a copy of the vector on all processors.
    */
-  Vector (const TrilinosWrappers::MPI::Vector &v);
+  explicit Vector (const TrilinosWrappers::MPI::Vector &v);
 
   /**
    * Another copy constructor: copy the values from a localized Trilinos
    * wrapper vector. This copy constructor is only available if Trilinos was
    * detected during configuration time.
    */
-  Vector (const TrilinosWrappers::Vector &v);
+  explicit Vector (const TrilinosWrappers::Vector &v);
 #endif
 
   /**
@@ -314,9 +346,10 @@ public:
    * swaps the pointers to the data of the two vectors and therefore does not
    * need to allocate temporary storage and move data around.
    *
-   * This function is analog to the the @p swap function of all C++ standard
-   * containers. Also, there is a global function <tt>swap(u,v)</tt> that
-   * simply calls <tt>u.swap(v)</tt>, again in analogy to standard functions.
+   * This function is analogous to the the @p swap function of all C++
+   * standard containers. Also, there is a global function <tt>swap(u,v)</tt>
+   * that simply calls <tt>u.swap(v)</tt>, again in analogy to standard
+   * functions.
    *
    * This function is virtual in order to allow for derived classes to handle
    * memory separately.
@@ -347,8 +380,9 @@ public:
 
 #ifdef DEAL_II_WITH_CXX11
   /**
-   * Move the given vector. This operator replaces the present vector with @p
-   * v by efficiently swapping the internal data structures.
+   * Move the given vector. This operator replaces the present vector with
+   * the internal data of the vector @p v and resets @p v to the state it would
+   * have after being newly default-constructed.
    *
    * @note This operator is only available if deal.II is configured with C++11
    * support.
@@ -909,7 +943,7 @@ public:
   //@{
 
   /**
-   * Returns true if the given global index is in the local range of this
+   * Return true if the given global index is in the local range of this
    * processor.  Since this is not a distributed vector the method always
    * returns true.
    */
@@ -983,6 +1017,12 @@ protected:
    */
   Number *val;
 
+  /**
+   * For parallel loops with TBB, this member variable stores the affinity
+   * information of loops.
+   */
+  mutable std_cxx11::shared_ptr<parallel::internal::TBBPartitioner> thread_loop_partitioner;
+
   /**
    * Make all other vector types friends.
    */
@@ -1019,6 +1059,22 @@ private:
 #ifndef DOXYGEN
 
 
+//------------------------ declarations for explicit specializations
+
+#ifdef DEAL_II_BOOST_BIND_COMPILER_BUG
+template <>
+Vector<std::complex<float> > &
+Vector<std::complex<float> >::operator= (const std::complex<float> s);
+#endif
+
+
+template <>
+Vector<int>::real_type
+Vector<int>::lp_norm (const real_type) const;
+
+
+//------------------------ inline functions
+
 template <typename Number>
 inline
 Vector<Number>::Vector ()
@@ -1026,7 +1082,9 @@ Vector<Number>::Vector ()
   vec_size(0),
   max_vec_size(0),
   val(0)
-{}
+{
+  reinit(0);
+}
 
 
 
@@ -1072,82 +1130,6 @@ Vector<Number>::~Vector ()
 
 
 
-template <typename Number>
-inline
-void Vector<Number>::reinit (const size_type n,
-                             const bool omit_zeroing_entries)
-{
-  if (n==0)
-    {
-      if (val) deallocate();
-      val = 0;
-      max_vec_size = vec_size = 0;
-      return;
-    };
-
-  if (n>max_vec_size)
-    {
-      if (val) deallocate();
-      max_vec_size = n;
-      allocate();
-    };
-  vec_size = n;
-  if (omit_zeroing_entries == false)
-    *this = static_cast<Number>(0);
-}
-
-
-
-// declare function that is implemented in vector.templates.h
-namespace internal
-{
-  namespace Vector
-  {
-    template <typename T, typename U>
-    void copy_vector (const dealii::Vector<T> &src,
-                      dealii::Vector<U>       &dst);
-  }
-}
-
-
-
-template <typename Number>
-inline
-Vector<Number> &
-Vector<Number>::operator= (const Vector<Number> &v)
-{
-  dealii::internal::Vector::copy_vector (v, *this);
-  return *this;
-}
-
-
-
-#ifdef DEAL_II_WITH_CXX11
-template <typename Number>
-inline
-Vector<Number> &
-Vector<Number>::operator= (Vector<Number> &&v)
-{
-  swap(v);
-
-  return *this;
-}
-#endif
-
-
-
-template <typename Number>
-template <typename Number2>
-inline
-Vector<Number> &
-Vector<Number>::operator= (const Vector<Number2> &v)
-{
-  internal::Vector::copy_vector (v, *this);
-  return *this;
-}
-
-
-
 template <typename Number>
 inline
 std::size_t Vector<Number>::size () const
@@ -1457,9 +1439,20 @@ operator << (LogStream &os, const Vector<number> &v)
   return os;
 }
 
-
 /*@}*/
 
+
+/**
+ * Declare dealii::Vector< Number > as serial vector.
+ *
+ * @author Uwe Koecher, 2017
+ */
+template <typename Number>
+struct is_serial_vector< Vector<Number> > : std_cxx11::true_type
+{
+};
+
+
 DEAL_II_NAMESPACE_CLOSE
 
 #endif
