@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2015 by the deal.II authors
+// Copyright (C) 1998 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -97,21 +97,26 @@ public:
    * @{
    */
 
-  /**
-   * No more available vectors.
-   */
-  DeclException0(ExcNoMoreVectors);
   /**
    * Vector was not allocated from this memory pool.
    */
-  DeclException0(ExcNotAllocatedHere);
+  DeclExceptionMsg(ExcNotAllocatedHere,
+                   "You are trying to deallocate a vector from a memory pool, but this "
+                   "vector has not actually been allocated by the same pool before.");
 
   //@}
   /**
-   * Pointer to vectors allocated from VectorMemory objects. This pointer is
-   * safe in the sense that it automatically calls free() when it is
-   * destroyed, thus relieving the user from using vector management functions
-   * at all.
+   * A class that looks like a pointer for all practical purposes and that
+   * upon construction time allocates a vector from a VectorMemory object
+   * (or an object of a derived class) that is passed to the constructor.
+   * The destructor then automatically returns the vector's ownership to
+   * the same VectorMemory object.
+   *
+   * Pointers of this type are therefore safe in the sense that they automatically
+   * call VectorMemory::free() when they are destroyed, whether that happens
+   * at the end of a code block or because local variables are destroyed during
+   * exception unwinding. These kinds of object thus relieve the user from
+   * using vector management functions explicitly.
    *
    * @author Guido Kanschat, 2009
    */
@@ -141,11 +146,13 @@ public:
      * Dereferencing operator.
      */
     VectorType *operator -> () const;
+
   private:
     /**
      * The memory pool used.
      */
     SmartPointer<VectorMemory<VectorType>,Pointer> pool;
+
     /**
      * The pointer to the vector.
      */
@@ -341,6 +348,16 @@ private:
   static Threads::Mutex mutex;
 };
 
+
+
+namespace internal
+{
+  namespace GrowingVectorMemory
+  {
+    void release_all_unused_memory();
+  }
+}
+
 /*@}*/
 
 #ifndef DOXYGEN
@@ -361,7 +378,12 @@ template <typename VectorType>
 inline
 VectorMemory<VectorType>::Pointer::~Pointer()
 {
-  pool->free(v);
+  try
+    {
+      pool->free(v);
+    }
+  catch (...)
+    {}
 }
 
 
