@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2011 - 2015 by the deal.II authors
+// Copyright (C) 2011 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,6 +13,9 @@
 //
 // ---------------------------------------------------------------------
 
+#ifndef dealii__matrix_free_dof_info_templates_h
+#define dealii__matrix_free_dof_info_templates_h
+
 
 #include <deal.II/base/memory_consumption.h>
 #include <deal.II/base/multithread_info.h>
@@ -335,7 +338,6 @@ no_constraint:
       const unsigned int n_owned  = (vector_partitioner->local_range().second-
                                      vector_partitioner->local_range().first);
       const std::size_t n_ghosts = ghost_dofs.size();
-      unsigned int      n_unique_ghosts= 0;
 #ifdef DEBUG
       for (std::vector<unsigned int>::iterator dof = dof_indices.begin();
            dof!=dof_indices.end(); ++dof)
@@ -346,6 +348,7 @@ no_constraint:
       IndexSet ghost_indices (vector_partitioner->size());
       if (n_ghosts > 0)
         {
+          unsigned int n_unique_ghosts = 0;
           // since we need to go back to the local_to_global indices and
           // replace the temporary numbering of ghosts by the real number in
           // the index set, we need to store these values
@@ -1218,6 +1221,7 @@ no_constraint:
 
               // adjust end of boundary cells to the remainder
               size_info.boundary_cells_end += (remainder+vectorization_length-1)/vectorization_length;
+              start_nonboundary = 0;
             }
           else
             {
@@ -2007,3 +2011,5 @@ not_connect:
 } // end of namespace internal
 
 DEAL_II_NAMESPACE_CLOSE
+
+#endif
