@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2011 - 2016 by the deal.II authors
+// Copyright (C) 2011 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -17,16 +17,18 @@
 #ifndef dealii__matrix_free_h
 #define dealii__matrix_free_h
 
+#include <deal.II/base/aligned_vector.h>
 #include <deal.II/base/exceptions.h>
 #include <deal.II/base/parallel.h>
 #include <deal.II/base/quadrature.h>
 #include <deal.II/base/vectorization.h>
+#include <deal.II/base/thread_local_storage.h>
 #include <deal.II/base/template_constraints.h>
 #include <deal.II/fe/fe.h>
 #include <deal.II/fe/mapping.h>
 #include <deal.II/fe/mapping_q1.h>
 #include <deal.II/lac/vector.h>
-#include <deal.II/lac/parallel_vector.h>
+#include <deal.II/lac/la_parallel_vector.h>
 #include <deal.II/lac/block_vector_base.h>
 #include <deal.II/lac/constraint_matrix.h>
 #include <deal.II/dofs/dof_handler.h>
@@ -47,6 +49,7 @@
 #include <stdlib.h>
 #include <memory>
 #include <limits>
+#include <list>
 
 
 DEAL_II_NAMESPACE_OPEN
@@ -97,11 +100,13 @@ DEAL_II_NAMESPACE_OPEN
  * operations for several cells with one CPU instruction and is one of the
  * main features of this framework.
  *
+ * For details on usage of this class, see the description of FEEvaluation.
+ *
  * @author Katharina Kormann, Martin Kronbichler, 2010, 2011
  */
 
 template <int dim, typename Number=double>
-class MatrixFree
+class MatrixFree : public Subscriptor
 {
 public:
 
@@ -136,14 +141,62 @@ public:
   struct AdditionalData
   {
     /**
-     * Collects options for task parallelism.
+     * Collects options for task parallelism. See the documentation of the
+     * member variable MatrixFree::AdditionalData::tasks_parallel_scheme for a
+     * thorough description.
      */
-    enum TasksParallelScheme {none, partition_partition, partition_color, color};
+    enum TasksParallelScheme
+    {
+      /**
+       * Perform application in serial.
+       */
+      none,
+      /**
+       * Partition the cells into two levels and afterwards form chunks.
+       */
+      partition_partition,
+      /**
+       * Partition on the global level and color cells within the partitions.
+       */
+      partition_color,
+      /**
+       * Use the traditional coloring algorithm: this is like
+       * TasksParallelScheme::partition_color, but only uses one partition.
+       */
+      color
+    };
+
+    // avoid warning about use of deprecated variables
+    DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 
     /**
      * Constructor for AdditionalData.
      */
-    AdditionalData (const MPI_Comm            mpi_communicator   = MPI_COMM_SELF,
+    AdditionalData (const TasksParallelScheme tasks_parallel_scheme = partition_partition,
+                    const unsigned int        tasks_block_size   = 0,
+                    const UpdateFlags         mapping_update_flags  = update_gradients | update_JxW_values,
+                    const unsigned int level_mg_handler = numbers::invalid_unsigned_int,
+                    const bool                store_plain_indices = true,
+                    const bool                initialize_indices = true,
+                    const bool                initialize_mapping = true)
+      :
+      mpi_communicator      (MPI_COMM_SELF),
+      tasks_parallel_scheme (tasks_parallel_scheme),
+      tasks_block_size      (tasks_block_size),
+      mapping_update_flags  (mapping_update_flags),
+      level_mg_handler      (level_mg_handler),
+      store_plain_indices   (store_plain_indices),
+      initialize_indices    (initialize_indices),
+      initialize_mapping    (initialize_mapping)
+    {};
+
+    /**
+     * Constructor for AdditionalData.
+     *
+     * @deprecated @p mpi_communicator should not be specified here
+     * since it is not used in the MatrixFree class.
+     */
+    AdditionalData (const MPI_Comm            mpi_communicator,
                     const TasksParallelScheme tasks_parallel_scheme = partition_partition,
                     const unsigned int        tasks_block_size   = 0,
                     const UpdateFlags         mapping_update_flags  = update_gradients | update_JxW_values,
@@ -160,18 +213,21 @@ public:
       store_plain_indices   (store_plain_indices),
       initialize_indices    (initialize_indices),
       initialize_mapping    (initialize_mapping)
-    {};
+    {} DEAL_II_DEPRECATED
 
     /**
-     * Sets the MPI communicator that the parallel layout of the operator
+     * Set the MPI communicator that the parallel layout of the operator
      * should be based upon. Defaults to MPI_COMM_SELF, but should be set to a
      * communicator similar to the one used for a distributed triangulation in
      * order to inform this class over all cells that are present.
+     *
+     * @deprecated This variable is not used anymore. The @p mpi_communicator
+     * is automatically set to the one used by the triangulation.
      */
-    MPI_Comm            mpi_communicator;
+    MPI_Comm            mpi_communicator DEAL_II_DEPRECATED;
 
     /**
-     * Sets the scheme for task parallelism. There are four options available.
+     * Set the scheme for task parallelism. There are four options available.
      * If set to @p none, the operator application is done in serial without
      * shared memory parallelism. If this class is used together with MPI and
      * MPI is also used for parallelism within the nodes, this flag should be
@@ -201,7 +257,7 @@ public:
     TasksParallelScheme tasks_parallel_scheme;
 
     /**
-     * Sets the number of so-called macro cells that should form one
+     * Set the number of so-called macro cells that should form one
      * partition. If zero size is given, the class tries to find a good size
      * for the blocks based on MultithreadInfo::n_threads() and the number of
      * cells present. Otherwise, the given number is used. If the given number
@@ -261,6 +317,8 @@ public:
     bool                initialize_mapping;
   };
 
+  DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
+
   /**
    * @name 1: Construction and initialization
    */
@@ -270,6 +328,11 @@ public:
    */
   MatrixFree ();
 
+  /**
+   * Copy constructor, calls copy_from
+   */
+  MatrixFree (const MatrixFree<dim,Number> &other);
+
   /**
    * Destructor.
    */
@@ -285,24 +348,6 @@ public:
    * or contain several copies of the same element. Mixing several different
    * elements into one FESystem is not allowed. In that case, use the
    * initialization function with several DoFHandler arguments.
-   *
-   * The @p IndexSet @p locally_owned_dofs is used to specify the parallel
-   * partitioning with MPI. Usually, this needs not be specified, and the
-   * other initialization function without and @p IndexSet description can be
-   * used, which gets the partitioning information builtin into the
-   * DoFHandler.
-   */
-  template <typename DoFHandlerType, typename QuadratureType>
-  void reinit (const Mapping<dim>     &mapping,
-               const DoFHandlerType   &dof_handler,
-               const ConstraintMatrix &constraint,
-               const IndexSet         &locally_owned_dofs,
-               const QuadratureType   &quad,
-               const AdditionalData    additional_data = AdditionalData());
-
-  /**
-   * Initializes the data structures. Same as above, but with index set stored
-   * in the DoFHandler for describing the locally owned degrees of freedom.
    */
   template <typename DoFHandlerType, typename QuadratureType>
   void reinit (const Mapping<dim>     &mapping,
@@ -321,6 +366,21 @@ public:
                const QuadratureType   &quad,
                const AdditionalData    additional_data = AdditionalData());
 
+  /**
+   * Same as above.
+   *
+   * @deprecated Setting the index set specifically is not supported any
+   * more. Use the reinit function without index set argument to choose the
+   * one provided by DoFHandler::locally_owned_dofs().
+   */
+  template <typename DoFHandlerType, typename QuadratureType>
+  void reinit (const Mapping<dim>     &mapping,
+               const DoFHandlerType   &dof_handler,
+               const ConstraintMatrix &constraint,
+               const IndexSet         &locally_owned_dofs,
+               const QuadratureType   &quad,
+               const AdditionalData    additional_data = AdditionalData()) DEAL_II_DEPRECATED;
+
   /**
    * Extracts the information needed to perform loops over cells. The
    * DoFHandler and ConstraintMatrix describe the layout of degrees of
@@ -340,25 +400,6 @@ public:
    * different degrees. However, the number of different quadrature formulas
    * can be sets independently from the number of DoFHandlers, when several
    * elements are always integrated with the same quadrature formula.
-   *
-   * The @p IndexSet @p locally_owned_dofs is used to specify the parallel
-   * partitioning with MPI. Usually, this needs not be specified, and the
-   * other initialization function without and @p IndexSet description can be
-   * used, which gets the partitioning information from the DoFHandler. This
-   * is the most general initialization function.
-   */
-  template <typename DoFHandlerType, typename QuadratureType>
-  void reinit (const Mapping<dim>                          &mapping,
-               const std::vector<const DoFHandlerType *>   &dof_handler,
-               const std::vector<const ConstraintMatrix *> &constraint,
-               const std::vector<IndexSet>                 &locally_owned_set,
-               const std::vector<QuadratureType>           &quad,
-               const AdditionalData                        additional_data = AdditionalData());
-
-  /**
-   * Initializes the data structures. Same as before, but now the index set
-   * description of the locally owned range of degrees of freedom is taken
-   * from the DoFHandler.
    */
   template <typename DoFHandlerType, typename QuadratureType>
   void reinit (const Mapping<dim>                          &mapping,
@@ -377,6 +418,21 @@ public:
                const std::vector<QuadratureType>           &quad,
                const AdditionalData                        additional_data = AdditionalData());
 
+  /**
+   * Same as above.
+   *
+   * @deprecated Setting the index set specifically is not supported any
+   * more. Use the reinit function without index set argument to choose the
+   * one provided by DoFHandler::locally_owned_dofs().
+   */
+  template <typename DoFHandlerType, typename QuadratureType>
+  void reinit (const Mapping<dim>                          &mapping,
+               const std::vector<const DoFHandlerType *>   &dof_handler,
+               const std::vector<const ConstraintMatrix *> &constraint,
+               const std::vector<IndexSet>                 &locally_owned_set,
+               const std::vector<QuadratureType>           &quad,
+               const AdditionalData                        additional_data = AdditionalData()) DEAL_II_DEPRECATED;
+
   /**
    * Initializes the data structures. Same as before, but now the index set
    * description of the locally owned range of degrees of freedom is taken
@@ -409,7 +465,7 @@ public:
   void copy_from (const MatrixFree<dim,Number> &matrix_free_base);
 
   /**
-   * Clears all data fields and brings the class into a condition similar to
+   * Clear all data fields and brings the class into a condition similar to
    * after having called the default constructor.
    */
   void clear();
@@ -506,10 +562,22 @@ public:
   /**
    * Initialize function for a general vector. The length of the vector is
    * equal to the total number of degrees in the DoFHandler. If the vector is
-   * of class parallel::distributed::Vector@<Number@>, the ghost entries are
+   * of class LinearAlgebra::distributed::Vector@<Number@>, the ghost entries are
    * set accordingly. For vector-valued problems with several DoFHandlers
    * underlying this class, the parameter @p vector_component defines which
    * component is to be used.
+   *
+   * For the vectors used with MatrixFree and in FEEvaluation, a vector needs
+   * to hold all
+   * @ref GlossLocallyActiveDof "locally active DoFs"
+   * and also some of the
+   * @ref GlossLocallyRelevantDof "locally relevant DoFs".
+   * The selection of DoFs is such that one can read all degrees of freedom on all
+   * locally relevant elements (locally active) plus the degrees of freedom
+   * that contraints expand into from the locally owned cells. However, not
+   * all locally relevant DoFs are stored because most of them would never be
+   * accessed in matrix-vector products and result in too much data sent
+   * around which impacts the performance.
    */
   template <typename VectorType>
   void initialize_dof_vector(VectorType &vec,
@@ -518,17 +586,29 @@ public:
   /**
    * Initialize function for a distributed vector. The length of the vector is
    * equal to the total number of degrees in the DoFHandler. If the vector is
-   * of class parallel::distributed::Vector@<Number@>, the ghost entries are
+   * of class LinearAlgebra::distributed::Vector@<Number@>, the ghost entries are
    * set accordingly. For vector-valued problems with several DoFHandlers
    * underlying this class, the parameter @p vector_component defines which
    * component is to be used.
+   *
+   * For the vectors used with MatrixFree and in FEEvaluation, a vector needs
+   * to hold all
+   * @ref GlossLocallyActiveDof "locally active DoFs"
+   * and also some of the
+   * @ref GlossLocallyRelevantDof "locally relevant DoFs".
+   * The selection of DoFs is such that one can read all degrees of freedom on all
+   * locally relevant elements (locally active) plus the degrees of freedom
+   * that contraints expand into from the locally owned cells. However, not
+   * all locally relevant DoFs are stored because most of them would never be
+   * accessed in matrix-vector products and result in too much data sent
+   * around which impacts the performance.
    */
   template <typename Number2>
-  void initialize_dof_vector(parallel::distributed::Vector<Number2> &vec,
+  void initialize_dof_vector(LinearAlgebra::distributed::Vector<Number2> &vec,
                              const unsigned int vector_component=0) const;
 
   /**
-   * Returns the partitioner that represents the locally owned data and the
+   * Return the partitioner that represents the locally owned data and the
    * ghost indices where access is needed to for the cell loop. The
    * partitioner is constructed from the locally owned dofs and ghost dofs
    * given by the respective fields. If you want to have specific information
@@ -541,19 +621,19 @@ public:
   get_vector_partitioner (const unsigned int vector_component=0) const;
 
   /**
-   * Returns the set of cells that are oned by the processor.
+   * Return the set of cells that are oned by the processor.
    */
   const IndexSet &
   get_locally_owned_set (const unsigned int fe_component = 0) const;
 
   /**
-   * Returns the set of ghost cells needed but not owned by the processor.
+   * Return the set of ghost cells needed but not owned by the processor.
    */
   const IndexSet &
   get_ghost_set (const unsigned int fe_component = 0) const;
 
   /**
-   * Returns a list of all degrees of freedom that are constrained. The list
+   * Return a list of all degrees of freedom that are constrained. The list
    * is returned in MPI-local index space for the locally owned range of the
    * vector, not in global MPI index space that spans all MPI processors. To
    * get numbers in global index space, call
@@ -578,12 +658,19 @@ public:
    */
   //@{
   /**
-   * Returns the number of different DoFHandlers specified at initialization.
+   * Returns whether a given FiniteElement @p fe is supported by this class.
+   */
+  template <int spacedim>
+  static
+  bool is_supported (const FiniteElement<dim, spacedim> &fe);
+
+  /**
+   * Return the number of different DoFHandlers specified at initialization.
    */
   unsigned int n_components () const;
 
   /**
-   * Returns the number of cells this structure is based on. If you are using
+   * Return the number of cells this structure is based on. If you are using
    * a usual DoFHandler, it corresponds to the number of (locally owned)
    * active cells. Note that most data structures in this class do not
    * directly act on this number but rather on n_macro_cells() which gives the
@@ -593,7 +680,7 @@ public:
   unsigned int n_physical_cells () const;
 
   /**
-   * Returns the number of macro cells that this structure works on, i.e., the
+   * Return the number of macro cells that this structure works on, i.e., the
    * number of cell chunks that are worked on after the application of
    * vectorization which in general works on several cells at once. The cell
    * range in @p cell_loop runs from zero to n_macro_cells() (exclusive), so
@@ -668,21 +755,21 @@ public:
   n_components_filled (const unsigned int macro_cell_number) const;
 
   /**
-   * Returns the number of degrees of freedom per cell for a given hp index.
+   * Return the number of degrees of freedom per cell for a given hp index.
    */
   unsigned int
   get_dofs_per_cell (const unsigned int fe_component = 0,
                      const unsigned int hp_active_fe_index = 0) const;
 
   /**
-   * Returns the number of quadrature points per cell for a given hp index.
+   * Return the number of quadrature points per cell for a given hp index.
    */
   unsigned int
   get_n_q_points (const unsigned int quad_index = 0,
                   const unsigned int hp_active_fe_index = 0) const;
 
   /**
-   * Returns the number of degrees of freedom on each face of the cell for
+   * Return the number of degrees of freedom on each face of the cell for
    * given hp index.
    */
   unsigned int
@@ -690,7 +777,7 @@ public:
                      const unsigned int hp_active_fe_index = 0) const;
 
   /**
-   * Returns the number of quadrature points on each face of the cell for
+   * Return the number of quadrature points on each face of the cell for
    * given hp index.
    */
   unsigned int
@@ -698,14 +785,14 @@ public:
                        const unsigned int hp_active_fe_index = 0) const;
 
   /**
-   * Returns the quadrature rule for given hp index.
+   * Return the quadrature rule for given hp index.
    */
   const Quadrature<dim> &
   get_quadrature (const unsigned int quad_index = 0,
                   const unsigned int hp_active_fe_index = 0) const;
 
   /**
-   * Returns the quadrature rule for given hp index.
+   * Return the quadrature rule for given hp index.
    */
   const Quadrature<dim-1> &
   get_face_quadrature (const unsigned int quad_index = 0,
@@ -724,7 +811,7 @@ public:
   bool mapping_initialized () const;
 
   /**
-   * Returns an approximation of the memory consumption of this class in
+   * Return an approximation of the memory consumption of this class in
    * bytes.
    */
   std::size_t memory_consumption() const;
@@ -749,43 +836,43 @@ public:
    */
   //@{
   /**
-   * Returns information on task graph.
+   * Return information on task graph.
    */
   const internal::MatrixFreeFunctions::TaskInfo &
   get_task_info () const;
 
   /**
-   * Returns information on system size.
+   * Return information on system size.
    */
   const internal::MatrixFreeFunctions::SizeInfo &
   get_size_info () const;
 
   /*
-   * Returns geometry-dependent information on the cells.
+   * Return geometry-dependent information on the cells.
    */
   const internal::MatrixFreeFunctions::MappingInfo<dim,Number> &
   get_mapping_info () const;
 
   /**
-   * Returns information on indexation degrees of freedom.
+   * Return information on indexation degrees of freedom.
    */
   const internal::MatrixFreeFunctions::DoFInfo &
   get_dof_info (const unsigned int fe_component = 0) const;
 
   /**
-   * Returns the number of weights in the constraint pool.
+   * Return the number of weights in the constraint pool.
    */
   unsigned int n_constraint_pool_entries() const;
 
   /**
-   * Returns a pointer to the first number in the constraint pool data with
+   * Return a pointer to the first number in the constraint pool data with
    * index @p pool_index (to be used together with @p constraint_pool_end()).
    */
   const Number *
   constraint_pool_begin (const unsigned int pool_index) const;
 
   /**
-   * Returns a pointer to one past the last number in the constraint pool data
+   * Return a pointer to one past the last number in the constraint pool data
    * with index @p pool_index (to be used together with @p
    * constraint_pool_begin()).
    */
@@ -793,7 +880,7 @@ public:
   constraint_pool_end (const unsigned int pool_index) const;
 
   /**
-   * Returns the unit cell information for given hp index.
+   * Return the unit cell information for given hp index.
    */
   const internal::MatrixFreeFunctions::ShapeInfo<Number> &
   get_shape_info (const unsigned int fe_component = 0,
@@ -801,6 +888,27 @@ public:
                   const unsigned int hp_active_fe_index = 0,
                   const unsigned int hp_active_quad_index = 0) const;
 
+  /**
+   * Obtains a scratch data object for internal use. Make sure to release it
+   * afterwards by passing the pointer you obtain from this object to the
+   * release_scratch_data() function. This interface is used by FEEvaluation
+   * objects for storing their data structures.
+   *
+   * The organization of the internal data structure is a thread-local storage
+   * of a list of vectors. Multiple threads will each get a separate storage
+   * field and separate vectors, ensuring thread safety. The mechanism to
+   * acquire and release objects is similar to the mechanisms used for the
+   * local contributions of WorkStream, see
+   * @ref workstream_paper "the WorkStream paper"
+   * .
+   */
+  AlignedVector<VectorizedArray<Number> > *acquire_scratch_data() const;
+
+  /**
+   * Makes the object of the scratchpad available again.
+   */
+  void release_scratch_data(const AlignedVector<VectorizedArray<Number> > *memory) const;
+
   //@}
 
 private:
@@ -855,10 +963,26 @@ private:
    */
   struct DoFHandlers
   {
-    DoFHandlers () : n_dof_handlers (0), level (numbers::invalid_unsigned_int) {};
+    DoFHandlers ()
+      :
+      active_dof_handler(usual),
+      n_dof_handlers (0),
+      level (numbers::invalid_unsigned_int)
+    {}
+
     std::vector<SmartPointer<const DoFHandler<dim> > >   dof_handler;
     std::vector<SmartPointer<const hp::DoFHandler<dim> > > hp_dof_handler;
-    enum ActiveDoFHandler { usual, hp } active_dof_handler;
+    enum ActiveDoFHandler
+    {
+      /**
+       * Use DoFHandler.
+       */
+      usual,
+      /**
+       * Use hp::DoFHandler.
+       */
+      hp
+    } active_dof_handler;
     unsigned int n_dof_handlers;
     unsigned int level;
   };
@@ -927,6 +1051,15 @@ private:
    * Stores whether indices have been initialized.
    */
   bool                               mapping_is_initialized;
+
+  /**
+   * Scratchpad memory for use in evaluation. We allow more than one
+   * evaluation object to attach to this field (this, the outer
+   * std::vector), so we need to keep tracked of whether a certain data
+   * field is already used (first part of pair) and keep a list of
+   * objects.
+   */
+  mutable Threads::ThreadLocalStorage<std::list<std::pair<bool, AlignedVector<VectorizedArray<Number> > > > > scratch_pad;
 };
 
 
@@ -953,7 +1086,7 @@ template <int dim, typename Number>
 template <typename Number2>
 inline
 void
-MatrixFree<dim,Number>::initialize_dof_vector(parallel::distributed::Vector<Number2> &vec,
+MatrixFree<dim,Number>::initialize_dof_vector(LinearAlgebra::distributed::Vector<Number2> &vec,
                                               const unsigned int comp) const
 {
   AssertIndexRange (comp, n_components());
@@ -1422,6 +1555,42 @@ MatrixFree<dim,Number>::mapping_initialized () const
 
 
 
+template <int dim,typename Number>
+AlignedVector<VectorizedArray<Number> > *
+MatrixFree<dim,Number>::acquire_scratch_data() const
+{
+  typedef std::list<std::pair<bool, AlignedVector<VectorizedArray<Number> > > > list_type;
+  list_type &data = scratch_pad.get();
+  for (typename list_type::iterator it=data.begin(); it!=data.end(); ++it)
+    if (it->first == false)
+      {
+        it->first = true;
+        return &it->second;
+      }
+  data.push_front(std::make_pair(true,AlignedVector<VectorizedArray<Number> >()));
+  return &data.front().second;
+}
+
+
+
+template <int dim, typename Number>
+void
+MatrixFree<dim,Number>::release_scratch_data(const AlignedVector<VectorizedArray<Number> > *scratch) const
+{
+  typedef std::list<std::pair<bool, AlignedVector<VectorizedArray<Number> > > > list_type;
+  list_type &data = scratch_pad.get();
+  for (typename list_type::iterator it=data.begin(); it!=data.end(); ++it)
+    if (&it->second == scratch)
+      {
+        Assert(it->first == true, ExcInternalError());
+        it->first = false;
+        return;
+      }
+  AssertThrow(false, ExcMessage("Tried to release invalid scratch pad"));
+}
+
+
+
 // ------------------------------ reinit functions ---------------------------
 
 namespace internal
@@ -1483,8 +1652,12 @@ reinit(const DoFHandlerType                                  &dof_handler,
   std::vector<IndexSet> locally_owned_sets =
     internal::MatrixFree::extract_locally_owned_index_sets
     (dof_handlers, additional_data.level_mg_handler);
-  reinit(StaticMappingQ1<dim>::mapping, dof_handlers,constraints, locally_owned_sets, quads,
-         additional_data);
+
+  std::vector<hp::QCollection<1> > quad_hp;
+  quad_hp.push_back (hp::QCollection<1>(quad));
+
+  internal_reinit(StaticMappingQ1<dim>::mapping, dof_handlers,constraints,
+                  locally_owned_sets, quad_hp, additional_data);
 }
 
 
@@ -1500,17 +1673,19 @@ reinit(const Mapping<dim>                                    &mapping,
 {
   std::vector<const DoFHandlerType *>   dof_handlers;
   std::vector<const ConstraintMatrix *> constraints;
-  std::vector<QuadratureType>           quads;
 
   dof_handlers.push_back(&dof_handler);
   constraints.push_back (&constraints_in);
-  quads.push_back (quad);
 
   std::vector<IndexSet> locally_owned_sets =
     internal::MatrixFree::extract_locally_owned_index_sets
     (dof_handlers, additional_data.level_mg_handler);
-  reinit(mapping, dof_handlers,constraints,locally_owned_sets, quads,
-         additional_data);
+
+  std::vector<hp::QCollection<1> > quad_hp;
+  quad_hp.push_back (hp::QCollection<1>(quad));
+
+  internal_reinit(mapping, dof_handlers,constraints,locally_owned_sets,
+                  quad_hp,  additional_data);
 }
 
 
@@ -1526,9 +1701,11 @@ reinit(const std::vector<const DoFHandlerType *>   &dof_handler,
   std::vector<IndexSet> locally_owned_set =
     internal::MatrixFree::extract_locally_owned_index_sets
     (dof_handler, additional_data.level_mg_handler);
-  reinit(StaticMappingQ1<dim>::mapping, dof_handler,constraint,locally_owned_set,
-         static_cast<const std::vector<Quadrature<1> >&>(quad),
-         additional_data);
+  std::vector<hp::QCollection<1> > quad_hp;
+  for (unsigned int q=0; q<quad.size(); ++q)
+    quad_hp.push_back (hp::QCollection<1>(quad[q]));
+  internal_reinit(StaticMappingQ1<dim>::mapping, dof_handler,constraint,
+                  locally_owned_set, quad_hp, additional_data);
 }
 
 
@@ -1541,13 +1718,13 @@ reinit(const std::vector<const DoFHandlerType *>             &dof_handler,
        const QuadratureType                                  &quad,
        const typename MatrixFree<dim,Number>::AdditionalData additional_data)
 {
-  std::vector<QuadratureType> quads;
-  quads.push_back(quad);
   std::vector<IndexSet> locally_owned_set =
     internal::MatrixFree::extract_locally_owned_index_sets
     (dof_handler, additional_data.level_mg_handler);
-  reinit(StaticMappingQ1<dim>::mapping, dof_handler,constraint,locally_owned_set, quads,
-         additional_data);
+  std::vector<hp::QCollection<1> > quad_hp;
+  quad_hp.push_back (hp::QCollection<1>(quad));
+  internal_reinit(StaticMappingQ1<dim>::mapping, dof_handler,constraint,
+                  locally_owned_set, quad_hp, additional_data);
 }
 
 
@@ -1561,13 +1738,13 @@ reinit(const Mapping<dim>                                    &mapping,
        const QuadratureType                                  &quad,
        const typename MatrixFree<dim,Number>::AdditionalData additional_data)
 {
-  std::vector<QuadratureType> quads;
-  quads.push_back(quad);
   std::vector<IndexSet> locally_owned_set =
     internal::MatrixFree::extract_locally_owned_index_sets
     (dof_handler, additional_data.level_mg_handler);
-  reinit(mapping, dof_handler,constraint,locally_owned_set, quads,
-         additional_data);
+  std::vector<hp::QCollection<1> > quad_hp;
+  quad_hp.push_back (hp::QCollection<1>(quad));
+  internal_reinit(mapping, dof_handler,constraint,
+                  locally_owned_set, quad_hp, additional_data);
 }
 
 
@@ -1584,8 +1761,11 @@ reinit(const Mapping<dim>                                   &mapping,
   std::vector<IndexSet> locally_owned_set =
     internal::MatrixFree::extract_locally_owned_index_sets
     (dof_handler, additional_data.level_mg_handler);
-  reinit(mapping, dof_handler,constraint,locally_owned_set,
-         quad, additional_data);
+  std::vector<hp::QCollection<1> > quad_hp;
+  for (unsigned int q=0; q<quad.size(); ++q)
+    quad_hp.push_back (hp::QCollection<1>(quad[q]));
+  internal_reinit(mapping, dof_handler,constraint,locally_owned_set,
+                  quad_hp, additional_data);
 }
 
 
@@ -1685,7 +1865,7 @@ namespace internal
 
   template<typename Number>
   inline
-  bool update_ghost_values_start (const parallel::distributed::Vector<Number> &vec,
+  bool update_ghost_values_start (const LinearAlgebra::distributed::Vector<Number> &vec,
                                   const unsigned int                  channel = 0)
   {
     bool return_value = !vec.has_ghost_elements();
@@ -1749,11 +1929,11 @@ namespace internal
 
   template<typename Number>
   inline
-  void reset_ghost_values (const parallel::distributed::Vector<Number> &vec,
+  void reset_ghost_values (const LinearAlgebra::distributed::Vector<Number> &vec,
                            const bool zero_out_ghosts)
   {
     if (zero_out_ghosts)
-      const_cast<parallel::distributed::Vector<Number>&>(vec).zero_out_ghosts();
+      const_cast<LinearAlgebra::distributed::Vector<Number>&>(vec).zero_out_ghosts();
   }
 
 
@@ -1804,7 +1984,7 @@ namespace internal
 
   template <typename Number>
   inline
-  void update_ghost_values_finish (const parallel::distributed::Vector<Number> &vec)
+  void update_ghost_values_finish (const LinearAlgebra::distributed::Vector<Number> &vec)
   {
     vec.update_ghost_values_finish();
   }
@@ -1855,7 +2035,7 @@ namespace internal
 
   template <typename Number>
   inline
-  void compress_start (parallel::distributed::Vector<Number> &vec,
+  void compress_start (LinearAlgebra::distributed::Vector<Number> &vec,
                        const unsigned int           channel = 0)
   {
     vec.compress_start(channel);
@@ -1907,7 +2087,7 @@ namespace internal
 
   template <typename Number>
   inline
-  void compress_finish (parallel::distributed::Vector<Number> &vec)
+  void compress_finish (LinearAlgebra::distributed::Vector<Number> &vec)
   {
     vec.compress_finish(::dealii::VectorOperation::add);
   }
@@ -1961,6 +2141,7 @@ namespace internal
                 const internal::MatrixFreeFunctions::TaskInfo &task_info_in,
                 const bool is_blocked_in)
         :
+        dummy (NULL),
         worker (worker_in),
         partition (partition_in),
         task_info (task_info_in),
@@ -1997,6 +2178,7 @@ namespace internal
                      const internal::MatrixFreeFunctions::TaskInfo &task_info_in,
                      const bool    is_blocked_in = false)
         :
+        dummy (NULL),
         function (function_in),
         partition (partition_in),
         task_info (task_info_in),
@@ -2124,6 +2306,7 @@ namespace internal
                      const internal::MatrixFreeFunctions::TaskInfo &task_info_in,
                      const bool    is_blocked_in)
         :
+        dummy (NULL),
         worker (worker_in),
         partition (partition_in),
         task_info (task_info_in),
@@ -2319,7 +2502,7 @@ MatrixFree<dim, Number>::cell_loop
               std::vector<internal::color::PartitionWork<Worker>*> worker(n_workers);
               std::vector<internal::color::PartitionWork<Worker>*> blocked_worker(n_blocked_workers);
               unsigned int worker_index = 0, slice_index = 0;
-              unsigned int spawn_index =  0, spawn_index_new = 0;
+              unsigned int spawn_index =  0;
               int spawn_index_child = -2;
               internal::MPIComCompress<OutVector> *worker_compr = new(root->allocate_child())
               internal::MPIComCompress<OutVector>(dst);
@@ -2327,7 +2510,7 @@ MatrixFree<dim, Number>::cell_loop
               for (unsigned int part=0;
                    part<task_info.partition_color_blocks_row_index.size()-1; part++)
                 {
-                  spawn_index_new = worker_index;
+                  const unsigned int spawn_index_new = worker_index;
                   if (part == 0)
                     worker[worker_index] = new(worker_compr->allocate_child())
                     internal::color::PartitionWork<Worker>(func,slice_index,task_info,false);
@@ -2352,9 +2535,11 @@ MatrixFree<dim, Number>::cell_loop
                       if (spawn_index_child == -1)
                         worker[spawn_index]->spawn(*blocked_worker[(part-1)/2]);
                       else
-                        worker[spawn_index]->spawn(*worker[spawn_index_child]);
+                        {
+                          Assert(spawn_index_child>=0, ExcInternalError());
+                          worker[spawn_index]->spawn(*worker[spawn_index_child]);
+                        }
                       spawn_index = spawn_index_new;
-                      spawn_index_child = -2;
                     }
                   else
                     {
@@ -2410,7 +2595,10 @@ MatrixFree<dim, Number>::cell_loop
                     }
                 }
               if (evens==odds)
-                worker[spawn_index]->spawn(*worker[spawn_index_child]);
+                {
+                  Assert(spawn_index_child>=0, ExcInternalError());
+                  worker[spawn_index]->spawn(*worker[spawn_index_child]);
+                }
               root->wait_for_all();
               root->destroy(*root);
             }
