@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2011 - 2015 by the deal.II authors
+// Copyright (C) 2011 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -85,7 +85,7 @@ namespace internal
                    const unsigned int base_element = 0);
 
       /**
-       * Returns the memory consumption of this class in bytes.
+       * Return the memory consumption of this class in bytes.
        */
       std::size_t memory_consumption () const;
 
@@ -194,6 +194,11 @@ namespace internal
        */
       unsigned int fe_degree;
 
+      /**
+       * Stores the number of quadrature points per dimension.
+       */
+      unsigned int n_q_points_1d;
+
       /**
        * Stores the number of quadrature points in @p dim dimensions for a
        * cell.
@@ -216,13 +221,13 @@ namespace internal
       unsigned int dofs_per_face;
 
       /**
-       * Checks whether we have symmetries in the shape values. In that case,
+       * Check whether we have symmetries in the shape values. In that case,
        * also fill the shape_???_evenodd fields.
        */
       bool check_1d_shapes_symmetric(const unsigned int n_q_points_1d);
 
       /**
-       * Checks whether symmetric 1D basis functions are such that the shape
+       * Check whether symmetric 1D basis functions are such that the shape
        * values form a diagonal matrix, which allows to use specialized
        * algorithms that save some operations.
        */
@@ -241,8 +246,11 @@ namespace internal
                                   const unsigned int base_element_number)
       :
       fe_degree (0),
+      n_q_points_1d (0),
       n_q_points (0),
-      dofs_per_cell (0)
+      dofs_per_cell (0),
+      n_q_points_face (0),
+      dofs_per_face (0)
     {
       reinit (quad, fe_in, base_element_number);
     }
