@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2011 - 2016 by the deal.II authors
+// Copyright (C) 2011 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,6 +13,9 @@
 //
 // ---------------------------------------------------------------------
 
+#ifndef dealii__matrix_free_shape_info_templates_h
+#define dealii__matrix_free_shape_info_templates_h
+
 
 #include <deal.II/base/utilities.h>
 #include <deal.II/base/memory_consumption.h>
@@ -40,8 +43,12 @@ namespace internal
     ShapeInfo<Number>::ShapeInfo ()
       :
       element_type (tensor_general),
+      fe_degree (numbers::invalid_unsigned_int),
+      n_q_points_1d(0),
       n_q_points (0),
-      dofs_per_cell (0)
+      dofs_per_cell (0),
+      n_q_points_face (0),
+      dofs_per_face (0)
     {}
 
 
@@ -53,13 +60,13 @@ namespace internal
                                const FiniteElement<dim> &fe_in,
                                const unsigned int base_element_number)
     {
-      const FiniteElement<dim> *fe = &fe_in;
-      fe = &fe_in.base_element(base_element_number);
+      const FiniteElement<dim> *fe = &fe_in.base_element(base_element_number);
 
       Assert (fe->n_components() == 1,
               ExcMessage("FEEvaluation only works for scalar finite elements."));
 
       fe_degree = fe->degree;
+      n_q_points_1d = quad.size();
 
       const unsigned int n_dofs_1d = fe_degree+1,
                          n_q_points_1d = quad.size();
@@ -246,8 +253,11 @@ namespace internal
         }
         case 1:
         {
-          this->face_indices(0,0) = 0;
-          this->face_indices(1,0) = n_dofs_1d-1;
+          if (this->dofs_per_face>0)
+            {
+              this->face_indices(0,0) = 0;
+              this->face_indices(1,0) = n_dofs_1d-1;
+            }
           break;
         }
         default:
@@ -417,3 +427,5 @@ namespace internal
 
 
 DEAL_II_NAMESPACE_CLOSE
+
+#endif
