@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2010 - 2016 by the deal.II authors
+// Copyright (C) 2010 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -549,7 +549,7 @@ namespace MeshWorker
     (const DOFINFO &info)
     {
       for (unsigned int i=0; i<residuals.size(); ++i)
-        assemble(*residuals(i), info.vector(i), info.indices);
+        assemble(*(residuals.entry<VectorType>(i)), info.vector(i), info.indices);
     }
 
 
@@ -562,8 +562,8 @@ namespace MeshWorker
     {
       for (unsigned int i=0; i<residuals.size(); ++i)
         {
-          assemble(*residuals(i), info1.vector(i), info1.indices);
-          assemble(*residuals(i), info2.vector(i), info2.indices);
+          assemble(*(residuals.entry<VectorType>(i)), info1.vector(i), info1.indices);
+          assemble(*(residuals.entry<VectorType>(i)), info2.vector(i), info2.indices);
         }
     }
 
