@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2014 by the deal.II authors
+// Copyright (C) 2009 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,6 +13,10 @@
 //
 // ---------------------------------------------------------------------
 
+#ifndef dealii__dof_info_templates_h
+#define dealii__dof_info_templates_h
+
+
 #include <deal.II/meshworker/dof_info.h>
 #include <deal.II/base/quadrature_lib.h>
 
@@ -24,6 +28,8 @@ namespace MeshWorker
   template <int dim, int spacedim, typename number>
   DoFInfo<dim,spacedim,number>::DoFInfo(const BlockInfo &info)
     :
+    face_number (numbers::invalid_unsigned_int),
+    sub_number (numbers::invalid_unsigned_int),
     block_info(&info, typeid(*this).name()),
     level_cell (false)
   {
@@ -58,3 +64,4 @@ namespace MeshWorker
 
 DEAL_II_NAMESPACE_CLOSE
 
+#endif
