@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2010 - 2015 by the deal.II authors
+// Copyright (C) 2010 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -94,6 +94,11 @@ namespace MeshWorker
     class CellsAndFaces
     {
     public:
+      /**
+       * Constructor. Initialize the member variables.
+       */
+      CellsAndFaces();
+
       /**
        * The initialization function, specifying the @p results vectors and
        * whether face data should be collected separately.
@@ -198,6 +203,14 @@ namespace MeshWorker
 
 //----------------------------------------------------------------------//
 
+    template <typename number>
+    inline
+    CellsAndFaces<number>::CellsAndFaces()
+      : separate_faces(true)
+    {}
+
+
+
     template <typename number>
     inline void
     CellsAndFaces<number>::initialize(AnyData &r, bool sep)
