@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2015 by the deal.II authors
+// Copyright (C) 2009 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,6 +13,10 @@
 //
 // ---------------------------------------------------------------------
 
+#ifndef dealii__integration_info_templates_h
+#define dealii__integration_info_templates_h
+
+
 #include <deal.II/meshworker/dof_info.h>
 #include <deal.II/meshworker/integration_info.h>
 #include <deal.II/base/quadrature_lib.h>
@@ -229,3 +233,5 @@ namespace MeshWorker
 
 
 DEAL_II_NAMESPACE_CLOSE
+
+#endif
