@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2006 - 2015 by the deal.II authors
+// Copyright (C) 2006 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -22,6 +22,7 @@
 #include <deal.II/base/work_stream.h>
 #include <deal.II/base/template_constraints.h>
 #include <deal.II/grid/tria.h>
+#include <deal.II/grid/filtered_iterator.h>
 #include <deal.II/meshworker/local_integrator.h>
 #include <deal.II/meshworker/dof_info.h>
 #include <deal.II/meshworker/integration_info.h>
@@ -32,7 +33,6 @@
 DEAL_II_NAMESPACE_OPEN
 
 template <typename> class TriaActiveIterator;
-template <typename> class FilteredIterator;
 
 namespace internal
 {
@@ -52,7 +52,7 @@ namespace internal
   }
 
   template <class ACCESSOR>
-  inline bool is_active_iterator(const FilteredIterator<TriaActiveIterator<ACCESSOR> > &)
+  inline bool is_active_iterator(const dealii::FilteredIterator<TriaActiveIterator<ACCESSOR> > &)
   {
     return true;
   }
@@ -95,10 +95,24 @@ namespace MeshWorker
      */
     bool ghost_cells;
 
+    /**
+     * Enumeration describing when to do assembly on a face: see, e.g.,
+     * MeshWorker::LoopControl::faces_to_ghost for an example of how the value
+     * of this enumeration is interpreted in a particular circumstance.
+     */
     enum FaceOption
     {
+      /**
+       * Do not perform assembly on a face.
+       */
       never,
+      /**
+       * Perform assembly on one face.
+       */
       one,
+      /**
+       * Perform assembly on both faces.
+       */
       both
     };
 
@@ -209,7 +223,7 @@ namespace MeshWorker
       for (unsigned int face_no=0; face_no < GeometryInfo<ITERATOR::AccessorType::Container::dimension>::faces_per_cell; ++face_no)
         {
           typename ITERATOR::AccessorType::Container::face_iterator face = cell->face(face_no);
-          if (cell->at_boundary(face_no))
+          if (cell->at_boundary(face_no) && !cell->has_periodic_neighbor(face_no))
             {
               // only integrate boundary faces of own cells
               if (integrate_boundary && own_cell)
@@ -223,7 +237,7 @@ namespace MeshWorker
           else if (integrate_interior_face)
             {
               // Interior face
-              TriaIterator<typename ITERATOR::AccessorType> neighbor = cell->neighbor(face_no);
+              TriaIterator<typename ITERATOR::AccessorType> neighbor = cell->neighbor_or_periodic_neighbor(face_no);
 
               types::subdomain_id neighbid = numbers::artificial_subdomain_id;
               if (neighbor->is_level_cell())
@@ -247,17 +261,12 @@ namespace MeshWorker
               if (own_cell != own_neighbor && loop_control.faces_to_ghost==LoopControl::never)
                 continue;
 
-              // Deal with
-              // refinement edges
-              // from the refined
-              // side. Assuming
-              // one-irregular
-              // meshes, this
-              // situation should
-              // only occur if
-              // both cells are
-              // active.
-              if (cell->neighbor_is_coarser(face_no))
+              // Deal with refinement edges from the refined side. Assuming one-irregular
+              // meshes, this situation should only occur if both cells are active.
+              const bool periodic_neighbor = cell->has_periodic_neighbor(face_no);
+
+              if ((!periodic_neighbor && cell->neighbor_is_coarser(face_no))
+                  || (periodic_neighbor && cell->periodic_neighbor_is_coarser(face_no)))
                 {
                   Assert(!cell->has_children(), ExcInternalError());
                   Assert(!neighbor->has_children(), ExcInternalError());
@@ -269,7 +278,9 @@ namespace MeshWorker
                     continue;
 
                   const std::pair<unsigned int, unsigned int> neighbor_face_no
-                    = cell->neighbor_of_coarser_neighbor(face_no);
+                    = periodic_neighbor?
+                      cell->periodic_neighbor_of_coarser_periodic_neighbor(face_no):
+                      cell->neighbor_of_coarser_neighbor(face_no);
                   const typename ITERATOR::AccessorType::Container::face_iterator nface
                     = neighbor->face(neighbor_face_no.first);
 
@@ -321,8 +332,10 @@ namespace MeshWorker
                       && (neighbid < csid))
                     continue;
 
-                  const unsigned int neighbor_face_no = cell->neighbor_face_no(face_no);
-                  Assert (neighbor->face(neighbor_face_no) == face, ExcInternalError());
+                  const unsigned int neighbor_face_no = periodic_neighbor?
+                                                        cell->periodic_neighbor_face_no(face_no):
+                                                        cell->neighbor_face_no(face_no);
+                  Assert (periodic_neighbor || neighbor->face(neighbor_face_no) == face, ExcInternalError());
                   // Regular interior face
                   dof_info.interior_face_available[face_no] = true;
                   dof_info.exterior_face_available[face_no] = true;
