@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2015 by the deal.II authors
+// Copyright (C) 2009 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,6 +13,9 @@
 //
 // ---------------------------------------------------------------------
 
+#ifndef dealii__vector_selector_templates_h
+#define dealii__vector_selector_templates_h
+
 
 #include <deal.II/meshworker/vector_selector.h>
 #include <deal.II/base/vector_slice.h>
@@ -244,3 +247,5 @@ namespace MeshWorker
 }
 
 DEAL_II_NAMESPACE_CLOSE
+
+#endif
