@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2010 - 2015 by the deal.II authors
+// Copyright (C) 2010 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -61,11 +61,28 @@ public:
    * This function internally calls the initialize() function above and the
    * constrains degrees on the external boundary of the domain by calling
    * MGTools::make_boundary_list() with the given second and third argument.
+   *
+   * @deprecated Use initialize() followed by make_zero_boundary_constraints() instead
    */
   template <int dim, int spacedim>
   void initialize(const DoFHandler<dim,spacedim> &dof,
                   const typename FunctionMap<dim>::type &function_map,
-                  const ComponentMask &component_mask = ComponentMask());
+                  const ComponentMask &component_mask = ComponentMask()) DEAL_II_DEPRECATED;
+
+  /**
+   * Fill the internal data structures with information
+   * about Dirichlet boundary dofs.
+   *
+   * The initialize() function has to be called before
+   * to set hanging node constraints.
+   *
+   * This function can be called multiple times to allow considering
+   * different sets of boundary_ids for different components.
+   */
+  template <int dim, int spacedim>
+  void make_zero_boundary_constraints(const DoFHandler<dim,spacedim> &dof,
+                                      const std::set<types::boundary_id> &boundary_ids,
+                                      const ComponentMask &component_mask = ComponentMask());
 
   /**
    * Reset the data structures.
@@ -128,9 +145,11 @@ void
 MGConstrainedDoFs::initialize(const DoFHandler<dim,spacedim> &dof)
 {
   boundary_indices.clear();
+  refinement_edge_indices.clear();
 
   const unsigned int nlevels = dof.get_triangulation().n_global_levels();
 
+  //At this point refinement_edge_indices is empty.
   refinement_edge_indices.resize(nlevels);
   for (unsigned int l=0; l<nlevels; ++l)
     refinement_edge_indices[l] = IndexSet(dof.n_dofs(l));
@@ -151,6 +170,7 @@ MGConstrainedDoFs::initialize(const DoFHandler<dim,spacedim> &dof,
   // allocate an IndexSet for each global level. Contents will be
   // overwritten inside make_boundary_list.
   const unsigned int n_levels = dof.get_triangulation().n_global_levels();
+  //At this point boundary_indices is empty.
   boundary_indices.resize(n_levels);
 
   MGTools::make_boundary_list (dof,
@@ -160,6 +180,27 @@ MGConstrainedDoFs::initialize(const DoFHandler<dim,spacedim> &dof,
 }
 
 
+template <int dim, int spacedim>
+inline
+void
+MGConstrainedDoFs::make_zero_boundary_constraints(const DoFHandler<dim,spacedim> &dof,
+                                                  const std::set<types::boundary_id> &boundary_ids,
+                                                  const ComponentMask &component_mask)
+{
+  // allocate an IndexSet for each global level. Contents will be
+  // overwritten inside make_boundary_list.
+  const unsigned int n_levels = dof.get_triangulation().n_global_levels();
+  Assert(boundary_indices.size() == 0 || boundary_indices.size() == n_levels,
+         ExcInternalError());
+  boundary_indices.resize(n_levels);
+
+  MGTools::make_boundary_list (dof,
+                               boundary_ids,
+                               boundary_indices,
+                               component_mask);
+}
+
+
 inline
 void
 MGConstrainedDoFs::clear()
