@@ -1,7 +1,6 @@
 // ---------------------------------------------------------------------
-// $Id: mg_dof_accessor.h 30036 2013-07-18 16:55:32Z maier $
 //
-// Copyright (C) 1998 - 2013 by the deal.II authors
+// Copyright (C) 1998 - 2015 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef __deal2__mg_dof_accessor_h
-#define __deal2__mg_dof_accessor_h
+#ifndef dealii__mg_dof_accessor_h
+#define dealii__mg_dof_accessor_h
 
 #warning This file is deprecated. Use <deal.II/dofs/dof_accessor.h> instead.
 #include <deal.II/dofs/dof_accessor.h>
