@@ -72,6 +72,8 @@ namespace mg
     virtual void vmult_add (const unsigned int level, VectorType &dst, const VectorType &src) const;
     virtual void Tvmult (const unsigned int level, VectorType &dst, const VectorType &src) const;
     virtual void Tvmult_add (const unsigned int level, VectorType &dst, const VectorType &src) const;
+    virtual unsigned int get_minlevel() const;
+    virtual unsigned int get_maxlevel() const;
 
     /**
      * Memory used by this object.
@@ -108,7 +110,7 @@ public:
 
   /**
    * Set the matrix object to be used. The matrix object must exist longer as
-   * the @p MGMatrix object, since only a pointer is stored.
+   * the @p MGMatrixSelect object, since only a pointer is stored.
    */
   void set_matrix (MGLevelObject<MatrixType> *M);
 
@@ -181,6 +183,7 @@ namespace mg
   }
 
 
+
   template <typename VectorType>
   template <typename MatrixType>
   inline
@@ -190,12 +193,14 @@ namespace mg
   }
 
 
+
   template <typename VectorType>
   inline
   Matrix<VectorType>::Matrix ()
   {}
 
 
+
   template <typename VectorType>
   inline
   const PointerMatrixBase<VectorType> &
@@ -205,6 +210,7 @@ namespace mg
   }
 
 
+
   template <typename VectorType>
   void
   Matrix<VectorType>::vmult (const unsigned int level,
@@ -215,6 +221,7 @@ namespace mg
   }
 
 
+
   template <typename VectorType>
   void
   Matrix<VectorType>::vmult_add (const unsigned int level,
@@ -225,6 +232,7 @@ namespace mg
   }
 
 
+
   template <typename VectorType>
   void
   Matrix<VectorType>::Tvmult (const unsigned int level,
@@ -235,6 +243,7 @@ namespace mg
   }
 
 
+
   template <typename VectorType>
   void
   Matrix<VectorType>::Tvmult_add (const unsigned int level,
@@ -245,6 +254,25 @@ namespace mg
   }
 
 
+
+  template <typename VectorType>
+  unsigned int
+  Matrix<VectorType>::get_minlevel() const
+  {
+    return matrices.min_level();
+  }
+
+
+
+  template <typename VectorType>
+  unsigned int
+  Matrix<VectorType>::get_maxlevel() const
+  {
+    return matrices.max_level();
+  }
+
+
+
   template <typename VectorType>
   inline
   std::size_t
@@ -278,6 +306,7 @@ MGMatrixSelect<MatrixType, number>::set_matrix (MGLevelObject<MatrixType> *p)
 }
 
 
+
 template <typename MatrixType, typename number>
 void
 MGMatrixSelect<MatrixType, number>::
@@ -289,6 +318,7 @@ select_block (const unsigned int brow,
 }
 
 
+
 template <typename MatrixType, typename number>
 void
 MGMatrixSelect<MatrixType, number>::
@@ -303,6 +333,7 @@ vmult  (const unsigned int    level,
 }
 
 
+
 template <typename MatrixType, typename number>
 void
 MGMatrixSelect<MatrixType, number>::
@@ -317,6 +348,7 @@ vmult_add  (const unsigned int    level,
 }
 
 
+
 template <typename MatrixType, typename number>
 void
 MGMatrixSelect<MatrixType, number>::
@@ -331,6 +363,7 @@ Tvmult  (const unsigned int    level,
 }
 
 
+
 template <typename MatrixType, typename number>
 void
 MGMatrixSelect<MatrixType, number>::
