@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2005 - 2015 by the deal.II authors
+// Copyright (C) 2005 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -27,14 +27,7 @@
 
 DEAL_II_NAMESPACE_OPEN
 
-template <class Object> class MGLevelObject;
 template <int dim, int spacedim> class DoFHandler;
-template <typename number> class Vector;
-template <typename number> class SparseMatrix;
-template <typename number> class BlockVector;
-template <typename number> class BlockSparseMatrix;
-template <typename number> class FullMatrix;
-template <typename number> class BlockSparseMatrix;
 
 /* !@addtogroup mg */
 /* @{ */
@@ -185,6 +178,9 @@ namespace MGTools
    * indices of degrees of freedom for each level that are at the part of the
    * boundary identified by the function_map argument. Its length has to match
    * the number of levels in the dof handler object.
+   *
+   * Previous content in @p boundary_indices is not overwritten,
+   * but added to.
    */
   template <int dim, int spacedim>
   void
@@ -196,6 +192,9 @@ namespace MGTools
   /**
    * The same function as above, but return an IndexSet rather than a
    * std::set<unsigned int> on each level.
+   *
+   * Previous content in @p boundary_indices is not overwritten,
+   * but added to.
    */
   template <int dim, int spacedim>
   void
@@ -204,6 +203,20 @@ namespace MGTools
                       std::vector<IndexSet>                 &boundary_indices,
                       const ComponentMask               &component_mask = ComponentMask());
 
+  /**
+   * The same function as above, but return an IndexSet rather than a
+   * std::set<unsigned int> on each level and use a std::set of boundary_ids
+   * as input.
+   *
+   * Previous content in @p boundary_indices is not overwritten, but added to.
+   */
+  template <int dim, int spacedim>
+  void
+  make_boundary_list (const DoFHandler<dim,spacedim>      &mg_dof,
+                      const std::set<types::boundary_id> &boundary_ids,
+                      std::vector<IndexSet>                 &boundary_indices,
+                      const ComponentMask               &component_mask = ComponentMask());
+
   /**
    * For each level in a multigrid hierarchy, produce an IndexSet that
    * indicates which of the degrees of freedom are along interfaces of this
