@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2001 - 2016 by the deal.II authors
+// Copyright (C) 2001 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -23,7 +23,7 @@
 #include <deal.II/lac/sparse_matrix.h>
 #include <deal.II/lac/block_sparsity_pattern.h>
 #include <deal.II/lac/trilinos_sparse_matrix.h>
-#include <deal.II/lac/parallel_vector.h>
+#include <deal.II/lac/la_parallel_vector.h>
 
 #include <deal.II/lac/vector_memory.h>
 
@@ -58,7 +58,7 @@ namespace internal
 
 #ifdef DEAL_II_WITH_TRILINOS
   template <typename Number>
-  struct MatrixSelector<parallel::distributed::Vector<Number> >
+  struct MatrixSelector<LinearAlgebra::distributed::Vector<Number> >
   {
     typedef ::dealii::TrilinosWrappers::SparsityPattern Sparsity;
     typedef ::dealii::TrilinosWrappers::SparseMatrix Matrix;
@@ -99,6 +99,23 @@ namespace internal
     {
     }
   };
+#else
+  // ! DEAL_II_WITH_TRILINOS
+  template <typename Number>
+  struct MatrixSelector<LinearAlgebra::distributed::Vector<Number> >
+  {
+    typedef ::dealii::SparsityPattern Sparsity;
+    typedef ::dealii::SparseMatrix<Number> Matrix;
+
+    template <typename SparsityPatternType, typename DoFHandlerType>
+    static void reinit(Matrix &, Sparsity &, int, const SparsityPatternType &, const DoFHandlerType &)
+    {
+      AssertThrow(false, ExcNotImplemented(
+                    "ERROR: MGTransferPrebuilt with LinearAlgebra::distributed::Vector currently "
+                    "needs deal.II to be configured with Trilinos."));
+    }
+  };
+
 #endif
 }
 
@@ -259,7 +276,7 @@ protected:
  * Implementation of transfer between the global vectors and the multigrid
  * levels for use in the derived class MGTransferPrebuilt and other classes.
  * This class is a specialization for the case of
- * parallel::distributed::Vector that requires a few different calling
+ * LinearAlgebra::distributed::Vector that requires a few different calling
  * routines as compared to the %parallel vectors in the PETScWrappers and
  * TrilinosWrappers namespaces.
  *
@@ -267,7 +284,7 @@ protected:
  * @date 2016
  */
 template <typename Number>
-class MGLevelGlobalTransfer<parallel::distributed::Vector<Number> > : public MGTransferBase<parallel::distributed::Vector<Number> >
+class MGLevelGlobalTransfer<LinearAlgebra::distributed::Vector<Number> > : public MGTransferBase<LinearAlgebra::distributed::Vector<Number> >
 {
 public:
 
@@ -283,8 +300,8 @@ public:
   template <int dim, typename Number2, int spacedim>
   void
   copy_to_mg (const DoFHandler<dim,spacedim>                        &mg_dof,
-              MGLevelObject<parallel::distributed::Vector<Number> > &dst,
-              const parallel::distributed::Vector<Number2>          &src) const;
+              MGLevelObject<LinearAlgebra::distributed::Vector<Number> > &dst,
+              const LinearAlgebra::distributed::Vector<Number2>          &src) const;
 
   /**
    * Transfer from multi-level vector to normal vector.
@@ -296,8 +313,8 @@ public:
   template <int dim, typename Number2, int spacedim>
   void
   copy_from_mg (const DoFHandler<dim,spacedim>                              &mg_dof,
-                parallel::distributed::Vector<Number2>                      &dst,
-                const MGLevelObject<parallel::distributed::Vector<Number> > &src) const;
+                LinearAlgebra::distributed::Vector<Number2>                      &dst,
+                const MGLevelObject<LinearAlgebra::distributed::Vector<Number> > &src) const;
 
   /**
    * Add a multi-level vector to a normal vector.
@@ -307,8 +324,8 @@ public:
   template <int dim, typename Number2, int spacedim>
   void
   copy_from_mg_add (const DoFHandler<dim,spacedim>                              &mg_dof,
-                    parallel::distributed::Vector<Number2>                      &dst,
-                    const MGLevelObject<parallel::distributed::Vector<Number> > &src) const;
+                    LinearAlgebra::distributed::Vector<Number2>                      &dst,
+                    const MGLevelObject<LinearAlgebra::distributed::Vector<Number> > &src) const;
 
   /**
    * If this object operates on BlockVector objects, we need to describe how
@@ -398,20 +415,20 @@ protected:
   /**
    * The mg_constrained_dofs of the level systems.
    */
-  SmartPointer<const MGConstrainedDoFs, MGLevelGlobalTransfer<parallel::distributed::Vector<Number> > > mg_constrained_dofs;
+  SmartPointer<const MGConstrainedDoFs, MGLevelGlobalTransfer<LinearAlgebra::distributed::Vector<Number> > > mg_constrained_dofs;
 
   /**
    * In the function copy_to_mg, we need to access ghosted entries of the
    * global vector for inserting into the level vectors. This vector is
    * populated with those entries.
    */
-  mutable parallel::distributed::Vector<Number> ghosted_global_vector;
+  mutable LinearAlgebra::distributed::Vector<Number> ghosted_global_vector;
 
   /**
    * In the function copy_from_mg, we access all level vectors with certain
    * ghost entries for inserting the result into a global vector.
    */
-  mutable MGLevelObject<parallel::distributed::Vector<Number> > ghosted_level_vector;
+  mutable MGLevelObject<LinearAlgebra::distributed::Vector<Number> > ghosted_level_vector;
 };
 
 
@@ -443,8 +460,16 @@ public:
    * Constructor with constraints. Equivalent to the default constructor
    * followed by initialize_constraints().
    */
+  MGTransferPrebuilt (const MGConstrainedDoFs &mg_constrained_dofs);
+
+  /**
+   * Constructor with constraints. Equivalent to the default constructor
+   * followed by initialize_constraints().
+   *
+   * @deprecated @p constraints is unused.
+   */
   MGTransferPrebuilt (const ConstraintMatrix &constraints,
-                      const MGConstrainedDoFs &mg_constrained_dofs);
+                      const MGConstrainedDoFs &mg_constrained_dofs) DEAL_II_DEPRECATED;
 
   /**
    * Destructor.
@@ -454,8 +479,15 @@ public:
   /**
    * Initialize the constraints to be used in build_matrices().
    */
+  void initialize_constraints (const MGConstrainedDoFs &mg_constrained_dofs);
+
+  /**
+   * Initialize the constraints to be used in build_matrices().
+   *
+   * @deprecated @p constraints is unused.
+   */
   void initialize_constraints (const ConstraintMatrix &constraints,
-                               const MGConstrainedDoFs &mg_constrained_dofs);
+                               const MGConstrainedDoFs &mg_constrained_dofs) DEAL_II_DEPRECATED;
 
   /**
    * Reset the object to the state it had right after the default constructor.
@@ -541,11 +573,6 @@ private:
    * boundary.
    */
   std::vector<std::vector<bool> > interface_dofs;
-
-  /**
-   * The constraints of the global system.
-   */
-  SmartPointer<const ConstraintMatrix, MGTransferPrebuilt<VectorType> > constraints;
 };
 
 
