@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2003 - 2015 by the deal.II authors
+// Copyright (C) 2003 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -181,12 +181,6 @@ MGTransferSelect<number>::do_copy_from_mg_add (
   OutVector                            &dst,
   const MGLevelObject<Vector<number> > &src) const
 {
-  const FiniteElement<dim> &fe = mg_dof_handler.get_fe();
-  const unsigned int dofs_per_cell = fe.dofs_per_cell;
-
-  std::vector<types::global_dof_index> global_dof_indices (dofs_per_cell);
-  std::vector<types::global_dof_index> level_dof_indices (dofs_per_cell);
-
   typename DoFHandler<dim,spacedim>::active_cell_iterator
   level_cell = mg_dof_handler.begin_active();
   const typename DoFHandler<dim,spacedim>::active_cell_iterator
