@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2015 by the deal.II authors
+// Copyright (C) 1999 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -23,78 +23,56 @@
 
 DEAL_II_NAMESPACE_OPEN
 
-
-template <typename VectorType>
-Multigrid<VectorType>::Multigrid (const unsigned int                    minlevel,
-                                  const unsigned int                    maxlevel,
-                                  const MGMatrixBase<VectorType>        &matrix,
-                                  const MGCoarseGridBase<VectorType>    &coarse,
-                                  const MGTransferBase<VectorType>      &transfer,
-                                  const MGSmootherBase<VectorType>      &pre_smooth,
-                                  const MGSmootherBase<VectorType>      &post_smooth,
-                                  typename Multigrid<VectorType>::Cycle cycle)
-  :
-  cycle_type(cycle),
-  minlevel(minlevel),
-  maxlevel(maxlevel),
-  defect(minlevel,maxlevel),
-  solution(minlevel,maxlevel),
-  t(minlevel,maxlevel),
-  matrix(&matrix, typeid(*this).name()),
-  coarse(&coarse, typeid(*this).name()),
-  transfer(&transfer, typeid(*this).name()),
-  pre_smooth(&pre_smooth, typeid(*this).name()),
-  post_smooth(&post_smooth, typeid(*this).name()),
-  edge_out(0, typeid(*this).name()),
-  edge_in(0, typeid(*this).name()),
-  edge_down(0, typeid(*this).name()),
-  edge_up(0, typeid(*this).name()),
-  debug(0)
-{}
-
-
-
 template <typename VectorType>
 void
 Multigrid<VectorType>::reinit (const unsigned int min_level,
                                const unsigned int max_level)
 {
+  Assert (min_level >= matrix->get_minlevel(),
+          ExcLowerRangeType<unsigned int>(min_level, matrix->get_minlevel()));
+  Assert (max_level <= matrix->get_maxlevel(),
+          ExcLowerRangeType<unsigned int>(matrix->get_maxlevel(), max_level));
+  Assert (min_level <= max_level,
+          ExcLowerRangeType<unsigned int>(max_level, min_level));
   minlevel=min_level;
   maxlevel=max_level;
+  // solution, t and defect2 are resized in cycle()
   defect.resize(minlevel, maxlevel);
 }
 
 
+
 template <typename VectorType>
 void
 Multigrid<VectorType>::set_maxlevel (const unsigned int l)
 {
-  Assert (l <= maxlevel, ExcIndexRange(l,minlevel,maxlevel+1));
-  Assert (l >= minlevel, ExcIndexRange(l,minlevel,maxlevel+1));
-  maxlevel = l;
+  reinit(minlevel, l);
 }
 
 
+
 template <typename VectorType>
 void
 Multigrid<VectorType>::set_minlevel (const unsigned int l,
                                      const bool relative)
 {
-  Assert (l <= maxlevel, ExcIndexRange(l,minlevel,maxlevel+1));
-  minlevel = (relative)
-             ? (maxlevel-l)
-             : l;
+  const unsigned int new_minlevel = (relative)
+                                    ? (maxlevel-l)
+                                    : l;
+  reinit(new_minlevel, maxlevel);
 }
 
 
+
 template <typename VectorType>
 void
-Multigrid<VectorType>::set_cycle(typename Multigrid<VectorType>::Cycle c)
+Multigrid<VectorType>::set_cycle (typename Multigrid<VectorType>::Cycle c)
 {
   cycle_type = c;
 }
 
 
+
 template <typename VectorType>
 void
 Multigrid<VectorType>::set_debug (const unsigned int d)
@@ -103,6 +81,7 @@ Multigrid<VectorType>::set_debug (const unsigned int d)
 }
 
 
+
 template <typename VectorType>
 void
 Multigrid<VectorType>::set_edge_matrices (const MGMatrixBase<VectorType> &down,
@@ -113,6 +92,7 @@ Multigrid<VectorType>::set_edge_matrices (const MGMatrixBase<VectorType> &down,
 }
 
 
+
 template <typename VectorType>
 void
 Multigrid<VectorType>::set_edge_flux_matrices (const MGMatrixBase<VectorType> &down,
@@ -123,6 +103,7 @@ Multigrid<VectorType>::set_edge_flux_matrices (const MGMatrixBase<VectorType> &d
 }
 
 
+
 template <typename VectorType>
 void
 Multigrid<VectorType>::level_v_step (const unsigned int level)
@@ -140,79 +121,56 @@ Multigrid<VectorType>::level_v_step (const unsigned int level)
       (*coarse)(level, solution[level], defect[level]);
       return;
     }
+
+  // smoothing of the residual
   if (debug>1)
     deallog << "Smoothing on     level " << level << std::endl;
-  // smoothing of the residual by
-  // modifying s
-//  defect[level].print(std::cout, 2,false);
-//  std::cout<<std::endl;
   pre_smooth->smooth(level, solution[level], defect[level]);
-//  solution[level].print(std::cout, 2,false);
 
   if (debug>2)
     deallog << "Solution norm          " << solution[level].l2_norm()
             << std::endl;
 
+  // compute residual on level, which includes the (CG) edge matrix
   if (debug>1)
     deallog << "Residual on      level " << level << std::endl;
-  // t = A*solution[level]
   matrix->vmult(level, t[level], solution[level]);
+  if (edge_out != 0)
+    {
+      edge_out->vmult_add(level, t[level], solution[level]);
+      if (debug>2)
+        deallog << "Norm     t[" << level << "] " << t[level].l2_norm() << std::endl;
+    }
+  t[level].sadd(-1.0, 1.0, defect[level]);
 
   if (debug>2)
     deallog << "Residual norm          " << t[level].l2_norm()
             << std::endl;
-//  std::cout<<std::endl;
-//  t[level].print(std::cout, 2,false);
 
-  // make t rhs of lower level The
-  // non-refined parts of the
-  // coarse-level defect already
-  // contain the global defect, the
-  // refined parts its restriction.
-  for (unsigned int l = level; l>minlevel; --l)
+  // Get the defect on the next coarser level as part of the (DG) edge matrix
+  // and then the main part by the restriction of the transfer
+  if (edge_down != 0)
     {
-      t[l-1] = 0.;
-      if (l==level && edge_out != 0)
-        {
-          edge_out->vmult_add(level, t[level], solution[level]);
-          if (debug>2)
-            deallog << "Norm     t[" << level << "] " << t[level].l2_norm() << std::endl;
-        }
-
-      if (l==level && edge_down != 0)
-        edge_down->vmult(level, t[level-1], solution[level]);
-
-      transfer->restrict_and_add (l, t[l-1], t[l]);
-      if (debug>3)
-        deallog << "restrict t[" << l-1 << "] " << t[l-1].l2_norm() << std::endl;
-      defect[l-1] -= t[l-1];
-      if (debug>3)
-        deallog << "defect   d[" << l-1 << "] " << defect[l-1].l2_norm() << std::endl;
+      edge_down->vmult(level, t[level-1], solution[level]);
+      defect[level-1] -= t[level-1];
     }
+  transfer->restrict_and_add(level, defect[level-1], t[level]);
 
   // do recursion
-  solution[level-1] = 0.;
   level_v_step(level-1);
 
-  // reset size of the auxiliary
-  // vector, since it has been
-  // resized in the recursive call to
-  // level_v_step directly above
-  t[level] = 0.;
-
   // do coarse grid correction
   transfer->prolongate(level, t[level], solution[level-1]);
   if (debug>2)
     deallog << "Prolongate norm        " << t[level].l2_norm() << std::endl;
-
   solution[level] += t[level];
 
+  // get in contribution from edge matrices to the defect
   if (edge_in != 0)
     {
       edge_in->Tvmult(level, t[level], solution[level]);
       defect[level] -= t[level];
     }
-
   if (edge_up != 0)
     {
       edge_up->Tvmult(level, t[level], solution[level-1]);
@@ -223,15 +181,10 @@ Multigrid<VectorType>::level_v_step (const unsigned int level)
     deallog << "V-cycle  Defect norm   " << defect[level].l2_norm()
             << std::endl;
 
+  // post-smoothing
   if (debug>1)
     deallog << "Smoothing on     level " << level << std::endl;
-  // post-smoothing
-
-//  std::cout<<std::endl;
-//  defect[level].print(std::cout, 2,false);
   post_smooth->smooth(level, solution[level], defect[level]);
-//  solution[level].print(std::cout, 2,false);
-//  std::cout<<std::endl;
 
   if (debug>2)
     deallog << "Solution norm          " << solution[level].l2_norm()
@@ -260,126 +213,123 @@ Multigrid<VectorType>::level_step(const unsigned int level,
     case w_cycle:
       cychar = 'W';
       break;
+    default:
+      Assert(false, ExcNotImplemented());
     }
 
   if (debug>0)
-    deallog << cychar << "-cycle entering level " << level << std::endl;
+    deallog << cychar << "-cycle entering level  " << level << std::endl;
 
-  // Not actually the defect yet, but
-  // we do not want to spend yet
-  // another vector.
-  if (level>minlevel)
-    {
-      defect2[level-1] = 0.;
-      transfer->restrict_and_add (level, defect2[level-1], defect2[level]);
-    }
-
-  // We get an update of the defect
-  // from the previous level in t and
-  // from two levels above in
-  // defect2. This is subtracted from
-  // the original defect.
-  t[level].equ(-1.,defect2[level],1.,defect[level]);
+  // Combine the defect from the initial copy_to_mg with the one that has come
+  // from the finer level by the transfer
+  defect2[level] += defect[level];
+  defect[level] = 0;
 
   if (debug>2)
-    deallog << cychar << "-cycle defect norm    " << t[level].l2_norm()
+    deallog << cychar << "-cycle defect norm     " << defect2[level].l2_norm()
             << std::endl;
 
   if (level == minlevel)
     {
       if (debug>0)
-        deallog << cychar << "-cycle coarse level   " << level << std::endl;
+        deallog << cychar << "-cycle coarse level    " << level << std::endl;
 
-      (*coarse)(level, solution[level], t[level]);
+      (*coarse)(level, solution[level], defect2[level]);
       return;
     }
+
+  // smoothing of the residual
   if (debug>1)
     deallog << cychar << "-cycle smoothing level " << level << std::endl;
-  // smoothing of the residual by
-  // modifying s
-  pre_smooth->smooth(level, solution[level], t[level]);
+  pre_smooth->smooth(level, solution[level], defect2[level]);
 
   if (debug>2)
-    deallog << cychar << "-cycle solution norm    "
+    deallog << cychar << "-cycle solution norm   "
             << solution[level].l2_norm() << std::endl;
 
+  // compute residual on level, which includes the (CG) edge matrix
   if (debug>1)
-    deallog << cychar << "-cycle residual level   " << level << std::endl;
-  // t = A*solution[level]
+    deallog << cychar << "-cycle residual level  " << level << std::endl;
   matrix->vmult(level, t[level], solution[level]);
-  // make t rhs of lower level The
-  // non-refined parts of the
-  // coarse-level defect already
-  // contain the global defect, the
-  // refined parts its restriction.
   if (edge_out != 0)
     edge_out->vmult_add(level, t[level], solution[level]);
-
-  if (edge_down != 0)
-    edge_down->vmult_add(level, defect2[level-1], solution[level]);
-
-  transfer->restrict_and_add (level, defect2[level-1], t[level]);
-  // do recursion
-  solution[level-1] = 0.;
-  // Every cycle need one recursion,
-  // the V-cycle, which is included
-  // here for the sake of the
-  // F-cycle, needs only one,
-  level_step(level-1, cycle);
-  // If we solve exactly, then we do
-  // not need a second coarse grid
-  // step.
-  if (level>minlevel+1)
-    {
-      // while the W-cycle repeats itself
-      if (cycle == w_cycle)
-        level_step(level-1, cycle);
-      // and the F-cycle does a V-cycle
-      // after an F-cycle.
-      else if (cycle == f_cycle)
-        level_step(level-1, v_cycle);
-    }
-
-  // reset size of the auxiliary
-  // vector, since it has been
-  // resized in the recursive call to
-  // level_v_step directly above
-  t[level] = 0.;
-  // do coarse grid correction
-  transfer->prolongate(level, t[level], solution[level-1]);
-
-  solution[level] += t[level];
-
-
-  if (edge_in != 0)
-    edge_in->Tvmult(level, t[level], solution[level]);
-
-  if (edge_up != 0)
-    edge_up->Tvmult(level, t[level], solution[level-1]);
-
-  t[level].sadd(-1.,-1.,defect2[level],1.,defect[level]);
+  t[level].sadd(-1.0, 1.0, defect2[level]);
 
   if (debug>2)
-    deallog << cychar << "-cycle  Defect norm    " << t[level].l2_norm()
+    deallog << cychar << "-cycle residual norm   " << t[level].l2_norm()
             << std::endl;
 
+  // Get the defect on the next coarser level as part of the (DG) edge matrix
+  // and then the main part by the restriction of the transfer
+  if (edge_down != 0)
+    edge_down->vmult(level, defect2[level-1], solution[level]);
+  else
+    defect2[level-1] = 0;
+
+  transfer->restrict_and_add (level, defect2[level-1], t[level]);
+
+  // Every cycle starts with a recursion of its type. As opposed to the
+  // V-cycle above where we do not need to set the solution vector to zero (as
+  // that has been done in the init functions), we need to zero out the values
+  // from previous cycles.
+  solution[level-1] = 0;
+  level_step(level-1, cycle);
+
+  // For W and F-cycle, repeat the process on the next coarser level except
+  // for the coarse solver which we invoke just once
+  if (level>minlevel+1)
+    {
+      // while the W-cycle repeats itself, ...
+      if (cycle == w_cycle)
+        level_step(level-1, cycle);
+      // ... the F-cycle does a V-cycle after an F-cycle, ...
+      else if (cycle == f_cycle)
+        level_step(level-1, v_cycle);
+      // ... and the V-cycle does nothing.
+    }
+
+  // do coarse grid correction
+  transfer->prolongate(level, t[level], solution[level-1]);
+  solution[level] += t[level];
+
+  // get in contribution from edge matrices to the defect
+  if (edge_in != 0)
+    {
+      edge_in->Tvmult(level, t[level], solution[level]);
+      defect2[level] -= t[level];
+    }
+
+  if (edge_up != 0)
+    {
+      edge_up->Tvmult(level, t[level], solution[level-1]);
+      defect2[level] -= t[level];
+    }
+
+  if (debug>2)
+    deallog << cychar << "-cycle  Defect norm    " << defect2[level].l2_norm()
+            << std::endl;
+
+  // post-smoothing
   if (debug>1)
     deallog << cychar << "-cycle smoothing level " << level << std::endl;
-  // post-smoothing
-  post_smooth->smooth(level, solution[level], t[level]);
+  post_smooth->smooth(level, solution[level], defect2[level]);
+
+  if (debug>2)
+    deallog << cychar << "-cycle solution norm   " << solution[level].l2_norm()
+            << std::endl;
 
   if (debug>1)
     deallog << cychar << "-cycle leaving level   " << level << std::endl;
 }
 
 
+
 template <typename VectorType>
 void
 Multigrid<VectorType>::cycle()
 {
-  // The defect vector has been
-  // initialized by copy_to_mg. Now
-  // adjust the other vectors.
+  // The defect vector has been initialized by copy_to_mg. Now adjust the
+  // other vectors.
   solution.resize(minlevel, maxlevel);
   t.resize(minlevel, maxlevel);
   if (cycle_type != v_cycle)
@@ -400,13 +350,13 @@ Multigrid<VectorType>::cycle()
 }
 
 
+
 template <typename VectorType>
 void
 Multigrid<VectorType>::vcycle()
 {
-  // The defect vector has been
-  // initialized by copy_to_mg. Now
-  // adjust the other vectors.
+  // The defect vector has been initialized by copy_to_mg. Now adjust the
+  // other vectors.
   solution.resize(minlevel, maxlevel);
   t.resize(minlevel, maxlevel);
 
