@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2015 by the deal.II authors
+// Copyright (C) 2000 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -31,14 +31,6 @@
 
 DEAL_II_NAMESPACE_OPEN
 
-template <int dim, int spacedim> class DoFHandler;
-namespace hp
-{
-  template <int dim, int spacedim> class DoFHandler;
-}
-
-
-
 /**
  * This namespace provides functions that compute a cell-wise approximation of
  * the norm of a derivative of a finite element field by taking difference
@@ -187,7 +179,7 @@ namespace DerivativeApproximation
                         const unsigned int                  component = 0);
 
   /**
-   * Calls the @p interpolate function, see above, with
+   * Call the @p interpolate function, see above, with
    * <tt>mapping=MappingQGeneric@<dim@>(1)</tt>.
    */
   template <int dim, template <int, int> class DoFHandlerType, class InputVector, int spacedim>
@@ -223,7 +215,7 @@ namespace DerivativeApproximation
                                  const unsigned int                  component = 0);
 
   /**
-   * Calls the @p interpolate function, see above, with
+   * Call the @p interpolate function, see above, with
    * <tt>mapping=MappingQGeneric@<dim@>(1)</tt>.
    */
   template <int dim, template <int, int> class DoFHandlerType, class InputVector, int spacedim>
@@ -246,24 +238,24 @@ namespace DerivativeApproximation
    * In a parallel computation the @p solution vector needs to contain the
    * locally relevant unknowns.
    */
-  template <typename DoFHandlerType, int dim, int spacedim, class InputVector, int order>
+  template <typename DoFHandlerType, class InputVector, int order>
   void
   approximate_derivative_tensor
-  (const Mapping<dim, spacedim>                        &mapping,
-   const DoFHandlerType                                &dof,
-   const InputVector                                   &solution,
+  (const Mapping<DoFHandlerType::dimension, DoFHandlerType::space_dimension>  &mapping,
+   const DoFHandlerType                                                       &dof,
+   const InputVector                                                          &solution,
 #ifndef _MSC_VER
-   const typename DoFHandlerType::active_cell_iterator &cell,
+   const typename DoFHandlerType::active_cell_iterator                        &cell,
 #else
    const TriaActiveIterator <dealii::DoFCellAccessor<DoFHandlerType, false> > &cell,
 #endif
-   Tensor<order, dim>                                  &derivative,
-   const unsigned int                                   component = 0);
+   Tensor<order, DoFHandlerType::dimension>                                   &derivative,
+   const unsigned int                                                         component = 0);
 
   /**
    * Same as above, with <tt>mapping=MappingQGeneric@<dim@>(1)</tt>.
    */
-  template <typename DoFHandlerType, int dim, int spacedim, class InputVector, int order>
+  template <typename DoFHandlerType, class InputVector, int order>
   void
   approximate_derivative_tensor
   (const DoFHandlerType                                &dof,
@@ -273,7 +265,7 @@ namespace DerivativeApproximation
 #else
    const TriaActiveIterator<dealii::DoFCellAccessor<DoFHandlerType, false> > &cell,
 #endif
-   Tensor<order, dim>                                  &derivative,
+   Tensor<order, DoFHandlerType::dimension>            &derivative,
    const unsigned int                                   component = 0);
 
   /**
