@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2015 by the deal.II authors
+// Copyright (C) 1998 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -27,13 +27,11 @@
 DEAL_II_NAMESPACE_OPEN
 
 
-template <int, int> class DoFHandler;
 template <int, int> class Mapping;
 template <int> class Quadrature;
 
 namespace hp
 {
-  template <int, int> class DoFHandler;
   template <int> class QCollection;
 }
 
@@ -95,12 +93,12 @@ namespace hp
  * \eta_K^2 = \sum_{F\in\partial K} c_F \int_{\partial K_F} \left[a
  * \frac{\partial u_h}{\partial n}\right]^2 do \f] be the error estimator for
  * cell $K$. $[\cdot]$ denotes the jump of the argument at the face. In the
- * paper of Ainsworth $ c_F=\frac h{24} $, but this factor is a bit esoteric,
+ * paper of Ainsworth $ c_F=\frac {h_K}{24} $, but this factor is a bit esoteric,
  * stemming from interpolation estimates and stability constants which may
  * hold for the Poisson problem, but may not hold for more general situations.
  * Alternatively, we consider the case when $ c_F=\frac {h_F}{2p_F} $, where $
  * h_F $ is face diagonal and $ p_F=max(p^+,p^-) $ is the maximum polynomial
- * degree of adjacent elements. The choice between the two is done by means of
+ * degree of adjacent elements; or $c_F=h_K$. The choice between these factors is done by means of
  * the enumerator, provided as the last argument in all functions.
  *
  * To perform the integration, use is made of the FEFaceValues and
@@ -112,21 +110,21 @@ namespace hp
  * times some factor) of each cell and take the square root.
  *
  * The integration is done using a quadrature formula on the face. For linear
- * trial functions (FEQ1), the QGauss2 or even the QMidpoint rule will
- * suffice. For higher order elements, it is necessary to utilize higher order
- * quadrature formulae as well.
+ * trial functions (FEQ1), QGauss (with two points) or even the QMidpoint rule
+ * will suffice. For higher order elements, it is necessary to utilize higher
+ * order quadrature formulae as well.
  *
  * We store the contribution of each face in a @p map, as provided by the C++
  * standard library, with the iterator pointing to that face being the key
  * into the map. When looping the second time over all cells, we have to sum
  * up the contributions of the faces and take the square root. For the Kelly
- * estimator, the multiplication with $\frac h{24}$ is done in the second
- * loop. By doing so we avoid problems to decide with which $h$ to multiply,
+ * estimator, the multiplication with $\frac {h_K}{24}$ is done in the second
+ * loop. By doing so we avoid problems to decide with which $h_K$ to multiply,
  * that of the cell on the one or that of the cell on the other side of the
  * face. Whereas for the hp-estimator the @p map stores integrals multiplied
  * by $\frac {h_F}{2p_F}$, which are then summed in the second loop.
  *
- * $h$ ($h_F$) is taken to be the greatest length of the diagonals of the cell
+ * $h_K$ ($h_F$) is taken to be the greatest length of the diagonals of the cell
  * (face). For more or less uniform cells (faces) without deformed angles,
  * this coincides with the diameter of the cell (face).
  *
@@ -178,7 +176,7 @@ namespace hp
  * <li> The face belongs to a Neumann boundary.  In this case, the
  * contribution of the face $F\in\partial K$ looks like \f[ n_F\int_F
  * \left|g-a\frac{\partial u_h}{\partial n}\right|^2 ds \f] where $g$ is the
- * Neumann boundary function, $n_F=\frac {h}{24}$ and $n_F=\frac {h_F}{p}$ for
+ * Neumann boundary function, $n_F=\frac {h_K}{24}$ and $n_F=\frac {h_F}{p}$ for
  * the Kelly and hp-estimator, respectively. If the finite element is vector-
  * valued, then obviously the function denoting the Neumann boundary
  * conditions needs to be vector-valued as well.
@@ -259,10 +257,12 @@ public:
    */
   enum Strategy
   {
-    //! Kelly error estimator with the factor $\frac {h}{24}$.
+    //! Kelly error estimator with the factor $\frac {h_K}{24}$.
     cell_diameter_over_24 = 0,
     //! the boundary residual estimator with the factor $\frac {h_F}{2 max(p^+,p^-)}$.
-    face_diameter_over_twice_max_degree
+    face_diameter_over_twice_max_degree,
+    //! Kelly error estimator with the factor $h_K$.
+    cell_diameter
   };
 
   /**
@@ -341,7 +341,7 @@ public:
    const Strategy                              strategy       = cell_diameter_over_24);
 
   /**
-   * Calls the @p estimate function, see above, with
+   * Call the @p estimate function, see above, with
    * <tt>mapping=MappingQGeneric@<dim@>(1)</tt>.
    */
   template <typename InputVector, typename DoFHandlerType>
@@ -387,7 +387,7 @@ public:
    const Strategy                              strategy       = cell_diameter_over_24);
 
   /**
-   * Calls the @p estimate function, see above, with
+   * Call the @p estimate function, see above, with
    * <tt>mapping=MappingQGeneric@<dim@>(1)</tt>.
    */
   template <typename InputVector, typename DoFHandlerType>
@@ -586,7 +586,7 @@ public:
    const types::material_id                    material_id    = numbers::invalid_material_id);
 
   /**
-   * Calls the @p estimate function, see above, with
+   * Call the @p estimate function, see above, with
    * <tt>mapping=MappingQGeneric1<1>()</tt>.
    */
   template <typename InputVector, typename DoFHandlerType>
@@ -630,7 +630,7 @@ public:
    const types::material_id                    material_id    = numbers::invalid_material_id);
 
   /**
-   * Calls the @p estimate function, see above, with
+   * Call the @p estimate function, see above, with
    * <tt>mapping=MappingQGeneric1<1>()</tt>.
    */
   template <typename InputVector, typename DoFHandlerType>
