@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2016 by the deal.II authors
+// Copyright (C) 1998 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,14 +13,19 @@
 //
 // ---------------------------------------------------------------------
 
+#ifndef dealii__error_estimator_templates_h
+#define dealii__error_estimator_templates_h
+
+#include <deal.II/base/numbers.h>
 #include <deal.II/base/thread_management.h>
 #include <deal.II/base/quadrature.h>
 #include <deal.II/base/quadrature_lib.h>
 #include <deal.II/base/work_stream.h>
 #include <deal.II/lac/vector.h>
-#include <deal.II/lac/parallel_vector.h>
+#include <deal.II/lac/la_vector.h>
+#include <deal.II/lac/la_parallel_vector.h>
 #include <deal.II/lac/block_vector.h>
-#include <deal.II/lac/parallel_block_vector.h>
+#include <deal.II/lac/la_parallel_block_vector.h>
 #include <deal.II/lac/petsc_vector.h>
 #include <deal.II/lac/petsc_block_vector.h>
 #include <deal.II/lac/trilinos_vector.h>
@@ -48,24 +53,6 @@
 
 DEAL_II_NAMESPACE_OPEN
 
-
-namespace
-{
-  template <typename CellIterator>
-  inline
-  void advance_by_n (CellIterator &cell,
-                     const unsigned int n)
-  {
-    // store a pointer to the end iterator, since we can't get at it any more
-    // once cell is already the end iterator (in that case dereferencing
-    // cell-> triggers an assertion)
-    const CellIterator endc = cell->get_dof_handler().end();
-    for (unsigned int t=0; ((t<n) && (cell!=endc)); ++t, ++cell)
-      ;
-  }
-}
-
-
 namespace internal
 {
   namespace
@@ -476,7 +463,7 @@ namespace internal
         for (unsigned int component=0; component<n_components; ++component)
           if (parallel_data.component_mask[component] == true)
             for (unsigned int p=0; p<n_q_points; ++p)
-              face_integral[n] += Utilities::fixed_power<2>(parallel_data.phi[n][p][component]) *
+              face_integral[n] += numbers::NumberTraits<number>::abs_square(parallel_data.phi[n][p][component]) *
                                   parallel_data.JxW_values[p];
 
       return face_integral;
@@ -498,6 +485,10 @@ namespace internal
         {
           return 1.0;
         }
+        case KellyErrorEstimator<DoFHandlerType::dimension,DoFHandlerType::space_dimension>::cell_diameter :
+        {
+          return 1.0;
+        }
         case KellyErrorEstimator<DoFHandlerType::dimension,DoFHandlerType::space_dimension>::face_diameter_over_twice_max_degree :
         {
           const double cell_degree = fe_face_values_cell.get_fe_collection()[cell->active_fe_index()].degree;
@@ -528,6 +519,10 @@ namespace internal
         {
           return 1.0;
         }
+        case KellyErrorEstimator<DoFHandlerType::dimension,DoFHandlerType::space_dimension>::cell_diameter :
+        {
+          return 1.0;
+        }
         case KellyErrorEstimator<DoFHandlerType::dimension,DoFHandlerType::space_dimension>::face_diameter_over_twice_max_degree :
         {
           const double cell_degree     = fe_face_values_cell.get_fe_collection()[cell->active_fe_index()].degree;
@@ -560,6 +555,10 @@ namespace internal
         {
           return 1.0;
         }
+        case KellyErrorEstimator<DoFHandlerType::dimension,DoFHandlerType::space_dimension>::cell_diameter :
+        {
+          return 1.0;
+        }
         case KellyErrorEstimator<DoFHandlerType::dimension,DoFHandlerType::space_dimension>::face_diameter_over_twice_max_degree :
         {
           const double cell_degree = fe_face_values.get_fe_collection()[cell->active_fe_index()].degree;
@@ -590,6 +589,10 @@ namespace internal
         {
           return cell->diameter()/24;
         }
+        case KellyErrorEstimator<DoFHandlerType::dimension,DoFHandlerType::space_dimension>::cell_diameter :
+        {
+          return cell->diameter();
+        }
         case KellyErrorEstimator<DoFHandlerType::dimension,DoFHandlerType::space_dimension>::face_diameter_over_twice_max_degree :
         {
           return 1.0;
@@ -1290,3 +1293,5 @@ void KellyErrorEstimator<dim, spacedim>::estimate
 }
 
 DEAL_II_NAMESPACE_CLOSE
+
+#endif
