@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2007 - 2015 by the deal.II authors
+// Copyright (C) 2007 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,6 +13,9 @@
 //
 // ---------------------------------------------------------------------
 
+#ifndef dealii__fe_field_function_templates_h
+#define dealii__fe_field_function_templates_h
+
 
 #include <deal.II/base/utilities.h>
 #include <deal.II/base/logstream.h>
@@ -37,12 +40,11 @@ namespace Functions
    const VectorType     &myv,
    const Mapping<dim>   &mymapping)
     :
-    Function<dim>(mydh.get_fe().n_components()),
+    Function<dim,typename VectorType::value_type>(mydh.get_fe().n_components()),
     dh(&mydh, "FEFieldFunction"),
     data_vector(myv),
     mapping(mymapping),
-    cell_hint(dh->end()),
-    n_components(mydh.get_fe().n_components())
+    cell_hint(dh->end())
   {
   }
 
@@ -60,10 +62,10 @@ namespace Functions
 
   template <int dim, typename DoFHandlerType, typename VectorType>
   void FEFieldFunction<dim, DoFHandlerType, VectorType>::vector_value (const Point<dim> &p,
-      Vector<double>   &values) const
+      Vector<typename VectorType::value_type>   &values) const
   {
-    Assert (values.size() == n_components,
-            ExcDimensionMismatch(values.size(), n_components));
+    Assert (values.size() == this->n_components,
+            ExcDimensionMismatch(values.size(), this->n_components));
     typename DoFHandlerType::active_cell_iterator cell = cell_hint.get();
     if (cell == dh->end())
       cell = dh->begin_active();
@@ -74,7 +76,7 @@ namespace Functions
       {
         const std::pair<typename dealii::internal::ActiveCellIterator<dim, dim, DoFHandlerType>::type, Point<dim> > my_pair
           = GridTools::find_active_cell_around_point (mapping, *dh, p);
-        AssertThrow (my_pair.first->is_locally_owned(),
+        AssertThrow (!my_pair.first->is_artificial(),
                      VectorTools::ExcPointNotAvailableHere());
 
         cell = my_pair.first;
@@ -83,6 +85,10 @@ namespace Functions
 
     cell_hint.get() = cell;
 
+    // check that the current cell is available:
+    AssertThrow (!cell->is_artificial(),
+                 VectorTools::ExcPointNotAvailableHere());
+
     // Now we can find out about the point
     Quadrature<dim> quad(qp.get());
     FEValues<dim> fe_v(mapping, cell->get_fe(), quad,
@@ -97,11 +103,11 @@ namespace Functions
 
 
   template <int dim, typename DoFHandlerType, typename VectorType>
-  double
+  typename VectorType::value_type
   FEFieldFunction<dim, DoFHandlerType, VectorType>::value (const Point<dim>   &p,
                                                            const unsigned int comp) const
   {
-    Vector<double> values(n_components);
+    Vector<typename VectorType::value_type> values(this->n_components);
     vector_value(p, values);
     return values(comp);
   }
@@ -112,10 +118,11 @@ namespace Functions
   void
   FEFieldFunction<dim, DoFHandlerType, VectorType>::
   vector_gradient (const Point<dim>            &p,
-                   std::vector<Tensor<1,dim> > &gradients) const
+                   std::vector<Tensor<1,dim,typename VectorType::value_type> > &gradients) const
   {
-    Assert (gradients.size() == n_components,
-            ExcDimensionMismatch(gradients.size(), n_components));
+    typedef typename VectorType::value_type number;
+    Assert (gradients.size() == this->n_components,
+            ExcDimensionMismatch(gradients.size(), this->n_components));
     typename DoFHandlerType::active_cell_iterator cell = cell_hint.get();
     if (cell == dh->end())
       cell = dh->begin_active();
@@ -126,13 +133,17 @@ namespace Functions
       {
         const std::pair<typename dealii::internal::ActiveCellIterator<dim, dim, DoFHandlerType>::type, Point<dim> > my_pair
           = GridTools::find_active_cell_around_point (mapping, *dh, p);
-        AssertThrow (my_pair.first->is_locally_owned(),
+        AssertThrow (!my_pair.first->is_artificial(),
                      VectorTools::ExcPointNotAvailableHere());
 
         cell = my_pair.first;
         qp = my_pair.second;
       }
 
+    // check that the current cell is available:
+    AssertThrow (!cell->is_artificial(),
+                 VectorTools::ExcPointNotAvailableHere());
+
     cell_hint.get() = cell;
 
     // Now we can find out about the point
@@ -141,24 +152,35 @@ namespace Functions
                        update_gradients);
     fe_v.reinit(cell);
 
-    // FIXME: we need a temp argument because get_function_values wants to put
-    // its data into an object storing the correct scalar type, but this
-    // function wants to return everything in a vector<double>
-    std::vector< std::vector<Tensor<1,dim,typename VectorType::value_type> > > vgrads
-    (1,  std::vector<Tensor<1,dim,typename VectorType::value_type> >(n_components) );
-    fe_v.get_function_gradients(data_vector, vgrads);
-    gradients = std::vector<Tensor<1,dim> >(vgrads[0].begin(), vgrads[0].end());
+    if (this->n_components == 1)
+      {
+        // the size of the @p gradients coincidentally coincides
+        // with the number of quadrature points we evaluate the function at.
+        fe_v.get_function_gradients(data_vector, gradients);
+      }
+    else
+      {
+        // Unfortunately we still need a temporary argument as we want to
+        // evaluate a gradient of a (generally) multicomponent function at
+        // a single quadrature point. Note that the first std::vector<> is related
+        // to the number of quadrature points (always one here), whereas the second
+        // to the number of components.
+        std::vector< std::vector<Tensor<1,dim,number> > > vgrads
+        (1,  std::vector<Tensor<1,dim,number> >(this->n_components) );
+        fe_v.get_function_gradients(data_vector, vgrads);
+        gradients = vgrads[0];
+      }
   }
 
 
 
   template <int dim, typename DoFHandlerType, typename VectorType>
-  Tensor<1,dim>
+  Tensor<1,dim,typename VectorType::value_type>
   FEFieldFunction<dim, DoFHandlerType, VectorType>::
   gradient (const Point<dim>   &p,
             const unsigned int comp) const
   {
-    std::vector<Tensor<1,dim> > grads(n_components);
+    std::vector<Tensor<1,dim,typename VectorType::value_type> > grads(this->n_components);
     vector_gradient(p, grads);
     return grads[comp];
   }
@@ -169,10 +191,10 @@ namespace Functions
   void
   FEFieldFunction<dim, DoFHandlerType, VectorType>::
   vector_laplacian (const Point<dim> &p,
-                    Vector<double>   &values) const
+                    Vector<typename VectorType::value_type>   &values) const
   {
-    Assert (values.size() == n_components,
-            ExcDimensionMismatch(values.size(), n_components));
+    Assert (values.size() == this->n_components,
+            ExcDimensionMismatch(values.size(), this->n_components));
     typename DoFHandlerType::active_cell_iterator cell = cell_hint.get();
     if (cell == dh->end())
       cell = dh->begin_active();
@@ -183,13 +205,17 @@ namespace Functions
       {
         const std::pair<typename dealii::internal::ActiveCellIterator<dim, dim, DoFHandlerType>::type, Point<dim> > my_pair
           = GridTools::find_active_cell_around_point (mapping, *dh, p);
-        AssertThrow (my_pair.first->is_locally_owned(),
+        AssertThrow (!my_pair.first->is_artificial(),
                      VectorTools::ExcPointNotAvailableHere());
 
         cell = my_pair.first;
         qp = my_pair.second;
       }
 
+    // check that the current cell is available:
+    AssertThrow (!cell->is_artificial(),
+                 VectorTools::ExcPointNotAvailableHere());
+
     cell_hint.get() = cell;
 
     // Now we can find out about the point
@@ -206,11 +232,11 @@ namespace Functions
 
 
   template <int dim, typename DoFHandlerType, typename VectorType>
-  double FEFieldFunction<dim, DoFHandlerType, VectorType>::laplacian
+  typename VectorType::value_type FEFieldFunction<dim, DoFHandlerType, VectorType>::laplacian
   (const Point<dim>   &p,
    const unsigned int  comp) const
   {
-    Vector<double> lap(n_components);
+    Vector<typename VectorType::value_type> lap(this->n_components);
     vector_laplacian(p, lap);
     return lap[comp];
   }
@@ -223,7 +249,7 @@ namespace Functions
   void
   FEFieldFunction<dim, DoFHandlerType, VectorType>::
   vector_value_list (const std::vector<Point< dim > > &points,
-                     std::vector< Vector<double> >    &values) const
+                     std::vector< Vector<typename VectorType::value_type> >    &values) const
   {
     Assert(points.size() == values.size(),
            ExcDimensionMismatch(points.size(), values.size()));
@@ -254,7 +280,7 @@ namespace Functions
       {
         fe_v.reinit(cells[i], i, 0);
         const unsigned int nq = qpoints[i].size();
-        std::vector< Vector<typename VectorType::value_type> > vvalues (nq, Vector<typename VectorType::value_type>(n_components));
+        std::vector< Vector<typename VectorType::value_type> > vvalues (nq, Vector<typename VectorType::value_type>(this->n_components));
         fe_v.get_present_fe_values ().get_function_values(data_vector, vvalues);
         for (unsigned int q=0; q<nq; ++q)
           values[maps[i][q]] = vvalues[q];
@@ -267,12 +293,12 @@ namespace Functions
   void
   FEFieldFunction<dim, DoFHandlerType, VectorType>::
   value_list (const std::vector<Point< dim > > &points,
-              std::vector< double >            &values,
+              std::vector< typename VectorType::value_type >            &values,
               const unsigned int                component) const
   {
     Assert(points.size() == values.size(),
            ExcDimensionMismatch(points.size(), values.size()));
-    std::vector< Vector<double> > vvalues(points.size(), Vector<double>(n_components));
+    std::vector< Vector<typename VectorType::value_type> > vvalues(points.size(), Vector<typename VectorType::value_type>(this->n_components));
     vector_value_list(points, vvalues);
     for (unsigned int q=0; q<points.size(); ++q)
       values[q] = vvalues[q](component);
@@ -284,7 +310,7 @@ namespace Functions
   void
   FEFieldFunction<dim, DoFHandlerType, VectorType>::
   vector_gradient_list (const std::vector<Point< dim > >           &points,
-                        std::vector<std::vector< Tensor<1,dim> > > &values) const
+                        std::vector<std::vector< Tensor<1,dim, typename VectorType::value_type> > > &values) const
   {
     Assert(points.size() == values.size(),
            ExcDimensionMismatch(points.size(), values.size()));
@@ -316,7 +342,7 @@ namespace Functions
         fe_v.reinit(cells[i], i, 0);
         const unsigned int nq = qpoints[i].size();
         std::vector< std::vector<Tensor<1,dim,typename VectorType::value_type> > >
-        vgrads (nq, std::vector<Tensor<1,dim,typename VectorType::value_type> >(n_components));
+        vgrads (nq, std::vector<Tensor<1,dim,typename VectorType::value_type> >(this->n_components));
         fe_v.get_present_fe_values ().get_function_gradients(data_vector, vgrads);
         for (unsigned int q=0; q<nq; ++q)
           {
@@ -332,13 +358,13 @@ namespace Functions
   void
   FEFieldFunction<dim, DoFHandlerType, VectorType>::
   gradient_list (const std::vector<Point< dim > > &points,
-                 std::vector< Tensor<1,dim> >     &values,
+                 std::vector< Tensor<1,dim, typename VectorType::value_type> >     &values,
                  const unsigned int                component) const
   {
     Assert(points.size() == values.size(),
            ExcDimensionMismatch(points.size(), values.size()));
-    std::vector< std::vector<Tensor<1,dim> > >
-    vvalues(points.size(), std::vector<Tensor<1,dim> >(n_components));
+    std::vector< std::vector<Tensor<1,dim, typename VectorType::value_type> > >
+    vvalues(points.size(), std::vector<Tensor<1,dim,typename VectorType::value_type> >(this->n_components));
     vector_gradient_list(points, vvalues);
     for (unsigned int q=0; q<points.size(); ++q)
       values[q] = vvalues[q][component];
@@ -349,7 +375,7 @@ namespace Functions
   void
   FEFieldFunction<dim, DoFHandlerType, VectorType>::
   vector_laplacian_list (const std::vector<Point< dim > > &points,
-                         std::vector< Vector<double> >    &values) const
+                         std::vector< Vector<typename VectorType::value_type> >    &values) const
   {
     Assert(points.size() == values.size(),
            ExcDimensionMismatch(points.size(), values.size()));
@@ -380,7 +406,7 @@ namespace Functions
       {
         fe_v.reinit(cells[i], i, 0);
         const unsigned int nq = qpoints[i].size();
-        std::vector< Vector<typename VectorType::value_type> > vvalues (nq, Vector<typename VectorType::value_type>(n_components));
+        std::vector< Vector<typename VectorType::value_type> > vvalues (nq, Vector<typename VectorType::value_type>(this->n_components));
         fe_v.get_present_fe_values ().get_function_laplacians(data_vector, vvalues);
         for (unsigned int q=0; q<nq; ++q)
           values[maps[i][q]] = vvalues[q];
@@ -391,12 +417,12 @@ namespace Functions
   void
   FEFieldFunction<dim, DoFHandlerType, VectorType>::
   laplacian_list (const std::vector<Point<dim> > &points,
-                  std::vector<double>            &values,
+                  std::vector<typename VectorType::value_type>            &values,
                   const unsigned int              component) const
   {
     Assert(points.size() == values.size(),
            ExcDimensionMismatch(points.size(), values.size()));
-    std::vector< Vector<double> > vvalues(points.size(), Vector<double>(n_components));
+    std::vector< Vector<typename VectorType::value_type> > vvalues(points.size(), Vector<typename VectorType::value_type>(this->n_components));
     vector_laplacian_list(points, vvalues);
     for (unsigned int q=0; q<points.size(); ++q)
       values[q] = vvalues[q](component);
@@ -458,7 +484,7 @@ namespace Functions
           const std::pair<typename dealii::internal::ActiveCellIterator<dim, dim, DoFHandlerType>::type, Point<dim> >
           my_pair  = GridTools::find_active_cell_around_point
                      (mapping, *dh, points[0]);
-          AssertThrow (my_pair.first->is_locally_owned(),
+          AssertThrow (!my_pair.first->is_artificial(),
                        VectorTools::ExcPointNotAvailableHere());
 
           cell = my_pair.first;
@@ -466,6 +492,10 @@ namespace Functions
           point_flags[0] = true;
         }
 
+      // check that the cell is available:
+      AssertThrow (!cell->is_artificial(),
+                   VectorTools::ExcPointNotAvailableHere());
+
       // Put in the first point.
       cells.push_back(cell);
       qpoints.push_back(std::vector<Point<dim> >(1, qp.get()));
@@ -522,7 +552,7 @@ namespace Functions
           {
             const std::pair<typename dealii::internal::ActiveCellIterator<dim, dim, DoFHandlerType>::type, Point<dim> > my_pair
               = GridTools::find_active_cell_around_point (mapping, *dh, points[first_outside]);
-            AssertThrow (my_pair.first->is_locally_owned(),
+            AssertThrow (!my_pair.first->is_artificial(),
                          VectorTools::ExcPointNotAvailableHere());
 
             cells.push_back(my_pair.first);
@@ -593,3 +623,5 @@ namespace Functions
 }
 
 DEAL_II_NAMESPACE_CLOSE
+
+#endif
