@@ -42,16 +42,6 @@
 
 DEAL_II_NAMESPACE_OPEN
 
-namespace internal
-{
-  namespace PointValueHistory
-  {
-    template <int dim> class PointGeometryData;
-  }
-}
-
-
-
 namespace internal
 {
   namespace PointValueHistory
@@ -419,7 +409,7 @@ public:
    * locations output.
    */
   void write_gnuplot (const std::string &base_name,
-                      const std::vector <Point <dim> > postprocessor_locations = std::vector <Point <dim> > ());
+                      const std::vector <Point <dim> > &postprocessor_locations = std::vector <Point <dim> > ());
 
 
   /**
