@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2015 by the deal.II authors
+// Copyright (C) 1999 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -433,7 +433,18 @@ private:
    */
   enum PreparationState
   {
-    none, pure_refinement, coarsening_and_refinement
+    /**
+     * The SolutionTransfer is not yet prepared.
+     */
+    none,
+    /**
+     * The SolutionTransfer is prepared for purely refinement.
+     */
+    pure_refinement,
+    /**
+     * The SolutionTransfer is prepared for coarsening and refinement.
+     */
+    coarsening_and_refinement
   };
 
   /**
