@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2005 - 2015 by the deal.II authors
+// Copyright (C) 2005 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -23,6 +23,7 @@
 
 DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #include <boost/random.hpp>
+#include <boost/math/special_functions/erf.hpp>
 DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 
 #ifdef DEAL_II_WITH_MUPARSER
@@ -46,9 +47,12 @@ DEAL_II_NAMESPACE_OPEN
 
 template <int dim>
 FunctionParser<dim>::FunctionParser(const unsigned int n_components,
-                                    const double       initial_time)
+                                    const double       initial_time,
+                                    const double       h)
   :
-  Function<dim>(n_components, initial_time)
+  AutoDerivativeFunction<dim>(h, n_components, initial_time),
+  initialized (false),
+  n_vars (0)
 {}
 
 
@@ -181,7 +185,7 @@ namespace internal
 
   double mu_erfc(double value)
   {
-    return erfc(value);
+    return boost::math::erfc(value);
   }
 
   // returns a random value in the range [0,1] initializing the generator
@@ -216,8 +220,9 @@ namespace internal
 }
 
 
+
 template <int dim>
-void FunctionParser<dim>:: init_muparser() const
+void FunctionParser<dim>::init_muparser() const
 {
   // check that we have not already initialized the parser on the
   // current thread, i.e., that the current function is only called
