@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2014 by the deal.II authors
+// Copyright (C) 1998 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -20,10 +20,6 @@
 # include <unistd.h>
 #endif
 
-#ifdef DEAM_II_MSVC
-#  include <WinSock2.h>
-#endif
-
 DEAL_II_NAMESPACE_OPEN
 
 
