@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2015 by the deal.II authors
+// Copyright (C) 1998 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -57,6 +57,7 @@ LogStream::LogStream()
   double_threshold(0.),
   float_threshold(0.),
   offset(0),
+  print_thread_id(false),
   old_cerr(0),
   at_newline(true)
 {
@@ -109,14 +110,17 @@ LogStream::~LogStream()
 
 
 void
-LogStream::test_mode(bool on)
+LogStream::test_mode(const bool on,
+                     const double double_threshold_,
+                     const float float_threshold_,
+                     const double offset_)
 {
   Threads::Mutex::ScopedLock lock(log_lock);
   if (on)
     {
-      double_threshold = 1.e-10;
-      float_threshold = 1.e-7f;
-      offset = 1.e-7;
+      double_threshold = double_threshold_;
+      float_threshold = float_threshold_;
+      offset = offset_;
     }
   else
     {
@@ -260,7 +264,9 @@ LogStream::get_console()
 std::ostream &
 LogStream::get_file_stream()
 {
-  Assert(file, ExcNoFileStreamGiven());
+  Assert(file,
+         ExcMessage("You can't ask for the std::ostream object for the output "
+                    "file if none had been set before."));
   return *file;
 }
 
