@@ -1,7 +1,6 @@
 // ---------------------------------------------------------------------
-// $Id: polynomials_raviart_thomas.cc 30037 2013-07-18 16:55:40Z maier $
 //
-// Copyright (C) 2004 - 2013 by the deal.II authors
+// Copyright (C) 2004 - 2015 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -60,7 +59,9 @@ void
 PolynomialsRaviartThomas<dim>::compute (const Point<dim>            &unit_point,
                                         std::vector<Tensor<1,dim> > &values,
                                         std::vector<Tensor<2,dim> > &grads,
-                                        std::vector<Tensor<3,dim> > &grad_grads) const
+                                        std::vector<Tensor<3, dim> > &grad_grads,
+                                        std::vector<Tensor<4, dim> > &third_derivatives,
+                                        std::vector<Tensor<5, dim> > &fourth_derivatives) const
 {
   Assert(values.size()==n_pols || values.size()==0,
          ExcDimensionMismatch(values.size(), n_pols));
@@ -68,6 +69,10 @@ PolynomialsRaviartThomas<dim>::compute (const Point<dim>            &unit_point,
          ExcDimensionMismatch(grads.size(), n_pols));
   Assert(grad_grads.size()==n_pols|| grad_grads.size()==0,
          ExcDimensionMismatch(grad_grads.size(), n_pols));
+  Assert(third_derivatives.size()==n_pols|| third_derivatives.size()==0,
+         ExcDimensionMismatch(third_derivatives.size(), n_pols));
+  Assert(fourth_derivatives.size()==n_pols|| fourth_derivatives.size()==0,
+         ExcDimensionMismatch(fourth_derivatives.size(), n_pols));
 
   // have a few scratch
   // arrays. because we don't want to
@@ -88,11 +93,15 @@ PolynomialsRaviartThomas<dim>::compute (const Point<dim>            &unit_point,
   static std::vector<double> p_values;
   static std::vector<Tensor<1,dim> > p_grads;
   static std::vector<Tensor<2,dim> > p_grad_grads;
+  static std::vector<Tensor<3,dim> > p_third_derivatives;
+  static std::vector<Tensor<4,dim> > p_fourth_derivatives;
 
   const unsigned int n_sub = polynomial_space.n();
   p_values.resize((values.size() == 0) ? 0 : n_sub);
   p_grads.resize((grads.size() == 0) ? 0 : n_sub);
   p_grad_grads.resize((grad_grads.size() == 0) ? 0 : n_sub);
+  p_third_derivatives.resize((third_derivatives.size() == 0) ? 0 : n_sub);
+  p_fourth_derivatives.resize((fourth_derivatives.size() == 0) ? 0 : n_sub);
 
   for (unsigned int d=0; d<dim; ++d)
     {
@@ -111,7 +120,7 @@ PolynomialsRaviartThomas<dim>::compute (const Point<dim>            &unit_point,
       for (unsigned int c=0; c<dim; ++c)
         p(c) = unit_point((c+d)%dim);
 
-      polynomial_space.compute (p, p_values, p_grads, p_grad_grads);
+      polynomial_space.compute (p, p_values, p_grads, p_grad_grads, p_third_derivatives, p_fourth_derivatives);
 
       for (unsigned int i=0; i<p_values.size(); ++i)
         values[i+d*n_sub][d] = p_values[i];
@@ -125,6 +134,21 @@ PolynomialsRaviartThomas<dim>::compute (const Point<dim>            &unit_point,
           for (unsigned int d2=0; d2<dim; ++d2)
             grad_grads[i+d*n_sub][d][(d1+d)%dim][(d2+d)%dim]
               = p_grad_grads[i][d1][d2];
+
+      for (unsigned int i=0; i<p_third_derivatives.size(); ++i)
+        for (unsigned int d1=0; d1<dim; ++d1)
+          for (unsigned int d2=0; d2<dim; ++d2)
+            for (unsigned int d3=0; d3<dim; ++d3)
+              third_derivatives[i+d*n_sub][d][(d1+d)%dim][(d2+d)%dim][(d3+d)%dim]
+                = p_third_derivatives[i][d1][d2][d3];
+
+      for (unsigned int i=0; i<p_fourth_derivatives.size(); ++i)
+        for (unsigned int d1=0; d1<dim; ++d1)
+          for (unsigned int d2=0; d2<dim; ++d2)
+            for (unsigned int d3=0; d3<dim; ++d3)
+              for (unsigned int d4=0; d4<dim; ++d4)
+                fourth_derivatives[i+d*n_sub][d][(d1+d)%dim][(d2+d)%dim][(d3+d)%dim][(d4+d)%dim]
+                  = p_fourth_derivatives[i][d1][d2][d3][d4];
     }
 }
 
