@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2015 by the deal.II authors
+// Copyright (C) 1998 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -15,7 +15,9 @@
 
 #include <deal.II/base/timer.h>
 #include <deal.II/base/exceptions.h>
+#include <deal.II/base/mpi.h>
 #include <deal.II/base/utilities.h>
+#include <deal.II/base/signaling_nan.h>
 #include <sstream>
 #include <iostream>
 #include <iomanip>
@@ -40,13 +42,24 @@ DEAL_II_NAMESPACE_OPEN
 // current process
 Timer::Timer()
   :
+  start_time (0.),
+  start_time_children (0.),
+  start_wall_time (0.),
   cumulative_time (0.),
-  cumulative_wall_time (0.)
+  cumulative_wall_time (0.),
+  last_lap_time (0.),
+  running (false)
 #ifdef DEAL_II_WITH_MPI
-  , mpi_communicator (MPI_COMM_SELF)
-  , sync_wall_time (false)
+  ,
+  mpi_communicator (MPI_COMM_SELF),
+  sync_wall_time (false)
 #endif
 {
+#ifdef DEAL_II_WITH_MPI
+  mpi_data.sum = mpi_data.min = mpi_data.max = mpi_data.avg = numbers::signaling_nan<double>();
+  mpi_data.min_index = mpi_data.max_index = numbers::invalid_unsigned_int;
+#endif
+
   start();
 }
 
@@ -56,17 +69,29 @@ Timer::Timer()
 // the communicator given from input
 #ifdef DEAL_II_WITH_MPI
 Timer::Timer(MPI_Comm mpi_communicator,
-             bool sync_wall_time_)
+             const bool sync_wall_time_)
   :
+  start_time (0.),
+  start_time_children (0.),
+  start_wall_time (0.),
   cumulative_time (0.),
   cumulative_wall_time (0.),
+  last_lap_time (0.),
+  running (false),
   mpi_communicator (mpi_communicator),
   sync_wall_time(sync_wall_time_)
 {
+#ifdef DEAL_II_WITH_MPI
+  mpi_data.sum = mpi_data.min = mpi_data.max = mpi_data.avg = numbers::signaling_nan<double>();
+  mpi_data.min_index = mpi_data.max_index = numbers::invalid_unsigned_int;
+#endif
+
   start();
 }
 #endif
 
+
+
 #ifdef DEAL_II_MSVC
 
 namespace
@@ -105,7 +130,10 @@ void Timer::start ()
 
 #ifdef DEAL_II_WITH_MPI
   if (sync_wall_time)
-    MPI_Barrier(mpi_communicator);
+    {
+      const int ierr = MPI_Barrier(mpi_communicator);
+      AssertThrowMPI(ierr);
+    }
 #endif
 
 #if defined(DEAL_II_HAVE_SYS_TIME_H) && defined(DEAL_II_HAVE_SYS_RESOURCE_H)
@@ -325,8 +353,13 @@ TimerOutput::TimerOutput (MPI_Comm      mpi_communicator,
 
 TimerOutput::~TimerOutput()
 {
-  while (active_sections.size() > 0)
-    leave_subsection();
+  try
+    {
+      while (active_sections.size() > 0)
+        leave_subsection();
+    }
+  catch (...)
+    {}
 
   if ( (output_frequency == summary || output_frequency == every_call_and_summary)
        && output_is_enabled == true)
@@ -505,9 +538,22 @@ TimerOutput::print_summary () const
           out_stream << std::setprecision(3);
           out_stream << i->second.total_cpu_time << "s |";
           out_stream << std::setw(10);
-          out_stream << std::setprecision(2);
           if (total_cpu_time != 0)
-            out_stream << i->second.total_cpu_time/total_cpu_time * 100 << "% |";
+            {
+              // if run time was less than 0.1%, just print a zero to avoid
+              // printing silly things such as "2.45e-6%". otherwise print
+              // the actual percentage
+              const double fraction = i->second.total_cpu_time/total_cpu_time;
+              if (fraction > 0.001)
+                {
+                  out_stream << std::setprecision(2);
+                  out_stream << fraction * 100;
+                }
+              else
+                out_stream << 0.0;
+
+              out_stream << "% |";
+            }
           else
             out_stream << 0.0 << "% |";
         }
@@ -563,11 +609,25 @@ TimerOutput::print_summary () const
           out_stream << std::setprecision(3);
           out_stream << i->second.total_wall_time << "s |";
           out_stream << std::setw(10);
-          out_stream << std::setprecision(2);
-          double value = i->second.total_wall_time/total_wall_time * 100;
-          if (!numbers::is_finite(value))
-            value = 0.0;
-          out_stream << value << "% |";
+
+          if (total_wall_time != 0)
+            {
+              // if run time was less than 0.1%, just print a zero to avoid
+              // printing silly things such as "2.45e-6%". otherwise print
+              // the actual percentage
+              const double fraction = i->second.total_wall_time/total_wall_time;
+              if (fraction > 0.001)
+                {
+                  out_stream << std::setprecision(2);
+                  out_stream << fraction * 100;
+                }
+              else
+                out_stream << 0.0;
+
+              out_stream << "% |";
+            }
+          else
+            out_stream << 0.0 << "% |";
         }
       out_stream << std::endl
                  << "+---------------------------------+-----------+"
