@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2005 - 2015 by the deal.II authors
+// Copyright (C) 2005 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -15,6 +15,7 @@
 
 
 #include <deal.II/base/utilities.h>
+#include <deal.II/base/mpi.h>
 #include <deal.II/base/exceptions.h>
 #include <deal.II/base/thread_local_storage.h>
 
@@ -46,6 +47,7 @@ DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 #endif
 
 
+DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #ifdef DEAL_II_WITH_TRILINOS
 #  ifdef DEAL_II_WITH_MPI
 #    include <Epetra_MpiComm.h>
@@ -56,6 +58,7 @@ DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 #  include "Teuchos_RCP.hpp"
 #  include "Epetra_SerialComm.h"
 #endif
+DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 
 
 
@@ -131,7 +134,7 @@ namespace Utilities
   trim(const std::string &input)
   {
     std::string::size_type left = 0;
-    std::string::size_type right = input.size() - 1;
+    std::string::size_type right = input.size() > 0 ? input.size() - 1 : 0;
 
     for (; left < input.size(); ++left)
       {
@@ -243,7 +246,7 @@ namespace Utilities
     errno = 0;
     const double d = std::strtod(s.c_str(), &p);
     AssertThrow ( !((errno != 0) || (s.size() == 0) || ((s.size()>0) && (*p != '\0'))),
-                  ExcMessage ("Can't convert <" + s + "> to an integer."));
+                  ExcMessage ("Can't convert <" + s + "> to a double."));
 
     return d;
   }
@@ -783,7 +786,8 @@ namespace Utilities
         {
           MPI_Comm comm = mpi_comm->GetMpiComm();
           *mpi_comm = Epetra_MpiComm(MPI_COMM_SELF);
-          MPI_Comm_free (&comm);
+          const int ierr = MPI_Comm_free (&comm);
+          AssertThrowMPI(ierr);
         }
 #endif
     }
