@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2015 by the deal.II authors
+// Copyright (C) 2009 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -20,8 +20,8 @@
 
 #include <deal.II/lac/vector.h>
 #include <deal.II/lac/block_vector.h>
-#include <deal.II/lac/parallel_vector.h>
-#include <deal.II/lac/parallel_block_vector.h>
+#include <deal.II/lac/la_parallel_vector.h>
+#include <deal.II/lac/la_parallel_block_vector.h>
 #include <deal.II/lac/petsc_vector.h>
 #include <deal.II/lac/petsc_block_vector.h>
 #include <deal.II/lac/trilinos_vector.h>
@@ -46,7 +46,8 @@ namespace parallel
     template<int dim, typename VectorType, typename DoFHandlerType>
     SolutionTransfer<dim, VectorType, DoFHandlerType>::SolutionTransfer (const DoFHandlerType &dof)
       :
-      dof_handler(&dof, typeid(*this).name())
+      dof_handler(&dof, typeid(*this).name()),
+      offset (numbers::invalid_unsigned_int)
     {
       Assert (dynamic_cast<const parallel::distributed::Triangulation<dim>*>
               (&dof_handler->get_triangulation()) != 0,
@@ -200,7 +201,7 @@ namespace parallel
     unsigned int
     SolutionTransfer<dim, VectorType, DoFHandlerType>::get_data_size() const
     {
-      return sizeof(double)* DoFTools::max_dofs_per_cell(*dof_handler);
+      return sizeof(typename VectorType::value_type)* DoFTools::max_dofs_per_cell(*dof_handler);
     }
 
 
@@ -222,7 +223,6 @@ namespace parallel
            ++it)
         {
           cell->get_interpolated_dof_values(*(*it), dofvalues);
-          Assert (typeid(typename VectorType::value_type) == typeid(double), ExcNotImplemented());
           std::memcpy(data_store, &dofvalues(0), sizeof(typename VectorType::value_type)*dofs_per_cell);
           data_store += dofs_per_cell;
         }
@@ -248,7 +248,6 @@ namespace parallel
            it != all_out.end();
            ++it)
         {
-          Assert (typeid(typename VectorType::value_type) == typeid(double), ExcNotImplemented());
           std::memcpy(&dofvalues(0), data_store, sizeof(typename VectorType::value_type)*dofs_per_cell);
           cell->set_dof_values_by_interpolation(dofvalues, *(*it));
           data_store += dofs_per_cell;
