@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2015 by the deal.II authors
+// Copyright (C) 1999 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -265,11 +265,8 @@ namespace DoFRenumbering
 
       int delta = 0;
 
-      typedef double Type;
-
       // must be BGL directed graph now
       typedef adjacency_list<vecS, vecS, directedS>  Graph;
-      typedef graph_traits<Graph>::vertex_descriptor Vertex;
 
       int n = dof_handler.n_dofs();
 
@@ -392,7 +389,12 @@ namespace DoFRenumbering
     // object will be empty and nothing happens
     ConstraintMatrix constraints;
     if (use_constraints)
-      DoFTools::make_hanging_node_constraints (dof_handler, constraints);
+      {
+        IndexSet relevant_dofs;
+        DoFTools::extract_locally_relevant_dofs(dof_handler, relevant_dofs);
+        constraints.reinit(relevant_dofs);
+        DoFTools::make_hanging_node_constraints (dof_handler, constraints);
+      }
     constraints.close ();
 
     const IndexSet locally_owned = dof_handler.locally_owned_dofs();
@@ -408,8 +410,7 @@ namespace DoFRenumbering
 
     // If the index set is not complete, need to get indices in local index
     // space.
-    if (locally_owned.n_elements() !=
-        locally_owned.size())
+    if (locally_owned.n_elements() != locally_owned.size())
       {
         // Create sparsity pattern from dsp by transferring its indices to
         // processor-local index space and doing Cuthill-McKee there
@@ -419,11 +420,14 @@ namespace DoFRenumbering
         for (unsigned int i=0; i<locally_owned.n_elements(); ++i)
           {
             const types::global_dof_index row = locally_owned.nth_index_in_set(i);
+            const unsigned int row_length = dsp.row_length(row);
             row_entries.clear();
-            for (DynamicSparsityPattern::iterator it =
-                   dsp.begin(row); it != dsp.end(row); ++it)
-              if (it->column() != row && locally_owned.is_element(it->column()))
-                row_entries.push_back(locally_owned.index_within_set(it->column()));
+            for (unsigned int j=0; j<row_length; ++j)
+              {
+                const unsigned int col = dsp.column_number(row, j);
+                if (col != row && locally_owned.is_element(col))
+                  row_entries.push_back(locally_owned.index_within_set(col));
+              }
             sparsity.add_entries(i, row_entries.begin(), row_entries.end(),
                                  true);
           }
@@ -766,11 +770,12 @@ namespace DoFRenumbering
         all_dof_counts(fe_collection.n_components() *
                        Utilities::MPI::n_mpi_processes (tria->get_communicator()));
 
-        MPI_Allgather ( &local_dof_count[0],
-                        n_buckets, DEAL_II_DOF_INDEX_MPI_TYPE,
-                        &all_dof_counts[0],
-                        n_buckets, DEAL_II_DOF_INDEX_MPI_TYPE,
-                        tria->get_communicator());
+        const int ierr = MPI_Allgather ( &local_dof_count[0],
+                                         n_buckets, DEAL_II_DOF_INDEX_MPI_TYPE,
+                                         &all_dof_counts[0],
+                                         n_buckets, DEAL_II_DOF_INDEX_MPI_TYPE,
+                                         tria->get_communicator());
+        AssertThrowMPI(ierr);
 
         for (unsigned int i=0; i<n_buckets; ++i)
           Assert (all_dof_counts[n_buckets*tria->locally_owned_subdomain()+i]
@@ -1053,11 +1058,12 @@ namespace DoFRenumbering
         all_dof_counts(fe_collection.n_components() *
                        Utilities::MPI::n_mpi_processes (tria->get_communicator()));
 
-        MPI_Allgather ( &local_dof_count[0],
-                        n_buckets, DEAL_II_DOF_INDEX_MPI_TYPE,
-                        &all_dof_counts[0],
-                        n_buckets, DEAL_II_DOF_INDEX_MPI_TYPE,
-                        tria->get_communicator());
+        const int ierr = MPI_Allgather ( &local_dof_count[0],
+                                         n_buckets, DEAL_II_DOF_INDEX_MPI_TYPE,
+                                         &all_dof_counts[0],
+                                         n_buckets, DEAL_II_DOF_INDEX_MPI_TYPE,
+                                         tria->get_communicator());
+        AssertThrowMPI(ierr);
 
         for (unsigned int i=0; i<n_buckets; ++i)
           Assert (all_dof_counts[n_buckets*tria->locally_owned_subdomain()+i]
@@ -1864,9 +1870,8 @@ namespace DoFRenumbering
     for (unsigned int i=0; i<n_dofs; ++i)
       new_indices[i] = i;
 
-    // shuffle the elements; the following is essentially the
-    // std::random_shuffle algorithm but uses a predictable
-    // random number generator
+    // shuffle the elements; the following is essentially std::shuffle (which
+    // is new in C++11) but with a boost URNG
     ::boost::mt19937 random_number_generator;
     for (unsigned int i=1; i<n_dofs; ++i)
       {
