@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2015 by the deal.II authors
+// Copyright (C) 1998 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -85,15 +85,14 @@ FiniteElement (const FiniteElementData<dim> &fe_data,
                       std::vector<ComponentMask> (fe_data.dofs_per_cell, nonzero_c[0])
                       :
                       nonzero_c),
-  n_nonzero_components_table (compute_n_nonzero_components(nonzero_components))
+  n_nonzero_components_table (compute_n_nonzero_components(nonzero_components)),
+  cached_primitivity (std::find_if (n_nonzero_components_table.begin(),
+                                    n_nonzero_components_table.end(),
+                                    std_cxx11::bind (std::not_equal_to<unsigned int>(),
+                                                     std_cxx11::_1,
+                                                     1U))
+                      == n_nonzero_components_table.end())
 {
-  this->set_primitivity(std::find_if (n_nonzero_components_table.begin(),
-                                      n_nonzero_components_table.end(),
-                                      std::bind2nd(std::not_equal_to<unsigned int>(),
-                                                   1U))
-                        == n_nonzero_components_table.end());
-
-
   Assert (restriction_is_additive_flags.size() == this->dofs_per_cell,
           ExcDimensionMismatch(restriction_is_additive_flags.size(),
                                this->dofs_per_cell));
@@ -1098,6 +1097,22 @@ FiniteElement<dim,spacedim>::get_constant_modes () const
 
 
 
+template <int dim, int spacedim>
+void
+FiniteElement<dim,spacedim>::
+convert_generalized_support_point_values_to_nodal_values (const std::vector<Vector<double> > &,
+                                                          std::vector<double> &) const
+{
+  Assert (has_generalized_support_points(),
+          ExcMessage ("The element for which you are calling the current "
+                      "function does not have generalized support points (see "
+                      "the glossary for a definition of generalized support "
+                      "points). Consequently, the current function can not "
+                      "be defined and is not implemented by the element."));
+  Assert (false, ExcNotImplemented());
+}
+
+
 template <int dim, int spacedim>
 void
 FiniteElement<dim,spacedim>::interpolate(
@@ -1121,9 +1136,9 @@ FiniteElement<dim,spacedim>::interpolate(
 template <int dim, int spacedim>
 void
 FiniteElement<dim,spacedim>::interpolate(
-  std::vector<double>    &local_dofs,
+  std::vector<double>                &local_dofs,
   const std::vector<Vector<double> > &values,
-  unsigned int offset) const
+  const unsigned int                  offset) const
 {
   Assert (has_support_points(), ExcFEHasNoSupportPoints());
   Assert (values.size() == unit_support_points.size(),
