@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2015 by the deal.II authors
+// Copyright (C) 2000 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -22,6 +22,8 @@
 #include <deal.II/base/tensor_product_polynomials_bubbles.h>
 #include <deal.II/base/polynomials_piecewise.h>
 #include <deal.II/fe/fe_q_base.h>
+#include <deal.II/fe/fe_dgq.h>
+#include <deal.II/fe/fe_dgp.h>
 #include <deal.II/fe/fe_nothing.h>
 #include <deal.II/fe/fe_tools.h>
 #include <deal.II/base/quadrature_lib.h>
@@ -862,7 +864,8 @@ compare_for_face_domination (const FiniteElement<dim,spacedim> &fe_other) const
       else
         return FiniteElementDomination::other_element_dominates;
     }
-  else if (const FE_Nothing<dim> *fe_nothing = dynamic_cast<const FE_Nothing<dim>*>(&fe_other))
+  else if (const FE_Nothing<dim,spacedim> *fe_nothing
+           = dynamic_cast<const FE_Nothing<dim,spacedim>*>(&fe_other))
     {
       if (fe_nothing->is_dominating())
         {
@@ -875,6 +878,14 @@ compare_for_face_domination (const FiniteElement<dim,spacedim> &fe_other) const
           return FiniteElementDomination::no_requirements;
         }
     }
+  else if ((dynamic_cast<const FE_DGQ<dim,spacedim>*>(&fe_other) != 0)
+           ||
+           (dynamic_cast<const FE_DGP<dim,spacedim>*>(&fe_other) != 0))
+    {
+      // there are no requirements between continuous and
+      // discontinuous elements
+      return FiniteElementDomination::no_requirements;
+    }
 
   Assert (false, ExcNotImplemented());
   return FiniteElementDomination::neither_element_dominates;
@@ -1118,12 +1129,13 @@ face_to_cell_index (const unsigned int face_index,
 
 template <class PolynomialType, int dim, int spacedim>
 std::vector<unsigned int>
-FE_Q_Base<PolynomialType,dim,spacedim>::get_dpo_vector(const unsigned int deg)
+FE_Q_Base<PolynomialType,dim,spacedim>::get_dpo_vector(const unsigned int degree)
 {
-  AssertThrow(deg>0,ExcMessage("FE_Q needs to be of degree > 0."));
+  typedef FE_Q_Base<PolynomialType,dim,spacedim> FEQ;
+  AssertThrow(degree>0, typename FEQ::ExcFEQCannotHaveDegree0());
   std::vector<unsigned int> dpo(dim+1, 1U);
   for (unsigned int i=1; i<dpo.size(); ++i)
-    dpo[i]=dpo[i-1]*(deg-1);
+    dpo[i]=dpo[i-1]*(degree-1);
   return dpo;
 }
 
@@ -1180,12 +1192,22 @@ FE_Q_Base<PolynomialType,dim,spacedim>
         {
           Assert (std::fabs (1.-this->poly_space.compute_value
                              (i, this->unit_support_points[i])) < eps,
-                  ExcInternalError());
+                  ExcInternalError("The Lagrange polynomial does not evaluate "
+                                   "to one or zero in a nodal point. "
+                                   "This typically indicates that the "
+                                   "polynomial interpolation is "
+                                   "ill-conditioned such that round-off "
+                                   "prevents the sum to be one."));
           for (unsigned int j=0; j<q_dofs_per_cell; ++j)
             if (j!=i)
               Assert (std::fabs (this->poly_space.compute_value
                                  (i, this->unit_support_points[j])) < eps,
-                      ExcInternalError());
+                      ExcInternalError("The Lagrange polynomial does not evaluate "
+                                       "to one or zero in a nodal point. "
+                                       "This typically indicates that the "
+                                       "polynomial interpolation is "
+                                       "ill-conditioned such that round-off "
+                                       "prevents the sum to be one."));
         }
 #endif
 
@@ -1301,7 +1323,14 @@ FE_Q_Base<PolynomialType,dim,spacedim>
           double sum = 0;
           for (unsigned int col=0; col<this->dofs_per_cell; ++col)
             sum += prolongate(row,col);
-          Assert (std::fabs(sum-1.) < eps, ExcInternalError());
+          Assert (std::fabs(sum-1.) <
+                  std::max(eps, 5e-16*std::sqrt(this->dofs_per_cell)),
+                  ExcInternalError("The entries in a row of the local "
+                                   "prolongation matrix do not add to one. "
+                                   "This typically indicates that the "
+                                   "polynomial interpolation is "
+                                   "ill-conditioned such that round-off "
+                                   "prevents the sum to be one."));
         }
 #endif
 
@@ -1339,31 +1368,27 @@ FE_Q_Base<PolynomialType,dim,spacedim>
           this->dofs_per_cell)
         return this->restriction[refinement_case-1][child];
 
-      FullMatrix<double> restriction(this->dofs_per_cell, this->dofs_per_cell);
+      FullMatrix<double> my_restriction(this->dofs_per_cell, this->dofs_per_cell);
       // distinguish q/q_dg0 case
       const unsigned int q_dofs_per_cell = Utilities::fixed_power<dim>(q_degree+1);
 
-      // for these Lagrange interpolation polynomials, construction of the
-      // restriction matrices is relatively simple. the reason is that the
-      // interpolation points on the mother cell are (except for the case with
-      // arbitrary nonequidistant nodes) always also interpolation points for
-      // some shape function on one or the other child, because we have chosen
-      // equidistant Lagrange interpolation points for the polynomials
+      // for Lagrange interpolation polynomials based on equidistant points,
+      // construction of the restriction matrices is relatively simple. the
+      // reason is that in this case the interpolation points on the mother
+      // cell are always also interpolation points for some shape function on
+      // one or the other child.
       //
-      // so the only thing we have to find out is: for each shape function on
-      // the mother cell, which is the child cell (possibly more than one) on
-      // which it is located, and which is the corresponding shape function
-      // there. rather than doing it for the shape functions on the mother
-      // cell, we take the interpolation points there
+      // in the general case with non-equidistant points, we need to actually
+      // do an interpolation. thus, we take the interpolation points on the
+      // mother cell and evaluate the shape functions of the child cell on
+      // those points. it does not hurt in the equidistant case because then
+      // simple one shape function evaluates to one and the others to zero.
       //
-      // note that the interpolation point of a shape function can be on the
-      // boundary between subcells. in that case, restriction from children to
-      // mother may produce two or more entries for a dof on the mother
-      // cell. however, this doesn't hurt: since the element is continuous,
-      // the contribution from each child should yield the same result, and
-      // since the element is non-additive we just overwrite one value
-      // (compute on one child) by the same value (compute on a later child),
-      // so we don't have to care about this
+      // this element is non-additive in all its degrees of freedom by
+      // default, which requires care in downstream use. fortunately, even the
+      // interpolation on non-equidistant points is invariant under the
+      // assumption that whenever a row makes a non-zero contribution to the
+      // mother's residual, the correct value is interpolated.
 
       const double eps = 1e-15*q_degree*dim;
       const std::vector<unsigned int> &index_map_inverse =
@@ -1372,7 +1397,7 @@ FE_Q_Base<PolynomialType,dim,spacedim>
       const unsigned int dofs1d = q_degree+1;
       std::vector<Tensor<1,dim> > evaluations1d (dofs1d);
 
-      restriction.reinit(this->dofs_per_cell, this->dofs_per_cell);
+      my_restriction.reinit(this->dofs_per_cell, this->dofs_per_cell);
 
       for (unsigned int i=0; i<q_dofs_per_cell; ++i)
         {
@@ -1420,26 +1445,33 @@ FE_Q_Base<PolynomialType,dim,spacedim>
                       const unsigned int child_dof =
                         index_map_inverse[j+jj];
                       if (std::fabs (val-1.) < eps)
-                        restriction(mother_dof,child_dof)=1.;
+                        my_restriction(mother_dof,child_dof)=1.;
                       else if (std::fabs(val) > eps)
-                        restriction(mother_dof,child_dof)=val;
+                        my_restriction(mother_dof,child_dof)=val;
                       sum_check += val;
                     }
                   FE_Q_Helper::increment_indices<dim> (j_indices, dofs1d);
                 }
-              Assert (std::fabs(sum_check-1) < eps,
-                      ExcInternalError());
+              Assert (std::fabs(sum_check-1) <
+                      std::max(eps, 5e-16*std::sqrt(this->dofs_per_cell)),
+                      ExcInternalError("The entries in a row of the local "
+                                       "restriction matrix do not add to one. "
+                                       "This typically indicates that the "
+                                       "polynomial interpolation is "
+                                       "ill-conditioned such that round-off "
+                                       "prevents the sum to be one."));
             }
 
           // part for FE_Q_DG0
           if (q_dofs_per_cell < this->dofs_per_cell)
-            restriction(this->dofs_per_cell-1,this->dofs_per_cell-1) =
+            my_restriction(this->dofs_per_cell-1,this->dofs_per_cell-1) =
               1./GeometryInfo<dim>::n_children(RefinementCase<dim>(refinement_case));
         }
 
-      // swap matrices
-      restriction.swap(const_cast<FullMatrix<double> &>
-                       (this->restriction[refinement_case-1][child]));
+      // swap the just computed restriction matrix into the
+      // element of the vector stored in the base class
+      my_restriction.swap(const_cast<FullMatrix<double> &>
+                          (this->restriction[refinement_case-1][child]));
     }
 
   return this->restriction[refinement_case-1][child];
