@@ -1,7 +1,6 @@
 // ---------------------------------------------------------------------
-// $Id$
 //
-// Copyright (C) 2012 - 2015 by the deal.II authors
+// Copyright (C) 2012 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -75,7 +74,7 @@ namespace FE_Q_Bubbles_Helper
           q_fine.reset(new QGauss<dim> (degree+1));
           break;
         default:
-          AssertThrow(false, ExcInternalError());
+          Assert(false, ExcInternalError());
         }
 
       Assert(q_fine.get() != NULL, ExcInternalError());
@@ -172,7 +171,7 @@ template <int dim, int spacedim>
 FE_Q_Bubbles<dim,spacedim>::FE_Q_Bubbles (const unsigned int q_degree)
   :
   FE_Q_Base<TensorProductPolynomialsBubbles<dim>, dim, spacedim> (
-    TensorProductPolynomialsBubbles<dim>(Polynomials::LagrangeEquidistant::generate_complete_basis(q_degree)),
+    TensorProductPolynomialsBubbles<dim>(Polynomials::generate_complete_Lagrange_basis(QGaussLobatto<1>(q_degree+1).get_points())),
     FiniteElementData<dim>(get_dpo_vector(q_degree),
                            1, q_degree+1,
                            FiniteElementData<dim>::H1),
@@ -183,11 +182,7 @@ FE_Q_Bubbles<dim,spacedim>::FE_Q_Bubbles (const unsigned int q_degree)
           ExcMessage ("This element can only be used for polynomial degrees "
                       "greater than zero"));
 
-  std::vector<Point<1> > support_points_1d(q_degree+1);
-  for (unsigned int i=0; i<=q_degree; ++i)
-    support_points_1d[i][0] = static_cast<double>(i)/q_degree;
-
-  this->initialize(support_points_1d);
+  this->initialize(QGaussLobatto<1>(q_degree+1).get_points());
 
   // adjust unit support point for discontinuous node
   Point<dim> point;
@@ -221,9 +216,7 @@ FE_Q_Bubbles<dim,spacedim>::FE_Q_Bubbles (const Quadrature<1> &points)
     get_riaf_vector(points.size()-1)),
   n_bubbles((points.size()-1<=1)?1:dim)
 {
-  const unsigned int q_degree = points.size()-1;
-  (void) q_degree;
-  Assert (q_degree > 0,
+  Assert (points.size() > 1,
           ExcMessage ("This element can only be used for polynomial degrees "
                       "at least one"));
 
@@ -253,7 +246,7 @@ template <int dim, int spacedim>
 std::string
 FE_Q_Bubbles<dim,spacedim>::get_name () const
 {
-  // note that the FETools::get_fe_from_name function depends on the
+  // note that the FETools::get_fe_by_name function depends on the
   // particular format of the string this function returns, so they have to be
   // kept in synch
 
@@ -294,7 +287,16 @@ FE_Q_Bubbles<dim,spacedim>::get_name () const
       }
 
   if (type == true)
-    namebuf << "FE_Q_Bubbles<" << dim << ">(" << this->degree-1 << ")";
+    {
+      if (this->degree > 3)
+        namebuf << "FE_Q_Bubbles<"
+                << Utilities::dim_string(dim,spacedim)
+                << ">(QIterated(QTrapez()," << this->degree-1 << "))";
+      else
+        namebuf << "FE_Q_Bubbles<"
+                << Utilities::dim_string(dim,spacedim)
+                << ">(" << this->degree-1 << ")";
+    }
   else
     {
 
@@ -308,9 +310,9 @@ FE_Q_Bubbles<dim,spacedim>::get_name () const
             break;
           }
       if (type == true)
-        namebuf << "FE_Q_Bubbles<" << dim << ">(QGaussLobatto(" << this->degree << "))";
+        namebuf << "FE_Q_Bubbles<" << dim << ">(" << this->degree-1 << ")";
       else
-        namebuf << "FE_Q_Bubbles<" << dim << ">(QUnknownNodes(" << this->degree-1 << "))";
+        namebuf << "FE_Q_Bubbles<" << dim << ">(QUnknownNodes(" << this->degree << "))";
     }
   return namebuf.str();
 }
@@ -326,6 +328,34 @@ FE_Q_Bubbles<dim,spacedim>::clone() const
 
 
 
+template <int dim, int spacedim>
+void
+FE_Q_Bubbles<dim,spacedim>::
+convert_generalized_support_point_values_to_nodal_values(const std::vector<Vector<double> > &support_point_values,
+                                                         std::vector<double>    &nodal_values) const
+{
+  Assert (support_point_values.size() == this->unit_support_points.size(),
+          ExcDimensionMismatch(support_point_values.size(),
+                               this->unit_support_points.size()));
+  Assert (nodal_values.size() == this->dofs_per_cell,
+          ExcDimensionMismatch(nodal_values.size(),this->dofs_per_cell));
+  Assert (support_point_values[0].size() == this->n_components(),
+          ExcDimensionMismatch(support_point_values[0].size(),this->n_components()));
+
+  for (unsigned int i=0; i<this->dofs_per_cell-1; ++i)
+    {
+      const std::pair<unsigned int, unsigned int> index
+        = this->system_to_component_index(i);
+      nodal_values[i] = support_point_values[i](index.first);
+    }
+
+  // We don't use the bubble functions for local interpolation
+  for (unsigned int i = 0; i<n_bubbles; ++i)
+    nodal_values[nodal_values.size()-i-1] = 0.;
+}
+
+
+
 template <int dim, int spacedim>
 void
 FE_Q_Bubbles<dim,spacedim>::interpolate(std::vector<double>       &local_dofs,
@@ -352,7 +382,7 @@ template <int dim, int spacedim>
 void
 FE_Q_Bubbles<dim,spacedim>::interpolate(std::vector<double>    &local_dofs,
                                         const std::vector<Vector<double> > &values,
-                                        unsigned int offset) const
+                                        const unsigned int offset) const
 {
   Assert (values.size() == this->unit_support_points.size(),
           ExcDimensionMismatch(values.size(),
