@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2012 - 2015 by the deal.II authors
+// Copyright (C) 2012 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -33,7 +33,7 @@ template <int dim, int spacedim>
 FE_Q_DG0<dim,spacedim>::FE_Q_DG0 (const unsigned int degree)
   :
   FE_Q_Base<TensorProductPolynomialsConst<dim>, dim, spacedim> (
-    TensorProductPolynomialsConst<dim>(Polynomials::LagrangeEquidistant::generate_complete_basis(degree)),
+    TensorProductPolynomialsConst<dim>(Polynomials::generate_complete_Lagrange_basis(QGaussLobatto<1>(degree+1).get_points())),
     FiniteElementData<dim>(get_dpo_vector(degree),
                            1, degree,
                            FiniteElementData<dim>::L2),
@@ -43,11 +43,7 @@ FE_Q_DG0<dim,spacedim>::FE_Q_DG0 (const unsigned int degree)
           ExcMessage ("This element can only be used for polynomial degrees "
                       "greater than zero"));
 
-  std::vector<Point<1> > support_points_1d(degree+1);
-  for (unsigned int i=0; i<=degree; ++i)
-    support_points_1d[i][0] = static_cast<double>(i)/degree;
-
-  this->initialize(support_points_1d);
+  this->initialize(QGaussLobatto<1>(degree+1).get_points());
 
   // adjust unit support point for discontinuous node
   Point<dim> point;
@@ -92,7 +88,7 @@ template <int dim, int spacedim>
 std::string
 FE_Q_DG0<dim,spacedim>::get_name () const
 {
-  // note that the FETools::get_fe_from_name function depends on the
+  // note that the FETools::get_fe_by_name function depends on the
   // particular format of the string this function returns, so they have to be
   // kept in synch
 
@@ -133,9 +129,16 @@ FE_Q_DG0<dim,spacedim>::get_name () const
       }
 
   if (type == true)
-    namebuf << "FE_Q_DG0<"
-            << Utilities::dim_string(dim,spacedim)
-            << ">(" << this->degree << ")";
+    {
+      if (this->degree > 2)
+        namebuf << "FE_Q_DG0<"
+                << Utilities::dim_string(dim,spacedim)
+                << ">(QIterated(QTrapez()," << this->degree << "))";
+      else
+        namebuf << "FE_Q_DG0<"
+                << Utilities::dim_string(dim,spacedim)
+                << ">(" << this->degree << ")";
+    }
   else
     {
 
@@ -151,7 +154,7 @@ FE_Q_DG0<dim,spacedim>::get_name () const
       if (type == true)
         namebuf << "FE_Q_DG0<"
                 << Utilities::dim_string(dim,spacedim)
-                << ">(QGaussLobatto(" << this->degree+1 << "))";
+                << ">(" << this->degree << ")";
       else
         namebuf << "FE_Q_DG0<"
                 << Utilities::dim_string(dim,spacedim)
@@ -171,6 +174,33 @@ FE_Q_DG0<dim,spacedim>::clone() const
 
 
 
+template <int dim, int spacedim>
+void
+FE_Q_DG0<dim,spacedim>::
+convert_generalized_support_point_values_to_nodal_values(const std::vector<Vector<double> > &support_point_values,
+                                                         std::vector<double>    &nodal_dofs) const
+{
+  Assert (support_point_values.size() == this->unit_support_points.size(),
+          ExcDimensionMismatch(support_point_values.size(),
+                               this->unit_support_points.size()));
+  Assert (nodal_dofs.size() == this->dofs_per_cell,
+          ExcDimensionMismatch(nodal_dofs.size(),this->dofs_per_cell));
+  Assert (support_point_values[0].size() == this->n_components(),
+          ExcDimensionMismatch(support_point_values[0].size(),this->n_components()));
+
+  for (unsigned int i=0; i<this->dofs_per_cell-1; ++i)
+    {
+      const std::pair<unsigned int, unsigned int> index
+        = this->system_to_component_index(i);
+      nodal_dofs[i] = support_point_values[i](index.first);
+    }
+
+  // We don't need the discontinuous function for local interpolation
+  nodal_dofs[nodal_dofs.size()-1] = 0.;
+}
+
+
+
 template <int dim, int spacedim>
 void
 FE_Q_DG0<dim,spacedim>::interpolate(std::vector<double>       &local_dofs,
@@ -196,7 +226,7 @@ template <int dim, int spacedim>
 void
 FE_Q_DG0<dim,spacedim>::interpolate(std::vector<double>    &local_dofs,
                                     const std::vector<Vector<double> > &values,
-                                    unsigned int offset) const
+                                    const unsigned int offset) const
 {
   Assert (values.size() == this->unit_support_points.size(),
           ExcDimensionMismatch(values.size(),
