@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2015 by the deal.II authors
+// Copyright (C) 2015 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -26,7 +26,7 @@ DEAL_II_NAMESPACE_OPEN
 
 
 template <int dim>
-FE_RannacherTurek<dim>::FE_RannacherTurek(const unsigned int degree,
+FE_RannacherTurek<dim>::FE_RannacherTurek(const unsigned int order,
                                           const unsigned int n_face_support_points) :
   FE_Poly<PolynomialsRannacherTurek<dim>, dim>(
     PolynomialsRannacherTurek<dim>(),
@@ -36,11 +36,11 @@ FE_RannacherTurek<dim>::FE_RannacherTurek(const unsigned int degree,
                            FiniteElementData<dim>::L2),
     std::vector<bool>(4, false), // restriction not implemented
     std::vector<ComponentMask>(4, std::vector<bool>(1, true))),
-  degree(degree),
+  order(order),
   n_face_support_points(n_face_support_points)
 {
   Assert(dim == 2, ExcNotImplemented());
-  Assert(degree == 0, ExcNotImplemented());
+  Assert(order == 0, ExcNotImplemented());
   this->initialize_support_points();
 }
 
@@ -63,7 +63,7 @@ std::string FE_RannacherTurek<dim>::get_name() const
   std::ostringstream namebuf;
   namebuf << "FE_RannacherTurek"
           << "<" << dim << ">"
-          << "(" << this->degree << ", " << this->n_face_support_points << ")";
+          << "(" << this->order << ", " << this->n_face_support_points << ")";
   return namebuf.str();
 }
 
@@ -72,7 +72,7 @@ std::string FE_RannacherTurek<dim>::get_name() const
 template <int dim>
 FiniteElement<dim> *FE_RannacherTurek<dim>::clone() const
 {
-  return new FE_RannacherTurek<dim>(this->degree, this->n_face_support_points);
+  return new FE_RannacherTurek<dim>(this->order, this->n_face_support_points);
 }
 
 
@@ -101,6 +101,26 @@ void FE_RannacherTurek<dim>::initialize_support_points()
 
 
 
+template <int dim>
+void
+FE_RannacherTurek<dim>::
+convert_generalized_support_point_values_to_nodal_values(const std::vector<Vector<double> > &support_point_values,
+                                                         std::vector<double> &nodal_dofs) const
+{
+  AssertDimension(support_point_values.size(), this->generalized_support_points.size());
+  AssertDimension(nodal_dofs.size(), this->dofs_per_cell);
+
+  // extract component and call scalar version of this function
+  std::vector<double> scalar_values(support_point_values.size());
+  for (unsigned int q = 0; q < support_point_values.size(); ++q)
+    {
+      scalar_values[q] = support_point_values[q][0];
+    }
+  this->interpolate(nodal_dofs, scalar_values);
+}
+
+
+
 template <int dim>
 void FE_RannacherTurek<dim>::interpolate(
   std::vector<double> &local_dofs,
@@ -133,7 +153,7 @@ template <int dim>
 void FE_RannacherTurek<dim>::interpolate(
   std::vector<double> &local_dofs,
   const std::vector<Vector<double> > &values,
-  unsigned int offset) const
+  const unsigned int offset) const
 {
   AssertDimension(values.size(), this->generalized_support_points.size());
   AssertDimension(local_dofs.size(), this->dofs_per_cell);
