@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2015 by the deal.II authors
+// Copyright (C) 2000 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -19,9 +19,10 @@
 #include <deal.II/base/thread_management.h>
 #include <deal.II/base/utilities.h>
 #include <deal.II/lac/vector.h>
+#include <deal.II/lac/la_vector.h>
 #include <deal.II/lac/block_vector.h>
-#include <deal.II/lac/parallel_vector.h>
-#include <deal.II/lac/parallel_block_vector.h>
+#include <deal.II/lac/la_parallel_vector.h>
+#include <deal.II/lac/la_parallel_block_vector.h>
 #include <deal.II/lac/petsc_parallel_vector.h>
 #include <deal.II/lac/petsc_block_vector.h>
 #include <deal.II/lac/petsc_parallel_block_vector.h>
@@ -85,11 +86,12 @@ namespace FETools
     Assert(u2.size()==dof2.n_dofs(),
            ExcDimensionMismatch(u2.size(), dof2.n_dofs()));
 
+
+    const IndexSet u2_elements = u2.locally_owned_elements();
 #ifdef DEBUG
     const IndexSet &dof1_local_dofs = dof1.locally_owned_dofs();
     const IndexSet &dof2_local_dofs = dof2.locally_owned_dofs();
     const IndexSet u1_elements = u1.locally_owned_elements();
-    const IndexSet u2_elements = u2.locally_owned_elements();
     Assert(u1_elements == dof1_local_dofs,
            ExcMessage("The provided vector and DoF handler should have the same"
                       " index sets."));
@@ -192,8 +194,13 @@ namespace FETools
 
           for (unsigned int i=0; i<dofs_per_cell2; ++i)
             {
-              u2(dofs[i])+=u2_local(i);
-              touch_count(dofs[i]) += 1;
+              // if dof is locally_owned
+              const types::global_dof_index gdi = dofs[i];
+              if (u2_elements.is_element(gdi))
+                {
+                  u2(dofs[i])+=u2_local(i);
+                  touch_count(dofs[i]) += 1;
+                }
             }
         }
     // cell1 is at the end, so should
@@ -217,7 +224,7 @@ namespace FETools
     for (types::global_dof_index i=0; i<dof2.n_dofs(); ++i)
       if (locally_owned_dofs.is_element(i))
         {
-          Assert(touch_count(i) != typename OutVector::value_type(),
+          Assert(static_cast<typename OutVector::value_type>(touch_count(i)) != typename OutVector::value_type(0),
                  ExcInternalError());
           u2(i) /= touch_count(i);
         }
@@ -230,87 +237,17 @@ namespace FETools
 
 
 
-  template <int dim, class InVector, class OutVector, int spacedim>
+  template <int dim,
+            template <int, int> class DoFHandlerType,
+            class InVector, class OutVector, int spacedim>
   void
-  back_interpolate(const DoFHandler<dim,spacedim>    &dof1,
-                   const InVector           &u1,
-                   const FiniteElement<dim,spacedim> &fe2,
-                   OutVector                &u1_interpolated)
+  back_interpolate(const DoFHandlerType<dim, spacedim> &dof1,
+                   const InVector                      &u1,
+                   const FiniteElement<dim,spacedim>   &fe2,
+                   OutVector                           &u1_interpolated)
   {
     Assert(dof1.get_fe().n_components() == fe2.n_components(),
            ExcDimensionMismatch(dof1.get_fe().n_components(), fe2.n_components()));
-    Assert(u1.size()==dof1.n_dofs(), ExcDimensionMismatch(u1.size(), dof1.n_dofs()));
-    Assert(u1_interpolated.size()==dof1.n_dofs(),
-           ExcDimensionMismatch(u1_interpolated.size(), dof1.n_dofs()));
-
-#ifdef DEBUG
-    const IndexSet &dof1_local_dofs = dof1.locally_owned_dofs();
-    const IndexSet u1_elements = u1.locally_owned_elements();
-    const IndexSet u1_interpolated_elements = u1_interpolated.locally_owned_elements();
-    Assert(u1_elements == dof1_local_dofs,
-           ExcMessage("The provided vector and DoF handler should have the same"
-                      " index sets."));
-    Assert(u1_interpolated_elements == dof1_local_dofs,
-           ExcMessage("The provided vector and DoF handler should have the same"
-                      " index sets."));
-#endif
-
-    // For continuous elements on grids
-    // with hanging nodes we need
-    // hanging node
-    // constraints. Consequently, when
-    // the elements are continuous no
-    // hanging node constraints are
-    // allowed.
-    const bool hanging_nodes_not_allowed=
-      (dof1.get_fe().dofs_per_vertex != 0) || (fe2.dofs_per_vertex != 0);
-
-    const unsigned int dofs_per_cell1=dof1.get_fe().dofs_per_cell;
-
-    Vector<typename OutVector::value_type> u1_local(dofs_per_cell1);
-    Vector<typename OutVector::value_type> u1_int_local(dofs_per_cell1);
-
-    const types::subdomain_id subdomain_id =
-      dof1.get_triangulation().locally_owned_subdomain();
-
-    typename DoFHandler<dim,spacedim>::active_cell_iterator cell = dof1.begin_active(),
-                                                            endc = dof1.end();
-
-    FullMatrix<double> interpolation_matrix(dofs_per_cell1, dofs_per_cell1);
-    get_back_interpolation_matrix(dof1.get_fe(), fe2,
-                                  interpolation_matrix);
-    for (; cell!=endc; ++cell)
-      if ((cell->subdomain_id() == subdomain_id)
-          ||
-          (subdomain_id == numbers::invalid_subdomain_id))
-        {
-          if (hanging_nodes_not_allowed)
-            for (unsigned int face=0; face<GeometryInfo<dim>::faces_per_cell; ++face)
-              Assert (cell->at_boundary(face) ||
-                      cell->neighbor(face)->level() == cell->level(),
-                      ExcHangingNodesNotAllowed(0));
-
-          cell->get_dof_values(u1, u1_local);
-          interpolation_matrix.vmult(u1_int_local, u1_local);
-          cell->set_dof_values(u1_int_local, u1_interpolated);
-        }
-
-    // if we work on a parallel PETSc vector
-    // we have to finish the work
-    u1_interpolated.compress(VectorOperation::insert);
-  }
-
-
-
-  template <int dim,
-            template <int> class DoFHandlerType,
-            class InVector, class OutVector, int spacedim>
-  void
-  back_interpolate(const DoFHandlerType<dim>         &dof1,
-                   const InVector                    &u1,
-                   const FiniteElement<dim,spacedim> &fe2,
-                   OutVector                         &u1_interpolated)
-  {
     Assert(u1.size() == dof1.n_dofs(),
            ExcDimensionMismatch(u1.size(), dof1.n_dofs()));
     Assert(u1_interpolated.size() == dof1.n_dofs(),
@@ -334,8 +271,8 @@ namespace FETools
     const types::subdomain_id subdomain_id =
       dof1.get_triangulation().locally_owned_subdomain();
 
-    typename DoFHandlerType<dim>::active_cell_iterator cell = dof1.begin_active(),
-                                                       endc = dof1.end();
+    typename DoFHandlerType<dim, spacedim>::active_cell_iterator cell = dof1.begin_active(),
+                                                                 endc = dof1.end();
 
     // map from possible fe objects in
     // dof1 to the back_interpolation
@@ -348,10 +285,6 @@ namespace FETools
           ||
           (subdomain_id == numbers::invalid_subdomain_id))
         {
-          Assert(cell->get_fe().n_components() == fe2.n_components(),
-                 ExcDimensionMismatch(cell->get_fe().n_components(),
-                                      fe2.n_components()));
-
           // For continuous elements on
           // grids with hanging nodes we
           // need hanging node
@@ -372,12 +305,12 @@ namespace FETools
 
           // make sure back_interpolation
           // matrix is available
-          if (interpolation_matrices[&cell->get_fe()] != 0)
+          if (interpolation_matrices[&cell->get_fe()] == 0)
             {
               interpolation_matrices[&cell->get_fe()] =
                 std_cxx11::shared_ptr<FullMatrix<double> >
                 (new FullMatrix<double>(dofs_per_cell1, dofs_per_cell1));
-              get_back_interpolation_matrix(dof1.get_fe(), fe2,
+              get_back_interpolation_matrix(cell->get_fe(), fe2,
                                             *interpolation_matrices[&cell->get_fe()]);
             }
 
@@ -389,8 +322,7 @@ namespace FETools
           cell->set_dof_values(u1_int_local, u1_interpolated);
         };
 
-    // if we work on a parallel PETSc vector
-    // we have to finish the work
+    // if we work on a parallel vector, we have to finish the work
     u1_interpolated.compress(VectorOperation::insert);
   }
 
@@ -471,21 +403,21 @@ namespace FETools
       }
 #endif
 
-      // special version for parallel::distributed::Vector
+      // special version for LinearAlgebra::distributed::Vector
       template <int dim, int spacedim, typename Number>
       void back_interpolate (const DoFHandler<dim,spacedim> &dof1,
                              const ConstraintMatrix &constraints1,
-                             const parallel::distributed::Vector<Number> &u1,
+                             const LinearAlgebra::distributed::Vector<Number> &u1,
                              const DoFHandler<dim,spacedim> &dof2,
                              const ConstraintMatrix &constraints2,
-                             parallel::distributed::Vector<Number> &u1_interpolated)
+                             LinearAlgebra::distributed::Vector<Number> &u1_interpolated)
       {
         IndexSet dof2_locally_owned_dofs = dof2.locally_owned_dofs();
         IndexSet dof2_locally_relevant_dofs;
         DoFTools::extract_locally_relevant_dofs (dof2,
                                                  dof2_locally_relevant_dofs);
 
-        parallel::distributed::Vector<Number>
+        LinearAlgebra::distributed::Vector<Number>
         u2 (dof2_locally_owned_dofs,
             dof2_locally_relevant_dofs,
             u1.get_mpi_communicator());
@@ -498,6 +430,7 @@ namespace FETools
   }
 
 
+
   template <int dim, class InVector, class OutVector, int spacedim>
   void back_interpolate(const DoFHandler<dim,spacedim> &dof1,
                         const ConstraintMatrix &constraints1,
@@ -615,7 +548,7 @@ namespace FETools
                                      OutVector &u1_difference)
       {
         back_interpolate(dof1, constraints1, u1, dof2, constraints2, u1_difference);
-        u1_difference.sadd(-1, u1);
+        u1_difference.sadd(-1., 1., u1);
       }
 
       // special version for Trilinos
@@ -688,8 +621,8 @@ namespace FETools
     const unsigned int n1 = dof1.get_fe().dofs_per_cell;
     const unsigned int n2 = dof2.get_fe().dofs_per_cell;
 
-    Vector<double> u1_local(n1);
-    Vector<double> u2_local(n2);
+    Vector<typename OutVector::value_type> u1_local(n1);
+    Vector<typename OutVector::value_type> u2_local(n2);
     std::vector<types::global_dof_index> dofs(n2);
 
     FullMatrix<double> matrix(n2,n1);
@@ -710,92 +643,6 @@ namespace FETools
         ++cell2;
       }
   }
-
-
-  template <int dim, class InVector, class OutVector, int spacedim>
-  void extrapolate(const DoFHandler<dim,spacedim> &dof1,
-                   const InVector &u1,
-                   const DoFHandler<dim,spacedim> &dof2,
-                   OutVector &u2)
-  {
-    ConstraintMatrix dummy;
-    dummy.close();
-    extrapolate(dof1, u1, dof2, dummy, u2);
-  }
-
-
-
-  template <int dim, class InVector, class OutVector, int spacedim>
-  void extrapolate(const DoFHandler<dim,spacedim> &dof1,
-                   const InVector &u1,
-                   const DoFHandler<dim,spacedim> &dof2,
-                   const ConstraintMatrix &constraints,
-                   OutVector &u2)
-  {
-    Assert(dof1.get_fe().n_components() == dof2.get_fe().n_components(),
-           ExcDimensionMismatch(dof1.get_fe().n_components(), dof2.get_fe().n_components()));
-    Assert(&dof1.get_triangulation()==&dof2.get_triangulation(), ExcTriangulationMismatch());
-    Assert(u1.size()==dof1.n_dofs(), ExcDimensionMismatch(u1.size(), dof1.n_dofs()));
-    Assert(u2.size()==dof2.n_dofs(), ExcDimensionMismatch(u2.size(), dof2.n_dofs()));
-
-    OutVector u3;
-    u3.reinit(u2);
-    interpolate(dof1, u1, dof2, constraints, u3);
-
-    const unsigned int dofs_per_cell  = dof2.get_fe().dofs_per_cell;
-    Vector<typename OutVector::value_type> dof_values(dofs_per_cell);
-
-    // make sure that each cell on the
-    // coarsest level is at least once
-    // refined. otherwise, we can't
-    // treat these cells and would
-    // generate a bogus result
-    {
-      typename DoFHandler<dim,spacedim>::cell_iterator cell = dof2.begin(0),
-                                                       endc = dof2.end(0);
-      for (; cell!=endc; ++cell)
-        Assert (cell->has_children(), ExcGridNotRefinedAtLeastOnce());
-    }
-
-    // then traverse grid bottom up
-    for (unsigned int level=0; level<dof1.get_triangulation().n_levels()-1; ++level)
-      {
-        typename DoFHandler<dim,spacedim>::cell_iterator cell=dof2.begin(level),
-                                                         endc=dof2.end(level);
-
-        for (; cell!=endc; ++cell)
-          if (!cell->active())
-            {
-              // check whether this
-              // cell has active
-              // children
-              bool active_children=false;
-              for (unsigned int child_n=0; child_n<cell->n_children(); ++child_n)
-                if (cell->child(child_n)->active())
-                  {
-                    active_children=true;
-                    break;
-                  }
-
-              // if there are active
-              // children, the we have
-              // to work on this
-              // cell. get the data
-              // from the one vector
-              // and set it on the
-              // other
-              if (active_children)
-                {
-                  cell->get_interpolated_dof_values(u3, dof_values);
-                  cell->set_dof_values_by_interpolation(dof_values, u2);
-                }
-            }
-      }
-
-    // Apply hanging node constraints.
-    constraints.distribute(u2);
-  }
-
 } // end of namespace FETools
 
 
