@@ -1,7 +1,6 @@
 // ---------------------------------------------------------------------
-// $Id: fe_values_inst2.cc 31527 2013-11-03 09:58:45Z maier $
 //
-// Copyright (C) 2013 by the deal.II authors
+// Copyright (C) 2013 - 2015 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -17,6 +16,5 @@
 // This file compiles the second half of the instantiations from fe_values.cc
 // to get the memory consumption below 1.5gb with gcc.
 
-#define FE_VALUES_INSTANTIATE_PART_TWO
-
+#define SPLIT_INSTANTIATIONS_INDEX 1
 #include "fe_values.cc"
