@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2001 - 2015 by the deal.II authors
+// Copyright (C) 2001 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -22,6 +22,16 @@
 #include <deal.II/base/tensor_product_polynomials.h>
 #include <deal.II/base/std_cxx11/unique_ptr.h>
 #include <deal.II/lac/full_matrix.h>
+#include <deal.II/lac/vector.h>
+#include <deal.II/lac/block_vector.h>
+#include <deal.II/lac/la_vector.h>
+#include <deal.II/lac/parallel_vector.h>
+#include <deal.II/lac/parallel_block_vector.h>
+#include <deal.II/lac/petsc_vector.h>
+#include <deal.II/lac/petsc_block_vector.h>
+#include <deal.II/lac/trilinos_vector.h>
+#include <deal.II/lac/trilinos_block_vector.h>
+#include <deal.II/lac/trilinos_parallel_block_vector.h>
 #include <deal.II/grid/tria_iterator.h>
 #include <deal.II/grid/tria_boundary.h>
 #include <deal.II/dofs/dof_accessor.h>
@@ -252,8 +262,8 @@ MappingFEField<dim,spacedim,DoFHandlerType,VectorType>::preserves_vertex_locatio
 template<int dim, int spacedim, typename VectorType, typename DoFHandlerType>
 void
 MappingFEField<dim,spacedim,VectorType,DoFHandlerType>::
-compute_shapes_virtual (const std::vector<Point<dim> >                       &unit_points,
-                        typename MappingFEField<dim, spacedim>::InternalData &data) const
+compute_shapes_virtual (const std::vector<Point<dim> >                    &unit_points,
+                        typename MappingFEField<dim, spacedim, VectorType, DoFHandlerType>::InternalData &data) const
 {
   const unsigned int n_points=unit_points.size();
 
@@ -409,27 +419,27 @@ MappingFEField<dim,spacedim,VectorType,DoFHandlerType>::compute_face_data
         {
           data.aux.resize (dim-1, std::vector<Tensor<1,spacedim> > (n_original_q_points));
 
-          // Compute tangentials to the
-          // unit cell.
-          const unsigned int nfaces = GeometryInfo<dim>::faces_per_cell;
-          data.unit_tangentials.resize (nfaces*(dim-1),
-                                        std::vector<Tensor<1,dim> > (n_original_q_points));
+          // Compute tangentials to the unit cell.
+          for (unsigned int i=0; i<data.unit_tangentials.size(); ++i)
+            data.unit_tangentials[i].resize (n_original_q_points);
+
           if (dim==2)
             {
               // ensure a counterclockwise
               // orientation of tangentials
               static const int tangential_orientation[4]= {-1,1,1,-1};
-              for (unsigned int i=0; i<nfaces; ++i)
+              for (unsigned int i=0; i<GeometryInfo<dim>::faces_per_cell; ++i)
                 {
                   Tensor<1,dim> tang;
                   tang[1-i/2]=tangential_orientation[i];
                   std::fill (data.unit_tangentials[i].begin(),
-                             data.unit_tangentials[i].end(), tang);
+                             data.unit_tangentials[i].end(),
+                             tang);
                 }
             }
           else if (dim==3)
             {
-              for (unsigned int i=0; i<nfaces; ++i)
+              for (unsigned int i=0; i<GeometryInfo<dim>::faces_per_cell; ++i)
                 {
                   Tensor<1,dim> tang1, tang2;
 
@@ -451,9 +461,11 @@ MappingFEField<dim,spacedim,VectorType,DoFHandlerType>::compute_face_data
                   // for all quadrature
                   // points on this face
                   std::fill (data.unit_tangentials[i].begin(),
-                             data.unit_tangentials[i].end(), tang1);
-                  std::fill (data.unit_tangentials[nfaces+i].begin(),
-                             data.unit_tangentials[nfaces+i].end(), tang2);
+                             data.unit_tangentials[i].end(),
+                             tang1);
+                  std::fill (data.unit_tangentials[GeometryInfo<dim>::faces_per_cell+i].begin(),
+                             data.unit_tangentials[GeometryInfo<dim>::faces_per_cell+i].end(),
+                             tang2);
                 }
             }
         }
@@ -516,10 +528,10 @@ namespace internal
      * have already been set), but only if the update_flags of the @p data
      * argument indicate so.
      */
-    template <int dim, int spacedim>
+    template <int dim, int spacedim, typename VectorType, typename DoFHandlerType>
     void
     maybe_compute_q_points (const typename dealii::QProjector<dim>::DataSetDescriptor data_set,
-                            const typename dealii::MappingFEField<dim,spacedim>::InternalData &data,
+                            const typename dealii::MappingFEField<dim,spacedim,VectorType,DoFHandlerType>::InternalData &data,
                             const FiniteElement<dim, spacedim>   &fe,
                             const ComponentMask                  &fe_mask,
                             const std::vector<unsigned int>      &fe_to_real,
@@ -553,11 +565,11 @@ namespace internal
      *
      * Skip the computation if possible as indicated by the first argument.
      */
-    template <int dim, int spacedim>
+    template <int dim, int spacedim, typename VectorType, typename DoFHandlerType>
     void
     maybe_update_Jacobians (const CellSimilarity::Similarity    cell_similarity,
                             const typename dealii::QProjector<dim>::DataSetDescriptor  data_set,
-                            const typename dealii::MappingFEField<dim,spacedim>::InternalData &data,
+                            const typename dealii::MappingFEField<dim,spacedim,VectorType,DoFHandlerType>::InternalData &data,
                             const FiniteElement<dim, spacedim> &fe,
                             const ComponentMask                &fe_mask,
                             const std::vector<unsigned int>    &fe_to_real)
@@ -622,11 +634,11 @@ namespace internal
      *
      * Skip the computation if possible as indicated by the first argument.
      */
-    template <int dim, int spacedim>
+    template <int dim, int spacedim, typename VectorType, typename DoFHandlerType>
     void
     maybe_update_jacobian_grads (const CellSimilarity::Similarity              cell_similarity,
                                  const typename dealii::QProjector<dim>::DataSetDescriptor data_set,
-                                 const typename dealii::MappingFEField<dim,spacedim>::InternalData &data,
+                                 const typename dealii::MappingFEField<dim,spacedim,VectorType,DoFHandlerType>::InternalData &data,
                                  const FiniteElement<dim, spacedim>           &fe,
                                  const ComponentMask                          &fe_mask,
                                  const std::vector<unsigned int>              &fe_to_real,
@@ -673,12 +685,12 @@ namespace internal
      *
      * Skip the computation if possible as indicated by the first argument.
      */
-    template <int dim, int spacedim>
+    template <int dim, int spacedim, typename VectorType, typename DoFHandlerType>
     void
     maybe_update_jacobian_pushed_forward_grads (
       const CellSimilarity::Similarity              cell_similarity,
       const typename dealii::QProjector<dim>::DataSetDescriptor data_set,
-      const typename dealii::MappingFEField<dim,spacedim>::InternalData &data,
+      const typename dealii::MappingFEField<dim,spacedim,VectorType,DoFHandlerType>::InternalData &data,
       const FiniteElement<dim, spacedim>           &fe,
       const ComponentMask                          &fe_mask,
       const std::vector<unsigned int>              &fe_to_real,
@@ -748,11 +760,11 @@ namespace internal
      *
      * Skip the computation if possible as indicated by the first argument.
      */
-    template <int dim, int spacedim>
+    template <int dim, int spacedim, typename VectorType, typename DoFHandlerType>
     void
     maybe_update_jacobian_2nd_derivatives (const CellSimilarity::Similarity              cell_similarity,
                                            const typename dealii::QProjector<dim>::DataSetDescriptor data_set,
-                                           const typename dealii::MappingFEField<dim,spacedim>::InternalData &data,
+                                           const typename dealii::MappingFEField<dim,spacedim,VectorType,DoFHandlerType>::InternalData &data,
                                            const FiniteElement<dim, spacedim>           &fe,
                                            const ComponentMask                          &fe_mask,
                                            const std::vector<unsigned int>              &fe_to_real,
@@ -801,12 +813,12 @@ namespace internal
      *
      * Skip the computation if possible as indicated by the first argument.
      */
-    template <int dim, int spacedim>
+    template <int dim, int spacedim, typename VectorType, typename DoFHandlerType>
     void
     maybe_update_jacobian_pushed_forward_2nd_derivatives (
       const CellSimilarity::Similarity              cell_similarity,
       const typename dealii::QProjector<dim>::DataSetDescriptor data_set,
-      const typename dealii::MappingFEField<dim,spacedim>::InternalData &data,
+      const typename dealii::MappingFEField<dim,spacedim,VectorType,DoFHandlerType>::InternalData &data,
       const FiniteElement<dim, spacedim>           &fe,
       const ComponentMask                          &fe_mask,
       const std::vector<unsigned int>              &fe_to_real,
@@ -894,11 +906,11 @@ namespace internal
    *
    * Skip the computation if possible as indicated by the first argument.
    */
-  template <int dim, int spacedim>
+  template <int dim, int spacedim, typename VectorType, typename DoFHandlerType>
   void
   maybe_update_jacobian_3rd_derivatives (const CellSimilarity::Similarity              cell_similarity,
                                          const typename dealii::QProjector<dim>::DataSetDescriptor data_set,
-                                         const typename dealii::MappingFEField<dim,spacedim>::InternalData &data,
+                                         const typename dealii::MappingFEField<dim,spacedim,VectorType,DoFHandlerType>::InternalData &data,
                                          const FiniteElement<dim, spacedim>           &fe,
                                          const ComponentMask                          &fe_mask,
                                          const std::vector<unsigned int>              &fe_to_real,
@@ -950,12 +962,12 @@ namespace internal
    *
    * Skip the computation if possible as indicated by the first argument.
    */
-  template <int dim, int spacedim>
+  template <int dim, int spacedim, typename VectorType, typename DoFHandlerType>
   void
   maybe_update_jacobian_pushed_forward_3rd_derivatives (
     const CellSimilarity::Similarity              cell_similarity,
     const typename dealii::QProjector<dim>::DataSetDescriptor data_set,
-    const typename dealii::MappingFEField<dim,spacedim>::InternalData &data,
+    const typename dealii::MappingFEField<dim,spacedim,VectorType,DoFHandlerType>::InternalData &data,
     const FiniteElement<dim, spacedim>           &fe,
     const ComponentMask                          &fe_mask,
     const std::vector<unsigned int>              &fe_to_real,
@@ -1065,14 +1077,14 @@ namespace internal
    *
    * The resulting data is put into the @p output_data argument.
    */
-  template <int dim, int spacedim>
+  template <int dim, int spacedim, typename VectorType, typename DoFHandlerType>
   void
-  maybe_compute_face_data (const dealii::MappingFEField<dim,spacedim> &mapping,
+  maybe_compute_face_data (const dealii::Mapping<dim,spacedim> &mapping,
                            const typename dealii::Triangulation<dim,spacedim>::cell_iterator &cell,
                            const unsigned int               face_no,
                            const unsigned int               subface_no,
                            const std::vector<double>        &weights,
-                           const typename dealii::MappingFEField<dim,spacedim>::InternalData &data,
+                           const typename dealii::MappingFEField<dim,spacedim,VectorType,DoFHandlerType>::InternalData &data,
                            internal::FEValues::MappingRelatedData<dim,spacedim>         &output_data)
   {
     const UpdateFlags update_flags = data.update_each;
@@ -1203,70 +1215,79 @@ namespace internal
    * 'data_set' to differentiate whether we will work on a face (and if so,
    * which one) or subface.
    */
-  template<int dim, int spacedim>
+  template<int dim, int spacedim, typename VectorType, typename DoFHandlerType>
   void
-  do_fill_fe_face_values (const dealii::MappingFEField<dim,spacedim>                        &mapping,
+  do_fill_fe_face_values (const dealii::Mapping<dim,spacedim>                               &mapping,
                           const typename dealii::Triangulation<dim,spacedim>::cell_iterator &cell,
                           const unsigned int                                                 face_no,
                           const unsigned int                                                 subface_no,
                           const typename dealii::QProjector<dim>::DataSetDescriptor          data_set,
                           const Quadrature<dim-1>                                           &quadrature,
-                          const typename dealii::MappingFEField<dim,spacedim>::InternalData &data,
+                          const typename dealii::MappingFEField<dim,spacedim,VectorType,DoFHandlerType>::InternalData    &data,
                           const FiniteElement<dim, spacedim>                                &fe,
                           const ComponentMask                                               &fe_mask,
                           const std::vector<unsigned int>                                   &fe_to_real,
                           internal::FEValues::MappingRelatedData<dim,spacedim>              &output_data)
   {
-    maybe_compute_q_points<dim,spacedim> (data_set,
-                                          data,
-                                          fe, fe_mask, fe_to_real,
-                                          output_data.quadrature_points);
+    maybe_compute_q_points<dim,spacedim,VectorType,DoFHandlerType>
+    (data_set,
+     data,
+     fe, fe_mask, fe_to_real,
+     output_data.quadrature_points);
 
-    maybe_update_Jacobians<dim,spacedim> (CellSimilarity::none,
-                                          data_set,
-                                          data,
-                                          fe, fe_mask, fe_to_real);
+    maybe_update_Jacobians<dim,spacedim,VectorType,DoFHandlerType>
+    (CellSimilarity::none,
+     data_set,
+     data,
+     fe, fe_mask, fe_to_real);
 
-    maybe_update_jacobian_grads<dim,spacedim> (CellSimilarity::none,
-                                               data_set,
-                                               data,
-                                               fe, fe_mask, fe_to_real,
-                                               output_data.jacobian_grads);
+    maybe_update_jacobian_grads<dim,spacedim,VectorType,DoFHandlerType>
+    (CellSimilarity::none,
+     data_set,
+     data,
+     fe, fe_mask, fe_to_real,
+     output_data.jacobian_grads);
 
-    maybe_update_jacobian_pushed_forward_grads<dim,spacedim> (CellSimilarity::none,
-                                                              data_set,
-                                                              data,
-                                                              fe, fe_mask, fe_to_real,
-                                                              output_data.jacobian_pushed_forward_grads);
+    maybe_update_jacobian_pushed_forward_grads<dim,spacedim,VectorType,DoFHandlerType>
+    (CellSimilarity::none,
+     data_set,
+     data,
+     fe, fe_mask, fe_to_real,
+     output_data.jacobian_pushed_forward_grads);
 
-    maybe_update_jacobian_2nd_derivatives<dim,spacedim> (CellSimilarity::none,
-                                                         data_set,
-                                                         data,
-                                                         fe, fe_mask, fe_to_real,
-                                                         output_data.jacobian_2nd_derivatives);
+    maybe_update_jacobian_2nd_derivatives<dim,spacedim,VectorType,DoFHandlerType>
+    (CellSimilarity::none,
+     data_set,
+     data,
+     fe, fe_mask, fe_to_real,
+     output_data.jacobian_2nd_derivatives);
 
-    maybe_update_jacobian_pushed_forward_2nd_derivatives<dim,spacedim> (CellSimilarity::none,
-        data_set,
-        data,
-        fe, fe_mask, fe_to_real,
-        output_data.jacobian_pushed_forward_2nd_derivatives);
+    maybe_update_jacobian_pushed_forward_2nd_derivatives<dim,spacedim,VectorType,DoFHandlerType>
+    (CellSimilarity::none,
+     data_set,
+     data,
+     fe, fe_mask, fe_to_real,
+     output_data.jacobian_pushed_forward_2nd_derivatives);
 
-    maybe_update_jacobian_3rd_derivatives<dim,spacedim> (CellSimilarity::none,
-                                                         data_set,
-                                                         data,
-                                                         fe, fe_mask, fe_to_real,
-                                                         output_data.jacobian_3rd_derivatives);
+    maybe_update_jacobian_3rd_derivatives<dim,spacedim,VectorType,DoFHandlerType>
+    (CellSimilarity::none,
+     data_set,
+     data,
+     fe, fe_mask, fe_to_real,
+     output_data.jacobian_3rd_derivatives);
 
-    maybe_update_jacobian_pushed_forward_3rd_derivatives<dim,spacedim> (CellSimilarity::none,
-        data_set,
-        data,
-        fe, fe_mask, fe_to_real,
-        output_data.jacobian_pushed_forward_3rd_derivatives);
+    maybe_update_jacobian_pushed_forward_3rd_derivatives<dim,spacedim,VectorType,DoFHandlerType>
+    (CellSimilarity::none,
+     data_set,
+     data,
+     fe, fe_mask, fe_to_real,
+     output_data.jacobian_pushed_forward_3rd_derivatives);
 
-    maybe_compute_face_data (mapping,
-                             cell, face_no, subface_no,
-                             quadrature.get_weights(), data,
-                             output_data);
+    maybe_compute_face_data<dim,spacedim,VectorType,DoFHandlerType>
+    (mapping,
+     cell, face_no, subface_no,
+     quadrature.get_weights(), data,
+     output_data);
   }
 }
 
@@ -1297,13 +1318,15 @@ fill_fe_values (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
 
   update_internal_dofs(cell, data);
 
-  internal::maybe_compute_q_points(QProjector<dim>::DataSetDescriptor::cell (),
-                                   data, *fe, fe_mask, fe_to_real,
-                                   output_data.quadrature_points);
+  internal::maybe_compute_q_points<dim,spacedim,VectorType,DoFHandlerType>
+  (QProjector<dim>::DataSetDescriptor::cell (),
+   data, *fe, fe_mask, fe_to_real,
+   output_data.quadrature_points);
 
-  internal::maybe_update_Jacobians(cell_similarity,
-                                   QProjector<dim>::DataSetDescriptor::cell (),
-                                   data, *fe, fe_mask, fe_to_real);
+  internal::maybe_update_Jacobians<dim,spacedim,VectorType,DoFHandlerType>
+  (cell_similarity,
+   QProjector<dim>::DataSetDescriptor::cell (),
+   data, *fe, fe_mask, fe_to_real);
 
   const UpdateFlags update_flags = data.update_each;
   const std::vector<double> &weights=quadrature.get_weights();
@@ -1404,41 +1427,47 @@ fill_fe_values (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
     }
 
   // calculate derivatives of the Jacobians
-  internal::maybe_update_jacobian_grads(cell_similarity,
-                                        QProjector<dim>::DataSetDescriptor::cell(),
-                                        data, *fe, fe_mask, fe_to_real,
-                                        output_data.jacobian_grads);
+  internal::maybe_update_jacobian_grads<dim,spacedim,VectorType,DoFHandlerType>
+  (cell_similarity,
+   QProjector<dim>::DataSetDescriptor::cell(),
+   data, *fe, fe_mask, fe_to_real,
+   output_data.jacobian_grads);
 
   // calculate derivatives of the Jacobians pushed forward to real cell coordinates
-  internal::maybe_update_jacobian_pushed_forward_grads(cell_similarity,
-                                                       QProjector<dim>::DataSetDescriptor::cell(),
-                                                       data, *fe, fe_mask, fe_to_real,
-                                                       output_data.jacobian_pushed_forward_grads);
+  internal::maybe_update_jacobian_pushed_forward_grads<dim,spacedim,VectorType,DoFHandlerType>
+  (cell_similarity,
+   QProjector<dim>::DataSetDescriptor::cell(),
+   data, *fe, fe_mask, fe_to_real,
+   output_data.jacobian_pushed_forward_grads);
 
   // calculate hessians of the Jacobians
-  internal::maybe_update_jacobian_2nd_derivatives(cell_similarity,
-                                                  QProjector<dim>::DataSetDescriptor::cell(),
-                                                  data, *fe, fe_mask, fe_to_real,
-                                                  output_data.jacobian_2nd_derivatives);
+  internal::maybe_update_jacobian_2nd_derivatives<dim,spacedim,VectorType,DoFHandlerType>
+  (cell_similarity,
+   QProjector<dim>::DataSetDescriptor::cell(),
+   data, *fe, fe_mask, fe_to_real,
+   output_data.jacobian_2nd_derivatives);
 
   // calculate hessians of the Jacobians pushed forward to real cell coordinates
-  internal::maybe_update_jacobian_pushed_forward_2nd_derivatives(cell_similarity,
-      QProjector<dim>::DataSetDescriptor::cell(),
-      data, *fe, fe_mask, fe_to_real,
-      output_data.jacobian_pushed_forward_2nd_derivatives);
+  internal::maybe_update_jacobian_pushed_forward_2nd_derivatives<dim,spacedim,VectorType,DoFHandlerType>
+  (cell_similarity,
+   QProjector<dim>::DataSetDescriptor::cell(),
+   data, *fe, fe_mask, fe_to_real,
+   output_data.jacobian_pushed_forward_2nd_derivatives);
 
   // calculate gradients of the hessians of the Jacobians
-  internal::maybe_update_jacobian_3rd_derivatives(cell_similarity,
-                                                  QProjector<dim>::DataSetDescriptor::cell(),
-                                                  data, *fe, fe_mask, fe_to_real,
-                                                  output_data.jacobian_3rd_derivatives);
+  internal::maybe_update_jacobian_3rd_derivatives<dim,spacedim,VectorType,DoFHandlerType>
+  (cell_similarity,
+   QProjector<dim>::DataSetDescriptor::cell(),
+   data, *fe, fe_mask, fe_to_real,
+   output_data.jacobian_3rd_derivatives);
 
   // calculate gradients of the hessians of the Jacobians pushed forward to real
   // cell coordinates
-  internal::maybe_update_jacobian_pushed_forward_3rd_derivatives(cell_similarity,
-      QProjector<dim>::DataSetDescriptor::cell(),
-      data, *fe, fe_mask, fe_to_real,
-      output_data.jacobian_pushed_forward_3rd_derivatives);
+  internal::maybe_update_jacobian_pushed_forward_3rd_derivatives<dim,spacedim,VectorType,DoFHandlerType>
+  (cell_similarity,
+   QProjector<dim>::DataSetDescriptor::cell(),
+   data, *fe, fe_mask, fe_to_real,
+   output_data.jacobian_pushed_forward_3rd_derivatives);
 
   return updated_cell_similarity;
 }
@@ -1462,18 +1491,19 @@ fill_fe_face_values (const typename Triangulation<dim,spacedim>::cell_iterator &
 
   update_internal_dofs(cell, data);
 
-  internal::do_fill_fe_face_values (*this,
-                                    cell, face_no, numbers::invalid_unsigned_int,
-                                    QProjector<dim>::DataSetDescriptor::
-                                    face (face_no,
-                                          cell->face_orientation(face_no),
-                                          cell->face_flip(face_no),
-                                          cell->face_rotation(face_no),
-                                          quadrature.size()),
-                                    quadrature,
-                                    data,
-                                    *fe, fe_mask, fe_to_real,
-                                    output_data);
+  internal::do_fill_fe_face_values<dim,spacedim,VectorType,DoFHandlerType>
+  (*this,
+   cell, face_no, numbers::invalid_unsigned_int,
+   QProjector<dim>::DataSetDescriptor::
+   face (face_no,
+         cell->face_orientation(face_no),
+         cell->face_flip(face_no),
+         cell->face_rotation(face_no),
+         quadrature.size()),
+   quadrature,
+   data,
+   *fe, fe_mask, fe_to_real,
+   output_data);
 }
 
 
@@ -1495,19 +1525,20 @@ fill_fe_subface_values (const typename Triangulation<dim,spacedim>::cell_iterato
 
   update_internal_dofs(cell, data);
 
-  internal::do_fill_fe_face_values (*this,
-                                    cell, face_no, numbers::invalid_unsigned_int,
-                                    QProjector<dim>::DataSetDescriptor::
-                                    subface (face_no, subface_no,
-                                             cell->face_orientation(face_no),
-                                             cell->face_flip(face_no),
-                                             cell->face_rotation(face_no),
-                                             quadrature.size(),
-                                             cell->subface_case(face_no)),
-                                    quadrature,
-                                    data,
-                                    *fe, fe_mask, fe_to_real,
-                                    output_data);
+  internal::do_fill_fe_face_values<dim,spacedim,VectorType,DoFHandlerType>
+  (*this,
+   cell, face_no, numbers::invalid_unsigned_int,
+   QProjector<dim>::DataSetDescriptor::
+   subface (face_no, subface_no,
+            cell->face_orientation(face_no),
+            cell->face_flip(face_no),
+            cell->face_rotation(face_no),
+            quadrature.size(),
+            cell->subface_case(face_no)),
+   quadrature,
+   data,
+   *fe, fe_mask, fe_to_real,
+   output_data);
 }
 
 
@@ -1945,7 +1976,7 @@ template<int dim, int spacedim, typename VectorType, typename DoFHandlerType>
 void
 MappingFEField<dim,spacedim,VectorType,DoFHandlerType>::update_internal_dofs
 (const typename Triangulation<dim,spacedim>::cell_iterator  &cell,
- const typename MappingFEField<dim, spacedim>::InternalData &data) const
+ const typename MappingFEField<dim, spacedim,VectorType,DoFHandlerType>::InternalData &data) const
 {
   Assert(euler_dof_handler != 0, ExcMessage("euler_dof_handler is empty"));
 
@@ -1959,6 +1990,10 @@ MappingFEField<dim,spacedim,VectorType,DoFHandlerType>::update_internal_dofs
 }
 
 // explicit instantiations
+#define SPLIT_INSTANTIATIONS_COUNT 2
+#ifndef SPLIT_INSTANTIATIONS_INDEX
+#define SPLIT_INSTANTIATIONS_INDEX 0
+#endif
 #include "mapping_fe_field.inst"
 
 
