@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2001 - 2015 by the deal.II authors
+// Copyright (C) 2001 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -16,7 +16,12 @@
 #include <deal.II/base/std_cxx11/array.h>
 #include <deal.II/fe/mapping_q1_eulerian.h>
 #include <deal.II/lac/vector.h>
+#include <deal.II/lac/block_vector.h>
+#include <deal.II/lac/la_vector.h>
+#include <deal.II/lac/la_parallel_vector.h>
+#include <deal.II/lac/la_parallel_block_vector.h>
 #include <deal.II/lac/petsc_vector.h>
+#include <deal.II/lac/petsc_block_vector.h>
 #include <deal.II/lac/trilinos_vector.h>
 #include <deal.II/lac/trilinos_block_vector.h>
 #include <deal.II/lac/trilinos_parallel_block_vector.h>
@@ -28,9 +33,20 @@
 DEAL_II_NAMESPACE_OPEN
 
 
-template <int dim, class EulerVectorType, int spacedim>
-MappingQ1Eulerian<dim, EulerVectorType, spacedim>::
-MappingQ1Eulerian (const EulerVectorType  &euler_transform_vectors,
+template <int dim, class VectorType, int spacedim>
+MappingQ1Eulerian<dim, VectorType, spacedim>::
+MappingQ1Eulerian (const DoFHandler<dim,spacedim> &shiftmap_dof_handler,
+                   const VectorType               &euler_transform_vectors)
+  :
+  MappingQGeneric<dim,spacedim>(1),
+  euler_transform_vectors(&euler_transform_vectors),
+  shiftmap_dof_handler(&shiftmap_dof_handler)
+{}
+
+
+template <int dim, class VectorType, int spacedim>
+MappingQ1Eulerian<dim, VectorType, spacedim>::
+MappingQ1Eulerian (const VectorType               &euler_transform_vectors,
                    const DoFHandler<dim,spacedim> &shiftmap_dof_handler)
   :
   MappingQGeneric<dim,spacedim>(1),
@@ -40,9 +56,9 @@ MappingQ1Eulerian (const EulerVectorType  &euler_transform_vectors,
 
 
 
-template <int dim, class EulerVectorType, int spacedim>
+template <int dim, class VectorType, int spacedim>
 std_cxx11::array<Point<spacedim>, GeometryInfo<dim>::vertices_per_cell>
-MappingQ1Eulerian<dim, EulerVectorType, spacedim>::
+MappingQ1Eulerian<dim, VectorType, spacedim>::
 get_vertices
 (const typename Triangulation<dim,spacedim>::cell_iterator &cell) const
 {
@@ -89,9 +105,9 @@ get_vertices
 
 
 
-template<int dim, class EulerVectorType, int spacedim>
+template<int dim, class VectorType, int spacedim>
 std::vector<Point<spacedim> >
-MappingQ1Eulerian<dim,EulerVectorType,spacedim>::
+MappingQ1Eulerian<dim,VectorType,spacedim>::
 compute_mapping_support_points(const typename Triangulation<dim,spacedim>::cell_iterator &cell) const
 {
   const std_cxx11::array<Point<spacedim>, GeometryInfo<dim>::vertices_per_cell>
@@ -108,18 +124,18 @@ compute_mapping_support_points(const typename Triangulation<dim,spacedim>::cell_
 
 
 
-template <int dim, class EulerVectorType, int spacedim>
-MappingQ1Eulerian<dim,EulerVectorType,spacedim> *
-MappingQ1Eulerian<dim, EulerVectorType, spacedim>::clone () const
+template <int dim, class VectorType, int spacedim>
+MappingQ1Eulerian<dim,VectorType,spacedim> *
+MappingQ1Eulerian<dim, VectorType, spacedim>::clone () const
 {
-  return new MappingQ1Eulerian<dim,EulerVectorType,spacedim>(*this);
+  return new MappingQ1Eulerian<dim,VectorType,spacedim>(*this);
 }
 
 
 
-template<int dim, class EulerVectorType, int spacedim>
+template<int dim, class VectorType, int spacedim>
 CellSimilarity::Similarity
-MappingQ1Eulerian<dim,EulerVectorType,spacedim>::
+MappingQ1Eulerian<dim,VectorType,spacedim>::
 fill_fe_values (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
                 const CellSimilarity::Similarity                           ,
                 const Quadrature<dim>                                     &quadrature,
