@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2006 - 2014 by the deal.II authors
+// Copyright (C) 2006 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,6 +14,7 @@
 // ---------------------------------------------------------------------
 
 #include <deal.II/base/memory_consumption.h>
+#include <deal.II/base/std_cxx11/bind.h>
 #include <deal.II/grid/tria_objects.h>
 #include <deal.II/grid/tria.h>
 #include <deal.II/grid/tria_iterator.h>
@@ -170,7 +171,7 @@ namespace internal
       const unsigned int new_size = new_hexes +
                                     std::count_if (used.begin(),
                                                    used.end(),
-                                                   std::bind2nd (std::equal_to<bool>(), true));
+                                                   std_cxx11::bind (std::equal_to<bool>(), std_cxx11::_1, true));
 
       // see above...
       if (new_size>cells.size())
@@ -195,7 +196,7 @@ namespace internal
                            4*new_size-children.size(),
                            -1);
 
-          // for the following two fields, we know exactly how many elements
+          // for the following fields, we know exactly how many elements
           // we need, so first reserve then resize (resize itself, at least
           // with some compiler libraries, appears to round up the size it
           // actually reserves)
