@@ -1,7 +1,6 @@
 // ---------------------------------------------------------------------
-// $Id: fe_values.cc 30037 2013-07-18 16:55:40Z maier $
 //
-// Copyright (C) 2003 - 2013 by the deal.II authors
+// Copyright (C) 2003 - 2015 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -26,12 +25,12 @@ namespace internal
   {
 // -------------------------- FEValuesBase -------------------------
 
-    template <int dim, int q_dim, class FEValues>
-    FEValuesBase<dim,q_dim,FEValues>::
-    FEValuesBase (const dealii::hp::MappingCollection<dim,FEValues::space_dimension> &mapping_collection,
-                  const dealii::hp::FECollection<dim,FEValues::space_dimension>      &fe_collection,
-                  const dealii::hp::QCollection<q_dim>     &q_collection,
-                  const UpdateFlags                   update_flags)
+    template <int dim, int q_dim, class FEValuesType>
+    FEValuesBase<dim,q_dim,FEValuesType>::FEValuesBase
+    (const dealii::hp::MappingCollection<dim,FEValuesType::space_dimension> &mapping_collection,
+     const dealii::hp::FECollection<dim,FEValuesType::space_dimension>      &fe_collection,
+     const dealii::hp::QCollection<q_dim>                                   &q_collection,
+     const UpdateFlags                                                       update_flags)
       :
       fe_collection (&fe_collection),
       mapping_collection (&mapping_collection),
@@ -46,14 +45,14 @@ namespace internal
     {}
 
 
-    template <int dim, int q_dim, class FEValues>
-    FEValuesBase<dim,q_dim,FEValues>::
-    FEValuesBase (const dealii::hp::FECollection<dim,FEValues::space_dimension>      &fe_collection,
-                  const dealii::hp::QCollection<q_dim> &q_collection,
-                  const UpdateFlags         update_flags)
+    template <int dim, int q_dim, class FEValuesType>
+    FEValuesBase<dim,q_dim,FEValuesType>::FEValuesBase
+    (const dealii::hp::FECollection<dim,FEValuesType::space_dimension> &fe_collection,
+     const dealii::hp::QCollection<q_dim>                              &q_collection,
+     const UpdateFlags                                                  update_flags)
       :
       fe_collection (&fe_collection),
-      mapping_collection (&dealii::hp::StaticMappingQ1<dim,FEValues::space_dimension>::
+      mapping_collection (&dealii::hp::StaticMappingQ1<dim,FEValuesType::space_dimension>::
                           mapping_collection),
       q_collection (q_collection),
       fe_values_table (fe_collection.size(),
@@ -67,12 +66,12 @@ namespace internal
 
 
 
-    template <int dim, int q_dim, class FEValues>
-    FEValues &
-    FEValuesBase<dim,q_dim,FEValues>::
-    select_fe_values (const unsigned int fe_index,
-                      const unsigned int mapping_index,
-                      const unsigned int q_index)
+    template <int dim, int q_dim, class FEValuesType>
+    FEValuesType &
+    FEValuesBase<dim,q_dim,FEValuesType>::select_fe_values
+    (const unsigned int fe_index,
+     const unsigned int mapping_index,
+     const unsigned int q_index)
     {
       Assert (fe_index < fe_collection->size(),
               ExcIndexRange (fe_index, 0, fe_collection->size()));
@@ -95,11 +94,11 @@ namespace internal
       if (fe_values_table(present_fe_values_index).get() == 0)
         fe_values_table(present_fe_values_index)
           =
-            std_cxx1x::shared_ptr<FEValues>
-            (new FEValues ((*mapping_collection)[mapping_index],
-                           (*fe_collection)[fe_index],
-                           q_collection[q_index],
-                           update_flags));
+            std_cxx11::shared_ptr<FEValuesType>
+            (new FEValuesType ((*mapping_collection)[mapping_index],
+                               (*fe_collection)[fe_index],
+                               q_collection[q_index],
+                               update_flags));
 
       // now there definitely is one!
       return *fe_values_table(present_fe_values_index);
@@ -140,9 +139,9 @@ namespace hp
 
 
   template <int dim, int spacedim>
-  template <class DH, bool lda>
+  template <typename DoFHandlerType, bool lda>
   void
-  FEValues<dim,spacedim>::reinit (const TriaIterator<DoFCellAccessor<DH,lda> > cell,
+  FEValues<dim,spacedim>::reinit (const TriaIterator<DoFCellAccessor<DoFHandlerType,lda> > cell,
                                   const unsigned int q_index,
                                   const unsigned int mapping_index,
                                   const unsigned int fe_index)
@@ -257,9 +256,9 @@ namespace hp
 
 
   template <int dim, int spacedim>
-  template <class DH, bool lda>
+  template <typename DoFHandlerType, bool lda>
   void
-  FEFaceValues<dim,spacedim>::reinit (const TriaIterator<DoFCellAccessor<DH,lda> > cell,
+  FEFaceValues<dim,spacedim>::reinit (const TriaIterator<DoFCellAccessor<DoFHandlerType,lda> > cell,
                                       const unsigned int face_no,
                                       const unsigned int q_index,
                                       const unsigned int mapping_index,
@@ -376,9 +375,9 @@ namespace hp
 
 
   template <int dim, int spacedim>
-  template <class DH, bool lda>
+  template <typename DoFHandlerType, bool lda>
   void
-  FESubfaceValues<dim,spacedim>::reinit (const TriaIterator<DoFCellAccessor<DH,lda> > cell,
+  FESubfaceValues<dim,spacedim>::reinit (const TriaIterator<DoFCellAccessor<DoFHandlerType,lda> > cell,
                                          const unsigned int face_no,
                                          const unsigned int subface_no,
                                          const unsigned int q_index,
