@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2015 by the deal.II authors
+// Copyright (C) 2000 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -47,14 +47,15 @@ template <class SparsityPatternBase>
 BlockSparsityPatternBase<SparsityPatternBase>::
 BlockSparsityPatternBase (const BlockSparsityPatternBase &s)
   :
-  Subscriptor ()
+  Subscriptor (),
+  rows (0),
+  columns (0)
 {
   (void)s;
-  Assert(s.rows==0, ExcInvalidConstructorCall());
-  Assert(s.columns==0, ExcInvalidConstructorCall());
-
-  rows = 0;
-  columns=0;
+  Assert (s.rows==0 && s.columns==0,
+          ExcMessage("This constructor can only be called if the provided argument "
+                     "is the sparsity pattern for an empty matrix. This constructor can "
+                     "not be used to copy-construct a non-empty sparsity pattern."));
 }
 
 
@@ -63,7 +64,12 @@ template <class SparsityPatternBase>
 BlockSparsityPatternBase<SparsityPatternBase>::~BlockSparsityPatternBase ()
 {
   // clear all memory
-  reinit (0,0);
+  try
+    {
+      reinit (0,0);
+    }
+  catch (...)
+    {}
 }
 
 
