@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2015 by the deal.II authors
+// Copyright (C) 1998 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -23,8 +23,9 @@
 #include <deal.II/lac/sparse_matrix_ez.h>
 #include <deal.II/lac/chunk_sparse_matrix.h>
 #include <deal.II/lac/block_sparse_matrix_ez.h>
-#include <deal.II/lac/parallel_vector.h>
-#include <deal.II/lac/parallel_block_vector.h>
+#include <deal.II/lac/la_vector.h>
+#include <deal.II/lac/la_parallel_vector.h>
+#include <deal.II/lac/la_parallel_block_vector.h>
 #include <deal.II/lac/petsc_vector.h>
 #include <deal.II/lac/petsc_block_vector.h>
 #include <deal.II/lac/petsc_sparse_matrix.h>
@@ -38,6 +39,7 @@
 #include <deal.II/lac/trilinos_sparse_matrix.h>
 #include <deal.II/lac/trilinos_block_sparse_matrix.h>
 #include <deal.II/lac/matrix_block.h>
+#include <deal.II/lac/diagonal_matrix.h>
 
 #include <algorithm>
 #include <numeric>
@@ -473,19 +475,19 @@ void ConstraintMatrix::close ()
 
 void
 ConstraintMatrix::merge (const ConstraintMatrix &other_constraints,
-                         const MergeConflictBehavior merge_conflict_behavior)
+                         const MergeConflictBehavior merge_conflict_behavior,
+                         const bool allow_different_local_lines)
 {
-  AssertThrow(local_lines == other_constraints.local_lines,
-              ExcNotImplemented());
+  (void) allow_different_local_lines;
+  Assert(allow_different_local_lines ||
+         local_lines == other_constraints.local_lines,
+         ExcMessage("local_lines for this and the other objects are not the same "
+                    "although allow_different_local_lines is false."));
 
   // store the previous state with respect to sorting
   const bool object_was_sorted = sorted;
   sorted = false;
 
-  if (other_constraints.lines_cache.size() > lines_cache.size())
-    lines_cache.resize(other_constraints.lines_cache.size(),
-                       numbers::invalid_size_type);
-
   // first action is to fold into the present object possible constraints
   // in the second object. we don't strictly need to do this any more since
   // the ConstraintMatrix has learned to deal with chains of constraints in
@@ -501,15 +503,16 @@ ConstraintMatrix::merge (const ConstraintMatrix &other_constraints,
       tmp.clear ();
       for (size_type i=0; i<line->entries.size(); ++i)
         {
-          // if the present dof is not constrained, or if we won't take the
+          // if the present dof is not stored, or not constrained, or if we won't take the
           // constraint from the other object, then simply copy it over
-          if (other_constraints.is_constrained(line->entries[i].first) == false
+          if ((other_constraints.local_lines.size() != 0
+               && other_constraints.local_lines.is_element(line->entries[i].first) == false)
+              ||
+              other_constraints.is_constrained(line->entries[i].first) == false
               ||
               ((merge_conflict_behavior != right_object_wins)
-               &&
-               other_constraints.is_constrained(line->entries[i].first)
-               &&
-               this->is_constrained(line->entries[i].first)))
+               && other_constraints.is_constrained(line->entries[i].first)
+               && this->is_constrained(line->entries[i].first)))
             tmp.push_back(line->entries[i]);
           else
             // otherwise resolve further constraints by replacing the old
@@ -528,56 +531,83 @@ ConstraintMatrix::merge (const ConstraintMatrix &other_constraints,
                 tmp.push_back (std::pair<size_type,double>(j->first,
                                                            j->second*weight));
 
-              line->inhomogeneity += other_constraints.get_inhomogeneity(line->entries[i].first) *
-                                     weight;
+              line->inhomogeneity
+              += other_constraints.get_inhomogeneity(line->entries[i].first) *
+                 weight;
             }
         }
       // finally exchange old and newly resolved line
       line->entries.swap (tmp);
     }
 
+  if (local_lines.size() != 0)
+    local_lines.add_indices(other_constraints.local_lines);
 
+  {
+    // do not bother to resize the lines cache exactly since it is pretty
+    // cheap to adjust it along the way.
+    std::fill(lines_cache.begin(), lines_cache.end(), numbers::invalid_size_type);
 
-  // next action: append those lines at the end that we want to add
-  for (std::vector<ConstraintLine>::const_iterator
-       line=other_constraints.lines.begin();
-       line!=other_constraints.lines.end(); ++line)
-    if (is_constrained(line->line) == false)
-      lines.push_back (*line);
-    else
+    // reset lines_cache for our own constraints
+    size_type index = 0;
+    for (std::vector<ConstraintLine>::const_iterator line = lines.begin();
+         line != lines.end(); ++line)
       {
-        // the constrained dof we want to copy from the other object is
-        // also constrained here. let's see what we should do with that
-        switch (merge_conflict_behavior)
+        size_type local_line_no = calculate_line_index(line->line);
+        if (local_line_no >= lines_cache.size())
+          lines_cache.resize(local_line_no+1, numbers::invalid_size_type);
+        lines_cache[local_line_no] = index++;
+      }
+
+    // Add other_constraints to lines cache and our list of constraints
+    for (std::vector<ConstraintLine>::const_iterator line = other_constraints.lines.begin();
+         line != other_constraints.lines.end(); ++line)
+      {
+        const size_type local_line_no = calculate_line_index(line->line);
+        if (local_line_no >= lines_cache.size())
           {
-          case no_conflicts_allowed:
-            AssertThrow (false,
-                         ExcDoFIsConstrainedFromBothObjects (line->line));
-            break;
+            lines_cache.resize(local_line_no+1, numbers::invalid_size_type);
+            lines.push_back(*line);
+            lines_cache[local_line_no] = index++;
+          }
+        else if (lines_cache[local_line_no] == numbers::invalid_size_type)
+          {
+            // there are no constraints for that line yet
+            lines.push_back(*line);
+            AssertIndexRange(local_line_no, lines_cache.size());
+            lines_cache[local_line_no] = index++;
+          }
+        else
+          {
+            // we already store that line
+            switch (merge_conflict_behavior)
+              {
+              case no_conflicts_allowed:
+                AssertThrow (false,
+                             ExcDoFIsConstrainedFromBothObjects (line->line));
+                break;
 
-          case left_object_wins:
-            // ignore this constraint
-            break;
+              case left_object_wins:
+                // ignore this constraint
+                break;
 
-          case right_object_wins:
-            // we need to replace the existing constraint by the one from
-            // the other object
-            lines[lines_cache[calculate_line_index(line->line)]].entries
-              = line->entries;
-            lines[lines_cache[calculate_line_index(line->line)]].inhomogeneity
-              = line->inhomogeneity;
-            break;
+              case right_object_wins:
+                AssertIndexRange(local_line_no, lines_cache.size());
+                lines[lines_cache[local_line_no]] = *line;
+                break;
 
-          default:
-            Assert (false, ExcNotImplemented());
+              default:
+                Assert (false, ExcNotImplemented());
+              }
           }
       }
 
-  // update the lines cache
-  size_type counter = 0;
-  for (std::vector<ConstraintLine>::const_iterator line=lines.begin();
-       line!=lines.end(); ++line, ++counter)
-    lines_cache[calculate_line_index(line->line)] = counter;
+    // check that we set the pointers correctly
+    for (size_type i=0; i<lines_cache.size(); ++i)
+      if (lines_cache[i] != numbers::invalid_size_type)
+        Assert (i == calculate_line_index(lines[lines_cache[i]].line),
+                ExcInternalError());
+  }
 
   // if the object was sorted before, then make sure it is so afterward as
   // well. otherwise leave everything in the unsorted state
@@ -589,11 +619,16 @@ ConstraintMatrix::merge (const ConstraintMatrix &other_constraints,
 
 void ConstraintMatrix::shift (const size_type offset)
 {
-  //TODO: this doesn't work with IndexSets yet. [TH]
-  AssertThrow(local_lines.size()==0, ExcNotImplemented());
-
-  lines_cache.insert (lines_cache.begin(), offset,
-                      numbers::invalid_size_type);
+  if (local_lines.size() == 0)
+    lines_cache.insert (lines_cache.begin(), offset,
+                        numbers::invalid_size_type);
+  else
+    {
+      // shift local_lines
+      IndexSet new_local_lines(local_lines.size());
+      new_local_lines.add_indices(local_lines, offset);
+      std::swap(local_lines, new_local_lines);
+    }
 
   for (std::vector<ConstraintLine>::iterator i = lines.begin();
        i != lines.end(); ++i)
@@ -604,6 +639,15 @@ void ConstraintMatrix::shift (const size_type offset)
            j != i->entries.end(); ++j)
         j->first += offset;
     }
+
+#ifdef DEBUG
+  // make sure that lines, lines_cache and local_lines
+  // are still linked correctly
+  for (size_type i=0; i<lines_cache.size(); ++i)
+    Assert(lines_cache[i] == numbers::invalid_size_type ||
+           calculate_line_index(lines[lines_cache[i]].line) == i,
+           ExcInternalError());
+#endif
 }
 
 
@@ -1222,18 +1266,31 @@ ConstraintMatrix::resolve_indices (std::vector<types::global_dof_index> &indices
                                                         VectorType       &condensed) const;\
   template void ConstraintMatrix::condense<VectorType >(VectorType &vec) const;\
   template void ConstraintMatrix:: \
-  distribute_local_to_global<VectorType > (const Vector<double>            &, \
+  distribute_local_to_global<VectorType > (const Vector<VectorType::value_type>            &, \
                                            const std::vector<ConstraintMatrix::size_type>  &, \
                                            VectorType                      &, \
-                                           const FullMatrix<double>        &) const
+                                           const FullMatrix<VectorType::value_type>        &) const;\
+  template void ConstraintMatrix:: \
+  distribute_local_to_global<VectorType > (const Vector<VectorType::value_type>            &, \
+                                           const std::vector<ConstraintMatrix::size_type>  &, \
+                                           const std::vector<ConstraintMatrix::size_type>  &, \
+                                           VectorType                      &, \
+                                           const FullMatrix<VectorType::value_type> &, \
+                                           bool) const
 
 #define PARALLEL_VECTOR_FUNCTIONS(VectorType) \
   template void ConstraintMatrix:: \
-  distribute_local_to_global<VectorType > (const Vector<double>            &, \
+  distribute_local_to_global<VectorType > (const Vector<VectorType::value_type>            &, \
                                            const std::vector<ConstraintMatrix::size_type>  &, \
                                            VectorType                      &, \
-                                           const FullMatrix<double>        &) const
-
+                                           const FullMatrix<VectorType::value_type>        &) const;\
+  template void ConstraintMatrix:: \
+  distribute_local_to_global<VectorType > (const Vector<VectorType::value_type>            &, \
+                                           const std::vector<ConstraintMatrix::size_type>  &, \
+                                           const std::vector<ConstraintMatrix::size_type>  &, \
+                                           VectorType                      &, \
+                                           const FullMatrix<VectorType::value_type> &, \
+                                           bool) const
 
 #ifdef DEAL_II_WITH_PETSC
 VECTOR_FUNCTIONS(PETScWrappers::MPI::Vector);
@@ -1287,6 +1344,8 @@ MATRIX_FUNCTIONS(SparseMatrix<float>);
 MATRIX_FUNCTIONS(FullMatrix<double>);
 MATRIX_FUNCTIONS(FullMatrix<float>);
 MATRIX_FUNCTIONS(FullMatrix<std::complex<double> >);
+MATRIX_FUNCTIONS(SparseMatrix<std::complex<double> >);
+MATRIX_FUNCTIONS(SparseMatrix<std::complex<float> >);
 
 BLOCK_MATRIX_FUNCTIONS(BlockSparseMatrix<double>);
 BLOCK_MATRIX_FUNCTIONS(BlockSparseMatrix<float>);
@@ -1403,10 +1462,8 @@ namespace internals
 
   SCRATCH_INITIALIZER(double,double);
   SCRATCH_INITIALIZER(float,float);
-  SCRATCH_INITIALIZER(long double,ldouble);
   SCRATCH_INITIALIZER(std::complex<double>,cdouble);
   SCRATCH_INITIALIZER(std::complex<float>,cfloat);
-  SCRATCH_INITIALIZER(std::complex<long double>,cldouble);
 #undef SCRATCH_INITIALIZER
 }
 
