@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2005 - 2015 by the deal.II authors
+// Copyright (C) 2005 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -130,6 +130,41 @@ LAPACKFullMatrix<number>::operator = (const double d)
 }
 
 
+template <typename number>
+LAPACKFullMatrix<number> &
+LAPACKFullMatrix<number>::operator*= (const number factor)
+{
+  Assert(state == LAPACKSupport::matrix ||
+         state == LAPACKSupport::inverse_matrix,
+         ExcState(state));
+
+  for (unsigned int column = 0; column<this->n(); ++column)
+    for (unsigned int row = 0; row<this->m(); ++row)
+      (*this)(row,column) *= factor;
+
+  return *this;
+}
+
+
+template <typename number>
+LAPACKFullMatrix<number> &
+LAPACKFullMatrix<number>::operator/= (const number factor)
+{
+  Assert(state == LAPACKSupport::matrix ||
+         state == LAPACKSupport::inverse_matrix,
+         ExcState(state));
+
+  AssertIsFinite(factor);
+  Assert (factor != number(0.), ExcZero() );
+
+  for (unsigned int column = 0; column<this->n(); ++column)
+    for (unsigned int row = 0; row<this->m(); ++row)
+      (*this)(row,column) /= factor;
+
+  return *this;
+}
+
+
 template <typename number>
 void
 LAPACKFullMatrix<number>::vmult (
@@ -461,6 +496,8 @@ void
 LAPACKFullMatrix<number>::compute_lu_factorization()
 {
   Assert(state == matrix, ExcState(state));
+  state = LAPACKSupport::unusable;
+
   const int mm = this->n_rows();
   const int nn = this->n_cols();
   number *values = const_cast<number *> (&this->values[0]);
@@ -468,10 +505,12 @@ LAPACKFullMatrix<number>::compute_lu_factorization()
   int info = 0;
   getrf(&mm, &nn, values, &mm, &ipiv[0], &info);
 
-  AssertThrow(info >= 0, ExcInternalError());
-  AssertThrow(info == 0, LACExceptions::ExcSingular());
+  Assert(info >= 0, ExcInternalError());
 
+  // if info >= 0, the factorization has been completed
   state = lu;
+
+  AssertThrow(info == 0, LACExceptions::ExcSingular());
 }
 
 
@@ -559,14 +598,14 @@ LAPACKFullMatrix<number>::invert()
     {
       getrf(&mm, &nn, values, &mm, &ipiv[0], &info);
 
-      AssertThrow(info >= 0, ExcInternalError());
+      Assert(info >= 0, ExcInternalError());
       AssertThrow(info == 0, LACExceptions::ExcSingular());
     }
 
   inv_work.resize (mm);
   getri(&mm, values, &mm, &ipiv[0], &inv_work[0], &mm, &info);
 
-  AssertThrow(info >= 0, ExcInternalError());
+  Assert(info >= 0, ExcInternalError());
   AssertThrow(info == 0, LACExceptions::ExcSingular());
 
   state = inverse_matrix;
@@ -591,7 +630,7 @@ LAPACKFullMatrix<number>::apply_lu_factorization(Vector<number> &v,
   getrs(trans, &nn, &one, values, &nn, &ipiv[0],
         v.begin(), &nn, &info);
 
-  AssertThrow(info == 0, ExcInternalError());
+  Assert(info == 0, ExcInternalError());
 }
 
 
@@ -613,7 +652,7 @@ LAPACKFullMatrix<number>::apply_lu_factorization(LAPACKFullMatrix<number> &B,
 
   getrs(trans, &nn, &kk, values, &nn, &ipiv[0], &B.values[0], &nn, &info);
 
-  AssertThrow(info == 0, ExcInternalError());
+  Assert(info == 0, ExcInternalError());
 }
 
 
@@ -906,7 +945,7 @@ LAPACKFullMatrix<number>::compute_generalized_eigenvalues_symmetric (
   lwork = static_cast<int>(work[0] + 1);
 
   // resize workspace array
-  work.resize((size_type) lwork);
+  work.resize(static_cast<size_type>(lwork));
 
   // Finally compute the generalized eigenvalues.
   sygv (&itype, jobz, uplo, &nn, values_A, &nn,
@@ -946,6 +985,9 @@ LAPACKFullMatrix<number>::print_formatted (
 
   Assert ((!this->empty()) || (this->n_cols()+this->n_rows()==0),
           ExcInternalError());
+  Assert (state == LAPACKSupport::matrix ||
+          state == LAPACKSupport::inverse_matrix,
+          ExcState(state));
 
   // set output format, but store old
   // state
