@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2015 by the deal.II authors
+// Copyright (C) 2004 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -17,6 +17,8 @@
 
 #ifdef DEAL_II_WITH_PETSC
 
+#  include <deal.II/lac/exceptions.h>
+#  include <deal.II/lac/petsc_compatibility.h>
 #  include <deal.II/lac/petsc_full_matrix.h>
 #  include <deal.II/lac/petsc_sparse_matrix.h>
 #  include <deal.II/lac/petsc_parallel_sparse_matrix.h>
@@ -49,10 +51,9 @@ namespace PETScWrappers
       const PetscInt    *colnums;
       const PetscScalar *values;
 
-      int ierr;
-      (void)ierr;
-      ierr = MatGetRow(*matrix, this->a_row, &ncols, &colnums, &values);
-      AssertThrow (ierr == 0, MatrixBase::ExcPETScError(ierr));
+      PetscErrorCode ierr = MatGetRow(*matrix, this->a_row, &ncols, &colnums,
+                                      &values);
+      AssertThrow (ierr == 0, ExcPETScError(ierr));
 
       // copy it into our caches if the line
       // isn't empty. if it is, then we've
@@ -66,7 +67,7 @@ namespace PETScWrappers
 
       // and finally restore the matrix
       ierr = MatRestoreRow(*matrix, this->a_row, &ncols, &colnums, &values);
-      AssertThrow (ierr == 0, MatrixBase::ExcPETScError(ierr));
+      AssertThrow (ierr == 0, ExcPETScError(ierr));
     }
   }
 
@@ -74,6 +75,7 @@ namespace PETScWrappers
 
   MatrixBase::MatrixBase ()
     :
+    matrix (NULL),
     last_action (VectorOperation::unknown)
   {}
 
@@ -81,12 +83,7 @@ namespace PETScWrappers
 
   MatrixBase::~MatrixBase ()
   {
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-    const int ierr = MatDestroy (matrix);
-#else
-    const int ierr = MatDestroy (&matrix);
-#endif
-    AssertThrow (ierr == 0, ExcPETScError(ierr));
+    destroy_matrix (matrix);
   }
 
 
@@ -95,17 +92,17 @@ namespace PETScWrappers
   MatrixBase::clear ()
   {
     // destroy the matrix...
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-    int ierr = MatDestroy (matrix);
-#else
-    int ierr = MatDestroy (&matrix);
-#endif
-    AssertThrow (ierr == 0, ExcPETScError(ierr));
+    {
+      const PetscErrorCode ierr = destroy_matrix (matrix);
+      AssertThrow(ierr == 0, ExcPETScError(ierr));
+    }
+
     // ...and replace it by an empty
     // sequential matrix
     const int m=0, n=0, n_nonzero_per_row=0;
-    ierr = MatCreateSeqAIJ(PETSC_COMM_SELF, m, n, n_nonzero_per_row,
-                           0, &matrix);
+    const PetscErrorCode ierr = MatCreateSeqAIJ(PETSC_COMM_SELF, m, n,
+                                                n_nonzero_per_row,
+                                                0, &matrix);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
@@ -119,7 +116,7 @@ namespace PETScWrappers
 
     assert_is_compressed ();
 
-    const int ierr = MatZeroEntries (matrix);
+    const PetscErrorCode ierr = MatZeroEntries (matrix);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return *this;
@@ -131,32 +128,8 @@ namespace PETScWrappers
   MatrixBase::clear_row (const size_type   row,
                          const PetscScalar new_diag_value)
   {
-    assert_is_compressed ();
-
-    // now set all the entries of this row to zero
-    const PetscInt petsc_row = row;
-
-    IS index_set;
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-    ISCreateGeneral (get_mpi_communicator(), 1, &petsc_row, &index_set);
-#else
-    ISCreateGeneral (get_mpi_communicator(), 1, &petsc_row, PETSC_COPY_VALUES, &index_set);
-#endif
-
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-    const int ierr
-      = MatZeroRowsIS(matrix, index_set, new_diag_value);
-#else
-    const int ierr
-      = MatZeroRowsIS(matrix, index_set, new_diag_value, PETSC_NULL, PETSC_NULL);
-#endif
-    AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-    ISDestroy (index_set);
-#else
-    ISDestroy (&index_set);
-#endif
+    std::vector<size_type> rows (1, row);
+    clear_rows(rows, new_diag_value);
   }
 
 
@@ -185,11 +158,10 @@ namespace PETScWrappers
 #endif
 
 #if DEAL_II_PETSC_VERSION_LT(3,2,0)
-    const int ierr
-      = MatZeroRowsIS(matrix, index_set, new_diag_value);
+    const PetscErrorCode ierr = MatZeroRowsIS(matrix, index_set, new_diag_value);
 #else
-    const int ierr
-      = MatZeroRowsIS(matrix, index_set, new_diag_value, PETSC_NULL, PETSC_NULL);
+    const PetscErrorCode ierr = MatZeroRowsIS(matrix, index_set, new_diag_value,
+                                              PETSC_NULL, PETSC_NULL);
 #endif
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
@@ -210,9 +182,8 @@ namespace PETScWrappers
 
     PetscScalar value;
 
-    const int ierr
-      = MatGetValues (matrix, 1, &petsc_i, 1, &petsc_j,
-                      &value);
+    const PetscErrorCode ierr = MatGetValues (matrix, 1, &petsc_i, 1, &petsc_j,
+                                              &value);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return value;
@@ -235,28 +206,32 @@ namespace PETScWrappers
   void
   MatrixBase::compress (const VectorOperation::values operation)
   {
+    {
 #ifdef DEBUG
 #ifdef DEAL_II_WITH_MPI
-    // Check that all processors agree that last_action is the same (or none!)
+      // Check that all processors agree that last_action is the same (or none!)
 
-    int my_int_last_action = last_action;
-    int all_int_last_action;
+      int my_int_last_action = last_action;
+      int all_int_last_action;
 
-    MPI_Allreduce(&my_int_last_action, &all_int_last_action, 1, MPI_INT,
-                  MPI_BOR, get_mpi_communicator());
+      const int ierr = MPI_Allreduce
+                       (&my_int_last_action, &all_int_last_action, 1, MPI_INT, MPI_BOR,
+                        get_mpi_communicator());
+      AssertThrowMPI(ierr);
 
-    AssertThrow(all_int_last_action != (VectorOperation::add | VectorOperation::insert),
-                ExcMessage("Error: not all processors agree on the last VectorOperation before this compress() call."));
+      AssertThrow(all_int_last_action != (VectorOperation::add | VectorOperation::insert),
+                  ExcMessage("Error: not all processors agree on the last "
+                             "VectorOperation before this compress() call."));
 #endif
 #endif
+    }
 
     AssertThrow(last_action == VectorOperation::unknown
                 || last_action == operation,
                 ExcMessage("Missing compress() or calling with wrong VectorOperation argument."));
 
     // flush buffers
-    int ierr;
-    ierr = MatAssemblyBegin (matrix,MAT_FINAL_ASSEMBLY);
+    PetscErrorCode ierr = MatAssemblyBegin (matrix,MAT_FINAL_ASSEMBLY);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     ierr = MatAssemblyEnd (matrix,MAT_FINAL_ASSEMBLY);
@@ -272,7 +247,7 @@ namespace PETScWrappers
   {
     PetscInt n_rows, n_cols;
 
-    int ierr = MatGetSize (matrix, &n_rows, &n_cols);
+    const PetscErrorCode ierr = MatGetSize (matrix, &n_rows, &n_cols);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return n_rows;
@@ -285,7 +260,7 @@ namespace PETScWrappers
   {
     PetscInt n_rows, n_cols;
 
-    int ierr = MatGetSize (matrix, &n_rows, &n_cols);
+    const PetscErrorCode ierr = MatGetSize (matrix, &n_rows, &n_cols);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return n_cols;
@@ -298,7 +273,7 @@ namespace PETScWrappers
   {
     PetscInt n_rows, n_cols;
 
-    int ierr = MatGetLocalSize (matrix, &n_rows, &n_cols);
+    const PetscErrorCode ierr = MatGetLocalSize (matrix, &n_rows, &n_cols);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return n_rows;
@@ -311,8 +286,8 @@ namespace PETScWrappers
   {
     PetscInt begin, end;
 
-    const int ierr = MatGetOwnershipRange (static_cast<const Mat &>(matrix),
-                                           &begin, &end);
+    const PetscErrorCode ierr = MatGetOwnershipRange (static_cast<const Mat &>(matrix),
+                                                      &begin, &end);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return std::make_pair (begin, end);
@@ -324,8 +299,7 @@ namespace PETScWrappers
   MatrixBase::n_nonzero_elements () const
   {
     MatInfo mat_info;
-    const int ierr
-      = MatGetInfo (matrix, MAT_GLOBAL_SUM, &mat_info);
+    const PetscErrorCode ierr = MatGetInfo (matrix, MAT_GLOBAL_SUM, &mat_info);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return static_cast<size_type>(mat_info.nz_used);
@@ -353,9 +327,8 @@ namespace PETScWrappers
 
 //TODO: this is probably horribly inefficient; we should lobby for a way to
 //query this information from PETSc
-    int ierr;
-    ierr = MatGetRow(*this, row, &ncols, &colnums, &values);
-    AssertThrow (ierr == 0, MatrixBase::ExcPETScError(ierr));
+    PetscErrorCode ierr = MatGetRow(*this, row, &ncols, &colnums, &values);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // then restore the matrix and return the number of columns in this row as
     // queried previously. Starting with PETSc 3.4, MatRestoreRow actually
@@ -364,7 +337,7 @@ namespace PETScWrappers
     // and return the saved value.
     const PetscInt ncols_saved = ncols;
     ierr = MatRestoreRow(*this, row, &ncols, &colnums, &values);
-    AssertThrow (ierr == 0, MatrixBase::ExcPETScError(ierr));
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return ncols_saved;
   }
@@ -375,8 +348,7 @@ namespace PETScWrappers
   {
     PetscReal result;
 
-    const int ierr
-      = MatNorm (matrix, NORM_1, &result);
+    const PetscErrorCode ierr = MatNorm (matrix, NORM_1, &result);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return result;
@@ -389,8 +361,7 @@ namespace PETScWrappers
   {
     PetscReal result;
 
-    const int ierr
-      = MatNorm (matrix, NORM_INFINITY, &result);
+    const PetscErrorCode ierr = MatNorm (matrix, NORM_INFINITY, &result);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return result;
@@ -403,8 +374,7 @@ namespace PETScWrappers
   {
     PetscReal result;
 
-    const int ierr
-      = MatNorm (matrix, NORM_FROBENIUS, &result);
+    const PetscErrorCode ierr = MatNorm (matrix, NORM_FROBENIUS, &result);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return result;
@@ -436,8 +406,7 @@ namespace PETScWrappers
   {
     PetscScalar result;
 
-    const int ierr
-      = MatGetTrace (matrix, &result);
+    const PetscErrorCode ierr = MatGetTrace (matrix, &result);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return result;
@@ -449,7 +418,7 @@ namespace PETScWrappers
   MatrixBase &
   MatrixBase::operator *= (const PetscScalar a)
   {
-    const int ierr = MatScale (matrix, a);
+    const PetscErrorCode ierr = MatScale (matrix, a);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return *this;
@@ -461,25 +430,31 @@ namespace PETScWrappers
   MatrixBase::operator /= (const PetscScalar a)
   {
     const PetscScalar factor = 1./a;
-    const int ierr = MatScale (matrix, factor);
-
+    const PetscErrorCode ierr = MatScale (matrix, factor);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return *this;
   }
 
 
+  MatrixBase &
+  MatrixBase::add (const PetscScalar factor,
+                   const MatrixBase &other)
+  {
+    const PetscErrorCode ierr = MatAXPY (matrix, factor,
+                                         other, DIFFERENT_NONZERO_PATTERN);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
+
+    return *this;
+  }
+
+
+
   MatrixBase &
   MatrixBase::add (const MatrixBase &other,
                    const PetscScalar factor)
   {
-    const int ierr = MatAXPY (matrix, factor,
-                              other, DIFFERENT_NONZERO_PATTERN);
-    (void)ierr;
-
-    Assert (ierr == 0, ExcPETScError(ierr));
-
-    return *this;
+    return add(factor, other);
   }
 
 
@@ -489,8 +464,7 @@ namespace PETScWrappers
   {
     Assert (&src != &dst, ExcSourceEqualsDestination());
 
-    const int ierr = MatMult (matrix, src, dst);
-    (void)ierr;
+    const PetscErrorCode ierr = MatMult (matrix, src, dst);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
@@ -502,8 +476,7 @@ namespace PETScWrappers
   {
     Assert (&src != &dst, ExcSourceEqualsDestination());
 
-    const int ierr = MatMultTranspose (matrix, src, dst);
-    (void)ierr;
+    const PetscErrorCode ierr = MatMultTranspose (matrix, src, dst);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
@@ -515,8 +488,7 @@ namespace PETScWrappers
   {
     Assert (&src != &dst, ExcSourceEqualsDestination());
 
-    const int ierr = MatMultAdd (matrix, src, dst, dst);
-    (void)ierr;
+    const PetscErrorCode ierr = MatMultAdd (matrix, src, dst, dst);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
@@ -528,8 +500,7 @@ namespace PETScWrappers
   {
     Assert (&src != &dst, ExcSourceEqualsDestination());
 
-    const int ierr = MatMultTransposeAdd (matrix, src, dst, dst);
-    (void)ierr;
+    const PetscErrorCode ierr = MatMultTransposeAdd (matrix, src, dst, dst);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
@@ -559,45 +530,28 @@ namespace PETScWrappers
   void
   MatrixBase::transpose ()
   {
-    int ierr = MatTranspose(matrix, MAT_REUSE_MATRIX, &matrix);
-    (void)ierr;
+    const PetscErrorCode ierr = MatTranspose(matrix, MAT_REUSE_MATRIX, &matrix);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-  PetscTruth
-#else
-  PetscBool
-#endif
+  PetscBooleanType
   MatrixBase::is_symmetric (const double tolerance)
   {
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-    PetscTruth
-#else
-    PetscBool
-#endif
-    truth;
+    PetscBooleanType truth;
     assert_is_compressed ();
-    MatIsSymmetric (matrix, tolerance, &truth);
+    const PetscErrorCode ierr = MatIsSymmetric (matrix, tolerance, &truth);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
     return truth;
   }
 
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-  PetscTruth
-#else
-  PetscBool
-#endif
+  PetscBooleanType
   MatrixBase::is_hermitian (const double tolerance)
   {
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-    PetscTruth
-#else
-    PetscBool
-#endif
-    truth;
+    PetscBooleanType truth;
 
     assert_is_compressed ();
-    MatIsHermitian (matrix, tolerance, &truth);
+    const PetscErrorCode ierr = MatIsHermitian (matrix, tolerance, &truth);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return truth;
   }
@@ -608,11 +562,13 @@ namespace PETScWrappers
     assert_is_compressed ();
 
     // Set options
-    PetscViewerSetFormat (PETSC_VIEWER_STDOUT_WORLD,
-                          format);
+    PetscErrorCode ierr = PetscViewerSetFormat (PETSC_VIEWER_STDOUT_WORLD,
+                                                format);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // Write to screen
-    MatView (matrix, PETSC_VIEWER_STDOUT_WORLD);
+    ierr = MatView (matrix, PETSC_VIEWER_STDOUT_WORLD);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
   void
@@ -629,9 +585,8 @@ namespace PETScWrappers
     MatrixBase::size_type row;
     for (row = loc_range.first; row < loc_range.second; ++row)
       {
-        int ierr = MatGetRow(*this, row, &ncols, &colnums, &values);
-        (void)ierr;
-        AssertThrow (ierr == 0, MatrixBase::ExcPETScError(ierr));
+        PetscErrorCode ierr = MatGetRow(*this, row, &ncols, &colnums, &values);
+        AssertThrow (ierr == 0, ExcPETScError(ierr));
 
         for (PetscInt col = 0; col < ncols; ++col)
           {
@@ -639,7 +594,7 @@ namespace PETScWrappers
           }
 
         ierr = MatRestoreRow(*this, row, &ncols, &colnums, &values);
-        AssertThrow (ierr == 0, MatrixBase::ExcPETScError(ierr));
+        AssertThrow (ierr == 0, ExcPETScError(ierr));
       }
 
     AssertThrow (out, ExcIO());
@@ -651,7 +606,8 @@ namespace PETScWrappers
   MatrixBase::memory_consumption() const
   {
     MatInfo info;
-    MatGetInfo(matrix, MAT_LOCAL, &info);
+    const PetscErrorCode ierr = MatGetInfo(matrix, MAT_LOCAL, &info);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return sizeof(*this) + static_cast<size_type>(info.memory);
   }
