@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2012 - 2014 by the deal.II authors
+// Copyright (C) 2012 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -18,6 +18,9 @@
 
 #ifdef DEAL_II_WITH_PETSC
 
+#include <deal.II/lac/exceptions.h>
+#include <deal.II/lac/petsc_compatibility.h>
+
 DEAL_II_NAMESPACE_OPEN
 
 namespace PETScWrappers
@@ -103,14 +106,9 @@ namespace PETScWrappers
   {
     this->communicator = communicator;
 
-    // destroy the matrix and
-    // generate a new one
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-    int ierr = MatDestroy (matrix);
-#else
-    int ierr = MatDestroy (&matrix);
-#endif
-    AssertThrow (ierr == 0, ExcPETScError(ierr));
+    // destroy the matrix and generate a new one
+    const PetscErrorCode ierr = destroy_matrix (matrix);
+    AssertThrow (ierr == 0, ExcPETScError (ierr));
 
     do_reinit (m, n, local_rows, local_columns);
   }
@@ -131,13 +129,8 @@ namespace PETScWrappers
             ExcInternalError());
 
     this->communicator = communicator;
-
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-    int ierr = MatDestroy (matrix);
-#else
-    int ierr = MatDestroy (&matrix);
-#endif
-    AssertThrow (ierr == 0, ExcPETScError(ierr));
+    const PetscErrorCode ierr = destroy_matrix (matrix);
+    AssertThrow (ierr != 0, ExcPETScError (ierr));
 
     do_reinit (m, n,
                local_rows_per_process[this_process],
@@ -169,12 +162,8 @@ namespace PETScWrappers
 
   void MatrixFree::clear ()
   {
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-    int ierr = MatDestroy (matrix);
-#else
-    int ierr = MatDestroy (&matrix);
-#endif
-    AssertThrow (ierr == 0, ExcPETScError(ierr));
+    const PetscErrorCode ierr = destroy_matrix (matrix);
+    AssertThrow (ierr == 0, ExcPETScError (ierr));
 
     const int m=0;
     do_reinit (m, m, m, m);
@@ -202,7 +191,8 @@ namespace PETScWrappers
     // and reinit x and y with
     // dealii::PETScWrappers::*::Vector:
     const char  *vec_type;
-    int ierr = VecGetType (src, &vec_type);
+    PetscErrorCode ierr = VecGetType (src, &vec_type);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     PetscInt  local_size;
     ierr = VecGetLocalSize (src, &local_size);
@@ -249,7 +239,7 @@ namespace PETScWrappers
     // to the matrix-vector multiplication
     // of this MatrixFree object,
     void  *this_object;
-    int ierr = MatShellGetContext (A, &this_object);
+    const PetscErrorCode ierr = MatShellGetContext (A, &this_object);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // call vmult of this object:
@@ -268,11 +258,11 @@ namespace PETScWrappers
     Assert (local_rows <= m, ExcDimensionMismatch (local_rows, m));
     Assert (local_columns <= n, ExcDimensionMismatch (local_columns, n));
 
-    int ierr;
     // create a PETSc MatShell matrix-type
     // object of dimension m x n and local size
     // local_rows x local_columns
-    ierr = MatCreateShell(communicator, local_rows, local_columns, m, n, (void *)this, &matrix);
+    PetscErrorCode ierr = MatCreateShell(communicator, local_rows, local_columns,
+                                         m, n, (void *)this, &matrix);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
     // register the MatrixFree::matrix_free_mult function
     // as the matrix multiplication used by this matrix
