@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2015 by the deal.II authors
+// Copyright (C) 2004 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -18,6 +18,7 @@
 #ifdef DEAL_II_WITH_PETSC
 
 #  include <deal.II/base/utilities.h>
+#  include <deal.II/lac/exceptions.h>
 #  include <deal.II/lac/petsc_compatibility.h>
 #  include <deal.II/lac/petsc_matrix_base.h>
 #  include <deal.II/lac/petsc_vector_base.h>
@@ -34,16 +35,29 @@ namespace PETScWrappers
     pc(NULL), matrix(NULL)
   {}
 
-
   PreconditionerBase::~PreconditionerBase ()
   {
+    try
+      {
+        clear();
+      }
+    catch (...)
+      {}
+  }
+
+  void
+  PreconditionerBase::clear ()
+  {
+    matrix = NULL;
+
     if (pc!=NULL)
       {
 #if DEAL_II_PETSC_VERSION_LT(3,2,0)
-        int ierr = PCDestroy(pc);
+        PetscErrorCode ierr = PCDestroy(pc);
 #else
-        int ierr = PCDestroy(&pc);
+        PetscErrorCode ierr = PCDestroy(&pc);
 #endif
+        pc = NULL;
         AssertThrow (ierr == 0, ExcPETScError(ierr));
       }
   }
@@ -55,8 +69,7 @@ namespace PETScWrappers
   {
     AssertThrow (pc != NULL, StandardExceptions::ExcInvalidState ());
 
-    int ierr;
-    ierr = PCApply(pc, src, dst);
+    const PetscErrorCode ierr = PCApply(pc, src, dst);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
@@ -69,11 +82,11 @@ namespace PETScWrappers
     AssertThrow (pc == NULL, StandardExceptions::ExcInvalidState ());
 
     MPI_Comm comm;
-    int ierr;
     // this ugly cast is necessary because the
     // type Mat and PETScObject are
     // unrelated.
-    ierr = PetscObjectGetComm(reinterpret_cast<PetscObject>(matrix), &comm);
+    PetscErrorCode ierr = PetscObjectGetComm(reinterpret_cast<PetscObject>(matrix),
+                                             &comm);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     ierr = PCCreate(comm, &pc);
@@ -107,7 +120,7 @@ namespace PETScWrappers
   {
     additional_data = additional_data_;
 
-    int ierr = PCCreate(comm, &pc);
+    PetscErrorCode ierr = PCCreate(comm, &pc);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     initialize();
@@ -127,8 +140,9 @@ namespace PETScWrappers
   void
   PreconditionJacobi::initialize()
   {
-    int ierr;
-    ierr = PCSetType (pc, const_cast<char *>(PCJACOBI));
+    AssertThrow (pc != NULL, StandardExceptions::ExcInvalidState ());
+
+    PetscErrorCode ierr = PCSetType (pc, const_cast<char *>(PCJACOBI));
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     ierr = PCSetFromOptions (pc);
@@ -139,13 +153,15 @@ namespace PETScWrappers
   PreconditionJacobi::initialize (const MatrixBase     &matrix_,
                                   const AdditionalData &additional_data_)
   {
+    clear ();
+
     matrix = static_cast<Mat>(matrix_);
     additional_data = additional_data_;
 
     create_pc();
     initialize();
 
-    int ierr = PCSetUp (pc);
+    PetscErrorCode ierr = PCSetUp (pc);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
@@ -156,7 +172,7 @@ namespace PETScWrappers
   {
     additional_data = additional_data_;
 
-    int ierr = PCCreate(comm, &pc);
+    PetscErrorCode ierr = PCCreate(comm, &pc);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     initialize();
@@ -177,8 +193,7 @@ namespace PETScWrappers
   void
   PreconditionBlockJacobi::initialize()
   {
-    int ierr;
-    ierr = PCSetType (pc, const_cast<char *>(PCBJACOBI));
+    PetscErrorCode ierr = PCSetType (pc, const_cast<char *>(PCBJACOBI));
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     ierr = PCSetFromOptions (pc);
@@ -190,13 +205,15 @@ namespace PETScWrappers
   PreconditionBlockJacobi::initialize (const MatrixBase     &matrix_,
                                        const AdditionalData &additional_data_)
   {
+    clear ();
+
     matrix = static_cast<Mat>(matrix_);
     additional_data = additional_data_;
 
     create_pc();
     initialize();
 
-    int ierr = PCSetUp (pc);
+    PetscErrorCode ierr = PCSetUp (pc);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
@@ -225,13 +242,14 @@ namespace PETScWrappers
   PreconditionSOR::initialize (const MatrixBase     &matrix_,
                                const AdditionalData &additional_data_)
   {
+    clear ();
+
     matrix = static_cast<Mat>(matrix_);
     additional_data = additional_data_;
 
     create_pc();
 
-    int ierr;
-    ierr = PCSetType (pc, const_cast<char *>(PCSOR));
+    PetscErrorCode ierr = PCSetType (pc, const_cast<char *>(PCSOR));
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // then set flags as given
@@ -270,13 +288,14 @@ namespace PETScWrappers
   PreconditionSSOR::initialize (const MatrixBase     &matrix_,
                                 const AdditionalData &additional_data_)
   {
+    clear ();
+
     matrix = static_cast<Mat>(matrix_);
     additional_data = additional_data_;
 
     create_pc();
 
-    int ierr;
-    ierr = PCSetType (pc, const_cast<char *>(PCSOR));
+    PetscErrorCode ierr = PCSetType (pc, const_cast<char *>(PCSOR));
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // then set flags as given
@@ -319,13 +338,14 @@ namespace PETScWrappers
   PreconditionEisenstat::initialize (const MatrixBase     &matrix_,
                                      const AdditionalData &additional_data_)
   {
+    clear ();
+
     matrix = static_cast<Mat>(matrix_);
     additional_data = additional_data_;
 
     create_pc();
 
-    int ierr;
-    ierr = PCSetType (pc, const_cast<char *>(PCEISENSTAT));
+    PetscErrorCode ierr = PCSetType (pc, const_cast<char *>(PCEISENSTAT));
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // then set flags as given
@@ -365,17 +385,18 @@ namespace PETScWrappers
   PreconditionICC::initialize (const MatrixBase     &matrix_,
                                const AdditionalData &additional_data_)
   {
+    clear ();
+
     matrix = static_cast<Mat>(matrix_);
     additional_data = additional_data_;
 
     create_pc();
 
-    int ierr;
-    ierr = PCSetType (pc, const_cast<char *>(PCICC));
+    PetscErrorCode ierr = PCSetType (pc, const_cast<char *>(PCICC));
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // then set flags
-    PCFactorSetLevels (pc, additional_data.levels);
+    ierr = PCFactorSetLevels (pc, additional_data.levels);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     ierr = PCSetFromOptions (pc);
@@ -410,17 +431,18 @@ namespace PETScWrappers
   PreconditionILU::initialize (const MatrixBase     &matrix_,
                                const AdditionalData &additional_data_)
   {
+    clear ();
+
     matrix = static_cast<Mat>(matrix_);
     additional_data = additional_data_;
 
     create_pc();
 
-    int ierr;
-    ierr = PCSetType (pc, const_cast<char *>(PCILU));
+    PetscErrorCode ierr = PCSetType (pc, const_cast<char *>(PCILU));
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // then set flags
-    PCFactorSetLevels (pc, additional_data.levels);
+    ierr = PCFactorSetLevels (pc, additional_data.levels);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     ierr = PCSetFromOptions (pc);
@@ -457,7 +479,7 @@ namespace PETScWrappers
   {
     additional_data = additional_data_;
 
-    int ierr = PCCreate(comm, &pc);
+    PetscErrorCode ierr = PCCreate(comm, &pc);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
 #ifdef PETSC_HAVE_HYPRE
@@ -480,8 +502,8 @@ namespace PETScWrappers
   void
   PreconditionBoomerAMG::initialize ()
   {
-    int ierr;
-    ierr = PCSetType (pc, const_cast<char *>(PCHYPRE));
+#ifdef PETSC_HAVE_HYPRE
+    PetscErrorCode ierr = PCSetType (pc, const_cast<char *>(PCHYPRE));
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     ierr = PCHYPRESetType(pc, "boomeramg");
@@ -518,24 +540,32 @@ namespace PETScWrappers
 
     ierr = PCSetFromOptions (pc);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
+#else
+    Assert (false,
+            ExcMessage ("Your PETSc installation does not include a copy of "
+                        "the hypre package necessary for this preconditioner."));
+#endif
   }
 
   void
   PreconditionBoomerAMG::initialize (const MatrixBase     &matrix_,
                                      const AdditionalData &additional_data_)
   {
+#ifdef PETSC_HAVE_HYPRE
+    clear ();
+
     matrix = static_cast<Mat>(matrix_);
     additional_data = additional_data_;
 
-#ifdef PETSC_HAVE_HYPRE
     create_pc();
     initialize ();
 
-    int ierr = PCSetUp (pc);
+    PetscErrorCode ierr = PCSetUp (pc);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
 #else // PETSC_HAVE_HYPRE
-    (void)pc;
+    (void)matrix_;
+    (void)additional_data_;
     Assert (false,
             ExcMessage ("Your PETSc installation does not include a copy of "
                         "the hypre package necessary for this preconditioner."));
@@ -575,14 +605,15 @@ namespace PETScWrappers
   PreconditionParaSails::initialize (const MatrixBase     &matrix_,
                                      const AdditionalData &additional_data_)
   {
+    clear ();
+
     matrix = static_cast<Mat>(matrix_);
     additional_data = additional_data_;
 
 #ifdef PETSC_HAVE_HYPRE
     create_pc();
 
-    int ierr;
-    ierr = PCSetType (pc, const_cast<char *>(PCHYPRE));
+    PetscErrorCode ierr = PCSetType (pc, const_cast<char *>(PCHYPRE));
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     ierr = PCHYPRESetType(pc, "parasails");
@@ -670,13 +701,14 @@ namespace PETScWrappers
   PreconditionNone::initialize (const MatrixBase     &matrix_,
                                 const AdditionalData &additional_data_)
   {
+    clear ();
+
     matrix = static_cast<Mat>(matrix_);
     additional_data = additional_data_;
 
     create_pc();
 
-    int ierr;
-    ierr = PCSetType (pc, const_cast<char *>(PCNONE));
+    PetscErrorCode ierr = PCSetType (pc, const_cast<char *>(PCNONE));
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     ierr = PCSetFromOptions (pc);
@@ -715,13 +747,14 @@ namespace PETScWrappers
   PreconditionLU::initialize (const MatrixBase     &matrix_,
                               const AdditionalData &additional_data_)
   {
+    clear ();
+
     matrix = static_cast<Mat>(matrix_);
     additional_data = additional_data_;
 
     create_pc();
 
-    int ierr;
-    ierr = PCSetType (pc, const_cast<char *>(PCLU));
+    PetscErrorCode ierr = PCSetType (pc, const_cast<char *>(PCLU));
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // set flags as given
