@@ -19,6 +19,8 @@
 
 #ifdef DEAL_II_WITH_PETSC
 
+#  include <deal.II/lac/exceptions.h>
+#  include <deal.II/lac/petsc_compatibility.h>
 #  include <deal.II/lac/petsc_matrix_base.h>
 #  include <deal.II/lac/petsc_vector_base.h>
 #  include <deal.II/lac/petsc_precondition.h>
@@ -33,17 +35,7 @@ namespace PETScWrappers
 
   SolverBase::SolverData::~SolverData ()
   {
-    if (ksp != NULL)
-      {
-        // destroy the solver object
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-        int ierr = KSPDestroy (ksp);
-#else
-        int ierr = KSPDestroy (&ksp);
-#endif
-
-        AssertThrow (ierr == 0, ExcPETScError(ierr));
-      }
+    destroy_krylov_solver (ksp);
   }
 
 
@@ -68,8 +60,6 @@ namespace PETScWrappers
                      const VectorBase         &b,
                      const PreconditionerBase &preconditioner)
   {
-    int ierr;
-
     /*
       TODO: PETSc duplicates communicators, so this does not work (you put MPI_COMM_SELF in, but get something other out when you ask PETSc for the communicator. This mainly fails due to the MatrixFree classes, that can not ask PETSc for a communicator. //Timo Heister
     Assert(A.get_mpi_communicator()==mpi_communicator, ExcMessage("PETSc Solver and Matrix need to use the same MPI_Comm."));
@@ -83,7 +73,7 @@ namespace PETScWrappers
       {
         solver_data.reset (new SolverData());
 
-        ierr = KSPCreate (mpi_communicator, &solver_data->ksp);
+        PetscErrorCode ierr = KSPCreate (mpi_communicator, &solver_data->ksp);
         AssertThrow (ierr == 0, ExcPETScError(ierr));
 
         // let derived classes set the solver
@@ -118,13 +108,14 @@ namespace PETScWrappers
         // checks with the solver_control
         // object we have in this object for
         // convergence
-        KSPSetConvergenceTest (solver_data->ksp, &convergence_test,
-                               reinterpret_cast<void *>(&solver_control),
-                               PETSC_NULL);
+        ierr = KSPSetConvergenceTest (solver_data->ksp, &convergence_test,
+                                      reinterpret_cast<void *>(&solver_control),
+                                      PETSC_NULL);
+        AssertThrow (ierr == 0, ExcPETScError(ierr));
       }
 
     // set the command line option prefix name
-    ierr = KSPSetOptionsPrefix(solver_data->ksp, prefix_name.c_str());
+    PetscErrorCode ierr = KSPSetOptionsPrefix(solver_data->ksp, prefix_name.c_str());
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // set the command line options provided
@@ -214,7 +205,7 @@ namespace PETScWrappers
   void
   SolverBase::initialize(const PreconditionerBase &preconditioner)
   {
-    int ierr;
+    PetscErrorCode ierr;
 
     solver_data.reset (new SolverData());
 
@@ -235,9 +226,10 @@ namespace PETScWrappers
     // checks with the solver_control
     // object we have in this object for
     // convergence
-    KSPSetConvergenceTest (solver_data->ksp, &convergence_test,
-                           reinterpret_cast<void *>(&solver_control),
-                           PETSC_NULL);
+    ierr = KSPSetConvergenceTest (solver_data->ksp, &convergence_test,
+                                  reinterpret_cast<void *>(&solver_control),
+                                  PETSC_NULL);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // set the command line options provided
     // by the user to override the defaults
@@ -269,8 +261,7 @@ namespace PETScWrappers
   void
   SolverRichardson::set_solver_type (KSP &ksp) const
   {
-    int ierr;
-    ierr = KSPSetType (ksp, KSPRICHARDSON);
+    PetscErrorCode ierr = KSPSetType (ksp, KSPRICHARDSON);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // set the damping factor from the data
@@ -280,7 +271,8 @@ namespace PETScWrappers
     // in the deal.II solvers, we always
     // honor the initial guess in the
     // solution vector. do so here as well:
-    KSPSetInitialGuessNonzero (ksp, PETSC_TRUE);
+    ierr = KSPSetInitialGuessNonzero (ksp, PETSC_TRUE);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // Hand over the absolute
     // tolerance and the maximum
@@ -295,8 +287,9 @@ namespace PETScWrappers
     // the Richardson iteration,
     // where no residual is
     // available.
-    KSPSetTolerances(ksp, PETSC_DEFAULT, this->solver_control.tolerance(),
-                     PETSC_DEFAULT, this->solver_control.max_steps()+1);
+    ierr = KSPSetTolerances(ksp, PETSC_DEFAULT, this->solver_control.tolerance(),
+                            PETSC_DEFAULT, this->solver_control.max_steps()+1);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
 
@@ -318,19 +311,18 @@ namespace PETScWrappers
     // completely pointless change in
     // spelling Chebyshev between PETSc 3.2
     // and 3.3...
-    int ierr;
-
 #if DEAL_II_PETSC_VERSION_LT(3,3,0)
-    ierr = KSPSetType (ksp, KSPCHEBYCHEV);
+    PetscErrorCode ierr = KSPSetType (ksp, KSPCHEBYCHEV);
 #else
-    ierr = KSPSetType (ksp, KSPCHEBYSHEV);
+    PetscErrorCode ierr = KSPSetType (ksp, KSPCHEBYSHEV);
 #endif
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // in the deal.II solvers, we always
     // honor the initial guess in the
     // solution vector. do so here as well:
-    KSPSetInitialGuessNonzero (ksp, PETSC_TRUE);
+    ierr = KSPSetInitialGuessNonzero (ksp, PETSC_TRUE);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
 
@@ -348,14 +340,14 @@ namespace PETScWrappers
   void
   SolverCG::set_solver_type (KSP &ksp) const
   {
-    int ierr;
-    ierr = KSPSetType (ksp, KSPCG);
+    PetscErrorCode ierr = KSPSetType (ksp, KSPCG);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // in the deal.II solvers, we always
     // honor the initial guess in the
     // solution vector. do so here as well:
-    KSPSetInitialGuessNonzero (ksp, PETSC_TRUE);
+    ierr = KSPSetInitialGuessNonzero (ksp, PETSC_TRUE);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
 
@@ -373,14 +365,14 @@ namespace PETScWrappers
   void
   SolverBiCG::set_solver_type (KSP &ksp) const
   {
-    int ierr;
-    ierr = KSPSetType (ksp, KSPBICG);
+    PetscErrorCode ierr = KSPSetType (ksp, KSPBICG);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // in the deal.II solvers, we always
     // honor the initial guess in the
     // solution vector. do so here as well:
-    KSPSetInitialGuessNonzero (ksp, PETSC_TRUE);
+    ierr = KSPSetInitialGuessNonzero (ksp, PETSC_TRUE);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
 
@@ -408,8 +400,7 @@ namespace PETScWrappers
   void
   SolverGMRES::set_solver_type (KSP &ksp) const
   {
-    int ierr;
-    ierr = KSPSetType (ksp, KSPGMRES);
+    PetscErrorCode ierr = KSPSetType (ksp, KSPGMRES);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // set the restart parameter from the
@@ -457,7 +448,8 @@ namespace PETScWrappers
     // in the deal.II solvers, we always
     // honor the initial guess in the
     // solution vector. do so here as well:
-    KSPSetInitialGuessNonzero (ksp, PETSC_TRUE);
+    ierr = KSPSetInitialGuessNonzero (ksp, PETSC_TRUE);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
 
@@ -475,14 +467,14 @@ namespace PETScWrappers
   void
   SolverBicgstab::set_solver_type (KSP &ksp) const
   {
-    int ierr;
-    ierr = KSPSetType (ksp, KSPBCGS);
+    PetscErrorCode ierr = KSPSetType (ksp, KSPBCGS);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // in the deal.II solvers, we always
     // honor the initial guess in the
     // solution vector. do so here as well:
-    KSPSetInitialGuessNonzero (ksp, PETSC_TRUE);
+    ierr = KSPSetInitialGuessNonzero (ksp, PETSC_TRUE);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
 
@@ -500,14 +492,14 @@ namespace PETScWrappers
   void
   SolverCGS::set_solver_type (KSP &ksp) const
   {
-    int ierr;
-    ierr = KSPSetType (ksp, KSPCGS);
+    PetscErrorCode ierr = KSPSetType (ksp, KSPCGS);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // in the deal.II solvers, we always
     // honor the initial guess in the
     // solution vector. do so here as well:
-    KSPSetInitialGuessNonzero (ksp, PETSC_TRUE);
+    ierr = KSPSetInitialGuessNonzero (ksp, PETSC_TRUE);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
 
@@ -525,14 +517,14 @@ namespace PETScWrappers
   void
   SolverTFQMR::set_solver_type (KSP &ksp) const
   {
-    int ierr;
-    ierr = KSPSetType (ksp, KSPTFQMR);
+    PetscErrorCode ierr = KSPSetType (ksp, KSPTFQMR);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // in the deal.II solvers, we always
     // honor the initial guess in the
     // solution vector. do so here as well:
-    KSPSetInitialGuessNonzero (ksp, PETSC_TRUE);
+    ierr = KSPSetInitialGuessNonzero (ksp, PETSC_TRUE);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
 
@@ -550,14 +542,14 @@ namespace PETScWrappers
   void
   SolverTCQMR::set_solver_type (KSP &ksp) const
   {
-    int ierr;
-    ierr = KSPSetType (ksp, KSPTCQMR);
+    PetscErrorCode ierr = KSPSetType (ksp, KSPTCQMR);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // in the deal.II solvers, we always
     // honor the initial guess in the
     // solution vector. do so here as well:
-    KSPSetInitialGuessNonzero (ksp, PETSC_TRUE);
+    ierr = KSPSetInitialGuessNonzero (ksp, PETSC_TRUE);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
 
@@ -575,14 +567,14 @@ namespace PETScWrappers
   void
   SolverCR::set_solver_type (KSP &ksp) const
   {
-    int ierr;
-    ierr = KSPSetType (ksp, KSPCR);
+    PetscErrorCode ierr = KSPSetType (ksp, KSPCR);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // in the deal.II solvers, we always
     // honor the initial guess in the
     // solution vector. do so here as well:
-    KSPSetInitialGuessNonzero (ksp, PETSC_TRUE);
+    ierr = KSPSetInitialGuessNonzero (ksp, PETSC_TRUE);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
 
@@ -600,14 +592,14 @@ namespace PETScWrappers
   void
   SolverLSQR::set_solver_type (KSP &ksp) const
   {
-    int ierr;
-    ierr = KSPSetType (ksp, KSPLSQR);
+    PetscErrorCode ierr = KSPSetType (ksp, KSPLSQR);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // in the deal.II solvers, we always
     // honor the initial guess in the
     // solution vector. do so here as well:
-    KSPSetInitialGuessNonzero (ksp, PETSC_TRUE);
+    ierr = KSPSetInitialGuessNonzero (ksp, PETSC_TRUE);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
 
@@ -625,8 +617,7 @@ namespace PETScWrappers
   void
   SolverPreOnly::set_solver_type (KSP &ksp) const
   {
-    int ierr;
-    ierr = KSPSetType (ksp, KSPPREONLY);
+    PetscErrorCode ierr = KSPSetType (ksp, KSPPREONLY);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // The KSPPREONLY solver of
@@ -642,7 +633,8 @@ namespace PETScWrappers
     // Using the PREONLY solver with
     // a nonzero initial guess leads
     // PETSc to produce some error messages.
-    KSPSetInitialGuessNonzero (ksp, PETSC_FALSE);
+    ierr = KSPSetInitialGuessNonzero (ksp, PETSC_FALSE);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
 
@@ -650,14 +642,7 @@ namespace PETScWrappers
 
   SparseDirectMUMPS::SolverDataMUMPS::~SolverDataMUMPS ()
   {
-    // destroy the solver object
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-    int ierr = KSPDestroy (ksp);
-#else
-    int ierr = KSPDestroy (&ksp);
-#endif
-
-    AssertThrow (ierr == 0, ExcPETScError(ierr));
+    destroy_krylov_solver (ksp);
   }
 
 
@@ -679,8 +664,7 @@ namespace PETScWrappers
     * preconditioner.  Its use is due to SparseDirectMUMPS being a direct
     * (rather than iterative) solver
     */
-    int ierr;
-    ierr = KSPSetType (ksp, KSPPREONLY);
+    PetscErrorCode ierr = KSPSetType (ksp, KSPPREONLY);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     /**
@@ -695,7 +679,8 @@ namespace PETScWrappers
      * Using a PREONLY solver with a nonzero initial guess leads PETSc to
      * produce some error messages.
      */
-    KSPSetInitialGuessNonzero (ksp, PETSC_FALSE);
+    ierr = KSPSetInitialGuessNonzero (ksp, PETSC_FALSE);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
   void
@@ -704,8 +689,6 @@ namespace PETScWrappers
                             const VectorBase &b)
   {
 #ifdef PETSC_HAVE_MUMPS
-    int ierr;
-
     /**
      * factorization matrix to be obtained from MUMPS
      */
@@ -737,7 +720,7 @@ namespace PETScWrappers
          * creates the default KSP context and puts it in the location
          * solver_data->ksp
          */
-        ierr = KSPCreate (mpi_communicator, &solver_data->ksp);
+        PetscErrorCode ierr = KSPCreate (mpi_communicator, &solver_data->ksp);
         AssertThrow (ierr == 0, ExcPETScError(ierr));
 
         /**
@@ -777,9 +760,10 @@ namespace PETScWrappers
          * convergence monitor function that checks with the solver_control
          * object for convergence
          */
-        KSPSetConvergenceTest (solver_data->ksp, &convergence_test,
-                               reinterpret_cast<void *>(&solver_control),
-                               PETSC_NULL);
+        ierr = KSPSetConvergenceTest (solver_data->ksp, &convergence_test,
+                                      reinterpret_cast<void *>(&solver_control),
+                                      PETSC_NULL);
+        AssertThrow (ierr == 0, ExcPETScError(ierr));
 
         /**
          * set the software that is to be used to perform the lu
@@ -831,7 +815,7 @@ namespace PETScWrappers
     /**
      * solve the linear system
      */
-    ierr = KSPSolve (solver_data->ksp, b, x);
+    PetscErrorCode ierr = KSPSolve (solver_data->ksp, b, x);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     /**
