@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2015 by the deal.II authors
+// Copyright (C) 2004 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -18,6 +18,8 @@
 #ifdef DEAL_II_WITH_PETSC
 
 #  include <deal.II/base/memory_consumption.h>
+#  include <deal.II/lac/exceptions.h>
+#  include <deal.II/lac/petsc_compatibility.h>
 #  include <deal.II/lac/petsc_vector.h>
 #  include <deal.II/lac/petsc_parallel_vector.h>
 #  include <cmath>
@@ -41,7 +43,7 @@ namespace PETScWrappers
         {
           PetscInt idx = index;
           PetscScalar value;
-          int ierr = VecGetValues(vector.vector, 1, &idx, &value);
+          PetscErrorCode ierr = VecGetValues(vector.vector, 1, &idx, &value);
           AssertThrow (ierr == 0, ExcPETScError(ierr));
           return value;
         }
@@ -49,8 +51,6 @@ namespace PETScWrappers
       // with a parallel vector
       else if (dynamic_cast<const PETScWrappers::MPI::Vector *>(&vector) != 0)
         {
-          int ierr;
-
           // there is the possibility
           // that the vector has
           // ghost elements. in that
@@ -71,7 +71,8 @@ namespace PETScWrappers
           if (vector.ghosted)
             {
               PetscInt begin, end;
-              ierr = VecGetOwnershipRange (vector.vector, &begin, &end);
+              PetscErrorCode ierr = VecGetOwnershipRange (vector.vector, &begin,
+                                                          &end);
               AssertThrow (ierr == 0, ExcPETScError(ierr));
 
               Vec locally_stored_elements = PETSC_NULL;
@@ -119,7 +120,7 @@ namespace PETScWrappers
           // available
           PetscInt begin, end;
 
-          ierr = VecGetOwnershipRange (vector.vector, &begin, &end);
+          PetscErrorCode ierr = VecGetOwnershipRange (vector.vector, &begin, &end);
           AssertThrow (ierr == 0, ExcPETScError(ierr));
 
 
@@ -148,6 +149,7 @@ namespace PETScWrappers
 
   VectorBase::VectorBase ()
     :
+    vector (NULL),
     ghosted(false),
     last_action (::dealii::VectorOperation::unknown),
     attained_ownership(true)
@@ -171,7 +173,7 @@ namespace PETScWrappers
             ExcMessage("PETSc does not support multi-threaded access, set "
                        "the thread limit to 1 in MPI_InitFinalize()."));
 
-    int ierr = VecDuplicate (v.vector, &vector);
+    PetscErrorCode ierr = VecDuplicate (v.vector, &vector);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     ierr = VecCopy (v.vector, vector);
@@ -200,11 +202,12 @@ namespace PETScWrappers
     if (attained_ownership)
       {
 #if DEAL_II_PETSC_VERSION_LT(3,2,0)
-        const int ierr = VecDestroy (vector);
+        const PetscErrorCode ierr = VecDestroy (vector);
 #else
-        const int ierr = VecDestroy (&vector);
+        const PetscErrorCode ierr = VecDestroy (&vector);
 #endif
-        AssertThrow (ierr == 0, ExcPETScError(ierr));
+        AssertNothrow (ierr == 0, ExcPETScError(ierr));
+        (void) ierr;
       }
   }
 
@@ -216,9 +219,9 @@ namespace PETScWrappers
     if (attained_ownership)
       {
 #if DEAL_II_PETSC_VERSION_LT(3,2,0)
-        const int ierr = VecDestroy (vector);
+        const PetscErrorCode ierr = VecDestroy (vector);
 #else
-        const int ierr = VecDestroy (&vector);
+        const PetscErrorCode ierr = VecDestroy (&vector);
 #endif
         AssertThrow (ierr == 0, ExcPETScError(ierr));
       }
@@ -238,7 +241,7 @@ namespace PETScWrappers
 
     //TODO[TH]: assert(is_compressed())
 
-    int ierr = VecSet (vector, s);
+    PetscErrorCode ierr = VecSet (vector, s);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     if (has_ghost_elements())
@@ -265,14 +268,8 @@ namespace PETScWrappers
     Assert (size() == v.size(),
             ExcDimensionMismatch(size(), v.size()));
 
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-    PetscTruth
-#else
-    PetscBool
-#endif
-    flag;
-
-    const int ierr = VecEqual (vector, v.vector, &flag);
+    PetscBooleanType flag;
+    const PetscErrorCode ierr = VecEqual (vector, v.vector, &flag);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return (flag == PETSC_TRUE);
@@ -286,14 +283,8 @@ namespace PETScWrappers
     Assert (size() == v.size(),
             ExcDimensionMismatch(size(), v.size()));
 
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-    PetscTruth
-#else
-    PetscBool
-#endif
-    flag;
-
-    const int ierr = VecEqual (vector, v.vector, &flag);
+    PetscBooleanType flag;
+    const PetscErrorCode ierr = VecEqual (vector, v.vector, &flag);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return (flag == PETSC_FALSE);
@@ -305,7 +296,7 @@ namespace PETScWrappers
   VectorBase::size () const
   {
     PetscInt sz;
-    const int ierr = VecGetSize (vector, &sz);
+    const PetscErrorCode ierr = VecGetSize (vector, &sz);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return sz;
@@ -317,7 +308,7 @@ namespace PETScWrappers
   VectorBase::local_size () const
   {
     PetscInt sz;
-    const int ierr = VecGetLocalSize (vector, &sz);
+    const PetscErrorCode ierr = VecGetLocalSize (vector, &sz);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return sz;
@@ -329,8 +320,8 @@ namespace PETScWrappers
   VectorBase::local_range () const
   {
     PetscInt begin, end;
-    const int ierr = VecGetOwnershipRange (static_cast<const Vec &>(vector),
-                                           &begin, &end);
+    const PetscErrorCode ierr = VecGetOwnershipRange (static_cast<const Vec &>(vector),
+                                                      &begin, &end);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return std::make_pair (begin, end);
@@ -393,7 +384,7 @@ namespace PETScWrappers
     //    val = (x,y) = y^H x,
     //where y^H denotes the conjugate transpose of y.
     //Note that this corresponds to the usual "mathematicians" complex inner product where the SECOND argument gets the complex conjugate.
-    const int ierr = VecDot (vec.vector, vector, &result);
+    const PetscErrorCode ierr = VecDot (vec.vector, vector, &result);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return result;
@@ -415,20 +406,26 @@ namespace PETScWrappers
   void
   VectorBase::compress (const VectorOperation::values operation)
   {
+    {
 #ifdef DEBUG
 #ifdef DEAL_II_WITH_MPI
-    // Check that all processors agree that last_action is the same (or none!)
+      // Check that all processors agree that last_action is the same (or none!)
 
-    int my_int_last_action = last_action;
-    int all_int_last_action;
+      int my_int_last_action = last_action;
+      int all_int_last_action;
 
-    MPI_Allreduce(&my_int_last_action, &all_int_last_action, 1, MPI_INT,
-                  MPI_BOR, get_mpi_communicator());
+      const int ierr = MPI_Allreduce
+                       (&my_int_last_action, &all_int_last_action, 1, MPI_INT, MPI_BOR,
+                        get_mpi_communicator());
+      AssertThrowMPI(ierr);
 
-    AssertThrow(all_int_last_action != (::dealii::VectorOperation::add | ::dealii::VectorOperation::insert),
-                ExcMessage("Error: not all processors agree on the last VectorOperation before this compress() call."));
+      AssertThrow(all_int_last_action !=
+                  (::dealii::VectorOperation::add | ::dealii::VectorOperation::insert),
+                  ExcMessage("Error: not all processors agree on the last "
+                             "VectorOperation before this compress() call."));
 #endif
 #endif
+    }
 
     AssertThrow(last_action == ::dealii::VectorOperation::unknown
                 || last_action == operation,
@@ -448,8 +445,7 @@ namespace PETScWrappers
     // we still need to call
     // VecAssemblyBegin/End on all
     // processors.
-    int ierr;
-    ierr = VecAssemblyBegin(vector);
+    PetscErrorCode ierr = VecAssemblyBegin(vector);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
     ierr = VecAssemblyEnd(vector);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
@@ -474,14 +470,12 @@ namespace PETScWrappers
   PetscScalar
   VectorBase::mean_value () const
   {
-    int ierr;
-
     // We can only use our more efficient
     // routine in the serial case.
     if (dynamic_cast<const PETScWrappers::MPI::Vector *>(this) != 0)
       {
         PetscScalar sum;
-        ierr = VecSum(vector, &sum);
+        const PetscErrorCode ierr = VecSum(vector, &sum);
         AssertThrow (ierr == 0, ExcPETScError(ierr));
         return sum/static_cast<PetscReal>(size());
       }
@@ -489,7 +483,7 @@ namespace PETScWrappers
     // get a representation of the vector and
     // loop over all the elements
     PetscScalar *start_ptr;
-    ierr = VecGetArray (vector, &start_ptr);
+    PetscErrorCode ierr = VecGetArray (vector, &start_ptr);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     PetscScalar mean = 0;
@@ -532,7 +526,7 @@ namespace PETScWrappers
   {
     real_type d;
 
-    const int ierr = VecNorm (vector, NORM_1, &d);
+    const PetscErrorCode ierr = VecNorm (vector, NORM_1, &d);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return d;
@@ -545,7 +539,7 @@ namespace PETScWrappers
   {
     real_type d;
 
-    const int ierr = VecNorm (vector, NORM_2, &d);
+    const PetscErrorCode ierr = VecNorm (vector, NORM_2, &d);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return d;
@@ -559,7 +553,7 @@ namespace PETScWrappers
     // get a representation of the vector and
     // loop over all the elements
     PetscScalar *start_ptr;
-    int ierr = VecGetArray (vector, &start_ptr);
+    PetscErrorCode ierr = VecGetArray (vector, &start_ptr);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     real_type norm = 0;
@@ -604,7 +598,7 @@ namespace PETScWrappers
   {
     real_type d;
 
-    const int ierr = VecNorm (vector, NORM_INFINITY, &d);
+    const PetscErrorCode ierr = VecNorm (vector, NORM_INFINITY, &d);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return d;
@@ -617,7 +611,7 @@ namespace PETScWrappers
   {
     real_type d;
     Assert (!has_ghost_elements(), ExcGhostsPresent());
-    const int ierr = VecNormalize (vector, &d);
+    const PetscErrorCode ierr = VecNormalize (vector, &d);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return d;
@@ -630,7 +624,7 @@ namespace PETScWrappers
     PetscInt  p;
     real_type d;
 
-    const int ierr = VecMin (vector, &p, &d);
+    const PetscErrorCode ierr = VecMin (vector, &p, &d);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return d;
@@ -643,7 +637,7 @@ namespace PETScWrappers
     PetscInt  p;
     real_type d;
 
-    const int ierr = VecMax (vector, &p, &d);
+    const PetscErrorCode ierr = VecMax (vector, &p, &d);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return d;
@@ -655,7 +649,7 @@ namespace PETScWrappers
   {
     Assert (!has_ghost_elements(), ExcGhostsPresent());
 
-    const int ierr = VecAbs (vector);
+    const PetscErrorCode ierr = VecAbs (vector);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return *this;
@@ -668,7 +662,7 @@ namespace PETScWrappers
   {
     Assert (!has_ghost_elements(), ExcGhostsPresent());
 
-    const int ierr = VecConjugate (vector);
+    const PetscErrorCode ierr = VecConjugate (vector);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return *this;
@@ -681,7 +675,7 @@ namespace PETScWrappers
   {
     Assert (!has_ghost_elements(), ExcGhostsPresent());
 
-    const int ierr = VecPointwiseMult (vector,vector,vector);
+    const PetscErrorCode ierr = VecPointwiseMult (vector,vector,vector);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return *this;
@@ -692,7 +686,7 @@ namespace PETScWrappers
   VectorBase::mult (const VectorBase &v)
   {
     Assert (!has_ghost_elements(), ExcGhostsPresent());
-    const int ierr = VecPointwiseMult (vector,vector,v);
+    const PetscErrorCode ierr = VecPointwiseMult (vector,vector,v);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return *this;
@@ -704,7 +698,7 @@ namespace PETScWrappers
                     const VectorBase &v)
   {
     Assert (!has_ghost_elements(), ExcGhostsPresent());
-    const int ierr = VecPointwiseMult (vector,u,v);
+    const PetscErrorCode ierr = VecPointwiseMult (vector,u,v);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return *this;
@@ -716,7 +710,7 @@ namespace PETScWrappers
     // get a representation of the vector and
     // loop over all the elements
     PetscScalar *start_ptr;
-    int ierr = VecGetArray (vector, &start_ptr);
+    PetscErrorCode ierr = VecGetArray (vector, &start_ptr);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     const PetscScalar *ptr  = start_ptr,
@@ -769,7 +763,7 @@ namespace PETScWrappers
     // get a representation of the vector and
     // loop over all the elements
     PetscScalar *start_ptr;
-    int ierr = VecGetArray (vector, &start_ptr);
+    PetscErrorCode ierr = VecGetArray (vector, &start_ptr);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     const PetscScalar *ptr  = start_ptr,
@@ -801,7 +795,7 @@ namespace PETScWrappers
     Assert (!has_ghost_elements(), ExcGhostsPresent());
     AssertIsFinite(a);
 
-    const int ierr = VecScale (vector, a);
+    const PetscErrorCode ierr = VecScale (vector, a);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return *this;
@@ -818,7 +812,7 @@ namespace PETScWrappers
     const PetscScalar factor = 1./a;
     AssertIsFinite(factor);
 
-    const int ierr = VecScale (vector, factor);
+    const PetscErrorCode ierr = VecScale (vector, factor);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return *this;
@@ -830,7 +824,7 @@ namespace PETScWrappers
   VectorBase::operator += (const VectorBase &v)
   {
     Assert (!has_ghost_elements(), ExcGhostsPresent());
-    const int ierr = VecAXPY (vector, 1, v);
+    const PetscErrorCode ierr = VecAXPY (vector, 1, v);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return *this;
@@ -842,7 +836,7 @@ namespace PETScWrappers
   VectorBase::operator -= (const VectorBase &v)
   {
     Assert (!has_ghost_elements(), ExcGhostsPresent());
-    const int ierr = VecAXPY (vector, -1, v);
+    const PetscErrorCode ierr = VecAXPY (vector, -1, v);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     return *this;
@@ -856,7 +850,7 @@ namespace PETScWrappers
     Assert (!has_ghost_elements(), ExcGhostsPresent());
     AssertIsFinite(s);
 
-    const int ierr = VecShift (vector, s);
+    const PetscErrorCode ierr = VecShift (vector, s);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
@@ -877,7 +871,7 @@ namespace PETScWrappers
     Assert (!has_ghost_elements(), ExcGhostsPresent());
     AssertIsFinite(a);
 
-    const int ierr = VecAXPY (vector, a, v);
+    const PetscErrorCode ierr = VecAXPY (vector, a, v);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
@@ -896,7 +890,7 @@ namespace PETScWrappers
     const PetscScalar weights[2] = {a,b};
     Vec               addends[2] = {v.vector, w.vector};
 
-    const int ierr = VecMAXPY (vector, 2, weights, addends);
+    const PetscErrorCode ierr = VecMAXPY (vector, 2, weights, addends);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
@@ -909,7 +903,7 @@ namespace PETScWrappers
     Assert (!has_ghost_elements(), ExcGhostsPresent());
     AssertIsFinite(s);
 
-    const int ierr = VecAYPX (vector, s, v);
+    const PetscErrorCode ierr = VecAYPX (vector, s, v);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
@@ -952,7 +946,7 @@ namespace PETScWrappers
     const PetscScalar weights[2] = {a,b};
     Vec               addends[2] = {v.vector,w.vector};
 
-    const int ierr = VecMAXPY (vector, 2, weights, addends);
+    const PetscErrorCode ierr = VecMAXPY (vector, 2, weights, addends);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
@@ -980,7 +974,7 @@ namespace PETScWrappers
     const PetscScalar weights[3] = {a,b,c};
     Vec               addends[3] = {v.vector, w.vector, x.vector};
 
-    const int ierr = VecMAXPY (vector, 3, weights, addends);
+    const PetscErrorCode ierr = VecMAXPY (vector, 3, weights, addends);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
@@ -990,7 +984,7 @@ namespace PETScWrappers
   VectorBase::scale (const VectorBase &factors)
   {
     Assert (!has_ghost_elements(), ExcGhostsPresent());
-    const int ierr
+    const PetscErrorCode ierr
       = VecPointwiseMult (vector, factors, vector);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
@@ -1010,7 +1004,7 @@ namespace PETScWrappers
     // there is no simple operation for this
     // in PETSc. there are multiple ways to
     // emulate it, we choose this one:
-    const int ierr = VecCopy (v.vector, vector);
+    const PetscErrorCode ierr = VecCopy (v.vector, vector);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     *this *= a;
@@ -1034,7 +1028,7 @@ namespace PETScWrappers
     // there is no simple operation for this
     // in PETSc. there are multiple ways to
     // emulate it, we choose this one:
-    const int ierr = VecCopy (v.vector, vector);
+    const PetscErrorCode ierr = VecCopy (v.vector, vector);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     sadd (a, b, w);
@@ -1047,7 +1041,7 @@ namespace PETScWrappers
                      const VectorBase &b)
   {
     Assert (!has_ghost_elements(), ExcGhostsPresent());
-    const int ierr = VecPointwiseDivide (vector, a, b);
+    const PetscErrorCode ierr = VecPointwiseDivide (vector, a, b);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
@@ -1059,11 +1053,13 @@ namespace PETScWrappers
     //TODO[TH]:assert(is_compressed())
 
     // Set options
-    PetscViewerSetFormat (PETSC_VIEWER_STDOUT_WORLD,
-                          format);
+    PetscErrorCode ierr = PetscViewerSetFormat (PETSC_VIEWER_STDOUT_WORLD,
+                                                format);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // Write to screen
-    VecView (vector, PETSC_VIEWER_STDOUT_WORLD);
+    ierr = VecView (vector, PETSC_VIEWER_STDOUT_WORLD);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
 
@@ -1079,7 +1075,7 @@ namespace PETScWrappers
     // get a representation of the vector and
     // loop over all the elements
     PetscScalar *val;
-    int ierr = VecGetArray (vector, &val);
+    PetscErrorCode ierr = VecGetArray (vector, &val);
 
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
@@ -1118,7 +1114,7 @@ namespace PETScWrappers
   void
   VectorBase::swap (VectorBase &v)
   {
-    const int ierr = VecSwap (vector, v.vector);
+    const PetscErrorCode ierr = VecSwap (vector, v.vector);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
@@ -1185,9 +1181,8 @@ namespace PETScWrappers
 #endif
 
         const InsertMode mode = (add_values ? ADD_VALUES : INSERT_VALUES);
-        const int ierr
-          = VecSetValues (vector, n_elements, petsc_indices, values,
-                          mode);
+        const PetscErrorCode ierr = VecSetValues (vector, n_elements, petsc_indices,
+                                                  values, mode);
         AssertThrow (ierr == 0, ExcPETScError(ierr));
       }
 
