@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2001 - 2015 by the deal.II authors
+// Copyright (C) 2001 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -367,6 +367,8 @@ SparseDirectUMFPACK::solve (const Matrix        &matrix,
 
 SparseDirectUMFPACK::SparseDirectUMFPACK ()
   :
+  _m(0),
+  _n(0),
   symbolic_decomposition (0),
   numeric_decomposition (0),
   control (0)
