@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2015 by the deal.II authors
+// Copyright (C) 2008 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -33,7 +33,12 @@ namespace TrilinosWrappers
   {
     // delete previous content of
     // the subobjects array
-    clear ();
+    try
+      {
+        clear ();
+      }
+    catch (...)
+      {}
   }
 
 
