@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2015 by the deal.II authors
+// Copyright (C) 2008 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -70,6 +70,37 @@ namespace TrilinosWrappers
   }
 
 
+  MPI_Comm
+  PreconditionBase::get_mpi_communicator () const
+  {
+#ifdef DEAL_II_WITH_MPI
+    return communicator.Comm();
+#else
+    return MPI_COMM_SELF;
+#endif
+  }
+
+
+  Epetra_Operator &
+  PreconditionBase::trilinos_operator () const
+  {
+    AssertThrow (preconditioner, ExcMessage("Trying to dereference a null pointer."));
+    return (*preconditioner);
+  }
+
+
+  IndexSet
+  PreconditionBase::locally_owned_domain_indices() const
+  {
+    return IndexSet(preconditioner->OperatorDomainMap());
+  }
+
+
+  IndexSet
+  PreconditionBase::locally_owned_range_indices() const
+  {
+    return IndexSet(preconditioner->OperatorRangeMap());
+  }
 
   /* -------------------------- PreconditionJacobi -------------------------- */
 
@@ -231,11 +262,11 @@ namespace TrilinosWrappers
   /* ----------------------- PreconditionBlockJacobi ---------------------- */
 
   PreconditionBlockJacobi::AdditionalData::
-  AdditionalData (const unsigned int block_size,
-                  const std::string  block_creation_type,
-                  const double omega,
-                  const double min_diagonal,
-                  const unsigned int n_sweeps)
+  AdditionalData (const unsigned int  block_size,
+                  const std::string  &block_creation_type,
+                  const double        omega,
+                  const double        min_diagonal,
+                  const unsigned int  n_sweeps)
     :
     block_size(block_size),
     block_creation_type(block_creation_type),
@@ -290,12 +321,12 @@ namespace TrilinosWrappers
   /* ----------------------- PreconditionBlockSSOR ------------------------ */
 
   PreconditionBlockSSOR::AdditionalData::
-  AdditionalData (const unsigned int block_size,
-                  const std::string  block_creation_type,
-                  const double       omega,
-                  const double       min_diagonal,
-                  const unsigned int overlap,
-                  const unsigned int n_sweeps)
+  AdditionalData (const unsigned int  block_size,
+                  const std::string  &block_creation_type,
+                  const double        omega,
+                  const double        min_diagonal,
+                  const unsigned int  overlap,
+                  const unsigned int  n_sweeps)
     :
     block_size(block_size),
     block_creation_type(block_creation_type),
@@ -351,12 +382,12 @@ namespace TrilinosWrappers
   /* ------------------------ PreconditionBlockSOR ------------------------ */
 
   PreconditionBlockSOR::AdditionalData::
-  AdditionalData (const unsigned int block_size,
-                  const std::string  block_creation_type,
-                  const double       omega,
-                  const double       min_diagonal,
-                  const unsigned int overlap,
-                  const unsigned int n_sweeps)
+  AdditionalData (const unsigned int  block_size,
+                  const std::string  &block_creation_type,
+                  const double        omega,
+                  const double        min_diagonal,
+                  const unsigned int  overlap,
+                  const unsigned int  n_sweeps)
     :
     block_size(block_size),
     block_creation_type(block_creation_type),
@@ -674,6 +705,49 @@ namespace TrilinosWrappers
 
   /* -------------------------- PreconditionIdentity --------------------- */
 
+  void
+  PreconditionIdentity::initialize (const SparseMatrix   &matrix,
+                                    const AdditionalData &)
+  {
+    // What follows just configures a dummy preconditioner that
+    // sets up the domain and range maps, as well as the communicator.
+    // It is never used as the vmult, Tvmult operations are
+    // given a custom defintion.
+    // Note: This is only required in order to wrap this
+    // preconditioner in a LinearOperator without an exemplar
+    // matrix.
+
+    // From PreconditionJacobi:
+    // release memory before reallocation
+    preconditioner.reset ();
+    preconditioner.reset (Ifpack().Create
+                          ("point relaxation",
+                           const_cast<Epetra_CrsMatrix *>(&matrix.trilinos_matrix()),
+                           0));
+
+    Ifpack_Preconditioner *ifpack = static_cast<Ifpack_Preconditioner *>
+                                    (preconditioner.get());
+    Assert (ifpack != 0, ExcMessage ("Trilinos could not create this "
+                                     "preconditioner"));
+
+    int ierr;
+
+    Teuchos::ParameterList parameter_list;
+    parameter_list.set ("relaxation: sweeps", 1);
+    parameter_list.set ("relaxation: type", "Jacobi");
+    parameter_list.set ("relaxation: damping factor", 1.0);
+    parameter_list.set ("relaxation: min diagonal value", 0.0);
+
+    ierr = ifpack->SetParameters(parameter_list);
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+
+    ierr = ifpack->Initialize();
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+
+    ierr = ifpack->Compute();
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+  }
+
   void
   PreconditionIdentity::vmult(VectorBase       &dst,
                               const VectorBase &src) const
@@ -703,15 +777,15 @@ namespace TrilinosWrappers
   }
 
   void
-  PreconditionIdentity::vmult(parallel::distributed::Vector<double>       &dst,
-                              const parallel::distributed::Vector<double> &src) const
+  PreconditionIdentity::vmult(LinearAlgebra::distributed::Vector<double>       &dst,
+                              const LinearAlgebra::distributed::Vector<double> &src) const
   {
     dst = src;
   }
 
   void
-  PreconditionIdentity::Tvmult(parallel::distributed::Vector<double>       &dst,
-                               const parallel::distributed::Vector<double> &src) const
+  PreconditionIdentity::Tvmult(LinearAlgebra::distributed::Vector<double>       &dst,
+                               const LinearAlgebra::distributed::Vector<double> &src) const
   {
     dst = src;
   }
