@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2015 by the deal.II authors
+// Copyright (C) 2008 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -100,6 +100,16 @@ namespace TrilinosWrappers
   {}
 
 
+
+  PreconditionAMGMueLu::PreconditionAMGMueLu()
+  {
+#ifdef DEAL_II_WITH_64BIT_INDICES
+    AssertThrow (false, ExcMessage("PreconditionAMGMueLu does not support 64bit-indices!"));
+#endif
+  }
+
+
+
   PreconditionAMGMueLu::~PreconditionAMGMueLu()
   {
     preconditioner.reset();
@@ -303,7 +313,7 @@ namespace TrilinosWrappers
   PreconditionAMGMueLu::size_type
   PreconditionAMGMueLu::memory_consumption() const
   {
-    unsigned int memory = sizeof(this);
+    unsigned int memory = sizeof(*this);
 
     // todo: find a way to read out ML's data
     // sizes
