@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2015 by the deal.II authors
+// Copyright (C) 2008 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -18,6 +18,7 @@
 #ifdef DEAL_II_WITH_TRILINOS
 
 #  include <deal.II/base/utilities.h>
+#  include <deal.II/base/mpi.h>
 #  include <deal.II/lac/sparsity_pattern.h>
 #  include <deal.II/lac/dynamic_sparsity_pattern.h>
 
@@ -66,9 +67,9 @@ namespace TrilinosWrappers
       return graph.NumGlobalEntries();
     }
 
-    int global_row_index(const Epetra_CrsGraph &graph, int i)
+    int local_to_global_index(const Epetra_BlockMap &map, int i)
     {
-      return graph.GRID(i);
+      return map.GID(i);
     }
 #else
     long long int n_global_elements (const Epetra_BlockMap &map)
@@ -101,9 +102,9 @@ namespace TrilinosWrappers
       return graph.NumGlobalEntries64();
     }
 
-    long long int global_row_index(const Epetra_CrsGraph &graph, int i)
+    long long int local_to_global_index(const Epetra_BlockMap &map, int i)
     {
-      return graph.GRID64(i);
+      return map.GID64(i);
     }
 #endif
   }
@@ -113,43 +114,34 @@ namespace TrilinosWrappers
     void
     Accessor::visit_present_row ()
     {
-      // if we are asked to visit the
-      // past-the-end line, then simply
-      // release all our caches and go on
-      // with life
+      // if we are asked to visit the past-the-end line, then simply
+      // release all our caches and go on with life
       if (this->a_row == sparsity_pattern->n_rows())
         {
           colnum_cache.reset ();
 
           return;
         }
-//TODO: Is this thread safe?
 
       // otherwise first flush Trilinos caches
       sparsity_pattern->compress ();
 
-      // get a representation of the present
-      // row
-      int ncols;
-      // TODO: casting a size_type to an int, could be a problem
-      int colnums = sparsity_pattern->n_cols();
+      colnum_cache.reset (new std::vector<size_type> (sparsity_pattern->row_length(this->a_row)));
 
-      int ierr;
-      ierr = sparsity_pattern->graph->ExtractGlobalRowCopy((TrilinosWrappers::types::int_type)this->a_row,
-                                                           colnums,
-                                                           ncols,
-                                                           (TrilinosWrappers::types::int_type *)&(*colnum_cache)[0]);
-      AssertThrow (ierr == 0, ExcTrilinosError(ierr));
 
-      // copy it into our caches if the
-      // line isn't empty. if it is, then
-      // we've done something wrong, since
-      // we shouldn't have initialized an
-      // iterator for an empty line (what
-      // would it point to?)
-      Assert (ncols != 0, ExcInternalError());
-      colnum_cache.reset (new std::vector<size_type> (colnums,
-                                                      colnums+ncols));
+      if (colnum_cache->size() > 0)
+        {
+          // get a representation of the present row
+          int ncols;
+          const int ierr = sparsity_pattern->graph->ExtractGlobalRowCopy
+                           ((TrilinosWrappers::types::int_type)this->a_row,
+                            colnum_cache->size(),
+                            ncols,
+                            (TrilinosWrappers::types::int_type *)&(*colnum_cache)[0]);
+          AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+          AssertThrow (static_cast<std::vector<size_type>::size_type>(ncols) == colnum_cache->size(),
+                       ExcInternalError());
+        }
     }
   }
 
@@ -1059,6 +1051,7 @@ namespace TrilinosWrappers
 
     const Epetra_MpiComm *mpi_comm
       = dynamic_cast<const Epetra_MpiComm *>(&graph->RangeMap().Comm());
+    Assert (mpi_comm != 0, ExcInternalError());
     return mpi_comm->Comm();
 #else
 
@@ -1096,7 +1089,8 @@ namespace TrilinosWrappers
           {
             graph->ExtractMyRowView (i, num_entries, indices);
             for (int j=0; j<num_entries; ++j)
-              out << "(" << i << "," << indices[global_row_index(*graph,j)] << ") "
+              out << "(" << local_to_global_index(graph->RowMap(), i)
+                  << "," << local_to_global_index(graph->ColMap(), indices[j]) << ") "
                   << std::endl;
           }
       }
@@ -1116,14 +1110,14 @@ namespace TrilinosWrappers
         int num_entries;
         graph->ExtractMyRowView (row, num_entries, indices);
 
-        for (unsigned int j=0; j<(unsigned int)num_entries; ++j)
+        for (int j=0; j<num_entries; ++j)
           // while matrix entries are usually
           // written (i,j), with i vertical and
           // j horizontal, gnuplot output is
           // x-y, that is we have to exchange
           // the order of output
-          out << indices[global_row_index(*graph,static_cast<int>(j))]
-              << " " << -static_cast<signed int>(row) << std::endl;
+          out << static_cast<int>(local_to_global_index(graph->ColMap(), indices[j]))
+              << " " << -static_cast<int>(local_to_global_index(graph->RowMap(), row)) << std::endl;
       }
 
     AssertThrow (out, ExcIO());
