@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2015 by the deal.II authors
+// Copyright (C) 2008 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -17,6 +17,7 @@
 
 #ifdef DEAL_II_WITH_TRILINOS
 
+#  include <deal.II/base/mpi.h>
 #  include <deal.II/lac/trilinos_sparse_matrix.h>
 #  include <deal.II/lac/trilinos_block_vector.h>
 
@@ -121,6 +122,7 @@ namespace TrilinosWrappers
       last_action = Zero;
       vector.reset (new Epetra_FEVector(*v.vector));
       has_ghosts = v.has_ghosts;
+      owned_elements = v.owned_elements;
     }
 
 
@@ -131,6 +133,7 @@ namespace TrilinosWrappers
       // initialize a minimal, valid object and swap
       last_action = Zero;
       vector.reset(new Epetra_FEVector(Epetra_Map(0,0,0,Utilities::Trilinos::comm_self())));
+      owned_elements.clear();
 
       swap(v);
     }
@@ -185,9 +188,7 @@ namespace TrilinosWrappers
       :
       VectorBase()
     {
-      IndexSet parallel_partitioning = local;
-      parallel_partitioning.add_indices(ghost);
-      reinit(parallel_partitioning, communicator);
+      reinit(local, ghost, communicator, false);
     }
 
 
@@ -199,20 +200,53 @@ namespace TrilinosWrappers
 
     void
     Vector::reinit (const Epetra_Map &input_map,
-                    const bool        omit_zeroing_entries)
+                    const bool      /*omit_zeroing_entries*/)
     {
       nonlocal_vector.reset();
 
-      if (vector->Map().SameAs(input_map)==false)
-        vector.reset (new Epetra_FEVector(input_map));
-      else if (omit_zeroing_entries == false)
-        {
-          const int ierr = vector->PutScalar(0.);
-          (void)ierr;
-          Assert (ierr == 0, ExcTrilinosError(ierr));
-        }
+      vector.reset (new Epetra_FEVector(input_map));
 
       has_ghosts = vector->Map().UniqueGIDs()==false;
+
+      // If the IndexSets are overlapping, we don't really know
+      // which process owns what. So we decide that no process
+      // owns anything in that case. In particular asking for
+      // the locally owned elements is not allowed.
+      owned_elements.clear();
+      if (has_ghosts)
+        owned_elements.set_size(0);
+      else
+        {
+          owned_elements.set_size(size());
+
+          // easy case: local range is contiguous
+          if (vector->Map().LinearMap())
+            {
+              const std::pair<size_type, size_type> x = local_range();
+              owned_elements.add_range (x.first, x.second);
+            }
+          else if (vector->Map().NumMyElements() > 0)
+            {
+              const size_type n_indices = vector->Map().NumMyElements();
+#ifndef DEAL_II_WITH_64BIT_INDICES
+              unsigned int *vector_indices = (unsigned int *)vector->Map().MyGlobalElements();
+#else
+              size_type *vector_indices = (size_type *)vector->Map().MyGlobalElements64();
+#endif
+              owned_elements.add_indices(vector_indices, vector_indices+n_indices);
+              owned_elements.compress();
+            }
+        }
+#if defined(DEBUG) && defined(DEAL_II_WITH_MPI)
+      const Epetra_MpiComm *comm_ptr
+        = dynamic_cast<const Epetra_MpiComm *>(&(vector->Comm()));
+      Assert (comm_ptr != 0, ExcInternalError());
+      const size_type n_elements_global
+        = Utilities::MPI::sum (owned_elements.n_elements(), comm_ptr->Comm());
+
+      Assert (has_ghosts || n_elements_global == size(), ExcInternalError());
+#endif
+
       last_action = Zero;
     }
 
@@ -221,13 +255,37 @@ namespace TrilinosWrappers
     void
     Vector::reinit (const IndexSet &parallel_partitioner,
                     const MPI_Comm &communicator,
-                    const bool      omit_zeroing_entries)
+                    const bool      /*omit_zeroing_entries*/)
     {
       nonlocal_vector.reset();
 
       Epetra_Map map = parallel_partitioner.make_trilinos_map (communicator,
                                                                true);
-      reinit (map, omit_zeroing_entries);
+
+      vector.reset (new Epetra_FEVector(map));
+
+      has_ghosts = vector->Map().UniqueGIDs()==false;
+
+      // If the IndexSets are overlapping, we don't really know
+      // which process owns what. So we decide that no process
+      // owns anything in that case. In particular asking for
+      // the locally owned elements is not allowed.
+      if (has_ghosts)
+        {
+          owned_elements.clear();
+          owned_elements.set_size(0);
+        }
+      else
+        owned_elements = parallel_partitioner;
+
+#ifdef DEBUG
+      const size_type n_elements_global
+        = Utilities::MPI::sum (owned_elements.n_elements(), communicator);
+
+      Assert (has_ghosts || n_elements_global == size(), ExcInternalError());
+#endif
+
+      last_action = Zero;
     }
 
 
@@ -244,19 +302,29 @@ namespace TrilinosWrappers
       // with the map in v, and generate the vector.
       if (allow_different_maps == false)
         {
-          if (vector->Map().SameAs(v.vector->Map()) == false)
+          // check equality for MPI communicators: We can only choose the fast
+          // version in case the underlying Epetra_MpiComm object is the same,
+          // otherwise we might access an MPI_Comm object that has been
+          // deleted
+#ifdef DEAL_II_WITH_MPI
+          const Epetra_MpiComm *my_comm = dynamic_cast<const Epetra_MpiComm *>(&vector->Comm());
+          const Epetra_MpiComm *v_comm = dynamic_cast<const Epetra_MpiComm *>(&v.vector->Comm());
+          const bool same_communicators = my_comm != NULL && v_comm != NULL &&
+                                          my_comm->DataPtr() == v_comm->DataPtr();
+#else
+          const bool same_communicators = true;
+#endif
+          if (!same_communicators || vector->Map().SameAs(v.vector->Map()) == false)
             {
               vector.reset (new Epetra_FEVector(v.vector->Map()));
               has_ghosts = v.has_ghosts;
               last_action = Zero;
+              owned_elements = v.owned_elements;
             }
           else if (omit_zeroing_entries == false)
             {
-              // old and new vectors
-              // have exactly the
-              // same map, i.e. size
-              // and parallel
-              // distribution
+              // old and new vectors have exactly the same map, i.e. size and
+              // parallel distribution
               int ierr;
               ierr = vector->GlobalAssemble (last_action);
               (void)ierr;
@@ -290,7 +358,15 @@ namespace TrilinosWrappers
 
           last_action = Insert;
         }
+#if defined(DEBUG) && defined(DEAL_II_WITH_MPI)
+      const Epetra_MpiComm *comm_ptr
+        = dynamic_cast<const Epetra_MpiComm *>(&(v.vector->Comm()));
+      Assert (comm_ptr != 0, ExcInternalError());
+      const size_type n_elements_global
+        = Utilities::MPI::sum (owned_elements.n_elements(), comm_ptr->Comm());
 
+      Assert (has_ghosts || n_elements_global == size(), ExcInternalError());
+#endif
     }
 
 
@@ -300,6 +376,8 @@ namespace TrilinosWrappers
                     const bool         import_data)
     {
       nonlocal_vector.reset();
+      owned_elements.clear();
+      owned_elements.set_size(v.size());
 
       // In case we do not allow to have different maps, this call means that
       // we have to reset the vector. So clear the vector, initialize our map
@@ -319,6 +397,8 @@ namespace TrilinosWrappers
             my_global_elements(v.block(block).vector_partitioner());
           for (size_type i=0; i<v.block(block).local_size(); ++i)
             global_ids[added_elements++] = glob_elements[i] + block_offset;
+          owned_elements.add_indices(v.block(block).owned_elements,
+                                     block_offset);
           block_offset += v.block(block).size();
         }
 
@@ -357,7 +437,15 @@ namespace TrilinosWrappers
 
           last_action = Insert;
         }
+#if defined(DEBUG) && defined(DEAL_II_WITH_MPI)
+      const Epetra_MpiComm *comm_ptr
+        = dynamic_cast<const Epetra_MpiComm *>(&(vector->Comm()));
+      Assert (comm_ptr != 0, ExcInternalError());
+      const size_type n_elements_global
+        = Utilities::MPI::sum (owned_elements.n_elements(), comm_ptr->Comm());
 
+      Assert (has_ghosts || n_elements_global == size(), ExcInternalError());
+#endif
     }
 
 
@@ -367,11 +455,17 @@ namespace TrilinosWrappers
                         const bool      vector_writable)
     {
       nonlocal_vector.reset();
+      owned_elements = locally_owned_entries;
       if (vector_writable == false)
         {
-          IndexSet parallel_partitioning = locally_owned_entries;
-          parallel_partitioning.add_indices(ghost_entries);
-          reinit(parallel_partitioning, communicator);
+          IndexSet parallel_partitioner = locally_owned_entries;
+          parallel_partitioner.add_indices(ghost_entries);
+          Epetra_Map map = parallel_partitioner.make_trilinos_map (communicator,
+                                                                   true);
+          vector.reset (new Epetra_FEVector(map));
+          has_ghosts = vector->Map().UniqueGIDs()==false;
+
+          last_action = Zero;
         }
       else
         {
@@ -391,17 +485,54 @@ namespace TrilinosWrappers
               nonlocal_vector.reset(new Epetra_MultiVector(nonlocal_map, 1));
             }
         }
+#ifdef DEBUG
+      const size_type n_elements_global
+        = Utilities::MPI::sum (owned_elements.n_elements(), communicator);
+
+      Assert (has_ghosts || n_elements_global == size(), ExcInternalError());
+#endif
     }
 
 
+
     Vector &
     Vector::operator = (const Vector &v)
     {
+      // check equality for MPI communicators to avoid accessing a possibly
+      // invalid MPI_Comm object
+#ifdef DEAL_II_WITH_MPI
+      const Epetra_MpiComm *my_comm = dynamic_cast<const Epetra_MpiComm *>(&vector->Comm());
+      const Epetra_MpiComm *v_comm = dynamic_cast<const Epetra_MpiComm *>(&v.vector->Comm());
+      const bool same_communicators = my_comm != NULL && v_comm != NULL &&
+                                      my_comm->DataPtr() == v_comm->DataPtr();
+      // Need to ask MPI whether the communicators are the same. We would like
+      // to use the following checks but currently we cannot make sure the
+      // memory of my_comm is not stale from some MPI_Comm_free
+      // somewhere. This can happen when a vector lives in GrowingVectorMemory
+      // data structures. Thus, the following code is commented out.
+      //
+      //if (my_comm != NULL && v_comm != NULL && my_comm->DataPtr() != v_comm->DataPtr())
+      //  {
+      //    int communicators_same = 0;
+      //    const int ierr = MPI_Comm_compare (my_comm->GetMpiComm(),
+      //                                       v_comm->GetMpiComm(),
+      //                                       &communicators_same);
+      //    AssertThrowMPI(ierr);
+      //    if (!(communicators_same == MPI_IDENT ||
+      //          communicators_same == MPI_CONGRUENT))
+      //      same_communicators = false;
+      //    else
+      //      same_communicators = true;
+      //  }
+#else
+      const bool same_communicators = true;
+#endif
+
       // distinguish three cases. First case: both vectors have the same
       // layout (just need to copy the local data, not reset the memory and
       // the underlying Epetra_Map). The third case means that we have to
       // rebuild the calling vector.
-      if (vector->Map().SameAs(v.vector->Map()))
+      if (same_communicators && v.vector->Map().SameAs(vector->Map()))
         {
           *vector = *v.vector;
           if (v.nonlocal_vector.get() != 0)
@@ -424,6 +555,7 @@ namespace TrilinosWrappers
           vector.reset (new Epetra_FEVector(*v.vector));
           last_action = Zero;
           has_ghosts = v.has_ghosts;
+          owned_elements = v.owned_elements;
         }
 
       if (v.nonlocal_vector.get() != 0)
@@ -505,20 +637,14 @@ namespace TrilinosWrappers
 
   Vector::Vector (const size_type n)
   {
-    last_action = Zero;
-    Epetra_LocalMap map ((TrilinosWrappers::types::int_type)n, 0, Utilities::Trilinos::comm_self());
-    vector.reset (new Epetra_FEVector (map));
+    reinit(n);
   }
 
 
 
   Vector::Vector (const Epetra_Map &input_map)
   {
-    last_action = Zero;
-    Epetra_LocalMap map (n_global_elements(input_map),
-                         input_map.IndexBase(),
-                         input_map.Comm());
-    vector.reset (new Epetra_FEVector(map));
+    reinit(input_map);
   }
 
 
@@ -526,16 +652,7 @@ namespace TrilinosWrappers
   Vector::Vector (const IndexSet &partitioning,
                   const MPI_Comm &communicator)
   {
-    last_action = Zero;
-    Epetra_LocalMap map (static_cast<TrilinosWrappers::types::int_type>(partitioning.size()),
-                         0,
-#ifdef DEAL_II_WITH_MPI
-                         Epetra_MpiComm(communicator));
-#else
-                         Epetra_SerialComm());
-    (void)communicator;
-#endif
-    vector.reset (new Epetra_FEVector(map));
+    reinit (partitioning, communicator);
   }
 
 
@@ -555,31 +672,18 @@ namespace TrilinosWrappers
       }
     else
       reinit (v, false, true);
-
   }
 
 
 
   void
   Vector::reinit (const size_type n,
-                  const bool      omit_zeroing_entries)
+                  const bool    /*omit_zeroing_entries*/)
   {
-    if (size() != n)
-      {
-        Epetra_LocalMap map ((TrilinosWrappers::types::int_type)n, 0,
-                             Utilities::Trilinos::comm_self());
-        vector.reset (new Epetra_FEVector (map));
-      }
-    else if (omit_zeroing_entries == false)
-      {
-        int ierr;
-        ierr = vector->GlobalAssemble(last_action);
-        (void)ierr;
-        Assert (ierr == 0, ExcTrilinosError(ierr));
-
-        ierr = vector->PutScalar(0.0);
-        Assert (ierr == 0, ExcTrilinosError(ierr));
-      }
+    owned_elements = complete_index_set(n);
+    Epetra_LocalMap map ((TrilinosWrappers::types::int_type)n, 0,
+                         Utilities::Trilinos::comm_self());
+    vector.reset (new Epetra_FEVector (map));
 
     last_action = Zero;
   }
@@ -588,25 +692,13 @@ namespace TrilinosWrappers
 
   void
   Vector::reinit (const Epetra_Map &input_map,
-                  const bool        omit_zeroing_entries)
+                  const bool     /*omit_zeroing_entries*/)
   {
-    if (n_global_elements(vector->Map()) != n_global_elements(input_map))
-      {
-        Epetra_LocalMap map (n_global_elements(input_map),
-                             input_map.IndexBase(),
-                             input_map.Comm());
-        vector.reset (new Epetra_FEVector (map));
-      }
-    else if (omit_zeroing_entries == false)
-      {
-        int ierr;
-        ierr = vector->GlobalAssemble(last_action);
-        (void)ierr;
-        Assert (ierr == 0, ExcTrilinosError(ierr));
-
-        ierr = vector->PutScalar(0.0);
-        Assert (ierr == 0, ExcTrilinosError(ierr));
-      }
+    Epetra_LocalMap map (n_global_elements(input_map),
+                         input_map.IndexBase(),
+                         input_map.Comm());
+    vector.reset (new Epetra_FEVector(map));
+    owned_elements = complete_index_set(n_global_elements(input_map));
 
     last_action = Zero;
   }
@@ -616,33 +708,20 @@ namespace TrilinosWrappers
   void
   Vector::reinit (const IndexSet &partitioning,
                   const MPI_Comm &communicator,
-                  const bool      omit_zeroing_entries)
+                  const bool    /*omit_zeroing_entries*/)
   {
-    if (n_global_elements(vector->Map()) !=
-        static_cast<TrilinosWrappers::types::int_type>(partitioning.size()))
-      {
-        Epetra_LocalMap map (static_cast<TrilinosWrappers::types::int_type>(partitioning.size()),
-                             0,
+    Epetra_LocalMap map (static_cast<TrilinosWrappers::types::int_type>(partitioning.size()),
+                         0,
 #ifdef DEAL_II_WITH_MPI
-                             Epetra_MpiComm(communicator));
+                         Epetra_MpiComm(communicator));
 #else
-                             Epetra_SerialComm());
-        (void)communicator;
+                         Epetra_SerialComm());
+    (void)communicator;
 #endif
-        vector.reset (new Epetra_FEVector(map));
-      }
-    else if (omit_zeroing_entries == false)
-      {
-        int ierr;
-        ierr = vector->GlobalAssemble(last_action);
-        (void)ierr;
-        Assert (ierr == 0, ExcTrilinosError(ierr));
-
-        ierr = vector->PutScalar(0.0);
-        Assert (ierr == 0, ExcTrilinosError(ierr));
-      }
+    vector.reset (new Epetra_FEVector(map));
 
     last_action = Zero;
+    owned_elements = partitioning;
   }
 
 
@@ -659,17 +738,26 @@ namespace TrilinosWrappers
     // the vector, initialize our
     // map with the map in v, and
     // generate the vector.
-    (void)omit_zeroing_entries;
     if (allow_different_maps == false)
       {
-        if (local_range() != v.local_range())
+        // check equality for MPI communicators
+#ifdef DEAL_II_WITH_MPI
+        const Epetra_MpiComm *my_comm = dynamic_cast<const Epetra_MpiComm *>(&vector->Comm());
+        const Epetra_MpiComm *v_comm = dynamic_cast<const Epetra_MpiComm *>(&v.vector->Comm());
+        const bool same_communicators = my_comm != NULL && v_comm != NULL &&
+                                        my_comm->DataPtr() == v_comm->DataPtr();
+#else
+        const bool same_communicators = true;
+#endif
+        if (!same_communicators || local_range() != v.local_range())
           {
             Epetra_LocalMap map (global_length(*(v.vector)),
                                  v.vector->Map().IndexBase(),
                                  v.vector->Comm());
             vector.reset (new Epetra_FEVector(map));
+            owned_elements = v.owned_elements;
           }
-        else
+        else if (omit_zeroing_entries)
           {
             int ierr;
             Assert (vector->Map().SameAs(v.vector->Map()) == true,
@@ -741,6 +829,7 @@ namespace TrilinosWrappers
                              v.vector->Map().IndexBase(),
                              v.vector->Comm());
         vector.reset (new Epetra_FEVector(map));
+        owned_elements = v.owned_elements;
       }
 
     const int ierr = vector->Update(1.0, *v.vector, 0.0);
