@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2015 by the deal.II authors
+// Copyright (C) 2008 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -18,11 +18,15 @@
 
 #ifdef DEAL_II_WITH_TRILINOS
 
+#  include <deal.II/base/mpi.h>
+
 #  include <cmath>
 
 DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #  include <Epetra_Import.h>
 #  include <Epetra_Export.h>
+
+#  include <boost/io/ios_state.hpp>
 DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 
 
@@ -98,7 +102,8 @@ namespace TrilinosWrappers
     last_action (Zero),
     compressed (true),
     has_ghosts  (v.has_ghosts),
-    vector(new Epetra_FEVector(*v.vector))
+    vector(new Epetra_FEVector(*v.vector)),
+    owned_elements(complete_index_set(v.size()))
   {}
 
 
@@ -137,6 +142,7 @@ namespace TrilinosWrappers
         last_action = Zero;
         vector.reset (new Epetra_FEVector(*v.vector));
         has_ghosts = v.has_ghosts;
+        owned_elements = v.owned_elements;
       }
     else
       {
@@ -206,10 +212,11 @@ namespace TrilinosWrappers
     // otherwise result in undefined behaviour in the call to
     // GlobalAssemble().
     double double_mode = mode;
+    const Epetra_MpiComm *comm_ptr
+      = dynamic_cast<const Epetra_MpiComm *>(&(vector_partitioner().Comm()));
+    Assert (comm_ptr != 0, ExcInternalError());
     Utilities::MPI::MinMaxAvg result
-      = Utilities::MPI::min_max_avg (double_mode,
-                                     dynamic_cast<const Epetra_MpiComm *>
-                                     (&vector_partitioner().Comm())->GetMpiComm());
+      = Utilities::MPI::min_max_avg (double_mode, comm_ptr->GetMpiComm());
     Assert(result.max-result.min<1e-5,
            ExcMessage ("Not all processors agree whether the last operation on "
                        "this vector was an addition or a set operation. This will "
@@ -226,7 +233,8 @@ namespace TrilinosWrappers
       {
         Epetra_Export exporter(nonlocal_vector->Map(), vector->Map());
         ierr = vector->Export(*nonlocal_vector, exporter, mode);
-        nonlocal_vector->PutScalar(0.);
+        AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+        ierr = nonlocal_vector->PutScalar(0.);
       }
     AssertThrow (ierr == 0, ExcTrilinosError(ierr));
     last_action = Zero;
@@ -364,6 +372,7 @@ namespace TrilinosWrappers
     // is zero on _all_ processors.
     const Epetra_MpiComm *mpi_comm
       = dynamic_cast<const Epetra_MpiComm *>(&vector->Map().Comm());
+    Assert(mpi_comm != 0, ExcInternalError());
     unsigned int num_nonzero = Utilities::MPI::sum(flag, mpi_comm->Comm());
     return num_nonzero == 0;
 #else
@@ -483,6 +492,7 @@ namespace TrilinosWrappers
                      const bool         across) const
   {
     AssertThrow (out, ExcIO());
+    boost::io::ios_flags_saver restore_flags(out);
 
     // get a representation of the
     // vector and loop over all
