@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2015 by the deal.II authors
+// Copyright (C) 1999 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -21,50 +21,32 @@ DEAL_II_NAMESPACE_OPEN
 
 // instantiate for integers:
 template class Vector<int>;
-namespace internal
-{
-  namespace Vector
-  {
-    template void copy_vector<int,double> (const dealii::Vector<int> &,
-                                           dealii::Vector<double> &);
-
-    template void copy_vector<int,int> (const dealii::Vector<int> &,
-                                        dealii::Vector<int> &);
-  }
-}
+template Vector<double> &Vector<double>::operator=<int> (const dealii::Vector<int> &);
 
 template
 void Vector<int>::reinit<double>(const Vector<double> &, const bool);
 
+// instantiate for long double manually because we use it in a few places:
+template class Vector<long double>;
+template long double Vector<long double>::operator *<long double>(const Vector<long double> &) const;
 
 // do a few functions that currently don't fit the scheme because they have
 // two template arguments that need to be different (the case of same
 // arguments is covered by the default copy constructor and copy operator that
 // is declared separately)
 
-#define TEMPL_COPY_CONSTRUCTOR(S1,S2)                   \
-  template Vector<S1>::Vector (const Vector<S2> &)
+#define TEMPL_COPY_CONSTRUCTOR(S1,S2)                           \
+  template Vector<S1>::Vector (const Vector<S2> &);             \
+  template Vector<S1>& Vector<S1>::operator=<S2> (const Vector<S2> &)
 
 #ifndef DEAL_II_EXPLICIT_CONSTRUCTOR_BUG
 TEMPL_COPY_CONSTRUCTOR(double,float);
 TEMPL_COPY_CONSTRUCTOR(float,double);
 
-TEMPL_COPY_CONSTRUCTOR(long double,double);
-TEMPL_COPY_CONSTRUCTOR(double,long double);
-
-TEMPL_COPY_CONSTRUCTOR(long double,float);
-TEMPL_COPY_CONSTRUCTOR(float,long double);
-
 
 TEMPL_COPY_CONSTRUCTOR(std::complex<double>,std::complex<float>);
 TEMPL_COPY_CONSTRUCTOR(std::complex<float>,std::complex<double>);
 
-TEMPL_COPY_CONSTRUCTOR(std::complex<long double>,std::complex<double>);
-TEMPL_COPY_CONSTRUCTOR(std::complex<double>,std::complex<long double>);
-
-TEMPL_COPY_CONSTRUCTOR(std::complex<long double>,std::complex<float>);
-TEMPL_COPY_CONSTRUCTOR(std::complex<float>,std::complex<long double>);
-
 #endif
 
 #undef TEMPL_COPY_CONSTRUCTOR
@@ -77,22 +59,33 @@ TEMPL_COPY_CONSTRUCTOR(std::complex<float>,std::complex<long double>);
 TEMPL_OP_EQ(double,float);
 TEMPL_OP_EQ(float,double);
 
-TEMPL_OP_EQ(long double,double);
-TEMPL_OP_EQ(double,long double);
-
-TEMPL_OP_EQ(long double,float);
-TEMPL_OP_EQ(float,long double);
-
 
 TEMPL_OP_EQ(std::complex<double>,std::complex<float>);
 TEMPL_OP_EQ(std::complex<float>,std::complex<double>);
 
-TEMPL_OP_EQ(std::complex<long double>,std::complex<double>);
-TEMPL_OP_EQ(std::complex<double>,std::complex<long double>);
-
-TEMPL_OP_EQ(std::complex<long double>,std::complex<float>);
-TEMPL_OP_EQ(std::complex<float>,std::complex<long double>);
-
 #undef TEMPL_OP_EQ
 
+#ifdef DEAL_II_BOOST_BIND_COMPILER_BUG
+template <>
+Vector<std::complex<float> > &
+Vector<std::complex<float> >::operator= (const std::complex<float> s)
+{
+  AssertIsFinite(s);
+  if (s != std::complex<float>())
+    Assert (vec_size!=0, ExcEmptyObject());
+  if (vec_size!=0)
+    std::fill (begin(), end(), s);
+
+  return *this;
+}
+#endif
+
+template <>
+Vector<int>::real_type
+Vector<int>::lp_norm (const real_type) const
+{
+  Assert(false, ExcMessage("No lp norm for integer vectors"));
+  return -1;
+}
+
 DEAL_II_NAMESPACE_CLOSE
