@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2007 - 2015 by the deal.II authors
+// Copyright (C) 2007 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -15,8 +15,9 @@
 
 #include <deal.II/lac/vector.h>
 #include <deal.II/lac/block_vector.h>
-#include <deal.II/lac/parallel_vector.h>
-#include <deal.II/lac/parallel_block_vector.h>
+#include <deal.II/lac/la_vector.h>
+#include <deal.II/lac/la_parallel_vector.h>
+#include <deal.II/lac/la_parallel_block_vector.h>
 #include <deal.II/lac/petsc_vector.h>
 #include <deal.II/lac/petsc_block_vector.h>
 #include <deal.II/lac/petsc_parallel_vector.h>
@@ -31,4 +32,15 @@ DEAL_II_NAMESPACE_OPEN
 
 #include "vector_memory.inst"
 
+namespace internal
+{
+  namespace GrowingVectorMemory
+  {
+    void release_all_unused_memory()
+    {
+#include "vector_memory_release.inst"
+    }
+  }
+}
+
 DEAL_II_NAMESPACE_CLOSE
