@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2003 - 2015 by the deal.II authors
+// Copyright (C) 2003 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -137,16 +137,12 @@ MGTransferBlockSelect<number>::copy_to_mg (
   // multilevel block is always the
   // first, since only one block is
   // selected.
-  bool first = true;
   for (unsigned int level=mg_dof_handler.get_triangulation().n_levels(); level != 0;)
     {
       --level;
       for (IT i= copy_indices[selected_block][level].begin();
            i != copy_indices[selected_block][level].end(); ++i)
         dst[level](i->second) = src.block(selected_block)(i->first);
-      if (!first)
-        restrict_and_add (level+1, dst[level], dst[level+1]);
-      first = false;
     }
 }
 
@@ -164,16 +160,12 @@ MGTransferBlockSelect<number>::copy_to_mg (
   // For MGTransferBlockSelect, the
   // multilevel block is always the
   // first, since only one block is selected.
-  bool first = true;
   for (unsigned int level=mg_dof_handler.get_triangulation().n_levels(); level != 0;)
     {
       --level;
       for (IT i= copy_indices[selected_block][level].begin();
            i != copy_indices[selected_block][level].end(); ++i)
         dst[level](i->second) = src(i->first);
-      if (!first)
-        restrict_and_add (level+1, dst[level], dst[level+1]);
-      first = false;
     }
 }
 
@@ -188,7 +180,6 @@ MGTransferBlock<number>::copy_to_mg (
   const BlockVector<number2> &src) const
 {
   reinit_vector_by_blocks(mg_dof_handler, dst, selected, sizes);
-  bool first = true;
   for (unsigned int level=mg_dof_handler.get_triangulation().n_levels(); level != 0;)
     {
       --level;
@@ -197,9 +188,6 @@ MGTransferBlock<number>::copy_to_mg (
           for (IT i= copy_indices[block][level].begin();
                i != copy_indices[block][level].end(); ++i)
             dst[level].block(mg_block[block])(i->second) = src.block(block)(i->first);
-      if (!first)
-        restrict_and_add (level+1, dst[level], dst[level+1]);
-      first = false;
     }
 }
 
@@ -527,8 +515,9 @@ void MGTransferBlockSelect<number>::build_matrices (
       // copy_indices.
       const types::global_dof_index n_active_dofs =
         std::count_if (temp_copy_indices.begin(), temp_copy_indices.end(),
-                       std::bind2nd(std::not_equal_to<types::global_dof_index>(),
-                                    numbers::invalid_dof_index));
+                       std_cxx11::bind (std::not_equal_to<types::global_dof_index>(),
+                                        std_cxx11::_1,
+                                        numbers::invalid_dof_index));
       copy_indices[selected_block][level].resize (n_active_dofs);
       types::global_dof_index counter = 0;
       for (types::global_dof_index i=0; i<temp_copy_indices.size(); ++i)
@@ -607,8 +596,9 @@ void MGTransferBlock<number>::build_matrices (
             const types::global_dof_index n_active_dofs =
               std::count_if (temp_copy_indices[block].begin(),
                              temp_copy_indices[block].end(),
-                             std::bind2nd(std::not_equal_to<types::global_dof_index>(),
-                                          numbers::invalid_dof_index));
+                             std_cxx11::bind (std::not_equal_to<types::global_dof_index>(),
+                                              std_cxx11::_1,
+                                              numbers::invalid_dof_index));
             copy_indices[block][level].resize (n_active_dofs);
             types::global_dof_index counter = 0;
             for (types::global_dof_index i=0; i<temp_copy_indices[block].size(); ++i)
