@@ -19,8 +19,8 @@
 
 #include <deal.II/lac/vector.h>
 #include <deal.II/lac/block_vector.h>
-#include <deal.II/lac/parallel_vector.h>
-#include <deal.II/lac/parallel_block_vector.h>
+#include <deal.II/lac/la_parallel_vector.h>
+#include <deal.II/lac/la_parallel_block_vector.h>
 #include <deal.II/lac/petsc_vector.h>
 #include <deal.II/lac/petsc_block_vector.h>
 #include <deal.II/lac/trilinos_vector.h>
@@ -48,9 +48,15 @@ MGTransferPrebuilt<VectorType>::MGTransferPrebuilt ()
 
 
 template<typename VectorType>
-MGTransferPrebuilt<VectorType>::MGTransferPrebuilt (const ConstraintMatrix &c, const MGConstrainedDoFs &mg_c)
-  :
-  constraints(&c)
+MGTransferPrebuilt<VectorType>::MGTransferPrebuilt (const MGConstrainedDoFs &mg_c)
+{
+  this->mg_constrained_dofs = &mg_c;
+}
+
+
+
+template<typename VectorType>
+MGTransferPrebuilt<VectorType>::MGTransferPrebuilt (const ConstraintMatrix &/*c*/, const MGConstrainedDoFs &mg_c)
 {
   this->mg_constrained_dofs = &mg_c;
 }
@@ -65,14 +71,22 @@ MGTransferPrebuilt<VectorType>::~MGTransferPrebuilt ()
 
 template <typename VectorType>
 void MGTransferPrebuilt<VectorType>::initialize_constraints
-(const ConstraintMatrix &c, const MGConstrainedDoFs &mg_c)
+(const MGConstrainedDoFs &mg_c)
 {
-  constraints = &c;
   this->mg_constrained_dofs = &mg_c;
 }
 
 
 
+template <typename VectorType>
+void MGTransferPrebuilt<VectorType>::initialize_constraints
+(const ConstraintMatrix &/*c*/, const MGConstrainedDoFs &mg_c)
+{
+  initialize_constraints(mg_c);
+}
+
+
+
 template <typename VectorType>
 void MGTransferPrebuilt<VectorType>::clear ()
 {
@@ -80,7 +94,6 @@ void MGTransferPrebuilt<VectorType>::clear ()
   prolongation_matrices.resize(0);
   prolongation_sparsities.resize(0);
   interface_dofs.resize(0);
-  constraints = 0;
 }
 
 
@@ -172,8 +185,8 @@ void MGTransferPrebuilt<VectorType>::build_matrices
       DynamicSparsityPattern dsp (this->sizes[level+1],
                                   this->sizes[level],
                                   level_p1_relevant_dofs);
-      for (typename DoFHandler<dim,spacedim>::cell_iterator cell=mg_dof.begin(level);
-           cell != mg_dof.end(level); ++cell)
+      typename DoFHandler<dim>::cell_iterator cell, endc = mg_dof.end(level);
+      for (cell=mg_dof.begin(level); cell != endc; ++cell)
         if (cell->has_children() &&
             ( mg_dof.get_triangulation().locally_owned_subdomain()==numbers::invalid_subdomain_id
               || cell->level_subdomain_id()==mg_dof.get_triangulation().locally_owned_subdomain()
@@ -219,8 +232,7 @@ void MGTransferPrebuilt<VectorType>::build_matrices
       FullMatrix<double> prolongation;
 
       // now actually build the matrices
-      for (typename DoFHandler<dim,spacedim>::cell_iterator cell=mg_dof.begin(level);
-           cell != mg_dof.end(level); ++cell)
+      for (cell=mg_dof.begin(level); cell != endc; ++cell)
         if (cell->has_children() &&
             (mg_dof.get_triangulation().locally_owned_subdomain()==numbers::invalid_subdomain_id
              || cell->level_subdomain_id()==mg_dof.get_triangulation().locally_owned_subdomain())
