@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2015 by the deal.II authors
+// Copyright (C) 2000 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -17,8 +17,9 @@
 #include <deal.II/base/work_stream.h>
 #include <deal.II/lac/vector.h>
 #include <deal.II/lac/block_vector.h>
-#include <deal.II/lac/parallel_vector.h>
-#include <deal.II/lac/parallel_block_vector.h>
+#include <deal.II/lac/la_vector.h>
+#include <deal.II/lac/la_parallel_vector.h>
+#include <deal.II/lac/la_parallel_block_vector.h>
 #include <deal.II/lac/petsc_vector.h>
 #include <deal.II/lac/petsc_block_vector.h>
 #include <deal.II/lac/trilinos_vector.h>
@@ -937,19 +938,19 @@ namespace DerivativeApproximation
      const unsigned int                  component)
     {
       // if the cell is not locally owned, then there is nothing to do
-      if (std_cxx11::get<0>(cell.iterators)->is_locally_owned() == false)
-        *std_cxx11::get<1>(cell.iterators) = 0;
+      if (std_cxx11::get<0>(*cell)->is_locally_owned() == false)
+        *std_cxx11::get<1>(*cell) = 0;
       else
         {
           typename DerivativeDescription::Derivative derivative;
           // call the function doing the actual
           // work on this cell
           approximate_cell<DerivativeDescription,dim,DoFHandlerType,InputVector, spacedim>
-          (mapping,dof_handler,solution,component,std_cxx11::get<0>(cell.iterators),derivative);
+          (mapping,dof_handler,solution,component,std_cxx11::get<0>(*cell),derivative);
 
           // evaluate the norm and fill the vector
           //*derivative_norm_on_this_cell
-          *std_cxx11::get<1>(cell.iterators) = DerivativeDescription::derivative_norm (derivative);
+          *std_cxx11::get<1>(*cell) = DerivativeDescription::derivative_norm (derivative);
         }
     }
 
@@ -1074,19 +1075,19 @@ namespace DerivativeApproximation
   }
 
 
-  template <typename DoFHandlerType, int dim, int spacedim, class InputVector, int order>
+  template <typename DoFHandlerType, class InputVector, int order>
   void
   approximate_derivative_tensor
-  (const Mapping<dim, spacedim>                        &mapping,
-   const DoFHandlerType                                &dof,
-   const InputVector                                   &solution,
+  (const Mapping<DoFHandlerType::dimension, DoFHandlerType::space_dimension> &mapping,
+   const DoFHandlerType                                                      &dof,
+   const InputVector                                                         &solution,
 #ifndef _MSC_VER
-   const typename DoFHandlerType::active_cell_iterator &cell,
+   const typename DoFHandlerType::active_cell_iterator                       &cell,
 #else
    const TriaActiveIterator < dealii::DoFCellAccessor < DoFHandlerType, false > > &cell,
 #endif
-   Tensor<order, dim>                                  &derivative,
-   const unsigned int                                   component)
+   Tensor<order, DoFHandlerType::dimension>                                  &derivative,
+   const unsigned int                                                         component)
   {
     internal::approximate_cell<typename internal::DerivativeSelector<order,DoFHandlerType::dimension>::DerivDescr>
     (mapping,
@@ -1099,7 +1100,7 @@ namespace DerivativeApproximation
 
 
 
-  template <typename DoFHandlerType, int dim, int spacedim, class InputVector, int order>
+  template <typename DoFHandlerType, class InputVector, int order>
   void
   approximate_derivative_tensor
   (const DoFHandlerType                                &dof,
@@ -1109,12 +1110,12 @@ namespace DerivativeApproximation
 #else
    const TriaActiveIterator < dealii::DoFCellAccessor < DoFHandlerType, false > > &cell,
 #endif
-   Tensor<order, dim>                                  &derivative,
+   Tensor<order, DoFHandlerType::dimension>            &derivative,
    const unsigned int                                   component)
   {
     // just call the respective function with Q1 mapping
-    approximate_derivative_tensor<DoFHandlerType, dim, spacedim, InputVector, order>
-    (StaticMappingQ1<dim, spacedim>::mapping,
+    approximate_derivative_tensor
+    (StaticMappingQ1<DoFHandlerType::dimension, DoFHandlerType::space_dimension>::mapping,
      dof,
      solution,
      cell,
