@@ -1,7 +1,6 @@
 // ---------------------------------------------------------------------
-// $Id: histogram.cc 30723 2013-09-16 03:34:44Z bangerth $
 //
-// Copyright (C) 1999 - 2013 by the deal.II authors
+// Copyright (C) 1999 - 2015 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -57,8 +56,10 @@ void Histogram::evaluate (const std::vector<Vector<number> > &values,
                           const unsigned int                  n_intervals,
                           const IntervalSpacing               interval_spacing)
 {
-  Assert (values.size() > 0, ExcEmptyData());
-  Assert (n_intervals > 0, ExcInvalidIntervals());
+  Assert (values.size() > 0,
+          ExcMessage("Your input data needs to contain at least one input vector."));
+  Assert (n_intervals > 0,
+          ExcMessage("The number of intervals needs to be at least one."));
   for (unsigned int i=0; i<values.size(); ++i)
     Assert (values[i].size() > 0, ExcEmptyData());
   Assert (values.size() == y_values_.size(),
@@ -217,7 +218,9 @@ void Histogram::evaluate (const Vector<number>    &values,
 void Histogram::write_gnuplot (std::ostream &out) const
 {
   AssertThrow (out, ExcIO());
-  Assert (!intervals.empty(), ExcEmptyData());
+  Assert (!intervals.empty(),
+          ExcMessage("There is nothing to write into the output file. "
+                     "Did you forget to call the evaluate() function?"));
 
   // do a simple 2d plot, if only
   // one data set is available
