@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2015 by the deal.II authors
+// Copyright (C) 1998 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -59,6 +59,9 @@
 
 DEAL_II_NAMESPACE_OPEN
 
+
+
+
 namespace MatrixTools
 {
   namespace
@@ -74,7 +77,7 @@ namespace MatrixTools
 //TODO:[WB] I don't think that the optimized storage of diagonals is needed (GK)
   template <typename number>
   void
-  apply_boundary_values (const std::map<types::global_dof_index,double> &boundary_values,
+  apply_boundary_values (const std::map<types::global_dof_index,number> &boundary_values,
                          SparseMatrix<number>  &matrix,
                          Vector<number>   &solution,
                          Vector<number>   &right_hand_side,
@@ -103,15 +106,15 @@ namespace MatrixTools
     // there is no such thing
     number first_nonzero_diagonal_entry = 1;
     for (unsigned int i=0; i<n_dofs; ++i)
-      if (matrix.diag_element(i) != 0)
+      if (matrix.diag_element(i) != number())
         {
           first_nonzero_diagonal_entry = matrix.diag_element(i);
           break;
         }
 
 
-    std::map<types::global_dof_index,double>::const_iterator dof  = boundary_values.begin(),
-                                                             endd = boundary_values.end();
+    typename std::map<types::global_dof_index,number>::const_iterator dof  = boundary_values.begin(),
+                                                                      endd = boundary_values.end();
     for (; dof != endd; ++dof)
       {
         Assert (dof->first < n_dofs, ExcInternalError());
@@ -141,7 +144,7 @@ namespace MatrixTools
         // store the new rhs entry to make
         // the gauss step more efficient
         number new_rhs;
-        if (matrix.diag_element(dof_number) != 0.0)
+        if (matrix.diag_element(dof_number) != number())
           {
             new_rhs = dof->second * matrix.diag_element(dof_number);
             right_hand_side(dof_number) = new_rhs;
@@ -215,7 +218,7 @@ namespace MatrixTools
                                    "function, see the documentation."));
 
                 // correct right hand side
-                right_hand_side(row) -= p->value() /
+                right_hand_side(row) -= static_cast<number>(p->value()) /
                                         diagonal_entry * new_rhs;
 
                 // set matrix entry to zero
@@ -232,7 +235,7 @@ namespace MatrixTools
 
   template <typename number>
   void
-  apply_boundary_values (const std::map<types::global_dof_index,double> &boundary_values,
+  apply_boundary_values (const std::map<types::global_dof_index,number> &boundary_values,
                          BlockSparseMatrix<number>  &matrix,
                          BlockVector<number>   &solution,
                          BlockVector<number>   &right_hand_side,
@@ -292,8 +295,8 @@ namespace MatrixTools
       first_nonzero_diagonal_entry = 1;
 
 
-    std::map<types::global_dof_index,double>::const_iterator dof  = boundary_values.begin(),
-                                                             endd = boundary_values.end();
+    typename std::map<types::global_dof_index,number>::const_iterator dof  = boundary_values.begin(),
+                                                                      endd = boundary_values.end();
     const BlockSparsityPattern &
     sparsity_pattern = matrix.get_sparsity_pattern();
 
@@ -491,12 +494,12 @@ namespace MatrixTools
 
 
 
-
+  template <typename number>
   void
-  local_apply_boundary_values (const std::map<types::global_dof_index,double> &boundary_values,
+  local_apply_boundary_values (const std::map<types::global_dof_index,number> &boundary_values,
                                const std::vector<types::global_dof_index> &local_dof_indices,
-                               FullMatrix<double> &local_matrix,
-                               Vector<double>     &local_rhs,
+                               FullMatrix<number> &local_matrix,
+                               Vector<number>     &local_rhs,
                                const bool          eliminate_columns)
   {
     Assert (local_dof_indices.size() == local_matrix.m(),
@@ -537,11 +540,11 @@ namespace MatrixTools
     //
     // we only compute this value lazily the
     // first time we need it.
-    double average_diagonal = 0;
+    number average_diagonal = 0;
     const unsigned int n_local_dofs = local_dof_indices.size();
     for (unsigned int i=0; i<n_local_dofs; ++i)
       {
-        const std::map<types::global_dof_index, double>::const_iterator
+        const typename std::map<types::global_dof_index, number>::const_iterator
         boundary_value = boundary_values.find (local_dof_indices[i]);
         if (boundary_value != boundary_values.end())
           {
