@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2015 by the deal.II authors
+// Copyright (C) 2009 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -16,8 +16,9 @@
 
 #include <deal.II/lac/vector.h>
 #include <deal.II/lac/block_vector.h>
-#include <deal.II/lac/parallel_vector.h>
-#include <deal.II/lac/parallel_block_vector.h>
+#include <deal.II/lac/la_vector.h>
+#include <deal.II/lac/la_parallel_vector.h>
+#include <deal.II/lac/la_parallel_block_vector.h>
 #include <deal.II/lac/petsc_vector.h>
 #include <deal.II/lac/petsc_block_vector.h>
 #include <deal.II/lac/trilinos_vector.h>
@@ -191,11 +192,6 @@ void PointValueHistory<dim>
   // points locations are dofs locations
   AssertThrow (dof_handler->get_fe ().has_support_points (), ExcNotImplemented ());
 
-  // FEValues object to extract quadrature
-  // points from
-  std::vector <Point <dim> >
-  unit_support_points = dof_handler->get_fe ().get_unit_support_points ();
-
   // While in general quadrature points seems
   // to refer to Gauss quadrature points, in
   // this case the quadrature points are
@@ -307,7 +303,7 @@ void PointValueHistory<dim>
 
   std::map <std::string, std::vector <std::vector <double> > >::iterator
   data_store_begin = data_store.begin ();
-  for (; data_store_begin != data_store.end (); data_store_begin++)
+  for (; data_store_begin != data_store.end (); ++data_store_begin)
     {
       // add an extra row to each vector
       // entry
@@ -342,10 +338,6 @@ void PointValueHistory<dim>
   // points locations are dofs locations
   AssertThrow (dof_handler->get_fe ().has_support_points (), ExcNotImplemented ());
 
-  // FEValues object to extract quadrature
-  // points from
-  std::vector <Point <dim> > unit_support_points = dof_handler->get_fe ().get_unit_support_points ();
-
   // While in general quadrature points seems
   // to refer to Gauss quadrature points, in
   // this case the quadrature points are
@@ -431,7 +423,7 @@ void PointValueHistory<dim>
 
       std::map <std::string, std::vector <std::vector <double> > >::iterator
       data_store_begin = data_store.begin ();
-      for (; data_store_begin != data_store.end (); data_store_begin++)
+      for (; data_store_begin != data_store.end (); ++data_store_begin)
         {
           // add an extra row to each vector
           // entry
@@ -591,7 +583,7 @@ void PointValueHistory<dim>
   unsigned int n_stored = mask->second.n_selected_components(dof_handler->get_fe ().n_components ());
 
   typename std::vector <internal::PointValueHistory::PointGeometryData <dim> >::iterator point = point_geometry_data.begin ();
-  for (unsigned int data_store_index = 0; point != point_geometry_data.end (); point++, data_store_index++)
+  for (unsigned int data_store_index = 0; point != point_geometry_data.end (); ++point, ++data_store_index)
     {
       // Look up the components to add
       // in the component_mask, and
@@ -645,7 +637,7 @@ void PointValueHistory<dim>
 
   // Loop over points and find correct cell
   typename std::vector <internal::PointValueHistory::PointGeometryData <dim> >::iterator point = point_geometry_data.begin ();
-  for (unsigned int data_store_index = 0; point != point_geometry_data.end (); point++, data_store_index++)
+  for (unsigned int data_store_index = 0; point != point_geometry_data.end (); ++point, ++data_store_index)
     {
       // we now have a point to query,
       // need to know what cell it is in
@@ -661,9 +653,9 @@ void PointValueHistory<dim>
         {
           // Extract data for the
           // PostProcessor object
-          std::vector< typename VectorType::value_type > uh (n_quadrature_points, 0.0);
-          std::vector< Tensor< 1, dim, typename VectorType::value_type > > duh (n_quadrature_points, Tensor <1, dim, typename VectorType::value_type> ());
-          std::vector< Tensor< 2, dim, typename VectorType::value_type > > dduh (n_quadrature_points, Tensor <2, dim, typename VectorType::value_type> ());
+          std::vector< typename VectorType::value_type > solution_values (n_quadrature_points, 0.0);
+          std::vector< Tensor< 1, dim, typename VectorType::value_type > > solution_gradients (n_quadrature_points, Tensor <1, dim, typename VectorType::value_type> ());
+          std::vector< Tensor< 2, dim, typename VectorType::value_type > > solution_hessians (n_quadrature_points, Tensor <2, dim, typename VectorType::value_type> ());
           std::vector<Point<dim> > dummy_normals (1, Point<dim> ());
           std::vector<Point<dim> > evaluation_points;
           // at each point there is
@@ -671,13 +663,13 @@ void PointValueHistory<dim>
           // value, gradient etc.
           if (update_flags & update_values)
             fe_values.get_function_values (solution,
-                                           uh);
+                                           solution_values);
           if (update_flags & update_gradients)
             fe_values.get_function_gradients (solution,
-                                              duh);
+                                              solution_gradients);
           if (update_flags & update_hessians)
             fe_values.get_function_hessians (solution,
-                                             dduh);
+                                             solution_hessians);
 
           // find the closest quadrature point
           evaluation_points = fe_values.get_quadrature_points();
@@ -696,9 +688,9 @@ void PointValueHistory<dim>
           // or compute_derived_quantities_scalar
           // TODO this function should also operate with typename VectorType::value_type
           data_postprocessor.
-          compute_derived_quantities_scalar(std::vector< double > (1, uh[selected_point]),
-                                            std::vector< Tensor< 1, dim > > (1, Tensor< 1, dim >(duh[selected_point]) ),
-                                            std::vector< Tensor< 2, dim > > (1, Tensor< 2, dim >(dduh[selected_point]) ),
+          compute_derived_quantities_scalar(std::vector< double > (1, solution_values[selected_point]),
+                                            std::vector< Tensor< 1, dim > > (1, Tensor< 1, dim >(solution_gradients[selected_point]) ),
+                                            std::vector< Tensor< 2, dim > > (1, Tensor< 2, dim >(solution_hessians[selected_point]) ),
                                             dummy_normals,
                                             std::vector<Point<dim> > (1, evaluation_points[selected_point]),
                                             computed_quantities);
@@ -707,9 +699,9 @@ void PointValueHistory<dim>
       else     // The case of a vector FE
         {
           // Extract data for the PostProcessor object
-          std::vector< Vector< typename VectorType::value_type > > uh (n_quadrature_points, Vector <typename VectorType::value_type> (n_components));
-          std::vector< std::vector< Tensor< 1, dim, typename VectorType::value_type > > > duh (n_quadrature_points, std::vector< Tensor< 1, dim, typename VectorType::value_type > > (n_components,  Tensor< 1, dim, typename VectorType::value_type >()));
-          std::vector< std::vector< Tensor< 2, dim, typename VectorType::value_type > > > dduh (n_quadrature_points, std::vector< Tensor< 2, dim, typename VectorType::value_type > > (n_components,  Tensor< 2, dim, typename VectorType::value_type >()));
+          std::vector< Vector< typename VectorType::value_type > > solution_values (n_quadrature_points, Vector <typename VectorType::value_type> (n_components));
+          std::vector< std::vector< Tensor< 1, dim, typename VectorType::value_type > > > solution_gradients (n_quadrature_points, std::vector< Tensor< 1, dim, typename VectorType::value_type > > (n_components,  Tensor< 1, dim, typename VectorType::value_type >()));
+          std::vector< std::vector< Tensor< 2, dim, typename VectorType::value_type > > > solution_hessians (n_quadrature_points, std::vector< Tensor< 2, dim, typename VectorType::value_type > > (n_components,  Tensor< 2, dim, typename VectorType::value_type >()));
           std::vector<Point<dim> > dummy_normals  (1, Point<dim> ());
           std::vector<Point<dim> > evaluation_points;
           // at each point there is
@@ -718,13 +710,13 @@ void PointValueHistory<dim>
           // derivative...
           if (update_flags & update_values)
             fe_values.get_function_values (solution,
-                                           uh);
+                                           solution_values);
           if (update_flags & update_gradients)
             fe_values.get_function_gradients (solution,
-                                              duh);
+                                              solution_gradients);
           if (update_flags & update_hessians)
             fe_values.get_function_hessians (solution,
-                                             dduh);
+                                             solution_hessians);
 
           // find the closest quadrature point
           evaluation_points = fe_values.get_quadrature_points();
@@ -742,20 +734,20 @@ void PointValueHistory<dim>
           // FIXME: We need tmp vectors below because the data
           // postprocessors are not equipped to deal with anything but
           // doubles (scalars and tensors).
-          const Vector< typename VectorType::value_type >                        &uh_s   = uh[selected_point];
-          const std::vector< Tensor< 1, dim, typename VectorType::value_type > > &duh_s  = duh[selected_point];
-          const std::vector< Tensor< 2, dim, typename VectorType::value_type > > &dduh_s = dduh[selected_point];
-          std::vector< Tensor< 1, dim > > tmp_d (duh_s.size());
-          for (unsigned int i = 0; i < duh_s.size(); i++)
-            tmp_d[i] = duh_s[i];
+          const Vector< typename VectorType::value_type >                        &solution_values_s   = solution_values[selected_point];
+          const std::vector< Tensor< 1, dim, typename VectorType::value_type > > &solution_gradients_s  = solution_gradients[selected_point];
+          const std::vector< Tensor< 2, dim, typename VectorType::value_type > > &solution_hessians_s = solution_hessians[selected_point];
+          std::vector< Tensor< 1, dim > > tmp_d (solution_gradients_s.size());
+          for (unsigned int i = 0; i < solution_gradients_s.size(); i++)
+            tmp_d[i] = solution_gradients_s[i];
 
-          std::vector< Tensor< 2, dim > > tmp_dd (dduh_s.size());
-          for (unsigned int i = 0; i < dduh_s.size(); i++)
-            tmp_dd[i] = dduh_s[i];
+          std::vector< Tensor< 2, dim > > tmp_dd (solution_hessians_s.size());
+          for (unsigned int i = 0; i < solution_hessians_s.size(); i++)
+            tmp_dd[i] = solution_hessians_s[i];
 
-          Vector< double > tmp(uh_s.size());
-          for (unsigned int i = 0; i < uh_s.size(); i++)
-            tmp[i] = uh_s[i];
+          Vector< double > tmp(solution_values_s.size());
+          for (unsigned int i = 0; i < solution_values_s.size(); i++)
+            tmp[i] = solution_values_s[i];
           // Call compute_derived_quantities_vector
           // or compute_derived_quantities_scalar
           data_postprocessor.
@@ -771,7 +763,7 @@ void PointValueHistory<dim>
       // we now have the data and need to save it
       // loop over data names
       typename std::vector<std::string>::const_iterator name = vector_names.begin();
-      for (; name != vector_names.end(); name++)
+      for (; name != vector_names.end(); ++name)
         {
           typename std::map <std::string, std::vector <std::vector <double> > >::iterator data_store_field = data_store.find(*name);
           Assert (data_store_field != data_store.end(), ExcMessage("vector_name not in class"));
@@ -817,6 +809,7 @@ void PointValueHistory<dim>
 ::evaluate_field_at_requested_location (const std::string &vector_name,
                                         const VectorType  &solution)
 {
+  typedef typename VectorType::value_type number;
   // must be closed to add data to internal
   // members.
   Assert (closed, ExcInvalidState ());
@@ -841,8 +834,8 @@ void PointValueHistory<dim>
   unsigned int n_stored = mask->second.n_selected_components(dof_handler->get_fe ().n_components ());
 
   typename std::vector <internal::PointValueHistory::PointGeometryData <dim> >::iterator point = point_geometry_data.begin ();
-  Vector <double> value (dof_handler->get_fe().n_components());
-  for (unsigned int data_store_index = 0; point != point_geometry_data.end (); point++, data_store_index++)
+  Vector <number> value (dof_handler->get_fe().n_components());
+  for (unsigned int data_store_index = 0; point != point_geometry_data.end (); ++point, ++data_store_index)
     {
       // Make a Vector <double> for the value
       // at the point. It will have as many
@@ -898,7 +891,8 @@ void PointValueHistory<dim>
 
 template <int dim>
 void PointValueHistory<dim>
-::write_gnuplot (const std::string &base_name, const std::vector <Point <dim> > postprocessor_locations)
+::write_gnuplot (const std::string &base_name,
+                 const std::vector <Point <dim> > &postprocessor_locations)
 {
   AssertThrow (closed, ExcInvalidState ());
   AssertThrow (!cleared, ExcInvalidState ());
@@ -968,7 +962,7 @@ void PointValueHistory<dim>
       //AssertThrow (!triangulation_changed, ExcDoFHandlerChanged ());
 
       typename std::vector <internal::PointValueHistory::PointGeometryData <dim> >::iterator point = point_geometry_data.begin ();
-      for (unsigned int data_store_index = 0; point != point_geometry_data.end (); point++, data_store_index++)
+      for (unsigned int data_store_index = 0; point != point_geometry_data.end (); ++point, ++data_store_index)
         {
           // for each point, open a file to
           // be written to
@@ -1088,7 +1082,7 @@ Vector<double> PointValueHistory<dim>
   Vector<double> dof_vector (dof_handler->n_dofs ());
 
   typename std::vector <internal::PointValueHistory::PointGeometryData <dim> >::iterator point = point_geometry_data.begin ();
-  for (; point != point_geometry_data.end (); point++)
+  for (; point != point_geometry_data.end (); ++point)
     {
       for (unsigned int component = 0; component < dof_handler->get_fe ().n_components (); component++)
         {
@@ -1110,7 +1104,7 @@ void PointValueHistory<dim>
   std::vector <std::vector <Point <dim> > > actual_points;
   typename std::vector <internal::PointValueHistory::PointGeometryData <dim> >::iterator point = point_geometry_data.begin ();
 
-  for (; point != point_geometry_data.end (); point++)
+  for (; point != point_geometry_data.end (); ++point)
     {
       actual_points.push_back (point->support_point_locations);
     }
@@ -1141,7 +1135,7 @@ void PointValueHistory<dim>
 
   // Loop over points and find correct cell
   typename std::vector <internal::PointValueHistory::PointGeometryData <dim> >::iterator point = point_geometry_data.begin ();
-  for (unsigned int data_store_index = 0; point != point_geometry_data.end (); point++, data_store_index++)
+  for (unsigned int data_store_index = 0; point != point_geometry_data.end (); ++point, ++data_store_index)
     {
       // we now have a point to query,
       // need to know what cell it is in
@@ -1187,7 +1181,7 @@ void PointValueHistory<dim>
     {
       if (!cleared)
         {
-          for (; point != point_geometry_data.end (); point++)
+          for (; point != point_geometry_data.end (); ++point)
             {
               out << "# Requested location: " << point->requested_location << "\n";
               out << "# DoF_index : Support location (for each component)\n";
@@ -1229,7 +1223,7 @@ void PointValueHistory<dim>
     {
       out << "Mnemonic: data set size (mask size, n true components) : n data sets\n";
     }
-  for (; data_store_begin != data_store.end (); data_store_begin++)
+  for (; data_store_begin != data_store.end (); ++data_store_begin)
     {
       // Find field mnemonic
       std::string vector_name = data_store_begin->first;
@@ -1285,7 +1279,7 @@ bool PointValueHistory<dim>
       data_store_begin = data_store.begin ();
       if (have_dof_handler)
         {
-          for (; data_store_begin != data_store.end (); data_store_begin++)
+          for (; data_store_begin != data_store.end (); ++data_store_begin)
             {
               Assert (data_store_begin->second.size() > 0,
                       ExcInternalError());
@@ -1313,7 +1307,7 @@ bool PointValueHistory<dim>
     {
       std::map <std::string, std::vector <std::vector <double> > >::iterator
       data_store_begin = data_store.begin ();
-      for (; data_store_begin != data_store.end (); data_store_begin++)
+      for (; data_store_begin != data_store.end (); ++data_store_begin)
         {
           Assert (data_store_begin->second.size() > 0,
                   ExcInternalError());
