@@ -1,7 +1,6 @@
 // ---------------------------------------------------------------------
-// $Id: solution_transfer_inst3.cc 31527 2013-11-03 09:58:45Z maier $
 //
-// Copyright (C) 2013 by the deal.II authors
+// Copyright (C) 2013 - 2015 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -17,7 +16,5 @@
 // This file compiles the third quarter of the instantiations from solution_transfer.cc
 // to reduce the compilation unit (and memory consumption)
 
-#define INSTANTIATE_HP_DH
-//#define SOLUTION_TRANSFER_INSTANTIATE_PART_TWO
-
+#define SPLIT_INSTANTIATIONS_INDEX 2
 #include "solution_transfer.cc"
