@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2015 by the deal.II authors
+// Copyright (C) 1999 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -18,6 +18,7 @@
 #include <deal.II/base/thread_management.h>
 #include <deal.II/base/utilities.h>
 #include <deal.II/base/parallel.h>
+#include <deal.II/base/std_cxx11/bind.h>
 #include <deal.II/grid/tria.h>
 #include <deal.II/grid/tria_accessor.h>
 #include <deal.II/grid/tria_iterator.h>
@@ -50,8 +51,13 @@ TimeDependent::TimeDependent (const TimeSteppingData &data_primal,
 
 TimeDependent::~TimeDependent ()
 {
-  while (timesteps.size() != 0)
-    delete_timestep (0);
+  try
+    {
+      while (timesteps.size() != 0)
+        delete_timestep (0);
+    }
+  catch (...)
+    {}
 }
 
 
@@ -154,8 +160,8 @@ void TimeDependent::delete_timestep (const unsigned int position)
 void
 TimeDependent::solve_primal_problem ()
 {
-  do_loop (std::mem_fun(&TimeStepBase::init_for_primal_problem),
-           std::mem_fun(&TimeStepBase::solve_primal_problem),
+  do_loop (std_cxx11::bind(&TimeStepBase::init_for_primal_problem, std_cxx11::_1),
+           std_cxx11::bind(&TimeStepBase::solve_primal_problem, std_cxx11::_1),
            timestepping_data_primal,
            forward);
 }
@@ -164,8 +170,8 @@ TimeDependent::solve_primal_problem ()
 void
 TimeDependent::solve_dual_problem ()
 {
-  do_loop (std::mem_fun(&TimeStepBase::init_for_dual_problem),
-           std::mem_fun(&TimeStepBase::solve_dual_problem),
+  do_loop (std_cxx11::bind(&TimeStepBase::init_for_dual_problem, std_cxx11::_1),
+           std_cxx11::bind(&TimeStepBase::solve_dual_problem, std_cxx11::_1),
            timestepping_data_dual,
            backward);
 }
@@ -174,8 +180,8 @@ TimeDependent::solve_dual_problem ()
 void
 TimeDependent::postprocess ()
 {
-  do_loop (std::mem_fun(&TimeStepBase::init_for_postprocessing),
-           std::mem_fun(&TimeStepBase::postprocess_timestep),
+  do_loop (std_cxx11::bind(&TimeStepBase::init_for_postprocessing, std_cxx11::_1),
+           std_cxx11::bind(&TimeStepBase::postprocess_timestep, std_cxx11::_1),
            timestepping_data_postprocess,
            forward);
 }
@@ -249,7 +255,8 @@ TimeStepBase::TimeStepBase (const double time) :
   next_timestep (0),
   sweep_no (numbers::invalid_unsigned_int),
   timestep_no (numbers::invalid_unsigned_int),
-  time (time)
+  time (time),
+  next_action(numbers::invalid_unsigned_int)
 {}
 
 
@@ -862,7 +869,7 @@ void TimeStepBase_Tria<dim>::refine_grid (const RefinementData refinement_data)
           if (cell->refine_flag_set())
             previous_cells += (GeometryInfo<dim>::max_children_per_cell-1);
           else if (cell->coarsen_flag_set())
-            previous_cells -= (GeometryInfo<dim>::max_children_per_cell-1) /
+            previous_cells -= (double)(GeometryInfo<dim>::max_children_per_cell-1) /
                               GeometryInfo<dim>::max_children_per_cell;
 
         // @p{previous_cells} now gives the
@@ -888,7 +895,7 @@ void TimeStepBase_Tria<dim>::refine_grid (const RefinementData refinement_data)
           if (cell->refine_flag_set())
             estimated_cells += (GeometryInfo<dim>::max_children_per_cell-1);
           else if (cell->coarsen_flag_set())
-            estimated_cells -= (GeometryInfo<dim>::max_children_per_cell-1) /
+            estimated_cells -= (double)(GeometryInfo<dim>::max_children_per_cell-1) /
                                GeometryInfo<dim>::max_children_per_cell;
 
         // calculate the allowed delta in
