@@ -1,3 +1,18 @@
+// ---------------------------------------------------------------------
+//
+// Copyright (C) 2014 - 2017 by the deal.II authors
+//
+// This file is part of the deal.II library.
+//
+// The deal.II library is free software; you can use it, redistribute
+// it, and/or modify it under the terms of the GNU Lesser General
+// Public License as published by the Free Software Foundation; either
+// version 2.1 of the License, or (at your option) any later version.
+// The full text of the license can be found in the file LICENSE at
+// the top level of the deal.II distribution.
+//
+// ---------------------------------------------------------------------
+
 #include <deal.II/opencascade/utilities.h>
 
 #ifdef DEAL_II_WITH_OPENCASCADE
@@ -28,7 +43,12 @@ DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #include <TopoDS_Edge.hxx>
 #include <TopExp_Explorer.hxx>
 
-#include <Handle_Standard_Transient.hxx>
+#include <Standard_Version.hxx>
+#if (OCC_VERSION_MAJOR < 7)
+#  include <Handle_Standard_Transient.hxx>
+#else
+#  include <Standard_Transient.hxx>
+#endif
 
 #include <TColStd_SequenceOfTransient.hxx>
 #include <TColStd_HSequenceOfTransient.hxx>
@@ -394,11 +414,10 @@ namespace OpenCASCADE
     Assert(Inters.IsDone(), ExcMessage("Could not project point."));
 
     double minDistance = 1e7;
-    double distance;
     Point<3> result;
     for (int i=0; i<Inters.NbPnt(); ++i)
       {
-        distance = point(origin).Distance(Inters.Pnt(i+1));
+        const double distance = point(origin).Distance(Inters.Pnt(i+1));
         //cout<<"Point "<<i<<": "<<point(Inters.Pnt(i+1))<<"  distance: "<<distance<<endl;
         if (distance < minDistance)
           {
@@ -531,7 +550,7 @@ namespace OpenCASCADE
     return std_cxx11::get<0>(ref);
   }
 
-  std_cxx11::tuple<Point<3>, Point<3>, double>
+  std_cxx11::tuple<Point<3>,  Tensor<1,3>, double, double>
   closest_point_and_differential_forms(const TopoDS_Shape &in_shape,
                                        const Point<3> &origin,
                                        const double tolerance)
@@ -586,7 +605,7 @@ namespace OpenCASCADE
     return Point<3>();
   }
 
-  std_cxx11::tuple<Point<3>, Point<3>, double >
+  std_cxx11::tuple<Point<3>,  Tensor<1,3>, double, double>
   push_forward_and_differential_forms(const TopoDS_Face &face,
                                       const double u,
                                       const double v,
@@ -598,9 +617,22 @@ namespace OpenCASCADE
     Assert(props.IsNormalDefined(), ExcMessage("Normal is not well defined!"));
     gp_Dir Normal = props.Normal();
     Assert(props.IsCurvatureDefined(), ExcMessage("Curvature is not well defined!"));
-    Standard_Real Mean_Curvature = props.MeanCurvature();
-    Point<3> normal = Point<3>(Normal.X(),Normal.Y(),Normal.Z());
-    return std_cxx11::tuple<Point<3>, Point<3>, double>(point(Value), normal, Mean_Curvature);
+    Standard_Real Min_Curvature = props.MinCurvature();
+    Standard_Real Max_Curvature = props.MaxCurvature();
+    Tensor<1,3> normal = Point<3>(Normal.X(),Normal.Y(),Normal.Z());
+
+    // In the case your manifold changes from convex to concave or viceversa
+    // the normal could jump from "inner" to "outer" normal.
+    // However, you should be able to change the normal sense preserving
+    // the manifold orientation:
+    if (face.Orientation()==TopAbs_REVERSED)
+      {
+        normal *= -1;
+        Min_Curvature *= -1;
+        Max_Curvature *= -1;
+      }
+
+    return std_cxx11::tuple<Point<3>, Tensor<1,3>, double, double>(point(Value), normal, Min_Curvature, Max_Curvature);
   }
 
 
