@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2013 - 2015 by the deal.II authors
+ * Copyright (C) 2013 - 2016 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -155,8 +155,10 @@ void LaplaceProblem::solve ()
   PETScWrappers::PreconditionBlockJacobi preconditioner (A);
   cg_solver.solve (A, x, b, preconditioner);
 
-  // some output
-  // ?
+  PETScWrappers::Vector res(x);
+  A.residual(res,x,b);
+  AssertThrow(res.l2_norm()<1e-3,
+              ExcInternalError());
 }
 
 void LaplaceProblem::run ()
