@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2005 - 2015 by the deal.II authors
+// Copyright (C) 2005 - 2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -42,7 +42,7 @@
 #include <deal.II/numerics/vector_tools.h>
 #include <deal.II/numerics/matrix_tools.h>
 
-#include <deal.II/lac/compressed_sparsity_pattern.h>
+#include <deal.II/lac/dynamic_sparsity_pattern.h>
 
 #include <algorithm>
 #include <iomanip>
@@ -87,7 +87,8 @@ template <int dim>
 LaplaceProblem<dim>::LaplaceProblem (const unsigned int mapping_degree) :
   fe (1),
   dof_handler (triangulation),
-  mapping (mapping_degree)
+  mapping (mapping_degree),
+  last_error(std::numeric_limits<double>::max())
 {
   deallog << "Using mapping with degree " << mapping_degree << ":"
           << std::endl
@@ -122,8 +123,8 @@ void LaplaceProblem<dim>::setup_system ()
                                         i, -1);
   mean_value_constraints.close ();
 
-  CompressedSparsityPattern csp (dof_handler.n_dofs(),
-                                 dof_handler.n_dofs());
+  DynamicSparsityPattern csp (dof_handler.n_dofs(),
+                              dof_handler.n_dofs());
   DoFTools::make_sparsity_pattern (dof_handler, csp);
   mean_value_constraints.condense (csp);
 
