@@ -1,7 +1,6 @@
 // ---------------------------------------------------------------------
-// $Id: tbb.cc 31930 2013-12-08 01:50:50Z maier $
 //
-// Copyright (C) 2013 by the deal.II authors
+// Copyright (C) 2013 - 2015 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -43,41 +42,39 @@ struct scratch_data
 
 struct copy_data
 {
-    int value;
+  int value;
 };
 
 
 void assemble(const std::vector<int>::iterator &it,
-	      scratch_data &scratch,
-	      copy_data &data)
+              scratch_data &scratch,
+              copy_data &data)
 {
-  std::cout << "hello!" << std::endl;
-
-  data.value +=(*it)*(*it);
+  data.value = (*it);
 }
 
-void copy(int & value, const copy_data &data)
+void copy(int &value, const copy_data &data)
 {
   value += data.value;
 }
 
 void test2()
-{ 
-  std::vector<int> v(3);
-  v[0] = 3;
-  v[1] = 5;
-  v[2] = 1;
+{
+  const int maxi = 10000;
+  std::vector<int> v(maxi);
+  for (unsigned int i=0; i<v.size(); ++i)
+    v[i] = i+1;
   int result = 0;
   WorkStream::run(v.begin(),
-		  v.end(),
-		  &assemble,
-		  std_cxx1x::bind(&copy,
-				  std_cxx1x::ref(result),
-				  std_cxx1x::_1),
-		  scratch_data(), copy_data());
+                  v.end(),
+                  &assemble,
+                  std_cxx11::bind(&copy,
+                                  std_cxx11::ref(result),
+                                  std_cxx11::_1),
+                  scratch_data(), copy_data());
   std::cout << "result: " << result << std::endl;
 
-  if (result != (3*3+5*5+1*1))
+  if (result != maxi*(maxi+1)/2)
     exit(2);
 }
 
